@@ -31,10 +31,12 @@ type Application interface {
 	ForceCharm() bool
 	Exposed() bool
 	MinUnits() int
+	PasswordHash() string
 
 	EndpointBindings() map[string]string
 
-	Settings() map[string]interface{}
+	CharmConfig() map[string]interface{}
+	ApplicationConfig() map[string]interface{}
 
 	Leader() string
 	LeadershipSettings() map[string]interface{}
@@ -76,7 +78,8 @@ type application struct {
 
 	EndpointBindings_ map[string]string `yaml:"endpoint-bindings,omitempty"`
 
-	Settings_ map[string]interface{} `yaml:"settings"`
+	CharmConfig_       map[string]interface{} `yaml:"settings"`
+	ApplicationConfig_ map[string]interface{} `yaml:"application-config,omitempty"`
 
 	Leader_             string                 `yaml:"leader,omitempty"`
 	LeadershipSettings_ map[string]interface{} `yaml:"leadership-settings"`
@@ -92,6 +95,7 @@ type application struct {
 
 	Constraints_        *constraints                  `yaml:"constraints,omitempty"`
 	StorageConstraints_ map[string]*storageconstraint `yaml:"storage-constraints,omitempty"`
+	PasswordHash_       string                        `yaml:"password-hash,omitempty"`
 }
 
 // ApplicationArgs is an argument struct used to add an application to the Model.
@@ -104,10 +108,12 @@ type ApplicationArgs struct {
 	Channel              string
 	CharmModifiedVersion int
 	ForceCharm           bool
+	PasswordHash         string
 	Exposed              bool
 	MinUnits             int
 	EndpointBindings     map[string]string
-	Settings             map[string]interface{}
+	ApplicationConfig    map[string]interface{}
+	CharmConfig          map[string]interface{}
 	Leader               string
 	LeadershipSettings   map[string]interface{}
 	StorageConstraints   map[string]StorageConstraintArgs
@@ -126,9 +132,11 @@ func newApplication(args ApplicationArgs) *application {
 		CharmModifiedVersion_: args.CharmModifiedVersion,
 		ForceCharm_:           args.ForceCharm,
 		Exposed_:              args.Exposed,
+		PasswordHash_:         args.PasswordHash,
 		MinUnits_:             args.MinUnits,
 		EndpointBindings_:     args.EndpointBindings,
-		Settings_:             args.Settings,
+		ApplicationConfig_:    args.ApplicationConfig,
+		CharmConfig_:          args.CharmConfig,
 		Leader_:               args.Leader,
 		LeadershipSettings_:   args.LeadershipSettings,
 		MetricsCredentials_:   creds,
@@ -195,6 +203,11 @@ func (a *application) Exposed() bool {
 	return a.Exposed_
 }
 
+// PasswordHash implements Application.
+func (a *application) PasswordHash() string {
+	return a.PasswordHash_
+}
+
 // MinUnits implements Application.
 func (a *application) MinUnits() int {
 	return a.MinUnits_
@@ -205,9 +218,14 @@ func (a *application) EndpointBindings() map[string]string {
 	return a.EndpointBindings_
 }
 
-// Settings implements Application.
-func (a *application) Settings() map[string]interface{} {
-	return a.Settings_
+// ApplicationConfig implements Application.
+func (a *application) ApplicationConfig() map[string]interface{} {
+	return a.ApplicationConfig_
+}
+
+// CharmConfig implements Application.
+func (a *application) CharmConfig() map[string]interface{} {
+	return a.CharmConfig_
 }
 
 // Leader implements Application.
@@ -392,6 +410,7 @@ type applicationDeserializationFunc func(map[string]interface{}) (*application,
 var applicationDeserializationFuncs = map[int]applicationDeserializationFunc{
 	1: importApplicationV1,
 	2: importApplicationV2,
+	3: importApplicationV3,
 }
 
 func applicationV1Fields() (schema.Fields, schema.Defaults) {
@@ -425,6 +444,7 @@ func applicationV1Fields() (schema.Fields, schema.Defaults) {
 		"metrics-creds":       "",
 		"storage-constraints": schema.Omit,
 		"endpoint-bindings":   schema.Omit,
+		"application-config":  schema.Omit,
 	}
 	addAnnotationSchema(fields, defaults)
 	addConstraintsSchema(fields, defaults)
@@ -432,17 +452,35 @@ func applicationV1Fields() (schema.Fields, schema.Defaults) {
 	return fields, defaults
 }
 
+func applicationV2Fields() (schema.Fields, schema.Defaults) {
+	fields, defaults := applicationV1Fields()
+	fields["type"] = schema.String()
+	return fields, defaults
+}
+
+func applicationV3Fields() (schema.Fields, schema.Defaults) {
+	fields, defaults := applicationV2Fields()
+	fields["application-config"] = schema.StringMap(schema.Any())
+	fields["password-hash"] = schema.String()
+	defaults["password-hash"] = ""
+	return fields, defaults
+}
+
 func importApplicationV1(source map[string]interface{}) (*application, error) {
 	fields, defaults := applicationV1Fields()
 	return importApplication(fields, defaults, 1, source)
 }
 
 func importApplicationV2(source map[string]interface{}) (*application, error) {
-	fields, defaults := applicationV1Fields()
-	fields["type"] = schema.String()
+	fields, defaults := applicationV2Fields()
 	return importApplication(fields, defaults, 2, source)
 }
 
+func importApplicationV3(source map[string]interface{}) (*application, error) {
+	fields, defaults := applicationV3Fields()
+	return importApplication(fields, defaults, 3, source)
+}
+
 func importApplication(fields schema.Fields, defaults schema.Defaults, importVersion int, source map[string]interface{}) (*application, error) {
 	checker := schema.FieldMap(fields, defaults)
 
@@ -465,7 +503,7 @@ func importApplication(fields schema.Fields, defaults schema.Defaults, importVer
 		Exposed_:              valid["exposed"].(bool),
 		MinUnits_:             int(valid["min-units"].(int64)),
 		EndpointBindings_:     convertToStringMap(valid["endpoint-bindings"]),
-		Settings_:             valid["settings"].(map[string]interface{}),
+		CharmConfig_:          valid["settings"].(map[string]interface{}),
 		Leader_:               valid["leader"].(string),
 		LeadershipSettings_:   valid["leadership-settings"].(map[string]interface{}),
 		StatusHistory_:        newStatusHistory(),
@@ -474,6 +512,9 @@ func importApplication(fields schema.Fields, defaults schema.Defaults, importVer
 	if importVersion >= 2 {
 		result.Type_ = valid["type"].(string)
 	}
+	if importVersion >= 3 {
+		result.PasswordHash_ = valid["password-hash"].(string)
+	}
 
 	result.importAnnotations(valid)
 
@@ -481,6 +522,14 @@ func importApplication(fields schema.Fields, defaults schema.Defaults, importVer
 		return nil, errors.Trace(err)
 	}
 
+	if configValues, ok := valid["application-config"]; ok {
+		configMap, ok := configValues.(map[string]interface{})
+		if !ok {
+			return nil, errors.Errorf("unexpected value for application-config, %T", configValues)
+		}
+		result.ApplicationConfig_ = configMap
+	}
+
 	if constraintsMap, ok := valid["constraints"]; ok {
 		constraints, err := importConstraints(constraintsMap.(map[string]interface{}))
 		if err != nil {
