@@ -109,7 +109,7 @@ def parse_args(argv):
     """Parse all arguments."""
     parser = argparse.ArgumentParser(
         description="Test model migration feature")
-    add_basic_testing_arguments(parser)
+    add_basic_testing_arguments(parser, existing=False)
     parser.add_argument(
         '--use-develop',
         action='store_true',
@@ -384,13 +384,13 @@ def deploy_simple_server_to_new_model(
         new_model, resource_contents, series)
     _, deploy_complete = new_model.deploy('cs:ubuntu', series=series)
     new_model.wait_for(deploy_complete)
-    new_model.deploy('cs:ntp', series=series)
-    new_model.juju('add-relation', ('ntp', application))
-    new_model.juju('add-relation', ('ntp', 'ubuntu'))
+    new_model.deploy('cs:nrpe', series=series)
+    new_model.juju('add-relation', ('nrpe', application))
+    new_model.juju('add-relation', ('nrpe', 'ubuntu'))
     # Need to wait for the subordinate charms too.
     new_model.wait_for(AllApplicationActive())
     new_model.wait_for(AllApplicationWorkloads())
-    new_model.wait_for(AgentsIdle(['ntp/0', 'ntp/1']))
+    new_model.wait_for(AgentsIdle(['nrpe/0', 'nrpe/1']))
     assert_deployed_charm_is_responding(new_model, resource_contents)
 
     return new_model, application
@@ -510,7 +510,7 @@ def migrate_model_to_controller(
         wait_for_model(migration_target_client, source_client.env.environment)
         migration_target_client.wait_for_started()
         wait_until_model_disappears(
-            source_client, source_client.env.environment)
+            source_client, source_client.env.environment, timeout=480)
     except JujuAssertionError as e:
         # Attempt to show model details as it might log migration failure
         # message.
