@@ -569,6 +569,9 @@ class ModelClient:
 
     controller_permissions = frozenset(['login', 'addmodel', 'superuser'])
 
+    # Granting 'login' will error as a created user has that at creation.
+    ignore_permissions = frozenset(['login'])
+
     reserved_spaces = frozenset([
         'endpoint-bindings-data', 'endpoint-bindings-public'])
 
@@ -841,6 +844,9 @@ class ModelClient:
         model_client = self.clone(env)
         with model_client._bootstrap_config() as config_file:
             self._add_model(env.environment, config_file)
+        # Make sure we track this in case it needs special cleanup (i.e. using
+        # an existing controller).
+        self._backend.track_model(model_client)
         return model_client
 
     def make_model_config(self):
@@ -855,6 +861,10 @@ class ModelClient:
             'admin-secret',
             'application-id',
             'application-password',
+            'audit-log-capture-args',
+            'audit-log-max-size',
+            'audit-log-max-backups',
+            'auditing-enabled',
             'auth-url',
             'bootstrap-host',
             'client-email',
@@ -947,8 +957,11 @@ class ModelClient:
         exit_status, _ = self.juju(
             'destroy-model',
             ('{}:{}'.format(self.env.controller.name, self.env.environment),
-            '-y', '--destroy-storage',),
+             '-y', '--destroy-storage',),
             include_e=False, timeout=get_teardown_timeout(self))
+        # Ensure things don't get confused at teardown time (i.e. if using an
+        #  existing controller)
+        self._backend.untrack_model(self)
         return exit_status
 
     def kill_controller(self, check=False):
@@ -1354,6 +1367,7 @@ class ModelClient:
                             break
                         status.raise_highest_error(ignore_recoverable=True)
                         reporter.update(states)
+                        time.sleep(1)
                     else:
                         if status is not None:
                             log.error(status.status_text)
@@ -1570,6 +1584,7 @@ class ModelClient:
                     status = self.get_status()
                     if status.get_service_count() >= service_count:
                         return
+                    time.sleep(1)
                 else:
                     raise ApplicationsNotStarted(self.env.environment, status)
 
@@ -1614,6 +1629,7 @@ class ModelClient:
                     return
                 if not quiet:
                     reporter.update(states)
+                time.sleep(1)
             else:
                 status.raise_highest_error(ignore_recoverable=False)
         except StatusTimeout:
@@ -1843,22 +1859,8 @@ class ModelClient:
         user_client = self.create_cloned_environment(juju_home,
                                                      controller_name,
                                                      username)
-
-        try:
-            child = user_client.expect('register', (token), include_e=False)
-            child.expect('(?i)password')
-            child.sendline(username + '_password')
-            child.expect('(?i)password')
-            child.sendline(username + '_password')
-            child.expect('(?i)name')
-            child.sendline(controller_name)
-            self._end_pexpect_session(child)
-        except pexpect.TIMEOUT:
-            log.error('Buffer: {}'.format(child.buffer))
-            log.error('Before: {}'.format(child.before))
-            raise Exception(
-                'Registering user failed: pexpect session timed out')
         user_client.env.user_name = username
+        register_user_interactively(user_client, token, controller_name)
         return user_client
 
     def login_user(self, username=None, password=None):
@@ -1933,6 +1935,9 @@ class ModelClient:
 
     def grant(self, user_name, permission, model=None):
         """Grant the user with model or controller permission."""
+        if permission in self.ignore_permissions:
+            log.info('Ignoring permission "{}".'.format(permission))
+            return
         if permission in self.controller_permissions:
             self.juju(
                 'grant',
@@ -2108,6 +2113,30 @@ class ModelClient:
         self.juju('switch', (':'.join(args),), include_e=False)
 
 
+def register_user_interactively(client, token, controller_name):
+    """Register a user with the supplied token and controller name.
+
+    :param client: ModelClient on which to register the user (using the models
+      controller.)
+    :param token: Token string to use when registering.
+    :param controller_name: String to use when naming the controller.
+    """
+    try:
+        child = client.expect('register', (token), include_e=False)
+        child.expect('(?i)password')
+        child.sendline(client.env.user_name + '_password')
+        child.expect('(?i)password')
+        child.sendline(client.env.user_name + '_password')
+        child.expect('(?i)name')
+        child.sendline(controller_name)
+        client._end_pexpect_session(child)
+    except pexpect.TIMEOUT:
+        log.error('Buffer: {}'.format(child.buffer))
+        log.error('Before: {}'.format(child.before))
+        raise Exception(
+            'Registering user failed: pexpect session timed out')
+
+
 def juju_home_path(juju_home, dir_name):
     return os.path.join(juju_home, 'juju-homes', dir_name)
 
