@@ -227,6 +227,10 @@ LXC_BRIDGE="ignored"`[1:])
 		"max-logs-age":            "72h",
 		"max-logs-size":           "4G",
 		"max-txn-log-size":        "10M",
+		"auditing-enabled":        false,
+		"audit-log-capture-args":  true,
+		"audit-log-max-size":      "200M",
+		"audit-log-max-backups":   5,
 	})
 
 	// Check that controller model configuration has been added, and
@@ -331,13 +335,16 @@ LXC_BRIDGE="ignored"`[1:])
 		"Provider",
 		"Version",
 	)
+	// Those attritbutes are configured during initialization, after "Open".
+	expectedCalledCfg, err := hostedCfg.Apply(map[string]interface{}{"container-networking-method": ""})
+	c.Assert(err, jc.ErrorIsNil)
 	envProvider.CheckCall(c, 2, "Open", environs.OpenParams{
 		Cloud: environs.CloudSpec{
 			Type:   "dummy",
 			Name:   "dummy",
 			Region: "dummy-region",
 		},
-		Config: hostedCfg,
+		Config: expectedCalledCfg,
 	})
 	envProvider.CheckCall(c, 3, "Create", environs.CreateParams{
 		ControllerUUID: controllerCfg.ControllerUUID(),
