@@ -11,6 +11,8 @@ import (
 	"net"
 	"net/http"
 	"net/http/httptest"
+	"os"
+	"reflect"
 	"sync"
 	"sync/atomic"
 	"time"
@@ -413,7 +415,9 @@ func (s *apiclientSuite) TestFallbackToSNIHostnameOnCertErrorAndNonNumericHostna
 			CACert:      jtesting.CACert,
 			SNIHostName: "foo.com",
 		},
-		expectOpenError: `unable to connect to API: x509: a root or intermediate certificate is not authorized to sign in this domain`,
+		// go 1.9 says "is not authorized to sign for this name"
+		// go 1.10 says "is not authorized to sign for this domain"
+		expectOpenError: `unable to connect to API: x509: a root or intermediate certificate is not authorized to sign.*`,
 		expectDials: []dialAttempt{{
 			// The first dial attempt should use the private CA cert.
 			check: func(info dialInfo) {
@@ -446,7 +450,9 @@ func (s *apiclientSuite) TestFailImmediatelyOnCertErrorAndNumericHostname(c *gc.
 			Addrs:  []string{"0.1.2.3:1234"},
 			CACert: jtesting.CACert,
 		},
-		expectOpenError: `unable to connect to API: x509: a root or intermediate certificate is not authorized to sign in this domain`,
+		// go 1.9 says "is not authorized to sign for this name"
+		// go 1.10 says "is not authorized to sign for this domain"
+		expectOpenError: `unable to connect to API: x509: a root or intermediate certificate is not authorized to sign.*`,
 		expectDials: []dialAttempt{{
 			// The first dial attempt should use the private CA cert.
 			check: func(info dialInfo) {
@@ -567,9 +573,7 @@ func (s *apiclientSuite) TestPublicDNSName(c *gc.C) {
 	listener, err := net.Listen("tcp", "localhost:0")
 	c.Assert(err, gc.IsNil)
 	machineTag := names.NewMachineTag("0")
-	statePool := state.NewStatePool(s.State)
-	defer statePool.Close()
-	srv, err := apiserver.NewServer(statePool, listener, apiserver.ServerConfig{
+	srv, err := apiserver.NewServer(s.StatePool, listener, apiserver.ServerConfig{
 		Clock:           clock.WallClock,
 		Cert:            jtesting.ServerCert,
 		Key:             jtesting.ServerKey,
@@ -790,6 +794,16 @@ func (s *apiclientSuite) TestOpenTimesOutOnLogin(c *gc.C) {
 		})
 		done <- err
 	}()
+	// Wait for Login to be entered before we advance the clock. Note that we don't actually unblock the request,
+	// we just ensure that the other side has gotten to the point where it wants to be blocked. Otherwise we might
+	// advance the clock before we even get the api.Dial to finish or before TLS handshaking finishes.
+	unblocked := make(chan struct{})
+	defer close(unblocked)
+	select {
+	case unblock <- unblocked:
+	case <-time.After(jtesting.LongWait):
+		c.Fatalf("timed out waiting for Login to be called")
+	}
 	err := clk.WaitAdvance(5*time.Second, time.Second, 1)
 	c.Assert(err, jc.ErrorIsNil)
 	select {
@@ -901,10 +915,10 @@ func (s *apiclientSuite) TestOpenDialTimeoutDoesNotAffectLogin(c *gc.C) {
 	err := clk.WaitAdvance(0, 0, 0)
 	c.Assert(err, jc.ErrorIsNil)
 
-	// unblock the login by receiving from "unblocked", and then the
+	// unblock the login by sending to "unblocked", and then the
 	// api.Open should return the result of the login.
 	select {
-	case <-unblocked:
+	case unblocked <- struct{}{}:
 	case <-time.After(jtesting.LongWait):
 		c.Fatalf("timed out waiting for login to be unblocked")
 	}
@@ -1087,6 +1101,39 @@ func (s *apiclientSuite) TestIsBrokenPingFailed(c *gc.C) {
 	c.Assert(conn.IsBroken(), jc.IsTrue)
 }
 
+func (s *apiclientSuite) TestLoginCapturesCLIArgs(c *gc.C) {
+	s.PatchValue(&os.Args, []string{"this", "is", "the test", "command"})
+
+	info := s.APIInfo(c)
+	conn := newRPCConnection()
+	conn.response = &params.LoginResult{
+		ControllerTag: "controller-" + s.ControllerConfig.ControllerUUID(),
+		ServerVersion: "2.3-rc2",
+	}
+	// Pass an already-closed channel so we don't wait for the monitor
+	// to signal the rpc connection is dead when closing the state
+	// (because there's no monitor running).
+	broken := make(chan struct{})
+	close(broken)
+	testState := api.NewTestingState(api.TestingStateParams{
+		RPCConnection: conn,
+		Clock:         &fakeClock{},
+		Address:       "localhost:1234",
+		Broken:        broken,
+		Closed:        make(chan struct{}),
+	})
+	err := testState.Login(info.Tag, info.Password, "", nil)
+	c.Assert(err, jc.ErrorIsNil)
+
+	calls := conn.stub.Calls()
+	c.Assert(calls, gc.HasLen, 1)
+	call := calls[0]
+	c.Assert(call.FuncName, gc.Equals, "Admin.Login")
+	c.Assert(call.Args, gc.HasLen, 2)
+	request := call.Args[1].(*params.LoginRequest)
+	c.Assert(request.CLIArgs, gc.Equals, `this is "the test" command`)
+}
+
 type fakeClock struct {
 	clock.Clock
 
@@ -1123,7 +1170,8 @@ func newRPCConnection(errs ...error) *fakeRPCConnection {
 }
 
 type fakeRPCConnection struct {
-	stub testing.Stub
+	stub     testing.Stub
+	response interface{}
 }
 
 func (f *fakeRPCConnection) Dead() <-chan struct{} {
@@ -1136,6 +1184,11 @@ func (f *fakeRPCConnection) Close() error {
 
 func (f *fakeRPCConnection) Call(req rpc.Request, params, response interface{}) error {
 	f.stub.AddCall(req.Type+"."+req.Action, req.Version, params)
+	if f.response != nil {
+		rv := reflect.ValueOf(response)
+		target := reflect.Indirect(rv)
+		target.Set(reflect.Indirect(reflect.ValueOf(f.response)))
+	}
 	return f.stub.NextErr()
 }
 
@@ -1255,7 +1308,7 @@ func (a *loginTimeoutAPIAdmin) Login(req params.LoginRequest) (params.LoginResul
 		return params.LoginResult{}, errors.New("timed out waiting to be unblocked")
 	}
 	select {
-	case unblocked <- struct{}{}:
+	case <-unblocked:
 	case <-time.After(jtesting.LongWait):
 		return params.LoginResult{}, errors.New("timed out sending on unblocked channel")
 	}
