@@ -6,10 +6,13 @@ package remoterelations
 import (
 	"github.com/juju/errors"
 	"gopkg.in/juju/names.v2"
+	"gopkg.in/macaroon.v1"
 
+	"github.com/juju/juju/api"
 	"github.com/juju/juju/api/base"
 	apiwatcher "github.com/juju/juju/api/watcher"
 	"github.com/juju/juju/apiserver/params"
+	"github.com/juju/juju/status"
 	"github.com/juju/juju/watcher"
 )
 
@@ -26,32 +29,11 @@ func NewClient(caller base.APICaller) *Client {
 	return &Client{facadeCaller}
 }
 
-// PublishLocalRelationChange publishes local relations changes to the
-// remote side offering those relations.
-func (c *Client) PublishLocalRelationChange(change params.RemoteRelationChangeEvent) error {
-	args := params.RemoteRelationsChanges{
-		Changes: []params.RemoteRelationChangeEvent{change},
-	}
-	var results params.ErrorResults
-	err := c.facade.FacadeCall("PublishLocalRelationChange", args, &results)
-	if err != nil {
-		return errors.Trace(err)
-	}
-	if len(results.Results) != 1 {
-		return errors.Errorf("expected 1 result, got %d", len(results.Results))
-	}
-	result := results.Results[0]
-	if result.Error != nil {
-		return errors.Trace(result.Error)
-	}
-	return nil
-}
-
 // ImportRemoteEntity adds an entity to the remote entities collection
 // with the specified opaque token.
-func (c *Client) ImportRemoteEntity(sourceModelUUID string, entity names.Tag, token string) error {
-	args := params.RemoteEntityArgs{Args: []params.RemoteEntityArg{
-		{ModelTag: names.NewModelTag(sourceModelUUID).String(), Tag: entity.String(), Token: token}},
+func (c *Client) ImportRemoteEntity(entity names.Tag, token string) error {
+	args := params.RemoteEntityTokenArgs{Args: []params.RemoteEntityTokenArg{
+		{Tag: entity.String(), Token: token}},
 	}
 	var results params.ErrorResults
 	err := c.facade.FacadeCall("ImportRemoteEntities", args, &results)
@@ -69,12 +51,12 @@ func (c *Client) ImportRemoteEntity(sourceModelUUID string, entity names.Tag, to
 }
 
 // ExportEntities allocates unique, remote entity IDs for the given entities in the local model.
-func (c *Client) ExportEntities(tags []names.Tag) ([]params.RemoteEntityIdResult, error) {
+func (c *Client) ExportEntities(tags []names.Tag) ([]params.TokenResult, error) {
 	args := params.Entities{Entities: make([]params.Entity, len(tags))}
 	for i, tag := range tags {
 		args.Entities[i].Tag = tag.String()
 	}
-	var results params.RemoteEntityIdResults
+	var results params.TokenResults
 	err := c.facade.FacadeCall("ExportEntities", args, &results)
 	if err != nil {
 		return nil, errors.Trace(err)
@@ -86,9 +68,9 @@ func (c *Client) ExportEntities(tags []names.Tag) ([]params.RemoteEntityIdResult
 }
 
 // GetToken returns the token associated with the entity with the given tag for the specified model.
-func (c *Client) GetToken(sourceModelUUID string, tag names.Tag) (string, error) {
+func (c *Client) GetToken(tag names.Tag) (string, error) {
 	args := params.GetTokenArgs{Args: []params.GetTokenArg{
-		{ModelTag: names.NewModelTag(sourceModelUUID).String(), Tag: tag.String()}},
+		{Tag: tag.String()}},
 	}
 	var results params.StringResults
 	err := c.facade.FacadeCall("GetTokens", args, &results)
@@ -101,20 +83,20 @@ func (c *Client) GetToken(sourceModelUUID string, tag names.Tag) (string, error)
 	result := results.Results[0]
 	if result.Error != nil {
 		if params.IsCodeNotFound(result.Error) {
-			return "", errors.NotFoundf("token for %v in model %v", tag, sourceModelUUID)
+			return "", errors.NotFoundf("token for %v", tag)
 		}
 		return "", errors.Trace(result.Error)
 	}
 	return result.Result, nil
 }
 
-// RemoveRemoteEntity removes the specified entity from the remote entities collection.
-func (c *Client) RemoveRemoteEntity(sourceModelUUID string, entity names.Tag) error {
-	args := params.RemoteEntityArgs{Args: []params.RemoteEntityArg{
-		{ModelTag: names.NewModelTag(sourceModelUUID).String(), Tag: entity.String()}},
+// SaveMacaroon saves the macaroon for the entity.
+func (c *Client) SaveMacaroon(entity names.Tag, mac *macaroon.Macaroon) error {
+	args := params.EntityMacaroonArgs{Args: []params.EntityMacaroonArg{
+		{Tag: entity.String(), Macaroon: mac}},
 	}
 	var results params.ErrorResults
-	err := c.facade.FacadeCall("RemoveRemoteEntities", args, &results)
+	err := c.facade.FacadeCall("SaveMacaroons", args, &results)
 	if err != nil {
 		return errors.Trace(err)
 	}
@@ -128,20 +110,6 @@ func (c *Client) RemoveRemoteEntity(sourceModelUUID string, entity names.Tag) er
 	return nil
 }
 
-// RegisterRemoteRelations sets up the local model to participate in the specified relations.
-func (c *Client) RegisterRemoteRelations(relations ...params.RegisterRemoteRelation) ([]params.RemoteEntityIdResult, error) {
-	args := params.RegisterRemoteRelations{Relations: relations}
-	var results params.RemoteEntityIdResults
-	err := c.facade.FacadeCall("RegisterRemoteRelations", args, &results)
-	if err != nil {
-		return nil, errors.Trace(err)
-	}
-	if len(results.Results) != len(relations) {
-		return nil, errors.Errorf("expected %d result(s), got %d", len(relations), len(results.Results))
-	}
-	return results.Results, nil
-}
-
 // RelationUnitSettings returns the relation unit settings for the given relation units in the local model.
 func (c *Client) RelationUnitSettings(relationUnits []params.RelationUnit) ([]params.SettingsResult, error) {
 	args := params.RelationUnits{relationUnits}
@@ -277,3 +245,53 @@ func (c *Client) WatchRemoteRelations() (watcher.StringsWatcher, error) {
 	w := apiwatcher.NewStringsWatcher(c.facade.RawAPICaller(), result)
 	return w, nil
 }
+
+// ConsumeRemoteRelationChange consumes a change to settings originating
+// from the remote/offering side of a relation.
+func (c *Client) ConsumeRemoteRelationChange(change params.RemoteRelationChangeEvent) error {
+	args := params.RemoteRelationsChanges{
+		Changes: []params.RemoteRelationChangeEvent{change},
+	}
+	var results params.ErrorResults
+	err := c.facade.FacadeCall("ConsumeRemoteRelationChanges", args, &results)
+	if err != nil {
+		return errors.Trace(err)
+	}
+	return results.OneError()
+}
+
+// ControllerAPIInfoForModel retrieves the controller API info for the specified model.
+func (c *Client) ControllerAPIInfoForModel(modelUUID string) (*api.Info, error) {
+	modelTag := names.NewModelTag(modelUUID)
+	args := params.Entities{[]params.Entity{{Tag: modelTag.String()}}}
+	var results params.ControllerAPIInfoResults
+	err := c.facade.FacadeCall("ControllerAPIInfoForModels", args, &results)
+	if err != nil {
+		return nil, errors.Trace(err)
+	}
+	if len(results.Results) != 1 {
+		return nil, errors.Errorf("expected 1 result, got %d", len(results.Results))
+	}
+	result := results.Results[0]
+	if result.Error != nil {
+		return nil, result.Error
+	}
+	return &api.Info{
+		Addrs:    result.Addresses,
+		CACert:   result.CACert,
+		ModelTag: modelTag,
+	}, nil
+}
+
+// SetRemoteApplicationStatus sets the status for the specified remote application.
+func (c *Client) SetRemoteApplicationStatus(applicationName string, status status.Status, message string) error {
+	args := params.SetStatus{Entities: []params.EntityStatusArgs{
+		{Tag: names.NewApplicationTag(applicationName).String(), Status: status.String(), Info: message},
+	}}
+	var results params.ErrorResults
+	err := c.facade.FacadeCall("SetRemoteApplicationsStatus", args, &results)
+	if err != nil {
+		return errors.Trace(err)
+	}
+	return results.OneError()
+}
