@@ -6,11 +6,14 @@ package api_test
 import (
 	stdtesting "testing"
 
-	"github.com/juju/names"
 	jc "github.com/juju/testing/checkers"
 	gc "gopkg.in/check.v1"
+	"gopkg.in/juju/names.v2"
+	"gopkg.in/macaroon.v1"
 
 	"github.com/juju/juju/api"
+	"github.com/juju/juju/api/modelmanager"
+	"github.com/juju/juju/api/usermanager"
 	jujutesting "github.com/juju/juju/juju/testing"
 	"github.com/juju/juju/network"
 	coretesting "github.com/juju/juju/testing"
@@ -41,15 +44,17 @@ func (s *stateSuite) TestCloseMultipleOk(c *gc.C) {
 // OpenAPIWithoutLogin connects to the API and returns an api.State without
 // actually calling st.Login already. The returned strings are the "tag" and
 // "password" that we would have used to login.
-func (s *stateSuite) OpenAPIWithoutLogin(c *gc.C) (*api.State, string, string) {
+func (s *stateSuite) OpenAPIWithoutLogin(c *gc.C) (api.Connection, names.Tag, string) {
 	info := s.APIInfo(c)
 	tag := info.Tag
 	password := info.Password
 	info.Tag = nil
 	info.Password = ""
+	info.Macaroons = nil
+	info.SkipLogin = true
 	apistate, err := api.Open(info, api.DialOpts{})
 	c.Assert(err, jc.ErrorIsNil)
-	return apistate, tag.String(), password
+	return apistate, tag, password
 }
 
 func (s *stateSuite) TestAPIHostPortsAlwaysIncludesTheConnection(c *gc.C) {
@@ -73,26 +78,69 @@ func (s *stateSuite) TestAPIHostPortsAlwaysIncludesTheConnection(c *gc.C) {
 	})
 }
 
-func (s *stateSuite) TestLoginSetsEnvironTag(c *gc.C) {
-	env, err := s.State.Environment()
+func (s *stateSuite) TestTags(c *gc.C) {
+	model, err := s.State.Model()
 	c.Assert(err, jc.ErrorIsNil)
 	apistate, tag, password := s.OpenAPIWithoutLogin(c)
 	defer apistate.Close()
-	// We haven't called Login yet, so the EnvironTag shouldn't be set.
-	envTag, err := apistate.EnvironTag()
-	c.Check(err, gc.ErrorMatches, `"" is not a valid tag`)
-	c.Check(envTag, gc.Equals, names.EnvironTag{})
-	err = apistate.Login(tag, password, "")
+	// Even though we haven't called Login, the model tag should
+	// still be set.
+	modelTag, ok := apistate.ModelTag()
+	c.Check(ok, jc.IsTrue)
+	c.Check(modelTag, gc.Equals, model.ModelTag())
+	err = apistate.Login(tag, password, "", nil)
 	c.Assert(err, jc.ErrorIsNil)
-	// Now that we've logged in, EnvironTag should be updated correctly.
-	envTag, err = apistate.EnvironTag()
-	c.Check(err, jc.ErrorIsNil)
-	c.Check(envTag, gc.Equals, env.EnvironTag())
-	// The server tag is also set, and since the environment is the
-	// state server environment, the uuid is the same.
-	srvTag, err := apistate.ServerTag()
-	c.Check(err, jc.ErrorIsNil)
-	c.Check(srvTag, gc.Equals, env.EnvironTag())
+	// Now that we've logged in, ModelTag should still be the same.
+	modelTag, ok = apistate.ModelTag()
+	c.Check(ok, jc.IsTrue)
+	c.Check(modelTag, gc.Equals, model.ModelTag())
+	controllerTag := apistate.ControllerTag()
+	c.Check(controllerTag, gc.Equals, coretesting.ControllerTag)
+}
+
+func (s *stateSuite) TestLoginSetsModelAccess(c *gc.C) {
+	// The default user has admin access.
+	c.Assert(s.APIState.ModelAccess(), gc.Equals, "admin")
+
+	manager := usermanager.NewClient(s.OpenControllerAPI(c))
+	defer manager.Close()
+	usertag, _, err := manager.AddUser("ro", "ro", "ro-password")
+	c.Assert(err, jc.ErrorIsNil)
+	mmanager := modelmanager.NewClient(s.OpenControllerAPI(c))
+	defer mmanager.Close()
+	modeltag, ok := s.APIState.ModelTag()
+	c.Assert(ok, jc.IsTrue)
+	err = mmanager.GrantModel(usertag.Id(), "read", modeltag.Id())
+	c.Assert(err, jc.ErrorIsNil)
+	conn := s.OpenAPIAs(c, usertag, "ro-password")
+	c.Assert(conn.ModelAccess(), gc.Equals, "read")
+}
+
+func (s *stateSuite) TestLoginSetsControllerAccess(c *gc.C) {
+	// The default user has admin access.
+	c.Assert(s.APIState.ControllerAccess(), gc.Equals, "superuser")
+
+	manager := usermanager.NewClient(s.OpenControllerAPI(c))
+	defer manager.Close()
+	usertag, _, err := manager.AddUser("ro", "ro", "ro-password")
+	c.Assert(err, jc.ErrorIsNil)
+	mmanager := modelmanager.NewClient(s.OpenControllerAPI(c))
+	defer mmanager.Close()
+	modeltag, ok := s.APIState.ModelTag()
+	c.Assert(ok, jc.IsTrue)
+	err = mmanager.GrantModel(usertag.Id(), "read", modeltag.Id())
+	c.Assert(err, jc.ErrorIsNil)
+	conn := s.OpenAPIAs(c, usertag, "ro-password")
+	c.Assert(conn.ControllerAccess(), gc.Equals, "login")
+}
+
+func (s *stateSuite) TestLoginMacaroonInvalidId(c *gc.C) {
+	apistate, tag, _ := s.OpenAPIWithoutLogin(c)
+	defer apistate.Close()
+	mac, err := macaroon.New([]byte("root-key"), "id", "juju")
+	c.Assert(err, jc.ErrorIsNil)
+	err = apistate.Login(tag, "", "", []macaroon.Slice{{mac}})
+	c.Assert(err, gc.ErrorMatches, "interaction required but not possible")
 }
 
 func (s *stateSuite) TestLoginTracksFacadeVersions(c *gc.C) {
@@ -100,14 +148,14 @@ func (s *stateSuite) TestLoginTracksFacadeVersions(c *gc.C) {
 	defer apistate.Close()
 	// We haven't called Login yet, so the Facade Versions should be empty
 	c.Check(apistate.AllFacadeVersions(), gc.HasLen, 0)
-	err := apistate.Login(tag, password, "")
+	err := apistate.Login(tag, password, "", nil)
 	c.Assert(err, jc.ErrorIsNil)
 	// Now that we've logged in, AllFacadeVersions should be updated.
 	allVersions := apistate.AllFacadeVersions()
 	c.Check(allVersions, gc.Not(gc.HasLen), 0)
 	// For sanity checking, ensure that we have a v2 of the Client facade
 	c.Assert(allVersions["Client"], gc.Not(gc.HasLen), 0)
-	c.Check(allVersions["Client"][0], gc.Equals, 0)
+	c.Check(allVersions["Client"][0], gc.Equals, 1)
 }
 
 func (s *stateSuite) TestAllFacadeVersionsSafeFromMutation(c *gc.C) {
@@ -124,7 +172,7 @@ func (s *stateSuite) TestAllFacadeVersionsSafeFromMutation(c *gc.C) {
 }
 
 func (s *stateSuite) TestBestFacadeVersion(c *gc.C) {
-	c.Check(s.APIState.BestFacadeVersion("Client"), gc.Equals, 0)
+	c.Check(s.APIState.BestFacadeVersion("Client"), gc.Equals, 1)
 }
 
 func (s *stateSuite) TestAPIHostPortsMovesConnectedValueFirst(c *gc.C) {
@@ -136,25 +184,31 @@ func (s *stateSuite) TestAPIHostPortsMovesConnectedValueFirst(c *gc.C) {
 	// the other addresses, but always see this one as well.
 	info := s.APIInfo(c)
 	// We intentionally set this to invalid values
-	badValue := network.HostPort{network.Address{
-		Value:       "0.1.2.3",
-		Type:        network.IPv4Address,
-		NetworkName: "",
-		Scope:       network.ScopeMachineLocal,
-	}, 1234}
+	badValue := network.HostPort{
+		Address: network.Address{
+			Value: "0.1.2.3",
+			Type:  network.IPv4Address,
+			Scope: network.ScopeMachineLocal,
+		},
+		Port: 1234,
+	}
 	badServer := []network.HostPort{badValue}
-	extraAddress := network.HostPort{network.Address{
-		Value:       "0.1.2.4",
-		Type:        network.IPv4Address,
-		NetworkName: "",
-		Scope:       network.ScopeMachineLocal,
-	}, 5678}
-	extraAddress2 := network.HostPort{network.Address{
-		Value:       "0.1.2.1",
-		Type:        network.IPv4Address,
-		NetworkName: "",
-		Scope:       network.ScopeMachineLocal,
-	}, 9012}
+	extraAddress := network.HostPort{
+		Address: network.Address{
+			Value: "0.1.2.4",
+			Type:  network.IPv4Address,
+			Scope: network.ScopeMachineLocal,
+		},
+		Port: 5678,
+	}
+	extraAddress2 := network.HostPort{
+		Address: network.Address{
+			Value: "0.1.2.1",
+			Type:  network.IPv4Address,
+			Scope: network.ScopeMachineLocal,
+		},
+		Port: 9012,
+	}
 	serverExtra := []network.HostPort{
 		extraAddress, goodAddress, extraAddress2,
 	}
