@@ -16,6 +16,7 @@ import (
 	"github.com/juju/juju/api/storage"
 	"github.com/juju/juju/apiserver/common"
 	"github.com/juju/juju/apiserver/params"
+	jujustorage "github.com/juju/juju/storage"
 	"github.com/juju/juju/testing"
 )
 
@@ -516,10 +517,13 @@ func (s *storageMockSuite) TestAddToUnit(c *gc.C) {
 
 	storageN := 3
 	expectedError := common.ServerError(errors.NotValidf("storage directive"))
-	one := func(u, s string, attrs params.StorageConstraints) params.ErrorResult {
-		result := params.ErrorResult{}
+	expectedDetails := &params.AddStorageDetails{[]string{"a/0", "b/1"}}
+	one := func(u, s string, attrs params.StorageConstraints) params.AddStorageResult {
+		result := params.AddStorageResult{}
 		if s == errOut {
 			result.Error = expectedError
+		} else {
+			result.Result = expectedDetails
 		}
 		return result
 	}
@@ -539,8 +543,8 @@ func (s *storageMockSuite) TestAddToUnit(c *gc.C) {
 			c.Assert(args.Storages, gc.HasLen, storageN)
 			c.Assert(args.Storages, gc.DeepEquals, unitStorages)
 
-			if results, k := result.(*params.ErrorResults); k {
-				out := []params.ErrorResult{}
+			if results, k := result.(*params.AddStorageResults); k {
+				out := []params.AddStorageResult{}
 				for _, s := range args.Storages {
 					out = append(out, one(s.UnitTag, s.StorageName, s.Constraints))
 				}
@@ -553,10 +557,10 @@ func (s *storageMockSuite) TestAddToUnit(c *gc.C) {
 	r, err := storageClient.AddToUnit(unitStorages)
 	c.Assert(err, jc.ErrorIsNil)
 	c.Assert(r, gc.HasLen, storageN)
-	expected := []params.ErrorResult{
-		{nil},
-		{expectedError},
-		{nil},
+	expected := []params.AddStorageResult{
+		{Result: expectedDetails},
+		{Error: expectedError},
+		{Result: expectedDetails},
 	}
 	c.Assert(r, jc.SameContents, expected)
 }
@@ -584,44 +588,109 @@ func (s *storageMockSuite) TestAddToUnitFacadeCallError(c *gc.C) {
 	c.Assert(found, gc.HasLen, 0)
 }
 
-func (s *storageMockSuite) TestDestroy(c *gc.C) {
-	apiCaller := basetesting.APICallerFunc(
-		func(objType string,
-			version int,
-			id, request string,
-			a, result interface{},
-		) error {
-			c.Check(objType, gc.Equals, "Storage")
-			c.Check(id, gc.Equals, "")
-			c.Check(request, gc.Equals, "Destroy")
-			c.Check(a, jc.DeepEquals, params.Entities{[]params.Entity{
-				{"storage-foo-0"},
-				{"storage-bar-1"},
-			}})
-			c.Assert(result, gc.FitsTypeOf, &params.ErrorResults{})
-			results := result.(*params.ErrorResults)
-			results.Results = []params.ErrorResult{
-				{},
-				{Error: &params.Error{Message: "baz"}},
-			}
-			return nil
-		},
-	)
+func (s *storageMockSuite) TestRemove(c *gc.C) {
+	apiCaller := basetesting.BestVersionCaller{
+		APICallerFunc: basetesting.APICallerFunc(
+			func(objType string,
+				version int,
+				id, request string,
+				a, result interface{},
+			) error {
+				c.Check(objType, gc.Equals, "Storage")
+				c.Check(id, gc.Equals, "")
+				c.Check(request, gc.Equals, "Remove")
+				c.Check(a, jc.DeepEquals, params.RemoveStorage{[]params.RemoveStorageInstance{
+					{Tag: "storage-foo-0", DestroyAttachments: false, DestroyStorage: false},
+					{Tag: "storage-bar-1", DestroyAttachments: false, DestroyStorage: false},
+				}})
+				c.Assert(result, gc.FitsTypeOf, &params.ErrorResults{})
+				results := result.(*params.ErrorResults)
+				results.Results = []params.ErrorResult{
+					{},
+					{Error: &params.Error{Message: "baz"}},
+				}
+				return nil
+			},
+		),
+		BestVersion: 4,
+	}
 	client := storage.NewClient(apiCaller)
-	results, err := client.Destroy([]string{"foo/0", "bar/1"})
+	results, err := client.Remove([]string{"foo/0", "bar/1"}, false, false)
 	c.Check(err, jc.ErrorIsNil)
 	c.Assert(results, gc.HasLen, 2)
 	c.Assert(results[0].Error, gc.IsNil)
 	c.Assert(results[1].Error, jc.DeepEquals, &params.Error{Message: "baz"})
 }
 
-func (s *storageMockSuite) TestDestroyInvalidStorageId(c *gc.C) {
+func (s *storageMockSuite) TestRemoveDestroyAttachments(c *gc.C) {
+	apiCaller := basetesting.BestVersionCaller{
+		APICallerFunc: basetesting.APICallerFunc(
+			func(objType string,
+				version int,
+				id, request string,
+				a, result interface{},
+			) error {
+				c.Check(a, jc.DeepEquals, params.RemoveStorage{[]params.RemoveStorageInstance{{
+					Tag:                "storage-foo-0",
+					DestroyAttachments: true,
+					DestroyStorage:     true,
+				}}})
+				results := result.(*params.ErrorResults)
+				results.Results = []params.ErrorResult{{}}
+				return nil
+			},
+		),
+		BestVersion: 4,
+	}
+	client := storage.NewClient(apiCaller)
+	results, err := client.Remove([]string{"foo/0"}, true, true)
+	c.Check(err, jc.ErrorIsNil)
+	c.Assert(results, gc.HasLen, 1)
+	c.Assert(results[0].Error, gc.IsNil)
+}
+
+func (s *storageMockSuite) TestRemoveDestroyV3(c *gc.C) {
+	apiCaller := basetesting.BestVersionCaller{
+		APICallerFunc: basetesting.APICallerFunc(
+			func(objType string,
+				version int,
+				id, request string,
+				a, result interface{},
+			) error {
+				c.Check(objType, gc.Equals, "Storage")
+				c.Check(id, gc.Equals, "")
+				c.Check(request, gc.Equals, "Destroy")
+				c.Check(a, jc.DeepEquals, params.Entities{[]params.Entity{
+					{Tag: "storage-foo-0"},
+				}})
+				results := result.(*params.ErrorResults)
+				results.Results = []params.ErrorResult{{}}
+				return nil
+			},
+		),
+		BestVersion: 3,
+	}
+	client := storage.NewClient(apiCaller)
+	results, err := client.Remove([]string{"foo/0"}, true, true)
+	c.Check(err, jc.ErrorIsNil)
+	c.Assert(results, gc.HasLen, 1)
+	c.Assert(results[0].Error, gc.IsNil)
+}
+
+func (s *storageMockSuite) TestRemoveDestroyV3NoDestroyStorage(c *gc.C) {
+	apiCaller := basetesting.BestVersionCaller{BestVersion: 3}
+	client := storage.NewClient(apiCaller)
+	_, err := client.Remove([]string{"foo/0"}, true, false)
+	c.Check(err, gc.ErrorMatches, "this juju controller does not support non-destructive removal of storage")
+}
+
+func (s *storageMockSuite) TestRemoveInvalidStorageId(c *gc.C) {
 	client := storage.NewClient(basetesting.APICallerFunc(
 		func(_ string, _ int, _, _ string, _, _ interface{}) error {
 			return nil
 		},
 	))
-	_, err := client.Destroy([]string{"foo/bar"})
+	_, err := client.Remove([]string{"foo/bar"}, false, false)
 	c.Check(err, gc.ErrorMatches, `storage ID "foo/bar" not valid`)
 }
 
@@ -718,3 +787,63 @@ func (s *storageMockSuite) TestAttachArityMismatch(c *gc.C) {
 	_, err := client.Attach("foo/0", []string{"bar/1", "baz/2"})
 	c.Check(err, gc.ErrorMatches, `expected 2 result\(s\), got 3`)
 }
+
+func (s *storageMockSuite) TestImport(c *gc.C) {
+	apiCaller := basetesting.APICallerFunc(
+		func(objType string,
+			version int,
+			id, request string,
+			a, result interface{},
+		) error {
+			c.Check(objType, gc.Equals, "Storage")
+			c.Check(id, gc.Equals, "")
+			c.Check(request, gc.Equals, "Import")
+			c.Check(a, jc.DeepEquals, params.BulkImportStorageParams{[]params.ImportStorageParams{{
+				Kind:        params.StorageKindBlock,
+				Pool:        "foo",
+				ProviderId:  "bar",
+				StorageName: "baz",
+			}}})
+			c.Assert(result, gc.FitsTypeOf, &params.ImportStorageResults{})
+			results := result.(*params.ImportStorageResults)
+			results.Results = []params.ImportStorageResult{{
+				Result: &params.ImportStorageDetails{
+					StorageTag: "storage-qux-0",
+				},
+			}}
+			return nil
+		},
+	)
+	client := storage.NewClient(apiCaller)
+	storageTag, err := client.Import(jujustorage.StorageKindBlock, "foo", "bar", "baz")
+	c.Assert(err, jc.ErrorIsNil)
+	c.Assert(storageTag, gc.Equals, names.NewStorageTag("qux/0"))
+}
+
+func (s *storageMockSuite) TestImportError(c *gc.C) {
+	apiCaller := basetesting.APICallerFunc(
+		func(objType string, version int, id, request string, a, result interface{}) error {
+			results := result.(*params.ImportStorageResults)
+			results.Results = []params.ImportStorageResult{{
+				Error: &params.Error{Message: "qux"},
+			}}
+			return nil
+		},
+	)
+	client := storage.NewClient(apiCaller)
+	_, err := client.Import(jujustorage.StorageKindBlock, "foo", "bar", "baz")
+	c.Check(err, gc.ErrorMatches, "qux")
+}
+
+func (s *storageMockSuite) TestImportArityMismatch(c *gc.C) {
+	apiCaller := basetesting.APICallerFunc(
+		func(objType string, version int, id, request string, a, result interface{}) error {
+			results := result.(*params.ImportStorageResults)
+			results.Results = []params.ImportStorageResult{{}, {}}
+			return nil
+		},
+	)
+	client := storage.NewClient(apiCaller)
+	_, err := client.Import(jujustorage.StorageKindBlock, "foo", "bar", "baz")
+	c.Check(err, gc.ErrorMatches, `expected 1 result, got 2`)
+}
