@@ -398,6 +398,39 @@ func (s *provisionerSuite) TestVolumeParams(c *gc.C) {
 	}})
 }
 
+func (s *provisionerSuite) TestRemoveVolumeParams(c *gc.C) {
+	apiCaller := testing.APICallerFunc(func(objType string, version int, id, request string, arg, result interface{}) error {
+		c.Check(objType, gc.Equals, "StorageProvisioner")
+		c.Check(version, gc.Equals, 0)
+		c.Check(id, gc.Equals, "")
+		c.Check(request, gc.Equals, "RemoveVolumeParams")
+		c.Check(arg, gc.DeepEquals, params.Entities{Entities: []params.Entity{{"volume-100"}}})
+		c.Assert(result, gc.FitsTypeOf, &params.RemoveVolumeParamsResults{})
+		*(result.(*params.RemoveVolumeParamsResults)) = params.RemoveVolumeParamsResults{
+			Results: []params.RemoveVolumeParamsResult{{
+				Result: params.RemoveVolumeParams{
+					Provider: "foo",
+					VolumeId: "bar",
+					Destroy:  true,
+				},
+			}},
+		}
+		return nil
+	})
+
+	st, err := storageprovisioner.NewState(apiCaller, names.NewMachineTag("123"))
+	c.Assert(err, jc.ErrorIsNil)
+	volumeParams, err := st.RemoveVolumeParams([]names.VolumeTag{names.NewVolumeTag("100")})
+	c.Check(err, jc.ErrorIsNil)
+	c.Assert(volumeParams, jc.DeepEquals, []params.RemoveVolumeParamsResult{{
+		Result: params.RemoveVolumeParams{
+			Provider: "foo",
+			VolumeId: "bar",
+			Destroy:  true,
+		},
+	}})
+}
+
 func (s *provisionerSuite) TestFilesystemParams(c *gc.C) {
 	var callCount int
 	apiCaller := testing.APICallerFunc(func(objType string, version int, id, request string, arg, result interface{}) error {
@@ -432,6 +465,39 @@ func (s *provisionerSuite) TestFilesystemParams(c *gc.C) {
 	}})
 }
 
+func (s *provisionerSuite) TestRemoveFilesystemParams(c *gc.C) {
+	apiCaller := testing.APICallerFunc(func(objType string, version int, id, request string, arg, result interface{}) error {
+		c.Check(objType, gc.Equals, "StorageProvisioner")
+		c.Check(version, gc.Equals, 0)
+		c.Check(id, gc.Equals, "")
+		c.Check(request, gc.Equals, "RemoveFilesystemParams")
+		c.Check(arg, gc.DeepEquals, params.Entities{Entities: []params.Entity{{"filesystem-100"}}})
+		c.Assert(result, gc.FitsTypeOf, &params.RemoveFilesystemParamsResults{})
+		*(result.(*params.RemoveFilesystemParamsResults)) = params.RemoveFilesystemParamsResults{
+			Results: []params.RemoveFilesystemParamsResult{{
+				Result: params.RemoveFilesystemParams{
+					Provider:     "foo",
+					FilesystemId: "bar",
+					Destroy:      true,
+				},
+			}},
+		}
+		return nil
+	})
+
+	st, err := storageprovisioner.NewState(apiCaller, names.NewMachineTag("123"))
+	c.Assert(err, jc.ErrorIsNil)
+	filesystemParams, err := st.RemoveFilesystemParams([]names.FilesystemTag{names.NewFilesystemTag("100")})
+	c.Check(err, jc.ErrorIsNil)
+	c.Assert(filesystemParams, jc.DeepEquals, []params.RemoveFilesystemParamsResult{{
+		Result: params.RemoveFilesystemParams{
+			Provider:     "foo",
+			FilesystemId: "bar",
+			Destroy:      true,
+		},
+	}})
+}
+
 func (s *provisionerSuite) TestVolumeAttachmentParams(c *gc.C) {
 	paramsResults := []params.VolumeAttachmentParamsResult{{
 		Result: params.VolumeAttachmentParams{
@@ -753,6 +819,27 @@ func (s *provisionerSuite) TestVolumeParamsClientError(c *gc.C) {
 	})
 }
 
+func (s *provisionerSuite) TestRemoveVolumeParamsClientError(c *gc.C) {
+	s.testClientError(c, func(st *storageprovisioner.State) error {
+		_, err := st.RemoveVolumeParams(nil)
+		return err
+	})
+}
+
+func (s *provisionerSuite) TestFilesystemParamsClientError(c *gc.C) {
+	s.testClientError(c, func(st *storageprovisioner.State) error {
+		_, err := st.FilesystemParams(nil)
+		return err
+	})
+}
+
+func (s *provisionerSuite) TestRemoveFilesystemParamsClientError(c *gc.C) {
+	s.testClientError(c, func(st *storageprovisioner.State) error {
+		_, err := st.RemoveFilesystemParams(nil)
+		return err
+	})
+}
+
 func (s *provisionerSuite) TestRemoveClientError(c *gc.C) {
 	s.testClientError(c, func(st *storageprovisioner.State) error {
 		_, err := st.Remove(nil)
@@ -844,6 +931,57 @@ func (s *provisionerSuite) TestVolumeParamsServerError(c *gc.C) {
 	c.Check(results[0].Error, gc.ErrorMatches, "MSG")
 }
 
+func (s *provisionerSuite) TestRemoveVolumeParamsServerError(c *gc.C) {
+	apiCaller := testing.APICallerFunc(func(objType string, version int, id, request string, arg, result interface{}) error {
+		*(result.(*params.RemoveVolumeParamsResults)) = params.RemoveVolumeParamsResults{
+			Results: []params.RemoveVolumeParamsResult{{
+				Error: &params.Error{Message: "MSG", Code: "621"},
+			}},
+		}
+		return nil
+	})
+	st, err := storageprovisioner.NewState(apiCaller, names.NewMachineTag("123"))
+	c.Assert(err, jc.ErrorIsNil)
+	results, err := st.RemoveVolumeParams([]names.VolumeTag{names.NewVolumeTag("100")})
+	c.Assert(err, jc.ErrorIsNil)
+	c.Assert(results, gc.HasLen, 1)
+	c.Check(results[0].Error, gc.ErrorMatches, "MSG")
+}
+
+func (s *provisionerSuite) TestFilesystemParamsServerError(c *gc.C) {
+	apiCaller := testing.APICallerFunc(func(objType string, version int, id, request string, arg, result interface{}) error {
+		*(result.(*params.FilesystemParamsResults)) = params.FilesystemParamsResults{
+			Results: []params.FilesystemParamsResult{{
+				Error: &params.Error{Message: "MSG", Code: "621"},
+			}},
+		}
+		return nil
+	})
+	st, err := storageprovisioner.NewState(apiCaller, names.NewMachineTag("123"))
+	c.Assert(err, jc.ErrorIsNil)
+	results, err := st.FilesystemParams([]names.FilesystemTag{names.NewFilesystemTag("100")})
+	c.Assert(err, jc.ErrorIsNil)
+	c.Assert(results, gc.HasLen, 1)
+	c.Check(results[0].Error, gc.ErrorMatches, "MSG")
+}
+
+func (s *provisionerSuite) TestRemoveFilesystemParamsServerError(c *gc.C) {
+	apiCaller := testing.APICallerFunc(func(objType string, version int, id, request string, arg, result interface{}) error {
+		*(result.(*params.RemoveFilesystemParamsResults)) = params.RemoveFilesystemParamsResults{
+			Results: []params.RemoveFilesystemParamsResult{{
+				Error: &params.Error{Message: "MSG", Code: "621"},
+			}},
+		}
+		return nil
+	})
+	st, err := storageprovisioner.NewState(apiCaller, names.NewMachineTag("123"))
+	c.Assert(err, jc.ErrorIsNil)
+	results, err := st.RemoveFilesystemParams([]names.FilesystemTag{names.NewFilesystemTag("100")})
+	c.Assert(err, jc.ErrorIsNil)
+	c.Assert(results, gc.HasLen, 1)
+	c.Check(results[0].Error, gc.ErrorMatches, "MSG")
+}
+
 func (s *provisionerSuite) TestSetVolumeInfoServerError(c *gc.C) {
 	apiCaller := testing.APICallerFunc(func(objType string, version int, id, request string, arg, result interface{}) error {
 		*(result.(*params.ErrorResults)) = params.ErrorResults{
