@@ -15,6 +15,7 @@ import (
 	"gopkg.in/macaroon-bakery.v1/bakery/checkers"
 	"gopkg.in/macaroon-bakery.v1/bakerytest"
 	"gopkg.in/macaroon-bakery.v1/httpbakery"
+	"gopkg.in/macaroon.v1"
 
 	"github.com/juju/juju/api"
 	"github.com/juju/juju/controller"
@@ -67,6 +68,12 @@ func (s *MacaroonSuite) TearDownTest(c *gc.C) {
 	s.JujuConnSuite.TearDownTest(c)
 }
 
+// DischargerLocation returns the URL of the third party caveat
+// discharger.
+func (s *MacaroonSuite) DischargerLocation() string {
+	return s.discharger.Location()
+}
+
 // AddModelUser is a convenience function that adds an external
 // user to the current model. It will panic
 // if the user name is local.
@@ -115,6 +122,11 @@ func (s *MacaroonSuite) APIInfo(c *gc.C) *api.Info {
 	info := s.JujuConnSuite.APIInfo(c)
 	info.Tag = nil
 	info.Password = ""
+	// Fill in any old macaroon to ensure we don't attempt
+	// an anonymous login.
+	mac, err := macaroon.New(nil, "test", "")
+	c.Assert(err, jc.ErrorIsNil)
+	info.Macaroons = []macaroon.Slice{{mac}}
 	return info
 }
 
