@@ -6,7 +6,7 @@ package uniter
 import (
 	"fmt"
 
-	"gopkg.in/juju/charm.v6-unstable"
+	"gopkg.in/juju/charm.v6"
 
 	"github.com/juju/juju/apiserver/params"
 )
