@@ -11,15 +11,15 @@ import (
 	jc "github.com/juju/testing/checkers"
 	"github.com/juju/utils"
 	gc "gopkg.in/check.v1"
-	"gopkg.in/juju/charm.v6-unstable"
+	"gopkg.in/juju/charm.v6"
 	"gopkg.in/juju/names.v2"
 
 	"github.com/juju/juju/api"
-	"github.com/juju/juju/api/base"
+	"github.com/juju/juju/api/base/testing"
 	"github.com/juju/juju/api/uniter"
 	"github.com/juju/juju/apiserver/common"
 	"github.com/juju/juju/apiserver/params"
-	"github.com/juju/juju/juju/testing"
+	jujutesting "github.com/juju/juju/juju/testing"
 	"github.com/juju/juju/network"
 	"github.com/juju/juju/state"
 	"github.com/juju/juju/status"
@@ -204,7 +204,7 @@ func (s *unitSuite) TestDestroyAllSubordinates(c *gc.C) {
 	c.Assert(monitoringSub.Life(), gc.Equals, state.Dying)
 }
 
-func (s *unitSuite) TestRefresh(c *gc.C) {
+func (s *unitSuite) TestRefreshLife(c *gc.C) {
 	c.Assert(s.apiUnit.Life(), gc.Equals, params.Alive)
 
 	err := s.apiUnit.EnsureDead()
@@ -216,6 +216,37 @@ func (s *unitSuite) TestRefresh(c *gc.C) {
 	c.Assert(s.apiUnit.Life(), gc.Equals, params.Dead)
 }
 
+func (s *unitSuite) TestRefreshResolve(c *gc.C) {
+	err := s.wordpressUnit.SetResolved(state.ResolvedRetryHooks)
+	c.Assert(err, jc.ErrorIsNil)
+
+	err = s.apiUnit.Refresh()
+	c.Assert(err, jc.ErrorIsNil)
+	mode := s.apiUnit.Resolved()
+	c.Assert(mode, gc.Equals, params.ResolvedRetryHooks)
+
+	err = s.apiUnit.ClearResolved()
+	c.Assert(err, jc.ErrorIsNil)
+	mode = s.apiUnit.Resolved()
+	c.Assert(mode, gc.Equals, params.ResolvedRetryHooks)
+
+	err = s.apiUnit.Refresh()
+	c.Assert(err, jc.ErrorIsNil)
+	mode = s.apiUnit.Resolved()
+	c.Assert(mode, gc.Equals, params.ResolvedNone)
+}
+
+func (s *unitSuite) TestRefreshSeries(c *gc.C) {
+	c.Assert(s.apiUnit.Series(), gc.Equals, "quantal")
+	err := s.wordpressMachine.UpdateMachineSeries("xenial", true)
+	c.Assert(err, gc.IsNil)
+	c.Assert(s.apiUnit.Series(), gc.Equals, "quantal")
+
+	err = s.apiUnit.Refresh()
+	c.Assert(err, jc.ErrorIsNil)
+	c.Assert(s.apiUnit.Series(), gc.Equals, "xenial")
+}
+
 func (s *unitSuite) TestWatch(c *gc.C) {
 	c.Assert(s.apiUnit.Life(), gc.Equals, params.Alive)
 
@@ -307,22 +338,6 @@ func (s *unitSuite) TestSubordinateWatchRelations(c *gc.C) {
 	wc.AssertNoChange()
 }
 
-func (s *unitSuite) TestResolve(c *gc.C) {
-	err := s.wordpressUnit.SetResolved(state.ResolvedRetryHooks)
-	c.Assert(err, jc.ErrorIsNil)
-
-	mode, err := s.apiUnit.Resolved()
-	c.Assert(err, jc.ErrorIsNil)
-	c.Assert(mode, gc.Equals, params.ResolvedRetryHooks)
-
-	err = s.apiUnit.ClearResolved()
-	c.Assert(err, jc.ErrorIsNil)
-
-	mode, err = s.apiUnit.Resolved()
-	c.Assert(err, jc.ErrorIsNil)
-	c.Assert(mode, gc.Equals, params.ResolvedNone)
-}
-
 func (s *unitSuite) TestAssignedMachine(c *gc.C) {
 	machineTag, err := s.apiUnit.AssignedMachine()
 	c.Assert(err, jc.ErrorIsNil)
@@ -378,44 +393,6 @@ func (s *unitSuite) TestPrivateAddress(c *gc.C) {
 	c.Assert(address, gc.Equals, "1.2.3.4")
 }
 
-func (s *unitSuite) TestNetworkConfig(c *gc.C) {
-	c.Skip("dimitern: temporarily disabled to pass a CI run until it can be fixed like its apiserver/uniter counterpart")
-
-	// Set some provider addresses bound to both "public" and "internal"
-	// spaces.
-	addresses := []network.Address{
-		network.NewAddressOnSpace("public", "8.8.8.8"),
-		network.NewAddressOnSpace("", "8.8.4.4"),
-		network.NewAddressOnSpace("internal", "10.0.0.1"),
-		network.NewAddressOnSpace("internal", "10.0.0.2"),
-		network.NewAddressOnSpace("public", "fc00::1"),
-	}
-	err := s.wordpressMachine.SetProviderAddresses(addresses...)
-	c.Assert(err, jc.ErrorIsNil)
-
-	netConfig, err := s.apiUnit.NetworkConfig("db") // relation name, bound to "internal"
-	c.Assert(err, jc.ErrorIsNil)
-	c.Assert(netConfig, jc.DeepEquals, []params.NetworkConfig{
-		{Address: "10.0.0.1"},
-		{Address: "10.0.0.2"},
-	})
-
-	netConfig, err = s.apiUnit.NetworkConfig("admin-api") // extra-binding name, bound to "public"
-	c.Assert(err, jc.ErrorIsNil)
-	c.Assert(netConfig, jc.DeepEquals, []params.NetworkConfig{
-		{Address: "8.8.8.8"},
-		{Address: "fc00::1"},
-	})
-
-	netConfig, err = s.apiUnit.NetworkConfig("unknown")
-	c.Assert(err, gc.ErrorMatches, `binding name "unknown" not defined by the unit's charm`)
-	c.Assert(netConfig, gc.IsNil)
-
-	netConfig, err = s.apiUnit.NetworkConfig("")
-	c.Assert(err, gc.ErrorMatches, "binding name cannot be empty")
-	c.Assert(netConfig, gc.IsNil)
-}
-
 func (s *unitSuite) TestAvailabilityZone(c *gc.C) {
 	uniter.PatchUnitResponse(s, s.apiUnit, "AvailabilityZone",
 		func(result interface{}) error {
@@ -489,6 +466,45 @@ func (s *unitSuite) TestGetSetCharmURL(c *gc.C) {
 	c.Assert(curl.String(), gc.Equals, s.wordpressCharm.String())
 }
 
+func (s *unitSuite) TestNetworkInfo(c *gc.C) {
+	var called int
+	relId := 2
+	apiCaller := testing.APICallerFunc(func(objType string, version int, id, request string, arg, result interface{}) error {
+		called++
+		if called == 1 {
+			*(result.(*params.UnitRefreshResults)) = params.UnitRefreshResults{
+				Results: []params.UnitRefreshResult{{Life: params.Alive, Resolved: params.ResolvedNone, Series: "quantal"}}}
+			return nil
+		}
+		c.Check(objType, gc.Equals, "Uniter")
+		c.Check(version, gc.Equals, expectedVersion)
+		c.Check(id, gc.Equals, "")
+		c.Check(request, gc.Equals, "NetworkInfo")
+		c.Check(arg, gc.DeepEquals, params.NetworkInfoParams{
+			Unit:       "unit-mysql-0",
+			Bindings:   []string{"server"},
+			RelationId: &relId,
+		})
+		c.Assert(result, gc.FitsTypeOf, &params.NetworkInfoResults{})
+		*(result.(*params.NetworkInfoResults)) = params.NetworkInfoResults{
+			Results: map[string]params.NetworkInfoResult{
+				"db": {
+					Error: &params.Error{Message: "FAIL"},
+				}},
+		}
+		return nil
+	})
+
+	ut := names.NewUnitTag("mysql/0")
+	st := uniter.NewState(apiCaller, ut)
+	unit, err := st.Unit(ut)
+	c.Assert(err, jc.ErrorIsNil)
+	result, err := unit.NetworkInfo([]string{"server"}, &relId)
+	c.Assert(err, jc.ErrorIsNil)
+	c.Assert(result["db"].Error, gc.ErrorMatches, "FAIL")
+	c.Assert(called, gc.Equals, 2)
+}
+
 func (s *unitSuite) TestConfigSettings(c *gc.C) {
 	// Make sure ConfigSettings returns an error when
 	// no charm URL is set, as its state counterpart does.
@@ -642,25 +658,32 @@ func (s *unitSuite) TestApplicationNameAndTag(c *gc.C) {
 	c.Assert(s.apiUnit.ApplicationTag(), gc.Equals, s.wordpressApplication.Tag())
 }
 
-func (s *unitSuite) TestJoinedRelations(c *gc.C) {
-	joinedRelations, err := s.apiUnit.JoinedRelations()
+func (s *unitSuite) TestRelationSuspended(c *gc.C) {
+	relationStatus, err := s.apiUnit.RelationsStatus()
 	c.Assert(err, jc.ErrorIsNil)
-	c.Assert(joinedRelations, gc.HasLen, 0)
+	c.Assert(relationStatus, gc.HasLen, 0)
 
 	rel1, _, _ := s.addRelatedApplication(c, "wordpress", "monitoring", s.wordpressUnit)
-	joinedRelations, err = s.apiUnit.JoinedRelations()
+	relationStatus, err = s.apiUnit.RelationsStatus()
 	c.Assert(err, jc.ErrorIsNil)
-	c.Assert(joinedRelations, gc.DeepEquals, []names.RelationTag{
-		rel1.Tag().(names.RelationTag),
-	})
+	c.Assert(relationStatus, gc.DeepEquals, []uniter.RelationStatus{{
+		Tag:       rel1.Tag().(names.RelationTag),
+		InScope:   true,
+		Suspended: false,
+	}})
 
-	rel2, _, _ := s.addRelatedApplication(c, "wordpress", "logging", s.wordpressUnit)
-	joinedRelations, err = s.apiUnit.JoinedRelations()
+	rel2 := s.addRelationSuspended(c, "wordpress", "logging", s.wordpressUnit)
+	relationStatus, err = s.apiUnit.RelationsStatus()
 	c.Assert(err, jc.ErrorIsNil)
-	c.Assert(joinedRelations, jc.SameContents, []names.RelationTag{
-		rel1.Tag().(names.RelationTag),
-		rel2.Tag().(names.RelationTag),
-	})
+	c.Assert(relationStatus, jc.SameContents, []uniter.RelationStatus{{
+		Tag:       rel1.Tag().(names.RelationTag),
+		InScope:   true,
+		Suspended: false,
+	}, {
+		Tag:       rel2.Tag().(names.RelationTag),
+		InScope:   false,
+		Suspended: true,
+	}})
 }
 
 func (s *unitSuite) TestWatchAddresses(c *gc.C) {
@@ -825,18 +848,8 @@ func (s *unitSuite) TestWatchMeterStatus(c *gc.C) {
 	wc.AssertOneChange()
 }
 
-func (s *unitSuite) patchNewState(
-	c *gc.C,
-	patchFunc func(_ base.APICaller, _ names.UnitTag) *uniter.State,
-) {
-	s.uniterSuite.patchNewState(c, patchFunc)
-	var err error
-	s.apiUnit, err = s.uniter.Unit(s.wordpressUnit.Tag().(names.UnitTag))
-	c.Assert(err, jc.ErrorIsNil)
-}
-
 type unitMetricBatchesSuite struct {
-	testing.JujuConnSuite
+	jujutesting.JujuConnSuite
 
 	st      api.Connection
 	uniter  *uniter.State
