@@ -7,13 +7,14 @@ import (
 	"fmt"
 
 	"github.com/juju/errors"
-	"gopkg.in/juju/charm.v6-unstable"
+	"gopkg.in/juju/charm.v6"
 	"gopkg.in/juju/names.v2"
 
 	"github.com/juju/juju/api/base"
 	"github.com/juju/juju/api/common"
 	apiwatcher "github.com/juju/juju/api/watcher"
 	"github.com/juju/juju/apiserver/params"
+	"github.com/juju/juju/core/relation"
 	"github.com/juju/juju/network"
 	"github.com/juju/juju/watcher"
 )
@@ -69,12 +70,12 @@ func newStateForVersionFn(version int) func(base.APICaller, names.UnitTag) *Stat
 	}
 }
 
-// newStateV6 creates a new client-side Uniter facade, version 6
-var newStateV6 = newStateForVersionFn(6)
+// newStateV7 creates a new client-side Uniter facade, version 7
+var newStateV7 = newStateForVersionFn(7)
 
 // NewState creates a new client-side Uniter facade.
 // Defined like this to allow patching during tests.
-var NewState = newStateV6
+var NewState = newStateV7
 
 // BestAPIVersion returns the API version that we were able to
 // determine is supported by both the client and the API Server.
@@ -89,7 +90,7 @@ func (st *State) Facade() base.FacadeCaller {
 
 // life requests the lifecycle of the given entity from the server.
 func (st *State) life(tag names.Tag) (params.Life, error) {
-	return common.Life(st.facade, tag)
+	return common.OneLife(st.facade, tag)
 }
 
 // relation requests relation information from the server.
@@ -114,6 +115,17 @@ func (st *State) relation(relationTag, unitTag names.Tag) (params.RelationResult
 	return result.Results[0], nil
 }
 
+func (st *State) setRelationStatus(id int, status relation.Status) error {
+	args := params.RelationStatusArgs{
+		Args: []params.RelationStatusArg{{RelationId: id, Status: params.RelationStatusValue(status)}},
+	}
+	var results params.ErrorResults
+	if err := st.facade.FacadeCall("SetRelationStatus", args, &results); err != nil {
+		return errors.Trace(err)
+	}
+	return results.OneError()
+}
+
 // getOneAction retrieves a single Action from the controller.
 func (st *State) getOneAction(tag *names.ActionTag) (params.ActionResult, error) {
 	nothing := params.ActionResult{}
@@ -145,15 +157,15 @@ func (st *State) getOneAction(tag *names.ActionTag) (params.ActionResult, error)
 
 // Unit provides access to methods of a state.Unit through the facade.
 func (st *State) Unit(tag names.UnitTag) (*Unit, error) {
-	life, err := st.life(tag)
+	unit := &Unit{
+		tag: tag,
+		st:  st,
+	}
+	err := unit.Refresh()
 	if err != nil {
 		return nil, err
 	}
-	return &Unit{
-		tag:  tag,
-		life: life,
-		st:   st,
-	}, nil
+	return unit, nil
 }
 
 // Application returns an application state by tag.
@@ -203,11 +215,12 @@ func (st *State) Relation(relationTag names.RelationTag) (*Relation, error) {
 		return nil, err
 	}
 	return &Relation{
-		id:       result.Id,
-		tag:      relationTag,
-		life:     result.Life,
-		st:       st,
-		otherApp: result.OtherApplication,
+		id:        result.Id,
+		tag:       relationTag,
+		life:      result.Life,
+		suspended: result.Suspended,
+		st:        st,
+		otherApp:  result.OtherApplication,
 	}, nil
 }
 
@@ -295,11 +308,12 @@ func (st *State) RelationById(id int) (*Relation, error) {
 	}
 	relationTag := names.NewRelationTag(result.Key)
 	return &Relation{
-		id:       result.Id,
-		tag:      relationTag,
-		life:     result.Life,
-		st:       st,
-		otherApp: result.OtherApplication,
+		id:        result.Id,
+		tag:       relationTag,
+		life:      result.Life,
+		suspended: result.Suspended,
+		st:        st,
+		otherApp:  result.OtherApplication,
 	}, nil
 }
 
