@@ -20,6 +20,7 @@ import (
 	"github.com/juju/juju/apiserver/facades/agent/presence"
 	"github.com/juju/juju/apiserver/observer"
 	"github.com/juju/juju/apiserver/params"
+	"github.com/juju/juju/core/auditlog"
 	"github.com/juju/juju/permission"
 	"github.com/juju/juju/rpc"
 	"github.com/juju/juju/rpc/rpcreflect"
@@ -51,12 +52,12 @@ func newAdminAPIV3(srv *Server, root *apiHandler, apiObserver observer.Observer)
 
 // Admin returns an object that provides API access to methods that can be
 // called even when not authenticated.
-func (r *admin) Admin(id string) (*admin, error) {
+func (a *admin) Admin(id string) (*admin, error) {
 	if id != "" {
 		// Safeguard id for possible future use.
 		return nil, common.ErrBadId
 	}
-	return r, nil
+	return a, nil
 }
 
 // Login logs in with the provided credentials.  All subsequent requests on the
@@ -135,7 +136,16 @@ func (a *admin) login(req params.LoginRequest, loginVersion int) (params.LoginRe
 		modelTag = a.root.model.Tag().String()
 	}
 
-	a.root.rpcConn.ServeRoot(apiRoot, serverError)
+	auditConfig := a.srv.GetAuditConfig()
+	auditRecorder, err := a.getAuditRecorder(req, authResult, auditConfig)
+	if err != nil {
+		return fail, errors.Trace(err)
+	}
+
+	recorderFactory := observer.NewRecorderFactory(
+		a.apiObserver, auditRecorder, auditConfig.CaptureAPIArgs)
+
+	a.root.rpcConn.ServeRoot(apiRoot, recorderFactory, serverError)
 	return params.LoginResult{
 		Servers:       params.FromNetworkHostsPorts(hostPorts),
 		ControllerTag: a.root.model.ControllerTag().String(),
@@ -147,6 +157,31 @@ func (a *admin) login(req params.LoginRequest, loginVersion int) (params.LoginRe
 	}, nil
 }
 
+func (a *admin) getAuditRecorder(req params.LoginRequest, authResult *authResult, cfg auditlog.Config) (*auditlog.Recorder, error) {
+	if !authResult.userLogin || !cfg.Enabled {
+		return nil, nil
+	}
+	// Wrap the audit logger in a filter that prevents us from logging
+	// lots of readonly conversations (like "juju status" requests).
+	filter := observer.MakeInterestingRequestFilter(cfg.ExcludeMethods)
+	result, err := auditlog.NewRecorder(
+		observer.NewAuditLogFilter(cfg.Target, filter),
+		a.srv.clock,
+		auditlog.ConversationArgs{
+			Who:          a.root.entity.Tag().Id(),
+			What:         req.CLIArgs,
+			ModelName:    a.root.model.Name(),
+			ModelUUID:    a.root.model.UUID(),
+			ConnectionID: a.root.connectionID,
+		},
+	)
+	if err != nil {
+		logger.Errorf("couldn't add login to audit log: %+v", err)
+		return nil, errors.Trace(err)
+	}
+	return result, nil
+}
+
 type authResult struct {
 	tag                    names.Tag // nil if external user login
 	anonymousLogin         bool
