@@ -14,8 +14,10 @@ import (
 
 	"github.com/juju/errors"
 	"github.com/juju/loggo"
+	jt "github.com/juju/testing"
 	jc "github.com/juju/testing/checkers"
 	"github.com/juju/utils"
+	"github.com/juju/utils/set"
 	gc "gopkg.in/check.v1"
 	"gopkg.in/juju/names.v2"
 	"gopkg.in/macaroon-bakery.v1/httpbakery"
@@ -27,12 +29,15 @@ import (
 	"github.com/juju/juju/apiserver/common"
 	"github.com/juju/juju/apiserver/facades/client/controller"
 	"github.com/juju/juju/apiserver/params"
+	servertesting "github.com/juju/juju/apiserver/testing"
 	"github.com/juju/juju/constraints"
+	"github.com/juju/juju/core/auditlog"
 	jujutesting "github.com/juju/juju/juju/testing"
 	"github.com/juju/juju/network"
 	"github.com/juju/juju/permission"
 	"github.com/juju/juju/rpc"
 	"github.com/juju/juju/state"
+	"github.com/juju/juju/state/multiwatcher"
 	coretesting "github.com/juju/juju/testing"
 	"github.com/juju/juju/testing/factory"
 )
@@ -41,12 +46,6 @@ type baseLoginSuite struct {
 	jujutesting.JujuConnSuite
 }
 
-type loginSuite struct {
-	baseLoginSuite
-}
-
-var _ = gc.Suite(&loginSuite{})
-
 func (s *baseLoginSuite) SetUpTest(c *gc.C) {
 	s.JujuConnSuite.SetUpTest(c)
 	loggo.GetLogger("juju.apiserver").SetLogLevel(loggo.TRACE)
@@ -62,6 +61,43 @@ func (s *baseLoginSuite) newMachineAndServer(c *gc.C) (*api.Info, *apiserver.Ser
 	return info, srv
 }
 
+func (s *baseLoginSuite) loginHostPorts(c *gc.C, info *api.Info) (connectedAddr string, hostPorts [][]network.HostPort) {
+	st, err := api.Open(info, fastDialOpts)
+	c.Assert(err, jc.ErrorIsNil)
+	defer st.Close()
+	return st.Addr(), st.APIHostPorts()
+}
+
+func (s *baseLoginSuite) addMachine(c *gc.C, job state.MachineJob) (*state.Machine, string) {
+	machine, err := s.State.AddMachine("quantal", job)
+	c.Assert(err, jc.ErrorIsNil)
+	password, err := utils.RandomPassword()
+	c.Assert(err, jc.ErrorIsNil)
+	err = machine.SetPassword(password)
+	c.Assert(err, jc.ErrorIsNil)
+	err = machine.SetProvisioned("foo", "fake_nonce", nil)
+	c.Assert(err, jc.ErrorIsNil)
+	return machine, password
+}
+
+func (s *baseLoginSuite) openAPIWithoutLogin(c *gc.C, info0 *api.Info) api.Connection {
+	info := *info0
+	info.Tag = nil
+	info.Password = ""
+	info.SkipLogin = true
+	info.Macaroons = nil
+	st, err := api.Open(&info, fastDialOpts)
+	c.Assert(err, jc.ErrorIsNil)
+	s.AddCleanup(func(*gc.C) { st.Close() })
+	return st
+}
+
+type loginSuite struct {
+	baseLoginSuite
+}
+
+var _ = gc.Suite(&loginSuite{})
+
 func (s *loginSuite) TestLoginWithInvalidTag(c *gc.C) {
 	info := s.APIInfo(c)
 	info.Tag = nil
@@ -162,6 +198,26 @@ func (s *loginSuite) TestLoginAsDeactivatedUser(c *gc.C) {
 	})
 }
 
+func (s *baseLoginSuite) runLoginSetsLogIdentifier(c *gc.C) {
+	info, srv := newServer(c, s.StatePool)
+	defer assertStop(c, srv)
+
+	machine, password := s.Factory.MakeMachineReturningPassword(
+		c, &factory.MachineParams{Nonce: "fake_nonce"})
+
+	info.Tag = machine.Tag()
+	info.Password = password
+	info.Nonce = "fake_nonce"
+
+	apiConn, err := api.Open(info, fastDialOpts)
+	c.Assert(err, jc.ErrorIsNil)
+	defer apiConn.Close()
+
+	apiMachine, err := apimachiner.NewState(apiConn).Machine(machine.MachineTag())
+	c.Assert(err, jc.ErrorIsNil)
+	c.Assert(apiMachine.Tag(), gc.Equals, machine.Tag())
+}
+
 func (s *loginSuite) TestLoginAddrs(c *gc.C) {
 	info, srv := s.newMachineAndServer(c)
 	defer assertStop(c, srv)
@@ -210,13 +266,6 @@ func (s *loginSuite) TestLoginAddrs(c *gc.C) {
 	c.Assert(hostPorts, gc.DeepEquals, stateAPIHostPorts)
 }
 
-func (s *baseLoginSuite) loginHostPorts(c *gc.C, info *api.Info) (connectedAddr string, hostPorts [][]network.HostPort) {
-	st, err := api.Open(info, fastDialOpts)
-	c.Assert(err, jc.ErrorIsNil)
-	defer st.Close()
-	return st.Addr(), st.APIHostPorts()
-}
-
 func startNLogins(c *gc.C, n int, info *api.Info) (chan error, *sync.WaitGroup) {
 	errResults := make(chan error, 100)
 	var doneWG sync.WaitGroup
@@ -541,30 +590,6 @@ func (s *loginSuite) TestControllerMachineLoginDuringMaintenance(c *gc.C) {
 	c.Assert(st.Close(), jc.ErrorIsNil)
 }
 
-func (s *baseLoginSuite) addMachine(c *gc.C, job state.MachineJob) (*state.Machine, string) {
-	machine, err := s.State.AddMachine("quantal", job)
-	c.Assert(err, jc.ErrorIsNil)
-	password, err := utils.RandomPassword()
-	c.Assert(err, jc.ErrorIsNil)
-	err = machine.SetPassword(password)
-	c.Assert(err, jc.ErrorIsNil)
-	err = machine.SetProvisioned("foo", "fake_nonce", nil)
-	c.Assert(err, jc.ErrorIsNil)
-	return machine, password
-}
-
-func (s *baseLoginSuite) openAPIWithoutLogin(c *gc.C, info0 *api.Info) api.Connection {
-	info := *info0
-	info.Tag = nil
-	info.Password = ""
-	info.SkipLogin = true
-	info.Macaroons = nil
-	st, err := api.Open(&info, fastDialOpts)
-	c.Assert(err, jc.ErrorIsNil)
-	s.AddCleanup(func(*gc.C) { st.Close() })
-	return st
-}
-
 func (s *loginSuite) TestAnonymousModelLogin(c *gc.C) {
 	info, srv := newServer(c, s.StatePool)
 	defer assertStop(c, srv)
@@ -922,6 +947,171 @@ func (s *loginSuite) TestLoginUpdatesLastLoginAndConnection(c *gc.C) {
 	c.Assert(when.After(startTime), jc.IsTrue)
 }
 
+func (s *loginSuite) TestLoginAddsAuditConversationEventually(c *gc.C) {
+	log := &servertesting.FakeAuditLog{}
+	cfg := defaultServerConfig(c)
+	cfg.AuditConfig.Enabled = true
+	cfg.AuditConfig.Target = log
+	cfg.Clock = jt.NewClock(cfg.Clock.Now())
+	info, srv := newServerWithConfig(c, s.StatePool, cfg)
+	defer assertStop(c, srv)
+	info.ModelTag = s.IAASModel.Tag().(names.ModelTag)
+
+	password := "shhh..."
+	user := s.Factory.MakeUser(c, &factory.UserParams{
+		Password: password,
+	})
+	conn := s.openAPIWithoutLogin(c, info)
+
+	var result params.LoginResult
+	request := &params.LoginRequest{
+		AuthTag:     user.Tag().String(),
+		Credentials: password,
+		CLIArgs:     "hey you guys",
+	}
+	err := conn.APICall("Admin", 3, "", "Login", request, &result)
+	c.Assert(err, jc.ErrorIsNil)
+	c.Assert(result.UserInfo, gc.NotNil)
+	// Nothing's logged at this point because there haven't been any
+	// interesting requests.
+	log.CheckCallNames(c)
+
+	var addResults params.AddMachinesResults
+	addReq := &params.AddMachines{
+		MachineParams: []params.AddMachineParams{{
+			Jobs: []multiwatcher.MachineJob{"JobHostUnits"},
+		}},
+	}
+	err = conn.APICall("Client", 1, "", "AddMachines", addReq, &addResults)
+	c.Assert(err, jc.ErrorIsNil)
+
+	log.CheckCallNames(c, "AddConversation", "AddRequest", "AddResponse")
+
+	convo := log.Calls()[0].Args[0].(auditlog.Conversation)
+	c.Assert(convo.ConversationID, gc.HasLen, 16)
+	// Blank out unknown fields.
+	convo.ConversationID = "0123456789abcdef"
+	convo.ConnectionID = "something"
+	c.Assert(convo, gc.Equals, auditlog.Conversation{
+		Who:            user.Tag().Id(),
+		What:           "hey you guys",
+		When:           cfg.Clock.Now().Format(time.RFC3339),
+		ModelName:      s.IAASModel.Name(),
+		ModelUUID:      s.IAASModel.UUID(),
+		ConnectionID:   "something",
+		ConversationID: "0123456789abcdef",
+	})
+
+	auditReq := log.Calls()[1].Args[0].(auditlog.Request)
+	auditReq.ConversationID = ""
+	auditReq.ConnectionID = ""
+	auditReq.RequestID = 0
+	c.Assert(auditReq, gc.Equals, auditlog.Request{
+		When:    cfg.Clock.Now().Format(time.RFC3339),
+		Facade:  "Client",
+		Method:  "AddMachines",
+		Version: 1,
+	})
+}
+
+func (s *loginSuite) TestAuditLoggingFailureOnInterestingRequest(c *gc.C) {
+	log := &servertesting.FakeAuditLog{}
+	log.SetErrors(errors.Errorf("bad news bears"))
+	cfg := defaultServerConfig(c)
+	cfg.AuditConfig.Enabled = true
+	cfg.AuditConfig.Target = log
+	info, srv := newServerWithConfig(c, s.StatePool, cfg)
+	defer assertStop(c, srv)
+
+	info.ModelTag = s.IAASModel.Tag().(names.ModelTag)
+
+	password := "shhh..."
+	user := s.Factory.MakeUser(c, &factory.UserParams{
+		Password: password,
+	})
+	conn := s.openAPIWithoutLogin(c, info)
+
+	var result params.LoginResult
+	request := &params.LoginRequest{
+		AuthTag:     user.Tag().String(),
+		Credentials: password,
+		CLIArgs:     "hey you guys",
+	}
+	err := conn.APICall("Admin", 3, "", "Login", request, &result)
+	// No error yet since logging the conversation is deferred until
+	// something happens.
+	c.Assert(err, jc.ErrorIsNil)
+
+	var addResults params.AddMachinesResults
+	addReq := &params.AddMachines{
+		MachineParams: []params.AddMachineParams{{
+			Jobs: []multiwatcher.MachineJob{"JobHostUnits"},
+		}},
+	}
+	err = conn.APICall("Client", 1, "", "AddMachines", addReq, &addResults)
+	c.Assert(err, gc.ErrorMatches, "bad news bears")
+}
+
+func (s *loginSuite) TestAuditLoggingUsesExcludeMethods(c *gc.C) {
+	log := &servertesting.FakeAuditLog{}
+	cfg := defaultServerConfig(c)
+	cfg.AuditConfig.Enabled = true
+	cfg.AuditConfig.ExcludeMethods = set.NewStrings("Client.AddMachines")
+	cfg.AuditConfig.Target = log
+	info, srv := newServerWithConfig(c, s.StatePool, cfg)
+	defer assertStop(c, srv)
+	info.ModelTag = s.IAASModel.Tag().(names.ModelTag)
+
+	password := "shhh..."
+	user := s.Factory.MakeUser(c, &factory.UserParams{
+		Password: password,
+	})
+	conn := s.openAPIWithoutLogin(c, info)
+
+	var result params.LoginResult
+	request := &params.LoginRequest{
+		AuthTag:     user.Tag().String(),
+		Credentials: password,
+		CLIArgs:     "hey you guys",
+	}
+	err := conn.APICall("Admin", 3, "", "Login", request, &result)
+	c.Assert(err, jc.ErrorIsNil)
+	c.Assert(result.UserInfo, gc.NotNil)
+	// Nothing's logged at this point because there haven't been any
+	// interesting requests.
+	log.CheckCallNames(c)
+
+	var addResults params.AddMachinesResults
+	addReq := &params.AddMachines{
+		MachineParams: []params.AddMachineParams{{
+			Jobs: []multiwatcher.MachineJob{"JobHostUnits"},
+		}},
+	}
+	err = conn.APICall("Client", 1, "", "AddMachines", addReq, &addResults)
+	c.Assert(err, jc.ErrorIsNil)
+
+	// Still nothing logged - the AddMachines call has been filtered out.
+	log.CheckCallNames(c)
+
+	// Call something else.
+	destroyReq := &params.DestroyMachines{
+		MachineNames: []string{addResults.Machines[0].Machine},
+	}
+	err = conn.APICall("Client", 1, "", "DestroyMachines", destroyReq, nil)
+	c.Assert(err, jc.ErrorIsNil)
+
+	// Now the conversation and both requests are logged.
+	log.CheckCallNames(c, "AddConversation", "AddRequest", "AddResponse", "AddRequest", "AddResponse")
+
+	req1 := log.Calls()[1].Args[0].(auditlog.Request)
+	c.Assert(req1.Facade, gc.Equals, "Client")
+	c.Assert(req1.Method, gc.Equals, "AddMachines")
+
+	req2 := log.Calls()[3].Args[0].(auditlog.Request)
+	c.Assert(req2.Facade, gc.Equals, "Client")
+	c.Assert(req2.Method, gc.Equals, "DestroyMachines")
+}
+
 var _ = gc.Suite(&macaroonLoginSuite{})
 
 type macaroonLoginSuite struct {
@@ -1251,7 +1441,7 @@ func (s *migrationSuite) TestExportingModel(c *gc.C) {
 }
 
 type loginV3Suite struct {
-	loginSuite
+	baseLoginSuite
 }
 
 var _ = gc.Suite(&loginV3Suite{})
