@@ -40,6 +40,7 @@ import (
 	"github.com/juju/juju/apiserver/observer"
 	"github.com/juju/juju/apiserver/params"
 	"github.com/juju/juju/apiserver/websocket"
+	"github.com/juju/juju/core/auditlog"
 	"github.com/juju/juju/resource"
 	"github.com/juju/juju/resource/resourceadapters"
 	"github.com/juju/juju/rpc"
@@ -51,30 +52,18 @@ var logger = loggo.GetLogger("juju.apiserver")
 
 var defaultHTTPMethods = []string{"GET", "POST", "HEAD", "PUT", "DELETE", "OPTIONS"}
 
-// These vars define how we rate limit incoming connections.
-const (
-	defaultLoginRateLimit         = 10 // concurrent login operations
-	defaultLoginMinPause          = 100 * time.Millisecond
-	defaultLoginMaxPause          = 1 * time.Second
-	defaultLoginRetryPause        = 5 * time.Second
-	defaultConnMinPause           = 0 * time.Millisecond
-	defaultConnMaxPause           = 5 * time.Second
-	defaultConnLookbackWindow     = 1 * time.Second
-	defaultConnLowerThreshold     = 1000   // connections per second
-	defaultConnUpperThreshold     = 100000 // connections per second
-	defaultLogSinkRateLimitBurst  = 1000
-	defaultLogSinkRateLimitRefill = time.Millisecond
-)
-
 // Server holds the server side of the API.
 type Server struct {
-	tomb                   tomb.Tomb
-	clock                  clock.Clock
-	pingClock              clock.Clock
-	wg                     sync.WaitGroup
-	statePool              *state.StatePool
-	lis                    net.Listener
-	tag                    names.Tag
+	tomb      tomb.Tomb
+	clock     clock.Clock
+	pingClock clock.Clock
+	wg        sync.WaitGroup
+	statePool *state.StatePool
+	lis       net.Listener
+
+	// Tag of the machine where the API server is running.
+	tag names.Tag
+
 	dataDir                string
 	logDir                 string
 	limiter                utils.Limiter
@@ -95,6 +84,8 @@ type Server struct {
 	logSinkWriter          io.WriteCloser
 	logsinkRateLimitConfig logsink.RateLimitConfig
 	dbloggers              dbloggers
+	auditConfigChanged     <-chan auditlog.Config
+	currentAuditConfig     auditlog.Config
 	upgradeComplete        func() bool
 	restoreStatus          func() state.RestoreStatus
 
@@ -180,6 +171,14 @@ type ServerConfig struct {
 	// DefaultLogSinkConfig() will be used.
 	LogSinkConfig *LogSinkConfig
 
+	// AuditConfig controls whether and how we track user-initiated
+	// API requests for auditing purposes.
+	AuditConfig auditlog.Config
+
+	// AuditConfigChanged is a channel which reports the new audit
+	// configuration whenever it changes.
+	AuditConfigChanged <-chan auditlog.Config
+
 	// PrometheusRegisterer registers Prometheus collectors.
 	PrometheusRegisterer prometheus.Registerer
 }
@@ -209,6 +208,9 @@ func (c ServerConfig) Validate() error {
 			return errors.Annotate(err, "validating logsink configuration")
 		}
 	}
+	if err := c.AuditConfig.Validate(); err != nil {
+		return errors.Annotatef(err, "validating audit log configuration")
+	}
 	return nil
 }
 
@@ -219,110 +221,6 @@ func (c ServerConfig) pingClock() clock.Clock {
 	return c.PingClock
 }
 
-// RateLimitConfig holds parameters to control
-// aspects of rate limiting connections and logins.
-type RateLimitConfig struct {
-	LoginRateLimit     int
-	LoginMinPause      time.Duration
-	LoginMaxPause      time.Duration
-	LoginRetryPause    time.Duration
-	ConnMinPause       time.Duration
-	ConnMaxPause       time.Duration
-	ConnLookbackWindow time.Duration
-	ConnLowerThreshold int
-	ConnUpperThreshold int
-}
-
-// DefaultRateLimitConfig returns a RateLimtConfig struct with
-// all attributes set to their default values.
-func DefaultRateLimitConfig() RateLimitConfig {
-	return RateLimitConfig{
-		LoginRateLimit:     defaultLoginRateLimit,
-		LoginMinPause:      defaultLoginMinPause,
-		LoginMaxPause:      defaultLoginMaxPause,
-		LoginRetryPause:    defaultLoginRetryPause,
-		ConnMinPause:       defaultConnMinPause,
-		ConnMaxPause:       defaultConnMaxPause,
-		ConnLookbackWindow: defaultConnLookbackWindow,
-		ConnLowerThreshold: defaultConnLowerThreshold,
-		ConnUpperThreshold: defaultConnUpperThreshold,
-	}
-}
-
-// Validate validates the rate limit configuration.
-// We apply arbitrary but sensible upper limits to prevent
-// typos from introducing obviously bad config.
-func (c RateLimitConfig) Validate() error {
-	if c.LoginRateLimit <= 0 || c.LoginRateLimit > 100 {
-		return errors.NotValidf("login-rate-limit %d <= 0 or > 100", c.LoginRateLimit)
-	}
-	if c.LoginMinPause < 0 || c.LoginMinPause > 100*time.Millisecond {
-		return errors.NotValidf("login-min-pause %d < 0 or > 100ms", c.LoginMinPause)
-	}
-	if c.LoginMaxPause < 0 || c.LoginMaxPause > 5*time.Second {
-		return errors.NotValidf("login-max-pause %d < 0 or > 5s", c.LoginMaxPause)
-	}
-	if c.LoginRetryPause < 0 || c.LoginRetryPause > 10*time.Second {
-		return errors.NotValidf("login-retry-pause %d < 0 or > 10s", c.LoginRetryPause)
-	}
-	if c.ConnMinPause < 0 || c.ConnMinPause > 100*time.Millisecond {
-		return errors.NotValidf("conn-min-pause %d < 0 or > 100ms", c.ConnMinPause)
-	}
-	if c.ConnMaxPause < 0 || c.ConnMaxPause > 10*time.Second {
-		return errors.NotValidf("conn-max-pause %d < 0 or > 10s", c.ConnMaxPause)
-	}
-	if c.ConnLookbackWindow < 0 || c.ConnLookbackWindow > 5*time.Second {
-		return errors.NotValidf("conn-lookback-window %d < 0 or > 5s", c.ConnMaxPause)
-	}
-	return nil
-}
-
-// LogSinkConfig holds parameters to control the API server's
-// logsink endpoint behaviour.
-type LogSinkConfig struct {
-	// DBLoggerBufferSize is the capacity of the database logger's buffer.
-	DBLoggerBufferSize int
-
-	// DBLoggerFlushInterval is the amount of time to allow a log record
-	// to sit in the buffer before being flushed to the database.
-	DBLoggerFlushInterval time.Duration
-
-	// RateLimitBurst defines the number of log messages that will be let
-	// through before we start rate limiting.
-	RateLimitBurst int64
-
-	// RateLimitRefill defines the rate at which log messages will be let
-	// through once the initial burst amount has been depleted.
-	RateLimitRefill time.Duration
-}
-
-// Validate validates the logsink endpoint configuration.
-func (cfg LogSinkConfig) Validate() error {
-	if cfg.DBLoggerBufferSize <= 0 || cfg.DBLoggerBufferSize > 1000 {
-		return errors.NotValidf("DBLoggerBufferSize %d <= 0 or > 1000", cfg.DBLoggerBufferSize)
-	}
-	if cfg.DBLoggerFlushInterval <= 0 || cfg.DBLoggerFlushInterval > 10*time.Second {
-		return errors.NotValidf("DBLoggerFlushInterval %s <= 0 or > 10 seconds", cfg.DBLoggerFlushInterval)
-	}
-	if cfg.RateLimitBurst <= 0 {
-		return errors.NotValidf("RateLimitBurst %d <= 0", cfg.RateLimitBurst)
-	}
-	if cfg.RateLimitRefill <= 0 {
-		return errors.NotValidf("RateLimitRefill %s <= 0", cfg.RateLimitRefill)
-	}
-	return nil
-}
-
-// DefaultLogSinkConfig returns a LogSinkConfig with default values.
-func DefaultLogSinkConfig() LogSinkConfig {
-	return LogSinkConfig{
-		DBLoggerBufferSize:    defaultDBLoggerBufferSize,
-		DBLoggerFlushInterval: defaultDBLoggerFlushInterval,
-		RateLimitBurst:        defaultLogSinkRateLimitBurst,
-		RateLimitRefill:       defaultLogSinkRateLimitRefill,
-	}
-}
-
 // NewServer serves the given state by accepting requests on the given
 // listener, using the given certificate and key (in PEM format) for
 // authentication.
@@ -383,6 +281,8 @@ func newServer(stPool *state.StatePool, lis net.Listener, cfg ServerConfig) (_ *
 			Burst:  cfg.LogSinkConfig.RateLimitBurst,
 			Clock:  cfg.Clock,
 		},
+		auditConfigChanged: cfg.AuditConfigChanged,
+		currentAuditConfig: cfg.AuditConfig,
 		dbloggers: dbloggers{
 			clock:                 cfg.Clock,
 			dbLoggerBufferSize:    cfg.LogSinkConfig.DBLoggerBufferSize,
@@ -581,6 +481,12 @@ func (srv *Server) run() {
 		srv.tomb.Kill(srv.processModelRemovals())
 	}()
 
+	srv.wg.Add(1)
+	go func() {
+		defer srv.wg.Done()
+		srv.tomb.Kill(srv.processAuditConfigChanges())
+	}()
+
 	// for pat based handlers, they are matched in-order of being
 	// registered, first match wins. So more specific ones have to be
 	// registered first.
@@ -914,15 +820,29 @@ func (srv *Server) apiHandler(w http.ResponseWriter, req *http.Request) {
 	websocket.Serve(w, req, func(conn *websocket.Conn) {
 		modelUUID := req.URL.Query().Get(":modeluuid")
 		logger.Tracef("got a request for model %q", modelUUID)
-		if err := srv.serveConn(conn, modelUUID, apiObserver, req.Host); err != nil {
+		if err := srv.serveConn(
+			conn,
+			modelUUID,
+			connectionID,
+			apiObserver,
+			req.Host,
+		); err != nil {
 			logger.Errorf("error serving RPCs: %v", err)
 		}
 	})
 }
 
-func (srv *Server) serveConn(wsConn *websocket.Conn, modelUUID string, apiObserver observer.Observer, host string) error {
+func (srv *Server) serveConn(
+	wsConn *websocket.Conn,
+	modelUUID string,
+	connectionID uint64,
+	apiObserver observer.Observer,
+	host string,
+) error {
 	codec := jsoncodec.NewWebsocket(wsConn.Conn)
-	conn := rpc.NewConn(codec, apiObserver)
+	recorderFactory := observer.NewRecorderFactory(
+		apiObserver, nil, observer.NoCaptureArgs)
+	conn := rpc.NewConn(codec, recorderFactory)
 
 	// Note that we don't overwrite modelUUID here because
 	// newAPIHandler treats an empty modelUUID as signifying
@@ -942,11 +862,11 @@ func (srv *Server) serveConn(wsConn *websocket.Conn, modelUUID string, apiObserv
 
 	if err == nil {
 		defer releaser()
-		h, err = newAPIHandler(srv, st, conn, modelUUID, host)
+		h, err = newAPIHandler(srv, st, conn, modelUUID, connectionID, host)
 	}
 
 	if err != nil {
-		conn.ServeRoot(&errRoot{errors.Trace(err)}, serverError)
+		conn.ServeRoot(&errRoot{errors.Trace(err)}, recorderFactory, serverError)
 	} else {
 		// Set up the admin apis used to accept logins and direct
 		// requests to the relevant business facade.
@@ -956,7 +876,7 @@ func (srv *Server) serveConn(wsConn *websocket.Conn, modelUUID string, apiObserv
 		for apiVersion, factory := range adminAPIFactories {
 			adminAPIs[apiVersion] = factory(srv, h, apiObserver)
 		}
-		conn.ServeRoot(newAdminRoot(h, adminAPIs), serverError)
+		conn.ServeRoot(newAdminRoot(h, adminAPIs), recorderFactory, serverError)
 	}
 	conn.Start()
 	select {
@@ -1096,3 +1016,32 @@ func (srv *Server) processModelRemovals() error {
 		}
 	}
 }
+
+func (srv *Server) processAuditConfigChanges() error {
+	for {
+		select {
+		case <-srv.tomb.Dying():
+			return tomb.ErrDying
+		case auditConfig := <-srv.auditConfigChanged:
+			if err := auditConfig.Validate(); err != nil {
+				logger.Warningf("discarding invalid audit config: %s", err)
+			} else {
+				srv.updateAuditConfig(auditConfig)
+			}
+		}
+	}
+}
+
+func (srv *Server) updateAuditConfig(newConfig auditlog.Config) {
+	srv.mu.Lock()
+	defer srv.mu.Unlock()
+	srv.currentAuditConfig = newConfig
+}
+
+// GetAuditConfig returns a copy of the current audit logging
+// configuration.
+func (srv *Server) GetAuditConfig() auditlog.Config {
+	srv.mu.Lock()
+	defer srv.mu.Unlock()
+	return srv.currentAuditConfig
+}
