@@ -21,7 +21,7 @@ import (
 	"gopkg.in/macaroon-bakery.v1/httpbakery"
 
 	"github.com/juju/juju/apiserver"
-	apiserverbackups "github.com/juju/juju/apiserver/backups"
+	apiserverbackups "github.com/juju/juju/apiserver/facades/client/backups"
 	"github.com/juju/juju/apiserver/params"
 	"github.com/juju/juju/state"
 	"github.com/juju/juju/state/backups"
@@ -38,7 +38,7 @@ func (s *backupsCommonSuite) SetUpTest(c *gc.C) {
 
 	s.fake = &backupstesting.FakeBackups{}
 	s.PatchValue(apiserver.NewBackups,
-		func(st *state.State) (backups.Backups, io.Closer) {
+		func(st *state.State, m *state.Model) (backups.Backups, io.Closer) {
 			return s.fake, ioutil.NopCloser(nil)
 		},
 	)
