@@ -4,8 +4,6 @@
 package common
 
 import (
-	"gopkg.in/juju/names.v2"
-
 	"github.com/juju/juju/environs"
 	"github.com/juju/juju/environs/config"
 )
@@ -14,7 +12,7 @@ import (
 // in a pluggable way.
 type EnvironConfigGetterFuncs struct {
 	ModelConfigFunc func() (*config.Config, error)
-	CloudSpecFunc   func(names.ModelTag) (environs.CloudSpec, error)
+	CloudSpecFunc   func() (environs.CloudSpec, error)
 }
 
 // ModelConfig implements EnvironConfigGetter.
@@ -23,6 +21,6 @@ func (f EnvironConfigGetterFuncs) ModelConfig() (*config.Config, error) {
 }
 
 // CloudSpec implements environs.EnvironConfigGetter.
-func (f EnvironConfigGetterFuncs) CloudSpec(model names.ModelTag) (environs.CloudSpec, error) {
-	return f.CloudSpecFunc(model)
+func (f EnvironConfigGetterFuncs) CloudSpec() (environs.CloudSpec, error) {
+	return f.CloudSpecFunc()
 }
