@@ -6,7 +6,6 @@ package common
 import (
 	"fmt"
 
-	"github.com/juju/errors"
 	"gopkg.in/juju/names.v2"
 
 	"github.com/juju/juju/apiserver/params"
@@ -14,14 +13,6 @@ import (
 	"github.com/juju/juju/status"
 )
 
-// ErrIsNotLeader is an error for operations that require for a
-// unit to be the leader but it was not the case.
-// TODO(fwereade) why do we have an alternative implementation of ErrPerm
-// that is exported (implying people will be able to meaningfully check it)
-// but not actually handled anywhere or converted into an error code by the
-// api server?
-var ErrIsNotLeader = errors.Errorf("this unit is not the leader")
-
 // StatusGetter implements a common Status method for use by
 // various facades.
 type StatusGetter struct {
@@ -84,26 +75,26 @@ func (s *StatusGetter) Status(args params.Entities) (params.StatusResults, error
 	return result, nil
 }
 
-// ServiceStatusGetter is a StatusGetter for combined service and unit statuses.
+// ApplicationStatusGetter is a StatusGetter for combined application and unit statuses.
 // TODO(fwereade) this is completely evil and should never have been created.
 // We have a perfectly adequate StatusGetter already, that accepts bulk args;
 // all this does is break the user model, break the api model, and lie about
 // unit statuses).
-type ServiceStatusGetter struct {
+type ApplicationStatusGetter struct {
 	st           *state.State
 	getCanAccess GetAuthFunc
 }
 
-// NewServiceStatusGetter returns a ServiceStatusGetter.
-func NewServiceStatusGetter(st *state.State, getCanAccess GetAuthFunc) *ServiceStatusGetter {
-	return &ServiceStatusGetter{
+// NewApplicationStatusGetter returns a ApplicationStatusGetter.
+func NewApplicationStatusGetter(st *state.State, getCanAccess GetAuthFunc) *ApplicationStatusGetter {
+	return &ApplicationStatusGetter{
 		st:           st,
 		getCanAccess: getCanAccess,
 	}
 }
 
-// Status returns the status of the Service for each given Unit tag.
-func (s *ServiceStatusGetter) Status(args params.Entities) (params.ApplicationStatusResults, error) {
+// Status returns the status of the Application for each given Unit tag.
+func (s *ApplicationStatusGetter) Status(args params.Entities) (params.ApplicationStatusResults, error) {
 	result := params.ApplicationStatusResults{
 		Results: make([]params.ApplicationStatusResult, len(args.Entities)),
 	}
@@ -117,11 +108,11 @@ func (s *ServiceStatusGetter) Status(args params.Entities) (params.ApplicationSt
 		// works by coincidence (and is happening at the wrong layer anyway).
 		// Read carefully.
 
-		// We "know" that arg.Tag is either the calling unit or its service
-		// (because getCanAccess is authUnitOrService, and we'll fail out if
+		// We "know" that arg.Tag is either the calling unit or its application
+		// (because getCanAccess is authUnitOrApplication, and we'll fail out if
 		// it isn't); and, in practice, it's always going to be the calling
-		// unit (because, /sigh, we don't actually use service tags to refer
-		// to services in this method).
+		// unit (because, /sigh, we don't actually use application tags to refer
+		// to applications in this method).
 		tag, err := names.ParseTag(arg.Tag)
 		if err != nil {
 			result.Results[i].Error = ServerError(err)
@@ -140,22 +131,22 @@ func (s *ServiceStatusGetter) Status(args params.Entities) (params.ApplicationSt
 		}
 		unitId := unitTag.Id()
 
-		// Now we have the unit, we can get the service that should have been
+		// Now we have the unit, we can get the application that should have been
 		// specified in the first place...
-		serviceId, err := names.UnitApplication(unitId)
+		applicationId, err := names.UnitApplication(unitId)
 		if err != nil {
 			result.Results[i].Error = ServerError(err)
 			continue
 		}
-		service, err := s.st.Application(serviceId)
+		application, err := s.st.Application(applicationId)
 		if err != nil {
 			result.Results[i].Error = ServerError(err)
 			continue
 		}
 
-		// ...so we can check the unit's service leadership...
+		// ...so we can check the unit's application leadership...
 		checker := s.st.LeadershipChecker()
-		token := checker.LeadershipCheck(serviceId, unitId)
+		token := checker.LeadershipCheck(applicationId, unitId)
 		if err := token.Check(nil); err != nil {
 			// TODO(fwereade) this should probably be ErrPerm is certain cases,
 			// but I don't think I implemented an exported ErrNotLeader. I
@@ -165,16 +156,16 @@ func (s *ServiceStatusGetter) Status(args params.Entities) (params.ApplicationSt
 		}
 
 		// ...and collect the results.
-		serviceStatus, unitStatuses, err := service.ServiceAndUnitsStatus()
+		applicationStatus, unitStatuses, err := application.ApplicationAndUnitsStatus()
 		if err != nil {
 			result.Results[i].Application.Error = ServerError(err)
 			result.Results[i].Error = ServerError(err)
 			continue
 		}
-		result.Results[i].Application.Status = serviceStatus.Status.String()
-		result.Results[i].Application.Info = serviceStatus.Message
-		result.Results[i].Application.Data = serviceStatus.Data
-		result.Results[i].Application.Since = serviceStatus.Since
+		result.Results[i].Application.Status = applicationStatus.Status.String()
+		result.Results[i].Application.Info = applicationStatus.Message
+		result.Results[i].Application.Data = applicationStatus.Data
+		result.Results[i].Application.Since = applicationStatus.Since
 
 		result.Results[i].Units = make(map[string]params.StatusResult, len(unitStatuses))
 		for uTag, r := range unitStatuses {
