@@ -7,6 +7,7 @@ import (
 	"time"
 
 	"github.com/juju/description"
+	"github.com/juju/errors"
 	"gopkg.in/juju/names.v2"
 
 	"github.com/juju/juju/controller"
@@ -28,28 +29,32 @@ type ModelManagerBackend interface {
 	state.CloudAccessor
 
 	ModelUUID() string
-	ModelsForUser(names.UserTag) ([]*state.UserModel, error)
+	ModelUUIDsForUser(names.UserTag) ([]string, error)
+	ModelBasicInfoForUser(user names.UserTag) ([]state.ModelAccessInfo, error)
+	ModelSummariesForUser(user names.UserTag, all bool) ([]state.ModelSummary, error)
 	IsControllerAdmin(user names.UserTag) (bool, error)
 	NewModel(state.ModelArgs) (Model, ModelManagerBackend, error)
+	Model() (Model, error)
+	AllModelUUIDs() ([]string, error)
+	GetModel(string) (Model, func() bool, error)
+	GetBackend(string) (ModelManagerBackend, func() bool, error)
 
 	ComposeNewModelConfig(modelAttr map[string]interface{}, regionSpec *environs.RegionSpec) (map[string]interface{}, error)
-	ControllerModel() (Model, error)
+	ControllerModelUUID() string
+	ControllerModelTag() names.ModelTag
 	ControllerConfig() (controller.Config, error)
-	ForModel(tag names.ModelTag) (ModelManagerBackend, error)
-	GetModel(names.ModelTag) (Model, error)
-	Model() (Model, error)
 	ModelConfigDefaultValues() (config.ModelDefaultAttributes, error)
 	UpdateModelConfigDefaultValues(update map[string]interface{}, remove []string, regionSpec *environs.RegionSpec) error
 	Unit(name string) (*state.Unit, error)
 	ModelTag() names.ModelTag
 	ModelConfig() (*config.Config, error)
-	AllModels() ([]Model, error)
-	AddModelUser(string, state.UserAccessSpec) (permission.UserAccess, error)
 	AddControllerUser(state.UserAccessSpec) (permission.UserAccess, error)
 	RemoveUserAccess(names.UserTag, names.Tag) error
 	UserAccess(names.UserTag, names.Tag) (permission.UserAccess, error)
 	AllMachines() (machines []Machine, err error)
 	AllApplications() (applications []Application, err error)
+	AllFilesystems() ([]state.Filesystem, error)
+	AllVolumes() ([]state.Volume, error)
 	ControllerUUID() string
 	ControllerTag() names.ControllerTag
 	Export() (description.Model, error)
@@ -57,7 +62,6 @@ type ModelManagerBackend interface {
 	SetUserAccess(subject names.UserTag, target names.Tag, access permission.Access) (permission.UserAccess, error)
 	SetModelMeterStatus(string, string) error
 	ReloadSpaces(environ environs.Environ) error
-	LastModelConnection(user names.UserTag) (time.Time, error)
 	LatestMigration() (state.ModelMigration, error)
 	DumpAll() (map[string]interface{}, error)
 	Close() error
@@ -84,85 +88,84 @@ type Model interface {
 	CloudCredential() (names.CloudCredentialTag, bool)
 	CloudRegion() string
 	Users() ([]permission.UserAccess, error)
-	Destroy() error
-	DestroyIncludingHosted() error
+	Destroy(state.DestroyModelParams) error
 	SLALevel() string
 	SLAOwner() string
 	MigrationMode() state.MigrationMode
 	Name() string
 	UUID() string
 	ControllerUUID() string
+	LastModelConnection(user names.UserTag) (time.Time, error)
+	AddUser(state.UserAccessSpec) (permission.UserAccess, error)
+	AutoConfigureContainerNetworking(environ environs.Environ) error
+	ModelConfigDefaultValues() (config.ModelDefaultAttributes, error)
 }
 
 var _ ModelManagerBackend = (*modelManagerStateShim)(nil)
 
 type modelManagerStateShim struct {
 	*state.State
+	model *state.Model
+	pool  *state.StatePool
 }
 
 // NewModelManagerBackend returns a modelManagerStateShim wrapping the passed
 // state, which implements ModelManagerBackend.
-func NewModelManagerBackend(st *state.State) ModelManagerBackend {
-	return modelManagerStateShim{st}
-}
-
-// ControllerModel implements ModelManagerBackend.
-func (st modelManagerStateShim) ControllerModel() (Model, error) {
-	m, err := st.State.ControllerModel()
-	if err != nil {
-		return nil, err
-	}
-	return modelShim{m}, nil
+func NewModelManagerBackend(m *state.Model, pool *state.StatePool) ModelManagerBackend {
+	return modelManagerStateShim{m.State(), m, pool}
 }
 
 // NewModel implements ModelManagerBackend.
 func (st modelManagerStateShim) NewModel(args state.ModelArgs) (Model, ModelManagerBackend, error) {
-	m, otherState, err := st.State.NewModel(args)
+	otherModel, otherState, err := st.State.NewModel(args)
 	if err != nil {
 		return nil, nil, err
 	}
-	return modelShim{m}, modelManagerStateShim{otherState}, nil
+	return modelShim{otherModel}, modelManagerStateShim{otherState, otherModel, st.pool}, nil
 }
 
-// ForModel implements ModelManagerBackend.
-func (st modelManagerStateShim) ForModel(tag names.ModelTag) (ModelManagerBackend, error) {
-	otherState, err := st.State.ForModel(tag)
+func (st modelManagerStateShim) ModelConfigDefaultValues() (config.ModelDefaultAttributes, error) {
+	return st.model.ModelConfigDefaultValues()
+}
+
+// UpdateModelConfigDefaultValues implements the ModelManagerBackend method.
+func (st modelManagerStateShim) UpdateModelConfigDefaultValues(update map[string]interface{}, remove []string, regionSpec *environs.RegionSpec) error {
+	return st.model.UpdateModelConfigDefaultValues(update, remove, regionSpec)
+}
+
+// ControllerTag exposes Model ControllerTag for ModelManagerBackend inteface
+func (st modelManagerStateShim) ControllerTag() names.ControllerTag {
+	return st.model.ControllerTag()
+}
+
+// GetBackend implements ModelManagerBackend.
+func (st modelManagerStateShim) GetBackend(modelUUID string) (ModelManagerBackend, func() bool, error) {
+	otherState, release, err := st.pool.Get(modelUUID)
 	if err != nil {
-		return nil, err
+		return nil, nil, errors.Trace(err)
 	}
-	return modelManagerStateShim{otherState}, nil
+	otherModel, err := otherState.Model()
+	if err != nil {
+		return nil, nil, err
+	}
+	return modelManagerStateShim{otherState, otherModel, st.pool}, release, nil
 }
 
 // GetModel implements ModelManagerBackend.
-func (st modelManagerStateShim) GetModel(tag names.ModelTag) (Model, error) {
-	m, err := st.State.GetModel(tag)
+func (st modelManagerStateShim) GetModel(modelUUID string) (Model, func() bool, error) {
+	model, release, err := st.pool.GetModel(modelUUID)
 	if err != nil {
-		return nil, err
+		return nil, nil, errors.Trace(err)
 	}
-	return modelShim{m}, nil
+	return modelShim{model}, release, nil
 }
 
 // Model implements ModelManagerBackend.
 func (st modelManagerStateShim) Model() (Model, error) {
-	m, err := st.State.Model()
-	if err != nil {
-		return nil, err
-	}
-	return modelShim{m}, nil
+	return modelShim{st.model}, nil
 }
 
-// AllModels implements ModelManagerBackend.
-func (st modelManagerStateShim) AllModels() ([]Model, error) {
-	allStateModels, err := st.State.AllModels()
-	if err != nil {
-		return nil, err
-	}
-	all := make([]Model, len(allStateModels))
-	for i, m := range allStateModels {
-		all[i] = modelShim{m}
-	}
-	return all, nil
-}
+var _ Model = (*modelShim)(nil)
 
 type modelShim struct {
 	*state.Model
@@ -216,25 +219,37 @@ func (st modelManagerStateShim) AllApplications() ([]Application, error) {
 	return all, nil
 }
 
-// BackendPool provides access to a pool of ModelManagerBackends.
-type BackendPool interface {
-	Get(modelUUID string) (ModelManagerBackend, func(), error)
-}
-
-// NewBackendPool returns a BackendPool wrapping the passed StatePool.
-func NewBackendPool(pool *state.StatePool) BackendPool {
-	return &statePoolShim{pool: pool}
-}
-
-type statePoolShim struct {
-	pool *state.StatePool
-}
-
-// Get implements BackendPool.
-func (p *statePoolShim) Get(modelUUID string) (ModelManagerBackend, func(), error) {
-	st, releaser, err := p.pool.Get(modelUUID)
-	closer := func() {
-		releaser()
+func (st modelManagerStateShim) AllFilesystems() ([]state.Filesystem, error) {
+	model, err := st.State.IAASModel()
+	if err != nil {
+		return nil, err
 	}
-	return NewModelManagerBackend(st), closer, err
+	return model.AllFilesystems()
+}
+
+func (st modelManagerStateShim) AllVolumes() ([]state.Volume, error) {
+	model, err := st.State.IAASModel()
+	if err != nil {
+		return nil, err
+	}
+	return model.AllVolumes()
+}
+
+// ModelConfig returns the underlying model's config. Exposed here to satisfy the
+// ModelBackend interface.
+func (st modelManagerStateShim) ModelConfig() (*config.Config, error) {
+	model, err := st.State.Model()
+	if err != nil {
+		return nil, errors.Trace(err)
+	}
+	return model.ModelConfig()
+}
+
+// [TODO: Eric Claude Jones] This method ignores an error for the purpose of
+// expediting refactoring for CAAS features (we are avoiding changing method
+// signatures so that refactoring doesn't spiral out of control). This method
+// should be deleted immediately upon the removal of the ModelTag method from
+// state.State.
+func (st modelManagerStateShim) ModelTag() names.ModelTag {
+	return names.NewModelTag(st.ModelUUID())
 }
