@@ -9,6 +9,7 @@ import (
 
 	"github.com/juju/juju/apiserver/facade"
 	"github.com/juju/juju/apiserver/params"
+	"github.com/juju/juju/state"
 )
 
 // ModelStatusAPI implements the ModelStatus() API.
@@ -16,34 +17,30 @@ type ModelStatusAPI struct {
 	authorizer facade.Authorizer
 	apiUser    names.UserTag
 	backend    ModelManagerBackend
-	pool       BackendPool
 }
 
 // NewModelStatusAPI creates an implementation providing the ModelStatus() API.
-func NewModelStatusAPI(st ModelManagerBackend, pool BackendPool, authorizer facade.Authorizer, apiUser names.UserTag) *ModelStatusAPI {
+func NewModelStatusAPI(backend ModelManagerBackend, authorizer facade.Authorizer, apiUser names.UserTag) *ModelStatusAPI {
 	return &ModelStatusAPI{
 		authorizer: authorizer,
 		apiUser:    apiUser,
-		backend:    st,
-		pool:       pool,
+		backend:    backend,
 	}
 }
 
 // ModelStatus returns a summary of the model.
 func (c *ModelStatusAPI) ModelStatus(req params.Entities) (params.ModelStatusResults, error) {
 	models := req.Entities
-	results := params.ModelStatusResults{}
-
 	status := make([]params.ModelStatus, len(models))
 	for i, model := range models {
 		modelStatus, err := c.modelStatus(model.Tag)
 		if err != nil {
-			return results, errors.Trace(err)
+			status[i].Error = ServerError(err)
+			continue
 		}
 		status[i] = modelStatus
 	}
-	results.Results = status
-	return results, nil
+	return params.ModelStatusResults{Results: status}, nil
 }
 
 func (c *ModelStatusAPI) modelStatus(tag string) (params.ModelStatus, error) {
@@ -54,11 +51,12 @@ func (c *ModelStatusAPI) modelStatus(tag string) (params.ModelStatus, error) {
 	}
 	st := c.backend
 	if modelTag != c.backend.ModelTag() {
-		var releaser func()
-		if st, releaser, err = c.pool.Get(modelTag.Id()); err != nil {
+		otherSt, releaser, err := c.backend.GetBackend(modelTag.Id())
+		if err != nil {
 			return status, errors.Trace(err)
 		}
 		defer releaser()
+		st = otherSt
 	}
 
 	model, err := st.Model()
@@ -95,6 +93,18 @@ func (c *ModelStatusAPI) modelStatus(tag string) (params.ModelStatus, error) {
 		return status, errors.Trace(err)
 	}
 
+	volumes, err := st.AllVolumes()
+	if err != nil {
+		return status, errors.Trace(err)
+	}
+	modelVolumes := ModelVolumeInfo(volumes)
+
+	filesystems, err := st.AllFilesystems()
+	if err != nil {
+		return status, errors.Trace(err)
+	}
+	modelFilesystems := ModelFilesystemInfo(filesystems)
+
 	return params.ModelStatus{
 		ModelTag:           tag,
 		OwnerTag:           model.Owner().String(),
@@ -102,5 +112,57 @@ func (c *ModelStatusAPI) modelStatus(tag string) (params.ModelStatus, error) {
 		HostedMachineCount: len(hostedMachines),
 		ApplicationCount:   len(applications),
 		Machines:           modelMachines,
+		Volumes:            modelVolumes,
+		Filesystems:        modelFilesystems,
 	}, nil
 }
+
+// ModelFilesystemInfo returns information about filesystems in the model.
+func ModelFilesystemInfo(in []state.Filesystem) []params.ModelFilesystemInfo {
+	out := make([]params.ModelFilesystemInfo, len(in))
+	for i, in := range in {
+		var statusString string
+		status, err := in.Status()
+		if err != nil {
+			statusString = err.Error()
+		} else {
+			statusString = string(status.Status)
+		}
+		var providerId string
+		if info, err := in.Info(); err == nil {
+			providerId = info.FilesystemId
+		}
+		out[i] = params.ModelFilesystemInfo{
+			Id:         in.Tag().Id(),
+			ProviderId: providerId,
+			Status:     statusString,
+			Detachable: in.Detachable(),
+		}
+	}
+	return out
+}
+
+// ModelVolumeInfo returns information about volumes in the model.
+func ModelVolumeInfo(in []state.Volume) []params.ModelVolumeInfo {
+	out := make([]params.ModelVolumeInfo, len(in))
+	for i, in := range in {
+		var statusString string
+		status, err := in.Status()
+		if err != nil {
+			statusString = err.Error()
+		} else {
+			statusString = string(status.Status)
+		}
+		var providerId string
+		if info, err := in.Info(); err == nil {
+			providerId = info.VolumeId
+		}
+		out[i] = params.ModelVolumeInfo{
+			Id:         in.Tag().Id(),
+			ProviderId: providerId,
+			Status:     statusString,
+			Detachable: in.Detachable(),
+		}
+	}
+	return out
+}
