@@ -5,6 +5,7 @@ package networkingcommon
 
 import (
 	"github.com/juju/errors"
+	"gopkg.in/juju/names.v2"
 
 	"github.com/juju/juju/apiserver/params"
 	"github.com/juju/juju/network"
@@ -82,15 +83,22 @@ func (s *spaceShim) Subnets() ([]BackingSubnet, error) {
 	return subnets, nil
 }
 
-func NewStateShim(st *state.State) *stateShim {
-	return &stateShim{stateenvirons.EnvironConfigGetter{st}, st}
+func NewStateShim(st *state.State) (*stateShim, error) {
+	m, err := st.Model()
+	if err != nil {
+		return nil, errors.Trace(err)
+	}
+	return &stateShim{stateenvirons.EnvironConfigGetter{st, m}, st, m}, nil
 }
 
 // stateShim forwards and adapts state.State methods to Backing
 // method.
+// TODO - CAAS(ericclaudejones): This should contain state and stateenvirons alone, model will be
+// removed once all relevant methods are moved from state to model.
 type stateShim struct {
 	stateenvirons.EnvironConfigGetter
 	st *state.State
+	m  *state.Model
 }
 
 func (s *stateShim) AddSpace(name string, providerId network.Id, subnetIds []string, public bool) error {
@@ -149,3 +157,7 @@ func (s *stateShim) AvailabilityZones() ([]providercommon.AvailabilityZone, erro
 func (s *stateShim) SetAvailabilityZones(zones []providercommon.AvailabilityZone) error {
 	return nil
 }
+
+func (s *stateShim) ModelTag() names.ModelTag {
+	return s.m.ModelTag()
+}
