@@ -187,6 +187,7 @@ func NetworkConfigFromInterfaceInfo(interfaceInfos []network.InterfaceInfo) []pa
 			DNSSearchDomains:    v.DNSSearchDomains,
 			GatewayAddress:      v.GatewayAddress.Value,
 			Routes:              routes,
+			IsDefaultGateway:    v.IsDefaultGateway,
 		}
 	}
 	return result
@@ -267,6 +268,7 @@ func NetworkConfigsToStateArgs(networkConfig []params.NetworkConfig) (
 			DNSServers:       netConfig.DNSServers,
 			DNSSearchDomains: netConfig.DNSSearchDomains,
 			GatewayAddress:   netConfig.GatewayAddress,
+			IsDefaultGateway: netConfig.IsDefaultGateway,
 		}
 		logger.Tracef("state address args for device: %+v", addr)
 		devicesAddrs = append(devicesAddrs, addr)
@@ -369,6 +371,7 @@ func networkConfigsByAddress(input []params.NetworkConfig) map[string]params.Net
 }
 
 func mergeObservedAndProviderInterfaceConfig(observedConfig, providerConfig params.NetworkConfig) params.NetworkConfig {
+	logger.Debugf("mergeObservedAndProviderInterfaceConfig %+v %+v", observedConfig, providerConfig)
 	finalConfig := observedConfig
 
 	// The following fields cannot be observed and are only known by the
@@ -391,6 +394,7 @@ func mergeObservedAndProviderInterfaceConfig(observedConfig, providerConfig para
 	if observedConfig.ParentInterfaceName == "" {
 		finalConfig.ParentInterfaceName = providerConfig.ParentInterfaceName
 	}
+	logger.Debugf("mergeObservedAndProviderInterfaceConfig %+v", finalConfig)
 
 	return finalConfig
 }
@@ -455,7 +459,7 @@ func networkToParamsNetworkInfo(info network.NetworkInfo) params.NetworkInfo {
 
 func MachineNetworkInfoResultToNetworkInfoResult(inResult state.MachineNetworkInfoResult) params.NetworkInfoResult {
 	if inResult.Error != nil {
-		return params.NetworkInfoResult{Error: common.ServerError(*inResult.Error)}
+		return params.NetworkInfoResult{Error: common.ServerError(inResult.Error)}
 	}
 	infos := make([]params.NetworkInfo, len(inResult.NetworkInfos))
 	for i, info := range inResult.NetworkInfos {
@@ -465,3 +469,28 @@ func MachineNetworkInfoResultToNetworkInfoResult(inResult state.MachineNetworkIn
 		Info: infos,
 	}
 }
+
+func FanConfigToFanConfigResult(config network.FanConfig) params.FanConfigResult {
+	result := params.FanConfigResult{make([]params.FanConfigEntry, len(config))}
+	for i, entry := range config {
+		result.Fans[i] = params.FanConfigEntry{entry.Underlay.String(), entry.Overlay.String()}
+	}
+	return result
+}
+
+func FanConfigResultToFanConfig(config params.FanConfigResult) (network.FanConfig, error) {
+	rv := make(network.FanConfig, len(config.Fans))
+	for i, entry := range config.Fans {
+		_, ipnet, err := net.ParseCIDR(entry.Underlay)
+		if err != nil {
+			return nil, err
+		}
+		rv[i].Underlay = ipnet
+		_, ipnet, err = net.ParseCIDR(entry.Overlay)
+		if err != nil {
+			return nil, err
+		}
+		rv[i].Overlay = ipnet
+	}
+	return rv, nil
+}
