@@ -14,17 +14,6 @@ import (
 	"github.com/juju/juju/storage/poolmanager"
 )
 
-type volumeAlreadyProvisionedError struct {
-	error
-}
-
-// IsVolumeAlreadyProvisioned returns true if the specified error
-// is caused by a volume already being provisioned.
-func IsVolumeAlreadyProvisioned(err error) bool {
-	_, ok := err.(*volumeAlreadyProvisionedError)
-	return ok
-}
-
 // VolumeParams returns the parameters for creating or destroying
 // the given volume.
 func VolumeParams(
