@@ -21,6 +21,7 @@ import (
 	apiservertesting "github.com/juju/juju/apiserver/testing"
 	"github.com/juju/juju/constraints"
 	"github.com/juju/juju/container"
+	"github.com/juju/juju/environs/config"
 	"github.com/juju/juju/instance"
 	"github.com/juju/juju/juju/testing"
 	"github.com/juju/juju/network"
@@ -56,9 +57,10 @@ func (s *provisionerSuite) SetUpTest(c *gc.C) {
 }
 
 func (s *provisionerSuite) setUpTest(c *gc.C, withController bool) {
-	s.JujuConnSuite.ConfigAttrs = map[string]interface{}{
-		"image-stream": "daily",
+	if s.JujuConnSuite.ConfigAttrs == nil {
+		s.JujuConnSuite.ConfigAttrs = make(map[string]interface{})
 	}
+	s.JujuConnSuite.ConfigAttrs["image-stream"] = "daily"
 	s.JujuConnSuite.SetUpTest(c)
 
 	// Reset previous machines (if any) and create 3 machines
@@ -916,7 +918,7 @@ func (s *withoutControllerSuite) TestDistributionGroup(c *gc.C) {
 	setProvisioned("3")
 
 	// Add a few controllers, provision two of them.
-	_, err = s.State.EnableHA(3, constraints.Value{}, "quantal", nil)
+	_, err = s.State.EnableHA(3, constraints.Value{}, "quantal", nil, "")
 	c.Assert(err, jc.ErrorIsNil)
 	setProvisioned("5")
 	setProvisioned("7")
@@ -1041,7 +1043,7 @@ func (s *withoutControllerSuite) TestDistributionGroupByMachineId(c *gc.C) {
 	setProvisioned("3")
 
 	// Add a few controllers, provision two of them.
-	_, err = s.State.EnableHA(3, constraints.Value{}, "quantal", nil)
+	_, err = s.State.EnableHA(3, constraints.Value{}, "quantal", nil, "")
 	c.Assert(err, jc.ErrorIsNil)
 	setProvisioned("5")
 	setProvisioned("7")
@@ -1342,19 +1344,45 @@ func (s *provisionerSuite) getManagerConfig(c *gc.C, typ instance.ContainerType)
 	return results.ManagerConfig
 }
 
-func (s *withoutControllerSuite) TestContainerManagerConfig(c *gc.C) {
+func (s *withoutControllerSuite) TestContainerManagerConfigDefaults(c *gc.C) {
 	cfg := s.getManagerConfig(c, instance.KVM)
 	c.Assert(cfg, jc.DeepEquals, map[string]string{
-		container.ConfigModelUUID: coretesting.ModelTag.Id(),
+		container.ConfigModelUUID:      coretesting.ModelTag.Id(),
+		config.ContainerImageStreamKey: "released",
+	})
+}
+
+type withImageMetadataSuite struct {
+	provisionerSuite
+}
+
+var _ = gc.Suite(&withImageMetadataSuite{})
+
+func (s *withImageMetadataSuite) SetUpTest(c *gc.C) {
+	s.ConfigAttrs = map[string]interface{}{
+		config.ContainerImageStreamKey:      "daily",
+		config.ContainerImageMetadataURLKey: "https://images.linuxcontainers.org/",
+	}
+	s.setUpTest(c, false)
+}
+
+func (s *withImageMetadataSuite) TestContainerManagerConfigImageMetadata(c *gc.C) {
+	cfg := s.getManagerConfig(c, instance.LXD)
+	c.Assert(cfg, jc.DeepEquals, map[string]string{
+		container.ConfigModelUUID:           coretesting.ModelTag.Id(),
+		config.ContainerImageStreamKey:      "daily",
+		config.ContainerImageMetadataURLKey: "https://images.linuxcontainers.org/",
 	})
 }
 
 func (s *withoutControllerSuite) TestContainerConfig(c *gc.C) {
 	attrs := map[string]interface{}{
-		"http-proxy":            "http://proxy.example.com:9000",
-		"apt-https-proxy":       "https://proxy.example.com:9000",
-		"allow-lxd-loop-mounts": true,
-		"apt-mirror":            "http://example.mirror.com",
+		"http-proxy":                   "http://proxy.example.com:9000",
+		"apt-https-proxy":              "https://proxy.example.com:9000",
+		"allow-lxd-loop-mounts":        true,
+		"apt-mirror":                   "http://example.mirror.com",
+		"cloudinit-userdata":           validCloudInitUserData,
+		"container-inherit-properties": "ca-certs,apt-primary",
 	}
 	err := s.Model.UpdateModelConfig(attrs, nil)
 	c.Assert(err, jc.ErrorIsNil)
@@ -1378,6 +1406,12 @@ func (s *withoutControllerSuite) TestContainerConfig(c *gc.C) {
 	c.Check(results.Proxy, gc.DeepEquals, expectedProxy)
 	c.Check(results.AptProxy, gc.DeepEquals, expectedAPTProxy)
 	c.Check(results.AptMirror, gc.DeepEquals, "http://example.mirror.com")
+	c.Check(results.CloudInitUserData, gc.DeepEquals, map[string]interface{}{
+		"packages":        []interface{}{"python-keystoneclient", "python-glanceclient"},
+		"preruncmd":       []interface{}{"mkdir /tmp/preruncmd", "mkdir /tmp/preruncmd2"},
+		"postruncmd":      []interface{}{"mkdir /tmp/postruncmd", "mkdir /tmp/postruncmd2"},
+		"package_upgrade": false})
+	c.Check(results.ContainerInheritProperties, gc.DeepEquals, "ca-certs,apt-primary")
 }
 
 func (s *withoutControllerSuite) TestSetSupportedContainers(c *gc.C) {
