@@ -13,7 +13,6 @@ import (
 	"github.com/juju/errors"
 	"github.com/juju/loggo"
 	jc "github.com/juju/testing/checkers"
-	"github.com/juju/utils/series"
 	"github.com/juju/version"
 	gc "gopkg.in/check.v1"
 	"gopkg.in/juju/charm.v6"
@@ -32,6 +31,7 @@ import (
 	"github.com/juju/juju/environs/manual/sshprovisioner"
 	toolstesting "github.com/juju/juju/environs/tools/testing"
 	"github.com/juju/juju/instance"
+	supportedversion "github.com/juju/juju/juju/version"
 	"github.com/juju/juju/network"
 	"github.com/juju/juju/permission"
 	"github.com/juju/juju/rpc"
@@ -563,7 +563,7 @@ func (s *clientSuite) TestForceDestroyMachines(c *gc.C) {
 	s.assertForceDestroyMachines(c)
 }
 
-func (s *clientSuite) testClientUnitResolved(c *gc.C, noretry bool, expectedResolvedMode state.ResolvedMode) {
+func (s *clientSuite) testClientUnitResolved(c *gc.C, retry bool, expectedResolvedMode state.ResolvedMode) {
 	// Setup:
 	s.setUpScenario(c)
 	u, err := s.State.Unit("wordpress/0")
@@ -577,7 +577,7 @@ func (s *clientSuite) testClientUnitResolved(c *gc.C, noretry bool, expectedReso
 	err = u.SetAgentStatus(sInfo)
 	c.Assert(err, jc.ErrorIsNil)
 	// Code under test:
-	err = s.APIState.Client().Resolved("wordpress/0", noretry)
+	err = s.APIState.Client().Resolved("wordpress/0", retry)
 	c.Assert(err, jc.ErrorIsNil)
 	// Freshen the unit's state.
 	err = u.Refresh()
@@ -589,11 +589,11 @@ func (s *clientSuite) testClientUnitResolved(c *gc.C, noretry bool, expectedReso
 }
 
 func (s *clientSuite) TestClientUnitResolved(c *gc.C) {
-	s.testClientUnitResolved(c, true, state.ResolvedNoHooks)
+	s.testClientUnitResolved(c, false, state.ResolvedNoHooks)
 }
 
 func (s *clientSuite) TestClientUnitResolvedRetry(c *gc.C) {
-	s.testClientUnitResolved(c, false, state.ResolvedRetryHooks)
+	s.testClientUnitResolved(c, true, state.ResolvedRetryHooks)
 }
 
 func (s *clientSuite) setupResolved(c *gc.C) *state.Unit {
@@ -612,7 +612,7 @@ func (s *clientSuite) setupResolved(c *gc.C) *state.Unit {
 }
 
 func (s *clientSuite) assertResolved(c *gc.C, u *state.Unit) {
-	err := s.APIState.Client().Resolved("wordpress/0", false)
+	err := s.APIState.Client().Resolved("wordpress/0", true)
 	c.Assert(err, jc.ErrorIsNil)
 	// Freshen the unit's state.
 	err = u.Refresh()
@@ -1017,7 +1017,7 @@ func (s *clientSuite) TestClientAddMachinesDefaultSeries(c *gc.C) {
 	c.Assert(len(machines), gc.Equals, 3)
 	for i, machineResult := range machines {
 		c.Assert(machineResult.Machine, gc.DeepEquals, strconv.Itoa(i))
-		s.checkMachine(c, machineResult.Machine, series.LatestLts(), apiParams[i].Constraints.String())
+		s.checkMachine(c, machineResult.Machine, supportedversion.SupportedLTS(), apiParams[i].Constraints.String())
 	}
 }
 
@@ -1033,7 +1033,7 @@ func (s *clientSuite) assertAddMachines(c *gc.C) {
 	c.Assert(len(machines), gc.Equals, 3)
 	for i, machineResult := range machines {
 		c.Assert(machineResult.Machine, gc.DeepEquals, strconv.Itoa(i))
-		s.checkMachine(c, machineResult.Machine, series.LatestLts(), apiParams[i].Constraints.String())
+		s.checkMachine(c, machineResult.Machine, supportedversion.SupportedLTS(), apiParams[i].Constraints.String())
 	}
 }
 
@@ -1109,7 +1109,7 @@ func (s *clientSuite) TestClientAddMachinesWithConstraints(c *gc.C) {
 	c.Assert(len(machines), gc.Equals, 3)
 	for i, machineResult := range machines {
 		c.Assert(machineResult.Machine, gc.DeepEquals, strconv.Itoa(i))
-		s.checkMachine(c, machineResult.Machine, series.LatestLts(), apiParams[i].Constraints.String())
+		s.checkMachine(c, machineResult.Machine, supportedversion.SupportedLTS(), apiParams[i].Constraints.String())
 	}
 }
 
@@ -1199,7 +1199,7 @@ func (s *clientSuite) TestClientAddMachinesWithInstanceIdSomeErrors(c *gc.C) {
 			c.Assert(machineResult.Error, gc.ErrorMatches, "cannot add a new machine: cannot add a machine with an instance id and no nonce")
 		} else {
 			c.Assert(machineResult.Machine, gc.DeepEquals, strconv.Itoa(i))
-			s.checkMachine(c, machineResult.Machine, series.LatestLts(), apiParams[i].Constraints.String())
+			s.checkMachine(c, machineResult.Machine, supportedversion.SupportedLTS(), apiParams[i].Constraints.String())
 			instanceId := fmt.Sprintf("1234-%d", i)
 			s.checkInstance(c, machineResult.Machine, instanceId, "foo", hc, addrs)
 		}
@@ -1378,9 +1378,10 @@ func (s *clientRepoSuite) TestResolveCharm(c *gc.C) {
 		url:        "cs:no-such",
 		resolveErr: `cannot resolve URL "cs:no-such": charm or bundle not found`,
 	}, {
-		about:    "invalid charm name",
-		url:      "cs:",
-		parseErr: `cannot parse URL "cs://": name "" not valid`,
+		about: "invalid charm name",
+		url:   "cs:",
+		// go-1.9 replaces 'cs:' with 'cs://', but not go-1.10
+		parseErr: `cannot parse URL "cs:/*": name "" not valid`,
 	}, {
 		about:      "local charm",
 		url:        "local:wordpress",
