@@ -75,8 +75,10 @@ const (
 	CodeDead                      = "dead"
 	CodeHasAssignedUnits          = "machine has assigned units"
 	CodeHasHostedModels           = "controller has hosted models"
+	CodeHasPersistentStorage      = "controller/model has persistent storage"
 	CodeModelNotEmpty             = "model not empty"
 	CodeMachineHasAttachedStorage = "machine has attached storage"
+	CodeStorageAttached           = "storage is attached"
 	CodeNotProvisioned            = "not provisioned"
 	CodeNoAddressSet              = "no address set"
 	CodeTryAgain                  = "try again"
@@ -95,6 +97,7 @@ const (
 	CodeDischargeRequired         = "macaroon discharge required"
 	CodeRedirect                  = "redirection required"
 	CodeRetry                     = "retry"
+	CodeIncompatibleSeries        = "incompatible series"
 )
 
 // ErrCode returns the error code associated with
@@ -185,6 +188,10 @@ func IsCodeHasHostedModels(err error) bool {
 	return ErrCode(err) == CodeHasHostedModels
 }
 
+func IsCodeHasPersistentStorage(err error) bool {
+	return ErrCode(err) == CodeHasPersistentStorage
+}
+
 func IsCodeModelNotEmpty(err error) bool {
 	return ErrCode(err) == CodeModelNotEmpty
 }
@@ -193,6 +200,10 @@ func IsCodeMachineHasAttachedStorage(err error) bool {
 	return ErrCode(err) == CodeMachineHasAttachedStorage
 }
 
+func IsCodeStorageAttached(err error) bool {
+	return ErrCode(err) == CodeStorageAttached
+}
+
 func IsCodeNotProvisioned(err error) bool {
 	return ErrCode(err) == CodeNotProvisioned
 }
@@ -244,3 +255,11 @@ func IsMethodNotAllowed(err error) bool {
 func IsRedirect(err error) bool {
 	return ErrCode(err) == CodeRedirect
 }
+
+func IsCodeIncompatibleSeries(err error) bool {
+	return ErrCode(err) == CodeIncompatibleSeries
+}
+
+func IsCodeForbidden(err error) bool {
+	return ErrCode(err) == CodeForbidden
+}
