@@ -689,7 +689,8 @@ type NetworkInfo struct {
 	Addresses []InterfaceAddress `json:"addresses"`
 }
 
-// NetworkInfoResult holds either and error or a list of NetworkInfos for given binding.
+// NetworkInfoResult Adds egress and ingress subnets and changes the serialized
+// `Info` key name in the yaml/json API protocol.
 type NetworkInfoResult struct {
 	Error            *Error        `json:"error,omitempty" yaml:"error,omitempty"`
 	Info             []NetworkInfo `json:"bind-addresses,omitempty" yaml:"bind-addresses,omitempty"`
@@ -702,6 +703,17 @@ type NetworkInfoResults struct {
 	Results map[string]NetworkInfoResult `json:"results"`
 }
 
+// NetworkInfoResultV6 holds either and error or a list of NetworkInfos for given binding.
+type NetworkInfoResultV6 struct {
+	Error *Error        `json:"error,omitempty" yaml:"error,omitempty"`
+	Info  []NetworkInfo `json:"network-info" yaml:"info"`
+}
+
+// NetworkInfoResults holds a mapping from binding name to NetworkInfoResultV6.
+type NetworkInfoResultsV6 struct {
+	Results map[string]NetworkInfoResultV6 `json:"results"`
+}
+
 // NetworkInfoParams holds a name of the unit and list of bindings for which we want to get NetworkInfos.
 type NetworkInfoParams struct {
 	Unit       string   `json:"unit"`
