@@ -529,6 +529,7 @@ type LoginRequest struct {
 	Credentials string           `json:"credentials"`
 	Nonce       string           `json:"nonce"`
 	Macaroons   []macaroon.Slice `json:"macaroons"`
+	CLIArgs     string           `json:"cli-args,omitempty"`
 	UserData    string           `json:"user-data"`
 }
 
@@ -668,12 +669,14 @@ type UpdateBehavior struct {
 // ContainerConfig contains information from the model config that is
 // needed for container cloud-init.
 type ContainerConfig struct {
-	ProviderType            string         `json:"provider-type"`
-	AuthorizedKeys          string         `json:"authorized-keys"`
-	SSLHostnameVerification bool           `json:"ssl-hostname-verification"`
-	Proxy                   proxy.Settings `json:"proxy"`
-	AptProxy                proxy.Settings `json:"apt-proxy"`
-	AptMirror               string         `json:"apt-mirror"`
+	ProviderType               string                 `json:"provider-type"`
+	AuthorizedKeys             string                 `json:"authorized-keys"`
+	SSLHostnameVerification    bool                   `json:"ssl-hostname-verification"`
+	Proxy                      proxy.Settings         `json:"proxy"`
+	AptProxy                   proxy.Settings         `json:"apt-proxy"`
+	AptMirror                  string                 `json:"apt-mirror"`
+	CloudInitUserData          map[string]interface{} `json:"cloudinit-userdata,omitempty"`
+	ContainerInheritProperties string                 `json:"container-inherit-properties,omitempty"`
 	*UpdateBehavior
 }
 
@@ -1011,7 +1014,7 @@ type ResumeReplicationParams struct {
 type MeterStatusParam struct {
 	Tag  string `json:"tag"`
 	Code string `json:"code"`
-	Info string `json:"info, omitempty"`
+	Info string `json:"info,omitempty"`
 }
 
 // MeterStatusParams holds parameters for making SetMeterStatus calls.
