@@ -77,11 +77,6 @@ func (s *toolsCommonSuite) assertUploadResponse(c *gc.C, resp *http.Response, ag
 	c.Check(toolsResponse.ToolsList, jc.DeepEquals, coretools.List{agentTools})
 }
 
-func (s *toolsCommonSuite) assertGetFileResponse(c *gc.C, resp *http.Response, expBody, expContentType string) {
-	body := assertResponse(c, resp, http.StatusOK, expContentType)
-	c.Check(string(body), gc.Equals, expBody)
-}
-
 func (s *toolsCommonSuite) assertErrorResponse(c *gc.C, resp *http.Response, expCode int, expError string) {
 	toolsResponse := s.assertResponse(c, resp, expCode)
 	c.Assert(toolsResponse.Error, gc.NotNil)
@@ -159,7 +154,7 @@ func (s *toolsSuite) TestUploadFailsWithNoTools(c *gc.C) {
 	c.Assert(err, jc.ErrorIsNil)
 
 	resp := s.uploadRequest(c, s.toolsURI(c, "?binaryVersion=1.18.0-quantal-amd64"), "application/x-tar-gz", tempFile.Name())
-	s.assertErrorResponse(c, resp, http.StatusBadRequest, "no tools uploaded")
+	s.assertErrorResponse(c, resp, http.StatusBadRequest, "no agent binaries uploaded")
 }
 
 func (s *toolsSuite) TestUploadFailsWithInvalidContentType(c *gc.C) {
@@ -461,7 +456,7 @@ func (s *toolsSuite) TestDownloadFetchesAndVerifiesSize(c *gc.C) {
 	c.Assert(err, jc.ErrorIsNil)
 
 	resp := s.downloadRequest(c, tools.Version, "")
-	s.assertErrorResponse(c, resp, http.StatusBadRequest, "error fetching tools: size mismatch for .*")
+	s.assertErrorResponse(c, resp, http.StatusBadRequest, "error fetching agent binaries: size mismatch for .*")
 	s.assertToolsNotStored(c, tools.Version.String())
 }
 
@@ -481,7 +476,7 @@ func (s *toolsSuite) TestDownloadFetchesAndVerifiesHash(c *gc.C) {
 	c.Assert(err, jc.ErrorIsNil)
 
 	resp := s.downloadRequest(c, tools.Version, "")
-	s.assertErrorResponse(c, resp, http.StatusBadRequest, "error fetching tools: hash mismatch for .*")
+	s.assertErrorResponse(c, resp, http.StatusBadRequest, "error fetching agent binaries: hash mismatch for .*")
 	s.assertToolsNotStored(c, tools.Version.String())
 }
 
