@@ -7,8 +7,10 @@ import (
 	"github.com/juju/errors"
 
 	"github.com/juju/juju/apiserver/common"
+	"github.com/juju/juju/apiserver/common/crossmodel"
 	"github.com/juju/juju/apiserver/common/storagecommon"
 	"github.com/juju/juju/apiserver/facade"
+	"github.com/juju/juju/apiserver/facades/controller/crossmodelrelations"
 	"github.com/juju/juju/apiserver/params"
 	"github.com/juju/juju/controller"
 	"github.com/juju/juju/core/migration"
@@ -99,7 +101,7 @@ func newNotifyWatcher(context facade.Context) (facade.Facade, error) {
 	auth := context.Auth()
 	resources := context.Resources()
 
-	if !isAgent(auth) {
+	if auth.GetAuthTag() != nil && !isAgent(auth) {
 		return nil, common.ErrPerm
 	}
 	watcher, ok := resources.Get(id).(state.NotifyWatcher)
@@ -140,7 +142,9 @@ func newStringsWatcher(context facade.Context) (facade.Facade, error) {
 	auth := context.Auth()
 	resources := context.Resources()
 
-	if !isAgent(auth) {
+	// TODO(wallyworld) - enhance this watcher to support
+	// anonymous api calls with macaroons.
+	if auth.GetAuthTag() != nil && !isAgent(auth) {
 		return nil, common.ErrPerm
 	}
 	watcher, ok := resources.Get(id).(state.StringsWatcher)
@@ -182,7 +186,9 @@ func newRelationUnitsWatcher(context facade.Context) (facade.Facade, error) {
 	auth := context.Auth()
 	resources := context.Resources()
 
-	if !isAgent(auth) {
+	// TODO(wallyworld) - enhance this watcher to support
+	// anonymous api calls with macaroons.
+	if auth.GetAuthTag() != nil && !isAgent(auth) {
 		return nil, common.ErrPerm
 	}
 	watcher, ok := resources.Get(id).(state.RelationUnitsWatcher)
@@ -211,6 +217,110 @@ func (w *srvRelationUnitsWatcher) Next() (params.RelationUnitsWatchResult, error
 	return params.RelationUnitsWatchResult{}, err
 }
 
+// srvRelationStatusWatcher defines the API wrapping a state.RelationStatusWatcher.
+type srvRelationStatusWatcher struct {
+	watcherCommon
+	st      *state.State
+	watcher state.StringsWatcher
+}
+
+func newRelationStatusWatcher(context facade.Context) (facade.Facade, error) {
+	id := context.ID()
+	auth := context.Auth()
+	resources := context.Resources()
+
+	// TODO(wallyworld) - enhance this watcher to support
+	// anonymous api calls with macaroons.
+	if auth.GetAuthTag() != nil && !isAgent(auth) {
+		return nil, common.ErrPerm
+	}
+	watcher, ok := resources.Get(id).(state.StringsWatcher)
+	if !ok {
+		return nil, common.ErrUnknownWatcher
+	}
+	return &srvRelationStatusWatcher{
+		watcherCommon: newWatcherCommon(context),
+		st:            context.State(),
+		watcher:       watcher,
+	}, nil
+}
+
+// Next returns when a change has occured to an entity of the
+// collection being watched since the most recent call to Next
+// or the Watch call that created the srvRelationStatusWatcher.
+func (w *srvRelationStatusWatcher) Next() (params.RelationLifeSuspendedStatusWatchResult, error) {
+	if changes, ok := <-w.watcher.Changes(); ok {
+		changesParams := make([]params.RelationLifeSuspendedStatusChange, len(changes))
+		for i, key := range changes {
+			change, err := crossmodel.GetRelationLifeSuspendedStatusChange(crossmodel.GetBackend(w.st), key)
+			if err != nil {
+				return params.RelationLifeSuspendedStatusWatchResult{
+					Error: common.ServerError(err),
+				}, nil
+			}
+			changesParams[i] = *change
+		}
+		return params.RelationLifeSuspendedStatusWatchResult{
+			Changes: changesParams,
+		}, nil
+	}
+	err := w.watcher.Err()
+	if err == nil {
+		err = common.ErrStoppedWatcher
+	}
+	return params.RelationLifeSuspendedStatusWatchResult{}, err
+}
+
+// srvOfferStatusWatcher defines the API wrapping a crossmodelrelations.OfferStatusWatcher.
+type srvOfferStatusWatcher struct {
+	watcherCommon
+	st      *state.State
+	watcher crossmodelrelations.OfferWatcher
+}
+
+func newOfferStatusWatcher(context facade.Context) (facade.Facade, error) {
+	id := context.ID()
+	auth := context.Auth()
+	resources := context.Resources()
+
+	// TODO(wallyworld) - enhance this watcher to support
+	// anonymous api calls with macaroons.
+	if auth.GetAuthTag() != nil && !isAgent(auth) {
+		return nil, common.ErrPerm
+	}
+	watcher, ok := resources.Get(id).(crossmodelrelations.OfferWatcher)
+	if !ok {
+		return nil, common.ErrUnknownWatcher
+	}
+	return &srvOfferStatusWatcher{
+		watcherCommon: newWatcherCommon(context),
+		st:            context.State(),
+		watcher:       watcher,
+	}, nil
+}
+
+// Next returns when a change has occured to an entity of the
+// collection being watched since the most recent call to Next
+// or the Watch call that created the srvOfferStatusWatcher.
+func (w *srvOfferStatusWatcher) Next() (params.OfferStatusWatchResult, error) {
+	if _, ok := <-w.watcher.Changes(); ok {
+		change, err := crossmodel.GetOfferStatusChange(crossmodel.GetBackend(w.st), w.watcher.OfferUUID())
+		if err != nil {
+			return params.OfferStatusWatchResult{
+				Error: common.ServerError(err),
+			}, nil
+		}
+		return params.OfferStatusWatchResult{
+			Changes: []params.OfferStatusChange{*change},
+		}, nil
+	}
+	err := w.watcher.Err()
+	if err == nil {
+		err = common.ErrStoppedWatcher
+	}
+	return params.OfferStatusWatchResult{}, err
+}
+
 // srvMachineStorageIdsWatcher defines the API wrapping a state.StringsWatcher
 // watching machine/storage attachments. This watcher notifies about storage
 // entities (volumes/filesystems) being attached to and detached from machines.
