@@ -35,12 +35,20 @@ type UserdataConfig interface {
 	// with appropriate configuration. It will run ConfigureBasic() and
 	// ConfigureJuju()
 	Configure() error
+
 	// ConfigureBasic updates the provided cloudinit.Config with
 	// basic configuration to initialise an OS image.
 	ConfigureBasic() error
+
 	// ConfigureJuju updates the provided cloudinit.Config with configuration
 	// to initialise a Juju machine agent.
 	ConfigureJuju() error
+
+	// ConfigureCustomOverrides updates the provided cloudinit.Config with
+	// user provided cloudinit data.  Data provided will overwrite current
+	// values with three exceptions: preruncmd was handled in ConfigureBasic()
+	// and packages and postruncmd were handled in ConfigureJuju().
+	ConfigureCustomOverrides() error
 }
 
 // NewUserdataConfig is supposed to take in an instanceConfig as well as a
