@@ -33,8 +33,8 @@ type RunCommand struct {
 	*runCommand
 }
 
-func (c *RunCommand) UnitTag() names.UnitTag {
-	return c.unitTag
+func (c *RunCommand) UnitTags() []names.UnitTag {
+	return c.unitTags
 }
 
 func (c *RunCommand) ActionName() string {
