@@ -1,5 +1,5 @@
 // Copyright 2015 Canonical Ltd.
-// Licensed under the LGPLv3, see LICENCE file for details.
+// Licensed under the AGPLv3, see LICENCE file for details.
 
 package application
 
@@ -346,8 +346,9 @@ func (h *bundleHandler) handleChanges() error {
 	}
 
 	// Deploy the bundle.
-	for _, change := range h.changes {
+	for i, change := range h.changes {
 		fmt.Fprintf(h.ctx.Stdout, "- %s\n", change.Description())
+		logger.Tracef("%d: change %s", i, pretty.Sprint(change))
 		switch change := change.(type) {
 		case *bundlechanges.AddCharmChange:
 			err = h.addCharm(change)
@@ -615,7 +616,6 @@ func (h *bundleHandler) addMachine(change *bundlechanges.AddMachineChange) error
 		Series:      p.Series,
 		Jobs:        []multiwatcher.MachineJob{multiwatcher.JobHostUnits},
 	}
-	logger.Debugf("machineParams: %s", pretty.Sprint(machineParams))
 	if ct := p.ContainerType; ct != "" {
 		// TODO(thumper): move the warning and translation into the bundle reading code.
 
@@ -635,10 +635,12 @@ func (h *bundleHandler) addMachine(change *bundlechanges.AddMachineChange) error
 		machineParams.ContainerType = containerType
 		if p.ParentId != "" {
 			logger.Debugf("p.ParentId: %q", p.ParentId)
-			machineParams.ParentId, err = h.resolveMachine(p.ParentId)
+			id, err := h.resolveMachine(p.ParentId)
 			if err != nil {
 				return errors.Annotatef(err, "cannot retrieve parent placement for %s", deployedApps())
 			}
+			// Never create nested containers for deployment.
+			machineParams.ParentId = h.topLevelMachine(id)
 		}
 	}
 	logger.Debugf("machineParams: %s", pretty.Sprint(machineParams))
@@ -982,6 +984,14 @@ func (h *bundleHandler) resolveMachine(placeholder string) (string, error) {
 	return h.unitStatus[machineOrUnit], nil
 }
 
+func (h *bundleHandler) topLevelMachine(id string) string {
+	if !names.IsContainerMachine(id) {
+		return id
+	}
+	tag := names.NewMachineTag(id)
+	return tag.Parent().Id()
+}
+
 // resolveRelation returns the relation name resolving the included application
 // placeholder.
 func resolveRelation(e string, results map[string]string) string {
@@ -1172,17 +1182,17 @@ func processSingleBundleOverlay(data *charm.BundleData, bundleOverlayFile string
 	// actually exist in the bundle data.
 	for appName, bc := range config.Applications {
 		app, found := data.Applications[appName]
-		if !found {
-			// Add it in.
-			data.Applications[appName] = bc
-			continue
-		}
 		// If bc is nil, that means to remove it from data.
 		if bc == nil {
 			delete(data.Applications, appName)
 			data.Relations = removeRelations(data.Relations, appName)
 			continue
 		}
+		if !found {
+			// Add it in.
+			data.Applications[appName] = bc
+			continue
+		}
 
 		fieldCheck := configCheck.Applications[appName]
 
@@ -1330,11 +1340,14 @@ func buildModelRepresentation(
 				Machine: unit.Machine,
 			})
 		}
-		// TODO: application constraints
 		applications[name] = application
 		annotationTags = append(annotationTags, names.NewApplicationTag(name).String())
 		appNames = append(appNames, name)
-		if len(appStatus.SubordinateTo) == 0 {
+		if len(appStatus.Units) > 0 {
+			// While this isn't entirely accurate, because an application
+			// without any units is still a principal, it is less bad than
+			// just using 'SubordinateTo' as a subordinate charm that isn't
+			// related to anything has that empty too.
 			principalApps = append(principalApps, name)
 		}
 	}
