@@ -15,6 +15,7 @@ import (
 	"time"
 
 	"github.com/juju/errors"
+	"github.com/juju/loggo"
 	jc "github.com/juju/testing/checkers"
 	gc "gopkg.in/check.v1"
 	"gopkg.in/juju/charm.v6"
@@ -247,6 +248,38 @@ func (s *BundleDeployCharmStoreSuite) TestDryRunTwice(c *gc.C) {
 	s.assertUnitsCreated(c, map[string]string{})
 }
 
+func (s *BundleDeployCharmStoreSuite) TestDryRunExistingModel(c *gc.C) {
+	testcharms.UploadCharm(c, s.client, "xenial/mysql-42", "mysql")
+	testcharms.UploadCharm(c, s.client, "xenial/wordpress-47", "wordpress")
+	testcharms.UploadCharm(c, s.client, "trusty/multi-series-subordinate-13", "multi-series-subordinate")
+	testcharms.UploadBundle(c, s.client, "bundle/wordpress-simple-1", "wordpress-simple")
+	// Start with a mysql that already has the right charm.
+	ch := s.Factory.MakeCharm(c, &factory.CharmParams{
+		Name: "mysql", Series: "xenial", Revision: "42"})
+	mysql := s.Factory.MakeApplication(c, &factory.ApplicationParams{
+		Name: "mysql", Charm: ch})
+	s.Factory.MakeUnit(c, &factory.UnitParams{Application: mysql})
+	// Also add a subordinate that isn't attached to anything.
+	sub := s.Factory.MakeCharm(c, &factory.CharmParams{
+		Name: "multi-series-subordinate", Series: "trusty", Revision: "13"})
+	s.Factory.MakeApplication(c, &factory.ApplicationParams{
+		Name: "sub", Charm: sub})
+
+	stdOut, _, err := runDeployWithOutput(c, "bundle/wordpress-simple", "--dry-run")
+	c.Assert(err, jc.ErrorIsNil)
+	expected := "" +
+		"Changes to deploy bundle:\n" +
+		"- upload charm cs:xenial/wordpress-47 for series xenial\n" +
+		"- deploy application wordpress on xenial using cs:xenial/wordpress-47\n" +
+		"- add relation wordpress:db - mysql:server\n" +
+		"- add unit wordpress/0 to new machine 1"
+
+	c.Check(stdOut, gc.Equals, expected)
+	stdOut, _, err = runDeployWithOutput(c, "bundle/wordpress-simple", "--dry-run")
+	c.Assert(err, jc.ErrorIsNil)
+	c.Check(stdOut, gc.Equals, expected)
+}
+
 func (s *BundleDeployCharmStoreSuite) TestDeployBundleGatedCharm(c *gc.C) {
 	testcharms.UploadCharm(c, s.client, "xenial/mysql-42", "mysql")
 	url, _ := testcharms.UploadCharm(c, s.client, "xenial/wordpress-47", "wordpress")
@@ -444,6 +477,11 @@ func (s *BundleDeployCharmStoreSuite) SetUpSuite(c *gc.C) {
 	s.PatchValue(&watcher.Period, 10*time.Millisecond)
 }
 
+func (s *BundleDeployCharmStoreSuite) SetUpTest(c *gc.C) {
+	s.charmStoreSuite.SetUpTest(c)
+	logger.SetLogLevel(loggo.TRACE)
+}
+
 func (s *BundleDeployCharmStoreSuite) Client() *csclient.Client {
 	return s.client
 }
@@ -1245,6 +1283,46 @@ func (s *BundleDeployCharmStoreSuite) TestDeployBundleUnitPlacedInApplication(c
 	})
 }
 
+func (s *BundleDeployCharmStoreSuite) TestDeployBundlePeerContainer(c *gc.C) {
+	testcharms.UploadCharm(c, s.client, "xenial/django-42", "dummy")
+	testcharms.UploadCharm(c, s.client, "xenial/wordpress-0", "wordpress")
+
+	stdOut, _, err := s.DeployBundleYAMLWithOutput(c, `
+        applications:
+            wordpress:
+                charm: wordpress
+                num_units: 2
+                to: ["lxd:new"]
+            django:
+                charm: cs:xenial/django-42
+                num_units: 2
+                to: ["lxd:wordpress"]
+    `)
+	c.Assert(err, jc.ErrorIsNil)
+	c.Check(stdOut, gc.Equals, ""+
+		"Executing changes:\n"+
+		"- upload charm cs:xenial/django-42 for series xenial\n"+
+		"- deploy application django on xenial using cs:xenial/django-42\n"+
+		"- upload charm cs:xenial/wordpress-0 for series xenial\n"+
+		"- deploy application wordpress on xenial using cs:xenial/wordpress-0\n"+
+		"- add lxd container 0/lxd/0 on new machine 0\n"+
+		"- add lxd container 1/lxd/0 on new machine 1\n"+
+		"- add unit wordpress/0 to 0/lxd/0\n"+
+		"- add unit wordpress/1 to 1/lxd/0\n"+
+		"- add lxd container 0/lxd/1 on new machine 0\n"+
+		"- add lxd container 1/lxd/1 on new machine 1\n"+
+		"- add unit django/0 to 0/lxd/1 to satisfy [lxd:wordpress]\n"+
+		"- add unit django/1 to 1/lxd/1 to satisfy [lxd:wordpress]",
+	)
+
+	s.assertUnitsCreated(c, map[string]string{
+		"django/0":    "0/lxd/1",
+		"django/1":    "1/lxd/1",
+		"wordpress/0": "0/lxd/0",
+		"wordpress/1": "1/lxd/0",
+	})
+}
+
 func (s *BundleDeployCharmStoreSuite) TestDeployBundleUnitColocationWithUnit(c *gc.C) {
 	testcharms.UploadCharm(c, s.client, "xenial/django-42", "dummy")
 	testcharms.UploadCharm(c, s.client, "xenial/mem-47", "dummy")
@@ -1851,6 +1929,20 @@ func (s *ProcessBundleOverlaySuite) TestRemoveApplication(c *gc.C) {
 	c.Assert(s.bundleData.Relations, gc.HasLen, 0)
 }
 
+func (s *ProcessBundleOverlaySuite) TestRemoveUnknownApplication(c *gc.C) {
+	config := `
+        applications:
+            unknown:
+    `
+	filename := s.writeFile(c, config)
+	err := processBundleOverlay(s.bundleData, filename)
+	c.Assert(err, jc.ErrorIsNil)
+	s.assertApplications(c, "django", "memcached")
+	c.Assert(s.bundleData.Relations, jc.DeepEquals, [][]string{
+		{"django", "memcached"},
+	})
+}
+
 func (s *ProcessBundleOverlaySuite) TestIncludes(c *gc.C) {
 	config := `
         applications:
