@@ -5,7 +5,7 @@ package application
 
 import (
 	"github.com/juju/cmd"
-	"gopkg.in/juju/charmrepo.v2-unstable/csclient"
+	"gopkg.in/juju/charmrepo.v2/csclient"
 	"gopkg.in/macaroon-bakery.v1/httpbakery"
 
 	"github.com/juju/juju/api"
@@ -47,10 +47,8 @@ func NewAddUnitCommandForTest(api serviceAddUnitAPI) cmd.Command {
 }
 
 // NewAddRelationCommandForTest returns an AddRelationCommand with the api provided as specified.
-func NewAddRelationCommandForTest(api ApplicationAddRelationAPI) modelcmd.ModelCommand {
-	cmd := &addRelationCommand{newAPIFunc: func() (ApplicationAddRelationAPI, error) {
-		return api, nil
-	}}
+func NewAddRelationCommandForTest(addAPI applicationAddRelationAPI, consumeAPI applicationConsumeDetailsAPI) modelcmd.ModelCommand {
+	cmd := &addRelationCommand{addRelationAPI: addAPI, consumeDetailsAPI: consumeAPI}
 	return modelcmd.Wrap(cmd)
 }
 
@@ -63,12 +61,51 @@ func NewRemoveRelationCommandForTest(api ApplicationDestroyRelationAPI) modelcmd
 }
 
 // NewConsumeCommandForTest returns a ConsumeCommand with the specified api.
-func NewConsumeCommandForTest(store jujuclient.ClientStore, api applicationConsumeAPI) cmd.Command {
-	c := &consumeCommand{api: api}
+func NewConsumeCommandForTest(
+	store jujuclient.ClientStore,
+	sourceAPI applicationConsumeDetailsAPI,
+	targetAPI applicationConsumeAPI,
+) cmd.Command {
+	c := &consumeCommand{sourceAPI: sourceAPI, targetAPI: targetAPI}
 	c.SetClientStore(store)
 	return modelcmd.Wrap(c)
 }
 
+func NewUpdateSeriesCommandForTest(
+	appAPI updateApplicationSeriesAPI,
+	machAPI updateMachineSeriesAPI,
+) modelcmd.ModelCommand {
+	cmd := &updateSeriesCommand{
+		updateApplicationSeriesClient: appAPI,
+		updateMachineSeriesClient:     machAPI,
+	}
+	return modelcmd.Wrap(cmd)
+}
+
+// NewSuspendRelationCommandForTest returns a SuspendRelationCommand with the api provided as specified.
+func NewSuspendRelationCommandForTest(api SetRelationSuspendedAPI) modelcmd.ModelCommand {
+	cmd := &suspendRelationCommand{newAPIFunc: func() (SetRelationSuspendedAPI, error) {
+		return api, nil
+	}}
+	return modelcmd.Wrap(cmd)
+}
+
+// NewResumeRelationCommandForTest returns a ResumeRelationCommand with the api provided as specified.
+func NewResumeRelationCommandForTest(api SetRelationSuspendedAPI) modelcmd.ModelCommand {
+	cmd := &resumeRelationCommand{newAPIFunc: func() (SetRelationSuspendedAPI, error) {
+		return api, nil
+	}}
+	return modelcmd.Wrap(cmd)
+}
+
+// NewRemoveSaasCommandForTest returns a RemoveSaasCommand with the api provided as specified.
+func NewRemoveSaasCommandForTest(api RemoveSaasAPI) modelcmd.ModelCommand {
+	cmd := &removeSaasCommand{newAPIFunc: func() (RemoveSaasAPI, error) {
+		return api, nil
+	}}
+	return modelcmd.Wrap(cmd)
+}
+
 type Patcher interface {
 	PatchValue(dest, value interface{})
 }
