@@ -6,7 +6,8 @@ package application
 import (
 	"github.com/juju/cmd"
 	"github.com/juju/errors"
-	"gopkg.in/juju/charm.v6-unstable"
+	"github.com/juju/gnuflag"
+	"gopkg.in/juju/charm.v6"
 	"gopkg.in/juju/names.v2"
 
 	"github.com/juju/juju/api/application"
@@ -23,6 +24,7 @@ func NewRemoveApplicationCommand() cmd.Command {
 // removeServiceCommand causes an existing application to be destroyed.
 type removeApplicationCommand struct {
 	modelcmd.ModelCommandBase
+	DestroyStorage   bool
 	ApplicationNames []string
 }
 
@@ -51,6 +53,11 @@ func (c *removeApplicationCommand) Info() *cmd.Info {
 	}
 }
 
+func (c *removeApplicationCommand) SetFlags(f *gnuflag.FlagSet) {
+	c.ModelCommandBase.SetFlags(f)
+	f.BoolVar(&c.DestroyStorage, "destroy-storage", false, "Destroy storage attached to application units")
+}
+
 func (c *removeApplicationCommand) Init(args []string) error {
 	if len(args) == 0 {
 		return errors.Errorf("no application specified")
@@ -66,9 +73,9 @@ func (c *removeApplicationCommand) Init(args []string) error {
 
 type removeApplicationAPI interface {
 	Close() error
-	DestroyApplications(appName ...string) ([]params.DestroyApplicationResult, error)
+	DestroyApplications(application.DestroyApplicationsParams) ([]params.DestroyApplicationResult, error)
 	DestroyDeprecated(appName string) error
-	DestroyUnits(unitNames ...string) ([]params.DestroyUnitResult, error)
+	DestroyUnits(application.DestroyUnitsParams) ([]params.DestroyUnitResult, error)
 	DestroyUnitsDeprecated(unitNames ...string) error
 	GetCharmURL(appName string) (*charm.URL, error)
 	ModelUUID() string
@@ -93,6 +100,9 @@ func (c *removeApplicationCommand) Run(ctx *cmd.Context) error {
 	if apiVersion < 4 {
 		return c.removeApplicationsDeprecated(ctx, client)
 	}
+	if c.DestroyStorage && apiVersion < 5 {
+		return errors.New("--destroy-storage is not supported by this controller")
+	}
 	return c.removeApplications(ctx, client)
 }
 
@@ -115,7 +125,10 @@ func (c *removeApplicationCommand) removeApplications(
 	ctx *cmd.Context,
 	client removeApplicationAPI,
 ) error {
-	results, err := client.DestroyApplications(c.ApplicationNames...)
+	results, err := client.DestroyApplications(application.DestroyApplicationsParams{
+		Applications:   c.ApplicationNames,
+		DestroyStorage: c.DestroyStorage,
+	})
 	if err := block.ProcessBlockedError(err, block.BlockRemove); err != nil {
 		return errors.Trace(err)
 	}
