@@ -21,7 +21,7 @@ type RemoveRelationSuite struct {
 
 func (s *RemoveRelationSuite) SetUpTest(c *gc.C) {
 	s.IsolationSuite.SetUpTest(c)
-	s.mockAPI = &mockRemoveAPI{Stub: &testing.Stub{}}
+	s.mockAPI = &mockRemoveAPI{Stub: &testing.Stub{}, version: 6}
 	s.mockAPI.removeRelationFunc = func(endpoints ...string) error {
 		return s.mockAPI.NextErr()
 	}
@@ -39,15 +39,22 @@ func (s *RemoveRelationSuite) TestRemoveRelationWrongNumberOfArguments(c *gc.C)
 	err := s.runRemoveRelation(c)
 	c.Assert(err, gc.ErrorMatches, "a relation must involve two applications")
 
-	// 1 argument
+	// 1 argument not an integer
 	err = s.runRemoveRelation(c, "application1")
-	c.Assert(err, gc.ErrorMatches, "a relation must involve two applications")
+	c.Assert(err, gc.ErrorMatches, `relation ID "application1" not valid`)
 
 	// More than 2 arguments
 	err = s.runRemoveRelation(c, "application1", "application2", "application3")
 	c.Assert(err, gc.ErrorMatches, "a relation must involve two applications")
 }
 
+func (s *RemoveRelationSuite) TestRemoveRelationIdOldServer(c *gc.C) {
+	s.mockAPI.version = 4
+	err := s.runRemoveRelation(c, "123")
+	c.Assert(err, gc.ErrorMatches, "removing a relation using its ID is not supported by this version of Juju")
+	s.mockAPI.CheckCall(c, 0, "Close")
+}
+
 func (s *RemoveRelationSuite) TestRemoveRelationSuccess(c *gc.C) {
 	err := s.runRemoveRelation(c, "application1", "application2")
 	c.Assert(err, jc.ErrorIsNil)
@@ -55,6 +62,13 @@ func (s *RemoveRelationSuite) TestRemoveRelationSuccess(c *gc.C) {
 	s.mockAPI.CheckCall(c, 1, "Close")
 }
 
+func (s *RemoveRelationSuite) TestRemoveRelationIdSuccess(c *gc.C) {
+	err := s.runRemoveRelation(c, "123")
+	c.Assert(err, jc.ErrorIsNil)
+	s.mockAPI.CheckCall(c, 0, "DestroyRelationId", 123)
+	s.mockAPI.CheckCall(c, 1, "Close")
+}
+
 func (s *RemoveRelationSuite) TestRemoveRelationFail(c *gc.C) {
 	msg := "fail remove-relation at API"
 	s.mockAPI.SetErrors(errors.New(msg))
@@ -74,6 +88,7 @@ func (s *RemoveRelationSuite) TestRemoveRelationBlocked(c *gc.C) {
 
 type mockRemoveAPI struct {
 	*testing.Stub
+	version            int
 	removeRelationFunc func(endpoints ...string) error
 }
 
@@ -86,3 +101,12 @@ func (s mockRemoveAPI) DestroyRelation(endpoints ...string) error {
 	s.MethodCall(s, "DestroyRelation", endpoints)
 	return s.removeRelationFunc(endpoints...)
 }
+
+func (s mockRemoveAPI) DestroyRelationId(relationId int) error {
+	s.MethodCall(s, "DestroyRelationId", relationId)
+	return nil
+}
+
+func (s mockRemoveAPI) BestAPIVersion() int {
+	return s.version
+}
