@@ -5,12 +5,12 @@ package backups_test
 
 import (
 	"github.com/juju/cmd"
+	"github.com/juju/cmd/cmdtesting"
 	"github.com/juju/errors"
 	jc "github.com/juju/testing/checkers"
 	gc "gopkg.in/check.v1"
 
 	"github.com/juju/juju/cmd/juju/backups"
-	"github.com/juju/juju/testing"
 )
 
 type showSuite struct {
@@ -27,7 +27,7 @@ func (s *showSuite) SetUpTest(c *gc.C) {
 
 func (s *showSuite) TestOkay(c *gc.C) {
 	s.setSuccess()
-	ctx, err := testing.RunCommand(c, s.subcommand, s.metaresult.ID)
+	ctx, err := cmdtesting.RunCommand(c, s.subcommand, s.metaresult.ID)
 	c.Check(err, jc.ErrorIsNil)
 
 	out := MetaResultString
@@ -36,6 +36,6 @@ func (s *showSuite) TestOkay(c *gc.C) {
 
 func (s *showSuite) TestError(c *gc.C) {
 	s.setFailure("failed!")
-	_, err := testing.RunCommand(c, s.subcommand, s.metaresult.ID)
+	_, err := cmdtesting.RunCommand(c, s.subcommand, s.metaresult.ID)
 	c.Check(errors.Cause(err), gc.ErrorMatches, "failed!")
 }
