@@ -43,6 +43,7 @@ us-west-2
 ca-central-1
 eu-west-1
 eu-west-2
+eu-west-3
 eu-central-1
 ap-south-1
 ap-southeast-1
@@ -80,6 +81,8 @@ eu-west-1:
   endpoint: https://ec2.eu-west-1.amazonaws.com
 eu-west-2:
   endpoint: https://ec2.eu-west-2.amazonaws.com
+eu-west-3:
+  endpoint: https://ec2.eu-west-3.amazonaws.com
 eu-central-1:
   endpoint: https://ec2.eu-central-1.amazonaws.com
 ap-south-1:
