@@ -49,6 +49,7 @@ import (
 	"github.com/juju/juju/instance"
 	"github.com/juju/juju/juju/keys"
 	"github.com/juju/juju/juju/osenv"
+	supportedversion "github.com/juju/juju/juju/version"
 	"github.com/juju/juju/jujuclient"
 	"github.com/juju/juju/jujuclient/jujuclienttesting"
 	"github.com/juju/juju/network"
@@ -85,7 +86,7 @@ func (s *BootstrapSuite) SetUpSuite(c *gc.C) {
 	s.MgoSuite.SetUpSuite(c)
 	s.PatchValue(&keys.JujuPublicKey, sstesting.SignedMetadataPublicKey)
 	s.PatchValue(&cert.NewCA, coretesting.NewCA)
-	s.PatchValue(&cert.NewLeafKeyBits, 128)
+	s.PatchValue(&cert.NewLeafKeyBits, 512)
 }
 
 func (s *BootstrapSuite) SetUpTest(c *gc.C) {
@@ -222,7 +223,7 @@ func (s *BootstrapSuite) run(c *gc.C, test bootstrapTest) testing.Restorer {
 	}
 	bootstrapVersion := v100p64
 	if test.version != "" {
-		useVersion := strings.Replace(test.version, "%LTS%", series.LatestLts(), 1)
+		useVersion := strings.Replace(test.version, "%LTS%", supportedversion.SupportedLTS(), 1)
 		bootstrapVersion = version.MustParseBinary(useVersion)
 		restore = restore.Add(testing.PatchValue(&jujuversion.Current, bootstrapVersion.Number))
 		restore = restore.Add(testing.PatchValue(&arch.HostArch, func() string { return bootstrapVersion.Arch }))
@@ -1212,7 +1213,7 @@ func (s *BootstrapSuite) setupAutoUploadTest(c *gc.C, vers, ser string) {
 }
 
 func (s *BootstrapSuite) TestAutoUploadAfterFailedSync(c *gc.C) {
-	s.PatchValue(&series.MustHostSeries, func() string { return series.LatestLts() })
+	s.PatchValue(&series.MustHostSeries, func() string { return supportedversion.SupportedLTS() })
 	s.setupAutoUploadTest(c, "1.7.3", "quantal")
 	// Run command and check for that upload has been run for tools matching
 	// the current juju version.
@@ -1413,7 +1414,11 @@ func (s *BootstrapSuite) TestBootstrapProviderFileCredential(c *gc.C) {
 
 	tmpFile, err := ioutil.TempFile("", "juju-bootstrap-test")
 	c.Assert(err, jc.ErrorIsNil)
-	defer func() { err := os.Remove(tmpFile.Name()); c.Assert(err, jc.ErrorIsNil) }()
+	defer func() {
+		tmpFile.Close()
+		err := os.Remove(tmpFile.Name())
+		c.Assert(err, jc.ErrorIsNil)
+	}()
 
 	contents := []byte("{something: special}\n")
 	err = ioutil.WriteFile(tmpFile.Name(), contents, 0644)
@@ -1435,8 +1440,8 @@ func (s *BootstrapSuite) TestBootstrapProviderFileCredential(c *gc.C) {
 	// that the state of the credential under test before finalization is
 	// indeed the file path itself and that the state of the credential
 	// after finalization is the contents of that file.
-	c.Assert(unfinalizedCredential.Attributes()["file"], gc.Matches, tmpFile.Name())
-	c.Assert(finalizedCredential.Attributes()["file"], gc.Matches, string(contents))
+	c.Assert(unfinalizedCredential.Attributes()["file"], gc.Equals, tmpFile.Name())
+	c.Assert(finalizedCredential.Attributes()["file"], gc.Equals, string(contents))
 }
 
 func (s *BootstrapSuite) TestBootstrapProviderDetectRegionsInvalid(c *gc.C) {
@@ -1776,6 +1781,7 @@ us-west-2
 ca-central-1
 eu-west-1
 eu-west-2
+eu-west-3
 eu-central-1
 ap-south-1
 ap-southeast-1
