@@ -19,6 +19,7 @@ import (
 
 	"github.com/juju/juju/api/base"
 	"github.com/juju/juju/api/controller"
+	"github.com/juju/juju/api/storage"
 	"github.com/juju/juju/apiserver/params"
 	"github.com/juju/juju/cmd/juju/block"
 	"github.com/juju/juju/cmd/modelcmd"
@@ -44,7 +45,10 @@ func NewDestroyCommand() cmd.Command {
 // destroyCommand destroys the specified controller.
 type destroyCommand struct {
 	destroyCommandBase
-	destroyModels bool
+	storageAPI     storageAPI
+	destroyModels  bool
+	destroyStorage bool
+	releaseStorage bool
 }
 
 // usageDetails has backticks which we want to keep for markdown processing.
@@ -55,9 +59,25 @@ All models (initial model plus all workload/hosted) associated with the
 controller will first need to be destroyed, either in advance, or by
 specifying `[1:] + "`--destroy-all-models`." + `
 
+If there is persistent storage in any of the models managed by the
+controller, then you must choose to either destroy or release the
+storage, using ` + "`--destroy-storage` or `--release-storage` respectively." + `
+
 Examples:
+    # Destroy the controller and all hosted models. If there is
+    # persistent storage remaining in any of the models, then
+    # this will prompt you to choose to either destroy or release
+    # the storage.
     juju destroy-controller --destroy-all-models mycontroller
 
+    # Destroy the controller and all hosted models, destroying
+    # any remaining persistent storage.
+    juju destroy-controller --destroy-all-models --destroy-storage
+
+    # Destroy the controller and all hosted models, releasing
+    # any remaining persistent storage from Juju's control.
+    juju destroy-controller --destroy-all-models --release-storage
+
 See also: 
     kill-controller
     unregister`
@@ -75,15 +95,21 @@ Continue? (y/N):`[1:]
 // that the destroy command calls.
 type destroyControllerAPI interface {
 	Close() error
+	BestAPIVersion() int
 	ModelConfig() (map[string]interface{}, error)
 	HostedModelConfigs() ([]controller.HostedConfig, error)
 	CloudSpec(names.ModelTag) (environs.CloudSpec, error)
-	DestroyController(destroyModels bool) error
+	DestroyController(controller.DestroyControllerParams) error
 	ListBlockedModels() ([]params.ModelBlockInfo, error)
 	ModelStatus(models ...names.ModelTag) ([]base.ModelStatus, error)
 	AllModels() ([]base.UserModel, error)
 }
 
+type storageAPI interface {
+	Close() error
+	ListStorageDetails() ([]params.StorageDetails, error)
+}
+
 // destroyClientAPI defines the methods on the client API endpoint that the
 // destroy command might call.
 type destroyClientAPI interface {
@@ -106,6 +132,16 @@ func (c *destroyCommand) Info() *cmd.Info {
 func (c *destroyCommand) SetFlags(f *gnuflag.FlagSet) {
 	c.destroyCommandBase.SetFlags(f)
 	f.BoolVar(&c.destroyModels, "destroy-all-models", false, "Destroy all hosted models in the controller")
+	f.BoolVar(&c.destroyStorage, "destroy-storage", false, "Destroy all storage instances managed by the controller")
+	f.BoolVar(&c.releaseStorage, "release-storage", false, "Release all storage instances from management of the controller, without destroying them")
+}
+
+// Init implements Command.Init.
+func (c *destroyCommand) Init(args []string) error {
+	if c.destroyStorage && c.releaseStorage {
+		return errors.New("--destroy-storage and --release-storage cannot both be specified")
+	}
+	return c.destroyCommandBase.Init(args)
 }
 
 // Run implements Command.Run
@@ -129,6 +165,48 @@ func (c *destroyCommand) Run(ctx *cmd.Context) error {
 	}
 	defer api.Close()
 
+	if api.BestAPIVersion() < 4 {
+		// Versions before 4 support only destroying the storage,
+		// and will not raise an error if there is storage in the
+		// controller. Force the user to specify up-front.
+		if c.releaseStorage {
+			return errors.New("this juju controller only supports destroying storage")
+		}
+		if !c.destroyStorage {
+			models, err := api.AllModels()
+			if err != nil {
+				return errors.Trace(err)
+			}
+			var anyStorage bool
+			for _, model := range models {
+				hasStorage, err := c.modelHasStorage(model.Name)
+				if err != nil {
+					return errors.Trace(err)
+				}
+				if hasStorage {
+					anyStorage = true
+					break
+				}
+			}
+			if anyStorage {
+				return errors.Errorf(`cannot destroy controller %q
+
+Destroying this controller will destroy the storage,
+but you have not indicated that you want to do that.
+
+Please run the the command again with --destroy-storage
+to confirm that you want to destroy the storage along
+with the controller.
+
+If instead you want to keep the storage, you must first
+upgrade the controller to version 2.3 or greater.
+
+`, controllerName)
+			}
+			c.destroyStorage = true
+		}
+	}
+
 	// Obtain controller environ so we can clean up afterwards.
 	controllerEnviron, err := c.getControllerEnviron(ctx, store, controllerName, api)
 	if err != nil {
@@ -139,10 +217,23 @@ func (c *destroyCommand) Run(ctx *cmd.Context) error {
 		// Attempt to destroy the controller.
 		ctx.Infof("Destroying controller")
 		var hasHostedModels bool
-		err = api.DestroyController(c.destroyModels)
+		var hasPersistentStorage bool
+		var destroyStorage *bool
+		if c.destroyStorage || c.releaseStorage {
+			// Set destroyStorage to true or false, if
+			// --destroy-storage or --release-storage
+			// is specified, respectively.
+			destroyStorage = &c.destroyStorage
+		}
+		err = api.DestroyController(controller.DestroyControllerParams{
+			DestroyModels:  c.destroyModels,
+			DestroyStorage: destroyStorage,
+		})
 		if err != nil {
 			if params.IsCodeHasHostedModels(err) {
 				hasHostedModels = true
+			} else if params.IsCodeHasPersistentStorage(err) {
+				hasPersistentStorage = true
 			} else {
 				return c.ensureUserFriendlyErrorLog(
 					errors.Annotate(err, "cannot destroy controller"),
@@ -165,6 +256,16 @@ func (c *destroyCommand) Run(ctx *cmd.Context) error {
 				continue
 			}
 		}
+		if !c.destroyStorage && !c.releaseStorage && hasPersistentStorage {
+			if err := c.checkNoPersistentStorage(ctx, envStatus); err != nil {
+				return errors.Trace(err)
+			}
+			// When we called DestroyController before, we were
+			// informed that there was persistent storage remaining.
+			// When we checked just now, there was none. We should
+			// try destroying again.
+			continue
+		}
 
 		// Even if we've not just requested for hosted models to be destroyed,
 		// there may be some being destroyed already. We need to wait for them.
@@ -182,6 +283,20 @@ func (c *destroyCommand) Run(ctx *cmd.Context) error {
 	}
 }
 
+func (c *destroyCommand) modelHasStorage(modelName string) (bool, error) {
+	client, err := c.getStorageAPI(modelName)
+	if err != nil {
+		return false, errors.Trace(err)
+	}
+	defer client.Close()
+
+	storage, err := client.ListStorageDetails()
+	if err != nil {
+		return false, errors.Trace(err)
+	}
+	return len(storage) > 0, nil
+}
+
 // checkNoAliveHostedModels ensures that the given set of hosted models
 // contains none that are Alive. If there are, an message is printed
 // out to
@@ -214,6 +329,69 @@ Models:
 %s`, controllerName, buf.String())
 }
 
+// checkNoPersistentStorage ensures that the controller contains
+// no persistent storage. If there is any, a message is printed
+// out informing the user that they must choose to destroy or
+// release the storage.
+func (c *destroyCommand) checkNoPersistentStorage(ctx *cmd.Context, envStatus environmentStatus) error {
+	models := append([]modelData{envStatus.controller.Model}, envStatus.models...)
+
+	var modelsWithPersistentStorage int
+	var persistentVolumesTotal int
+	var persistentFilesystemsTotal int
+	for _, m := range models {
+		if m.PersistentVolumeCount+m.PersistentFilesystemCount == 0 {
+			continue
+		}
+		modelsWithPersistentStorage++
+		persistentVolumesTotal += m.PersistentVolumeCount
+		persistentFilesystemsTotal += m.PersistentFilesystemCount
+	}
+
+	var buf bytes.Buffer
+	if n := persistentVolumesTotal; n > 0 {
+		fmt.Fprintf(&buf, "%d volume", n)
+		if n > 1 {
+			buf.WriteRune('s')
+		}
+		if persistentFilesystemsTotal > 0 {
+			buf.WriteString(" and ")
+		}
+	}
+	if n := persistentFilesystemsTotal; n > 0 {
+		fmt.Fprintf(&buf, "%d filesystem", n)
+		if n > 1 {
+			buf.WriteRune('s')
+		}
+	}
+	buf.WriteRune(' ')
+	if n := modelsWithPersistentStorage; n == 1 {
+		buf.WriteString("in 1 model")
+	} else {
+		fmt.Fprintf(&buf, "across %d models", n)
+	}
+
+	controllerName, err := c.ControllerName()
+	if err != nil {
+		return errors.Trace(err)
+	}
+
+	return errors.Errorf(`cannot destroy controller %q
+
+The controller has persistent storage remaining:
+	%s
+
+To destroy the storage, run the destroy-controller
+command again with the "--destroy-storage" flag.
+
+To release the storage from Juju's management
+without destroying it, use the "--release-storage"
+flag instead. The storage can then be imported
+into another Juju model.
+
+`, controllerName, buf.String())
+}
+
 // ensureUserFriendlyErrorLog ensures that error will be logged and displayed
 // in a user-friendly manner with readable and digestable error message.
 func (c *destroyCommand) ensureUserFriendlyErrorLog(destroyErr error, ctx *cmd.Context, api destroyControllerAPI) error {
@@ -241,9 +419,6 @@ func (c *destroyCommand) ensureUserFriendlyErrorLog(destroyErr error, ctx *cmd.C
 		}
 		return cmd.ErrSilent
 	}
-	if params.IsCodeHasHostedModels(destroyErr) {
-		return destroyErr
-	}
 	controllerName, err := c.ControllerName()
 	if err != nil {
 		return errors.Trace(err)
@@ -303,6 +478,17 @@ func (c *destroyCommandBase) getControllerAPI() (destroyControllerAPI, error) {
 	return controller.NewClient(root), nil
 }
 
+func (c *destroyCommand) getStorageAPI(modelName string) (storageAPI, error) {
+	if c.storageAPI != nil {
+		return c.storageAPI, nil
+	}
+	root, err := c.NewModelAPIRoot(modelName)
+	if err != nil {
+		return nil, errors.Trace(err)
+	}
+	return storage.NewClient(root), nil
+}
+
 // SetFlags implements Command.SetFlags.
 func (c *destroyCommandBase) SetFlags(f *gnuflag.FlagSet) {
 	c.ControllerCommandBase.SetFlags(f)
