@@ -12,6 +12,7 @@ import (
 	"github.com/juju/gnuflag"
 	"github.com/juju/utils/clock"
 
+	"github.com/juju/juju/api/controller"
 	"github.com/juju/juju/apiserver/common"
 	"github.com/juju/juju/cmd/modelcmd"
 	"github.com/juju/juju/environs"
@@ -125,8 +126,12 @@ func (c *killCommand) Run(ctx *cmd.Context) error {
 		return environs.Destroy(controllerName, controllerEnviron, store)
 	}
 
-	// Attempt to destroy the controller and all environments.
-	err = api.DestroyController(true)
+	// Attempt to destroy the controller and all models and storage.
+	destroyStorage := true
+	err = api.DestroyController(controller.DestroyControllerParams{
+		DestroyModels:  true,
+		DestroyStorage: &destroyStorage,
+	})
 	if err != nil {
 		ctx.Infof("Unable to destroy controller through the API: %s\nDestroying through provider", err)
 		return environs.Destroy(controllerName, controllerEnviron, store)
@@ -177,6 +182,16 @@ func (c *killCommand) DirectDestroyRemaining(ctx *cmd.Context, api destroyContro
 		logger.Errorf("unable to retrieve hosted model config: %v", err)
 	}
 	for _, model := range hostedConfig {
+		if model.Error != nil {
+			// We can only display model name here since
+			// the error coming from api can be anything
+			// including the parsing of the model owner tag.
+			// Only model name is guaranteed to be set in the result
+			// when an error is returned.
+			hasErrors = true
+			logger.Errorf("could not kill %s directly: %v", model.Name, model.Error)
+			continue
+		}
 		ctx.Infof("Killing %s/%s directly", model.Owner.Id(), model.Name)
 		cfg, err := config.New(config.NoDefaults, model.Config)
 		if err != nil {
