@@ -37,7 +37,7 @@ func NewOfferCommandForTest(
 }
 
 func NewShowEndpointsCommandForTest(store jujuclient.ClientStore, api ShowAPI) cmd.Command {
-	aCmd := &showCommand{newAPIFunc: func() (ShowAPI, error) {
+	aCmd := &showCommand{newAPIFunc: func(controllerName string) (ShowAPI, error) {
 		return api, nil
 	}}
 	aCmd.SetClientStore(store)
@@ -45,15 +45,26 @@ func NewShowEndpointsCommandForTest(store jujuclient.ClientStore, api ShowAPI) c
 }
 
 func NewListEndpointsCommandForTest(store jujuclient.ClientStore, api ListAPI) cmd.Command {
-	aCmd := &listCommand{newAPIFunc: func() (ListAPI, error) {
+	aCmd := &listCommand{
+		newAPIFunc: func() (ListAPI, error) {
+			return api, nil
+		},
+		refreshModels: noOpRefresh,
+	}
+	aCmd.SetClientStore(store)
+	return modelcmd.Wrap(aCmd)
+}
+
+func NewFindEndpointsCommandForTest(store jujuclient.ClientStore, api FindAPI) cmd.Command {
+	aCmd := &findCommand{newAPIFunc: func(controllerName string) (FindAPI, error) {
 		return api, nil
 	}}
 	aCmd.SetClientStore(store)
 	return modelcmd.WrapController(aCmd)
 }
 
-func NewFindEndpointsCommandForTest(store jujuclient.ClientStore, api FindAPI) cmd.Command {
-	aCmd := &findCommand{newAPIFunc: func() (FindAPI, error) {
+func NewRemoveCommandForTest(store jujuclient.ClientStore, api RemoveAPI) cmd.Command {
+	aCmd := &removeCommand{newAPIFunc: func(controllerName string) (RemoveAPI, error) {
 		return api, nil
 	}}
 	aCmd.SetClientStore(store)
