@@ -7,8 +7,8 @@ import (
 	"github.com/juju/cmd"
 	"github.com/juju/errors"
 	"github.com/juju/gnuflag"
+	"gopkg.in/juju/names.v2"
 
-	"github.com/juju/juju/apiserver/params"
 	"github.com/juju/juju/cmd/modelcmd"
 	"github.com/juju/juju/core/crossmodel"
 )
@@ -18,49 +18,47 @@ Find which offered application endpoints are available to the current user.
 
 This command is aimed for a user who wants to discover what endpoints are available to them.
 
-options:
--o, --output (= "")
-   specify an output file
---format (= tabular)
-   specify output format (tabular|json|yaml)
-
 Examples:
-   $ juju find-endpoints
-   $ juju find-endpoints mycontroller:
-   $ juju find-endpoints fred/prod
-   $ juju find-endpoints --interface mysql --url fred/prod
-   $ juju find-endpoints --url fred/prod.db2
+   $ juju find-offers
+   $ juju find-offers mycontroller:
+   $ juju find-offers fred/prod
+   $ juju find-offers --interface mysql
+   $ juju find-offers --url fred/prod.db2
+   $ juju find-offers --offer db2
    
 See also:
-   show-endpoints   
+   show-offer   
 `
 
 type findCommand struct {
 	RemoteEndpointsCommandBase
 
 	url            string
+	source         string
 	modelOwnerName string
 	modelName      string
 	offerName      string
 	interfaceName  string
-	endpoint       string
 
 	out        cmd.Output
-	newAPIFunc func() (FindAPI, error)
+	newAPIFunc func(string) (FindAPI, error)
 }
 
 // NewFindEndpointsCommand constructs command that
 // allows to find offered application endpoints.
 func NewFindEndpointsCommand() cmd.Command {
 	findCmd := &findCommand{}
-	findCmd.newAPIFunc = func() (FindAPI, error) {
-		return findCmd.NewRemoteEndpointsAPI()
+	findCmd.newAPIFunc = func(controllerName string) (FindAPI, error) {
+		return findCmd.NewRemoteEndpointsAPI(controllerName)
 	}
 	return modelcmd.WrapController(findCmd)
 }
 
 // Init implements Command.Init.
 func (c *findCommand) Init(args []string) (err error) {
+	if c.offerName != "" && c.url != "" {
+		return errors.New("cannot specify both a URL term and offer term")
+	}
 	url, err := cmd.ZeroOrOneArgs(args)
 	if err != nil {
 		return errors.Trace(err)
@@ -77,8 +75,8 @@ func (c *findCommand) Init(args []string) (err error) {
 // Info implements Command.Info.
 func (c *findCommand) Info() *cmd.Info {
 	return &cmd.Info{
-		Name:    "find-endpoints",
-		Purpose: "Find offered application endpoints",
+		Name:    "find-offers",
+		Purpose: "Find offered application endpoints.",
 		Doc:     findCommandDoc,
 	}
 }
@@ -86,9 +84,9 @@ func (c *findCommand) Info() *cmd.Info {
 // SetFlags implements Command.SetFlags.
 func (c *findCommand) SetFlags(f *gnuflag.FlagSet) {
 	c.RemoteEndpointsCommandBase.SetFlags(f)
-	f.StringVar(&c.url, "url", "", "application URL")
+	f.StringVar(&c.url, "url", "", "return results matching the offer URL")
 	f.StringVar(&c.interfaceName, "interface", "", "return results matching the interface name")
-	f.StringVar(&c.endpoint, "endpoint", "", "return results matching the endpoint name")
+	f.StringVar(&c.offerName, "offer", "", "return results matching the offer name")
 	c.out.AddFlags(f, "tabular", map[string]cmd.Formatter{
 		"yaml":    cmd.FormatYaml,
 		"json":    cmd.FormatJson,
@@ -101,7 +99,13 @@ func (c *findCommand) Run(ctx *cmd.Context) (err error) {
 	if err := c.validateOrSetURL(); err != nil {
 		return errors.Trace(err)
 	}
-	api, err := c.newAPIFunc()
+	accountDetails, err := c.CurrentAccountDetails()
+	if err != nil {
+		return err
+	}
+	loggedInUser := accountDetails.User
+
+	api, err := c.newAPIFunc(c.source)
 	if err != nil {
 		return err
 	}
@@ -111,13 +115,10 @@ func (c *findCommand) Run(ctx *cmd.Context) (err error) {
 		OwnerName: c.modelOwnerName,
 		ModelName: c.modelName,
 		OfferName: c.offerName,
-		// TODO(wallyworld): interface
-		// TODO(wallyworld): endpoint
 	}
-	if c.interfaceName != "" || c.endpoint != "" {
+	if c.interfaceName != "" {
 		filter.Endpoints = []crossmodel.EndpointFilterTerm{{
 			Interface: c.interfaceName,
-			Name:      c.endpoint,
 		}}
 	}
 	found, err := api.FindApplicationOffers(filter)
@@ -125,7 +126,7 @@ func (c *findCommand) Run(ctx *cmd.Context) (err error) {
 		return err
 	}
 
-	output, err := convertFoundOffers(found...)
+	output, err := convertFoundOffers(c.source, names.NewUserTag(loggedInUser), found...)
 	if err != nil {
 		return err
 	}
@@ -142,12 +143,18 @@ func (c *findCommand) validateOrSetURL() error {
 	}
 	if c.url == "" {
 		c.url = controllerName + ":"
+		c.source = controllerName
 		return nil
 	}
-	urlParts, err := crossmodel.ParseApplicationURLParts(c.url)
+	urlParts, err := crossmodel.ParseOfferURLParts(c.url)
 	if err != nil {
 		return errors.Trace(err)
 	}
+	if urlParts.Source != "" {
+		c.source = urlParts.Source
+	} else {
+		c.source = controllerName
+	}
 	user := urlParts.User
 	if user == "" {
 		accountDetails, err := c.CurrentAccountDetails()
@@ -159,41 +166,61 @@ func (c *findCommand) validateOrSetURL() error {
 	c.modelOwnerName = user
 	c.modelName = urlParts.ModelName
 	c.offerName = urlParts.ApplicationName
-	if urlParts.Source != "" && urlParts.Source != controllerName {
-		return errors.NotSupportedf("finding endpoints from another controller %q", urlParts.Source)
-	}
 	return nil
 }
 
 // FindAPI defines the API methods that cross model find command uses.
 type FindAPI interface {
 	Close() error
-	FindApplicationOffers(filters ...crossmodel.ApplicationOfferFilter) ([]params.ApplicationOffer, error)
+	FindApplicationOffers(filters ...crossmodel.ApplicationOfferFilter) ([]*crossmodel.ApplicationOfferDetails, error)
 }
 
 // ApplicationOfferResult defines the serialization behaviour of an application offer.
-// This is used in map-style yaml output where remote application URL is the key.
+// This is used in map-style yaml output where offer URL is the key.
 type ApplicationOfferResult struct {
 	// Access is the level of access the user has on the offer.
 	Access string `yaml:"access" json:"access"`
 
 	// Endpoints is the list of offered application endpoints.
 	Endpoints map[string]RemoteEndpoint `yaml:"endpoints" json:"endpoints"`
+
+	// Users are the users who can access the offer.
+	Users map[string]OfferUser `yaml:"users,omitempty" json:"users,omitempty"`
+}
+
+func accessForUser(user names.UserTag, users []crossmodel.OfferUserDetails) string {
+	for _, u := range users {
+		if u.UserName == user.Id() {
+			return string(u.Access)
+		}
+	}
+	return "-"
 }
 
 // convertFoundOffers takes any number of api-formatted remote applications and
 // creates a collection of ui-formatted applications.
-func convertFoundOffers(services ...params.ApplicationOffer) (map[string]ApplicationOfferResult, error) {
-	if len(services) == 0 {
+func convertFoundOffers(
+	store string, loggedInUser names.UserTag, offers ...*crossmodel.ApplicationOfferDetails,
+) (map[string]ApplicationOfferResult, error) {
+	if len(offers) == 0 {
 		return nil, nil
 	}
-	output := make(map[string]ApplicationOfferResult, len(services))
-	for _, one := range services {
+	output := make(map[string]ApplicationOfferResult, len(offers))
+	for _, one := range offers {
+		access := accessForUser(loggedInUser, one.Users)
 		app := ApplicationOfferResult{
-			Access:    one.Access,
+			Access:    access,
 			Endpoints: convertRemoteEndpoints(one.Endpoints...),
+			Users:     convertUsers(one.Users...),
 		}
-		output[one.OfferURL] = app
+		url, err := crossmodel.ParseOfferURL(one.OfferURL)
+		if err != nil {
+			return nil, err
+		}
+		if url.Source == "" {
+			url.Source = store
+		}
+		output[url.String()] = app
 	}
 	return output, nil
 }
