@@ -11,9 +11,8 @@ import (
 	"github.com/juju/errors"
 	jc "github.com/juju/testing/checkers"
 	gc "gopkg.in/check.v1"
-	"gopkg.in/juju/charm.v6-unstable"
+	"gopkg.in/juju/charm.v6"
 
-	"github.com/juju/juju/apiserver/params"
 	"github.com/juju/juju/cmd/juju/crossmodel"
 	jujucrossmodel "github.com/juju/juju/core/crossmodel"
 )
@@ -45,8 +44,8 @@ func (s *findSuite) TestFindNoArgs(c *gc.C) {
 		c,
 		[]string{},
 		`
-URL                   Access   Interfaces
-fred/test.hosted-db2  consume  http:db2, http:log
+Store   URL                   Access   Interfaces
+master  fred/test.hosted-db2  consume  http:db2, http:log
 
 `[1:],
 	)
@@ -56,8 +55,8 @@ func (s *findSuite) TestFindDuplicateUrl(c *gc.C) {
 	s.assertFindError(c, []string{"url", "--url", "urlparam"}, ".*URL term cannot be specified twice.*")
 }
 
-func (s *findSuite) TestFindDifferentController(c *gc.C) {
-	s.assertFindError(c, []string{"different:user/model.offer"}, `finding endpoints from another controller "different" not supported`)
+func (s *findSuite) TestFindOfferandUrl(c *gc.C) {
+	s.assertFindError(c, []string{"--offer", "offer", "--url", "urlparam"}, ".*cannot specify both a URL term and offer term.*")
 }
 
 func (s *findSuite) TestNoResults(c *gc.C) {
@@ -66,11 +65,14 @@ func (s *findSuite) TestNoResults(c *gc.C) {
 	s.mockAPI.expectedFilter = &jujucrossmodel.ApplicationOfferFilter{
 		OwnerName: "bob",
 		ModelName: "none",
+		Endpoints: []jujucrossmodel.EndpointFilterTerm{{
+			Interface: "mysql",
+		}},
 	}
-	s.mockAPI.results = []params.ApplicationOffer{}
+	s.mockAPI.results = []*jujucrossmodel.ApplicationOfferDetails{}
 	s.assertFindError(
 		c,
-		[]string{"--url", "none"},
+		[]string{"--url", "none", "--interface", "mysql"},
 		`no matching application offers found`,
 	)
 }
@@ -88,8 +90,8 @@ func (s *findSuite) TestSimpleFilter(c *gc.C) {
 		c,
 		[]string{"--format", "tabular", "--url", "fred/model.hosted-db2"},
 		`
-URL                    Access   Interfaces
-fred/model.hosted-db2  consume  http:db2, http:log
+Store   URL                    Access   Interfaces
+master  fred/model.hosted-db2  consume  http:db2, http:log
 
 `[1:],
 	)
@@ -102,16 +104,15 @@ func (s *findSuite) TestEndpointFilter(c *gc.C) {
 		ModelName: "model",
 		Endpoints: []jujucrossmodel.EndpointFilterTerm{{
 			Interface: "mysql",
-			Name:      "db",
 		}},
 	}
 	s.mockAPI.expectedModelName = "model"
 	s.assertFind(
 		c,
-		[]string{"--format", "tabular", "--url", "fred/model", "--endpoint", "db", "--interface", "mysql"},
+		[]string{"--format", "tabular", "--url", "fred/model", "--interface", "mysql"},
 		`
-URL                    Access   Interfaces
-fred/model.hosted-db2  consume  http:db2, http:log
+Store   URL                    Access   Interfaces
+master  fred/model.hosted-db2  consume  http:db2, http:log
 
 `[1:],
 	)
@@ -128,7 +129,7 @@ func (s *findSuite) TestFindYaml(c *gc.C) {
 		c,
 		[]string{"fred/model.hosted-db2", "--format", "yaml"},
 		`
-fred/model.hosted-db2:
+master:fred/model.hosted-db2:
   access: consume
   endpoints:
     db2:
@@ -137,6 +138,10 @@ fred/model.hosted-db2:
     log:
       interface: http
       role: provider
+  users:
+    bob:
+      display-name: Bob
+      access: consume
 `[1:],
 	)
 }
@@ -147,8 +152,22 @@ func (s *findSuite) TestFindTabular(c *gc.C) {
 		c,
 		[]string{"fred/model.hosted-db2", "--format", "tabular"},
 		`
-URL                    Access   Interfaces
-fred/model.hosted-db2  consume  http:db2, http:log
+Store   URL                    Access   Interfaces
+master  fred/model.hosted-db2  consume  http:db2, http:log
+
+`[1:],
+	)
+}
+
+func (s *findSuite) TestFindDifferentController(c *gc.C) {
+	s.mockAPI.expectedModelName = "model"
+	s.mockAPI.controllerName = "different"
+	s.assertFind(
+		c,
+		[]string{"fred/model.hosted-db2", "--format", "tabular"},
+		`
+Store      URL                    Access   Interfaces
+different  fred/model.hosted-db2  consume  http:db2, http:log
 
 `[1:],
 	)
@@ -169,17 +188,18 @@ func (s *findSuite) assertFindError(c *gc.C, args []string, expected string) {
 
 type mockFindAPI struct {
 	c                 *gc.C
+	controllerName    string
 	msg, offerName    string
 	expectedModelName string
 	expectedFilter    *jujucrossmodel.ApplicationOfferFilter
-	results           []params.ApplicationOffer
+	results           []*jujucrossmodel.ApplicationOfferDetails
 }
 
 func (s mockFindAPI) Close() error {
 	return nil
 }
 
-func (s mockFindAPI) FindApplicationOffers(filters ...jujucrossmodel.ApplicationOfferFilter) ([]params.ApplicationOffer, error) {
+func (s mockFindAPI) FindApplicationOffers(filters ...jujucrossmodel.ApplicationOfferFilter) ([]*jujucrossmodel.ApplicationOfferDetails, error) {
 	if s.msg != "" {
 		return nil, errors.New(s.msg)
 	}
@@ -191,13 +211,20 @@ func (s mockFindAPI) FindApplicationOffers(filters ...jujucrossmodel.Application
 	if s.results != nil {
 		return s.results, nil
 	}
-	return []params.ApplicationOffer{{
-		OfferURL:  fmt.Sprintf("fred/%s.%s", s.expectedModelName, s.offerName),
+	store := s.controllerName
+	if store == "" {
+		store = "master"
+	}
+	offerURL := fmt.Sprintf("%s:fred/%s.%s", store, s.expectedModelName, s.offerName)
+	return []*jujucrossmodel.ApplicationOfferDetails{{
+		OfferURL:  offerURL,
 		OfferName: s.offerName,
-		Endpoints: []params.RemoteEndpoint{
+		Endpoints: []charm.Relation{
 			{Name: "log", Interface: "http", Role: charm.RoleProvider},
 			{Name: "db2", Interface: "http", Role: charm.RoleRequirer},
 		},
-		Access: "consume",
+		Users: []jujucrossmodel.OfferUserDetails{{
+			UserName: "bob", DisplayName: "Bob", Access: "consume",
+		}},
 	}}, nil
 }
