@@ -12,6 +12,7 @@ import (
 	"github.com/juju/gnuflag"
 	"gopkg.in/juju/names.v2"
 
+	"github.com/juju/juju/api/applicationoffers"
 	"github.com/juju/juju/apiserver/params"
 	"github.com/juju/juju/cmd/modelcmd"
 	jujucrossmodel "github.com/juju/juju/core/crossmodel"
@@ -48,7 +49,7 @@ func NewOfferCommand() cmd.Command {
 }
 
 type offerCommand struct {
-	ApplicationOffersCommandBase
+	modelcmd.ControllerCommandBase
 	newAPIFunc    func() (OfferAPI, error)
 	refreshModels func(jujuclient.ClientStore, string) error
 	endpointsSpec string
@@ -66,11 +67,21 @@ type offerCommand struct {
 	QualifiedModelName string
 }
 
+// NewApplicationOffersAPI returns an application offers api for the root api endpoint
+// that the command returns.
+func (c *offerCommand) NewApplicationOffersAPI() (*applicationoffers.Client, error) {
+	root, err := c.NewAPIRoot()
+	if err != nil {
+		return nil, err
+	}
+	return applicationoffers.NewClient(root), nil
+}
+
 // Info implements Command.Info.
 func (c *offerCommand) Info() *cmd.Info {
 	return &cmd.Info{
 		Name:    "offer",
-		Purpose: "Offer application endpoints for use in other models",
+		Purpose: "Offer application endpoints for use in other models.",
 		Args:    "[model-name.]<application-name>:<endpoint-name>[,...] [offer-name]",
 		Doc:     offerCommandDoc,
 	}
@@ -87,15 +98,12 @@ func (c *offerCommand) Init(args []string) error {
 		argCount = 2
 		c.OfferName = args[1]
 	}
-	if c.OfferName == "" {
-		c.OfferName = c.Application
-	}
 	return cmd.CheckEmpty(args[argCount:])
 }
 
 // SetFlags implements Command.SetFlags.
 func (c *offerCommand) SetFlags(f *gnuflag.FlagSet) {
-	c.ApplicationOffersCommandBase.SetFlags(f)
+	c.ControllerCommandBase.SetFlags(f)
 }
 
 // Run implements Command.Run.
@@ -138,6 +146,9 @@ func (c *offerCommand) Run(ctx *cmd.Context) error {
 		return errors.Annotate(err, "getting model details")
 	}
 
+	if c.OfferName == "" {
+		c.OfferName = c.Application
+	}
 	// TODO (anastasiamac 2015-11-16) Add a sensible way for user to specify long-ish (at times) description when offering
 	results, err := api.Offer(modelDetails.ModelUUID, c.Application, c.Endpoints, c.OfferName, "")
 	if err != nil {
@@ -181,11 +192,12 @@ func (c *offerCommand) parseEndpoints(controllerName, arg string) error {
 	)
 	if modelNameArg != "" && !jujuclient.IsQualifiedModelName(modelNameArg) {
 		modelName = modelNameArg
-		account, err := c.ClientStore().AccountDetails(controllerName)
+		store := modelcmd.QualifyingClientStore{c.ClientStore()}
+		var err error
+		c.QualifiedModelName, err = store.QualifiedModelName(controllerName, modelName)
 		if err != nil {
 			return errors.Trace(err)
 		}
-		c.QualifiedModelName = jujuclient.JoinOwnerModelName(names.NewUserTag(account.User), modelName)
 	} else if modelNameArg != "" {
 		c.QualifiedModelName = modelNameArg
 		modelName, _, err = jujuclient.SplitModelName(modelNameArg)
