@@ -8,6 +8,7 @@ import (
 	"strings"
 
 	"github.com/juju/cmd"
+	"github.com/juju/cmd/cmdtesting"
 	"github.com/juju/errors"
 	jc "github.com/juju/testing/checkers"
 	gc "gopkg.in/check.v1"
@@ -70,6 +71,10 @@ func (s *AddMachineSuite) TestInit(c *gc.C) {
 			args:      []string{"ssh:user@10.10.0.3"},
 			count:     1,
 			placement: "ssh:user@10.10.0.3",
+		}, {
+			args:      []string{"winrm:user@10.10.0.3"},
+			count:     1,
+			placement: "winrm:user@10.10.0.3",
 		}, {
 			args:      []string{"zone=us-east-1a"},
 			count:     1,
@@ -89,7 +94,7 @@ func (s *AddMachineSuite) TestInit(c *gc.C) {
 	} {
 		c.Logf("test %d", i)
 		wrappedCommand, addCmd := machine.NewAddCommandForTest(s.fakeAddMachine, s.fakeAddMachine, s.fakeMachineManager)
-		err := testing.InitCommand(wrappedCommand, test.args)
+		err := cmdtesting.InitCommand(wrappedCommand, test.args)
 		if test.errorString == "" {
 			c.Check(err, jc.ErrorIsNil)
 			c.Check(addCmd.Series, gc.Equals, test.series)
@@ -108,13 +113,13 @@ func (s *AddMachineSuite) TestInit(c *gc.C) {
 
 func (s *AddMachineSuite) run(c *gc.C, args ...string) (*cmd.Context, error) {
 	add, _ := machine.NewAddCommandForTest(s.fakeAddMachine, s.fakeAddMachine, s.fakeMachineManager)
-	return testing.RunCommand(c, add, args...)
+	return cmdtesting.RunCommand(c, add, args...)
 }
 
 func (s *AddMachineSuite) TestAddMachine(c *gc.C) {
 	context, err := s.run(c)
 	c.Assert(err, jc.ErrorIsNil)
-	c.Assert(testing.Stderr(context), gc.Equals, "created machine 0\n")
+	c.Assert(cmdtesting.Stderr(context), gc.Equals, "created machine 0\n")
 
 	c.Assert(s.fakeAddMachine.args, gc.HasLen, 1)
 	param := s.fakeAddMachine.args[0]
@@ -129,26 +134,26 @@ func (s *AddMachineSuite) TestAddMachineUnauthorizedMentionsJujuGrant(c *gc.C) {
 		Code:    params.CodeUnauthorized,
 	}
 	ctx, _ := s.run(c)
-	errString := strings.Replace(testing.Stderr(ctx), "\n", " ", -1)
+	errString := strings.Replace(cmdtesting.Stderr(ctx), "\n", " ", -1)
 	c.Assert(errString, gc.Matches, `.*juju grant.*`)
 }
 
 func (s *AddMachineSuite) TestSSHPlacement(c *gc.C) {
-	s.PatchValue(machine.ManualProvisioner, func(args manual.ProvisionMachineArgs) (string, error) {
+	s.PatchValue(machine.SSHProvisioner, func(args manual.ProvisionMachineArgs) (string, error) {
 		return "42", nil
 	})
 	context, err := s.run(c, "ssh:10.1.2.3")
 	c.Assert(err, jc.ErrorIsNil)
-	c.Assert(testing.Stderr(context), gc.Equals, "created machine 42\n")
+	c.Assert(cmdtesting.Stderr(context), gc.Equals, "created machine 42\n")
 }
 
 func (s *AddMachineSuite) TestSSHPlacementError(c *gc.C) {
-	s.PatchValue(machine.ManualProvisioner, func(args manual.ProvisionMachineArgs) (string, error) {
+	s.PatchValue(machine.SSHProvisioner, func(args manual.ProvisionMachineArgs) (string, error) {
 		return "", errors.New("failed to initialize warp core")
 	})
 	context, err := s.run(c, "ssh:10.1.2.3")
 	c.Assert(err, gc.ErrorMatches, "failed to initialize warp core")
-	c.Assert(testing.Stderr(context), gc.Equals, "")
+	c.Assert(cmdtesting.Stderr(context), gc.Equals, "")
 }
 
 func (s *AddMachineSuite) TestParamsPassedOn(c *gc.C) {
@@ -179,7 +184,7 @@ failed to create 2 machines
 `
 	context, err := s.run(c, "-n", "3")
 	c.Assert(err, gc.ErrorMatches, "something went wrong, something went wrong")
-	c.Assert(testing.Stderr(context), gc.Equals, expectedOutput)
+	c.Assert(cmdtesting.Stderr(context), gc.Equals, expectedOutput)
 }
 
 func (s *AddMachineSuite) TestBlockedError(c *gc.C) {
@@ -250,6 +255,10 @@ func (f *fakeAddMachineAPI) ForceDestroyMachines(machines ...string) error {
 	return errors.NotImplementedf("ForceDestroyMachines")
 }
 
+func (f *fakeAddMachineAPI) DestroyMachinesWithParams(force, keep bool, machines ...string) error {
+	return errors.NotImplementedf("ForceDestroyMachinesWithParams")
+}
+
 func (f *fakeAddMachineAPI) ProvisioningScript(params.ProvisioningScriptParams) (script string, err error) {
 	return "", errors.NotImplementedf("ProvisioningScript")
 }
