@@ -4,6 +4,7 @@
 package model
 
 import (
+	"bytes"
 	"fmt"
 	"time"
 
@@ -18,11 +19,11 @@ import (
 	"github.com/juju/juju/api/base"
 	"github.com/juju/juju/api/modelconfig"
 	"github.com/juju/juju/api/modelmanager"
+	"github.com/juju/juju/api/storage"
 	"github.com/juju/juju/apiserver/params"
 	jujucmd "github.com/juju/juju/cmd"
 	"github.com/juju/juju/cmd/juju/block"
 	"github.com/juju/juju/cmd/modelcmd"
-	"github.com/juju/juju/jujuclient"
 )
 
 const (
@@ -34,7 +35,6 @@ var logger = loggo.GetLogger("juju.cmd.juju.model")
 // NewDestroyCommand returns a command used to destroy a model.
 func NewDestroyCommand() cmd.Command {
 	destroyCmd := &destroyCommand{}
-	destroyCmd.RefreshModels = destroyCmd.ModelCommandBase.RefreshModels
 	destroyCmd.sleepFunc = time.Sleep
 	return modelcmd.Wrap(
 		destroyCmd,
@@ -46,19 +46,17 @@ func NewDestroyCommand() cmd.Command {
 // destroyCommand destroys the specified model.
 type destroyCommand struct {
 	modelcmd.ModelCommandBase
-	// RefreshModels hides the RefreshModels function defined
-	// in ModelCommandBase. This allows overriding for testing.
-	// NOTE: ideal solution would be to have the base implement a method
-	// like store.ModelByName which auto-refreshes.
-	RefreshModels func(jujuclient.ClientStore, string) error
 
 	// sleepFunc is used when calling the timed function to get model status updates.
 	sleepFunc func(time.Duration)
 
-	envName   string
-	assumeYes bool
-	api       DestroyModelAPI
-	configApi ModelConfigAPI
+	envName        string
+	assumeYes      bool
+	destroyStorage bool
+	releaseStorage bool
+	api            DestroyModelAPI
+	configAPI      ModelConfigAPI
+	storageAPI     StorageAPI
 }
 
 var destroyDoc = `
@@ -68,10 +66,16 @@ there. Due to the irreversible nature of the command, it will prompt for
 confirmation (unless overridden with the '-y' option) before taking any
 action.
 
+If there is persistent storage in any of the models managed by the
+controller, then you must choose to either destroy or release the
+storage, using --destroy-storage or --release-storage respectively.
+
 Examples:
 
     juju destroy-model test
     juju destroy-model -y mymodel
+    juju destroy-model -y mymodel --destroy-storage
+    juju destroy-model -y mymodel --release-storage
 
 See also:
     destroy-controller
@@ -86,7 +90,8 @@ Continue [y/N]? `[1:]
 // API that the destroy command calls. It is exported for mocking in tests.
 type DestroyModelAPI interface {
 	Close() error
-	DestroyModel(names.ModelTag) error
+	BestAPIVersion() int
+	DestroyModel(tag names.ModelTag, destroyStorage *bool) error
 	ModelStatus(models ...names.ModelTag) ([]base.ModelStatus, error)
 }
 
@@ -112,10 +117,15 @@ func (c *destroyCommand) SetFlags(f *gnuflag.FlagSet) {
 	c.ModelCommandBase.SetFlags(f)
 	f.BoolVar(&c.assumeYes, "y", false, "Do not prompt for confirmation")
 	f.BoolVar(&c.assumeYes, "yes", false, "")
+	f.BoolVar(&c.destroyStorage, "destroy-storage", false, "Destroy all storage instances in the model")
+	f.BoolVar(&c.releaseStorage, "release-storage", false, "Release all storage instances from the model, and management of the controller, without destroying them")
 }
 
 // Init implements Command.Init.
 func (c *destroyCommand) Init(args []string) error {
+	if c.destroyStorage && c.releaseStorage {
+		return errors.New("--destroy-storage and --release-storage cannot both be specified")
+	}
 	switch len(args) {
 	case 0:
 		return errors.New("no model specified")
@@ -138,8 +148,8 @@ func (c *destroyCommand) getAPI() (DestroyModelAPI, error) {
 }
 
 func (c *destroyCommand) getModelConfigAPI() (ModelConfigAPI, error) {
-	if c.configApi != nil {
-		return c.configApi, nil
+	if c.configAPI != nil {
+		return c.configAPI, nil
 	}
 	root, err := c.NewAPIRoot()
 	if err != nil {
@@ -148,6 +158,17 @@ func (c *destroyCommand) getModelConfigAPI() (ModelConfigAPI, error) {
 	return modelconfig.NewClient(root), nil
 }
 
+func (c *destroyCommand) getStorageAPI() (StorageAPI, error) {
+	if c.storageAPI != nil {
+		return c.storageAPI, nil
+	}
+	root, err := c.NewAPIRoot()
+	if err != nil {
+		return nil, errors.Trace(err)
+	}
+	return storage.NewClient(root), nil
+}
+
 // Run implements Command.Run
 func (c *destroyCommand) Run(ctx *cmd.Context) error {
 	store := c.ClientStore()
@@ -155,25 +176,14 @@ func (c *destroyCommand) Run(ctx *cmd.Context) error {
 	if err != nil {
 		return errors.Trace(err)
 	}
-	modelName, err := c.ModelName()
-	if err != nil {
-		return errors.Trace(err)
-	}
 
 	controllerDetails, err := store.ControllerByName(controllerName)
 	if err != nil {
 		return errors.Annotate(err, "cannot read controller details")
 	}
-	modelDetails, err := store.ModelByName(controllerName, modelName)
-	if errors.IsNotFound(err) {
-		if err := c.RefreshModels(store, controllerName); err != nil {
-			return errors.Annotate(err, "refreshing models cache")
-		}
-		// Now try again.
-		modelDetails, err = store.ModelByName(controllerName, modelName)
-	}
+	modelName, modelDetails, err := c.ModelDetails()
 	if err != nil {
-		return errors.Annotate(err, "cannot read model info")
+		return errors.Trace(err)
 	}
 
 	if modelDetails.ModelUUID == controllerDetails.ControllerUUID {
@@ -195,26 +205,70 @@ func (c *destroyCommand) Run(ctx *cmd.Context) error {
 	}
 	defer api.Close()
 
-	configApi, err := c.getModelConfigAPI()
+	configAPI, err := c.getModelConfigAPI()
 	if err != nil {
 		return errors.Annotate(err, "cannot connect to API")
 	}
-	defer configApi.Close()
+	defer configAPI.Close()
 
 	// Check if the model has an SLA set.
 	slaIsSet := false
-	slaLevel, err := configApi.SLALevel()
+	slaLevel, err := configAPI.SLALevel()
 	if err == nil {
 		slaIsSet = slaLevel != "" && slaLevel != slaUnsupported
 	} else {
 		logger.Debugf("could not determine model SLA level: %v", err)
 	}
 
+	if api.BestAPIVersion() < 4 {
+		// Versions before 4 support only destroying the storage,
+		// and will not raise an error if there is storage in the
+		// controller. Force the user to specify up-front.
+		if c.releaseStorage {
+			return errors.New("this juju controller only supports destroying storage")
+		}
+		if !c.destroyStorage {
+			storageAPI, err := c.getStorageAPI()
+			if err != nil {
+				return errors.Trace(err)
+			}
+			defer storageAPI.Close()
+
+			storage, err := storageAPI.ListStorageDetails()
+			if err != nil {
+				return errors.Trace(err)
+			}
+			if len(storage) > 0 {
+				return errors.Errorf(`cannot destroy model %q
+
+Destroying this model will destroy the storage, but you
+have not indicated that you want to do that.
+
+Please run the the command again with --destroy-storage
+to confirm that you want to destroy the storage along
+with the model.
+
+If instead you want to keep the storage, you must first
+upgrade the controller to version 2.3 or greater.
+
+`, modelName)
+			}
+			c.destroyStorage = true
+		}
+	}
+
 	// Attempt to destroy the model.
 	ctx.Infof("Destroying model")
-	err = api.DestroyModel(names.NewModelTag(modelDetails.ModelUUID))
-	if err != nil {
-		return c.handleError(errors.Annotate(err, "cannot destroy model"), modelName)
+	var destroyStorage *bool
+	if c.destroyStorage || c.releaseStorage {
+		destroyStorage = &c.destroyStorage
+	}
+	modelTag := names.NewModelTag(modelDetails.ModelUUID)
+	if err := api.DestroyModel(modelTag, destroyStorage); err != nil {
+		return c.handleError(
+			modelTag, modelName, api,
+			errors.Annotate(err, "cannot destroy model"),
+		)
 	}
 
 	// Wait for model to be destroyed.
@@ -263,6 +317,8 @@ func (c *destroyCommand) removeModelBudget(uuid string) error {
 type modelData struct {
 	machineCount     int
 	applicationCount int
+	volumeCount      int
+	filesystemCount  int
 }
 
 // newTimedModelStatus returns a function which waits a given period of time
@@ -271,9 +327,16 @@ func newTimedModelStatus(ctx *cmd.Context, api DestroyModelAPI, tag names.ModelT
 	return func(wait time.Duration) *modelData {
 		sleepFunc(wait)
 		status, err := api.ModelStatus(tag)
+		if err == nil && len(status) == 1 && status[0].Error != nil {
+			// In 2.2 an error of one model generate an error for the entire request,
+			// in 2.3 this was corrected to just be an error for the requested model.
+			err = status[0].Error
+		}
 		if err != nil {
 			if params.ErrCode(err) != params.CodeNotFound {
 				ctx.Infof("Unable to get the model status from the API: %v.", err)
+			} else {
+				ctx.Infof("Model destroyed.")
 			}
 			return nil
 		}
@@ -284,6 +347,8 @@ func newTimedModelStatus(ctx *cmd.Context, api DestroyModelAPI, tag names.ModelT
 		return &modelData{
 			machineCount:     status[0].HostedMachineCount,
 			applicationCount: status[0].ServiceCount,
+			volumeCount:      len(status[0].Volumes),
+			filesystemCount:  len(status[0].Filesystems),
 		}
 	}
 }
@@ -299,20 +364,97 @@ func formatDestroyModelInfo(data *modelData) string {
 	if data.applicationCount > 0 {
 		out += fmt.Sprintf(", %d application(s)", data.applicationCount)
 	}
+	if data.volumeCount > 0 {
+		out += fmt.Sprintf(", %d volume(s)", data.volumeCount)
+	}
+	if data.filesystemCount > 0 {
+		out += fmt.Sprintf(", %d filesystems(s)", data.filesystemCount)
+	}
 	return out
 }
 
-func (c *destroyCommand) handleError(err error, modelName string) error {
-	if err == nil {
-		return nil
-	}
+func (c *destroyCommand) handleError(
+	modelTag names.ModelTag,
+	modelName string,
+	api DestroyModelAPI,
+	err error,
+) error {
 	if params.IsCodeOperationBlocked(err) {
 		return block.ProcessBlockedError(err, block.BlockDestroy)
 	}
+	if params.IsCodeHasPersistentStorage(err) {
+		return handlePersistentStorageError(modelTag, modelName, api)
+	}
 	logger.Errorf(`failed to destroy model %q`, modelName)
 	return err
 }
 
+func handlePersistentStorageError(
+	modelTag names.ModelTag,
+	modelName string,
+	api DestroyModelAPI,
+) error {
+	modelStatuses, err := api.ModelStatus(modelTag)
+	if err != nil {
+		return errors.Annotate(err, "getting model status")
+	}
+	if l := len(modelStatuses); l != 1 {
+		return errors.Errorf("error finding model status: expected one result, got %d", l)
+	}
+	modelStatus := modelStatuses[0]
+	if modelStatus.Error != nil {
+		if errors.IsNotFound(modelStatus.Error) {
+			// This most likely occurred because a model was
+			// destroyed half-way through the call.
+			return nil
+		}
+		return errors.Annotate(err, "getting model status")
+	}
+
+	var buf bytes.Buffer
+	var persistentVolumes, persistentFilesystems int
+	for _, v := range modelStatus.Volumes {
+		if v.Detachable {
+			persistentVolumes++
+		}
+	}
+	for _, f := range modelStatus.Filesystems {
+		if f.Detachable {
+			persistentFilesystems++
+		}
+	}
+	if n := persistentVolumes; n > 0 {
+		fmt.Fprintf(&buf, "%d volume", n)
+		if n > 1 {
+			buf.WriteRune('s')
+		}
+		if persistentFilesystems > 0 {
+			buf.WriteString(" and ")
+		}
+	}
+	if n := persistentFilesystems; n > 0 {
+		fmt.Fprintf(&buf, "%d filesystem", n)
+		if n > 1 {
+			buf.WriteRune('s')
+		}
+	}
+
+	return errors.Errorf(`cannot destroy model %q
+
+The model has persistent storage remaining:
+	%s
+
+To destroy the storage, run the destroy-model
+command again with the "--destroy-storage" flag.
+
+To release the storage from Juju's management
+without destroying it, use the "--release-storage"
+flag instead. The storage can then be imported
+into another Juju model.
+
+`, modelName, buf.String())
+}
+
 var getBudgetAPIClient = getBudgetAPIClientImpl
 
 func getBudgetAPIClientImpl(bakeryClient *httpbakery.Client) BudgetAPIClient {
@@ -323,3 +465,9 @@ func getBudgetAPIClientImpl(bakeryClient *httpbakery.Client) BudgetAPIClient {
 type BudgetAPIClient interface {
 	DeleteBudget(string) (string, error)
 }
+
+// StorageAPI defines the storage client API interface.
+type StorageAPI interface {
+	Close() error
+	ListStorageDetails() ([]params.StorageDetails, error)
+}
