@@ -31,6 +31,7 @@ type DestroySuite struct {
 	testing.FakeJujuXDGDataHomeSuite
 	api             *fakeAPI
 	configAPI       *fakeConfigAPI
+	storageAPI      *mockStorageAPI
 	stub            *jutesting.Stub
 	budgetAPIClient *mockBudgetAPIClient
 	store           *jujuclient.MemStore
@@ -41,25 +42,43 @@ var _ = gc.Suite(&DestroySuite{})
 
 // fakeDestroyAPI mocks out the cient API
 type fakeAPI struct {
+	*jutesting.Stub
 	err             error
 	env             map[string]interface{}
 	statusCallCount int
+	bestAPIVersion  int
 	modelInfoErr    []*params.Error
 }
 
 func (f *fakeAPI) Close() error { return nil }
 
-func (f *fakeAPI) DestroyModel(names.ModelTag) error {
-	return f.err
+func (f *fakeAPI) BestAPIVersion() int {
+	return f.bestAPIVersion
+}
+
+func (f *fakeAPI) DestroyModel(tag names.ModelTag, destroyStorage *bool) error {
+	f.MethodCall(f, "DestroyModel", tag, destroyStorage)
+	return f.NextErr()
 }
 
 func (f *fakeAPI) ModelStatus(models ...names.ModelTag) ([]base.ModelStatus, error) {
-	var err *params.Error = &params.Error{Code: params.CodeNotFound}
+	var err error
 	if f.statusCallCount < len(f.modelInfoErr) {
-		err = f.modelInfoErr[f.statusCallCount]
+		modelInfoErr := f.modelInfoErr[f.statusCallCount]
+		if modelInfoErr != nil {
+			err = modelInfoErr
+		}
+	} else {
+		err = &params.Error{Code: params.CodeNotFound}
 	}
 	f.statusCallCount++
-	return []base.ModelStatus{{}}, err
+	return []base.ModelStatus{{
+		Volumes: []base.Volume{
+			{Detachable: true},
+			{Detachable: true},
+		},
+		Filesystems: []base.Filesystem{{Detachable: true}},
+	}}, err
 }
 
 // faceConfiAPI mocks out the ModelConfigAPI.
@@ -76,10 +95,13 @@ func (f *fakeConfigAPI) Close() error { return nil }
 
 func (s *DestroySuite) SetUpTest(c *gc.C) {
 	s.FakeJujuXDGDataHomeSuite.SetUpTest(c)
-	s.api = &fakeAPI{}
-	s.api.err = nil
+	s.stub = &jutesting.Stub{}
+	s.api = &fakeAPI{
+		Stub:           s.stub,
+		bestAPIVersion: 4,
+	}
 	s.configAPI = &fakeConfigAPI{}
-	s.configAPI.err = nil
+	s.storageAPI = &mockStorageAPI{Stub: s.stub}
 
 	s.store = jujuclient.NewMemStore()
 	s.store.CurrentControllerName = "test1"
@@ -95,18 +117,17 @@ func (s *DestroySuite) SetUpTest(c *gc.C) {
 	}
 	s.sleep = func(time.Duration) {}
 
-	s.stub = &jutesting.Stub{}
 	s.budgetAPIClient = &mockBudgetAPIClient{Stub: s.stub}
 	s.PatchValue(model.GetBudgetAPIClient, func(*httpbakery.Client) model.BudgetAPIClient { return s.budgetAPIClient })
 }
 
 func (s *DestroySuite) runDestroyCommand(c *gc.C, args ...string) (*cmd.Context, error) {
-	cmd := model.NewDestroyCommandForTest(s.api, s.configAPI, noOpRefresh, s.store, s.sleep)
+	cmd := model.NewDestroyCommandForTest(s.api, s.configAPI, s.storageAPI, noOpRefresh, s.store, s.sleep)
 	return cmdtesting.RunCommand(c, cmd, args...)
 }
 
 func (s *DestroySuite) NewDestroyCommand() cmd.Command {
-	return model.NewDestroyCommandForTest(s.api, s.configAPI, noOpRefresh, s.store, s.sleep)
+	return model.NewDestroyCommandForTest(s.api, s.configAPI, s.storageAPI, noOpRefresh, s.store, s.sleep)
 }
 
 func checkModelExistsInStore(c *gc.C, name string, store jujuclient.ClientStore) {
@@ -143,14 +164,14 @@ func (s *DestroySuite) TestDestroyUnknownModelCallsRefresh(c *gc.C) {
 		return nil
 	}
 
-	cmd := model.NewDestroyCommandForTest(s.api, s.configAPI, refresh, s.store, s.sleep)
+	cmd := model.NewDestroyCommandForTest(s.api, s.configAPI, s.storageAPI, refresh, s.store, s.sleep)
 	_, err := cmdtesting.RunCommand(c, cmd, "foo")
 	c.Check(called, jc.IsTrue)
-	c.Check(err, gc.ErrorMatches, `cannot read model info: model test1:admin/foo not found`)
+	c.Check(err, gc.ErrorMatches, `model test1:admin/foo not found`)
 }
 
 func (s *DestroySuite) TestDestroyCannotConnectToAPI(c *gc.C) {
-	s.api.err = errors.New("connection refused")
+	s.stub.SetErrors(errors.New("connection refused"))
 	_, err := s.runDestroyCommand(c, "test2", "-y")
 	c.Assert(err, gc.ErrorMatches, "cannot destroy model: connection refused")
 	c.Check(c.GetTestLog(), jc.Contains, "failed to destroy model \"test2\"")
@@ -168,7 +189,9 @@ func (s *DestroySuite) TestDestroy(c *gc.C) {
 	_, err := s.runDestroyCommand(c, "test2", "-y")
 	c.Assert(err, jc.ErrorIsNil)
 	checkModelRemovedFromStore(c, "test1:admin/test2", s.store)
-	s.stub.CheckNoCalls(c)
+	s.stub.CheckCalls(c, []jutesting.StubCall{
+		{"DestroyModel", []interface{}{names.NewModelTag("test2-uuid"), (*bool)(nil)}},
+	})
 }
 
 func (s *DestroySuite) TestDestroyBlocks(c *gc.C) {
@@ -181,7 +204,7 @@ func (s *DestroySuite) TestDestroyBlocks(c *gc.C) {
 }
 
 func (s *DestroySuite) TestFailedDestroyModel(c *gc.C) {
-	s.api.err = errors.New("permission denied")
+	s.stub.SetErrors(errors.New("permission denied"))
 	_, err := s.runDestroyCommand(c, "test1:test2", "-y")
 	c.Assert(err, gc.ErrorMatches, "cannot destroy model: permission denied")
 	checkModelExistsInStore(c, "test1:admin/test2", s.store)
@@ -191,26 +214,99 @@ func (s *DestroySuite) TestDestroyWithUnsupportedSLA(c *gc.C) {
 	s.configAPI.slaLevel = "unsupported"
 	_, err := s.runDestroyCommand(c, "test1:test2", "-y")
 	c.Assert(err, jc.ErrorIsNil)
-	s.stub.CheckNoCalls(c)
+	s.stub.CheckCallNames(c, "DestroyModel")
 }
 
 func (s *DestroySuite) TestDestroyWithSupportedSLA(c *gc.C) {
 	s.configAPI.slaLevel = "standard"
 	_, err := s.runDestroyCommand(c, "test2", "-y")
 	c.Assert(err, jc.ErrorIsNil)
-	s.stub.CheckCalls(c, []jutesting.StubCall{{
-		"DeleteBudget", []interface{}{"test2-uuid"},
-	}})
+	s.stub.CheckCalls(c, []jutesting.StubCall{
+		{"DestroyModel", []interface{}{names.NewModelTag("test2-uuid"), (*bool)(nil)}},
+		{"DeleteBudget", []interface{}{"test2-uuid"}},
+	})
 }
 
 func (s *DestroySuite) TestDestroyWithSupportedSLAFailure(c *gc.C) {
 	s.configAPI.slaLevel = "standard"
-	s.stub.SetErrors(errors.New("bah"))
+	s.stub.SetErrors(nil, errors.New("bah"))
+	_, err := s.runDestroyCommand(c, "test2", "-y")
+	c.Assert(err, jc.ErrorIsNil)
+	s.stub.CheckCalls(c, []jutesting.StubCall{
+		{"DestroyModel", []interface{}{names.NewModelTag("test2-uuid"), (*bool)(nil)}},
+		{"DeleteBudget", []interface{}{"test2-uuid"}},
+	})
+}
+
+func (s *DestroySuite) TestDestroyDestroyStorage(c *gc.C) {
+	_, err := s.runDestroyCommand(c, "test2", "-y", "--destroy-storage")
+	c.Assert(err, jc.ErrorIsNil)
+	destroyStorage := true
+	s.stub.CheckCalls(c, []jutesting.StubCall{
+		{"DestroyModel", []interface{}{names.NewModelTag("test2-uuid"), &destroyStorage}},
+	})
+}
+
+func (s *DestroySuite) TestDestroyReleaseStorage(c *gc.C) {
+	_, err := s.runDestroyCommand(c, "test2", "-y", "--release-storage")
+	c.Assert(err, jc.ErrorIsNil)
+	destroyStorage := false
+	s.stub.CheckCalls(c, []jutesting.StubCall{
+		{"DestroyModel", []interface{}{names.NewModelTag("test2-uuid"), &destroyStorage}},
+	})
+}
+
+func (s *DestroySuite) TestDestroyDestroyReleaseStorageFlagsMutuallyExclusive(c *gc.C) {
+	_, err := s.runDestroyCommand(c, "test2", "-y", "--destroy-storage", "--release-storage")
+	c.Assert(err, gc.ErrorMatches, "--destroy-storage and --release-storage cannot both be specified")
+}
+
+func (s *DestroySuite) TestDestroyDestroyStorageFlagUnspecified(c *gc.C) {
+	s.stub.SetErrors(&params.Error{Code: params.CodeHasPersistentStorage})
+	s.api.modelInfoErr = []*params.Error{nil}
+	_, err := s.runDestroyCommand(c, "test2", "-y")
+	c.Assert(err, gc.ErrorMatches, `cannot destroy model "test2"
+
+The model has persistent storage remaining:
+	2 volumes and 1 filesystem
+
+To destroy the storage, run the destroy-model
+command again with the "--destroy-storage" flag.
+
+To release the storage from Juju's management
+without destroying it, use the "--release-storage"
+flag instead. The storage can then be imported
+into another Juju model.
+
+`)
+}
+
+func (s *DestroySuite) TestDestroyDestroyStorageFlagUnspecifiedOldController(c *gc.C) {
+	s.api.bestAPIVersion = 3
+	s.storageAPI.storage = []params.StorageDetails{{}}
+
+	_, err := s.runDestroyCommand(c, "test2", "-y")
+	c.Assert(err, gc.ErrorMatches, `cannot destroy model "test2"
+
+Destroying this model will destroy the storage, but you
+have not indicated that you want to do that.
+
+Please run the the command again with --destroy-storage
+to confirm that you want to destroy the storage along
+with the model.
+
+If instead you want to keep the storage, you must first
+upgrade the controller to version 2.3 or greater.
+
+`)
+}
+
+func (s *DestroySuite) TestDestroyDestroyStorageFlagUnspecifiedOldControllerNoStorage(c *gc.C) {
+	s.api.bestAPIVersion = 3
+	s.storageAPI.storage = nil // no storage in model
+
 	_, err := s.runDestroyCommand(c, "test2", "-y")
 	c.Assert(err, jc.ErrorIsNil)
-	s.stub.CheckCalls(c, []jutesting.StubCall{{
-		"DeleteBudget", []interface{}{"test2-uuid"},
-	}})
 }
 
 func (s *DestroySuite) resetModel(c *gc.C) {
@@ -273,7 +369,7 @@ func (s *DestroySuite) TestDestroyCommandConfirmation(c *gc.C) {
 }
 
 func (s *DestroySuite) TestBlockedDestroy(c *gc.C) {
-	s.api.err = common.OperationBlockedError("TestBlockedDestroy")
+	s.stub.SetErrors(common.OperationBlockedError("TestBlockedDestroy"))
 	_, err := s.runDestroyCommand(c, "test2", "-y")
 	testing.AssertOperationWasBlocked(c, err, ".*TestBlockedDestroy.*")
 }
@@ -287,3 +383,15 @@ func (c *mockBudgetAPIClient) DeleteBudget(model string) (string, error) {
 	c.MethodCall(c, "DeleteBudget", model)
 	return "Budget removed.", c.NextErr()
 }
+
+type mockStorageAPI struct {
+	*jutesting.Stub
+	storage []params.StorageDetails
+}
+
+func (*mockStorageAPI) Close() error { return nil }
+
+func (m *mockStorageAPI) ListStorageDetails() ([]params.StorageDetails, error) {
+	m.MethodCall(m, "ListStorageDetails")
+	return m.storage, m.NextErr()
+}
