@@ -44,8 +44,9 @@ func NewRetryProvisioningCommandForTest(api RetryProvisioningAPI) cmd.Command {
 
 // NewShowCommandForTest returns a ShowCommand with the api provided as specified.
 func NewShowCommandForTest(api ShowModelAPI, refreshFunc func(jujuclient.ClientStore, string) error, store jujuclient.ClientStore) cmd.Command {
-	cmd := &showModelCommand{api: api, RefreshModels: refreshFunc}
+	cmd := &showModelCommand{api: api}
 	cmd.SetClientStore(store)
+	cmd.SetModelRefresh(refreshFunc)
 	return modelcmd.Wrap(cmd, modelcmd.WrapSkipModelFlags)
 }
 
@@ -53,30 +54,32 @@ func NewShowCommandForTest(api ShowModelAPI, refreshFunc func(jujuclient.ClientS
 func NewDumpCommandForTest(api DumpModelAPI, store jujuclient.ClientStore) cmd.Command {
 	cmd := &dumpCommand{api: api}
 	cmd.SetClientStore(store)
-	return modelcmd.WrapController(cmd)
+	return modelcmd.Wrap(cmd)
 }
 
 // NewDumpDBCommandForTest returns a DumpDBCommand with the api provided as specified.
 func NewDumpDBCommandForTest(api DumpDBAPI, store jujuclient.ClientStore) cmd.Command {
 	cmd := &dumpDBCommand{api: api}
 	cmd.SetClientStore(store)
-	return modelcmd.WrapController(cmd)
+	return modelcmd.Wrap(cmd)
 }
 
 // NewDestroyCommandForTest returns a DestroyCommand with the api provided as specified.
 func NewDestroyCommandForTest(
 	api DestroyModelAPI,
-	configApi ModelConfigAPI,
+	configAPI ModelConfigAPI,
+	storageAPI StorageAPI,
 	refreshFunc func(jujuclient.ClientStore, string) error, store jujuclient.ClientStore,
 	sleepFunc func(time.Duration),
 ) cmd.Command {
 	cmd := &destroyCommand{
-		api:           api,
-		configApi:     configApi,
-		RefreshModels: refreshFunc,
-		sleepFunc:     sleepFunc,
+		api:        api,
+		configAPI:  configAPI,
+		storageAPI: storageAPI,
+		sleepFunc:  sleepFunc,
 	}
 	cmd.SetClientStore(store)
+	cmd.SetModelRefresh(refreshFunc)
 	return modelcmd.Wrap(
 		cmd,
 		modelcmd.WrapSkipDefaultModel,
