@@ -14,7 +14,6 @@ import (
 	"github.com/juju/juju/apiserver/common"
 	"github.com/juju/juju/cmd/juju/model"
 	"github.com/juju/juju/core/crossmodel"
-	"github.com/juju/juju/feature"
 	"github.com/juju/juju/jujuclient"
 	"github.com/juju/juju/testing"
 )
@@ -36,7 +35,6 @@ const (
 )
 
 func (s *grantRevokeSuite) SetUpTest(c *gc.C) {
-	s.SetInitialFeatureFlags(feature.CrossModelRelations)
 	s.FakeJujuXDGDataHomeSuite.SetUpTest(c)
 	s.fakeModelAPI = &fakeModelGrantRevokeAPI{}
 	s.fakeOffersAPI = &fakeOffersGrantRevokeAPI{}
@@ -149,11 +147,11 @@ func (s *grantSuite) TestInitOffers(c *gc.C) {
 	c.Assert(err, jc.ErrorIsNil)
 
 	c.Assert(grantCmd.User, gc.Equals, "bob")
-	url1, err := crossmodel.ParseApplicationURL("fred/model.offer1")
+	url1, err := crossmodel.ParseOfferURL("fred/model.offer1")
 	c.Assert(err, jc.ErrorIsNil)
-	url2, err := crossmodel.ParseApplicationURL("mary/model.offer2")
+	url2, err := crossmodel.ParseOfferURL("mary/model.offer2")
 	c.Assert(err, jc.ErrorIsNil)
-	c.Assert(grantCmd.OfferURLs, jc.DeepEquals, []*crossmodel.ApplicationURL{url1, url2})
+	c.Assert(grantCmd.OfferURLs, jc.DeepEquals, []*crossmodel.OfferURL{url1, url2})
 	c.Assert(grantCmd.ModelNames, gc.HasLen, 0)
 }
 
