@@ -16,25 +16,18 @@ import (
 	"github.com/juju/juju/cmd/juju/common"
 	"github.com/juju/juju/cmd/modelcmd"
 	"github.com/juju/juju/cmd/output"
-	"github.com/juju/juju/jujuclient"
 )
 
 const showModelCommandDoc = `Show information about the current or specified model`
 
 func NewShowCommand() cmd.Command {
 	showCmd := &showModelCommand{}
-	showCmd.RefreshModels = showCmd.ModelCommandBase.RefreshModels
 	return modelcmd.Wrap(showCmd, modelcmd.WrapSkipModelFlags)
 }
 
 // showModelCommand shows all the users with access to the current model.
 type showModelCommand struct {
 	modelcmd.ModelCommandBase
-	// RefreshModels hides the RefreshModels function defined
-	// in ModelCommandBase. This allows overriding for testing.
-	// NOTE: ideal solution would be to have the base implement a method
-	// like store.ModelByName which auto-refreshes.
-	RefreshModels func(jujuclient.ClientStore, string) error
 
 	out cmd.Output
 	api ShowModelAPI
@@ -96,30 +89,18 @@ func (c *showModelCommand) Run(ctx *cmd.Context) error {
 	if err != nil {
 		return errors.Trace(err)
 	}
-	modelName, err := c.ModelName()
+	_, modelDetails, err := c.ModelDetails()
 	if err != nil {
 		return errors.Trace(err)
 	}
+	modelTag := names.NewModelTag(modelDetails.ModelUUID)
+
 	api, err := c.getAPI()
 	if err != nil {
 		return err
 	}
 	defer api.Close()
 
-	store := c.ClientStore()
-	modelDetails, err := store.ModelByName(controllerName, modelName)
-	if errors.IsNotFound(err) {
-		if err := c.RefreshModels(store, controllerName); err != nil {
-			return errors.Annotate(err, "refreshing models cache")
-		}
-		// Now try again.
-		modelDetails, err = store.ModelByName(controllerName, modelName)
-	}
-	if err != nil {
-		return errors.Annotate(err, "getting model details")
-	}
-
-	modelTag := names.NewModelTag(modelDetails.ModelUUID)
 	results, err := api.ModelInfo([]names.ModelTag{modelTag})
 	if err != nil {
 		return err
