@@ -7,6 +7,7 @@ package budget
 import (
 	"fmt"
 	"strconv"
+	"strings"
 
 	"github.com/juju/cmd"
 	"github.com/juju/errors"
@@ -22,7 +23,8 @@ type budgetCommand struct {
 	modelcmd.ModelCommandBase
 	modelUUID string
 	api       apiClient
-	Value     string
+	Wallet    string
+	Limit     string
 }
 
 // NewBudgetCommand returns a new budgetCommand.
@@ -39,7 +41,7 @@ func (c *budgetCommand) newAPIClient(bakery *httpbakery.Client) (apiClient, erro
 }
 
 type apiClient interface {
-	UpdateBudget(string, string) (string, error)
+	UpdateBudget(string, string, string) (string, error)
 }
 
 const doc = `
@@ -48,13 +50,15 @@ Updates an existing budget for a model.
 Examples:
     # Sets the budget for the current model to 10.
     juju budget 10
+    # Moves the budget for the current model to wallet 'personal' and sets the limit to 10.
+    juju budget personal:10
 `
 
 // Info implements cmd.Command.Info.
 func (c *budgetCommand) Info() *cmd.Info {
 	return &cmd.Info{
 		Name:    "budget",
-		Args:    "<value>",
+		Args:    "[<wallet>:]<limit>",
 		Purpose: "Update a budget.",
 		Doc:     doc,
 	}
@@ -65,15 +69,33 @@ func (c *budgetCommand) SetFlags(f *gnuflag.FlagSet) {
 	c.ModelCommandBase.SetFlags(f)
 }
 
+func budgetDefinition(input string) (wallet, limit string, err error) {
+	tokens := strings.Split(input, ":")
+	switch len(tokens) {
+	case 1:
+		return "", tokens[0], nil
+	case 2:
+		return tokens[0], tokens[1], nil
+	default:
+		return "", "", errors.Errorf("invalid budget definition: %v", input)
+	}
+}
+
 // Init implements cmd.Command.Init.
 func (c *budgetCommand) Init(args []string) error {
 	if len(args) < 1 {
 		return errors.New("value required")
 	}
-	c.Value = args[0]
-	if _, err := strconv.ParseInt(c.Value, 10, 32); err != nil {
-		return errors.New("value needs to be a whole number")
+
+	wallet, limit, err := budgetDefinition(args[0])
+	if err != nil {
+		return errors.Trace(err)
 	}
+	if _, err := strconv.ParseInt(limit, 10, 32); err != nil {
+		return errors.New("budget limit needs to be a whole number")
+	}
+	c.Wallet = wallet
+	c.Limit = limit
 	if c.modelUUID != "" {
 		if !utils.IsValidUUIDString(c.modelUUID) {
 			return errors.NotValidf("provided model UUID %q", c.modelUUID)
@@ -107,7 +129,7 @@ func (c *budgetCommand) Run(ctx *cmd.Context) error {
 	if err != nil {
 		return errors.Annotate(err, "failed to create an api client")
 	}
-	resp, err := api.UpdateBudget(modelUUID, c.Value)
+	resp, err := api.UpdateBudget(modelUUID, c.Wallet, c.Limit)
 	if err != nil {
 		return errors.Annotate(err, "failed to update the budget")
 	}
