@@ -20,8 +20,7 @@ var _ = gc.Suite(&UpdateSuite{})
 func (s *UpdateSuite) SetUpTest(c *gc.C) {
 	s.BaseSuite.SetFeatureFlags(feature.PostNetCLIMVP)
 	s.BaseSpaceSuite.SetUpTest(c)
-	s.command = space.NewUpdateCommandForTest(s.api)
-	c.Assert(s.command, gc.NotNil)
+	s.newCommand = space.NewUpdateCommand
 }
 
 func (s *UpdateSuite) TestRunWithSubnetsSucceeds(c *gc.C) {
