@@ -31,11 +31,18 @@ func (s *addSuite) SetUpTest(c *gc.C) {
 	s.SubStorageSuite.SetUpTest(c)
 
 	s.mockAPI = &mockAddAPI{
-		addToUnitFunc: func(storages []params.StorageAddParams) ([]params.ErrorResult, error) {
-			result := make([]params.ErrorResult, len(storages))
+		addToUnitFunc: func(storages []params.StorageAddParams) ([]params.AddStorageResult, error) {
+			result := make([]params.AddStorageResult, len(storages))
 			for i, one := range storages {
 				if strings.HasPrefix(one.StorageName, "err") {
 					result[i].Error = common.ServerError(errors.Errorf("test failure"))
+					continue
+				}
+				result[i].Result = &params.AddStorageDetails{
+					StorageTags: []string{
+						"storage-foo-0",
+						"storage-foo-1",
+					},
 				}
 			}
 			return result, nil
@@ -82,7 +89,7 @@ func (s *addSuite) TestAddArgs(c *gc.C) {
 	for i, t := range errorTsts {
 		c.Logf("test %d for %q", i, t.args)
 		s.args = t.args
-		s.assertAddErrorOutput(c, t.expectedErr, "", visibleErrorMessage(t.visibleErr))
+		s.assertAddErrorOutput(c, t.expectedErr, visibleErrorMessage(t.visibleErr))
 	}
 }
 
@@ -90,63 +97,65 @@ func (s *addSuite) TestAddInvalidUnit(c *gc.C) {
 	s.args = []string{"tst-123", "data=676"}
 
 	expectedErr := `unit name "tst-123" not valid`
-	s.assertAddErrorOutput(c, expectedErr, "", visibleErrorMessage(expectedErr))
-}
-
-var validArgs = [][]string{
-	[]string{"tst/123", "data=676"},
-	[]string{"tst/123", "data"},
+	s.assertAddErrorOutput(c, expectedErr, visibleErrorMessage(expectedErr))
 }
 
 func (s *addSuite) TestAddSuccess(c *gc.C) {
+	validArgs := [][]string{
+		[]string{"tst/123", "data=676"},
+		[]string{"tst/123", "data"},
+	}
+	expectedStderr := `
+added storage foo/0 to tst/123
+added storage foo/1 to tst/123
+`[1:]
+
 	for i, args := range validArgs {
 		c.Logf("test %d for %q", i, args)
-		s.args = args
-		s.assertAddOutput(c, "added \"data\"\n", "")
+		context, err := s.runAdd(c, args...)
+		c.Assert(err, jc.ErrorIsNil)
+		s.assertExpectedOutput(c, context, expectedStderr)
 	}
 }
 
 func (s *addSuite) TestAddOperationAborted(c *gc.C) {
 	s.args = []string{"tst/123", "data=676"}
-	s.mockAPI.addToUnitFunc = func(storages []params.StorageAddParams) ([]params.ErrorResult, error) {
+	s.mockAPI.addToUnitFunc = func(storages []params.StorageAddParams) ([]params.AddStorageResult, error) {
 		return nil, errors.New("aborted")
 	}
-	s.assertAddErrorOutput(c, ".*aborted.*", "", "")
+	s.assertAddErrorOutput(c, ".*aborted.*", "")
 }
 
 func (s *addSuite) TestAddFailure(c *gc.C) {
 	s.args = []string{"tst/123", "err=676"}
-	s.assertAddErrorOutput(c, "cmd: error out silently", "", "failed to add \"err\": test failure\n")
+	s.assertAddErrorOutput(c, "cmd: error out silently", "failed to add storage \"err\" to tst/123: test failure\n")
 }
 
 func (s *addSuite) TestAddMixOrderPreserved(c *gc.C) {
-	expectedOut := `
-added "a"
-`[1:]
 	expectedErr := `
-failed to add "err": test failure
+added storage foo/0 to tst/123
+added storage foo/1 to tst/123
+failed to add storage "err" to tst/123: test failure
 `[1:]
 
 	s.args = []string{"tst/123", "a=676", "err=676"}
-	s.assertAddErrorOutput(c, "cmd: error out silently", expectedOut, expectedErr)
+	s.assertAddErrorOutput(c, "cmd: error out silently", expectedErr)
 
 	s.args = []string{"tst/123", "err=676", "a=676"}
-	s.assertAddErrorOutput(c, "cmd: error out silently", expectedOut, expectedErr)
+	s.assertAddErrorOutput(c, "cmd: error out silently", expectedErr)
 }
 
 func (s *addSuite) TestAddAllDistinctErrors(c *gc.C) {
-	expectedOut := `
-added "storage0"
-added "storage1"
-`[1:]
 	expectedErr := `
-failed to add "storage2": storage pool "barf" not found
-failed to add "storage42": storage "storage42" not found
+added storage "storage0" to tst/123
+added storage "storage1" to tst/123
+failed to add storage "storage2" to tst/123: storage pool "barf" not found
+failed to add storage "storage42" to tst/123: storage "storage42" not found
 `[1:]
 
 	s.args = []string{"tst/123", "storage0=ebs", "storage2=barf", "storage1=123", "storage42=loop"}
-	s.mockAPI.addToUnitFunc = func(storages []params.StorageAddParams) ([]params.ErrorResult, error) {
-		result := make([]params.ErrorResult, len(storages))
+	s.mockAPI.addToUnitFunc = func(storages []params.StorageAddParams) ([]params.AddStorageResult, error) {
+		result := make([]params.AddStorageResult, len(storages))
 		for i, one := range storages {
 			if one.StorageName == "storage2" {
 				result[i].Error = common.ServerError(errors.Errorf(`storage pool "barf" not found`))
@@ -158,21 +167,19 @@ failed to add "storage42": storage "storage42" not found
 		return result, nil
 	}
 
-	s.assertAddErrorOutput(c, "cmd: error out silently", expectedOut, expectedErr)
+	s.assertAddErrorOutput(c, "cmd: error out silently", expectedErr)
 }
 
 func (s *addSuite) TestAddStorageOnlyDistinctErrors(c *gc.C) {
-	expectedOut := `
-added "storage0"
-`[1:]
 	expectedErr := `
-failed to add "storage2": storage "storage2" not found
-failed to add "storage42": storage "storage42" not found
+added storage "storage0" to tst/123
+failed to add storage "storage2" to tst/123: storage "storage2" not found
+failed to add storage "storage42" to tst/123: storage "storage42" not found
 `[1:]
 
 	s.args = []string{"tst/123", "storage0=ebs", "storage2=barf", "storage42=loop"}
-	s.mockAPI.addToUnitFunc = func(storages []params.StorageAddParams) ([]params.ErrorResult, error) {
-		result := make([]params.ErrorResult, len(storages))
+	s.mockAPI.addToUnitFunc = func(storages []params.StorageAddParams) ([]params.AddStorageResult, error) {
+		result := make([]params.AddStorageResult, len(storages))
 		for i, one := range storages {
 			if one.StorageName == "storage42" || one.StorageName == "storage2" {
 				result[i].Error = common.ServerError(errors.Errorf(`storage "%v" not found`, one.StorageName))
@@ -181,11 +188,10 @@ failed to add "storage42": storage "storage42" not found
 		return result, nil
 	}
 
-	s.assertAddErrorOutput(c, "cmd: error out silently", expectedOut, expectedErr)
+	s.assertAddErrorOutput(c, "cmd: error out silently", expectedErr)
 }
 
 func (s *addSuite) TestAddStorageMixDistinctAndNonDistinctErrors(c *gc.C) {
-	expectedOut := ``
 	expectedErr := `
 some unit error
 storage "storage0" not found
@@ -193,8 +199,8 @@ storage "storage0" not found
 
 	unitErr := `some unit error`
 	s.args = []string{"tst/123", "storage0=ebs", "storage2=barf", "storage42=loop"}
-	s.mockAPI.addToUnitFunc = func(storages []params.StorageAddParams) ([]params.ErrorResult, error) {
-		result := make([]params.ErrorResult, len(storages))
+	s.mockAPI.addToUnitFunc = func(storages []params.StorageAddParams) ([]params.AddStorageResult, error) {
+		result := make([]params.AddStorageResult, len(storages))
 		for i, one := range storages {
 			if one.StorageName == "storage42" || one.StorageName == "storage2" {
 				result[i].Error = common.ServerError(errors.New(unitErr))
@@ -205,27 +211,27 @@ storage "storage0" not found
 		return result, nil
 	}
 
-	s.assertAddErrorOutput(c, "cmd: error out silently", expectedOut, expectedErr)
+	s.assertAddErrorOutput(c, "cmd: error out silently", expectedErr)
 }
 
 func (s *addSuite) TestCollapseUnitErrors(c *gc.C) {
 	expectedErr := `some unit error`
 
 	s.args = []string{"tst/123", "storage0=ebs", "storage2=barf", "storage1=123", "storage42=loop"}
-	s.mockAPI.addToUnitFunc = func(storages []params.StorageAddParams) ([]params.ErrorResult, error) {
-		result := make([]params.ErrorResult, len(storages))
+	s.mockAPI.addToUnitFunc = func(storages []params.StorageAddParams) ([]params.AddStorageResult, error) {
+		result := make([]params.AddStorageResult, len(storages))
 		for i, _ := range storages {
 			result[i].Error = common.ServerError(errors.New(expectedErr))
 		}
 		return result, nil
 	}
 
-	s.assertAddErrorOutput(c, "cmd: error out silently", "", fmt.Sprintf("%v\n", expectedErr))
+	s.assertAddErrorOutput(c, "cmd: error out silently", expectedErr+"\n")
 }
 
 func (s *addSuite) TestUnauthorizedMentionsJujuGrant(c *gc.C) {
 	s.args = []string{"tst/123", "data"}
-	s.mockAPI.addToUnitFunc = func(storages []params.StorageAddParams) ([]params.ErrorResult, error) {
+	s.mockAPI.addToUnitFunc = func(storages []params.StorageAddParams) ([]params.AddStorageResult, error) {
 		return nil, &params.Error{
 			Message: "permission denied",
 			Code:    params.CodeUnauthorized,
@@ -237,25 +243,15 @@ func (s *addSuite) TestUnauthorizedMentionsJujuGrant(c *gc.C) {
 	c.Assert(errString, gc.Matches, `.*juju grant.*`)
 }
 
-func (s *addSuite) assertAddOutput(c *gc.C, expectedOut, expectedErr string) {
-	context, err := s.runAdd(c, s.args...)
-	c.Assert(err, jc.ErrorIsNil)
-
-	s.assertExpectedOutput(c, context, expectedOut, expectedErr)
-}
-
-func (s *addSuite) assertAddErrorOutput(c *gc.C, expected string, expectedOut, expectedErr string) {
+func (s *addSuite) assertAddErrorOutput(c *gc.C, expected string, expectedErr string) {
 	context, err := s.runAdd(c, s.args...)
 	c.Assert(errors.Cause(err), gc.ErrorMatches, expected)
-	s.assertExpectedOutput(c, context, expectedOut, expectedErr)
+	s.assertExpectedOutput(c, context, expectedErr)
 }
 
-func (s *addSuite) assertExpectedOutput(c *gc.C, context *cmd.Context, expectedOut, expectedErr string) {
-	obtainedErr := cmdtesting.Stderr(context)
-	c.Assert(obtainedErr, gc.Equals, expectedErr)
-
-	obtainedValid := cmdtesting.Stdout(context)
-	c.Assert(obtainedValid, gc.Equals, expectedOut)
+func (s *addSuite) assertExpectedOutput(c *gc.C, context *cmd.Context, expectedErr string) {
+	c.Assert(cmdtesting.Stdout(context), gc.Equals, "")
+	c.Assert(cmdtesting.Stderr(context), gc.Equals, expectedErr)
 }
 
 func (s *addSuite) runAdd(c *gc.C, args ...string) (*cmd.Context, error) {
@@ -267,13 +263,13 @@ func visibleErrorMessage(errMsg string) string {
 }
 
 type mockAddAPI struct {
-	addToUnitFunc func(storages []params.StorageAddParams) ([]params.ErrorResult, error)
+	addToUnitFunc func(storages []params.StorageAddParams) ([]params.AddStorageResult, error)
 }
 
 func (s mockAddAPI) Close() error {
 	return nil
 }
 
-func (s mockAddAPI) AddToUnit(storages []params.StorageAddParams) ([]params.ErrorResult, error) {
+func (s mockAddAPI) AddToUnit(storages []params.StorageAddParams) ([]params.AddStorageResult, error) {
 	return s.addToUnitFunc(storages)
 }
