@@ -8,6 +8,7 @@ import (
 	"time"
 
 	"github.com/juju/cmd"
+	"github.com/juju/cmd/cmdtesting"
 	jc "github.com/juju/testing/checkers"
 	gc "gopkg.in/check.v1"
 	"gopkg.in/juju/names.v2"
@@ -15,7 +16,6 @@ import (
 	"github.com/juju/juju/apiserver/params"
 	"github.com/juju/juju/cmd/juju/storage"
 	_ "github.com/juju/juju/provider/dummy"
-	"github.com/juju/juju/testing"
 )
 
 // epoch is the time we use for "since" in statuses. The time
@@ -38,7 +38,7 @@ func (s *ShowSuite) SetUpTest(c *gc.C) {
 }
 
 func (s *ShowSuite) runShow(c *gc.C, args []string) (*cmd.Context, error) {
-	return testing.RunCommand(c, storage.NewShowCommandForTest(s.mockAPI, s.store), args...)
+	return cmdtesting.RunCommand(c, storage.NewShowCommandForTest(s.mockAPI, s.store), args...)
 }
 
 func (s *ShowSuite) TestShowNoMatch(c *gc.C) {
@@ -126,7 +126,7 @@ func (s *ShowSuite) assertValidShow(c *gc.C, args []string, expected string) {
 	context, err := s.runShow(c, args)
 	c.Assert(err, jc.ErrorIsNil)
 
-	obtained := testing.Stdout(context)
+	obtained := cmdtesting.Stdout(context)
 	c.Assert(obtained, gc.Matches, expected)
 }
 
