@@ -3,7 +3,6 @@
 package user_test
 
 import (
-	"github.com/juju/cmd"
 	"github.com/juju/cmd/cmdtesting"
 	jc "github.com/juju/testing/checkers"
 	gc "gopkg.in/check.v1"
@@ -34,11 +33,6 @@ func (m *mockRemoveUserAPI) RemoveUser(username string) error {
 	return nil
 }
 
-func (s *RemoveUserCommandSuite) run(c *gc.C, name string) (*cmd.Context, error) {
-	removeCommand, _ := user.NewRemoveCommandForTest(s.mockAPI, s.store)
-	return cmdtesting.RunCommand(c, removeCommand, name)
-}
-
 func (s *RemoveUserCommandSuite) TestInit(c *gc.C) {
 	table := []struct {
 		args        []string
