@@ -4,9 +4,10 @@
 package engine
 
 import (
+	worker "gopkg.in/juju/worker.v1"
+
 	"github.com/juju/juju/agent"
 	"github.com/juju/juju/api/base"
-	"github.com/juju/juju/worker"
 	"github.com/juju/juju/worker/dependency"
 )
 
