@@ -10,7 +10,6 @@ import (
 	"os"
 	"path/filepath"
 	"strconv"
-	"strings"
 	"sync"
 	"time"
 
@@ -48,7 +47,6 @@ import (
 	"github.com/juju/juju/apiserver/observer"
 	"github.com/juju/juju/apiserver/observer/metricobserver"
 	"github.com/juju/juju/apiserver/params"
-	"github.com/juju/juju/audit"
 	"github.com/juju/juju/cert"
 	"github.com/juju/juju/cmd/jujud/agent/machine"
 	"github.com/juju/juju/cmd/jujud/agent/model"
@@ -57,8 +55,8 @@ import (
 	"github.com/juju/juju/container"
 	"github.com/juju/juju/container/kvm"
 	"github.com/juju/juju/controller"
+	"github.com/juju/juju/core/auditlog"
 	"github.com/juju/juju/environs"
-	"github.com/juju/juju/environs/simplestreams"
 	"github.com/juju/juju/instance"
 	jujunames "github.com/juju/juju/juju/names"
 	"github.com/juju/juju/juju/paths"
@@ -77,13 +75,13 @@ import (
 	"github.com/juju/juju/watcher"
 	jworker "github.com/juju/juju/worker"
 	"github.com/juju/juju/worker/apicaller"
+	"github.com/juju/juju/worker/auditconfigupdater"
 	"github.com/juju/juju/worker/catacomb"
 	"github.com/juju/juju/worker/certupdater"
 	"github.com/juju/juju/worker/conv2state"
 	"github.com/juju/juju/worker/dependency"
 	"github.com/juju/juju/worker/deployer"
 	"github.com/juju/juju/worker/gate"
-	"github.com/juju/juju/worker/imagemetadataworker"
 	"github.com/juju/juju/worker/introspection"
 	"github.com/juju/juju/worker/logsender"
 	"github.com/juju/juju/worker/logsender/logsendermetrics"
@@ -96,11 +94,12 @@ import (
 )
 
 var (
-	logger         = loggo.GetLogger("juju.cmd.jujud")
-	jujuRun        = paths.MustSucceed(paths.JujuRun(series.MustHostSeries()))
-	jujuDumpLogs   = paths.MustSucceed(paths.JujuDumpLogs(series.MustHostSeries()))
-	jujuIntrospect = paths.MustSucceed(paths.JujuIntrospect(series.MustHostSeries()))
-	jujudSymlinks  = []string{jujuRun, jujuDumpLogs, jujuIntrospect}
+	logger           = loggo.GetLogger("juju.cmd.jujud")
+	jujuRun          = paths.MustSucceed(paths.JujuRun(series.MustHostSeries()))
+	jujuDumpLogs     = paths.MustSucceed(paths.JujuDumpLogs(series.MustHostSeries()))
+	jujuIntrospect   = paths.MustSucceed(paths.JujuIntrospect(series.MustHostSeries()))
+	jujuUpdateSeries = paths.MustSucceed(paths.JujuUpdateSeries(series.MustHostSeries()))
+	jujudSymlinks    = []string{jujuRun, jujuDumpLogs, jujuIntrospect, jujuUpdateSeries}
 
 	// The following are defined as variables to allow the tests to
 	// intercept calls to the functions. In every case, they should
@@ -110,7 +109,6 @@ var (
 	useMultipleCPUs       = utils.UseMultipleCPUs
 	peergrouperNew        = peergrouper.New
 	newCertificateUpdater = certupdater.NewCertificateUpdater
-	newMetadataUpdater    = imagemetadataworker.NewWorker
 	reportOpenedState     = func(*state.State) {}
 
 	modelManifolds   = model.Manifolds
@@ -751,24 +749,6 @@ func (a *MachineAgent) startAPIWorkers(apiConn api.Connection) (_ worker.Worker,
 	}
 
 	if isModelManager {
-
-		// Published image metadata for some providers are in simple streams.
-		// Providers that do not depend on simple streams do not need this worker.
-		apiSt, err := apiagent.NewState(apiConn)
-		if err != nil {
-			return nil, errors.Annotate(err, "getting API facade")
-		}
-		env, err := environs.GetEnviron(apiSt, newEnvirons)
-		if err != nil {
-			return nil, errors.Annotate(err, "getting environ")
-		}
-		if _, ok := env.(simplestreams.HasRegion); ok {
-			// Start worker that stores published image metadata in state.
-			runner.StartWorker("imagemetadata", func() (worker.Worker, error) {
-				return newMetadataUpdater(apiConn.MetadataUpdater()), nil
-			})
-		}
-
 		// We don't have instance info set and the network config for the
 		// bootstrap machine only, so update it now. All the other machines will
 		// have instance info including network config set at provisioning time.
@@ -1111,6 +1091,12 @@ func (a *MachineAgent) startStateWorkers(
 			//
 			// TODO(ericsnow) For now we simply do not close the channel.
 			certChangedChan := make(chan params.StateServingInfo, 10)
+
+			// Similarly, buffer the audit config change channel so
+			// that controller config changes coming in don't block
+			// apiserver startup.
+			auditConfigChan := make(chan auditlog.Config, 1)
+
 			// Each time apiserver worker is restarted, we need a fresh copy of state due
 			// to the fact that state holds lease managers which are killed and need to be reset.
 			dialOpts, err := mongoDialOptions(
@@ -1130,11 +1116,25 @@ func (a *MachineAgent) startStateWorkers(
 				)
 				return st, err
 			}
+
+			controllerConfig, err := st.ControllerConfig()
+			if err != nil {
+				return nil, errors.Trace(err)
+			}
+			auditConfig := getAuditLogConfig(controllerConfig)
+			if auditConfig.Enabled {
+				auditConfig.Target = auditlog.NewLogFile(
+					agentConfig.LogDir(), auditConfig.MaxSizeMB, auditConfig.MaxBackups)
+			}
+
 			runner.StartWorker("apiserver", a.apiserverWorkerStarter(
 				stateOpener,
 				certChangedChan,
+				auditConfigChan,
+				auditConfig,
 				dependencyReporter,
 			))
+
 			var stateServingSetter certupdater.StateServingInfoSetter = func(info params.StateServingInfo, done <-chan struct{}) error {
 				return a.ChangeConfig(func(config agent.ConfigSetter) error {
 					config.SetStateServingInfo(info)
@@ -1150,6 +1150,13 @@ func (a *MachineAgent) startStateWorkers(
 			a.startWorkerAfterUpgrade(runner, "certupdater", func() (worker.Worker, error) {
 				return newCertificateUpdater(m, agentConfig, st, st, stateServingSetter), nil
 			})
+
+			a.startWorkerAfterUpgrade(runner, "auditconfigupdater", func() (worker.Worker, error) {
+				logFactory := func(cfg auditlog.Config) auditlog.AuditLog {
+					return auditlog.NewLogFile(agentConfig.LogDir(), cfg.MaxSizeMB, cfg.MaxBackups)
+				}
+				return auditconfigupdater.New(st, auditConfig, logFactory, auditConfigChan)
+			})
 		default:
 			return nil, errors.Errorf("unknown job type %q", job)
 		}
@@ -1206,7 +1213,9 @@ var stateWorkerDialOpts mongo.DialOpts
 
 func (a *MachineAgent) apiserverWorkerStarter(
 	stateOpener func() (*state.State, error),
-	certChanged chan params.StateServingInfo,
+	certChanged <-chan params.StateServingInfo,
+	auditConfigChanged <-chan auditlog.Config,
+	auditConfig auditlog.Config,
 	dependencyReporter dependency.Reporter,
 ) func() (worker.Worker, error) {
 	return func() (worker.Worker, error) {
@@ -1216,7 +1225,7 @@ func (a *MachineAgent) apiserverWorkerStarter(
 		}
 		statePool := state.NewStatePool(st)
 		w, err := a.newAPIserverWorker(
-			st, statePool, certChanged, dependencyReporter,
+			st, statePool, certChanged, auditConfigChanged, auditConfig, dependencyReporter,
 		)
 		if err != nil {
 			statePool.Close()
@@ -1230,7 +1239,9 @@ func (a *MachineAgent) apiserverWorkerStarter(
 func (a *MachineAgent) newAPIserverWorker(
 	st *state.State,
 	statePool *state.StatePool,
-	certChanged chan params.StateServingInfo,
+	certChanged <-chan params.StateServingInfo,
+	auditConfigChanged <-chan auditlog.Config,
+	auditConfig auditlog.Config,
 	dependencyReporter dependency.Reporter,
 ) (worker.Worker, error) {
 	agentConfig := a.CurrentConfig()
@@ -1258,13 +1269,6 @@ func (a *MachineAgent) newAPIserverWorker(
 		return nil, err
 	}
 
-	// TODO(katco): We should be doing something more serious than
-	// logging audit errors. Failures in the auditing systems should
-	// stop the api server until the problem can be corrected.
-	auditErrorHandler := func(err error) {
-		logger.Criticalf("%v", err)
-	}
-
 	controllerConfig, err := st.ControllerConfig()
 	if err != nil {
 		return nil, errors.Annotate(err, "cannot fetch the controller config")
@@ -1275,8 +1279,6 @@ func (a *MachineAgent) newAPIserverWorker(
 		clock.WallClock,
 		jujuversion.Current,
 		agentConfig.Model().Id(),
-		newAuditEntrySink(st, logDir),
-		auditErrorHandler,
 		a.prometheusRegistry,
 	)
 	if err != nil {
@@ -1300,7 +1302,7 @@ func (a *MachineAgent) newAPIserverWorker(
 		return nil, errors.Annotate(err, "getting log sink config")
 	}
 
-	server, err := apiserver.NewServer(statePool, listener, apiserver.ServerConfig{
+	serverConfig := apiserver.ServerConfig{
 		Clock:                         clock.WallClock,
 		Cert:                          cert,
 		Key:                           key,
@@ -1319,7 +1321,11 @@ func (a *MachineAgent) newAPIserverWorker(
 		RateLimitConfig:               rateLimitConfig,
 		LogSinkConfig:                 &logSinkConfig,
 		PrometheusRegisterer:          a.prometheusRegistry,
-	})
+		AuditConfig:                   auditConfig,
+		AuditConfigChanged:            auditConfigChanged,
+	}
+
+	server, err := apiserver.NewServer(statePool, listener, serverConfig)
 	if err != nil {
 		return nil, errors.Annotate(err, "cannot start api server worker")
 	}
@@ -1451,37 +1457,11 @@ func getRateLimitConfig(cfg agent.Config) (apiserver.RateLimitConfig, error) {
 	return result, nil
 }
 
-func newAuditEntrySink(st *state.State, logDir string) audit.AuditEntrySinkFn {
-	persistFn := st.PutAuditEntryFn()
-	fileSinkFn := audit.NewLogFileSink(logDir)
-	return func(entry audit.AuditEntry) error {
-		// We don't care about auditing anything but user actions.
-		if _, err := names.ParseUserTag(entry.OriginName); err != nil {
-			return nil
-		}
-		// TODO(wallyworld) - Pinger requests should not originate as a user action.
-		if strings.HasPrefix(entry.Operation, "Pinger:") {
-			return nil
-		}
-		persistErr := persistFn(entry)
-		sinkErr := fileSinkFn(entry)
-		if persistErr == nil {
-			return errors.Annotate(sinkErr, "cannot save audit record to file")
-		}
-		if sinkErr == nil {
-			return errors.Annotate(persistErr, "cannot save audit record to database")
-		}
-		return errors.Annotate(persistErr, "cannot save audit record to file or database")
-	}
-}
-
 func newObserverFn(
 	controllerConfig controller.Config,
 	clock clock.Clock,
 	jujuServerVersion version.Number,
 	modelUUID string,
-	persistAuditEntry audit.AuditEntrySinkFn,
-	auditErrorHandler observer.ErrorHandler,
 	prometheusRegisterer prometheus.Registerer,
 ) (observer.ObserverFactory, error) {
 
@@ -1497,18 +1477,6 @@ func newObserverFn(
 		return observer.NewRequestObserver(ctx)
 	})
 
-	// Auditing observer
-	// TODO(katco): Auditing needs feature tests (lp:1604551)
-	if controllerConfig.AuditingEnabled() {
-		observerFactories = append(observerFactories, func() observer.Observer {
-			ctx := &observer.AuditContext{
-				JujuServerVersion: jujuServerVersion,
-				ModelUUID:         modelUUID,
-			}
-			return observer.NewAudit(ctx, persistAuditEntry, auditErrorHandler)
-		})
-	}
-
 	// Metrics observer.
 	metricObserver, err := metricobserver.NewObserverFactory(metricobserver.Config{
 		Clock:                clock,
@@ -1550,7 +1518,12 @@ func (a *MachineAgent) ensureMongoServer(agentConfig agent.Config) (err error) {
 
 	// Mongo is installed, record the version.
 	err = a.ChangeConfig(func(config agent.ConfigSetter) error {
-		config.SetMongoVersion(mongo.InstalledVersion())
+		finder := mongo.NewMongodFinder()
+		_, version, err := finder.FindBest()
+		if err != nil {
+			return errors.Trace(err)
+		}
+		config.SetMongoVersion(version)
 		return nil
 	})
 	if err != nil {
@@ -1824,3 +1797,14 @@ func getLogSinkConfig(cfg agent.Config) (apiserver.LogSinkConfig, error) {
 	}
 	return result, nil
 }
+
+func getAuditLogConfig(cfg controller.Config) auditlog.Config {
+	result := auditlog.Config{
+		Enabled:        cfg.AuditingEnabled(),
+		CaptureAPIArgs: cfg.AuditLogCaptureArgs(),
+		MaxSizeMB:      cfg.AuditLogMaxSizeMB(),
+		MaxBackups:     cfg.AuditLogMaxBackups(),
+		ExcludeMethods: cfg.AuditLogExcludeMethods(),
+	}
+	return result
+}
