@@ -8,17 +8,20 @@ import (
 	"time"
 
 	"github.com/juju/errors"
+	"github.com/juju/loggo"
 	"github.com/juju/pubsub"
 	"github.com/juju/utils/clock"
 	"github.com/juju/utils/proxy"
 	"github.com/juju/utils/voyeur"
 	"github.com/juju/version"
 	"github.com/prometheus/client_golang/prometheus"
+	"gopkg.in/juju/names.v2"
 	"gopkg.in/juju/worker.v1"
 
 	coreagent "github.com/juju/juju/agent"
 	"github.com/juju/juju/api"
 	"github.com/juju/juju/api/base"
+	"github.com/juju/juju/api/crosscontroller"
 	apideployer "github.com/juju/juju/api/deployer"
 	"github.com/juju/juju/cmd/jujud/agent/engine"
 	"github.com/juju/juju/container/lxd"
@@ -31,11 +34,15 @@ import (
 	"github.com/juju/juju/worker/apiconfigwatcher"
 	"github.com/juju/juju/worker/authenticationworker"
 	"github.com/juju/juju/worker/centralhub"
+	"github.com/juju/juju/worker/dblogpruner"
 	"github.com/juju/juju/worker/dependency"
 	"github.com/juju/juju/worker/deployer"
 	"github.com/juju/juju/worker/diskmanager"
+	"github.com/juju/juju/worker/externalcontrollerupdater"
+	"github.com/juju/juju/worker/fanconfigurer"
 	"github.com/juju/juju/worker/fortress"
 	"github.com/juju/juju/worker/gate"
+	"github.com/juju/juju/worker/globalclockupdater"
 	"github.com/juju/juju/worker/hostkeyreporter"
 	"github.com/juju/juju/worker/identityfilewriter"
 	"github.com/juju/juju/worker/logger"
@@ -45,17 +52,30 @@ import (
 	"github.com/juju/juju/worker/migrationflag"
 	"github.com/juju/juju/worker/migrationminion"
 	"github.com/juju/juju/worker/proxyupdater"
+	psworker "github.com/juju/juju/worker/pubsub"
 	"github.com/juju/juju/worker/reboot"
 	"github.com/juju/juju/worker/resumer"
+	"github.com/juju/juju/worker/singular"
 	workerstate "github.com/juju/juju/worker/state"
 	"github.com/juju/juju/worker/stateconfigwatcher"
 	"github.com/juju/juju/worker/storageprovisioner"
 	"github.com/juju/juju/worker/terminationworker"
 	"github.com/juju/juju/worker/toolsversionchecker"
+	"github.com/juju/juju/worker/txnpruner"
 	"github.com/juju/juju/worker/upgrader"
 	"github.com/juju/juju/worker/upgradesteps"
 )
 
+const (
+	// globalClockUpdaterUpdateInterval is the interval between
+	// global clock updates.
+	globalClockUpdaterUpdateInterval = 1 * time.Second
+
+	// globalClockUpdaterBackoffDelay is the amount of time to
+	// delay when a concurrent global clock update is detected.
+	globalClockUpdaterBackoffDelay = 10 * time.Second
+)
+
 // ManifoldsConfig allows specialisation of the result of Manifolds.
 type ManifoldsConfig struct {
 
@@ -86,6 +106,10 @@ type ManifoldsConfig struct {
 	// upgrader worker completes it's first check.
 	UpgradeCheckLock gate.Lock
 
+	// OpenController is function used by the controller manifold to
+	// create a *state.Controller.
+	OpenController func(coreagent.Config) (*state.Controller, error)
+
 	// OpenState is function used by the state manifold to create a
 	// *state.State.
 	OpenState func(coreagent.Config) (*state.State, error)
@@ -137,12 +161,28 @@ type ManifoldsConfig struct {
 	// CentralHub is the primary hub that exists in the apiserver.
 	CentralHub *pubsub.StructuredHub
 
-	// DepEngineReporter is a dependency engine reporter.
-	DepEngineReporter dependency.Reporter
+	// PubSubReporter is the introspection reporter for the pubsub forwarding
+	// worker.
+	PubSubReporter psworker.Reporter
 
 	// UpdateLoggerConfig is a function that will save the specified
 	// config value as the logging config in the agent.conf file.
 	UpdateLoggerConfig func(string) error
+
+	// NewAgentStatusSetter provides upgradesteps.StatusSetter.
+	NewAgentStatusSetter func(apiConn api.Connection) (upgradesteps.StatusSetter, error)
+
+	// ControllerLeaseDuration defines for how long this agent will ask
+	// for controller administration rights.
+	ControllerLeaseDuration time.Duration
+
+	// LogPruneInterval defines how frequently logs are pruned from
+	// the database.
+	LogPruneInterval time.Duration
+
+	// TransactionPruneInterval defines how frequently mgo/txn transactions
+	// are pruned from the database.
+	TransactionPruneInterval time.Duration
 }
 
 // Manifolds returns a set of co-configured manifolds covering the
@@ -175,6 +215,20 @@ func Manifolds(config ManifoldsConfig) dependency.Manifolds {
 		externalUpdateProxyFunc = lxd.ConfigureLXDProxies
 	}
 
+	newExternalControllerWatcherClient := func(apiInfo *api.Info) (
+		externalcontrollerupdater.ExternalControllerWatcherClientCloser, error,
+	) {
+		conn, err := apicaller.NewExternalControllerConnection(apiInfo)
+		if err != nil {
+			return nil, errors.Trace(err)
+		}
+		return crosscontroller.NewClient(conn), nil
+	}
+
+	agentConfig := config.Agent.CurrentConfig()
+	machineTag := agentConfig.Tag().(names.MachineTag)
+	controllerTag := agentConfig.Controller()
+
 	return dependency.Manifolds{
 		// The agent manifold references the enclosing agent, and is the
 		// foundation stone on which most other manifolds ultimately depend.
@@ -189,6 +243,12 @@ func Manifolds(config ManifoldsConfig) dependency.Manifolds {
 		// with this code.
 		terminationName: terminationworker.Manifold(),
 
+		clockName: clockManifold(config.Clock),
+
+		// Each machine agent has a flag manifold/worker which
+		// reports whether or not the agent is a controller.
+		isControllerFlagName: isControllerFlagManifold(),
+
 		// The stateconfigwatcher manifold watches the machine agent's
 		// configuration and reports if state serving info is
 		// present. It will bounce itself if state serving info is
@@ -210,6 +270,37 @@ func Manifolds(config ManifoldsConfig) dependency.Manifolds {
 			Hub: config.CentralHub,
 		}),
 
+		// The pubsub manifold gets the APIInfo from the agent config,
+		// and uses this as a basis to talk to the other API servers.
+		// The worker subscribes to the messages sent by the peergrouper
+		// that defines the set of machines that are the API servers.
+		// All non-local messages that originate from the machine that
+		// is running the worker get forwarded to the other API servers.
+		// This worker does not run in non-API server machines through
+		// the hub dependency, as that is only available if the machine
+		// is an API server.
+		pubSubName: psworker.Manifold(psworker.ManifoldConfig{
+			AgentName:      agentName,
+			CentralHubName: centralHubName,
+			Clock:          config.Clock,
+			Logger:         loggo.GetLogger("juju.worker.pubsub"),
+			NewWorker:      psworker.NewWorker,
+			Reporter:       config.PubSubReporter,
+		}),
+
+		/* TODO(menn0) - this is currently unused, pending further
+		 * refactoring in the state package.
+
+			// The controller manifold creates a *state.Controller and
+			// makes it available to other manifolds. It pings the MongoDB
+			// session regularly and will die if pings fail.
+			controllerName: workercontroller.Manifold(workercontroller.ManifoldConfig{
+				AgentName:              agentName,
+				StateConfigWatcherName: stateConfigWatcherName,
+				OpenController:         config.OpenController,
+			}),
+		*/
+
 		// The state manifold creates a *state.State and makes it
 		// available to other manifolds. It pings the mongodb session
 		// regularly and will die if pings fail.
@@ -300,6 +391,7 @@ func Manifolds(config ManifoldsConfig) dependency.Manifolds {
 			UpgradeStepsGateName: upgradeStepsGateName,
 			OpenStateForUpgrade:  config.OpenStateForUpgrade,
 			PreUpgradeSteps:      config.PreUpgradeSteps,
+			NewAgentStatusSetter: config.NewAgentStatusSetter,
 		}),
 
 		// The migration workers collaborate to run migrations;
@@ -332,6 +424,32 @@ func Manifolds(config ManifoldsConfig) dependency.Manifolds {
 			NewWorker:         migrationminion.NewWorker,
 		}),
 
+		// We run a global clock updater for every controller machine.
+		//
+		// The global clock updater is primary for detecting and
+		// preventing concurrent updates, to ensure global time is
+		// monotonic and increases at a rate no faster than real time.
+		globalClockUpdaterName: globalclockupdater.Manifold(globalclockupdater.ManifoldConfig{
+			ClockName:      clockName,
+			StateName:      stateName,
+			NewWorker:      globalclockupdater.NewWorker,
+			UpdateInterval: globalClockUpdaterUpdateInterval,
+			BackoffDelay:   globalClockUpdaterBackoffDelay,
+		}),
+
+		// Each controller machine runs a singular worker which will
+		// attempt to claim responsibility for running certain workers
+		// that must not be run concurrently by multiple agents.
+		isPrimaryControllerFlagName: ifController(singular.Manifold(singular.ManifoldConfig{
+			ClockName:     clockName,
+			APICallerName: apiCallerName,
+			Duration:      config.ControllerLeaseDuration,
+			Claimant:      machineTag,
+			Entity:        controllerTag,
+			NewFacade:     singular.NewFacade,
+			NewWorker:     singular.NewWorker,
+		})),
+
 		// The serving-info-setter manifold sets grabs the state
 		// serving info from the API connection and writes it to the
 		// agent config.
@@ -395,12 +513,19 @@ func Manifolds(config ManifoldsConfig) dependency.Manifolds {
 			APICallerName: apiCallerName,
 		})),
 
+		fanConfigurerName: ifNotMigrating(fanconfigurer.Manifold(fanconfigurer.ManifoldConfig{
+			APICallerName: apiCallerName,
+			Clock:         config.Clock,
+		})),
+
 		// The machiner Worker will wait for the identified machine to become
 		// Dying and make it Dead; or until the machine becomes Dead by other
-		// means.
+		// means. This worker needs to be launched after fanconfigurer
+		// so that it reports interfaces created by it.
 		machinerName: ifNotMigrating(machiner.Manifold(machiner.ManifoldConfig{
-			AgentName:     agentName,
-			APICallerName: apiCallerName,
+			AgentName:         agentName,
+			APICallerName:     apiCallerName,
+			FanConfigurerName: fanConfigurerName,
 		})),
 
 		// The log sender is a leaf worker that sends log messages to some
@@ -416,7 +541,7 @@ func Manifolds(config ManifoldsConfig) dependency.Manifolds {
 			LogSource:     config.LogSource,
 		})),
 
-		// The deployer worker is responsible for deploying and recalling unit
+		// The deployer worker is primary for deploying and recalling unit
 		// agents, according to changes in a set of state units; and for the
 		// final removal of its agents' units from state when they are no
 		// longer needed.
@@ -473,6 +598,38 @@ func Manifolds(config ManifoldsConfig) dependency.Manifolds {
 			NewFacade:     hostkeyreporter.NewFacade,
 			NewWorker:     hostkeyreporter.NewWorker,
 		})),
+
+		externalControllerUpdaterName: ifNotMigrating(ifPrimaryController(externalcontrollerupdater.Manifold(
+			externalcontrollerupdater.ManifoldConfig{
+				APICallerName:                      apiCallerName,
+				NewExternalControllerWatcherClient: newExternalControllerWatcherClient,
+			},
+		))),
+		logPrunerName: ifNotMigrating(ifPrimaryController(dblogpruner.Manifold(
+			dblogpruner.ManifoldConfig{
+				ClockName:     clockName,
+				StateName:     stateName,
+				PruneInterval: config.LogPruneInterval,
+				NewWorker:     dblogpruner.NewWorker,
+			},
+		))),
+		txnPrunerName: ifNotMigrating(ifPrimaryController(txnpruner.Manifold(
+			txnpruner.ManifoldConfig{
+				ClockName:     clockName,
+				StateName:     stateName,
+				PruneInterval: config.TransactionPruneInterval,
+				NewWorker:     txnpruner.New,
+			},
+		))),
+	}
+}
+
+func clockManifold(clock clock.Clock) dependency.Manifold {
+	return dependency.Manifold{
+		Start: func(_ dependency.Context) (worker.Worker, error) {
+			return engine.NewValueWorker(clock)
+		},
+		Output: engine.ValueWorkerOutput,
 	}
 }
 
@@ -490,15 +647,30 @@ var ifNotMigrating = engine.Housing{
 	Occupy: migrationFortressName,
 }.Decorate
 
+var ifPrimaryController = engine.Housing{
+	Flags: []string{
+		isPrimaryControllerFlagName,
+	},
+}.Decorate
+
+var ifController = engine.Housing{
+	Flags: []string{
+		isControllerFlagName,
+	},
+}.Decorate
+
 const (
 	agentName              = "agent"
 	terminationName        = "termination-signal-handler"
 	stateConfigWatcherName = "state-config-watcher"
+	controllerName         = "controller"
 	stateName              = "state"
 	stateWorkersName       = "unconverted-state-workers"
 	apiCallerName          = "api-caller"
 	apiConfigWatcherName   = "api-config-watcher"
 	centralHubName         = "central-hub"
+	pubSubName             = "pubsub-forwarder"
+	clockName              = "clock"
 
 	upgraderName         = "upgrader"
 	upgradeStepsName     = "upgrade-steps-runner"
@@ -511,21 +683,28 @@ const (
 	migrationInactiveFlagName = "migration-inactive-flag"
 	migrationMinionName       = "migration-minion"
 
-	servingInfoSetterName    = "serving-info-setter"
-	apiWorkersName           = "unconverted-api-workers"
-	rebootName               = "reboot-executor"
-	loggingConfigUpdaterName = "logging-config-updater"
-	diskManagerName          = "disk-manager"
-	proxyConfigUpdater       = "proxy-config-updater"
-	apiAddressUpdaterName    = "api-address-updater"
-	machinerName             = "machiner"
-	logSenderName            = "log-sender"
-	deployerName             = "unit-agent-deployer"
-	authenticationWorkerName = "ssh-authkeys-updater"
-	storageProvisionerName   = "storage-provisioner"
-	resumerName              = "mgo-txn-resumer"
-	identityFileWriterName   = "ssh-identity-writer"
-	toolsVersionCheckerName  = "tools-version-checker"
-	machineActionName        = "machine-action-runner"
-	hostKeyReporterName      = "host-key-reporter"
+	servingInfoSetterName         = "serving-info-setter"
+	apiWorkersName                = "unconverted-api-workers"
+	rebootName                    = "reboot-executor"
+	loggingConfigUpdaterName      = "logging-config-updater"
+	diskManagerName               = "disk-manager"
+	proxyConfigUpdater            = "proxy-config-updater"
+	apiAddressUpdaterName         = "api-address-updater"
+	machinerName                  = "machiner"
+	logSenderName                 = "log-sender"
+	deployerName                  = "unit-agent-deployer"
+	authenticationWorkerName      = "ssh-authkeys-updater"
+	storageProvisionerName        = "storage-provisioner"
+	resumerName                   = "mgo-txn-resumer"
+	identityFileWriterName        = "ssh-identity-writer"
+	toolsVersionCheckerName       = "tools-version-checker"
+	machineActionName             = "machine-action-runner"
+	hostKeyReporterName           = "host-key-reporter"
+	fanConfigurerName             = "fan-configurer"
+	externalControllerUpdaterName = "external-controller-updater"
+	globalClockUpdaterName        = "global-clock-updater"
+	isPrimaryControllerFlagName   = "is-primary-controller-flag"
+	isControllerFlagName          = "is-controller-flag"
+	logPrunerName                 = "log-pruner"
+	txnPrunerName                 = "transaction-pruner"
 )
