@@ -11,7 +11,6 @@ import (
 	gc "gopkg.in/check.v1"
 	worker "gopkg.in/juju/worker.v1"
 
-	"github.com/juju/juju/agent"
 	"github.com/juju/juju/cmd/jujud/agent/machine"
 	"github.com/juju/juju/testing"
 	"github.com/juju/juju/worker/dependency"
@@ -26,7 +25,7 @@ var _ = gc.Suite(&ManifoldsSuite{})
 
 func (*ManifoldsSuite) TestStartFuncs(c *gc.C) {
 	manifolds := machine.Manifolds(machine.ManifoldsConfig{
-		Agent: fakeAgent{},
+		Agent: &mockAgent{},
 	})
 	for name, manifold := range manifolds {
 		c.Logf("checking %q manifold", name)
@@ -35,7 +34,9 @@ func (*ManifoldsSuite) TestStartFuncs(c *gc.C) {
 }
 
 func (*ManifoldsSuite) TestManifoldNames(c *gc.C) {
-	manifolds := machine.Manifolds(machine.ManifoldsConfig{})
+	manifolds := machine.Manifolds(machine.ManifoldsConfig{
+		Agent: &mockAgent{},
+	})
 	keys := make([]string, 0, len(manifolds))
 	for k := range manifolds {
 		keys = append(keys, k)
@@ -47,8 +48,15 @@ func (*ManifoldsSuite) TestManifoldNames(c *gc.C) {
 		"api-caller",
 		"api-config-watcher",
 		"central-hub",
+		"clock",
 		"disk-manager",
+		"external-controller-updater",
+		"fan-configurer",
+		"global-clock-updater",
 		"host-key-reporter",
+		"is-controller-flag",
+		"is-primary-controller-flag",
+		"log-pruner",
 		"log-sender",
 		"logging-config-updater",
 		"machine-action-runner",
@@ -58,6 +66,7 @@ func (*ManifoldsSuite) TestManifoldNames(c *gc.C) {
 		"migration-minion",
 		"migration-inactive-flag",
 		"proxy-config-updater",
+		"pubsub-forwarder",
 		"reboot-executor",
 		"serving-info-setter",
 		"ssh-authkeys-updater",
@@ -67,6 +76,7 @@ func (*ManifoldsSuite) TestManifoldNames(c *gc.C) {
 		"storage-provisioner",
 		"termination-signal-handler",
 		"tools-version-checker",
+		"transaction-pruner",
 		"unconverted-api-workers",
 		"unconverted-state-workers",
 		"unit-agent-deployer",
@@ -81,7 +91,9 @@ func (*ManifoldsSuite) TestManifoldNames(c *gc.C) {
 }
 
 func (*ManifoldsSuite) TestUpgradesBlockMigration(c *gc.C) {
-	manifolds := machine.Manifolds(machine.ManifoldsConfig{})
+	manifolds := machine.Manifolds(machine.ManifoldsConfig{
+		Agent: &mockAgent{},
+	})
 	manifold, ok := manifolds["migration-fortress"]
 	c.Assert(ok, jc.IsTrue)
 
@@ -95,7 +107,12 @@ func (*ManifoldsSuite) TestMigrationGuardsUsed(c *gc.C) {
 		"api-caller",
 		"api-config-watcher",
 		"central-hub",
+		"clock",
+		"global-clock-updater",
+		"is-controller-flag",
+		"is-primary-controller-flag",
 		"log-forwarder",
+		"pubsub-forwarder",
 		"state",
 		"state-config-watcher",
 		"termination-signal-handler",
@@ -110,7 +127,9 @@ func (*ManifoldsSuite) TestMigrationGuardsUsed(c *gc.C) {
 		"upgrade-steps-runner",
 		"upgrader",
 	)
-	manifolds := machine.Manifolds(machine.ManifoldsConfig{})
+	manifolds := machine.Manifolds(machine.ManifoldsConfig{
+		Agent: &mockAgent{},
+	})
 	for name, manifold := range manifolds {
 		c.Logf(name)
 		if !exempt.Contains(name) {
@@ -120,6 +139,26 @@ func (*ManifoldsSuite) TestMigrationGuardsUsed(c *gc.C) {
 	}
 }
 
+func (*ManifoldsSuite) TestSingularGuardsUsed(c *gc.C) {
+	manifolds := machine.Manifolds(machine.ManifoldsConfig{
+		Agent: &mockAgent{},
+	})
+	for name, manifold := range manifolds {
+		c.Logf(name)
+		switch name {
+		case "is-primary-controller-flag":
+			checkContains(c, manifold.Inputs, "is-controller-flag")
+			checkNotContains(c, manifold.Inputs, "is-primary-controller-flag")
+		case "external-controller-updater", "log-pruner", "transaction-pruner":
+			checkNotContains(c, manifold.Inputs, "is-controller-flag")
+			checkContains(c, manifold.Inputs, "is-primary-controller-flag")
+		default:
+			checkNotContains(c, manifold.Inputs, "is-controller-flag")
+			checkNotContains(c, manifold.Inputs, "is-primary-controller-flag")
+		}
+	}
+}
+
 func checkContains(c *gc.C, names []string, seek string) {
 	for _, name := range names {
 		if name == seek {
@@ -129,10 +168,20 @@ func checkContains(c *gc.C, names []string, seek string) {
 	c.Errorf("%q not found in %v", seek, names)
 }
 
+func checkNotContains(c *gc.C, names []string, seek string) {
+	for _, name := range names {
+		if name == seek {
+			c.Errorf("%q found in %v", seek, names)
+			return
+		}
+	}
+}
+
 func (*ManifoldsSuite) TestUpgradeGates(c *gc.C) {
 	upgradeStepsLock := gate.NewLock()
 	upgradeCheckLock := gate.NewLock()
 	manifolds := machine.Manifolds(machine.ManifoldsConfig{
+		Agent:            &mockAgent{},
 		UpgradeStepsLock: upgradeStepsLock,
 		UpgradeCheckLock: upgradeCheckLock,
 	})
@@ -163,7 +212,3 @@ func assertGate(c *gc.C, manifold dependency.Manifold, unlocker gate.Unlocker) {
 		c.Fatalf("expected gate to be unlocked")
 	}
 }
-
-type fakeAgent struct {
-	agent.Agent
-}
