@@ -4,6 +4,8 @@
 package agent
 
 import (
+	"bufio"
+	"encoding/json"
 	"io/ioutil"
 	"os"
 	"path/filepath"
@@ -35,10 +37,11 @@ import (
 
 	"github.com/juju/juju/agent"
 	"github.com/juju/juju/api"
-	"github.com/juju/juju/api/imagemetadata"
 	apimachiner "github.com/juju/juju/api/machiner"
 	"github.com/juju/juju/apiserver/params"
 	"github.com/juju/juju/cmd/jujud/agent/model"
+	"github.com/juju/juju/controller"
+	"github.com/juju/juju/core/auditlog"
 	"github.com/juju/juju/core/migration"
 	"github.com/juju/juju/environs"
 	envtesting "github.com/juju/juju/environs/testing"
@@ -46,8 +49,8 @@ import (
 	"github.com/juju/juju/network"
 	"github.com/juju/juju/provider/dummy"
 	"github.com/juju/juju/state"
+	"github.com/juju/juju/state/multiwatcher"
 	"github.com/juju/juju/state/watcher"
-	"github.com/juju/juju/status"
 	"github.com/juju/juju/storage"
 	coretesting "github.com/juju/juju/testing"
 	"github.com/juju/juju/testing/factory"
@@ -74,6 +77,9 @@ type MachineSuite struct {
 var _ = gc.Suite(&MachineSuite{})
 
 func (s *MachineSuite) SetUpTest(c *gc.C) {
+	s.ControllerConfigAttrs = map[string]interface{}{
+		controller.AuditingEnabled: true,
+	}
 	s.commonMachineSuite.SetUpTest(c)
 	// Most of these tests normally finish sub-second on a fast machine.
 	// If any given test hits a minute, we have almost certainly become
@@ -239,136 +245,6 @@ func (s *MachineSuite) TestDyingMachine(c *gc.C) {
 	c.Assert(m.Life(), gc.Equals, state.Dead)
 }
 
-func (s *MachineSuite) TestHostUnits(c *gc.C) {
-	m, _, _ := s.primeAgent(c, state.JobHostUnits)
-	a := s.newAgent(c, m)
-	ctx, reset := patchDeployContext(c, s.BackingState)
-	defer reset()
-	go func() { c.Check(a.Run(nil), jc.ErrorIsNil) }()
-	defer func() { c.Check(a.Stop(), jc.ErrorIsNil) }()
-
-	// check that unassigned units don't trigger any deployments.
-	app := s.AddTestingApplication(c, "wordpress", s.AddTestingCharm(c, "wordpress"))
-	u0, err := app.AddUnit(state.AddUnitParams{})
-	c.Assert(err, jc.ErrorIsNil)
-	u1, err := app.AddUnit(state.AddUnitParams{})
-	c.Assert(err, jc.ErrorIsNil)
-
-	ctx.waitDeployed(c)
-
-	// assign u0, check it's deployed.
-	err = u0.AssignToMachine(m)
-	c.Assert(err, jc.ErrorIsNil)
-	ctx.waitDeployed(c, u0.Name())
-
-	// "start the agent" for u0 to prevent short-circuited remove-on-destroy;
-	// check that it's kept deployed despite being Dying.
-	// lp:1558657
-	now := time.Now()
-	sInfo := status.StatusInfo{
-		Status:  status.Idle,
-		Message: "",
-		Since:   &now,
-	}
-	err = u0.SetAgentStatus(sInfo)
-	c.Assert(err, jc.ErrorIsNil)
-	err = u0.Destroy()
-	c.Assert(err, jc.ErrorIsNil)
-	ctx.waitDeployed(c, u0.Name())
-
-	// add u1 to the machine, check it's deployed.
-	err = u1.AssignToMachine(m)
-	c.Assert(err, jc.ErrorIsNil)
-	ctx.waitDeployed(c, u0.Name(), u1.Name())
-
-	// make u0 dead; check the deployer recalls the unit and removes it from
-	// state.
-	err = u0.EnsureDead()
-	c.Assert(err, jc.ErrorIsNil)
-	ctx.waitDeployed(c, u1.Name())
-
-	// The deployer actually removes the unit just after
-	// removing its deployment, so we need to poll here
-	// until it actually happens.
-	for attempt := coretesting.LongAttempt.Start(); attempt.Next(); {
-		if !attempt.HasNext() {
-			c.Fatalf("timeout waiting for unit %q to be removed", u0.Name())
-		}
-		if err := u0.Refresh(); err == nil {
-			c.Logf("waiting unit %q to be removed...", u0.Name())
-			continue
-		} else {
-			c.Assert(err, jc.Satisfies, errors.IsNotFound)
-			break
-		}
-	}
-
-	// short-circuit-remove u1 after it's been deployed; check it's recalled
-	// and removed from state.
-	err = u1.Destroy()
-	c.Assert(err, jc.ErrorIsNil)
-	err = u1.Refresh()
-	c.Assert(err, jc.Satisfies, errors.IsNotFound)
-	ctx.waitDeployed(c)
-}
-
-func (s *MachineSuite) TestManageModel(c *gc.C) {
-	usefulVersion := version.Binary{
-		Number: jujuversion.Current,
-		Arch:   arch.HostArch(),
-		Series: "quantal", // to match the charm created below
-	}
-	envtesting.AssertUploadFakeToolsVersions(c, s.DefaultToolsStorage, s.Environ.Config().AgentStream(), s.Environ.Config().AgentStream(), usefulVersion)
-	m, _, _ := s.primeAgent(c, state.JobManageModel)
-	op := make(chan dummy.Operation, 200)
-	dummy.Listen(op)
-
-	a := s.newAgent(c, m)
-	// Make sure the agent is stopped even if the test fails.
-	defer a.Stop()
-	done := make(chan error)
-	go func() {
-		done <- a.Run(nil)
-	}()
-	c.Logf("started test agent, waiting for workers...")
-
-	// Check that the provisioner and firewaller are alive by doing
-	// a rudimentary check that it responds to state changes.
-
-	// Create an exposed service, and add a unit.
-	charm := s.AddTestingCharm(c, "dummy")
-	app := s.AddTestingApplication(c, "test-service", charm)
-	err := app.SetExposed()
-	c.Assert(err, jc.ErrorIsNil)
-	unit, err := app.AddUnit(state.AddUnitParams{})
-	c.Assert(err, jc.ErrorIsNil)
-	err = s.State.AssignUnit(unit, state.AssignCleanEmpty)
-	c.Assert(err, jc.ErrorIsNil)
-
-	// It should be allocated to a machine, which should then be provisioned.
-	c.Logf("application %q added with 1 unit, waiting for unit %q's machine to be started...", app.Name(), unit.Name())
-	c.Check(opRecvTimeout(c, s.State, op, dummy.OpStartInstance{}), gc.NotNil)
-	c.Logf("machine hosting unit %q started, waiting for the unit to be deployed...", unit.Name())
-	s.waitProvisioned(c, unit)
-
-	// Open a port on the unit; it should be handled by the firewaller.
-	c.Logf("unit %q deployed, opening port tcp/999...", unit.Name())
-	err = unit.OpenPort("tcp", 999)
-	c.Assert(err, jc.ErrorIsNil)
-	c.Check(opRecvTimeout(c, s.State, op, dummy.OpOpenPorts{}), gc.NotNil)
-	c.Logf("unit %q port tcp/999 opened, cleaning up...", unit.Name())
-
-	err = a.Stop()
-	c.Assert(err, jc.ErrorIsNil)
-	select {
-	case err := <-done:
-		c.Assert(err, jc.ErrorIsNil)
-	case <-time.After(coretesting.LongWait):
-		c.Fatalf("timed out waiting for agent to terminate")
-	}
-	c.Logf("test agent stopped successfully.")
-}
-
 func (s *MachineSuite) TestManageModelRunsInstancePoller(c *gc.C) {
 	s.AgentSuite.PatchValue(&instancepoller.ShortPoll, 500*time.Millisecond)
 	usefulVersion := version.Binary{
@@ -635,6 +511,96 @@ func (s *MachineSuite) TestManageModelServesAPI(c *gc.C) {
 	})
 }
 
+func (s *MachineSuite) TestManageModelAuditsAPI(c *gc.C) {
+	password := "shhh..."
+	user := s.Factory.MakeUser(c, &factory.UserParams{
+		Password: password,
+	})
+
+	err := s.State.UpdateControllerConfig(map[string]interface{}{
+		"audit-log-exclude-methods": []interface{}{"Client.FullStatus"},
+	}, nil)
+	c.Assert(err, jc.ErrorIsNil)
+
+	s.assertJobWithState(c, state.JobManageModel, func(conf agent.Config, agentState *state.State) {
+		logPath := filepath.Join(conf.LogDir(), "audit.log")
+
+		makeAPIRequest := func(doRequest func(*api.Client)) {
+			apiInfo, ok := conf.APIInfo()
+			c.Assert(ok, jc.IsTrue)
+			apiInfo.Tag = user.Tag()
+			apiInfo.Password = password
+			st, err := api.Open(apiInfo, fastDialOpts)
+			c.Assert(err, jc.ErrorIsNil)
+			defer st.Close()
+			doRequest(st.Client())
+		}
+
+		// Make requests in separate API connections so they're separate conversations.
+		makeAPIRequest(func(client *api.Client) {
+			_, err = client.Status(nil)
+			c.Assert(err, jc.ErrorIsNil)
+		})
+		makeAPIRequest(func(client *api.Client) {
+			_, err = client.AddMachines([]params.AddMachineParams{{
+				Jobs: []multiwatcher.MachineJob{"JobHostUnits"},
+			}})
+			c.Assert(err, jc.ErrorIsNil)
+		})
+
+		// Check that there's a call to Client.AddMachinesV2 in the
+		// log, but no call to Client.FullStatus.
+		records := readAuditLog(c, logPath)
+		c.Assert(records, gc.HasLen, 3)
+		c.Assert(records[1].Request, gc.NotNil)
+		c.Assert(records[1].Request.Facade, gc.Equals, "Client")
+		c.Assert(records[1].Request.Method, gc.Equals, "AddMachinesV2")
+
+		// Now update the controller config to remove the exclusion.
+		err := s.State.UpdateControllerConfig(map[string]interface{}{
+			"audit-log-exclude-methods": []interface{}{},
+		}, nil)
+		c.Assert(err, jc.ErrorIsNil)
+
+		prevRecords := len(records)
+
+		// We might need to wait until the controller config change is
+		// propagated to the apiserver.
+		for a := coretesting.LongAttempt.Start(); a.Next(); {
+			makeAPIRequest(func(client *api.Client) {
+				_, err = client.Status(nil)
+				c.Assert(err, jc.ErrorIsNil)
+			})
+			// Check to see whether there are more logged requests.
+			records = readAuditLog(c, logPath)
+			if prevRecords < len(records) {
+				break
+			}
+		}
+		// Now there should also be a call to Client.FullStatus (and a response).
+		lastRequest := records[len(records)-2]
+		c.Assert(lastRequest.Request, gc.NotNil)
+		c.Assert(lastRequest.Request.Facade, gc.Equals, "Client")
+		c.Assert(lastRequest.Request.Method, gc.Equals, "FullStatus")
+	})
+}
+
+func readAuditLog(c *gc.C, logPath string) []auditlog.Record {
+	file, err := os.Open(logPath)
+	c.Assert(err, jc.ErrorIsNil)
+	defer file.Close()
+
+	scanner := bufio.NewScanner(file)
+	var results []auditlog.Record
+	for scanner.Scan() {
+		var record auditlog.Record
+		err := json.Unmarshal(scanner.Bytes(), &record)
+		c.Assert(err, jc.ErrorIsNil)
+		results = append(results, record)
+	}
+	return results
+}
+
 func (s *MachineSuite) assertAgentSetsToolsVersion(c *gc.C, job state.MachineJob) {
 	vers := version.Binary{
 		Number: jujuversion.Current,
@@ -925,31 +891,6 @@ func (s *MachineSuite) TestDiskManagerWorkerUpdatesState(c *gc.C) {
 	c.Fatalf("timeout while waiting for block devices to be recorded")
 }
 
-func (s *MachineSuite) TestMachineAgentDoesNotRunMetadataWorkerForHostUnits(c *gc.C) {
-	s.checkMetadataWorkerNotRun(c, state.JobHostUnits, "can host units")
-}
-
-func (s *MachineSuite) TestMachineAgentDoesNotRunMetadataWorkerForNonSimpleStreamDependentProviders(c *gc.C) {
-	s.checkMetadataWorkerNotRun(c, state.JobManageModel, "has provider which doesn't depend on simple streams")
-}
-
-func (s *MachineSuite) checkMetadataWorkerNotRun(c *gc.C, job state.MachineJob, suffix string) {
-	// Patch out the worker func before starting the agent.
-	started := newSignal()
-	newWorker := func(cl *imagemetadata.Client) worker.Worker {
-		started.trigger()
-		return jworker.NewNoOpWorker()
-	}
-	s.PatchValue(&newMetadataUpdater, newWorker)
-
-	// Start the machine agent.
-	m, _, _ := s.primeAgent(c, job)
-	a := s.newAgent(c, m)
-	go func() { c.Check(a.Run(nil), jc.ErrorIsNil) }()
-	defer func() { c.Check(a.Stop(), jc.ErrorIsNil) }()
-	started.assertNotTriggered(c, startWorkerWait, "metadata update worker started")
-}
-
 func (s *MachineSuite) TestMachineAgentRunsMachineStorageWorker(c *gc.C) {
 	m, _, _ := s.primeAgent(c, state.JobHostUnits)
 
@@ -1005,39 +946,6 @@ func (s *MachineSuite) TestMachineAgentDoesNotRunsCertificateUpdateWorkerForNonC
 	started.assertNotTriggered(c, startWorkerWait, "certificate was updated")
 }
 
-func (s *MachineSuite) TestCertificateUpdateWorkerUpdatesCertificate(c *gc.C) {
-	coretesting.SkipFlaky(c, "lp:1466514")
-	// Set up the machine agent.
-	m, _, _ := s.primeAgent(c, state.JobManageModel)
-	a := s.newAgent(c, m)
-	a.ReadConfig(names.NewMachineTag(m.Id()).String())
-
-	// Set up check that certificate has been updated.
-	updated := make(chan struct{})
-	go func() {
-		for {
-			stateInfo, _ := a.CurrentConfig().StateServingInfo()
-			srvCert, err := cert.ParseCert(stateInfo.Cert)
-			if !c.Check(err, jc.ErrorIsNil) {
-				break
-			}
-			sanIPs := make([]string, len(srvCert.IPAddresses))
-			for i, ip := range srvCert.IPAddresses {
-				sanIPs[i] = ip.String()
-			}
-			if len(sanIPs) == 1 && sanIPs[0] == "0.1.2.3" {
-				close(updated)
-				break
-			}
-			time.Sleep(100 * time.Millisecond)
-		}
-	}()
-
-	go func() { c.Check(a.Run(nil), jc.ErrorIsNil) }()
-	defer func() { c.Check(a.Stop(), jc.ErrorIsNil) }()
-	s.assertChannelActive(c, updated, "certificate to be updated")
-}
-
 func (s *MachineSuite) TestCertificateDNSUpdated(c *gc.C) {
 	m, _, _ := s.primeAgent(c, state.JobManageModel)
 	a := s.newAgent(c, m)
@@ -1365,6 +1273,20 @@ func (s *MachineSuite) TestReplicasetInitForNewController(c *gc.C) {
 	c.Assert(s.fakeEnsureMongo.InitiateCount, gc.Equals, 0)
 }
 
+func (s *MachineSuite) TestGetAuditLogConfig(c *gc.C) {
+	cfg := coretesting.FakeControllerConfig()
+	cfg["auditing-enabled"] = true
+	cfg["audit-log-exclude-methods"] = []interface{}{"Exclude.This"}
+	result := getAuditLogConfig(cfg)
+	c.Assert(result, gc.DeepEquals, auditlog.Config{
+		Enabled:        true,
+		CaptureAPIArgs: true,
+		MaxSizeMB:      200,
+		MaxBackups:     5,
+		ExcludeMethods: set.NewStrings("Exclude.This"),
+	})
+}
+
 type nullWorker struct {
 	tomb tomb.Tomb
 }
