@@ -7,8 +7,8 @@ import (
 	"time"
 
 	"github.com/juju/utils/clock"
-	"github.com/juju/utils/featureflag"
 	"github.com/juju/utils/voyeur"
+	"gopkg.in/juju/names.v2"
 	"gopkg.in/juju/worker.v1"
 
 	coreagent "github.com/juju/juju/agent"
@@ -17,7 +17,6 @@ import (
 	"github.com/juju/juju/cmd/jujud/agent/engine"
 	"github.com/juju/juju/core/life"
 	"github.com/juju/juju/environs"
-	"github.com/juju/juju/feature"
 	"github.com/juju/juju/worker/actionpruner"
 	"github.com/juju/juju/worker/agent"
 	"github.com/juju/juju/worker/apicaller"
@@ -106,6 +105,7 @@ type ManifoldsConfig struct {
 // run together to administer a model, as configured.
 func Manifolds(config ManifoldsConfig) dependency.Manifolds {
 	agentConfig := config.Agent.CurrentConfig()
+	machineTag := agentConfig.Tag().(names.MachineTag)
 	modelTag := agentConfig.Model()
 	controllerTag := agentConfig.Controller()
 	result := dependency.Manifolds{
@@ -150,9 +150,10 @@ func Manifolds(config ManifoldsConfig) dependency.Manifolds {
 		}),
 		isResponsibleFlagName: singular.Manifold(singular.ManifoldConfig{
 			ClockName:     clockName,
-			AgentName:     agentName,
 			APICallerName: apiCallerName,
 			Duration:      config.RunFlagDuration,
+			Claimant:      machineTag,
+			Entity:        modelTag,
 
 			NewFacade: singular.NewFacade,
 			NewWorker: singular.NewWorker,
@@ -261,10 +262,10 @@ func Manifolds(config ManifoldsConfig) dependency.Manifolds {
 			Scope:         modelTag,
 		})),
 		firewallerName: ifNotMigrating(firewaller.Manifold(firewaller.ManifoldConfig{
-			AgentName:          agentName,
-			APICallerName:      apiCallerName,
-			EnvironName:        environTrackerName,
-			NewAPIConnForModel: api.NewConnectionForModel,
+			AgentName:               agentName,
+			APICallerName:           apiCallerName,
+			EnvironName:             environTrackerName,
+			NewControllerConnection: apicaller.NewExternalControllerConnection,
 
 			NewFirewallerWorker:      firewaller.NewWorker,
 			NewFirewallerFacade:      firewaller.NewFirewallerFacade,
@@ -328,15 +329,13 @@ func Manifolds(config ManifoldsConfig) dependency.Manifolds {
 			}},
 		})),
 	}
-	if featureflag.Enabled(feature.CrossModelRelations) {
-		result[remoteRelationsName] = ifNotMigrating(remoterelations.Manifold(remoterelations.ManifoldConfig{
-			AgentName:                agentName,
-			APICallerName:            apiCallerName,
-			NewAPIConnForModel:       api.NewConnectionForModel,
-			NewRemoteRelationsFacade: remoterelations.NewRemoteRelationsFacade,
-			NewWorker:                remoterelations.NewWorker,
-		}))
-	}
+	result[remoteRelationsName] = ifNotMigrating(remoterelations.Manifold(remoterelations.ManifoldConfig{
+		AgentName:                agentName,
+		APICallerName:            apiCallerName,
+		NewControllerConnection:  apicaller.NewExternalControllerConnection,
+		NewRemoteRelationsFacade: remoterelations.NewRemoteRelationsFacade,
+		NewWorker:                remoterelations.NewWorker,
+	}))
 	return result
 }
 
