@@ -21,10 +21,13 @@ import (
 	"github.com/juju/juju/api/uniter"
 	"github.com/juju/juju/cmd/jujud/agent/unit"
 	cmdutil "github.com/juju/juju/cmd/jujud/util"
+	"github.com/juju/juju/upgrades"
 	jworker "github.com/juju/juju/worker"
 	"github.com/juju/juju/worker/dependency"
+	"github.com/juju/juju/worker/gate"
 	"github.com/juju/juju/worker/introspection"
 	"github.com/juju/juju/worker/logsender"
+	"github.com/juju/juju/worker/upgradesteps"
 )
 
 var (
@@ -48,8 +51,9 @@ type UnitAgent struct {
 	// Used to signal that the upgrade worker will not
 	// reboot the agent on startup because there are no
 	// longer any immediately pending agent upgrades.
-	// Channel used as a selectable bool (closed means true).
-	initialUpgradeCheckComplete chan struct{}
+	initialUpgradeCheckComplete gate.Lock
+	preUpgradeSteps             upgrades.PreUpgradeStepsFunc
+	upgradeComplete             gate.Lock
 
 	prometheusRegistry *prometheus.Registry
 }
@@ -64,9 +68,10 @@ func NewUnitAgent(ctx *cmd.Context, bufferedLogger *logsender.BufferedLogWriter)
 		AgentConf:        NewAgentConf(""),
 		configChangedVal: voyeur.NewValue(true),
 		ctx:              ctx,
-		initialUpgradeCheckComplete: make(chan struct{}),
+		initialUpgradeCheckComplete: gate.NewLock(),
 		bufferedLogger:              bufferedLogger,
 		prometheusRegistry:          prometheusRegistry,
+		preUpgradeSteps:             upgrades.PreUpgradeSteps,
 	}, nil
 }
 
@@ -127,6 +132,14 @@ func (a *UnitAgent) Stop() error {
 	return a.tomb.Wait()
 }
 
+func (a *UnitAgent) isUpgradeRunning() bool {
+	return !a.upgradeComplete.IsUnlocked()
+}
+
+func (a *UnitAgent) isInitialUpgradeCheckPending() bool {
+	return !a.initialUpgradeCheckComplete.IsUnlocked()
+}
+
 // Run runs a unit agent.
 func (a *UnitAgent) Run(ctx *cmd.Context) error {
 	defer a.tomb.Done()
@@ -150,6 +163,9 @@ func (a *UnitAgent) APIWorkers() (worker.Worker, error) {
 		})
 	}
 
+	agentConfig := a.AgentConf.CurrentConfig()
+	a.upgradeComplete = upgradesteps.NewLock(agentConfig)
+
 	manifolds := unitManifolds(unit.ManifoldsConfig{
 		Agent:                agent.APIHostPortsSetter{a},
 		LogSource:            a.bufferedLogger.Logs(),
@@ -158,6 +174,10 @@ func (a *UnitAgent) APIWorkers() (worker.Worker, error) {
 		ValidateMigration:    a.validateMigration,
 		PrometheusRegisterer: a.prometheusRegistry,
 		UpdateLoggerConfig:   updateAgentConfLogging,
+		PreviousAgentVersion: agentConfig.UpgradedToVersion(),
+		PreUpgradeSteps:      a.preUpgradeSteps,
+		UpgradeStepsLock:     a.upgradeComplete,
+		UpgradeCheckLock:     a.initialUpgradeCheckComplete,
 	})
 
 	config := dependency.EngineConfig{
