@@ -528,7 +528,7 @@ func storeImageMetadataInState(st *state.State, env environs.Environ, source str
 		}
 		metadataState[i] = m
 	}
-	if err := st.CloudImageMetadataStorage.SaveMetadata(metadataState); err != nil {
+	if err := st.CloudImageMetadataStorage.SaveMetadataNoExpiry(metadataState); err != nil {
 		return errors.Annotatef(err, "cannot cache image metadata")
 	}
 	return nil
