@@ -58,12 +58,13 @@ type ModelAPI interface {
 // an API connection.
 type CommandBase struct {
 	cmd.CommandBase
-	cmdContext  *cmd.Context
-	apiContexts map[string]*apiContext
-	modelAPI_   ModelAPI
-	apiOpenFunc api.OpenFunc
-	authOpts    AuthOpts
-	runStarted  bool
+	cmdContext    *cmd.Context
+	apiContexts   map[string]*apiContext
+	modelAPI_     ModelAPI
+	apiOpenFunc   api.OpenFunc
+	authOpts      AuthOpts
+	runStarted    bool
+	refreshModels func(jujuclient.ClientStore, string) error
 }
 
 func (c *CommandBase) assertRunStarted() {
@@ -102,6 +103,11 @@ func (c *CommandBase) SetAPIOpen(apiOpen api.OpenFunc) {
 	c.apiOpenFunc = apiOpen
 }
 
+// SetModelRefresh sets the function used for refreshing models.
+func (c *CommandBase) SetModelRefresh(refresh func(jujuclient.ClientStore, string) error) {
+	c.refreshModels = refresh
+}
+
 func (c *CommandBase) modelAPI(store jujuclient.ClientStore, controllerName string) (ModelAPI, error) {
 	c.assertRunStarted()
 	if c.modelAPI_ != nil {
@@ -164,15 +170,7 @@ func (c *CommandBase) missingModelError(store jujuclient.ClientStore, controller
 			logger.Warningf("cannot reset current model: %v", err)
 		}
 	}
-	errorMessage := "model %q has been removed from the controller, run 'juju models' and switch to one of them."
-	modelInfoMessage := "\nThere are %d accessible models on controller %q."
-	models, err := store.AllModels(controllerName)
-	if err == nil {
-		modelInfoMessage = fmt.Sprintf(modelInfoMessage, len(models), controllerName)
-	} else {
-		modelInfoMessage = ""
-	}
-	return errors.Errorf(errorMessage+modelInfoMessage, modelName)
+	return errors.Errorf("model %q has been removed from the controller, run 'juju models' and switch to one of them.", modelName)
 }
 
 // NewAPIConnectionParams returns a juju.NewAPIConnectionParams with the
@@ -256,6 +254,13 @@ func (c *CommandBase) apiOpen(info *api.Info, opts api.DialOpts) (api.Connection
 // RefreshModels refreshes the local models cache for the current user
 // on the specified controller.
 func (c *CommandBase) RefreshModels(store jujuclient.ClientStore, controllerName string) error {
+	if c.refreshModels == nil {
+		return c.doRefreshModels(store, controllerName)
+	}
+	return c.refreshModels(store, controllerName)
+}
+
+func (c *CommandBase) doRefreshModels(store jujuclient.ClientStore, controllerName string) error {
 	c.assertRunStarted()
 	modelManager, err := c.modelAPI(store, controllerName)
 	if err != nil {
@@ -272,13 +277,22 @@ func (c *CommandBase) RefreshModels(store jujuclient.ClientStore, controllerName
 	if err != nil {
 		return errors.Trace(err)
 	}
+	if err := c.SetControllerModels(store, controllerName, models); err != nil {
+		return errors.Trace(err)
+	}
+	return nil
+}
+
+func (c *CommandBase) SetControllerModels(store jujuclient.ClientStore, controllerName string, models []base.UserModel) error {
+	modelsToStore := make(map[string]jujuclient.ModelDetails, len(models))
 	for _, model := range models {
 		modelDetails := jujuclient.ModelDetails{model.UUID}
 		owner := names.NewUserTag(model.Owner)
 		modelName := jujuclient.JoinOwnerModelName(owner, model.Name)
-		if err := store.UpdateModel(controllerName, modelName, modelDetails); err != nil {
-			return errors.Trace(err)
-		}
+		modelsToStore[modelName] = modelDetails
+	}
+	if err := store.SetModels(controllerName, modelsToStore); err != nil {
+		return errors.Trace(err)
 	}
 	return nil
 }
@@ -431,18 +445,6 @@ type bootstrapConfigGetter struct {
 	registry environs.ProviderRegistry
 }
 
-func (g bootstrapConfigGetter) getBootstrapConfig(controllerName string) (*config.Config, error) {
-	bootstrapConfig, params, err := g.getBootstrapConfigParams(controllerName)
-	if err != nil {
-		return nil, errors.Trace(err)
-	}
-	provider, err := g.registry.Provider(bootstrapConfig.CloudType)
-	if err != nil {
-		return nil, errors.Trace(err)
-	}
-	return provider.PrepareConfig(*params)
-}
-
 func (g bootstrapConfigGetter) getBootstrapConfigParams(controllerName string) (*jujuclient.BootstrapConfig, *environs.PrepareConfigParams, error) {
 	controllerDetails, err := g.store.ControllerByName(controllerName)
 	if err != nil {
@@ -454,13 +456,13 @@ func (g bootstrapConfigGetter) getBootstrapConfigParams(controllerName string) (
 	}
 
 	var credential *cloud.Credential
+	bootstrapCloud := cloud.Cloud{
+		Name:             bootstrapConfig.Cloud,
+		Type:             bootstrapConfig.CloudType,
+		Endpoint:         bootstrapConfig.CloudEndpoint,
+		IdentityEndpoint: bootstrapConfig.CloudIdentityEndpoint,
+	}
 	if bootstrapConfig.Credential != "" {
-		bootstrapCloud := cloud.Cloud{
-			Name:             bootstrapConfig.Cloud,
-			Type:             bootstrapConfig.CloudType,
-			Endpoint:         bootstrapConfig.CloudEndpoint,
-			IdentityEndpoint: bootstrapConfig.CloudIdentityEndpoint,
-		}
 		if bootstrapConfig.CloudRegion != "" {
 			bootstrapCloud.Regions = []cloud.Region{{
 				Name:             bootstrapConfig.CloudRegion,
@@ -491,8 +493,14 @@ func (g bootstrapConfigGetter) getBootstrapConfigParams(controllerName string) (
 		}
 		// DetectCredential ensures that there is only one credential
 		// to choose from. It's still in a map, though, hence for..range.
-		for _, one := range cloudCredential.AuthCredentials {
+		var credentialName string
+		for name, one := range cloudCredential.AuthCredentials {
 			credential = &one
+			credentialName = name
+		}
+		credential, err = FinalizeFileContent(credential, provider)
+		if err != nil {
+			return nil, nil, AnnotateWithFinalizationError(err, credentialName, bootstrapCloud.Name)
 		}
 		credential, err = provider.FinalizeCredential(
 			g.ctx, environs.FinalizeCredentialParams{
