@@ -16,6 +16,7 @@ import (
 	"gopkg.in/macaroon-bakery.v1/httpbakery"
 
 	"github.com/juju/juju/api"
+	"github.com/juju/juju/api/modelmanager"
 	"github.com/juju/juju/environs"
 	"github.com/juju/juju/juju/osenv"
 	"github.com/juju/juju/jujuclient"
@@ -202,33 +203,39 @@ func (c *ModelCommandBase) NewAPIClient() (*api.Client, error) {
 	return root.Client(), nil
 }
 
-// NewAPIRoot returns a new connection to the API server for the environment
-// directed to the model specified on the command line.
-func (c *ModelCommandBase) NewAPIRoot() (api.Connection, error) {
+func (c *ModelCommandBase) ModelDetails() (string, *jujuclient.ModelDetails, error) {
 	modelName, err := c.ModelName()
 	if err != nil {
-		return nil, errors.Trace(err)
+		return "", nil, errors.Trace(err)
 	}
 	controllerName, err := c.ControllerName()
 	if err != nil {
-		return nil, errors.Trace(err)
+		return "", nil, errors.Trace(err)
 	}
-	// This is work in progress as we remove the ModelName from downstream code.
-	// We want to be able to specify the environment in a number of ways, one of
-	// which is the connection name on the client machine.
 	if modelName == "" {
-		return nil, errors.Trace(ErrNoModelSpecified)
+		return "", nil, errors.Trace(ErrNoModelSpecified)
 	}
-	_, err = c.store.ModelByName(controllerName, modelName)
+	details, err := c.store.ModelByName(controllerName, modelName)
 	if err != nil {
 		if !errors.IsNotFound(err) {
-			return nil, errors.Trace(err)
+			return "", nil, errors.Trace(err)
 		}
 		// The model isn't known locally, so query the models
 		// available in the controller, and cache them locally.
 		if err := c.RefreshModels(c.store, controllerName); err != nil {
-			return nil, errors.Annotate(err, "refreshing models")
+			return "", nil, errors.Annotate(err, "refreshing models")
 		}
+		details, err = c.store.ModelByName(controllerName, modelName)
+	}
+	return modelName, details, err
+}
+
+// NewAPIRoot returns a new connection to the API server for the environment
+// directed to the model specified on the command line.
+func (c *ModelCommandBase) NewAPIRoot() (api.Connection, error) {
+	modelName, _, err := c.ModelDetails()
+	if err != nil {
+		return nil, errors.Trace(err)
 	}
 	return c.newAPIRoot(modelName)
 }
@@ -270,6 +277,16 @@ func (c *ModelCommandBase) CurrentAccountDetails() (*jujuclient.AccountDetails,
 	return c.ClientStore().AccountDetails(controllerName)
 }
 
+// NewModelManagerAPIClient returns an API client for the
+// ModelManager on the current controller using the current credentials.
+func (c *ModelCommandBase) NewModelManagerAPIClient() (*modelmanager.Client, error) {
+	root, err := c.NewControllerAPIRoot()
+	if err != nil {
+		return nil, errors.Trace(err)
+	}
+	return modelmanager.NewClient(root), nil
+}
+
 // WrapOption specifies an option to the Wrap function.
 type WrapOption func(*modelCommandWrapper)
 
