@@ -12,7 +12,6 @@ import (
 	"github.com/juju/errors"
 	"github.com/juju/gnuflag"
 	"github.com/juju/utils/arch"
-	"github.com/juju/utils/series"
 
 	"github.com/juju/juju/cmd/modelcmd"
 	"github.com/juju/juju/environs"
@@ -21,6 +20,7 @@ import (
 	"github.com/juju/juju/environs/imagemetadata"
 	"github.com/juju/juju/environs/simplestreams"
 	"github.com/juju/juju/environs/storage"
+	"github.com/juju/juju/juju/version"
 )
 
 type imageMetadataCommandBase struct {
@@ -149,7 +149,7 @@ func (c *imageMetadataCommand) setParams(context *cmd.Context) error {
 		logger.Infof("no model found, creating image metadata using user supplied data")
 	}
 	if c.Series == "" {
-		c.Series = series.LatestLts()
+		c.Series = version.SupportedLTS()
 	}
 	if c.ImageId == "" {
 		return errors.Errorf("image id must be specified")
