@@ -4,21 +4,14 @@
 package all
 
 import (
-	"os"
-
 	jujucmd "github.com/juju/cmd"
 	"github.com/juju/errors"
 	"gopkg.in/juju/names.v2"
 
 	"github.com/juju/juju/api/base"
-	"github.com/juju/juju/apiserver/charmrevisionupdater"
-	"github.com/juju/juju/cmd/juju/charmcmd"
-	"github.com/juju/juju/cmd/juju/commands"
-	"github.com/juju/juju/cmd/modelcmd"
+	"github.com/juju/juju/apiserver/facades/controller/charmrevisionupdater"
 	"github.com/juju/juju/resource"
-	"github.com/juju/juju/resource/api/client"
 	internalclient "github.com/juju/juju/resource/api/private/client"
-	"github.com/juju/juju/resource/cmd"
 	"github.com/juju/juju/resource/context"
 	contextcmd "github.com/juju/juju/resource/context/cmd"
 	"github.com/juju/juju/resource/resourceadapters"
@@ -42,24 +35,11 @@ func (r resources) registerForServer() error {
 // RegisterForClient is the top-level registration method
 // for the component in a "juju" command context.
 func (r resources) registerForClient() error {
-	r.registerPublicCommands()
-
-	// needed for help-tool
+	// needed for hook-tool
 	r.registerHookContextCommands()
 	return nil
 }
 
-// resourcesAPIClient adds a Close() method to the resources public API client.
-type resourcesAPIClient struct {
-	*client.Client
-	closeConnFunc func() error
-}
-
-// Close implements io.Closer.
-func (client resourcesAPIClient) Close() error {
-	return client.closeConnFunc()
-}
-
 // registerAgentWorkers adds the resources workers to the agents.
 func (r resources) registerAgentWorkers() {
 	if !markRegistered(resource.ComponentName, "agent-workers") {
@@ -76,44 +56,6 @@ func (resources) registerState() {
 	}
 }
 
-// registerPublicCommands adds the resources-related commands
-// to the "juju" supercommand.
-func (r resources) registerPublicCommands() {
-	if !markRegistered(resource.ComponentName, "public-commands") {
-		return
-	}
-
-	charmcmd.RegisterSubCommand(cmd.NewListCharmResourcesCommand())
-
-	commands.RegisterEnvCommand(func() modelcmd.ModelCommand {
-		return cmd.NewUploadCommand(cmd.UploadDeps{
-			NewClient: func(c *cmd.UploadCommand) (cmd.UploadClient, error) {
-				apiRoot, err := c.NewAPIRoot()
-				if err != nil {
-					return nil, errors.Trace(err)
-				}
-				return resourceadapters.NewAPIClient(apiRoot)
-			},
-			OpenResource: func(s string) (cmd.ReadSeekCloser, error) {
-				return os.Open(s)
-			},
-		})
-
-	})
-
-	commands.RegisterEnvCommand(func() modelcmd.ModelCommand {
-		return cmd.NewShowServiceCommand(cmd.ShowServiceDeps{
-			NewClient: func(c *cmd.ShowServiceCommand) (cmd.ShowServiceClient, error) {
-				apiRoot, err := c.NewAPIRoot()
-				if err != nil {
-					return nil, errors.Trace(err)
-				}
-				return resourceadapters.NewAPIClient(apiRoot)
-			},
-		})
-	})
-}
-
 func (r resources) registerHookContext() {
 	if markRegistered(resource.ComponentName, "hook-context") == false {
 		return
