@@ -10,17 +10,16 @@ import (
 
 	"github.com/juju/juju/container"
 	"github.com/juju/juju/container/kvm"
-	"github.com/juju/juju/container/lxc"
+	"github.com/juju/juju/container/lxd"
 	"github.com/juju/juju/instance"
 )
 
 // NewContainerManager creates the appropriate container.Manager for the
 // specified container type.
-func NewContainerManager(forType instance.ContainerType, conf container.ManagerConfig, imageURLGetter container.ImageURLGetter,
-) (container.Manager, error) {
+func NewContainerManager(forType instance.ContainerType, conf container.ManagerConfig) (container.Manager, error) {
 	switch forType {
-	case instance.LXC:
-		return lxc.NewContainerManager(conf, imageURLGetter)
+	case instance.LXD:
+		return lxd.NewContainerManager(conf)
 	case instance.KVM:
 		return kvm.NewContainerManager(conf)
 	}
