@@ -8,6 +8,7 @@ import (
 
 	"github.com/juju/juju/instance"
 	"github.com/juju/juju/network"
+	"github.com/juju/juju/status"
 )
 
 type kvmInstance struct {
@@ -23,11 +24,17 @@ func (kvm *kvmInstance) Id() instance.Id {
 }
 
 // Status implements instance.Instance.Status.
-func (kvm *kvmInstance) Status() string {
+func (kvm *kvmInstance) Status() instance.InstanceStatus {
 	if kvm.container.IsRunning() {
-		return "running"
+		return instance.InstanceStatus{
+			Status:  status.Running,
+			Message: "running",
+		}
+	}
+	return instance.InstanceStatus{
+		Status:  status.Stopped,
+		Message: "stopped",
 	}
-	return "stopped"
 }
 
 func (*kvmInstance) Refresh() error {
@@ -40,17 +47,17 @@ func (kvm *kvmInstance) Addresses() ([]network.Address, error) {
 }
 
 // OpenPorts implements instance.Instance.OpenPorts.
-func (kvm *kvmInstance) OpenPorts(machineId string, ports []network.Port) error {
+func (kvm *kvmInstance) OpenPorts(machineId string, rules []network.IngressRule) error {
 	return fmt.Errorf("not implemented")
 }
 
 // ClosePorts implements instance.Instance.ClosePorts.
-func (kvm *kvmInstance) ClosePorts(machineId string, ports []network.Port) error {
+func (kvm *kvmInstance) ClosePorts(machineId string, rules []network.IngressRule) error {
 	return fmt.Errorf("not implemented")
 }
 
-// Ports implements instance.Instance.Ports.
-func (kvm *kvmInstance) Ports(machineId string) ([]network.Port, error) {
+// IngressRules implements instance.Instance.IngressRules.
+func (kvm *kvmInstance) IngressRules(machineId string) ([]network.IngressRule, error) {
 	return nil, fmt.Errorf("not implemented")
 }
 
