@@ -6,12 +6,14 @@ package controller
 import (
 	"fmt"
 	"net/url"
+	"regexp"
 	"time"
 
 	"github.com/juju/errors"
 	"github.com/juju/schema"
 	"github.com/juju/utils"
 	utilscert "github.com/juju/utils/cert"
+	"github.com/juju/utils/set"
 	"gopkg.in/macaroon-bakery.v1/bakery"
 
 	"github.com/juju/juju/cert"
@@ -32,6 +34,33 @@ const (
 	// auditing information.
 	AuditingEnabled = "auditing-enabled"
 
+	// AuditLogCaptureArgs determines whether the audit log will
+	// contain the arguments passed to API methods.
+	AuditLogCaptureArgs = "audit-log-capture-args"
+
+	// AuditLogMaxSize is the maximum size for the current audit log
+	// file, eg "250M".
+	AuditLogMaxSize = "audit-log-max-size"
+
+	// AuditLogMaxBackups is the number of old audit log files to keep
+	// (compressed).
+	AuditLogMaxBackups = "audit-log-max-backups"
+
+	// AuditLogExcludeMethods is a list of Facade.Method names that
+	// aren't interesting for audit logging purposes. A conversation
+	// with only calls to these will be excluded from the
+	// log. (They'll still appear in conversations that have other
+	// interesting calls though.)
+	AuditLogExcludeMethods = "audit-log-exclude-methods"
+
+	// ReadOnlyMethodsWildcard is the special value that can be added
+	// to the exclude-methods list that represents all of the read
+	// only methods (see apiserver/observer/auditfilter.go). This
+	// value will be stored in the DB (rather than being expanded at
+	// write time) so any changes to the set of read-only methods in
+	// new versions of Juju will be honoured.
+	ReadOnlyMethodsWildcard = "ReadOnlyMethods"
+
 	// StatePort is the port used for mongo connections.
 	StatePort = "state-port"
 
@@ -72,7 +101,7 @@ const (
 	// detault
 	MongoMemoryProfile = "mongo-memory-profile"
 
-	// MaxLogsAge is the maximum age for log entries, ef "72h"
+	// MaxLogsAge is the maximum age for log entries, eg "72h"
 	MaxLogsAge = "max-logs-age"
 
 	// MaxLogsSize is the maximum size the log collection can grow to
@@ -86,7 +115,19 @@ const (
 
 	// DefaultAuditingEnabled contains the default value for the
 	// AuditingEnabled config value.
-	DefaultAuditingEnabled = false
+	DefaultAuditingEnabled = true
+
+	// DefaultAuditLogCaptureArgs is the default for the
+	// AuditLogCaptureArgs setting (which is not to capture them).
+	DefaultAuditLogCaptureArgs = false
+
+	// DefaultAuditLogMaxSizeMB is the default size in MB at which we
+	// roll the audit log file.
+	DefaultAuditLogMaxSizeMB = 300
+
+	// DefaultAuditLogMaxBackups is the default number of files to
+	// keep.
+	DefaultAuditLogMaxBackups = 10
 
 	// DefaultNUMAControlPolicy should not be used by default.
 	// Only use numactl if user specifically requests it
@@ -112,24 +153,53 @@ const (
 	DefaultMaxTxnLogCollectionMB = 10 // 10 MB
 )
 
-// ControllerOnlyConfigAttributes are attributes which are only relevant
-// for a controller, never a model.
-var ControllerOnlyConfigAttributes = []string{
-	AllowModelAccessKey,
-	APIPort,
-	AutocertDNSNameKey,
-	AutocertURLKey,
-	CACertKey,
-	ControllerUUIDKey,
-	IdentityPublicKey,
-	IdentityURL,
-	SetNUMAControlPolicyKey,
-	StatePort,
-	MongoMemoryProfile,
-	MaxLogsSize,
-	MaxLogsAge,
-	MaxTxnLogSize,
-}
+var (
+	// ControllerOnlyConfigAttributes are attributes which are only relevant
+	// for a controller, never a model.
+	ControllerOnlyConfigAttributes = []string{
+		AllowModelAccessKey,
+		APIPort,
+		AutocertDNSNameKey,
+		AutocertURLKey,
+		CACertKey,
+		ControllerUUIDKey,
+		IdentityPublicKey,
+		IdentityURL,
+		SetNUMAControlPolicyKey,
+		StatePort,
+		MongoMemoryProfile,
+		MaxLogsSize,
+		MaxLogsAge,
+		MaxTxnLogSize,
+		AuditingEnabled,
+		AuditLogCaptureArgs,
+		AuditLogMaxSize,
+		AuditLogMaxBackups,
+		AuditLogExcludeMethods,
+	}
+
+	// AllowedUpdateConfigAttributes contains all of the controller
+	// config attributes that are allowed to be updated after the
+	// controller has been created.
+	// TODO(babbageclunk): initially this will only be audit log
+	// values, but we should work out which others can also be changed
+	// safely.
+	AllowedUpdateConfigAttributes = set.NewStrings(
+		AuditingEnabled,
+		AuditLogCaptureArgs,
+		AuditLogExcludeMethods,
+	)
+
+	// DefaultAuditLogExcludeMethods is the default list of methods to
+	// exclude from the audit log.
+	DefaultAuditLogExcludeMethods = []string{
+		// This special value means we exclude any methods in the set
+		// listed in apiserver/observer/auditfilter.go
+		ReadOnlyMethodsWildcard,
+	}
+
+	methodNameRE = regexp.MustCompile(`[[:alpha:]][[:alnum:]]*\.[[:alpha:]][[:alnum:]]*`)
+)
 
 // ControllerOnlyAttribute returns true if the specified attribute name
 // is only relevant for a controller.
@@ -216,7 +286,52 @@ func (c Config) AuditingEnabled() bool {
 	if v, ok := c[AuditingEnabled]; ok {
 		return v.(bool)
 	}
-	return false
+	return DefaultAuditingEnabled
+}
+
+// AuditLogCaptureArgs returns whether audit logging should capture
+// the arguments to API methods. The default is false.
+func (c Config) AuditLogCaptureArgs() bool {
+	if v, ok := c[AuditLogCaptureArgs]; ok {
+		return v.(bool)
+	}
+	return DefaultAuditLogCaptureArgs
+}
+
+// AuditLogMaxSizeMB returns the maximum size for an audit log file in
+// MB.
+func (c Config) AuditLogMaxSizeMB() int {
+	// Value has already been validated.
+	value, _ := utils.ParseSize(c.asString(AuditLogMaxSize))
+	return int(value)
+}
+
+// AuditLogMaxBackups returns the maximum number of backup audit log
+// files to keep.
+func (c Config) AuditLogMaxBackups() int {
+	if value, ok := c[AuditLogMaxBackups]; ok {
+		// Values obtained over the API are encoded as float64.
+		if floatValue, ok := value.(float64); ok {
+			return int(floatValue)
+		}
+		return value.(int)
+	}
+	return DefaultAuditLogMaxBackups
+}
+
+// AuditLogExcludeMethods returns the set of method names that are
+// considered uninteresting for audit logging. Conversations
+// containing only these will be excluded from the audit log.
+func (c Config) AuditLogExcludeMethods() set.Strings {
+	if value, ok := c[AuditLogExcludeMethods]; ok {
+		value := value.([]interface{})
+		items := set.NewStrings()
+		for _, item := range value {
+			items.Add(item.(string))
+		}
+		return items
+	}
+	return set.NewStrings(DefaultAuditLogExcludeMethods...)
 }
 
 // ControllerUUID returns the uuid for the model's controller.
@@ -372,6 +487,40 @@ func Validate(c Config) error {
 		}
 	}
 
+	var auditLogMaxSize int
+	if v, ok := c[AuditLogMaxSize].(string); ok {
+		if size, err := utils.ParseSize(v); err != nil {
+			return errors.Annotate(err, "invalid audit log max size in configuration")
+		} else {
+			auditLogMaxSize = int(size)
+		}
+	}
+
+	if v, ok := c[AuditingEnabled].(bool); ok {
+		if v && auditLogMaxSize == 0 {
+			return errors.Errorf("invalid audit log max size: can't be 0 if auditing is enabled")
+		}
+	}
+
+	if v, ok := c[AuditLogMaxBackups].(int); ok {
+		if v < 0 {
+			return errors.Errorf("invalid audit log max backups: should be a number of files (or 0 to keep all), got %d", v)
+		}
+	}
+
+	if v, ok := c[AuditLogExcludeMethods].([]interface{}); ok {
+		for i, name := range v {
+			name := name.(string)
+			if name != ReadOnlyMethodsWildcard && !methodNameRE.MatchString(name) {
+				return errors.Errorf(
+					`invalid audit log exclude methods: should be a list of "Facade.Method" names (or "ReadOnlyMethods"), got %q at position %d`,
+					name,
+					i+1,
+				)
+			}
+		}
+	}
+
 	return nil
 }
 
@@ -383,6 +532,10 @@ func GenerateControllerCertAndKey(caCert, caKey string, hostAddresses []string)
 
 var configChecker = schema.FieldMap(schema.Fields{
 	AuditingEnabled:         schema.Bool(),
+	AuditLogCaptureArgs:     schema.Bool(),
+	AuditLogMaxSize:         schema.String(),
+	AuditLogMaxBackups:      schema.ForceInt(),
+	AuditLogExcludeMethods:  schema.List(schema.String()),
 	APIPort:                 schema.ForceInt(),
 	StatePort:               schema.ForceInt(),
 	IdentityURL:             schema.String(),
@@ -398,6 +551,10 @@ var configChecker = schema.FieldMap(schema.Fields{
 }, schema.Defaults{
 	APIPort:                 DefaultAPIPort,
 	AuditingEnabled:         DefaultAuditingEnabled,
+	AuditLogCaptureArgs:     DefaultAuditLogCaptureArgs,
+	AuditLogMaxSize:         fmt.Sprintf("%vM", DefaultAuditLogMaxSizeMB),
+	AuditLogMaxBackups:      DefaultAuditLogMaxBackups,
+	AuditLogExcludeMethods:  DefaultAuditLogExcludeMethods,
 	StatePort:               DefaultStatePort,
 	IdentityURL:             schema.Omit,
 	IdentityPublicKey:       schema.Omit,
