@@ -15,7 +15,7 @@ import (
 	"github.com/juju/schema"
 	"github.com/juju/utils"
 	"github.com/juju/utils/proxy"
-	"github.com/juju/utils/series"
+	"github.com/juju/utils/set"
 	"github.com/juju/version"
 	"gopkg.in/juju/charmrepo.v2"
 	"gopkg.in/juju/environschema.v1"
@@ -25,6 +25,7 @@ import (
 	"github.com/juju/juju/controller"
 	"github.com/juju/juju/environs/tags"
 	"github.com/juju/juju/juju/osenv"
+	jujuversion "github.com/juju/juju/juju/version"
 	"github.com/juju/juju/logfwd/syslog"
 	"github.com/juju/juju/network"
 )
@@ -79,6 +80,14 @@ const (
 	// AgentMetadataURLKey stores the key for this setting.
 	AgentMetadataURLKey = "agent-metadata-url"
 
+	// ContainerImageStreamKey is the key used to specify the stream
+	// to for container OS images.
+	ContainerImageStreamKey = "container-image-stream"
+
+	// ContainerImageMetadataURLKey is the key used to specify the location
+	// of OS image metadata for containers.
+	ContainerImageMetadataURLKey = "container-image-metadata-url"
+
 	// HTTPProxyKey stores the key for this setting.
 	HTTPProxyKey = "http-proxy"
 
@@ -177,10 +186,16 @@ const (
 	// FanConfig defines the configuration for FAN network running in the model.
 	FanConfig = "fan-config"
 
-	// CloudInitUserDataKey is the key to specify cloud-init yaml the user wants to add
-	// into the cloud-config data produced by Juju when provisioning machines.
+	// CloudInitUserDataKey is the key to specify cloud-init yaml the user
+	// wants to add into the cloud-config data produced by Juju when
+	// provisioning machines.
 	CloudInitUserDataKey = "cloudinit-userdata"
 
+	// ContainerInheritProperiesKey is the key to specify a list of properties
+	// to be copied from a machine to a container during provisioning. The
+	// list will be comma separated.
+	ContainerInheritProperiesKey = "container-inherit-properties"
+
 	//
 	// Deprecated Settings Attributes
 	//
@@ -265,7 +280,7 @@ func PreferredSeries(cfg HasDefaultSeries) string {
 	if series, ok := cfg.DefaultSeries(); ok {
 		return series
 	}
-	return series.LatestLts()
+	return jujuversion.SupportedLTS()
 }
 
 // Config holds an immutable environment configuration.
@@ -374,26 +389,29 @@ var defaultConfigValues = map[string]interface{}{
 	NetBondReconfigureDelayKey: 17,
 	ContainerNetworkingMethod:  "",
 
-	"default-series":           series.LatestLts(),
-	ProvisionerHarvestModeKey:  HarvestDestroyed.String(),
-	ResourceTagsKey:            "",
-	"logging-config":           "",
-	AutomaticallyRetryHooks:    true,
-	"enable-os-refresh-update": true,
-	"enable-os-upgrade":        true,
-	"development":              false,
-	"test-mode":                false,
-	TransmitVendorMetricsKey:   true,
-	UpdateStatusHookInterval:   DefaultUpdateStatusHookInterval,
-	EgressSubnets:              "",
-	FanConfig:                  "",
-	CloudInitUserDataKey:       "",
+	"default-series":             jujuversion.SupportedLTS(),
+	ProvisionerHarvestModeKey:    HarvestDestroyed.String(),
+	ResourceTagsKey:              "",
+	"logging-config":             "",
+	AutomaticallyRetryHooks:      true,
+	"enable-os-refresh-update":   true,
+	"enable-os-upgrade":          true,
+	"development":                false,
+	"test-mode":                  false,
+	TransmitVendorMetricsKey:     true,
+	UpdateStatusHookInterval:     DefaultUpdateStatusHookInterval,
+	EgressSubnets:                "",
+	FanConfig:                    "",
+	CloudInitUserDataKey:         "",
+	ContainerInheritProperiesKey: "",
 
 	// Image and agent streams and URLs.
-	"image-stream":       "released",
-	"image-metadata-url": "",
-	AgentStreamKey:       "released",
-	AgentMetadataURLKey:  "",
+	"image-stream":               "released",
+	"image-metadata-url":         "",
+	AgentStreamKey:               "released",
+	AgentMetadataURLKey:          "",
+	ContainerImageStreamKey:      "released",
+	ContainerImageMetadataURLKey: "",
 
 	// Log forward settings.
 	LogForwardEnabled: false,
@@ -603,9 +621,9 @@ func Validate(cfg, old *Config) error {
 	}
 
 	if raw, ok := cfg.defined[CloudInitUserDataKey].(string); ok && raw != "" {
-		userDataMap := make(map[string]interface{})
-		if err := yaml.Unmarshal([]byte(raw), &userDataMap); err != nil {
-			return errors.Annotate(err, "cloudinit-userdata: must be valid YAML")
+		userDataMap, err := ensureStringMaps(raw)
+		if err != nil {
+			return errors.Annotate(err, "cloudinit-userdata")
 		}
 
 		// if there packages, ensure they are strings
@@ -627,6 +645,25 @@ func Validate(cfg, old *Config) error {
 		if _, ok := userDataMap["runcmd"]; ok {
 			return errors.New("cloudinit-userdata: runcmd not allowed, use preruncmd or postruncmd instead")
 		}
+
+		// error if bootcmd is specified
+		if _, ok := userDataMap["bootcmd"]; ok {
+			return errors.New("cloudinit-userdata: bootcmd not allowed")
+		}
+	}
+
+	if raw, ok := cfg.defined[ContainerInheritProperiesKey].(string); ok && raw != "" {
+		rawProperties := strings.Split(raw, ",")
+		propertySet := set.NewStrings()
+		for _, prop := range rawProperties {
+			propertySet.Add(strings.TrimSpace(prop))
+		}
+		whiteListSet := set.NewStrings("apt-primary", "apt-sources", "apt-security", "ca-certs")
+		diffSet := propertySet.Difference(whiteListSet)
+
+		if !diffSet.IsEmpty() {
+			return fmt.Errorf("container-inherit-properties: %s not allowed", strings.Join(diffSet.SortedValues(), ", "))
+		}
 	}
 
 	// Check the immutable config values.  These can't change
@@ -651,6 +688,20 @@ func Validate(cfg, old *Config) error {
 	return nil
 }
 
+// ensureStringMaps takes in a string and returns YAML in a map
+// where all keys of any nested maps are strings.
+func ensureStringMaps(in string) (map[string]interface{}, error) {
+	userDataMap := make(map[string]interface{})
+	if err := yaml.Unmarshal([]byte(in), &userDataMap); err != nil {
+		return nil, errors.Annotate(err, "must be valid YAML")
+	}
+	out, err := utils.ConformYAML(userDataMap)
+	if err != nil {
+		return nil, err
+	}
+	return out.(map[string]interface{}), nil
+}
+
 func isEmpty(val interface{}) bool {
 	switch val := val.(type) {
 	case nil:
@@ -898,8 +949,8 @@ func (c *Config) AgentMetadataURL() (string, bool) {
 	return "", false
 }
 
-// ImageMetadataURL returns the URL at which the metadata used to locate image ids is located,
-// and wether it has been set.
+// ImageMetadataURL returns the URL at which the metadata used to locate image
+// ids is located, and whether it has been set.
 func (c *Config) ImageMetadataURL() (string, bool) {
 	if url, ok := c.defined["image-metadata-url"]; ok && url != "" {
 		return url.(string), true
@@ -907,6 +958,15 @@ func (c *Config) ImageMetadataURL() (string, bool) {
 	return "", false
 }
 
+// ContainerImageMetadataURL returns the URL at which the metadata used to
+// locate container OS image ids is located, and whether it has been set.
+func (c *Config) ContainerImageMetadataURL() (string, bool) {
+	if url, ok := c.defined[ContainerImageMetadataURLKey]; ok && url != "" {
+		return url.(string), true
+	}
+	return "", false
+}
+
 // Development returns whether the environment is in development mode.
 func (c *Config) Development() bool {
 	value, _ := c.defined["development"].(bool)
@@ -1002,6 +1062,16 @@ func (c *Config) AgentStream() string {
 	return "released"
 }
 
+// ContainerImageStream returns the simplestreams stream used to identify which
+// image ids to search when starting a container.
+func (c *Config) ContainerImageStream() string {
+	v, _ := c.defined[ContainerImageStreamKey].(string)
+	if v != "" {
+		return v
+	}
+	return "released"
+}
+
 // TestMode indicates if the environment is intended for testing.
 // In this case, accessing the charm store does not affect statistical
 // data of the store.
@@ -1134,11 +1204,17 @@ func (c *Config) FanConfig() (network.FanConfig, error) {
 func (c *Config) CloudInitUserData() map[string]interface{} {
 	raw := c.asString(CloudInitUserDataKey)
 	if raw == "" {
-		return map[string]interface{}{}
+		return nil
 	}
-	userDataMap := make(map[string]interface{})
-	yaml.Unmarshal([]byte(raw), &userDataMap)
-	return userDataMap
+	// The raw data has already passed Validate()
+	conformingUserDataMap, _ := ensureStringMaps(raw)
+	return conformingUserDataMap
+}
+
+// ContainerInheritProperies returns a copy of the raw user data keys
+// that were specified by the user.
+func (c *Config) ContainerInheritProperies() string {
+	return c.asString(ContainerInheritProperiesKey)
 }
 
 // UnknownAttrs returns a copy of the raw configuration attributes
@@ -1237,6 +1313,8 @@ var alwaysOptional = schema.Defaults{
 	"image-stream":               schema.Omit,
 	"image-metadata-url":         schema.Omit,
 	AgentMetadataURLKey:          schema.Omit,
+	ContainerImageStreamKey:      schema.Omit,
+	ContainerImageMetadataURLKey: schema.Omit,
 	"default-series":             schema.Omit,
 	"development":                schema.Omit,
 	"ssl-hostname-verification":  schema.Omit,
@@ -1256,6 +1334,7 @@ var alwaysOptional = schema.Defaults{
 	EgressSubnets:                schema.Omit,
 	FanConfig:                    schema.Omit,
 	CloudInitUserDataKey:         schema.Omit,
+	ContainerInheritProperiesKey: schema.Omit,
 }
 
 func allowEmpty(attr string) bool {
@@ -1531,6 +1610,16 @@ global or per instance security groups.`,
 		Type:        environschema.Tstring,
 		Group:       environschema.EnvironGroup,
 	},
+	ContainerImageMetadataURLKey: {
+		Description: "The URL at which the metadata used to locate container OS image ids is located",
+		Type:        environschema.Tstring,
+		Group:       environschema.EnvironGroup,
+	},
+	ContainerImageStreamKey: {
+		Description: `The simplestreams stream used to identify which image ids to search when starting a container.`,
+		Type:        environschema.Tstring,
+		Group:       environschema.EnvironGroup,
+	},
 	"logging-config": {
 		Description: `The configuration string to use when configuring Juju agent logging (see http://godoc.org/github.com/juju/loggo#ParseConfigurationString for details)`,
 		Type:        environschema.Tstring,
@@ -1686,4 +1775,9 @@ data of the store. (default false)`,
 		Type:        environschema.Tstring,
 		Group:       environschema.EnvironGroup,
 	},
+	ContainerInheritProperiesKey: {
+		Description: "List of properties to be copied from the host machine to new containers created in this model (comma-separated)",
+		Type:        environschema.Tstring,
+		Group:       environschema.EnvironGroup,
+	},
 }
