@@ -14,7 +14,6 @@ import (
 	gitjujutesting "github.com/juju/testing"
 	jc "github.com/juju/testing/checkers"
 	"github.com/juju/utils/proxy"
-	"github.com/juju/utils/series"
 	"github.com/juju/version"
 	gc "gopkg.in/check.v1"
 	"gopkg.in/juju/charmrepo.v2"
@@ -23,6 +22,7 @@ import (
 	"github.com/juju/juju/cert"
 	"github.com/juju/juju/environs/config"
 	"github.com/juju/juju/juju/osenv"
+	jujuversion "github.com/juju/juju/juju/version"
 	"github.com/juju/juju/testing"
 )
 
@@ -55,7 +55,7 @@ var sampleConfig = testing.Attrs{
 	"unknown":                    "my-unknown",
 	"ssl-hostname-verification":  true,
 	"development":                false,
-	"default-series":             series.LatestLts(),
+	"default-series":             jujuversion.SupportedLTS(),
 	"disable-network-management": false,
 	"ignore-machine-addresses":   false,
 	"automatically-retry-hooks":  true,
@@ -90,18 +90,20 @@ var configTests = []configTest{
 		useDefaults: config.UseDefaults,
 		attrs:       minimalConfigAttrs,
 	}, {
-		about:       "Agent Stream",
+		about:       "Streams",
 		useDefaults: config.UseDefaults,
 		attrs: minimalConfigAttrs.Merge(testing.Attrs{
-			"image-metadata-url": "image-url",
-			"agent-stream":       "released",
+			"image-stream":           "released",
+			"agent-stream":           "released",
+			"container-image-stream": "daily",
 		}),
 	}, {
 		about:       "Metadata URLs",
 		useDefaults: config.UseDefaults,
 		attrs: minimalConfigAttrs.Merge(testing.Attrs{
-			"image-metadata-url": "image-url",
-			"agent-metadata-url": "agent-metadata-url-value",
+			"image-metadata-url":           "image-url",
+			"agent-metadata-url":           "agent-metadata-url-value",
+			"container-image-metadata-url": "container-image-metadata-url-value",
 		}),
 	}, {
 		about:       "Explicit series",
@@ -550,6 +552,19 @@ var configTests = []configTest{
 			"syslog-client-cert": testing.ServerCert,
 			"syslog-client-key":  testing.ServerKey,
 		}),
+	}, {
+		about:       "Valid container-inherit-properties",
+		useDefaults: config.UseDefaults,
+		attrs: minimalConfigAttrs.Merge(testing.Attrs{
+			"container-inherit-properties": "apt-primary, ca-certs",
+		}),
+	}, {
+		about:       "Invalid container-inherit-properties",
+		useDefaults: config.UseDefaults,
+		attrs: minimalConfigAttrs.Merge(testing.Attrs{
+			"container-inherit-properties": "apt-security, write_files,users,apt-sources",
+		}),
+		err: `container-inherit-properties: users, write_files not allowed`,
 	},
 }
 
@@ -615,7 +630,7 @@ func (test configTest) check(c *gc.C, home *gitjujutesting.FakeHome) {
 	if seriesAttr != "" {
 		c.Assert(defaultSeries, gc.Equals, seriesAttr)
 	} else {
-		c.Assert(defaultSeries, gc.Equals, series.LatestLts())
+		c.Assert(defaultSeries, gc.Equals, jujuversion.SupportedLTS())
 	}
 
 	if m, _ := test.attrs["firewall-mode"].(string); m != "" {
@@ -702,6 +717,20 @@ func (test configTest) check(c *gc.C, home *gitjujutesting.FakeHome) {
 	}
 	c.Assert(agentStreamValue, gc.Equals, expectedAgentStreamAttr)
 
+	containerURL, urlPresent := cfg.ContainerImageMetadataURL()
+	if v, _ := test.attrs["container-image-metadata-url"].(string); v != "" {
+		c.Assert(containerURL, gc.Equals, v)
+		c.Assert(urlPresent, jc.IsTrue)
+	} else {
+		c.Assert(urlPresent, jc.IsFalse)
+	}
+
+	if v, ok := test.attrs["container-image-stream"]; ok {
+		c.Assert(cfg.ContainerImageStream(), gc.Equals, v)
+	} else {
+		c.Assert(cfg.ContainerImageStream(), gc.Equals, "released")
+	}
+
 	resourceTags, cfgHasResourceTags := cfg.ResourceTags()
 	c.Assert(cfgHasResourceTags, jc.IsTrue)
 	if tags, ok := test.attrs["resource-tags"]; ok {
@@ -730,6 +759,10 @@ func (test configTest) check(c *gc.C, home *gitjujutesting.FakeHome) {
 	if val, ok := test.attrs[config.NetBondReconfigureDelayKey].(int); ok {
 		c.Assert(cfg.NetBondReconfigureDelay(), gc.Equals, val)
 	}
+
+	if val, ok := test.attrs[config.ContainerInheritProperiesKey].(string); ok && val != "" {
+		c.Assert(cfg.ContainerInheritProperies(), gc.Equals, val)
+	}
 }
 
 func (s *ConfigSuite) TestConfigAttrs(c *gc.C) {
@@ -743,7 +776,7 @@ func (s *ConfigSuite) TestConfigAttrs(c *gc.C) {
 		"firewall-mode":              config.FwInstance,
 		"unknown":                    "my-unknown",
 		"ssl-hostname-verification":  true,
-		"default-series":             series.LatestLts(),
+		"default-series":             jujuversion.SupportedLTS(),
 		"disable-network-management": false,
 		"ignore-machine-addresses":   false,
 		"automatically-retry-hooks":  true,
@@ -860,8 +893,12 @@ var configValidateCloudInitUserDataTests = []configValidateCloudInitUserDataTest
 		err:   `cloudinit-userdata: users not allowed`,
 	}, {
 		about: "Invalid cloud init user data values: runcmd",
-		value: invalidCloudInitUserDataRuncmd,
+		value: invalidCloudInitUserDataRunCmd,
 		err:   `cloudinit-userdata: runcmd not allowed, use preruncmd or postruncmd instead`,
+	}, {
+		about: "Invalid cloud init user data values: bootcmd",
+		value: invalidCloudInitUserDataBootCmd,
+		err:   `cloudinit-userdata: bootcmd not allowed`,
 	}, {
 		about: "Invalid cloud init user data: yaml",
 		value: invalidCloudInitUserDataInvalidYAML,
@@ -879,7 +916,7 @@ func (s *ConfigSuite) TestValidateCloudInitUserData(c *gc.C) {
 	}
 	s.FakeHomeSuite.Home.AddFiles(c, files...)
 	for i, test := range configValidateCloudInitUserDataTests {
-		c.Logf("test %d. %s", i, test.about)
+		c.Logf("test %d of %d. %s", i+1, len(configValidateCloudInitUserDataTests), test.about)
 		test.checkNew(c)
 	}
 }
@@ -1202,6 +1239,13 @@ func (s *ConfigSuite) TestCloudInitUserDataFromEnvironment(c *gc.C) {
 	)
 }
 
+func (s *ConfigSuite) TestContainerInheritProperies(c *gc.C) {
+	cfg := newTestConfig(c, testing.Attrs{
+		"container-inherit-properties": "ca-certs,apt-primary",
+	})
+	c.Assert(cfg.ContainerInheritProperies(), gc.Equals, "ca-certs,apt-primary")
+}
+
 func (s *ConfigSuite) TestSchemaNoExtra(c *gc.C) {
 	schema, err := config.Schema(nil)
 	c.Assert(err, gc.IsNil)
@@ -1315,7 +1359,8 @@ MIIBOgIBAAJAZabKgKInuOxj5vDWLwHHQtK3/45KB+32D15w94Nt83BmuGxo90lw
 -----END CERTIFICATE-----
 `[1:]
 
-var validCloudInitUserData = `packages:
+var validCloudInitUserData = `
+packages:
   - 'python-keystoneclient'
   - 'python-glanceclient'
 preruncmd:
@@ -1325,35 +1370,48 @@ postruncmd:
   - mkdir /tmp/postruncmd
   - mkdir /tmp/postruncmd2
 package_upgrade: false
-`
+`[1:]
 
-var invalidCloudInitUserDataPackageInt = `packages:
+var invalidCloudInitUserDataPackageInt = `
+packages:
     - 76
 postruncmd:
     - mkdir /tmp/runcmd
 package_upgrade: true
-`
+`[1:]
 
-var invalidCloudInitUserDataRuncmd = `packages:
+var invalidCloudInitUserDataRunCmd = `
+packages:
     - 'string1'
     - 'string2'
 runcmd:
     - mkdir /tmp/runcmd
 package_upgrade: true
-`
+`[1:]
 
-var invalidCloudInitUserDataUsers = `packages:
+var invalidCloudInitUserDataBootCmd = `
+packages:
+    - 'string1'
+    - 'string2'
+bootcmd:
+    - mkdir /tmp/bootcmd
+package_upgrade: true
+`[1:]
+
+var invalidCloudInitUserDataUsers = `
+packages:
     - 'string1'
     - 'string2'
 users:
     name: test-user
 package_upgrade: true
-`
+`[1:]
 
-var invalidCloudInitUserDataInvalidYAML = `packages:
+var invalidCloudInitUserDataInvalidYAML = `
+packages:
     - 'string1'
      'string2'
 runcmd:
     - mkdir /tmp/runcmd
 package_upgrade: true
-`
+`[1:]
