@@ -3,11 +3,11 @@
 
 package imagemetadata
 
-var CurrentStreamsVersion = currentStreamsVersion
+const CurrentStreamsVersion = currentStreamsVersion
 
 // SetSigningPublicKey sets a new public key for testing and returns the original key.
 func SetSigningPublicKey(key string) string {
-	oldKey := simplestreamsImagesPublicKey
-	simplestreamsImagesPublicKey = key
+	oldKey := SimplestreamsImagesPublicKey
+	SimplestreamsImagesPublicKey = key
 	return oldKey
 }
