@@ -21,7 +21,6 @@ import (
 	"github.com/juju/juju/instance"
 	"github.com/juju/juju/juju/testing"
 	"github.com/juju/juju/jujuclient"
-	"github.com/juju/juju/jujuclient/jujuclienttesting"
 	coretesting "github.com/juju/juju/testing"
 	jujuversion "github.com/juju/juju/version"
 )
@@ -118,7 +117,7 @@ func (t *Tests) SetUpTest(c *gc.C) {
 	c.Assert(err, jc.ErrorIsNil)
 	t.UploadFakeTools(c, stor, "released", "released")
 	t.toolsStorage = stor
-	t.ControllerStore = jujuclienttesting.NewMemStore()
+	t.ControllerStore = jujuclient.NewMemStore()
 	t.ControllerUUID = coretesting.FakeControllerConfig().ControllerUUID()
 }
 
@@ -192,8 +191,8 @@ func (t *Tests) TestBootstrap(c *gc.C) {
 
 	args := bootstrap.BootstrapParams{
 		ControllerConfig: coretesting.FakeControllerConfig(),
-		CloudName:        t.TestConfig["type"].(string),
 		Cloud: cloud.Cloud{
+			Name:      t.TestConfig["type"].(string),
 			Type:      t.TestConfig["type"].(string),
 			AuthTypes: []cloud.AuthType{credential.AuthType()},
 			Regions:   regions,
