@@ -1,9 +1,13 @@
+// Copyright 2013 Canonical Ltd.
+// Licensed under the AGPLv3, see LICENCE file for details.
+
 package simplestreams_test
 
 import (
 	"encoding/json"
 
-	gc "launchpad.net/gocheck"
+	jc "github.com/juju/testing/checkers"
+	gc "gopkg.in/check.v1"
 
 	"github.com/juju/juju/environs/simplestreams"
 )
@@ -22,13 +26,13 @@ func (s *jsonSuite) TestItemCollectionMarshalling(c *gc.C) {
             "c": 123 
         }
     }`), &m)
-	c.Assert(err, gc.IsNil)
+	c.Assert(err, jc.ErrorIsNil)
 	c.Assert(m.Items, gc.DeepEquals, map[string]interface{}{
 		"a": "b",
 		"c": float64(123),
 	})
 	// Ensure marshalling works as expected, too.
 	b, err := json.Marshal(&m)
-	c.Assert(err, gc.IsNil)
+	c.Assert(err, jc.ErrorIsNil)
 	c.Assert(string(b), gc.Equals, `{"items":{"a":"b","c":123}}`)
 }
