@@ -4,13 +4,12 @@
 package simplestreams_test
 
 import (
-	"bytes"
 	"strings"
 	"testing"
 
 	jc "github.com/juju/testing/checkers"
 	"github.com/juju/utils"
-	gc "launchpad.net/gocheck"
+	gc "gopkg.in/check.v1"
 
 	"github.com/juju/juju/environs/simplestreams"
 	sstesting "github.com/juju/juju/environs/simplestreams/testing"
@@ -18,7 +17,6 @@ import (
 
 func Test(t *testing.T) {
 	registerSimpleStreamsTests()
-	gc.Suite(&signingSuite{})
 	gc.Suite(&jsonSuite{})
 	gc.TestingT(t)
 }
@@ -26,7 +24,7 @@ func Test(t *testing.T) {
 func registerSimpleStreamsTests() {
 	gc.Suite(&simplestreamsSuite{
 		LocalLiveSimplestreamsSuite: sstesting.LocalLiveSimplestreamsSuite{
-			Source:         simplestreams.NewURLDataSource("test", "test:", utils.VerifySSLHostnames),
+			Source:         simplestreams.NewURLDataSource("test", "test:", utils.VerifySSLHostnames, simplestreams.DEFAULT_CLOUD_DATA, false),
 			RequireSigned:  false,
 			DataType:       "image-ids",
 			StreamsVersion: "v1",
@@ -59,9 +57,9 @@ func (s *simplestreamsSuite) TearDownSuite(c *gc.C) {
 
 func (s *simplestreamsSuite) TestGetProductsPath(c *gc.C) {
 	indexRef, err := s.GetIndexRef(sstesting.Index_v1)
-	c.Assert(err, gc.IsNil)
+	c.Assert(err, jc.ErrorIsNil)
 	path, err := indexRef.GetProductsPath(s.ValidConstraint)
-	c.Assert(err, gc.IsNil)
+	c.Assert(err, jc.ErrorIsNil)
 	c.Assert(path, gc.Equals, "streams/v1/image_metadata.json")
 }
 
@@ -113,15 +111,15 @@ func (*simplestreamsSuite) TestExtractCatalogsForProductsPreservesOrder(c *gc.C)
 		})
 }
 
-func (*simplestreamsSuite) TestExtractIndexesAcceptsNil(c *gc.C) {
+func (*simplestreamsSuite) TestExtractIndexesAcceptsEmpty(c *gc.C) {
 	ind := simplestreams.Indices{}
-	c.Check(simplestreams.ExtractIndexes(ind), gc.HasLen, 0)
+	c.Check(simplestreams.ExtractIndexes(ind, nil), gc.HasLen, 0)
 }
 
 func (*simplestreamsSuite) TestExtractIndexesReturnsIndex(c *gc.C) {
 	metadata := simplestreams.IndexMetadata{}
 	ind := simplestreams.Indices{Indexes: map[string]*simplestreams.IndexMetadata{"foo": &metadata}}
-	c.Check(simplestreams.ExtractIndexes(ind), gc.DeepEquals, simplestreams.IndexMetadataSlice{&metadata})
+	c.Check(simplestreams.ExtractIndexes(ind, nil), gc.DeepEquals, simplestreams.IndexMetadataSlice{&metadata})
 }
 
 func (*simplestreamsSuite) TestExtractIndexesReturnsAllIndexes(c *gc.C) {
@@ -132,7 +130,7 @@ func (*simplestreamsSuite) TestExtractIndexesReturnsAllIndexes(c *gc.C) {
 		},
 	}
 
-	array := simplestreams.ExtractIndexes(ind)
+	array := simplestreams.ExtractIndexes(ind, nil)
 
 	c.Assert(array, gc.HasLen, len(ind.Indexes))
 	c.Check(array[0], gc.NotNil)
@@ -148,6 +146,20 @@ func (*simplestreamsSuite) TestExtractIndexesReturnsAllIndexes(c *gc.C) {
 		(array[1] == ind.Indexes["bar"]))
 }
 
+func (*simplestreamsSuite) TestExtractIndexesReturnsSpecifiedIndexes(c *gc.C) {
+	ind := simplestreams.Indices{
+		Indexes: map[string]*simplestreams.IndexMetadata{
+			"foo":    {},
+			"bar":    {},
+			"foobar": {},
+		},
+	}
+
+	array := simplestreams.ExtractIndexes(ind, []string{"foobar"})
+	c.Assert(array, gc.HasLen, 1)
+	c.Assert(array[0], gc.Equals, ind.Indexes["foobar"])
+}
+
 func (*simplestreamsSuite) TestHasCloudAcceptsNil(c *gc.C) {
 	metadata := simplestreams.IndexMetadata{Clouds: nil}
 	c.Check(simplestreams.HasCloud(metadata, simplestreams.CloudSpec{}), jc.IsTrue)
@@ -316,11 +328,14 @@ func (s *countingSource) URL(path string) (string, error) {
 func (s *simplestreamsSuite) TestGetMetadataNoMatching(c *gc.C) {
 	source := &countingSource{
 		DataSource: simplestreams.NewURLDataSource(
-			"test", "test:/daily", utils.VerifySSLHostnames,
+			"test",
+			"test:/daily",
+			utils.VerifySSLHostnames,
+			simplestreams.DEFAULT_CLOUD_DATA,
+			false,
 		),
 	}
 	sources := []simplestreams.DataSource{source, source, source}
-	params := simplestreams.ValueParams{DataType: "image-ids"}
 	constraint := sstesting.NewTestConstraint(simplestreams.LookupParams{
 		CloudSpec: simplestreams.CloudSpec{
 			Region:   "us-east-1",
@@ -329,15 +344,14 @@ func (s *simplestreamsSuite) TestGetMetadataNoMatching(c *gc.C) {
 		Series: []string{"precise"},
 		Arches: []string{"not-a-real-arch"}, // never matches
 	})
+	params := simplestreams.GetMetadataParams{
+		StreamsVersion:   s.StreamsVersion,
+		LookupConstraint: constraint,
+		ValueParams:      simplestreams.ValueParams{DataType: "image-ids"},
+	}
 
-	items, resolveInfo, err := simplestreams.GetMetadata(
-		sources,
-		s.StreamsVersion,
-		constraint,
-		false,
-		params,
-	)
-	c.Assert(err, gc.IsNil)
+	items, resolveInfo, err := simplestreams.GetMetadata(sources, params)
+	c.Assert(err, jc.ErrorIsNil)
 	c.Assert(items, gc.HasLen, 0)
 	c.Assert(resolveInfo, gc.DeepEquals, &simplestreams.ResolveInfo{
 		Source:    "test",
@@ -346,14 +360,14 @@ func (s *simplestreamsSuite) TestGetMetadataNoMatching(c *gc.C) {
 		MirrorURL: "",
 	})
 
-	// There should be 2 calls to each data-source:
-	// one for .sjson, one for .json.
-	c.Assert(source.count, gc.Equals, 2*len(sources))
+	// There should be 4 calls to each data-source:
+	// one for .sjson, one for .json, repeated for legacy vs new index files.
+	c.Assert(source.count, gc.Equals, 4*len(sources))
 }
 
 func (s *simplestreamsSuite) TestMetadataCatalog(c *gc.C) {
 	metadata := s.AssertGetMetadata(c)
-	c.Check(len(metadata.Products), gc.Equals, 3)
+	c.Check(len(metadata.Products), gc.Equals, 6)
 	c.Check(len(metadata.Aliases), gc.Equals, 1)
 	metadataCatalog := metadata.Products["com.ubuntu.cloud:server:12.04:amd64"]
 	c.Check(len(metadataCatalog.Items), gc.Equals, 2)
@@ -411,6 +425,43 @@ func (s *simplestreamsSuite) TestDealiasing(c *gc.C) {
 	c.Check(ti.Endpoint, gc.Equals, "https://ec2.us-west-3.amazonaws.com")
 }
 
+type storageVirtTest struct {
+	product, coll, item, storage, virt string
+}
+
+func (s *simplestreamsSuite) TestStorageVirtFromTopLevel(c *gc.C) {
+	s.assertImageMetadata(c,
+		storageVirtTest{"com.ubuntu.cloud:server:13.04:amd64", "20160318", "nzww1pe", "ebs", "pv"},
+	)
+}
+
+func (s *simplestreamsSuite) TestStorageVirtFromCatalog(c *gc.C) {
+	s.assertImageMetadata(c,
+		storageVirtTest{"com.ubuntu.cloud:server:14.10:amd64", "20160218", "nzww1pe", "ebs", "pv"},
+	)
+}
+
+func (s *simplestreamsSuite) TestStorageVirtFromCollection(c *gc.C) {
+	s.assertImageMetadata(c,
+		storageVirtTest{"com.ubuntu.cloud:server:12.10:amd64", "20160118", "nzww1pe", "ebs", "pv"},
+	)
+}
+
+func (s *simplestreamsSuite) TestStorageVirtFromItem(c *gc.C) {
+	s.assertImageMetadata(c,
+		storageVirtTest{"com.ubuntu.cloud:server:14.04:amd64", "20140118", "nzww1pe", "ssd", "hvm"},
+	)
+}
+
+func (s *simplestreamsSuite) assertImageMetadata(c *gc.C, one storageVirtTest) {
+	metadata := s.AssertGetMetadata(c)
+	metadataCatalog := metadata.Products[one.product]
+	ic := metadataCatalog.Items[one.coll]
+	ti := ic.Items[one.item].(*sstesting.TestItem)
+	c.Check(ti.Storage, gc.Equals, one.storage)
+	c.Check(ti.VirtType, gc.Equals, one.virt)
+}
+
 var getMirrorTests = []struct {
 	region    string
 	endpoint  string
@@ -450,108 +501,23 @@ func (s *simplestreamsSuite) TestGetMirrorMetadata(c *gc.C) {
 			MirrorContentId: "com.ubuntu.juju:released:tools",
 		}
 		indexRef, err := simplestreams.GetIndexWithFormat(
-			s.Source, simplestreams.UnsignedIndex("v1"), sstesting.Index_v1,
+			s.Source, s.IndexPath(), sstesting.Index_v1,
 			simplestreams.MirrorsPath("v1"), s.RequireSigned, cloud, params)
-		if !c.Check(err, gc.IsNil) {
+		if !c.Check(err, jc.ErrorIsNil) {
 			continue
 		}
 		if t.err != "" {
 			c.Check(err, gc.ErrorMatches, t.err)
 			continue
 		}
-		if !c.Check(err, gc.IsNil) {
+		if !c.Check(err, jc.ErrorIsNil) {
 			continue
 		}
 		mirrorURL, err := indexRef.Source.URL("")
-		if !c.Check(err, gc.IsNil) {
+		if !c.Check(err, jc.ErrorIsNil) {
 			continue
 		}
 		c.Check(mirrorURL, gc.Equals, t.mirrorURL)
 		c.Check(indexRef.MirroredProductsPath, gc.Equals, t.path)
 	}
 }
-
-var testSigningKey = `-----BEGIN PGP PRIVATE KEY BLOCK-----
-Version: GnuPG v1.4.10 (GNU/Linux)
-
-lQHYBE2rFNoBBADFwqWQIW/DSqcB4yCQqnAFTJ27qS5AnB46ccAdw3u4Greeu3Bp
-idpoHdjULy7zSKlwR1EA873dO/k/e11Ml3dlAFUinWeejWaK2ugFP6JjiieSsrKn
-vWNicdCS4HTWn0X4sjl0ZiAygw6GNhqEQ3cpLeL0g8E9hnYzJKQ0LWJa0QARAQAB
-AAP/TB81EIo2VYNmTq0pK1ZXwUpxCrvAAIG3hwKjEzHcbQznsjNvPUihZ+NZQ6+X
-0HCfPAdPkGDCLCb6NavcSW+iNnLTrdDnSI6+3BbIONqWWdRDYJhqZCkqmG6zqSfL
-IdkJgCw94taUg5BWP/AAeQrhzjChvpMQTVKQL5mnuZbUCeMCAN5qrYMP2S9iKdnk
-VANIFj7656ARKt/nf4CBzxcpHTyB8+d2CtPDKCmlJP6vL8t58Jmih+kHJMvC0dzn
-gr5f5+sCAOOe5gt9e0am7AvQWhdbHVfJU0TQJx+m2OiCJAqGTB1nvtBLHdJnfdC9
-TnXXQ6ZXibqLyBies/xeY2sCKL5qtTMCAKnX9+9d/5yQxRyrQUHt1NYhaXZnJbHx
-q4ytu0eWz+5i68IYUSK69jJ1NWPM0T6SkqpB3KCAIv68VFm9PxqG1KmhSrQIVGVz
-dCBLZXmIuAQTAQIAIgUCTasU2gIbAwYLCQgHAwIGFQgCCQoLBBYCAwECHgECF4AA
-CgkQO9o98PRieSoLhgQAkLEZex02Qt7vGhZzMwuN0R22w3VwyYyjBx+fM3JFETy1
-ut4xcLJoJfIaF5ZS38UplgakHG0FQ+b49i8dMij0aZmDqGxrew1m4kBfjXw9B/v+
-eIqpODryb6cOSwyQFH0lQkXC040pjq9YqDsO5w0WYNXYKDnzRV0p4H1pweo2VDid
-AdgETasU2gEEAN46UPeWRqKHvA99arOxee38fBt2CI08iiWyI8T3J6ivtFGixSqV
-bRcPxYO/qLpVe5l84Nb3X71GfVXlc9hyv7CD6tcowL59hg1E/DC5ydI8K8iEpUmK
-/UnHdIY5h8/kqgGxkY/T/hgp5fRQgW1ZoZxLajVlMRZ8W4tFtT0DeA+JABEBAAEA
-A/0bE1jaaZKj6ndqcw86jd+QtD1SF+Cf21CWRNeLKnUds4FRRvclzTyUMuWPkUeX
-TaNNsUOFqBsf6QQ2oHUBBK4VCHffHCW4ZEX2cd6umz7mpHW6XzN4DECEzOVksXtc
-lUC1j4UB91DC/RNQqwX1IV2QLSwssVotPMPqhOi0ZLNY7wIA3n7DWKInxYZZ4K+6
-rQ+POsz6brEoRHwr8x6XlHenq1Oki855pSa1yXIARoTrSJkBtn5oI+f8AzrnN0BN
-oyeQAwIA/7E++3HDi5aweWrViiul9cd3rcsS0dEnksPhvS0ozCJiHsq/6GFmy7J8
-QSHZPteedBnZyNp5jR+H7cIfVN3KgwH/Skq4PsuPhDq5TKK6i8Pc1WW8MA6DXTdU
-nLkX7RGmMwjC0DBf7KWAlPjFaONAX3a8ndnz//fy1q7u2l9AZwrj1qa1iJ8EGAEC
-AAkFAk2rFNoCGwwACgkQO9o98PRieSo2/QP/WTzr4ioINVsvN1akKuekmEMI3LAp
-BfHwatufxxP1U+3Si/6YIk7kuPB9Hs+pRqCXzbvPRrI8NHZBmc8qIGthishdCYad
-AHcVnXjtxrULkQFGbGvhKURLvS9WnzD/m1K2zzwxzkPTzT9/Yf06O6Mal5AdugPL
-VrM0m72/jnpKo04=
-=zNCn
------END PGP PRIVATE KEY BLOCK-----
-`
-
-var validClearsignInput = `
------BEGIN PGP SIGNED MESSAGE-----
-Hash: SHA1
-
-Hello world
-line 2
-`
-
-var invalidClearsignInput = `
------BEGIN PGP SIGNED MESSAGE-----
-Hash: SHA1
-
-Invalid
-`
-
-var testSig = `-----BEGIN PGP SIGNATURE-----
-Version: GnuPG v1.4.10 (GNU/Linux)
-
-iJwEAQECAAYFAk8kMuEACgkQO9o98PRieSpMsAQAhmY/vwmNpflrPgmfWsYhk5O8
-pjnBUzZwqTDoDeINjZEoPDSpQAHGhjFjgaDx/Gj4fAl0dM4D0wuUEBb6QOrwflog
-2A2k9kfSOMOtk0IH/H5VuFN1Mie9L/erYXjTQIptv9t9J7NoRBMU0QOOaFU0JaO9
-MyTpno24AjIAGb+mH1U=
-=hIJ6
------END PGP SIGNATURE-----
-`
-
-type signingSuite struct{}
-
-func (s *signingSuite) TestDecodeCheckValidSignature(c *gc.C) {
-	r := bytes.NewReader([]byte(validClearsignInput + testSig))
-	txt, err := simplestreams.DecodeCheckSignature(r, testSigningKey)
-	c.Assert(err, gc.IsNil)
-	c.Assert(txt, gc.DeepEquals, []byte("Hello world\nline 2\n"))
-}
-
-func (s *signingSuite) TestDecodeCheckInvalidSignature(c *gc.C) {
-	r := bytes.NewReader([]byte(invalidClearsignInput + testSig))
-	_, err := simplestreams.DecodeCheckSignature(r, testSigningKey)
-	c.Assert(err, gc.Not(gc.IsNil))
-	_, ok := err.(*simplestreams.NotPGPSignedError)
-	c.Assert(ok, jc.IsFalse)
-}
-
-func (s *signingSuite) TestDecodeCheckMissingSignature(c *gc.C) {
-	r := bytes.NewReader([]byte("foo"))
-	_, err := simplestreams.DecodeCheckSignature(r, testSigningKey)
-	_, ok := err.(*simplestreams.NotPGPSignedError)
-	c.Assert(ok, jc.IsTrue)
-}
