@@ -13,10 +13,11 @@ import (
 	"github.com/juju/loggo"
 	jc "github.com/juju/testing/checkers"
 	gc "gopkg.in/check.v1"
-	"gopkg.in/juju/charm.v6-unstable"
-	"gopkg.in/juju/charmrepo.v2-unstable"
+	"gopkg.in/juju/charm.v6"
+	"gopkg.in/juju/charmrepo.v2"
 	"gopkg.in/juju/names.v2"
 
+	"github.com/juju/juju/cmd/juju/application"
 	"github.com/juju/juju/cmd/juju/commands"
 	"github.com/juju/juju/cmd/juju/crossmodel"
 	"github.com/juju/juju/cmd/juju/model"
@@ -34,9 +35,9 @@ type crossmodelSuite struct {
 
 func (s *crossmodelSuite) TestListEndpoints(c *gc.C) {
 	ch := s.AddTestingCharm(c, "riak")
-	s.AddTestingService(c, "riakservice", ch)
+	s.AddTestingApplication(c, "riakservice", ch)
 	ch = s.AddTestingCharm(c, "varnish")
-	s.AddTestingService(c, "varnishservice", ch)
+	s.AddTestingApplication(c, "varnishservice", ch)
 
 	_, err := cmdtesting.RunCommand(c, crossmodel.NewOfferCommand(), "riakservice:endpoint", "riak")
 	c.Assert(err, jc.ErrorIsNil)
@@ -49,29 +50,62 @@ func (s *crossmodelSuite) TestListEndpoints(c *gc.C) {
 	c.Assert(err, jc.ErrorIsNil)
 	c.Assert(ctx.Stdout.(*bytes.Buffer).String(), gc.Equals, `
 riak:
+  application: riakservice
   store: kontroll
-  charm: riak
-  url: admin/controller.riak
+  charm: local:quantal/riak-7
+  offer-url: admin/controller.riak
   endpoints:
     endpoint:
       interface: http
       role: provider
+  users:
+    admin:
+      display-name: admin
+      access: admin
+    everyone@external:
+      access: read
 varnish:
+  application: varnishservice
   store: kontroll
-  charm: varnish
-  url: admin/controller.varnish
+  charm: local:quantal/varnish-1
+  offer-url: admin/controller.varnish
   endpoints:
     webcache:
       interface: varnish
       role: provider
+  users:
+    admin:
+      display-name: admin
+      access: admin
+    everyone@external:
+      access: read
 `[1:])
 }
 
+func (s *crossmodelSuite) TestRemove(c *gc.C) {
+	ch := s.AddTestingCharm(c, "riak")
+	s.AddTestingApplication(c, "riakservice", ch)
+	ch = s.AddTestingCharm(c, "varnish")
+	s.AddTestingApplication(c, "varnishservice", ch)
+
+	_, err := cmdtesting.RunCommand(c, crossmodel.NewOfferCommand(),
+		"riakservice:endpoint", "riak")
+	c.Assert(err, jc.ErrorIsNil)
+
+	_, err = cmdtesting.RunCommand(c, crossmodel.NewRemoveOfferCommand(),
+		"admin/controller.riak")
+	c.Assert(err, jc.ErrorIsNil)
+
+	_, err = cmdtesting.RunCommand(c, crossmodel.NewShowOfferedEndpointCommand(),
+		"admin/controller.riak")
+	c.Assert(err, gc.ErrorMatches, `application offer "admin/controller\.riak" not found`)
+}
+
 func (s *crossmodelSuite) TestShow(c *gc.C) {
 	ch := s.AddTestingCharm(c, "riak")
-	s.AddTestingService(c, "riakservice", ch)
+	s.AddTestingApplication(c, "riakservice", ch)
 	ch = s.AddTestingCharm(c, "varnish")
-	s.AddTestingService(c, "varnishservice", ch)
+	s.AddTestingApplication(c, "varnishservice", ch)
 
 	_, err := cmdtesting.RunCommand(c, crossmodel.NewOfferCommand(),
 		"riakservice:endpoint", "riak")
@@ -85,13 +119,19 @@ func (s *crossmodelSuite) TestShow(c *gc.C) {
 		"admin/controller.varnish", "--format", "yaml")
 	c.Assert(err, jc.ErrorIsNil)
 	c.Assert(ctx.Stdout.(*bytes.Buffer).String(), gc.Equals, `
-admin/controller.varnish:
+kontroll:admin/controller.varnish:
+  description: Another popular database
   access: admin
   endpoints:
     webcache:
       interface: varnish
       role: provider
-  description: Another popular database
+  users:
+    admin:
+      display-name: admin
+      access: admin
+    everyone@external:
+      access: read
 `[1:])
 }
 
@@ -101,21 +141,30 @@ func (s *crossmodelSuite) TestShowOtherModel(c *gc.C) {
 	ctx, err := cmdtesting.RunCommand(c, crossmodel.NewShowOfferedEndpointCommand(),
 		"otheruser/othermodel.hosted-mysql", "--format", "yaml")
 	c.Assert(err, jc.ErrorIsNil)
-	c.Assert(ctx.Stdout.(*bytes.Buffer).String(), gc.Equals, `
-otheruser/othermodel.hosted-mysql:
+	otherUser, err := s.State.User(names.NewUserTag("otheruser"))
+	c.Assert(err, jc.ErrorIsNil)
+	c.Assert(ctx.Stdout.(*bytes.Buffer).String(), gc.Equals, fmt.Sprintf(`
+kontroll:otheruser/othermodel.hosted-mysql:
   access: admin
   endpoints:
     database:
       interface: mysql
       role: provider
-`[1:])
+  users:
+    admin:
+      display-name: admin
+      access: admin
+    otheruser:
+      display-name: %s
+      access: admin
+`, otherUser.DisplayName())[1:])
 }
 
 func (s *crossmodelSuite) setupOffers(c *gc.C) {
 	ch := s.AddTestingCharm(c, "riak")
-	s.AddTestingService(c, "riakservice", ch)
+	s.AddTestingApplication(c, "riakservice", ch)
 	ch = s.AddTestingCharm(c, "varnish")
-	s.AddTestingService(c, "varnishservice", ch)
+	s.AddTestingApplication(c, "varnishservice", ch)
 
 	_, err := cmdtesting.RunCommand(c, crossmodel.NewOfferCommand(),
 		"riakservice:endpoint", "riak")
@@ -130,18 +179,30 @@ func (s *crossmodelSuite) TestFind(c *gc.C) {
 		"admin/controller", "--format", "yaml")
 	c.Assert(err, jc.ErrorIsNil)
 	c.Assert(ctx.Stdout.(*bytes.Buffer).String(), gc.Equals, `
-admin/controller.riak:
+kontroll:admin/controller.riak:
   access: admin
   endpoints:
     endpoint:
       interface: http
       role: provider
-admin/controller.varnish:
+  users:
+    admin:
+      display-name: admin
+      access: admin
+    everyone@external:
+      access: read
+kontroll:admin/controller.varnish:
   access: admin
   endpoints:
     webcache:
       interface: varnish
       role: provider
+  users:
+    admin:
+      display-name: admin
+      access: admin
+    everyone@external:
+      access: read
 `[1:])
 }
 
@@ -151,14 +212,23 @@ func (s *crossmodelSuite) TestFindOtherModel(c *gc.C) {
 	ctx, err := cmdtesting.RunCommand(c, crossmodel.NewFindEndpointsCommand(),
 		"otheruser/othermodel", "--format", "yaml")
 	c.Assert(err, jc.ErrorIsNil)
-	c.Assert(ctx.Stdout.(*bytes.Buffer).String(), gc.Equals, `
-otheruser/othermodel.hosted-mysql:
+	otherUser, err := s.State.User(names.NewUserTag("otheruser"))
+	c.Assert(err, jc.ErrorIsNil)
+	c.Assert(ctx.Stdout.(*bytes.Buffer).String(), gc.Equals, fmt.Sprintf(`
+kontroll:otheruser/othermodel.hosted-mysql:
   access: admin
   endpoints:
     database:
       interface: mysql
       role: provider
-`[1:])
+  users:
+    admin:
+      display-name: admin
+      access: admin
+    otheruser:
+      display-name: %s
+      access: admin
+`, otherUser.DisplayName())[1:])
 }
 
 func (s *crossmodelSuite) TestFindAllModels(c *gc.C) {
@@ -167,40 +237,59 @@ func (s *crossmodelSuite) TestFindAllModels(c *gc.C) {
 
 	ctx, err := cmdtesting.RunCommand(c, crossmodel.NewFindEndpointsCommand(), "kontroll:", "--format", "yaml")
 	c.Assert(err, jc.ErrorIsNil)
-	c.Assert(ctx.Stdout.(*bytes.Buffer).String(), gc.Equals, `
-admin/controller.riak:
+	otherUser, err := s.State.User(names.NewUserTag("otheruser"))
+	c.Assert(err, jc.ErrorIsNil)
+	c.Assert(ctx.Stdout.(*bytes.Buffer).String(), gc.Equals, fmt.Sprintf(`
+kontroll:admin/controller.riak:
   access: admin
   endpoints:
     endpoint:
       interface: http
       role: provider
-admin/controller.varnish:
+  users:
+    admin:
+      display-name: admin
+      access: admin
+    everyone@external:
+      access: read
+kontroll:admin/controller.varnish:
   access: admin
   endpoints:
     webcache:
       interface: varnish
       role: provider
-otheruser/othermodel.hosted-mysql:
+  users:
+    admin:
+      display-name: admin
+      access: admin
+    everyone@external:
+      access: read
+kontroll:otheruser/othermodel.hosted-mysql:
   access: admin
   endpoints:
     database:
       interface: mysql
       role: provider
-`[1:])
+  users:
+    admin:
+      display-name: admin
+      access: admin
+    otheruser:
+      display-name: %s
+      access: admin
+`, otherUser.DisplayName())[1:])
 }
 
 func (s *crossmodelSuite) TestAddRelationFromURL(c *gc.C) {
-	c.Skip("add relation from URL not currently supported")
-
 	ch := s.AddTestingCharm(c, "wordpress")
-	s.AddTestingService(c, "wordpress", ch)
+	s.AddTestingApplication(c, "wordpress", ch)
 	ch = s.AddTestingCharm(c, "mysql")
-	s.AddTestingService(c, "mysql", ch)
+	s.AddTestingApplication(c, "mysql", ch)
 
 	_, err := cmdtesting.RunCommand(c, crossmodel.NewOfferCommand(),
-		"mysql:server", "me/model.hosted-mysql")
+		"mysql:server", "hosted-mysql")
 	c.Assert(err, jc.ErrorIsNil)
-	_, err = runJujuCommand(c, "add-relation", "wordpress", "me/model.hosted-mysql")
+	_, err = runJujuCommand(c, "add-relation", "wordpress", "admin/controller.hosted-mysql")
 	c.Assert(err, jc.ErrorIsNil)
 	svc, err := s.State.RemoteApplication("hosted-mysql")
 	c.Assert(err, jc.ErrorIsNil)
@@ -221,8 +310,7 @@ func (s *crossmodelSuite) TestAddRelationFromURL(c *gc.C) {
 			ApplicationName: "hosted-mysql",
 			Relation: charm.Relation{Name: "server",
 				Role:      "provider",
-				Interface: "mysql",
-				Scope:     "global"},
+				Interface: "mysql"},
 		},
 	})
 }
@@ -247,17 +335,16 @@ func (s *crossmodelSuite) assertAddRelationSameControllerSuccess(c *gc.C, otherM
 			},
 		}, {
 			ApplicationName: "hosted-mysql",
-			Relation: charm.Relation{Name: "server",
+			Relation: charm.Relation{Name: "database",
 				Role:      "provider",
-				Interface: "mysql",
-				Scope:     "global"},
+				Interface: "mysql"},
 		},
 	})
 }
 
 func (s *crossmodelSuite) TestAddRelationSameControllerSameOwner(c *gc.C) {
 	ch := s.AddTestingCharm(c, "wordpress")
-	s.AddTestingService(c, "wordpress", ch)
+	s.AddTestingApplication(c, "wordpress", ch)
 
 	otherModel := s.Factory.MakeModel(c, &factory.ModelParams{Name: "othermodel"})
 	s.AddCleanup(func(*gc.C) { otherModel.Close() })
@@ -356,26 +443,26 @@ func (s *crossmodelSuite) loginAdminUser(c *gc.C) {
 
 func (s *crossmodelSuite) TestAddRelationSameControllerPermissionDenied(c *gc.C) {
 	ch := s.AddTestingCharm(c, "wordpress")
-	s.AddTestingService(c, "wordpress", ch)
+	s.AddTestingApplication(c, "wordpress", ch)
 	s.addOtherModelApplication(c)
 
 	s.createTestUser(c)
 	s.loginTestUser(c)
 	context, err := runJujuCommand(c, "add-relation", "-m", "admin/controller", "wordpress", "otheruser/othermodel.hosted-mysql")
 	c.Assert(err, gc.NotNil)
-	c.Assert(cmdtesting.Stderr(context), jc.Contains, "You do not have permission to add a relation")
+	c.Assert(cmdtesting.Stderr(context), jc.Contains, `application offer "otheruser/othermodel.hosted-mysql" not found`)
 }
 
 func (s *crossmodelSuite) TestAddRelationSameControllerPermissionAllowed(c *gc.C) {
 	ch := s.AddTestingCharm(c, "wordpress")
-	s.AddTestingService(c, "wordpress", ch)
-	//otherModel := s.addOtherModelApplication(c)
+	s.AddTestingApplication(c, "wordpress", ch)
 	s.addOtherModelApplication(c)
 
 	s.createTestUser(c)
 
 	// Users with consume permission to the offer can add relations.
 	runJujuCommand(c, "grant", "test", "consume", "otheruser/othermodel.hosted-mysql")
+	runJujuCommand(c, "grant", "test", "write", "admin/controller")
 
 	s.loginTestUser(c)
 	s.assertAddRelationSameControllerSuccess(c, "otheruser")
@@ -398,29 +485,33 @@ func (s *crossmodelSuite) TestFindOffersWithPermission(c *gc.C) {
 		"otheruser/othermodel", "--format", "yaml")
 	c.Assert(err, jc.ErrorIsNil)
 	c.Assert(ctx.Stdout.(*bytes.Buffer).String(), gc.Equals, `
-otheruser/othermodel.hosted-mysql:
+kontroll:otheruser/othermodel.hosted-mysql:
   access: read
   endpoints:
     database:
       interface: mysql
       role: provider
+  users:
+    test:
+      access: read
 `[1:])
 }
 
 func (s *crossmodelSuite) assertOfferGrant(c *gc.C) {
 	ch := s.AddTestingCharm(c, "riak")
-	s.AddTestingService(c, "riakservice", ch)
+	s.AddTestingApplication(c, "riakservice", ch)
 
 	_, err := cmdtesting.RunCommand(c, crossmodel.NewOfferCommand(), "riakservice:endpoint", "riak")
 	c.Assert(err, jc.ErrorIsNil)
 
 	// Check the default access levels.
-	offerTag := names.NewApplicationOfferTag("riak")
+	offer, err := state.NewApplicationOffers(s.State).ApplicationOffer("riak")
+	c.Assert(err, jc.ErrorIsNil)
 	userTag := names.NewUserTag("everyone@external")
-	access, err := s.State.GetOfferAccess(offerTag, userTag)
+	access, err := s.State.GetOfferAccess(offer.OfferUUID, userTag)
 	c.Assert(err, jc.ErrorIsNil)
 	c.Assert(access, gc.Equals, permission.ReadAccess)
-	access, err = s.State.GetOfferAccess(offerTag, names.NewUserTag("admin"))
+	access, err = s.State.GetOfferAccess(offer.OfferUUID, names.NewUserTag("admin"))
 	c.Assert(err, jc.ErrorIsNil)
 	c.Assert(access, gc.Equals, permission.AdminAccess)
 
@@ -428,7 +519,7 @@ func (s *crossmodelSuite) assertOfferGrant(c *gc.C) {
 	s.Factory.MakeUser(c, &factory.UserParams{Name: "bob"})
 	_, err = cmdtesting.RunCommand(c, model.NewGrantCommand(), "bob", "consume", "admin/controller.riak")
 	c.Assert(err, jc.ErrorIsNil)
-	access, err = s.State.GetOfferAccess(offerTag, names.NewUserTag("bob"))
+	access, err = s.State.GetOfferAccess(offer.OfferUUID, names.NewUserTag("bob"))
 	c.Assert(err, jc.ErrorIsNil)
 	c.Assert(access, gc.Equals, permission.ConsumeAccess)
 
@@ -440,12 +531,54 @@ func (s *crossmodelSuite) TestOfferGrant(c *gc.C) {
 
 func (s *crossmodelSuite) TestOfferRevoke(c *gc.C) {
 	s.assertOfferGrant(c)
-	offerTag := names.NewApplicationOfferTag("riak")
 
 	// Revoke consume access.
 	_, err := cmdtesting.RunCommand(c, model.NewRevokeCommand(), "bob", "consume", "admin/controller.riak")
 	c.Assert(err, jc.ErrorIsNil)
-	access, err := s.State.GetOfferAccess(offerTag, names.NewUserTag("bob"))
+	offer, err := state.NewApplicationOffers(s.State).ApplicationOffer("riak")
+	c.Assert(err, jc.ErrorIsNil)
+	access, err := s.State.GetOfferAccess(offer.OfferUUID, names.NewUserTag("bob"))
 	c.Assert(err, jc.ErrorIsNil)
 	c.Assert(access, gc.Equals, permission.ReadAccess)
 }
+
+func (s *crossmodelSuite) TestConsumeWithPermission(c *gc.C) {
+	s.addOtherModelApplication(c)
+	s.createTestUser(c)
+	s.loginTestUser(c)
+	ctx, err := cmdtesting.RunCommand(c, application.NewConsumeCommand(),
+		"otheruser/othermodel.hosted-mysql")
+	c.Assert(err, gc.ErrorMatches, `application offer "otheruser/othermodel.hosted-mysql" not found`)
+
+	s.loginAdminUser(c)
+	_, err = cmdtesting.RunCommand(c, model.NewGrantCommand(), "test", "consume", "otheruser/othermodel.hosted-mysql")
+	c.Assert(err, jc.ErrorIsNil)
+	_, err = cmdtesting.RunCommand(c, model.NewGrantCommand(), "test", "write", "admin/controller")
+	c.Assert(err, jc.ErrorIsNil)
+
+	s.loginTestUser(c)
+	ctx, err = cmdtesting.RunCommand(c, application.NewConsumeCommand(),
+		"-m", "admin/controller", "otheruser/othermodel.hosted-mysql", "othermysql")
+	c.Assert(err, jc.ErrorIsNil)
+	c.Assert(ctx.Stderr.(*bytes.Buffer).String(), gc.Equals, `
+Added otheruser/othermodel.hosted-mysql as othermysql
+`[1:])
+}
+
+func (s *crossmodelSuite) TestRemoveSaas(c *gc.C) {
+	s.addOtherModelApplication(c)
+	_, err := cmdtesting.RunCommand(c, application.NewConsumeCommand(),
+		"otheruser/othermodel.hosted-mysql")
+
+	_, err = cmdtesting.RunCommand(c, application.NewRemoveSaasCommand(),
+		"-m", "admin/controller", "hosted-mysql")
+	c.Assert(err, jc.ErrorIsNil)
+
+	// A second time it's no longer there.
+	ctx, err := cmdtesting.RunCommand(c, application.NewRemoveSaasCommand(),
+		"-m", "admin/controller", "hosted-mysql")
+	c.Check(err, gc.ErrorMatches, "cmd: error out silently")
+	c.Assert(ctx.Stderr.(*bytes.Buffer).String(), gc.Equals, `
+removing SAAS application hosted-mysql failed: remote application "hosted-mysql" not found
+`[1:])
+}
