@@ -55,12 +55,52 @@ func (s *UserSuite) TestUserChangePassword(c *gc.C) {
 	c.Assert(user.PasswordValid("dummy-secret"), jc.IsTrue)
 	_, err = s.RunUserCommand(c, "not-dummy-secret\nnot-dummy-secret\n", "change-user-password")
 	c.Assert(err, jc.ErrorIsNil)
-	user.Refresh()
+	err = user.Refresh()
 	c.Assert(err, jc.ErrorIsNil)
 	c.Assert(user.PasswordValid("dummy-secret"), jc.IsFalse)
 	c.Assert(user.PasswordValid("not-dummy-secret"), jc.IsTrue)
 }
 
+func (s *UserSuite) TestUserResetPasswordForSelf(c *gc.C) {
+	user, err := s.State.User(s.AdminUserTag(c))
+	c.Assert(err, jc.ErrorIsNil)
+	c.Assert(user.PasswordValid("dummy-secret"), jc.IsTrue)
+
+	// Should not be able to reset own password
+	context, err := s.RunUserCommand(c, "", "change-user-password", "--reset")
+	c.Assert(err, jc.ErrorIsNil)
+	c.Assert(cmdtesting.Stdout(context), gc.Equals, "")
+	c.Assert(cmdtesting.Stderr(context), gc.Matches, `
+You cannot reset your own password.
+If you want to change it, please call `[1:]+"`juju change-user-password`"+` without --reset option.
+`)
+	err = user.Refresh()
+	c.Assert(err, jc.ErrorIsNil)
+	c.Assert(user.PasswordValid("dummy-secret"), jc.IsTrue)
+}
+
+func (s *UserSuite) TestUserResetPasswordForOther(c *gc.C) {
+	username := "bob"
+	context := run(c, nil, "add-user", username, "Bob Dobbs")
+	c.Check(cmdtesting.Stderr(context), gc.Equals, "")
+
+	context, err := s.RunUserCommand(c, "", "change-user-password", "--reset", username)
+	c.Assert(err, jc.ErrorIsNil)
+	c.Assert(cmdtesting.Stdout(context), gc.Equals, "")
+	c.Assert(cmdtesting.Stderr(context), gc.Matches, `
+Password for "bob" has been reset.
+Ask the user to run:
+     juju register (.+)
+`[1:])
+}
+
+func (s *UserSuite) TestUserResetPasswordFail(c *gc.C) {
+	context, err := s.RunUserCommand(c, "", "change-user-password", "--reset", "bob")
+	c.Assert(err, gc.ErrorMatches, "cmd: error out silently")
+	c.Assert(cmdtesting.Stdout(context), gc.Equals, "")
+	c.Assert(cmdtesting.Stderr(context), gc.Matches, "ERROR permission denied\n")
+}
+
 func (s *UserSuite) TestUserInfo(c *gc.C) {
 	user, err := s.State.User(s.AdminUserTag(c))
 	c.Assert(err, jc.ErrorIsNil)
