@@ -114,6 +114,7 @@ func (s *upgradeSuite) TestLoginsDuringUpgrade(c *gc.C) {
 	// when upgrades have started and can control when upgrades
 	// should finish.
 	upgradeCh := make(chan bool)
+	upgradeChClosed := false
 	abort := make(chan bool)
 	fakePerformUpgrade := func(version.Number, []upgrades.Target, upgrades.Context) error {
 		// Signal that upgrade has started.
@@ -139,12 +140,21 @@ func (s *upgradeSuite) TestLoginsDuringUpgrade(c *gc.C) {
 
 	c.Assert(waitForUpgradeToStart(upgradeCh), jc.IsTrue)
 
+	// The test will hang if there's a failure in the assertions below
+	// and upgradeCh isn't closed.
+	defer func() {
+		if !upgradeChClosed {
+			close(upgradeCh)
+		}
+	}()
+
 	// Only user and local logins are allowed during upgrade. Users get a restricted API.
 	s.checkLoginToAPIAsUser(c, machine0Conf, RestrictedAPIExposed)
 	c.Assert(canLoginToAPIAsMachine(c, machine0Conf, machine0Conf), jc.IsTrue)
 	c.Assert(canLoginToAPIAsMachine(c, machine1Conf, machine0Conf), jc.IsFalse)
 
 	close(upgradeCh) // Allow upgrade to complete
+	upgradeChClosed = true
 
 	waitForUpgradeToFinish(c, machine0Conf)
 
@@ -170,7 +180,7 @@ func (s *upgradeSuite) TestDowngradeOnMasterWhenOtherControllerDoesntStartUpgrad
 
 	// Create 3 controllers
 	machineA, _ := s.makeStateAgentVersion(c, s.oldVersion)
-	changes, err := s.State.EnableHA(3, constraints.Value{}, "quantal", nil)
+	changes, err := s.State.EnableHA(3, constraints.Value{}, "quantal", nil, "")
 	c.Assert(err, jc.ErrorIsNil)
 	c.Assert(len(changes.Added), gc.Equals, 2)
 	machineB, _, _ := s.configureMachine(c, changes.Added[0], s.oldVersion)
