@@ -21,25 +21,33 @@ type InstanceSuite struct{}
 var _ = gc.Suite(&InstanceSuite{})
 
 func (s *InstanceSuite) TestParseContainerType(c *gc.C) {
-	ctype, err := instance.ParseContainerType("lxc")
+	ctype, err := instance.ParseContainerType("lxd")
 	c.Assert(err, jc.ErrorIsNil)
-	c.Assert(ctype, gc.Equals, instance.LXC)
+	c.Assert(ctype, gc.Equals, instance.LXD)
+
+	ctype, err = instance.ParseContainerType("lxd")
+	c.Assert(err, jc.ErrorIsNil)
+	c.Assert(ctype, gc.Equals, instance.LXD)
 
 	ctype, err = instance.ParseContainerType("kvm")
 	c.Assert(err, jc.ErrorIsNil)
 	c.Assert(ctype, gc.Equals, instance.KVM)
 
-	ctype, err = instance.ParseContainerType("none")
+	_, err = instance.ParseContainerType("none")
 	c.Assert(err, gc.ErrorMatches, `invalid container type "none"`)
 
-	ctype, err = instance.ParseContainerType("omg")
+	_, err = instance.ParseContainerType("omg")
 	c.Assert(err, gc.ErrorMatches, `invalid container type "omg"`)
 }
 
 func (s *InstanceSuite) TestParseContainerTypeOrNone(c *gc.C) {
-	ctype, err := instance.ParseContainerTypeOrNone("lxc")
+	ctype, err := instance.ParseContainerTypeOrNone("lxd")
 	c.Assert(err, jc.ErrorIsNil)
-	c.Assert(ctype, gc.Equals, instance.LXC)
+	c.Assert(ctype, gc.Equals, instance.LXD)
+
+	ctype, err = instance.ParseContainerTypeOrNone("lxd")
+	c.Assert(err, jc.ErrorIsNil)
+	c.Assert(ctype, gc.Equals, instance.LXD)
 
 	ctype, err = instance.ParseContainerTypeOrNone("kvm")
 	c.Assert(err, jc.ErrorIsNil)
@@ -49,6 +57,6 @@ func (s *InstanceSuite) TestParseContainerTypeOrNone(c *gc.C) {
 	c.Assert(err, jc.ErrorIsNil)
 	c.Assert(ctype, gc.Equals, instance.NONE)
 
-	ctype, err = instance.ParseContainerTypeOrNone("omg")
+	_, err = instance.ParseContainerTypeOrNone("omg")
 	c.Assert(err, gc.ErrorMatches, `invalid container type "omg"`)
 }
