@@ -240,9 +240,6 @@ type UpdateControllerParams struct {
 	// PublicDNSName (when set) holds the public host name of the controller.
 	PublicDNSName *string
 
-	// ModelCount (when set) is the number of models visible to the user.
-	ModelCount *int
-
 	// ControllerMachineCount (when set) is the total number of controller machines in the environment.
 	ControllerMachineCount *int
 
@@ -288,9 +285,6 @@ func updateControllerDetailsFromLogin(
 	newDetails.AgentVersion = params.AgentVersion
 	newDetails.APIEndpoints = hostPorts
 	newDetails.DNSCache = params.DNSCache
-	if params.ModelCount != nil {
-		newDetails.ModelCount = params.ModelCount
-	}
 	if params.MachineCount != nil {
 		newDetails.MachineCount = params.MachineCount
 	}
