@@ -203,15 +203,6 @@ func (s *ControllersSuite) TestSetCurrentControllerNoneExists(c *gc.C) {
 	c.Assert(err, gc.ErrorMatches, "controller test.controller not found")
 }
 
-func (s *ControllersSuite) assertWriteFails(c *gc.C, failureMessage string) {
-	err := s.store.UpdateController(s.controllerName, s.controller)
-	c.Assert(err, gc.ErrorMatches, failureMessage)
-
-	found, err := s.store.ControllerByName(s.controllerName)
-	c.Assert(err, gc.ErrorMatches, fmt.Sprintf("controller %v not found", s.controllerName))
-	c.Assert(found, gc.IsNil)
-}
-
 func (s *ControllersSuite) assertControllerNotExists(c *gc.C) {
 	all := writeTestControllersFile(c)
 	_, exists := all.Controllers[s.controllerName]
