@@ -329,10 +329,7 @@ func (s *store) UpdateModel(controllerName, modelName string, details ModelDetai
 	if err := ValidateControllerName(controllerName); err != nil {
 		return errors.Trace(err)
 	}
-	if err := ValidateModelName(modelName); err != nil {
-		return errors.Trace(err)
-	}
-	if err := ValidateModelDetails(details); err != nil {
+	if err := ValidateModel(modelName, details); err != nil {
 		return errors.Trace(err)
 	}
 
@@ -469,8 +466,9 @@ func (s *store) ModelByName(controllerName, modelName string) (*ModelDetails, er
 	controllerModels, ok := all[controllerName]
 	if !ok {
 		return nil, errors.NotFoundf(
-			"models for controller %s",
+			"model %s:%s",
 			controllerName,
+			modelName,
 		)
 	}
 	details, ok := controllerModels.Models[modelName]
@@ -518,10 +516,9 @@ func (s *store) RemoveModel(controllerName, modelName string) error {
 	))
 }
 
-func updateModels(
-	controllerName string,
-	update func(*ControllerModels) (bool, error),
-) error {
+type updateModelFunc func(storedModels *ControllerModels) (bool, error)
+
+func updateModels(controllerName string, update updateModelFunc) error {
 	all, err := ReadModelsFile(JujuModelsPath())
 	if err != nil {
 		return errors.Trace(err)
@@ -547,6 +544,52 @@ func updateModels(
 	return nil
 }
 
+// SetModels implements ModelUpdater.
+func (s *store) SetModels(controllerName string, models map[string]ModelDetails) error {
+	if err := ValidateControllerName(controllerName); err != nil {
+		return errors.Trace(err)
+	}
+
+	for modelName, details := range models {
+		if err := ValidateModel(modelName, details); err != nil {
+			return errors.Trace(err)
+		}
+	}
+
+	releaser, err := s.acquireLock()
+	if err != nil {
+		return errors.Trace(err)
+	}
+	defer releaser.Release()
+
+	err = updateModels(controllerName, func(storedModels *ControllerModels) (bool, error) {
+		changed := len(storedModels.Models) != len(models)
+		// Add or update controller models based on a new collection.
+		for modelName, details := range models {
+			oldDetails, ok := storedModels.Models[modelName]
+			if ok && details == oldDetails {
+				continue
+			}
+			storedModels.Models[modelName] = details
+			changed = true
+		}
+		// Delete models that are not in the new collection.
+		for modelName, _ := range storedModels.Models {
+			if _, ok := models[modelName]; !ok {
+				delete(storedModels.Models, modelName)
+				if storedModels.CurrentModel == modelName {
+					storedModels.CurrentModel = ""
+				}
+			}
+		}
+		return changed, nil
+	})
+	if err != nil {
+		return errors.Trace(err)
+	}
+	return nil
+}
+
 // UpdateAccount implements AccountUpdater.
 func (s *store) UpdateAccount(controllerName string, details AccountDetails) error {
 	if err := ValidateControllerName(controllerName); err != nil {
@@ -727,12 +770,6 @@ func (s *store) BootstrapConfigForController(controllerName string) (*BootstrapC
 	if !ok {
 		return nil, errors.NotFoundf("bootstrap config for controller %s", controllerName)
 	}
-	if cfg.CloudType == "" {
-		// TODO(axw) 2016-07-25 #1603841
-		// Drop this when we get to 2.0. This exists only for
-		// compatibility with previous beta releases.
-		cfg.CloudType, _ = cfg.Config["type"].(string)
-	}
 	return &cfg, nil
 }
 
