@@ -58,11 +58,6 @@ type ControllerDetails struct {
 	// does not need to hit the server.
 	ActiveControllerMachineCount int `yaml:"active-controller-machine-count"`
 
-	// ModelCount is the number of models to which a user has access.
-	// It is cached here so under normal usage list-controllers
-	// does not need to hit the server.
-	ModelCount *int `yaml:"model-count,omitempty"`
-
 	// MachineCount is the number of machines in all models to
 	// which a user has access. It is cached here so under normal
 	// usage list-controllers does not need to hit the server.
@@ -191,6 +186,11 @@ type ModelUpdater interface {
 	// Otherwise, it will be overwritten with the new details.
 	UpdateModel(controllerName, modelName string, details ModelDetails) error
 
+	// SetModels updates the list of currently stored controller
+	// models in model store - models will be added, updated or removed from the
+	// store based on the supplied models collection.
+	SetModels(controllerName string, models map[string]ModelDetails) error
+
 	// SetCurrentModel sets the name of the current model for
 	// the specified controller and account. If there exists no
 	// model with the specified names, an error satisfying
