@@ -8,7 +8,7 @@ import (
 	jc "github.com/juju/testing/checkers"
 	"github.com/juju/version"
 	gc "gopkg.in/check.v1"
-	"gopkg.in/juju/charm.v6-unstable"
+	"gopkg.in/juju/charm.v6"
 	"gopkg.in/juju/names.v2"
 
 	"github.com/juju/juju/cloud"
@@ -199,7 +199,7 @@ func (s *SourcePrecheckSuite) TestUnitVersionsDontMatch(c *gc.C) {
 		},
 	}
 	err := migration.SourcePrecheck(backend)
-	c.Assert(err.Error(), gc.Equals, "unit bar/1 tools don't match model (1.2.4 != 1.2.3)")
+	c.Assert(err.Error(), gc.Equals, "unit bar/1 agent binaries don't match model (1.2.4 != 1.2.3)")
 }
 
 func (s *SourcePrecheckSuite) TestDeadUnit(c *gc.C) {
@@ -281,7 +281,7 @@ func (s *SourcePrecheckSuite) TestControllerMachineVersionsDontMatch(c *gc.C) {
 	backend := newFakeBackend()
 	backend.controllerBackend = newBackendWithMismatchingTools()
 	err := migration.SourcePrecheck(backend)
-	c.Assert(err, gc.ErrorMatches, "controller: machine . tools don't match model.+")
+	c.Assert(err, gc.ErrorMatches, "controller: machine . agent binaries don't match model.+")
 }
 
 func (s *SourcePrecheckSuite) TestControllerMachineRequiresReboot(c *gc.C) {
@@ -315,6 +315,76 @@ func (s *SourcePrecheckSuite) TestProvisioningControllerMachine(c *gc.C) {
 	c.Assert(err.Error(), gc.Equals, "controller: machine 0 not running (allocating)")
 }
 
+func (s *SourcePrecheckSuite) TestUnitsAllInScope(c *gc.C) {
+	backend := newHappyBackend()
+	backend.relations = []migration.PrecheckRelation{&fakeRelation{
+		endpoints: []state.Endpoint{
+			{ApplicationName: "foo"},
+			{ApplicationName: "bar"},
+		},
+		relUnits: map[string]*fakeRelationUnit{
+			"foo/0": {valid: true, inScope: true},
+			"bar/0": {valid: true, inScope: true},
+			"bar/1": {valid: true, inScope: true},
+		},
+	}}
+	err := migration.SourcePrecheck(backend)
+	c.Assert(err, jc.ErrorIsNil)
+}
+
+func (s *SourcePrecheckSuite) TestSubordinatesNotYetInScope(c *gc.C) {
+	backend := newHappyBackend()
+	backend.relations = []migration.PrecheckRelation{&fakeRelation{
+		key: "foo:db bar:db",
+		endpoints: []state.Endpoint{
+			{ApplicationName: "foo"},
+			{ApplicationName: "bar"},
+		},
+		relUnits: map[string]*fakeRelationUnit{
+			"foo/0": {valid: true, inScope: true},
+			"bar/0": {valid: true, inScope: true},
+			"bar/1": {valid: true, inScope: false},
+		},
+	}}
+	err := migration.SourcePrecheck(backend)
+	c.Assert(err, gc.ErrorMatches, "unit bar/1 hasn't joined relation foo:db bar:db yet")
+}
+
+func (s *SourcePrecheckSuite) TestSubordinatesInvalidUnitsNotYetInScope(c *gc.C) {
+	backend := newHappyBackend()
+	backend.relations = []migration.PrecheckRelation{&fakeRelation{
+		key: "foo:db bar:db",
+		endpoints: []state.Endpoint{
+			{ApplicationName: "foo"},
+			{ApplicationName: "bar"},
+		},
+		relUnits: map[string]*fakeRelationUnit{
+			"foo/0": {valid: true, inScope: true},
+			"bar/0": {valid: true, inScope: true},
+			"bar/1": {valid: false, inScope: false},
+		},
+	}}
+	err := migration.SourcePrecheck(backend)
+	c.Assert(err, jc.ErrorIsNil)
+}
+
+func (s *SourcePrecheckSuite) TestCrossModelUnitsNotYetInScope(c *gc.C) {
+	backend := newHappyBackend()
+	backend.relations = []migration.PrecheckRelation{&fakeRelation{
+		key:        "foo:db bar:db",
+		crossModel: true,
+		endpoints: []state.Endpoint{
+			{ApplicationName: "foo"},
+			{ApplicationName: "remote-mysql"},
+		},
+		relUnits: map[string]*fakeRelationUnit{
+			"foo/0": {valid: true, inScope: false},
+		},
+	}}
+	err := migration.SourcePrecheck(backend)
+	c.Assert(err, jc.ErrorIsNil)
+}
+
 type TargetPrecheckSuite struct {
 	precheckBaseSuite
 	modelInfo coremigration.ModelInfo
@@ -332,11 +402,11 @@ func (s *TargetPrecheckSuite) SetUpTest(c *gc.C) {
 }
 
 func (s *TargetPrecheckSuite) runPrecheck(backend migration.PrecheckBackend) error {
-	return migration.TargetPrecheck(backend, s.modelInfo)
+	return migration.TargetPrecheck(backend, nil, s.modelInfo)
 }
 
 func (s *TargetPrecheckSuite) TestSuccess(c *gc.C) {
-	err := migration.TargetPrecheck(newHappyBackend(), s.modelInfo)
+	err := migration.TargetPrecheck(newHappyBackend(), nil, s.modelInfo)
 	c.Assert(err, jc.ErrorIsNil)
 }
 
@@ -347,7 +417,7 @@ func (s *TargetPrecheckSuite) TestModelVersionAheadOfTarget(c *gc.C) {
 	sourceVersion.Patch++
 	s.modelInfo.AgentVersion = sourceVersion
 
-	err := migration.TargetPrecheck(backend, s.modelInfo)
+	err := migration.TargetPrecheck(backend, nil, s.modelInfo)
 	c.Assert(err.Error(), gc.Equals,
 		`model has higher version than target controller (1.2.4 > 1.2.3)`)
 }
@@ -361,7 +431,7 @@ func (s *TargetPrecheckSuite) TestSourceControllerMajorAhead(c *gc.C) {
 	sourceVersion.Patch = 0
 	s.modelInfo.ControllerAgentVersion = sourceVersion
 
-	err := migration.TargetPrecheck(backend, s.modelInfo)
+	err := migration.TargetPrecheck(backend, nil, s.modelInfo)
 	c.Assert(err.Error(), gc.Equals,
 		`source controller has higher version than target controller (2.0.0 > 1.2.3)`)
 }
@@ -374,7 +444,7 @@ func (s *TargetPrecheckSuite) TestSourceControllerMinorAhead(c *gc.C) {
 	sourceVersion.Patch = 0
 	s.modelInfo.ControllerAgentVersion = sourceVersion
 
-	err := migration.TargetPrecheck(backend, s.modelInfo)
+	err := migration.TargetPrecheck(backend, nil, s.modelInfo)
 	c.Assert(err.Error(), gc.Equals,
 		`source controller has higher version than target controller (1.3.0 > 1.2.3)`)
 }
@@ -386,7 +456,7 @@ func (s *TargetPrecheckSuite) TestSourceControllerPatchAhead(c *gc.C) {
 	sourceVersion.Patch++
 	s.modelInfo.ControllerAgentVersion = sourceVersion
 
-	c.Assert(migration.TargetPrecheck(backend, s.modelInfo), jc.ErrorIsNil)
+	c.Assert(migration.TargetPrecheck(backend, nil, s.modelInfo), jc.ErrorIsNil)
 }
 
 func (s *TargetPrecheckSuite) TestSourceControllerBuildAhead(c *gc.C) {
@@ -396,7 +466,7 @@ func (s *TargetPrecheckSuite) TestSourceControllerBuildAhead(c *gc.C) {
 	sourceVersion.Build++
 	s.modelInfo.ControllerAgentVersion = sourceVersion
 
-	c.Assert(migration.TargetPrecheck(backend, s.modelInfo), jc.ErrorIsNil)
+	c.Assert(migration.TargetPrecheck(backend, nil, s.modelInfo), jc.ErrorIsNil)
 }
 
 func (s *TargetPrecheckSuite) TestSourceControllerTagMismatch(c *gc.C) {
@@ -406,13 +476,13 @@ func (s *TargetPrecheckSuite) TestSourceControllerTagMismatch(c *gc.C) {
 	sourceVersion.Tag = "alpha"
 	s.modelInfo.ControllerAgentVersion = sourceVersion
 
-	c.Assert(migration.TargetPrecheck(backend, s.modelInfo), jc.ErrorIsNil)
+	c.Assert(migration.TargetPrecheck(backend, nil, s.modelInfo), jc.ErrorIsNil)
 }
 
 func (s *TargetPrecheckSuite) TestDying(c *gc.C) {
 	backend := newFakeBackend()
 	backend.model.life = state.Dying
-	err := migration.TargetPrecheck(backend, s.modelInfo)
+	err := migration.TargetPrecheck(backend, nil, s.modelInfo)
 	c.Assert(err, gc.ErrorMatches, "model is dying")
 }
 
@@ -428,7 +498,7 @@ func (s *TargetPrecheckSuite) TestIsUpgradingError(c *gc.C) {
 	backend := &fakeBackend{
 		isUpgradingErr: errors.New("boom"),
 	}
-	err := migration.TargetPrecheck(backend, s.modelInfo)
+	err := migration.TargetPrecheck(backend, nil, s.modelInfo)
 	c.Assert(err, gc.ErrorMatches, "checking for upgrades: boom")
 }
 
@@ -436,19 +506,19 @@ func (s *TargetPrecheckSuite) TestIsUpgrading(c *gc.C) {
 	backend := &fakeBackend{
 		isUpgrading: true,
 	}
-	err := migration.TargetPrecheck(backend, s.modelInfo)
+	err := migration.TargetPrecheck(backend, nil, s.modelInfo)
 	c.Assert(err, gc.ErrorMatches, "upgrade in progress")
 }
 
 func (s *TargetPrecheckSuite) TestIsMigrationActiveError(c *gc.C) {
 	backend := &fakeBackend{migrationActiveErr: errors.New("boom")}
-	err := migration.TargetPrecheck(backend, s.modelInfo)
+	err := migration.TargetPrecheck(backend, nil, s.modelInfo)
 	c.Assert(err, gc.ErrorMatches, "checking for active migration: boom")
 }
 
 func (s *TargetPrecheckSuite) TestIsMigrationActive(c *gc.C) {
 	backend := &fakeBackend{migrationActive: true}
-	err := migration.TargetPrecheck(backend, s.modelInfo)
+	err := migration.TargetPrecheck(backend, nil, s.modelInfo)
 	c.Assert(err, gc.ErrorMatches, "model is being migrated out of target controller")
 }
 
@@ -458,67 +528,80 @@ func (s *TargetPrecheckSuite) TestMachineVersionsDontMatch(c *gc.C) {
 
 func (s *TargetPrecheckSuite) TestDyingMachine(c *gc.C) {
 	backend := newBackendWithDyingMachine()
-	err := migration.TargetPrecheck(backend, s.modelInfo)
+	err := migration.TargetPrecheck(backend, nil, s.modelInfo)
 	c.Assert(err, gc.ErrorMatches, "machine 0 is dying")
 }
 
 func (s *TargetPrecheckSuite) TestNonStartedMachine(c *gc.C) {
 	backend := newBackendWithDownMachine()
-	err := migration.TargetPrecheck(backend, s.modelInfo)
+	err := migration.TargetPrecheck(backend, nil, s.modelInfo)
 	c.Assert(err.Error(), gc.Equals, "machine 0 agent not functioning at this time (down)")
 }
 
 func (s *TargetPrecheckSuite) TestProvisioningMachine(c *gc.C) {
 	backend := newBackendWithProvisioningMachine()
-	err := migration.TargetPrecheck(backend, s.modelInfo)
+	err := migration.TargetPrecheck(backend, nil, s.modelInfo)
 	c.Assert(err.Error(), gc.Equals, "machine 0 not running (allocating)")
 }
 
 func (s *TargetPrecheckSuite) TestDownMachineAgent(c *gc.C) {
 	backend := newBackendWithDownMachineAgent()
-	err := migration.TargetPrecheck(backend, s.modelInfo)
+	err := migration.TargetPrecheck(backend, nil, s.modelInfo)
 	c.Assert(err.Error(), gc.Equals, "machine 1 agent not functioning at this time (down)")
 }
 
 func (s *TargetPrecheckSuite) TestModelNameAlreadyInUse(c *gc.C) {
-	backend := newFakeBackend()
-	backend.models = []migration.PrecheckModel{
-		&fakeModel{
-			name:  modelName,
-			owner: modelOwner,
+	pool := &fakePool{
+		models: []migration.PrecheckModel{
+			&fakeModel{
+				uuid:  "uuid",
+				name:  modelName,
+				owner: modelOwner,
+			},
 		},
 	}
-	err := migration.TargetPrecheck(backend, s.modelInfo)
+	backend := newFakeBackend()
+	backend.models = pool.uuids()
+	err := migration.TargetPrecheck(backend, pool, s.modelInfo)
 	c.Assert(err, gc.ErrorMatches, "model named \"model-name\" already exists")
 }
 
 func (s *TargetPrecheckSuite) TestModelNameOverlapOkForDifferentOwner(c *gc.C) {
-	backend := newFakeBackend()
-	backend.models = []migration.PrecheckModel{
-		&fakeModel{name: modelName, owner: names.NewUserTag("someone.else")},
+	pool := &fakePool{
+		models: []migration.PrecheckModel{
+			&fakeModel{name: modelName, owner: names.NewUserTag("someone.else")},
+		},
 	}
-	err := migration.TargetPrecheck(backend, s.modelInfo)
+	backend := newFakeBackend()
+	backend.models = pool.uuids()
+	err := migration.TargetPrecheck(backend, pool, s.modelInfo)
 	c.Assert(err, jc.ErrorIsNil)
 }
 
 func (s *TargetPrecheckSuite) TestUUIDAlreadyExists(c *gc.C) {
-	backend := newFakeBackend()
-	backend.models = []migration.PrecheckModel{
-		&fakeModel{uuid: modelUUID},
+	pool := &fakePool{
+		models: []migration.PrecheckModel{
+			&fakeModel{uuid: modelUUID},
+		},
 	}
-	err := migration.TargetPrecheck(backend, s.modelInfo)
+	backend := newFakeBackend()
+	backend.models = pool.uuids()
+	err := migration.TargetPrecheck(backend, pool, s.modelInfo)
 	c.Assert(err.Error(), gc.Equals, "model with same UUID already exists (model-uuid)")
 }
 
 func (s *TargetPrecheckSuite) TestUUIDAlreadyExistsButImporting(c *gc.C) {
-	backend := newFakeBackend()
-	backend.models = []migration.PrecheckModel{
-		&fakeModel{
-			uuid:          modelUUID,
-			migrationMode: state.MigrationModeImporting,
+	pool := &fakePool{
+		models: []migration.PrecheckModel{
+			&fakeModel{
+				uuid:          modelUUID,
+				migrationMode: state.MigrationModeImporting,
+			},
 		},
 	}
-	err := migration.TargetPrecheck(backend, s.modelInfo)
+	backend := newFakeBackend()
+	backend.models = pool.uuids()
+	err := migration.TargetPrecheck(backend, pool, s.modelInfo)
 	c.Assert(err, jc.ErrorIsNil)
 }
 
@@ -543,7 +626,7 @@ func (*precheckBaseSuite) checkAgentVersionError(c *gc.C, runPrecheck precheckRu
 
 func (*precheckBaseSuite) checkMachineVersionsDontMatch(c *gc.C, runPrecheck precheckRunner) {
 	err := runPrecheck(newBackendWithMismatchingTools())
-	c.Assert(err.Error(), gc.Equals, "machine 1 tools don't match model (1.3.1 != 1.2.3)")
+	c.Assert(err.Error(), gc.Equals, "machine 1 agent binaries don't match model (1.3.1 != 1.2.3)")
 }
 
 func newHappyBackend() *fakeBackend {
@@ -631,7 +714,7 @@ type fakeBackend struct {
 	agentVersionErr error
 
 	model  fakeModel
-	models []migration.PrecheckModel
+	models []string
 
 	cleanupNeeded bool
 	cleanupErr    error
@@ -648,6 +731,9 @@ type fakeBackend struct {
 	apps       []migration.PrecheckApplication
 	allAppsErr error
 
+	relations  []migration.PrecheckRelation
+	allRelsErr error
+
 	credentials    cloud.Credential
 	credentialsErr error
 
@@ -665,7 +751,7 @@ func (b *fakeBackend) Model() (migration.PrecheckModel, error) {
 	return &b.model, nil
 }
 
-func (b *fakeBackend) AllModels() ([]migration.PrecheckModel, error) {
+func (b *fakeBackend) AllModelUUIDs() ([]string, error) {
 	return b.models, nil
 }
 
@@ -695,7 +781,10 @@ func (b *fakeBackend) AllMachines() ([]migration.PrecheckMachine, error) {
 
 func (b *fakeBackend) AllApplications() ([]migration.PrecheckApplication, error) {
 	return b.apps, b.allAppsErr
+}
 
+func (b *fakeBackend) AllRelations() ([]migration.PrecheckRelation, error) {
+	return b.relations, b.allRelsErr
 }
 
 func (b *fakeBackend) ListPendingResources(app string) ([]resource.Resource, error) {
@@ -709,6 +798,27 @@ func (b *fakeBackend) ControllerBackend() (migration.PrecheckBackendCloser, erro
 	return b.controllerBackend, nil
 }
 
+type fakePool struct {
+	models []migration.PrecheckModel
+}
+
+func (p *fakePool) uuids() []string {
+	out := make([]string, len(p.models))
+	for i, model := range p.models {
+		out[i] = model.UUID()
+	}
+	return out
+}
+
+func (p *fakePool) GetModel(uuid string) (migration.PrecheckModel, func(), error) {
+	for _, model := range p.models {
+		if model.UUID() == uuid {
+			return model, func() {}, nil
+		}
+	}
+	return nil, nil, errors.NotFoundf("model %v", uuid)
+}
+
 type fakeModel struct {
 	uuid          string
 	name          string
@@ -889,3 +999,41 @@ func (u *fakeUnit) Status() (status.StatusInfo, error) {
 func (u *fakeUnit) AgentPresence() (bool, error) {
 	return !u.lost, nil
 }
+
+type fakeRelation struct {
+	key           string
+	crossModel    bool
+	crossModelErr error
+	endpoints     []state.Endpoint
+	relUnits      map[string]*fakeRelationUnit
+	unitErr       error
+}
+
+func (r *fakeRelation) String() string {
+	return r.key
+}
+
+func (r *fakeRelation) IsCrossModel() (bool, error) {
+	return r.crossModel, r.crossModelErr
+}
+
+func (r *fakeRelation) Endpoints() []state.Endpoint {
+	return r.endpoints
+}
+
+func (r *fakeRelation) Unit(u migration.PrecheckUnit) (migration.PrecheckRelationUnit, error) {
+	return r.relUnits[u.Name()], r.unitErr
+}
+
+type fakeRelationUnit struct {
+	valid, inScope     bool
+	validErr, scopeErr error
+}
+
+func (ru *fakeRelationUnit) Valid() (bool, error) {
+	return ru.valid, ru.validErr
+}
+
+func (ru *fakeRelationUnit) InScope() (bool, error) {
+	return ru.inScope, ru.scopeErr
+}
