@@ -69,7 +69,7 @@ type Query interface {
 	Explain(result interface{}) error
 	For(result interface{}, f func() error) error
 	Hint(indexKey ...string) Query
-	Iter() *mgo.Iter
+	Iter() Iterator
 	Limit(n int) Query
 	LogReplay() Query
 	MapReduce(job *mgo.MapReduce, result interface{}) (info *mgo.MapReduceInfo, err error)
@@ -84,6 +84,14 @@ type Query interface {
 	Tail(timeout time.Duration) *mgo.Iter
 }
 
+// Iterator defines the parts of the mgo.Iter that we use - this
+// interface allows us to switch out the querying for testing.
+type Iterator interface {
+	Next(interface{}) bool
+	Timeout() bool
+	Close() error
+}
+
 // WrapCollection returns a Collection that wraps the supplied *mgo.Collection.
 func WrapCollection(coll *mgo.Collection) Collection {
 	return collectionWrapper{coll}
@@ -176,3 +184,7 @@ func (qw queryWrapper) Snapshot() Query {
 func (qw queryWrapper) Sort(fields ...string) Query {
 	return queryWrapper{qw.Query.Sort(fields...)}
 }
+
+func (qw queryWrapper) Iter() Iterator {
+	return qw.Query.Iter()
+}
