@@ -372,9 +372,9 @@ func (s *MongoSuite) TestInstallMongod(c *gc.C) {
 
 	tests := []installs{
 		{"precise", [][]string{{"--target-release", "precise-updates/cloud-tools", "mongodb-server"}}},
-		{"trusty", [][]string{{"juju-mongodb3.2"}, {"juju-mongo-tools3.2"}}},
-		{"wily", [][]string{{"juju-mongodb3.2"}, {"juju-mongo-tools3.2"}}},
+		{"trusty", [][]string{{"juju-mongodb"}}},
 		{"xenial", [][]string{{"juju-mongodb3.2"}, {"juju-mongo-tools3.2"}}},
+		{"bionic", [][]string{{"mongodb-server-core"}}},
 	}
 
 	testing.PatchExecutableAsEchoArgs(c, s, "add-apt-repository")
@@ -432,9 +432,9 @@ func (s *MongoSuite) TestInstallMongodFallsBack(c *gc.C) {
 
 	tests := []installs{
 		{"precise", "mongodb-server"},
-		{"trusty", "juju-mongodb3.2\njuju-mongodb"},
-		{"wily", "juju-mongodb3.2\njuju-mongodb"},
-		{"xenial", "juju-mongodb3.2\njuju-mongodb"},
+		{"trusty", "juju-mongodb"},
+		{"xenial", "juju-mongodb3.2"},
+		{"bionic", "mongodb-server-core"},
 	}
 
 	dataDir := c.MkDir()
