@@ -56,6 +56,10 @@ func (s *AddressSuite) TestNewScopedAddressIPv4(c *gc.C) {
 		value:         "8.8.8.8",
 		scope:         network.ScopeUnknown,
 		expectedScope: network.ScopePublic,
+	}, {
+		value:         "241.1.2.3",
+		scope:         network.ScopeUnknown,
+		expectedScope: network.ScopeFanLocal,
 	}}
 
 	for i, t := range tests {
@@ -211,6 +215,10 @@ func (s *AddressSuite) TestNewAddresses(c *gc.C) {
 		[]string{"169.254.1.23", "169.254.1.1"},
 		network.IPv4Address,
 		network.ScopeLinkLocal,
+	}, {
+		[]string{"243.1.5.7", "245.3.1.2"},
+		network.IPv4Address,
+		network.ScopeFanLocal,
 	}, {
 		[]string{"ff01::2", "ff01::1"},
 		network.IPv6Address,
@@ -291,72 +299,80 @@ var selectPublicTests = []selectTest{{
 	},
 	1,
 }, {
-	"a machine IPv4 local address is not selected",
+	"the cloud local address is selected when a fan-local fallback exists",
 	[]network.Address{
-		network.NewScopedAddress("127.0.0.1", network.ScopeMachineLocal),
-	},
-	-1,
-}, {
-	"a machine IPv6 local address is not selected",
-	[]network.Address{
-		network.NewScopedAddress("::1", network.ScopeMachineLocal),
-	},
-	-1,
-}, {
-	"a link-local IPv4 address is not selected",
-	[]network.Address{
-		network.NewScopedAddress("169.254.1.1", network.ScopeLinkLocal),
-	},
-	-1,
-}, {
-	"a link-local (multicast or not) IPv6 address is not selected",
-	[]network.Address{
-		network.NewScopedAddress("fe80::1", network.ScopeLinkLocal),
-		network.NewScopedAddress("ff01::2", network.ScopeLinkLocal),
-		network.NewScopedAddress("ff02::1:1", network.ScopeLinkLocal),
-	},
-	-1,
-}, {
-	"a public name is preferred to an unknown or cloud local address",
-	[]network.Address{
-		network.NewScopedAddress("127.0.0.1", network.ScopeUnknown),
-		network.NewScopedAddress("10.0.0.1", network.ScopeCloudLocal),
-		network.NewScopedAddress("fc00::1", network.ScopeCloudLocal),
-		network.NewScopedAddress("public.invalid.testing", network.ScopePublic),
-	},
-	3,
-}, {
-	"first unknown address selected",
-	// NOTE(dimitern): Not using NewScopedAddress() below as it derives the
-	// scope internally from the value when given ScopeUnknown.
-	[]network.Address{
-		{Value: "10.0.0.1", Scope: network.ScopeUnknown},
-		{Value: "8.8.8.8", Scope: network.ScopeUnknown},
-	},
-	0,
-}, {
-	"public IP address is picked when both public IPs and public hostnames exist",
-	[]network.Address{
-		network.NewScopedAddress("10.0.0.1", network.ScopeUnknown),
-		network.NewScopedAddress("example.com", network.ScopePublic),
-		network.NewScopedAddress("8.8.8.8", network.ScopePublic),
-	},
-	2,
-}, {
-	"hostname is picked over cloud local address",
-	[]network.Address{
-		network.NewScopedAddress("10.0.0.1", network.ScopeUnknown),
-		network.NewScopedAddress("example.com", network.ScopePublic),
+		network.NewScopedAddress("243.1.1.1", network.ScopeFanLocal),
+		network.NewScopedAddress("172.16.1.1", network.ScopeCloudLocal),
 	},
 	1,
-}, {
-	"IPv4 preferred over IPv6",
-	[]network.Address{
-		network.NewScopedAddress("2001:db8::1", network.ScopePublic),
-		network.NewScopedAddress("8.8.8.8", network.ScopePublic),
-	},
-	1,
-}}
+},
+	{
+		"a machine IPv4 local address is not selected",
+		[]network.Address{
+			network.NewScopedAddress("127.0.0.1", network.ScopeMachineLocal),
+		},
+		-1,
+	}, {
+		"a machine IPv6 local address is not selected",
+		[]network.Address{
+			network.NewScopedAddress("::1", network.ScopeMachineLocal),
+		},
+		-1,
+	}, {
+		"a link-local IPv4 address is not selected",
+		[]network.Address{
+			network.NewScopedAddress("169.254.1.1", network.ScopeLinkLocal),
+		},
+		-1,
+	}, {
+		"a link-local (multicast or not) IPv6 address is not selected",
+		[]network.Address{
+			network.NewScopedAddress("fe80::1", network.ScopeLinkLocal),
+			network.NewScopedAddress("ff01::2", network.ScopeLinkLocal),
+			network.NewScopedAddress("ff02::1:1", network.ScopeLinkLocal),
+		},
+		-1,
+	}, {
+		"a public name is preferred to an unknown or cloud local address",
+		[]network.Address{
+			network.NewScopedAddress("127.0.0.1", network.ScopeUnknown),
+			network.NewScopedAddress("10.0.0.1", network.ScopeCloudLocal),
+			network.NewScopedAddress("fc00::1", network.ScopeCloudLocal),
+			network.NewScopedAddress("public.invalid.testing", network.ScopePublic),
+		},
+		3,
+	}, {
+		"first unknown address selected",
+		// NOTE(dimitern): Not using NewScopedAddress() below as it derives the
+		// scope internally from the value when given ScopeUnknown.
+		[]network.Address{
+			{Value: "10.0.0.1", Scope: network.ScopeUnknown},
+			{Value: "8.8.8.8", Scope: network.ScopeUnknown},
+		},
+		0,
+	}, {
+		"public IP address is picked when both public IPs and public hostnames exist",
+		[]network.Address{
+			network.NewScopedAddress("10.0.0.1", network.ScopeUnknown),
+			network.NewScopedAddress("example.com", network.ScopePublic),
+			network.NewScopedAddress("8.8.8.8", network.ScopePublic),
+		},
+		2,
+	}, {
+		"hostname is picked over cloud local address",
+		[]network.Address{
+			network.NewScopedAddress("10.0.0.1", network.ScopeUnknown),
+			network.NewScopedAddress("example.com", network.ScopePublic),
+		},
+		1,
+	}, {
+		"IPv4 preferred over IPv6",
+		[]network.Address{
+			network.NewScopedAddress("2001:db8::1", network.ScopePublic),
+			network.NewScopedAddress("8.8.8.8", network.ScopePublic),
+		},
+		1,
+	}}
 
 func (s *AddressSuite) TestSelectPublicAddress(c *gc.C) {
 	for i, t := range selectPublicTests {
@@ -702,6 +718,7 @@ func (*AddressSuite) TestSortAddresses(c *gc.C) {
 		"fc00::1",
 		"169.254.1.2",
 		"localhost",
+		"243.5.1.2",
 		"2001:db8::1",
 		"fe80::2",
 		"7.8.8.8",
@@ -723,6 +740,8 @@ func (*AddressSuite) TestSortAddresses(c *gc.C) {
 		"172.16.0.1",
 		// Then IPv6 cloud-local addresses.
 		"fc00::1",
+		// Then fan-local addresses.
+		"243.5.1.2",
 		// Then machine-local IPv4 addresses.
 		"127.0.0.1",
 		// Then machine-local IPv6 addresses.
