@@ -7,7 +7,6 @@ import (
 	"fmt"
 	"net"
 	"strings"
-	"time"
 
 	"github.com/juju/errors"
 	jc "github.com/juju/testing/checkers"
@@ -214,25 +213,25 @@ func (*HostPortSuite) TestParseHostPortsErrors(c *gc.C) {
 		err   string
 	}{{
 		input: "",
-		err:   `cannot parse "" as address:port: missing port in address`,
+		err:   `cannot parse "" as address:port: .*missing port in address.*`,
 	}, {
 		input: " ",
-		err:   `cannot parse " " as address:port: missing port in address  `,
+		err:   `cannot parse " " as address:port: .*missing port in address.*`,
 	}, {
 		input: ":",
 		err:   `cannot parse ":" port: strconv.(ParseInt|Atoi): parsing "": invalid syntax`,
 	}, {
 		input: "host",
-		err:   `cannot parse "host" as address:port: missing port in address host`,
+		err:   `cannot parse "host" as address:port: .*missing port in address.*`,
 	}, {
 		input: "host:port",
 		err:   `cannot parse "host:port" port: strconv.(ParseInt|Atoi): parsing "port": invalid syntax`,
 	}, {
 		input: "::1",
-		err:   `cannot parse "::1" as address:port: too many colons in address ::1`,
+		err:   `cannot parse "::1" as address:port: .*too many colons in address.*`,
 	}, {
 		input: "1.2.3.4",
-		err:   `cannot parse "1.2.3.4" as address:port: missing port in address 1.2.3.4`,
+		err:   `cannot parse "1.2.3.4" as address:port: .*missing port in address.*`,
 	}, {
 		input: "1.2.3.4:foo",
 		err:   `cannot parse "1.2.3.4:foo" port: strconv.(ParseInt|Atoi): parsing "foo": invalid syntax`,
@@ -245,7 +244,7 @@ func (*HostPortSuite) TestParseHostPortsErrors(c *gc.C) {
 	}
 	// Finally, test with mixed valid and invalid args.
 	hps, err := network.ParseHostPorts("1.2.3.4:42", "[fc00::1]:12", "foo")
-	c.Assert(err, gc.ErrorMatches, `cannot parse "foo" as address:port: missing port in address foo`)
+	c.Assert(err, gc.ErrorMatches, `cannot parse "foo" as address:port: .*missing port in address.*`)
 	c.Assert(hps, gc.IsNil)
 }
 
@@ -501,30 +500,6 @@ func (s *HostPortSuite) TestUniqueHostPortsHugeUniqueInput(c *gc.C) {
 	c.Assert(results, jc.DeepEquals, expected)
 }
 
-func (s *HostPortSuite) TestReachableHostPortAllUnreachable(c *gc.C) {
-	dialer := &net.Dialer{Timeout: 100 * time.Millisecond}
-	unreachableHPs := s.manyHostPorts(c, maxTCPPort, nil) // use IANA reserved port
-	timeout := 300 * time.Millisecond
-
-	best, err := network.ReachableHostPort(unreachableHPs, dialer, timeout)
-	c.Check(err, gc.ErrorMatches, "cannot connect to any address: .*")
-	c.Check(best, gc.Equals, network.HostPort{})
-}
-
-func (s *HostPortSuite) TestReachableHostPortRealDial(c *gc.C) {
-	fakeHostPort := network.NewHostPorts(1234, "127.0.0.1")[0]
-	hostPorts := []network.HostPort{
-		fakeHostPort,
-		testTCPServer(c),
-	}
-	timeout := 300 * time.Millisecond
-
-	dialer := &net.Dialer{Timeout: 100 * time.Millisecond}
-	best, err := network.ReachableHostPort(hostPorts, dialer, timeout)
-	c.Check(err, jc.ErrorIsNil)
-	c.Check(best, jc.DeepEquals, hostPorts[1]) // the only real listener
-}
-
 const maxTCPPort = 65535
 
 func (s *HostPortSuite) manyHostPorts(c *gc.C, count int, addressFunc func(index int) string) []network.HostPort {
@@ -542,24 +517,3 @@ func (s *HostPortSuite) manyHostPorts(c *gc.C, count int, addressFunc func(index
 	}
 	return results
 }
-
-func testTCPServer(c *gc.C) network.HostPort {
-	listener, err := net.Listen("tcp", "127.0.0.1:0")
-	c.Assert(err, jc.ErrorIsNil)
-
-	listenAddress := listener.Addr().String()
-	hostPort, err := network.ParseHostPort(listenAddress)
-	c.Assert(err, jc.ErrorIsNil)
-	c.Logf("listening on %q", hostPort)
-
-	go func() {
-		conn, _ := listener.Accept()
-		if conn != nil {
-			c.Logf("accepted connection on %q from %s", hostPort, conn.RemoteAddr())
-			conn.Close()
-		}
-		listener.Close()
-	}()
-
-	return *hostPort
-}
