@@ -139,13 +139,6 @@ type SpaceInfo struct {
 	ProviderId Id
 	Subnets    []SubnetInfo
 }
-type BySpaceName []SpaceInfo
-
-func (s BySpaceName) Len() int      { return len(s) }
-func (s BySpaceName) Swap(i, j int) { s[i], s[j] = s[j], s[i] }
-func (s BySpaceName) Less(i, j int) bool {
-	return s[i].Name < s[j].Name
-}
 
 // InterfaceConfigType defines valid network interface configuration
 // types. See interfaces(5) for details
@@ -269,6 +262,9 @@ type InterfaceInfo struct {
 	// Routes defines a list of routes that should be added when this interface
 	// is brought up, and removed when this interface is stopped.
 	Routes []Route
+
+	// IsDefaultGateway is set if this device is a default gw on a machine.
+	IsDefaultGateway bool
 }
 
 // Route defines a single route to a subnet via a defined gateway.
@@ -410,6 +406,9 @@ type DeviceToBridge struct {
 
 	// BridgeName is the name of the bridge that we want created.
 	BridgeName string
+
+	// MACAddress is the MAC address of the device to be bridged
+	MACAddress string
 }
 
 // LXCNetDefaultConfig is the location of the default network config
@@ -580,3 +579,51 @@ func QuoteSpaces(vals []string) string {
 func QuoteSpaceSet(vals set.Strings) string {
 	return QuoteSpaces(vals.SortedValues())
 }
+
+// firstLastAddresses returns the first and last addresses of the subnet.
+func firstLastAddresses(subnet *net.IPNet) (net.IP, net.IP) {
+	firstIP := subnet.IP
+	lastIP := make([]byte, len(firstIP))
+	copy(lastIP, firstIP)
+
+	for i, b := range lastIP {
+		lastIP[i] = b ^ (^subnet.Mask[i])
+	}
+	return firstIP, lastIP
+}
+
+func cidrContains(cidr *net.IPNet, subnet *net.IPNet) bool {
+	first, last := firstLastAddresses(subnet)
+	return cidr.Contains(first) && cidr.Contains(last)
+}
+
+// SubnetInAnyRange returns true if the subnet's address range is fully
+// contained in any of the specified subnet blocks.
+func SubnetInAnyRange(cidrs []*net.IPNet, subnet *net.IPNet) bool {
+	for _, cidr := range cidrs {
+		if cidrContains(cidr, subnet) {
+			return true
+		}
+	}
+	return false
+}
+
+// FormatAsCIDR converts the specified IP addresses to
+// a slice of CIDRs.
+func FormatAsCIDR(addresses []string) []string {
+	result := make([]string, len(addresses))
+	for i, a := range addresses {
+		cidr := a
+		// If address is not already a cidr, add a /32 (ipv4) or /128 (ipv6).
+		if _, _, err := net.ParseCIDR(a); err != nil {
+			ip := net.ParseIP(a)
+			if ip.To4() != nil {
+				cidr = a + "/32"
+			} else {
+				cidr = a + "/128"
+			}
+		}
+		result[i] = cidr
+	}
+	return result
+}
