@@ -42,7 +42,12 @@ type ProviderConfig struct {
 	// clients.
 	NewStorageClient azurestorage.NewClientFunc
 
-	// RetryClock is used when retrying API calls due to rate-limiting.
+	// RetryClock is used for retrying some operations, like
+	// waiting for deployments to complete.
+	//
+	// Retries due to rate-limiting are handled by the go-autorest
+	// package, which uses "time" directly. We cannot mock the
+	// waiting in that case.
 	RetryClock clock.Clock
 
 	// RandomWindowsAdminPassword is a function used to generate
@@ -137,7 +142,15 @@ func (prov *azureEnvironProvider) PrepareConfig(args environs.PrepareConfigParam
 	if err := validateCloudSpec(args.Cloud); err != nil {
 		return nil, errors.Annotate(err, "validating cloud spec")
 	}
-	return args.Config, nil
+	// Set the default block-storage source.
+	attrs := make(map[string]interface{})
+	if _, ok := args.Config.StorageDefaultBlockSource(); !ok {
+		attrs[config.StorageDefaultBlockSourceKey] = azureStorageProviderType
+	}
+	if len(attrs) == 0 {
+		return args.Config, nil
+	}
+	return args.Config.Apply(attrs)
 }
 
 func validateCloudSpec(spec environs.CloudSpec) error {
