@@ -6,9 +6,11 @@ package azure
 import (
 	"github.com/juju/errors"
 	"github.com/juju/utils/clock"
+	"github.com/juju/utils/ssh"
 
 	"github.com/juju/juju/environs"
 	"github.com/juju/juju/provider/azure/internal/azureauth"
+	"github.com/juju/juju/provider/azure/internal/azurecli"
 	"github.com/juju/juju/provider/azure/internal/azurestorage"
 )
 
@@ -28,10 +30,12 @@ func NewProvider(config ProviderConfig) (environs.EnvironProvider, error) {
 
 func init() {
 	environProvider, err := NewProvider(ProviderConfig{
-		NewStorageClient:                  azurestorage.NewClient,
-		RetryClock:                        &clock.WallClock,
-		RandomWindowsAdminPassword:        randomAdminPassword,
-		InteractiveCreateServicePrincipal: azureauth.InteractiveCreateServicePrincipal,
+		NewStorageClient:           azurestorage.NewClient,
+		RetryClock:                 &clock.WallClock,
+		RandomWindowsAdminPassword: randomAdminPassword,
+		GenerateSSHKey:             ssh.GenerateKey,
+		ServicePrincipalCreator:    &azureauth.ServicePrincipalCreator{},
+		AzureCLI:                   azurecli.AzureCLI{},
 	})
 	if err != nil {
 		panic(err)
