@@ -5,6 +5,7 @@ package ec2
 
 import (
 	"regexp"
+	"strings"
 	"sync"
 	"time"
 
@@ -41,15 +42,24 @@ const (
 	// Specifies whether the volume should be encrypted.
 	EBS_Encrypted = "encrypted"
 
-	volumeTypeMagnetic        = "magnetic"         // standard
-	volumeTypeSSD             = "ssd"              // gp2
-	volumeTypeProvisionedIops = "provisioned-iops" // io1
-	volumeTypeStandard        = "standard"
-	volumeTypeGP2             = "gp2"
-	volumeTypeIO1             = "io1"
+	// Volume Aliases
+	volumeAliasMagnetic        = "magnetic"         // standard
+	volumeAliasOptimizedHDD    = "optimized-hdd"    // sc1
+	volumeAliasColdStorage     = "cold-storage"     // sc1
+	volumeAliasSSD             = "ssd"              // gp2
+	volumeAliasProvisionedIops = "provisioned-iops" // io1
+
+	// Volume types
+	volumeTypeStandard = "standard"
+	volumeTypeGP2      = "gp2"
+	volumeTypeIO1      = "io1"
+	volumeTypeST1      = "st1"
+	volumeTypeSC1      = "sc1"
 
 	rootDiskDeviceName = "/dev/sda1"
 
+	nvmeDeviceLinkPrefix = "/dev/disk/by-id/nvme-Amazon_Elastic_Block_Store_"
+
 	// defaultControllerDiskSizeMiB is the default size for the
 	// root disk of controller machines, if no root-disk constraint
 	// is specified.
@@ -107,6 +117,18 @@ const (
 	// volumes. We take the minimum of volumeSize*maxProvisionedIopsSizeRatio and
 	// maxProvisionedIops.
 	maxProvisionedIops = 20000
+
+	// minSt1VolumeSizeGiB is the minimum volume size for st1 volume instances.
+	minSt1VolumeSizeGiB = 500
+
+	// maxSt1VolumeSizeGiB is the maximum volume size for st1 volume instances.
+	maxSt1VolumeSizeGiB = 16 * 1024
+
+	// minSc1VolumeSizeGiB is the minimum volume size for sc1 volume instances.
+	minSc1VolumeSizeGiB = 500
+
+	// maxSc1VolumeSizeGiB is the maximum volume size for sc1 volume instances.
+	maxSc1VolumeSizeGiB = 16 * 1024
 )
 
 const (
@@ -143,12 +165,16 @@ var _ storage.Provider = (*ebsProvider)(nil)
 
 var ebsConfigFields = schema.Fields{
 	EBS_VolumeType: schema.OneOf(
-		schema.Const(volumeTypeMagnetic),
-		schema.Const(volumeTypeSSD),
-		schema.Const(volumeTypeProvisionedIops),
+		schema.Const(volumeAliasMagnetic),
+		schema.Const(volumeAliasOptimizedHDD),
+		schema.Const(volumeAliasColdStorage),
+		schema.Const(volumeAliasSSD),
+		schema.Const(volumeAliasProvisionedIops),
 		schema.Const(volumeTypeStandard),
 		schema.Const(volumeTypeGP2),
 		schema.Const(volumeTypeIO1),
+		schema.Const(volumeTypeST1),
+		schema.Const(volumeTypeSC1),
 	),
 	EBS_IOPS:      schema.ForceInt(),
 	EBS_Encrypted: schema.Bool(),
@@ -157,7 +183,7 @@ var ebsConfigFields = schema.Fields{
 var ebsConfigChecker = schema.FieldMap(
 	ebsConfigFields,
 	schema.Defaults{
-		EBS_VolumeType: volumeTypeMagnetic,
+		EBS_VolumeType: volumeAliasSSD,
 		EBS_IOPS:       schema.Omit,
 		EBS_Encrypted:  false,
 	},
@@ -183,11 +209,15 @@ func newEbsConfig(attrs map[string]interface{}) (*ebsConfig, error) {
 		encrypted:  coerced[EBS_Encrypted].(bool),
 	}
 	switch ebsConfig.volumeType {
-	case volumeTypeMagnetic:
+	case volumeAliasMagnetic:
 		ebsConfig.volumeType = volumeTypeStandard
-	case volumeTypeSSD:
+	case volumeAliasColdStorage:
+		ebsConfig.volumeType = volumeTypeSC1
+	case volumeAliasOptimizedHDD:
+		ebsConfig.volumeType = volumeTypeST1
+	case volumeAliasSSD:
 		ebsConfig.volumeType = volumeTypeGP2
-	case volumeTypeProvisionedIops:
+	case volumeAliasProvisionedIops:
 		ebsConfig.volumeType = volumeTypeIO1
 	}
 	if ebsConfig.iops > 0 && ebsConfig.volumeType != volumeTypeIO1 {
@@ -227,7 +257,7 @@ func (*ebsProvider) Releasable() bool {
 // DefaultPools is defined on the Provider interface.
 func (e *ebsProvider) DefaultPools() []*storage.Config {
 	ssdPool, _ := storage.NewConfig("ebs-ssd", EBS_ProviderType, map[string]interface{}{
-		EBS_VolumeType: volumeTypeSSD,
+		EBS_VolumeType: volumeAliasSSD,
 	})
 	return []*storage.Config{ssdPool}
 }
@@ -634,6 +664,12 @@ func (v *ebsVolumeSource) ValidateVolumeParams(params storage.VolumeParams) erro
 	case volumeTypeIO1:
 		minVolumeSize = minProvisionedIopsVolumeSizeGiB
 		maxVolumeSize = maxProvisionedIopsVolumeSizeGiB
+	case volumeTypeST1:
+		minVolumeSize = minSt1VolumeSizeGiB
+		maxVolumeSize = maxSt1VolumeSizeGiB
+	case volumeTypeSC1:
+		minVolumeSize = minSc1VolumeSizeGiB
+		maxVolumeSize = maxSc1VolumeSizeGiB
 	}
 	if vol.VolumeSize < minVolumeSize {
 		return errors.Errorf(
@@ -652,25 +688,30 @@ func (v *ebsVolumeSource) ValidateVolumeParams(params storage.VolumeParams) erro
 
 // AttachVolumes is specified on the storage.VolumeSource interface.
 func (v *ebsVolumeSource) AttachVolumes(attachParams []storage.VolumeAttachmentParams) ([]storage.AttachVolumesResult, error) {
-	// We need the virtualisation types for each instance we are
-	// attaching to so we can determine the device name.
+	// We need the instance type for each instance we are
+	// attaching to so we can determine how to identify the
+	// volume attachment
 	instIds := set.NewStrings()
 	for _, p := range attachParams {
 		instIds.Add(string(p.InstanceId))
 	}
 	instances := make(instanceCache)
-	if instIds.Size() > 1 {
-		if err := instances.update(v.env.ec2, instIds.Values()...); err != nil {
-			logger.Debugf("querying running instances: %v", err)
-			// We ignore the error, because we don't want an invalid
-			// InstanceId reference from one VolumeParams to prevent
-			// the creation of another volume.
-		}
+	if err := instances.update(v.env.ec2, instIds.Values()...); err != nil {
+		logger.Debugf("querying running instances: %v", err)
+		// We ignore the error, because we don't want an invalid
+		// InstanceId reference from one VolumeParams to prevent
+		// the creation of another volume.
 	}
 
 	results := make([]storage.AttachVolumesResult, len(attachParams))
 	for i, params := range attachParams {
 		instId := string(params.InstanceId)
+		inst, err := instances.get(instId)
+		if err != nil {
+			results[i].Error = err
+			continue
+		}
+
 		// By default we should allocate device names without the
 		// trailing number. Block devices with a trailing number are
 		// not liked by some applications, e.g. Ceph, which want full
@@ -686,12 +727,33 @@ func (v *ebsVolumeSource) AttachVolumes(attachParams []storage.VolumeAttachmentP
 			results[i].Error = err
 			continue
 		}
+
+		var attachmentInfo storage.VolumeAttachmentInfo
+		if strings.HasPrefix(inst.InstanceType, "c5.") || strings.HasPrefix(inst.InstanceType, "m5.") {
+			// The newer hypervisor attaches EBS volumes
+			// as NVMe devices, and the block device names
+			// are unpredictable from here.
+			//
+			// Instead of using device name, we fill in
+			// the device link, based on the statically
+			// defined model name ("Amazon Elastic Block Store",
+			// and serial (vol123456abcdef...); the serial
+			// is the same as the volume ID without the "-".
+			//
+			// NOTE(axw) inst.Hypervisor still says "xen" for
+			// m5 and c5 instance types, which would seem to
+			// be a lie. This is why we check the prefix,
+			// rather than the hypervisor name.
+			sn := strings.Replace(params.VolumeId, "-", "", 1)
+			attachmentInfo.DeviceLink = nvmeDeviceLinkPrefix + sn
+		} else {
+			attachmentInfo.DeviceName = deviceName
+		}
+
 		results[i].VolumeAttachment = &storage.VolumeAttachment{
 			params.Volume,
 			params.Machine,
-			storage.VolumeAttachmentInfo{
-				DeviceName: deviceName,
-			},
+			attachmentInfo,
 		}
 	}
 	return results, nil
