@@ -34,6 +34,7 @@ import (
 	"github.com/juju/juju/provider/common"
 	"github.com/juju/juju/provider/ec2/internal/ec2instancetypes"
 	"github.com/juju/juju/status"
+	"github.com/juju/juju/storage"
 	"github.com/juju/juju/tools"
 )
 
@@ -73,8 +74,13 @@ type environ struct {
 	defaultVPCMutex   sync.Mutex
 	defaultVPCChecked bool
 	defaultVPC        *ec2.VPC
+
+	ensureGroupMutex sync.Mutex
 }
 
+var _ environs.Environ = (*environ)(nil)
+var _ environs.Networking = (*environ)(nil)
+
 func (e *environ) Config() *config.Config {
 	return e.ecfg().Config
 }
@@ -240,6 +246,15 @@ func (e *environ) InstanceAvailabilityZoneNames(ids []instance.Id) ([]string, er
 	return zones, err
 }
 
+// DeriveAvailabilityZones is part of the common.ZonedEnviron interface.
+func (e *environ) DeriveAvailabilityZones(args environs.StartInstanceParams) ([]string, error) {
+	availabilityZone, err := e.deriveAvailabilityZone(args)
+	if availabilityZone != "" {
+		return []string{availabilityZone}, errors.Trace(err)
+	}
+	return nil, errors.Trace(err)
+}
+
 type ec2Placement struct {
 	availabilityZone *ec2.AvailabilityZoneInfo
 	subnet           *ec2.Subnet
@@ -283,7 +298,7 @@ func (e *environ) parsePlacement(placement string) (*ec2Placement, error) {
 		for _, subnet := range subnetResp.Subnets {
 			allSubnets = append(allSubnets, fmt.Sprintf("%q:%q", subnet.Id, subnet.CIDRBlock))
 			if matcher.Match(subnet) {
-				// We found the CIDR, now see if we can find the AZ
+				// We found the CIDR, now see if we can find the AZs.
 				for _, zone := range zones {
 					if zone.Name() == subnet.AvailZone {
 						ec2AZ := zone.(*ec2AvailabilityZone)
@@ -301,14 +316,17 @@ func (e *environ) parsePlacement(placement string) (*ec2Placement, error) {
 	return nil, fmt.Errorf("unknown placement directive: %v", placement)
 }
 
-// PrecheckInstance is defined on the state.Prechecker interface.
-func (e *environ) PrecheckInstance(series string, cons constraints.Value, placement string) error {
-	if placement != "" {
-		if _, err := e.parsePlacement(placement); err != nil {
-			return err
-		}
+// PrecheckInstance is defined on the environs.InstancePrechecker interface.
+func (e *environ) PrecheckInstance(args environs.PrecheckInstanceParams) error {
+	if _, _, err := e.deriveAvailabilityZoneAndSubnetID(
+		environs.StartInstanceParams{
+			Placement:         args.Placement,
+			VolumeAttachments: args.VolumeAttachments,
+		},
+	); err != nil {
+		return errors.Trace(err)
 	}
-	if !cons.HasInstanceType() {
+	if !args.Constraints.HasInstanceType() {
 		return nil
 	}
 	// Constraint has an instance-type constraint so let's see if it is valid.
@@ -317,17 +335,17 @@ func (e *environ) PrecheckInstance(series string, cons constraints.Value, placem
 		return errors.Trace(err)
 	}
 	for _, itype := range instanceTypes {
-		if itype.Name != *cons.InstanceType {
+		if itype.Name != *args.Constraints.InstanceType {
 			continue
 		}
-		if archMatches(itype.Arches, cons.Arch) {
+		if archMatches(itype.Arches, args.Constraints.Arch) {
 			return nil
 		}
 	}
-	if cons.Arch == nil {
-		return fmt.Errorf("invalid AWS instance type %q specified", *cons.InstanceType)
+	if args.Constraints.Arch == nil {
+		return fmt.Errorf("invalid AWS instance type %q specified", *args.Constraints.InstanceType)
 	}
-	return fmt.Errorf("invalid AWS instance type %q and arch %q specified", *cons.InstanceType, *cons.Arch)
+	return fmt.Errorf("invalid AWS instance type %q and arch %q specified", *args.Constraints.InstanceType, *args.Constraints.Arch)
 }
 
 // MetadataLookupParams returns parameters which are used to query simplestreams metadata.
@@ -371,8 +389,6 @@ func (e *environ) DistributeInstances(candidates, distributionGroup []instance.I
 	return common.DistributeInstances(e, candidates, distributionGroup)
 }
 
-var availabilityZoneAllocations = common.AvailabilityZoneAllocations
-
 // MaintainInstance is specified in the InstanceBroker interface.
 func (*environ) MaintainInstance(args environs.StartInstanceParams) error {
 	return nil
@@ -386,9 +402,6 @@ func resourceName(tag names.Tag, envName string) string {
 
 // StartInstance is specified in the InstanceBroker interface.
 func (e *environ) StartInstance(args environs.StartInstanceParams) (_ *environs.StartInstanceResult, resultErr error) {
-	if args.ControllerUUID == "" {
-		return nil, errors.New("missing controller UUID")
-	}
 	var inst *ec2Instance
 	callback := args.StatusCallback
 	defer func() {
@@ -401,56 +414,29 @@ func (e *environ) StartInstance(args environs.StartInstanceParams) (_ *environs.
 		}
 	}()
 
-	var availabilityZones []string
-	var placementSubnetID string
-	if args.Placement != "" {
-		placement, err := e.parsePlacement(args.Placement)
-		if err != nil {
-			return nil, err
-		}
-		if placement.availabilityZone.State != availableState {
-			return nil, errors.Errorf("availability zone %q is %q", placement.availabilityZone.Name, placement.availabilityZone.State)
-		}
-		availabilityZones = append(availabilityZones, placement.availabilityZone.Name)
-		if placement.subnet != nil {
-			if placement.subnet.State != availableState {
-				return nil, errors.Errorf("subnet %q is %q", placement.subnet.CIDRBlock, placement.subnet.State)
-			}
-			placementSubnetID = placement.subnet.Id
-		}
-	}
+	callback(status.Allocating, "Verifying availability zone", nil)
 
-	callback(status.Allocating, "Determining availability zones", nil)
-	// If no availability zone is specified, then automatically spread across
-	// the known zones for optimal spread across the instance distribution
-	// group.
-	var zoneInstances []common.AvailabilityZoneInstances
-	if len(availabilityZones) == 0 {
-		var err error
-		var group []instance.Id
-		if args.DistributionGroup != nil {
-			group, err = args.DistributionGroup()
-			if err != nil {
-				return nil, err
-			}
-		}
-		zoneInstances, err = availabilityZoneAllocations(e, group)
-		if err != nil {
-			return nil, err
-		}
-		for _, z := range zoneInstances {
-			availabilityZones = append(availabilityZones, z.ZoneName)
-		}
-		if len(availabilityZones) == 0 {
-			return nil, errors.New("failed to determine availability zones")
+	// Verify the provided availability zone to start the instance in.  It's
+	// provided via StartInstanceParams Constraints or AvailabilityZone.
+	// The availability zone of existing volumes that are to be
+	// attached to the machine must all match, and must be the same
+	// as specified zone (if any).
+	availabilityZone, placementSubnetID, err := e.deriveAvailabilityZoneAndSubnetID(args)
+	if err != nil {
+		// An IsNotValid error is returned if the zone is invalid;
+		// this is a zone-specific error.
+		zoneSpecific := errors.IsNotValid(err)
+		if !zoneSpecific {
+			err = common.ZoneIndependentError(err)
 		}
+		return nil, err
 	}
 
 	arches := args.Tools.Arches()
 
 	instanceTypes, err := e.supportedInstanceTypes()
 	if err != nil {
-		return nil, errors.Trace(err)
+		return nil, common.ZoneIndependentError(err)
 	}
 
 	spec, err := findInstanceSpec(
@@ -466,11 +452,13 @@ func (e *environ) StartInstance(args environs.StartInstanceParams) (_ *environs.
 		},
 	)
 	if err != nil {
-		return nil, err
+		return nil, common.ZoneIndependentError(err)
 	}
 	tools, err := args.Tools.Match(tools.Filter{Arch: spec.Image.Arch})
 	if err != nil {
-		return nil, errors.Errorf("chosen architecture %v not present in %v", spec.Image.Arch, arches)
+		return nil, common.ZoneIndependentError(
+			errors.Errorf("chosen architecture %v not present in %v", spec.Image.Arch, arches),
+		)
 	}
 
 	if spec.InstanceType.Deprecated {
@@ -478,16 +466,18 @@ func (e *environ) StartInstance(args environs.StartInstanceParams) (_ *environs.
 	}
 
 	if err := args.InstanceConfig.SetTools(tools); err != nil {
-		return nil, errors.Trace(err)
+		return nil, common.ZoneIndependentError(err)
 	}
 	if err := instancecfg.FinishInstanceConfig(args.InstanceConfig, e.Config()); err != nil {
-		return nil, err
+		return nil, common.ZoneIndependentError(err)
 	}
 
 	callback(status.Allocating, "Making user data", nil)
 	userData, err := providerinit.ComposeUserData(args.InstanceConfig, nil, AmazonRenderer{})
 	if err != nil {
-		return nil, errors.Annotate(err, "cannot make user data")
+		return nil, common.ZoneIndependentError(
+			errors.Annotate(err, "cannot make user data"),
+		)
 	}
 	logger.Debugf("ec2 user data; %d bytes", len(userData))
 	var apiPort int
@@ -498,9 +488,10 @@ func (e *environ) StartInstance(args environs.StartInstanceParams) (_ *environs.
 	}
 	callback(status.Allocating, "Setting up groups", nil)
 	groups, err := e.setUpGroups(args.ControllerUUID, args.InstanceConfig.MachineId, apiPort)
-
 	if err != nil {
-		return nil, errors.Annotate(err, "cannot set up groups")
+		return nil, common.ZoneIndependentError(
+			errors.Annotate(err, "cannot set up groups"),
+		)
 	}
 
 	blockDeviceMappings := getBlockDeviceMappings(
@@ -534,67 +525,61 @@ func (e *environ) StartInstance(args environs.StartInstanceParams) (_ *environs.
 		ImageId:             spec.Image.Id,
 	}
 
+	runArgs := commonRunArgs
+	runArgs.AvailZone = availabilityZone
+
 	haveVPCID := isVPCIDSet(e.ecfg().vpcID())
-
-	for _, zone := range availabilityZones {
-		runArgs := commonRunArgs
-		runArgs.AvailZone = zone
-
-		var subnetIDsForZone []string
-		var subnetErr error
-		if haveVPCID {
-			var allowedSubnetIDs []string
-			if placementSubnetID != "" {
-				allowedSubnetIDs = []string{placementSubnetID}
+	var subnetIDsForZone []string
+	var subnetErr error
+	if haveVPCID {
+		var allowedSubnetIDs []string
+		if placementSubnetID != "" {
+			allowedSubnetIDs = []string{placementSubnetID}
+		} else {
+			for subnetID, _ := range args.SubnetsToZones {
+				allowedSubnetIDs = append(allowedSubnetIDs, string(subnetID))
+			}
+		}
+		subnetIDsForZone, subnetErr = getVPCSubnetIDsForAvailabilityZone(e.ec2, e.ecfg().vpcID(), availabilityZone, allowedSubnetIDs)
+	} else if args.Constraints.HaveSpaces() {
+		subnetIDsForZone, subnetErr = findSubnetIDsForAvailabilityZone(availabilityZone, args.SubnetsToZones)
+		if subnetErr == nil && placementSubnetID != "" {
+			asSet := set.NewStrings(subnetIDsForZone...)
+			if asSet.Contains(placementSubnetID) {
+				subnetIDsForZone = []string{placementSubnetID}
 			} else {
-				for subnetID, _ := range args.SubnetsToZones {
-					allowedSubnetIDs = append(allowedSubnetIDs, string(subnetID))
-				}
-			}
-			subnetIDsForZone, subnetErr = getVPCSubnetIDsForAvailabilityZone(e.ec2, e.ecfg().vpcID(), zone, allowedSubnetIDs)
-		} else if args.Constraints.HaveSpaces() {
-			subnetIDsForZone, subnetErr = findSubnetIDsForAvailabilityZone(zone, args.SubnetsToZones)
-			if subnetErr == nil && placementSubnetID != "" {
-				asSet := set.NewStrings(subnetIDsForZone...)
-				if asSet.Contains(placementSubnetID) {
-					subnetIDsForZone = []string{placementSubnetID}
-				} else {
-					subnetIDsForZone = nil
-					subnetErr = errors.NotFoundf("subnets %q in AZ %q", placementSubnetID, zone)
-				}
+				subnetIDsForZone = nil
+				subnetErr = errors.NotFoundf("subnets %q in AZ %q", placementSubnetID, availabilityZone)
 			}
 		}
-
-		switch {
-		case subnetErr != nil && errors.IsNotFound(subnetErr):
-			logger.Infof("no matching subnets in zone %q; assuming zone is constrained and trying another", zone)
-			continue
-		case subnetErr != nil:
-			return nil, errors.Annotatef(subnetErr, "getting subnets for zone %q", zone)
-		case len(subnetIDsForZone) > 1:
-			// With multiple equally suitable subnets, picking one at random
-			// will allow for better instance spread within the same zone, and
-			// still work correctly if we happen to pick a constrained subnet
-			// (we'll just treat this the same way we treat constrained zones
-			// and retry).
-			runArgs.SubnetId = subnetIDsForZone[rand.Intn(len(subnetIDsForZone))]
-			logger.Debugf("selected random subnet %q from all matching in zone %q", runArgs.SubnetId, zone)
-		case len(subnetIDsForZone) == 1:
-			runArgs.SubnetId = subnetIDsForZone[0]
-			logger.Debugf("selected subnet %q in zone %q", runArgs.SubnetId, zone)
-		}
-
-		callback(status.Allocating, fmt.Sprintf("Trying to start instance in availability zone %q", zone), nil)
-		instResp, err = runInstances(e.ec2, runArgs, callback)
-		if err == nil || !isZoneOrSubnetConstrainedError(err) {
-			break
-		}
-
-		logger.Infof("%q is constrained, trying another availability zone", zone)
 	}
 
+	switch {
+	case subnetErr != nil && errors.IsNotFound(subnetErr):
+		return nil, errors.Trace(subnetErr)
+	case subnetErr != nil:
+		return nil, errors.Annotatef(subnetErr, "getting subnets for zone %q", availabilityZone)
+	case len(subnetIDsForZone) > 1:
+		// With multiple equally suitable subnets, picking one at random
+		// will allow for better instance spread within the same zone, and
+		// still work correctly if we happen to pick a constrained subnet
+		// (we'll just treat this the same way we treat constrained zones
+		// and retry).
+		runArgs.SubnetId = subnetIDsForZone[rand.Intn(len(subnetIDsForZone))]
+		logger.Debugf("selected random subnet %q from all matching in zone %q", runArgs.SubnetId, availabilityZone)
+	case len(subnetIDsForZone) == 1:
+		runArgs.SubnetId = subnetIDsForZone[0]
+		logger.Debugf("selected subnet %q in zone %q", runArgs.SubnetId, availabilityZone)
+	}
+
+	callback(status.Allocating, fmt.Sprintf("Trying to start instance in availability zone %q", availabilityZone), nil)
+	instResp, err = runInstances(e.ec2, runArgs, callback)
 	if err != nil {
-		return nil, errors.Annotate(err, "cannot run instances")
+		err := errors.Annotate(err, "cannot run instances")
+		if !isZoneOrSubnetConstrainedError(err) {
+			err = common.ZoneIndependentError(err)
+		}
+		return nil, err
 	}
 	if len(instResp.Instances) != 1 {
 		return nil, errors.Errorf("expected 1 started instance, got %d", len(instResp.Instances))
@@ -619,7 +604,9 @@ func (e *environ) StartInstance(args environs.StartInstanceParams) (_ *environs.
 	)
 	args.InstanceConfig.Tags[tagName] = instanceName
 	if err := tagResources(e.ec2, args.InstanceConfig.Tags, string(inst.Id())); err != nil {
-		return nil, errors.Annotate(err, "tagging instance")
+		return nil, common.ZoneIndependentError(
+			errors.Annotate(err, "tagging instance"),
+		)
 	}
 
 	// Tag the machine's root EBS volume, if it has one.
@@ -632,7 +619,9 @@ func (e *environ) StartInstance(args environs.StartInstanceParams) (_ *environs.
 		)
 		tags[tagName] = instanceName + "-root"
 		if err := tagRootDisk(e.ec2, tags, inst.Instance); err != nil {
-			return nil, errors.Annotate(err, "tagging root disk")
+			return nil, common.ZoneIndependentError(
+				errors.Annotate(err, "tagging root disk"),
+			)
 		}
 	}
 
@@ -651,6 +640,117 @@ func (e *environ) StartInstance(args environs.StartInstanceParams) (_ *environs.
 	}, nil
 }
 
+func (e *environ) deriveAvailabilityZone(args environs.StartInstanceParams) (string, error) {
+	availabilityZone, _, err := e.deriveAvailabilityZoneAndSubnetID(args)
+	return availabilityZone, err
+}
+
+func (e *environ) deriveAvailabilityZoneAndSubnetID(args environs.StartInstanceParams) (string, string, error) {
+	// Determine the availability zones of existing volumes that are to be
+	// attached to the machine. They must all match, and must be the same
+	// as specified zone (if any).
+	volumeAttachmentsZone, err := volumeAttachmentsZone(e.ec2, args.VolumeAttachments)
+	if err != nil {
+		return "", "", errors.Trace(err)
+	}
+	placementZone, placementSubnetID, err := e.instancePlacementZone(args.Placement, volumeAttachmentsZone)
+	if err != nil {
+		return "", "", errors.Trace(err)
+	}
+	var availabilityZone string
+	if placementZone != "" {
+		availabilityZone = placementZone
+	} else if args.AvailabilityZone != "" {
+		// Validate and check state of the AvailabilityZone
+		zones, err := e.AvailabilityZones()
+		if err != nil {
+			return "", "", err
+		}
+		for _, z := range zones {
+			if z.Name() == args.AvailabilityZone {
+				ec2AZ := z.(*ec2AvailabilityZone)
+				if ec2AZ.AvailabilityZoneInfo.State != availableState {
+					return "", "", errors.Errorf(
+						"availability zone %q is %q",
+						ec2AZ.AvailabilityZoneInfo.Name,
+						ec2AZ.AvailabilityZoneInfo.State,
+					)
+				} else {
+					availabilityZone = args.AvailabilityZone
+				}
+				break
+			}
+		}
+		if availabilityZone == "" {
+			return "", "", errors.NotValidf("availability zone %q", availabilityZone)
+		}
+	}
+	return availabilityZone, placementSubnetID, nil
+}
+
+func (e *environ) instancePlacementZone(placement, volumeAttachmentsZone string) (zone, subnet string, _ error) {
+	if placement == "" {
+		return volumeAttachmentsZone, "", nil
+	}
+	var placementSubnetID string
+	instPlacement, err := e.parsePlacement(placement)
+	if err != nil {
+		return "", "", errors.Trace(err)
+	}
+	if instPlacement.availabilityZone.State != availableState {
+		return "", "", errors.Errorf(
+			"availability zone %q is %q",
+			instPlacement.availabilityZone.Name,
+			instPlacement.availabilityZone.State,
+		)
+	}
+	if volumeAttachmentsZone != "" && volumeAttachmentsZone != instPlacement.availabilityZone.Name {
+		return "", "", errors.Errorf(
+			"cannot create instance with placement %q, as this will prevent attaching the requested EBS volumes in zone %q",
+			placement, volumeAttachmentsZone,
+		)
+	}
+	if instPlacement.subnet != nil {
+		if instPlacement.subnet.State != availableState {
+			return "", "", errors.Errorf("subnet %q is %q", instPlacement.subnet.CIDRBlock, instPlacement.subnet.State)
+		}
+		placementSubnetID = instPlacement.subnet.Id
+	}
+	return instPlacement.availabilityZone.Name, placementSubnetID, nil
+}
+
+// volumeAttachmentsZone determines the availability zone for each volume
+// identified in the volume attachment parameters, checking that they are
+// all the same, and returns the availability zone name.
+func volumeAttachmentsZone(ec2 *ec2.EC2, attachments []storage.VolumeAttachmentParams) (string, error) {
+	volumeIds := make([]string, 0, len(attachments))
+	for _, a := range attachments {
+		if a.Provider != EBS_ProviderType {
+			continue
+		}
+		volumeIds = append(volumeIds, a.VolumeId)
+	}
+	if len(volumeIds) == 0 {
+		return "", nil
+	}
+	resp, err := ec2.Volumes(volumeIds, nil)
+	if err != nil {
+		return "", errors.Annotatef(err, "getting volume details (%s)", volumeIds)
+	}
+	if len(resp.Volumes) == 0 {
+		return "", nil
+	}
+	for i, v := range resp.Volumes[1:] {
+		if v.AvailZone != resp.Volumes[i].AvailZone {
+			return "", errors.Errorf(
+				"cannot attach volumes from multiple availability zones: %s is in %s, %s is in %s",
+				resp.Volumes[i].Id, resp.Volumes[i].AvailZone, v.Id, v.AvailZone,
+			)
+		}
+	}
+	return resp.Volumes[0].AvailZone, nil
+}
+
 // tagResources calls ec2.CreateTags, tagging each of the specified resources
 // with the given tags. tagResources will retry for a short period of time
 // if it receives a *.NotFound error response from EC2.
@@ -1188,7 +1288,7 @@ func (e *environ) destroyControllerManagedEnvirons(controllerUUID string) error
 	if err != nil {
 		return errors.Annotate(err, "listing volumes")
 	}
-	errs := destroyVolumes(e.ec2, volIds)
+	errs := foreachVolume(e.ec2, volIds, destroyVolume)
 	for i, err := range errs {
 		if err == nil {
 			continue
@@ -1651,6 +1751,12 @@ func (e *environ) securityGroupsByNameOrID(groupName string) (*ec2.SecurityGroup
 // Any entries in perms without SourceIPs will be granted for
 // the named group only.
 func (e *environ) ensureGroup(controllerUUID, name string, perms []ec2.IPPerm) (g ec2.SecurityGroup, err error) {
+	// Due to parallelization of the provisioner, it's possible that we try
+	// to create the model security group a second time before the first time
+	// is complete causing failures.
+	e.ensureGroupMutex.Lock()
+	defer e.ensureGroupMutex.Unlock()
+
 	// Specify explicit VPC ID if needed (not for default VPC or EC2-classic).
 	chosenVPCID := e.ecfg().vpcID()
 	inVPCLogSuffix := fmt.Sprintf(" (in VPC %q)", chosenVPCID)
@@ -1795,7 +1901,7 @@ func isZoneOrSubnetConstrainedError(err error) bool {
 // constrained for the instance type being provisioned, or is
 // otherwise unusable for the specific request made.
 func isZoneConstrainedError(err error) bool {
-	switch err := err.(type) {
+	switch err := errors.Cause(err).(type) {
 	case *ec2.Error:
 		switch err.Code {
 		case "Unsupported", "InsufficientInstanceCapacity":
@@ -1821,7 +1927,7 @@ func isZoneConstrainedError(err error) bool {
 // the instance type being provisioned, or is otherwise unusable for the
 // specific request made.
 func isSubnetConstrainedError(err error) bool {
-	switch err := err.(type) {
+	switch err := errors.Cause(err).(type) {
 	case *ec2.Error:
 		switch err.Code {
 		case "InsufficientFreeAddressesInSubnet", "InsufficientInstanceCapacity":
@@ -1912,3 +2018,21 @@ func (*environ) AreSpacesRoutable(space1, space2 *environs.ProviderSpaceInfo) (b
 func (*environ) SSHAddresses(addresses []network.Address) ([]network.Address, error) {
 	return addresses, nil
 }
+
+// SuperSubnets implements NetworkingEnviron.SuperSubnets
+func (e *environ) SuperSubnets() ([]string, error) {
+	vpcId := e.ecfg().vpcID()
+	if !isVPCIDSet(vpcId) {
+		if hasDefaultVPC, err := e.hasDefaultVPC(); err == nil && hasDefaultVPC {
+			vpcId = e.defaultVPC.Id
+		}
+	}
+	if !isVPCIDSet(vpcId) {
+		return nil, errors.NotSupportedf("Not a VPC environment")
+	}
+	cidr, err := getVPCCIDR(e.ec2, vpcId)
+	if err != nil {
+		return nil, err
+	}
+	return []string{cidr}, nil
+}
