@@ -54,13 +54,12 @@ func AllModelGroups(e environs.Environ) ([]string, error) {
 }
 
 var (
-	EC2AvailabilityZones        = &ec2AvailabilityZones
-	AvailabilityZoneAllocations = &availabilityZoneAllocations
-	RunInstances                = &runInstances
-	BlockDeviceNamer            = blockDeviceNamer
-	GetBlockDeviceMappings      = getBlockDeviceMappings
-	IsVPCNotUsableError         = isVPCNotUsableError
-	IsVPCNotRecommendedError    = isVPCNotRecommendedError
+	EC2AvailabilityZones     = &ec2AvailabilityZones
+	RunInstances             = &runInstances
+	BlockDeviceNamer         = blockDeviceNamer
+	GetBlockDeviceMappings   = getBlockDeviceMappings
+	IsVPCNotUsableError      = isVPCNotUsableError
+	IsVPCNotRecommendedError = isVPCNotRecommendedError
 )
 
 const VPCIDNone = vpcIDNone
@@ -84,10 +83,6 @@ var (
 	TerminateInstancesById         = &terminateInstancesById
 )
 
-func EC2ErrCode(err error) string {
-	return ec2ErrCode(err)
-}
-
 // FabricateInstance creates a new fictitious instance
 // given an existing instance and a new id.
 func FabricateInstance(inst instance.Instance, newId string) instance.Instance {
