@@ -320,21 +320,23 @@ func parseMem(s string) (uint64, error) {
 
 func locationToRegion(loc string) (string, bool) {
 	regions := map[string]string{
-		"US East (N. Virginia)":     "us-east-1",
-		"US East (Ohio)":            "us-east-2",
-		"US West (N. California)":   "us-west-1",
-		"US West (Oregon)":          "us-west-2",
-		"Asia Pacific (Mumbai)":     "ap-south-1",
-		"Asia Pacific (Seoul)":      "ap-northeast-2",
-		"Asia Pacific (Singapore)":  "ap-southeast-1",
-		"Asia Pacific (Sydney)":     "ap-southeast-2",
-		"Asia Pacific (Tokyo)":      "ap-northeast-1",
-		"Canada (Central)":          "ca-central-1",
-		"EU (Frankfurt)":            "eu-central-1",
-		"EU (Ireland)":              "eu-west-1",
-		"EU (London)":               "eu-west-2",
-		"South America (Sao Paulo)": "sa-east-1",
-		"AWS GovCloud (US)":         "us-gov-west-1",
+		"US East (N. Virginia)":      "us-east-1",
+		"US East (Ohio)":             "us-east-2",
+		"US West (N. California)":    "us-west-1",
+		"US West (Oregon)":           "us-west-2",
+		"Canada (Central)":           "ca-central-1",
+		"EU (Frankfurt)":             "eu-central-1",
+		"EU (Ireland)":               "eu-west-1",
+		"EU (London)":                "eu-west-2",
+		"EU (Paris)":                 "eu-west-3",
+		"Asia Pacific (Tokyo)":       "ap-northeast-1",
+		"Asia Pacific (Seoul)":       "ap-northeast-2",
+		"Asia Pacific (Osaka-Local)": "ap-northeast-3",
+		"Asia Pacific (Singapore)":   "ap-southeast-1",
+		"Asia Pacific (Sydney)":      "ap-southeast-2",
+		"Asia Pacific (Mumbai)":      "ap-south-1",
+		"South America (Sao Paulo)":  "sa-east-1",
+		"AWS GovCloud (US)":          "us-gov-west-1",
 
 		// NOTE(axw) at the time of writing, there is no
 		// pricing information for cn-north-1.
