@@ -4,23 +4,15 @@
 package joyent_test
 
 import (
-	"fmt"
-	"io/ioutil"
-	"os"
-
-	jc "github.com/juju/testing/checkers"
-	"github.com/juju/utils"
 	gc "gopkg.in/check.v1"
 
 	envtesting "github.com/juju/juju/environs/testing"
 	coretesting "github.com/juju/juju/testing"
-	"github.com/juju/juju/version"
 )
 
 const (
-	testUser        = "test"
-	testKeyFileName = "provider_id_rsa"
-	testPrivateKey  = `-----BEGIN RSA PRIVATE KEY-----
+	testUser       = "test"
+	testPrivateKey = `-----BEGIN RSA PRIVATE KEY-----
 MIIEpAIBAAKCAQEAza+KvczCrcpQGRq9e347VHx9oEvuhseJt0ydR+UMAveyQprU
 4JHvzwUUhGnG147GJQYyfQ4nzaSG62az/YThoZJzw8gtxGkVHv0wlAlRkYhxbKbq
 8WQIh73xDQkHLw2lXLvf7Tt0Mhow0qGEmkOjTb5fPsj2evphrV3jJ15QlhL4cv33
@@ -50,59 +42,38 @@ C+4FYwKBgQDE9yZTUpJjG2424z6bl/MHzwl5RB4pMronp0BbeVqPwhCBfj0W5I42
 	testKeyFingerprint = "66:ca:1c:09:75:99:35:69:be:91:08:25:03:c0:17:c0"
 )
 
-type providerSuite struct {
-	coretesting.FakeJujuHomeSuite
+type baseSuite struct {
+	coretesting.FakeJujuXDGDataHomeSuite
 	envtesting.ToolsFixture
 	restoreTimeouts func()
 }
 
-var _ = gc.Suite(&providerSuite{})
+var _ = gc.Suite(&baseSuite{})
 
-func (s *providerSuite) SetUpSuite(c *gc.C) {
+func (s *baseSuite) SetUpSuite(c *gc.C) {
+	s.FakeJujuXDGDataHomeSuite.SetUpSuite(c)
 	s.restoreTimeouts = envtesting.PatchAttemptStrategies()
-	s.FakeJujuHomeSuite.SetUpSuite(c)
 }
 
-func (s *providerSuite) TearDownSuite(c *gc.C) {
+func (s *baseSuite) TearDownSuite(c *gc.C) {
 	s.restoreTimeouts()
-	s.FakeJujuHomeSuite.TearDownSuite(c)
+	s.FakeJujuXDGDataHomeSuite.TearDownSuite(c)
 }
 
-func (s *providerSuite) SetUpTest(c *gc.C) {
-	s.FakeJujuHomeSuite.SetUpTest(c)
+func (s *baseSuite) SetUpTest(c *gc.C) {
+	s.FakeJujuXDGDataHomeSuite.SetUpTest(c)
 	s.ToolsFixture.SetUpTest(c)
-	s.AddCleanup(CreateTestKey(c))
 }
 
-func (s *providerSuite) TearDownTest(c *gc.C) {
+func (s *baseSuite) TearDownTest(c *gc.C) {
 	s.ToolsFixture.TearDownTest(c)
-	s.FakeJujuHomeSuite.TearDownTest(c)
+	s.FakeJujuXDGDataHomeSuite.TearDownTest(c)
 }
 
-func GetFakeConfig(sdcUrl, mantaUrl string) coretesting.Attrs {
+func GetFakeConfig() coretesting.Attrs {
 	return coretesting.FakeConfig().Merge(coretesting.Attrs{
-		"name":             "joyent test environment",
-		"type":             "joyent",
-		"sdc-user":         testUser,
-		"sdc-key-id":       testKeyFingerprint,
-		"sdc-url":          sdcUrl,
-		"manta-user":       testUser,
-		"manta-key-id":     testKeyFingerprint,
-		"manta-url":        mantaUrl,
-		"private-key-path": fmt.Sprintf("~/.ssh/%s", testKeyFileName),
-		"algorithm":        "rsa-sha256",
-		"control-dir":      "juju-test",
-		"agent-version":    version.Current.Number.String(),
+		"name":          "joyent-test-model",
+		"type":          "joyent",
+		"agent-version": coretesting.FakeVersionNumber.String(),
 	})
 }
-
-func CreateTestKey(c *gc.C) func(*gc.C) {
-	keyFile := fmt.Sprintf("~/.ssh/%s", testKeyFileName)
-	keyFilePath, err := utils.NormalizePath(keyFile)
-	c.Assert(err, jc.ErrorIsNil)
-	err = ioutil.WriteFile(keyFilePath, []byte(testPrivateKey), 400)
-	c.Assert(err, jc.ErrorIsNil)
-	return func(c *gc.C) {
-		os.Remove(keyFilePath)
-	}
-}
