@@ -156,18 +156,14 @@ func (env *environ) Bootstrap(ctx environs.BootstrapContext, params environs.Boo
 // Destroy shuts down all known machines and destroys the rest of the
 // known environment.
 func (env *environ) Destroy() error {
-	rules, err := env.IngressRules()
-	if err != nil {
-		return errors.Trace(err)
-	}
-	if len(rules) > 0 {
-		if err := env.ClosePorts(rules); err != nil {
-			return errors.Trace(err)
-		}
-	}
 	if err := env.base.DestroyEnv(); err != nil {
 		return errors.Trace(err)
 	}
+	if env.storageSupported() {
+		if err := destroyModelFilesystems(env); err != nil {
+			return errors.Annotate(err, "destroying LXD filesystems for model")
+		}
+	}
 	return nil
 }
 
@@ -179,6 +175,11 @@ func (env *environ) DestroyController(controllerUUID string) error {
 	if err := env.destroyHostedModelResources(controllerUUID); err != nil {
 		return errors.Trace(err)
 	}
+	if env.storageSupported() {
+		if err := destroyControllerFilesystems(env, controllerUUID); err != nil {
+			return errors.Annotate(err, "destroying LXD filesystems for controller")
+		}
+	}
 	return nil
 }
 
