@@ -9,13 +9,13 @@ import (
 	"github.com/juju/cmd/cmdtesting"
 	gitjujutesting "github.com/juju/testing"
 	jc "github.com/juju/testing/checkers"
+	"github.com/lxc/lxd/shared/api"
 	gc "gopkg.in/check.v1"
 
 	"github.com/juju/juju/cloudconfig/instancecfg"
 	"github.com/juju/juju/cmd/modelcmd"
 	"github.com/juju/juju/environs"
 	envtesting "github.com/juju/juju/environs/testing"
-	"github.com/juju/juju/provider/common"
 	"github.com/juju/juju/provider/lxd"
 	coretesting "github.com/juju/juju/testing"
 	"github.com/juju/juju/tools/lxdclient"
@@ -101,33 +101,61 @@ func (s *environSuite) TestBootstrapAPI(c *gc.C) {
 }
 
 func (s *environSuite) TestDestroy(c *gc.C) {
-	err := s.Env.Destroy()
+	s.Client.Volumes = map[string][]api.StorageVolume{
+		"juju": []api.StorageVolume{{
+			StorageVolumePut: api.StorageVolumePut{
+				Name: "not-ours",
+				Config: map[string]string{
+					"user.juju-model-uuid": "other",
+				},
+			},
+		}, {
+			StorageVolumePut: api.StorageVolumePut{
+				Name: "ours",
+				Config: map[string]string{
+					"user.juju-model-uuid": s.Config.UUID(),
+				},
+			},
+		}},
+	}
 
-	c.Check(err, jc.ErrorIsNil)
-}
-
-func (s *environSuite) TestDestroyAPI(c *gc.C) {
 	err := s.Env.Destroy()
 	c.Assert(err, jc.ErrorIsNil)
 
-	fwname := common.EnvFullName(s.Env.Config().UUID())
-	s.Stub.CheckCalls(c, []gitjujutesting.StubCall{{
-		FuncName: "Ports",
-		Args: []interface{}{
-			fwname,
-		},
-	}, {
-		FuncName: "Destroy",
-		Args:     nil,
-	}})
+	s.Stub.CheckCalls(c, []gitjujutesting.StubCall{
+		{"Destroy", nil},
+		{"StorageSupported", nil},
+		{"StoragePools", nil},
+		{"VolumeList", []interface{}{"juju"}},
+		{"VolumeDelete", []interface{}{"juju", "ours"}},
+		{"VolumeList", []interface{}{"juju-zfs"}},
+	})
 }
 
-func (s *environSuite) TestDestroyHostedModels(c *gc.C) {
+func (s *environSuite) TestDestroyController(c *gc.C) {
 	s.UpdateConfig(c, map[string]interface{}{
 		"controller-uuid": s.Config.UUID(),
 	})
 	s.Stub.ResetCalls()
 
+	s.Client.Volumes = map[string][]api.StorageVolume{
+		"juju": []api.StorageVolume{{
+			StorageVolumePut: api.StorageVolumePut{
+				Name: "not-ours",
+				Config: map[string]string{
+					"user.juju-controller-uuid": "other",
+				},
+			},
+		}, {
+			StorageVolumePut: api.StorageVolumePut{
+				Name: "ours",
+				Config: map[string]string{
+					"user.juju-controller-uuid": s.Config.UUID(),
+				},
+			},
+		}},
+	}
+
 	// machine0 is in the controller model.
 	machine0 := s.NewRawInstance(c, "juju-controller-machine-0")
 	machine0.InstanceSummary.Metadata["juju-model-uuid"] = s.Config.UUID()
@@ -146,11 +174,18 @@ func (s *environSuite) TestDestroyHostedModels(c *gc.C) {
 	err := s.Env.DestroyController(s.Config.UUID())
 	c.Assert(err, jc.ErrorIsNil)
 
-	fwname := common.EnvFullName(s.Env.Config().UUID())
 	s.Stub.CheckCalls(c, []gitjujutesting.StubCall{
-		{"Ports", []interface{}{fwname}},
 		{"Destroy", nil},
+		{"StorageSupported", nil},
+		{"StoragePools", nil},
+		{"VolumeList", []interface{}{"juju"}},
+		{"VolumeList", []interface{}{"juju-zfs"}},
 		{"Instances", []interface{}{"juju-", lxdclient.AliveStatuses}},
 		{"RemoveInstances", []interface{}{"juju-", []string{machine1.Name}}},
+		{"StorageSupported", nil},
+		{"StoragePools", nil},
+		{"VolumeList", []interface{}{"juju"}},
+		{"VolumeDelete", []interface{}{"juju", "ours"}},
+		{"VolumeList", []interface{}{"juju-zfs"}},
 	})
 }
