@@ -18,7 +18,7 @@ type lxdRenderer struct{}
 // EncodeUserdata implements renderers.ProviderRenderer.
 func (lxdRenderer) Render(cfg cloudinit.CloudConfig, os jujuos.OSType) ([]byte, error) {
 	switch os {
-	case jujuos.Ubuntu, jujuos.CentOS:
+	case jujuos.Ubuntu, jujuos.CentOS, jujuos.OpenSUSE:
 		return renderers.RenderYAML(cfg)
 	default:
 		return nil, errors.Errorf("cannot encode userdata for OS %q", os)
