@@ -206,12 +206,12 @@ func (suite *environSuite) TestSelectNodeValidZone(c *gc.C) {
 	suite.testMAASObject.TestServer.NewNode(`{"system_id": "node0", "hostname": "host0", "zone": "bar"}`)
 
 	snArgs := selectNodeArgs{
-		AvailabilityZones: []string{"foo", "bar"},
-		Constraints:       constraints.Value{},
+		AvailabilityZone: "bar",
+		Constraints:      constraints.Value{},
 	}
 
 	node, err := env.selectNode(snArgs)
-	c.Assert(err, jc.ErrorIsNil)
+	c.Assert(err, gc.IsNil)
 	c.Assert(node, gc.NotNil)
 }
 
@@ -219,20 +219,21 @@ func (suite *environSuite) TestSelectNodeInvalidZone(c *gc.C) {
 	env := suite.makeEnviron()
 
 	snArgs := selectNodeArgs{
-		AvailabilityZones: []string{"foo", "bar"},
-		Constraints:       constraints.Value{},
+		AvailabilityZone: "foo",
+		Constraints:      constraints.Value{},
 	}
 
 	_, err := env.selectNode(snArgs)
 	c.Assert(err, gc.NotNil)
-	c.Assert(err, gc.ErrorMatches, `cannot run instances: ServerError: 409 Conflict \(\)`)
+	c.Assert(err.noMatch, jc.IsTrue)
+	c.Assert(err, gc.ErrorMatches, ".*409.*")
 }
 
 func (suite *environSuite) TestAcquireNode(c *gc.C) {
 	env := suite.makeEnviron()
 	suite.testMAASObject.TestServer.NewNode(`{"system_id": "node0", "hostname": "host0"}`)
 
-	_, err := env.acquireNode("", "", constraints.Value{}, nil, nil)
+	_, err := env.acquireNode("", "", "", constraints.Value{}, nil, nil)
 
 	c.Check(err, jc.ErrorIsNil)
 	operations := suite.testMAASObject.TestServer.NodeOperations()
@@ -250,7 +251,7 @@ func (suite *environSuite) TestAcquireNodeByName(c *gc.C) {
 	env := suite.makeEnviron()
 	suite.testMAASObject.TestServer.NewNode(`{"system_id": "node0", "hostname": "host0"}`)
 
-	_, err := env.acquireNode("host0", "", constraints.Value{}, nil, nil)
+	_, err := env.acquireNode("host0", "", "", constraints.Value{}, nil, nil)
 
 	c.Check(err, jc.ErrorIsNil)
 	operations := suite.testMAASObject.TestServer.NodeOperations()
@@ -271,7 +272,7 @@ func (suite *environSuite) TestAcquireNodeTakesConstraintsIntoAccount(c *gc.C) {
 	)
 	constraints := constraints.Value{Arch: stringp("arm"), Mem: uint64p(1024)}
 
-	_, err := env.acquireNode("", "", constraints, nil, nil)
+	_, err := env.acquireNode("", "", "", constraints, nil, nil)
 
 	c.Check(err, jc.ErrorIsNil)
 	requestValues := suite.testMAASObject.TestServer.NodeOperationRequestValues()
@@ -344,7 +345,7 @@ func (suite *environSuite) TestAcquireNodePassedAgentName(c *gc.C) {
 	env := suite.makeEnviron()
 	suite.testMAASObject.TestServer.NewNode(`{"system_id": "node0", "hostname": "host0"}`)
 
-	_, err := env.acquireNode("", "", constraints.Value{}, nil, nil)
+	_, err := env.acquireNode("", "", "", constraints.Value{}, nil, nil)
 
 	c.Check(err, jc.ErrorIsNil)
 	requestValues := suite.testMAASObject.TestServer.NodeOperationRequestValues()
@@ -358,7 +359,7 @@ func (suite *environSuite) TestAcquireNodePassesPositiveAndNegativeTags(c *gc.C)
 	suite.testMAASObject.TestServer.NewNode(`{"system_id": "node0", "tag_names": "tag1,tag3"}`)
 
 	_, err := env.acquireNode(
-		"", "",
+		"", "", "",
 		constraints.Value{Tags: stringslicep("tag1", "^tag2", "tag3", "^tag4")},
 		nil, nil,
 	)
@@ -377,7 +378,7 @@ func (suite *environSuite) TestAcquireNodePassesPositiveAndNegativeSpaces(c *gc.
 	suite.testMAASObject.TestServer.NewNode(`{"system_id": "node0"}`)
 
 	_, err := env.acquireNode(
-		"", "",
+		"", "", "",
 		constraints.Value{Spaces: stringslicep("space-1", "^space-2", "space-3", "^space-4")},
 		nil, nil,
 	)
@@ -410,7 +411,7 @@ func (suite *environSuite) TestAcquireNodeDisambiguatesNamedLabelsFromIndexedUpT
 	suite.testMAASObject.TestServer.NewNode(`{"system_id": "node0"}`)
 
 	_, err := env.acquireNode(
-		"", "",
+		"", "", "",
 		constraints.Value{Spaces: stringslicep("space-1", "^space-2", "space-3", "^space-4")},
 		[]interfaceBinding{{"0", "first-clash"}, {"1", "final-clash"}},
 		nil,
@@ -447,7 +448,7 @@ func (suite *environSuite) TestAcquireNodeStorage(c *gc.C) {
 		server.NewSpace(spaceJSON(gomaasapi.CreateSpace{Name: "space-1"}))
 		server.NewNode(`{"system_id": "node0", "hostname": "host0"}`)
 		suite.addSubnet(c, 1, 1, "node0")
-		_, err := env.acquireNode("", "", constraints.Value{}, nil, test.volumes)
+		_, err := env.acquireNode("", "", "", constraints.Value{}, nil, test.volumes)
 		c.Check(err, jc.ErrorIsNil)
 		requestValues := server.NodeOperationRequestValues()
 		nodeRequestValues, found := requestValues["node0"]
@@ -548,7 +549,7 @@ func (suite *environSuite) TestAcquireNodeInterfaces(c *gc.C) {
 		suite.addSubnetWithSpace(c, 7, 7, "bar", "node1")
 		env := suite.makeEnviron()
 		server.NewNode(`{"system_id": "node0", "hostname": "host0"}`)
-		_, err := env.acquireNode("", "", cons, test.interfaces, nil)
+		_, err := env.acquireNode("", "", "", cons, test.interfaces, nil)
 		if test.expectedError != "" {
 			c.Check(err, gc.ErrorMatches, test.expectedError)
 			c.Check(err, jc.Satisfies, errors.IsNotValid)
@@ -582,7 +583,7 @@ func (suite *environSuite) TestAcquireNodeConvertsSpaceNames(c *gc.C) {
 	}
 	env := suite.makeEnviron()
 	server.NewNode(`{"system_id": "node0", "hostname": "host0"}`)
-	_, err := env.acquireNode("", "", cons, nil, nil)
+	_, err := env.acquireNode("", "", "", cons, nil, nil)
 	c.Assert(err, jc.ErrorIsNil)
 	requestValues := server.NodeOperationRequestValues()
 	nodeRequestValues, found := requestValues["node0"]
@@ -599,7 +600,7 @@ func (suite *environSuite) TestAcquireNodeTranslatesSpaceNames(c *gc.C) {
 	}
 	env := suite.makeEnviron()
 	server.NewNode(`{"system_id": "node0", "hostname": "host0"}`)
-	_, err := env.acquireNode("", "", cons, nil, nil)
+	_, err := env.acquireNode("", "", "", cons, nil, nil)
 	c.Assert(err, jc.ErrorIsNil)
 	requestValues := server.NodeOperationRequestValues()
 	nodeRequestValues, found := requestValues["node0"]
@@ -616,6 +617,6 @@ func (suite *environSuite) TestAcquireNodeUnrecognisedSpace(c *gc.C) {
 	}
 	env := suite.makeEnviron()
 	server.NewNode(`{"system_id": "node0", "hostname": "host0"}`)
-	_, err := env.acquireNode("", "", cons, nil, nil)
+	_, err := env.acquireNode("", "", "", cons, nil, nil)
 	c.Assert(err, gc.ErrorMatches, `unrecognised space in constraint "baz"`)
 }
