@@ -4,8 +4,6 @@
 package maas
 
 import (
-	"github.com/juju/gomaasapi"
-
 	"github.com/juju/juju/cloudconfig/cloudinit"
 	"github.com/juju/juju/environs"
 )
@@ -14,10 +12,6 @@ var (
 	ShortAttempt = &shortAttempt
 )
 
-func GetMAASClient(env environs.Environ) *gomaasapi.MAASObject {
-	return env.(*maasEnviron).getMAASClient()
-}
-
 func NewCloudinitConfig(env environs.Environ, hostname, series string) (cloudinit.CloudConfig, error) {
 	return env.(*maasEnviron).newCloudinitConfig(hostname, series)
 }
