@@ -8,7 +8,6 @@ import (
 	"encoding/base64"
 	"io/ioutil"
 	"math/rand"
-	"net/http"
 	"net/url"
 
 	"github.com/juju/errors"
@@ -214,19 +213,6 @@ func (s *storageSuite) TestListOperatesOnFlatNamespace(c *gc.C) {
 	c.Check(listing, gc.DeepEquals, []string{"a/b/c/d"})
 }
 
-// getFileAtURL requests, and returns, the file at the given URL.
-func getFileAtURL(fileURL string) ([]byte, error) {
-	response, err := http.Get(fileURL)
-	if err != nil {
-		return nil, err
-	}
-	body, err := ioutil.ReadAll(response.Body)
-	if err != nil {
-		return nil, err
-	}
-	return body, nil
-}
-
 func (s *storageSuite) TestURLReturnsURLCorrespondingToFile(c *gc.C) {
 	const filename = "my-file.txt"
 	stor := NewStorage(s.makeEnviron()).(*maas1Storage)
