@@ -314,7 +314,7 @@ exit 0
 	return err
 }
 
-func (*manualEnviron) PrecheckInstance(series string, _ constraints.Value, placement string) error {
+func (*manualEnviron) PrecheckInstance(environs.PrecheckInstanceParams) error {
 	return errors.New(`use "juju add-machine ssh:[user@]<host>" to provision machines`)
 }
 
