@@ -4,7 +4,6 @@
 package manual
 
 import (
-	"fmt"
 	"strings"
 
 	"github.com/juju/errors"
@@ -144,13 +143,6 @@ func (p ManualProvider) open(host, user string, cfg *environConfig) (environs.En
 	return env, nil
 }
 
-func checkImmutableString(cfg, old *environConfig, key string) error {
-	if old.attrs[key] != cfg.attrs[key] {
-		return fmt.Errorf("cannot change %s from %q to %q", key, old.attrs[key], cfg.attrs[key])
-	}
-	return nil
-}
-
 func (p ManualProvider) validate(cfg, old *config.Config) (*environConfig, error) {
 	// Check for valid changes for the base config values.
 	if err := config.Validate(cfg, old); err != nil {
