@@ -4,6 +4,7 @@
 package openstack
 
 import (
+	"fmt"
 	"math"
 	"net/url"
 	"sync"
@@ -13,6 +14,7 @@ import (
 	"github.com/juju/schema"
 	"github.com/juju/utils"
 	"gopkg.in/goose.v2/cinder"
+	gooseerrors "gopkg.in/goose.v2/errors"
 	"gopkg.in/goose.v2/identity"
 	"gopkg.in/goose.v2/nova"
 
@@ -373,10 +375,7 @@ func destroyVolume(storageAdapter OpenstackStorage, volumeId string) error {
 				args[i].InstanceId = instance.Id(a.ServerId)
 			}
 			if len(args) > 0 {
-				results, err := detachVolumes(storageAdapter, args)
-				if err != nil {
-					return false, errors.Trace(err)
-				}
+				results := detachVolumes(storageAdapter, args)
 				for _, err := range results {
 					if err != nil {
 						return false, errors.Trace(err)
@@ -388,6 +387,11 @@ func destroyVolume(storageAdapter OpenstackStorage, volumeId string) error {
 		return false, nil
 	})
 	if err != nil {
+		if errors.IsNotFound(err) {
+			// The volume wasn't found; nothing
+			// to destroy, so we're done.
+			return nil
+		}
 		return errors.Trace(err)
 	}
 	if volume.Status == volumeStatusDeleting {
@@ -521,22 +525,15 @@ func waitVolume(
 
 // DetachVolumes implements storage.VolumeSource.
 func (s *cinderVolumeSource) DetachVolumes(args []storage.VolumeAttachmentParams) ([]error, error) {
-	return detachVolumes(s.storageAdapter, args)
+	return detachVolumes(s.storageAdapter, args), nil
 }
 
-func detachVolumes(storageAdapter OpenstackStorage, args []storage.VolumeAttachmentParams) ([]error, error) {
+func detachVolumes(storageAdapter OpenstackStorage, args []storage.VolumeAttachmentParams) []error {
 	results := make([]error, len(args))
 	for i, arg := range args {
-		// Check to see if the volume is already detached.
-		attachments, err := storageAdapter.ListVolumeAttachments(string(arg.InstanceId))
-		if err != nil {
-			results[i] = errors.Annotate(err, "listing volume attachments")
-			continue
-		}
 		if err := detachVolume(
 			string(arg.InstanceId),
 			arg.VolumeId,
-			attachments,
 			storageAdapter,
 		); err != nil {
 			results[i] = errors.Annotatef(
@@ -546,7 +543,7 @@ func detachVolumes(storageAdapter OpenstackStorage, args []storage.VolumeAttachm
 			continue
 		}
 	}
-	return results, nil
+	return results
 }
 
 func cinderToJujuVolumeInfos(volumes []cinder.Volume) []storage.VolumeInfo {
@@ -565,15 +562,14 @@ func cinderToJujuVolumeInfo(volume *cinder.Volume) storage.VolumeInfo {
 	}
 }
 
-func detachVolume(instanceId, volumeId string, attachments []nova.VolumeAttachment, storageAdapter OpenstackStorage) error {
-	// TODO(axw) verify whether we need to do this find step. From looking at the example
-	// responses in the OpenStack docs, the "attachment ID" is always the same as the
-	// volume ID. So we should just be able to issue a blind detach request, and then
-	// ignore errors that indicate the volume is already detached.
-	if findAttachment(volumeId, attachments) == nil {
-		return nil
+func detachVolume(instanceId, volumeId string, storageAdapter OpenstackStorage) error {
+	err := storageAdapter.DetachVolume(instanceId, volumeId)
+	if err != nil && !errors.IsNotFound(err) {
+		return errors.Trace(err)
 	}
-	return storageAdapter.DetachVolume(instanceId, volumeId)
+	// The volume was successfully detached, or was
+	// already detached (i.e. NotFound error case).
+	return nil
 }
 
 func findAttachment(volId string, attachments []nova.VolumeAttachment) *nova.VolumeAttachment {
@@ -656,6 +652,9 @@ func (ga *openstackStorageAdapter) GetVolumesDetail() ([]cinder.Volume, error) {
 func (ga *openstackStorageAdapter) GetVolume(volumeId string) (*cinder.Volume, error) {
 	resp, err := ga.cinderClient.GetVolume(volumeId)
 	if err != nil {
+		if gooseerrors.IsNotFound(err) {
+			return nil, errors.NotFoundf("volume %q", volumeId)
+		}
 		return nil, err
 	}
 	return &resp.Volume, nil
@@ -665,3 +664,29 @@ func (ga *openstackStorageAdapter) GetVolume(volumeId string) (*cinder.Volume, e
 func (ga *openstackStorageAdapter) SetVolumeMetadata(volumeId string, metadata map[string]string) (map[string]string, error) {
 	return ga.cinderClient.SetVolumeMetadata(volumeId, metadata)
 }
+
+// DeleteVolume is part of the OpenstackStorage interface.
+func (ga *openstackStorageAdapter) DeleteVolume(volumeId string) error {
+	if err := ga.cinderClient.DeleteVolume(volumeId); err != nil {
+		if gooseerrors.IsNotFound(err) {
+			return errors.NotFoundf("volume %q", volumeId)
+		}
+		return err
+	}
+	return nil
+}
+
+// DetachVolume is part of the OpenstackStorage interface.
+func (ga *openstackStorageAdapter) DetachVolume(serverId, attachmentId string) error {
+	if err := ga.novaClient.DetachVolume(serverId, attachmentId); err != nil {
+		if gooseerrors.IsNotFound(err) {
+			return errors.NewNotFound(nil,
+				fmt.Sprintf("volume %q is not attached to server %q",
+					attachmentId, serverId,
+				),
+			)
+		}
+		return err
+	}
+	return nil
+}
