@@ -8,6 +8,7 @@ package openstack
 import (
 	"fmt"
 	"net/url"
+	"path"
 	"strings"
 	"sync"
 	"time"
@@ -842,21 +843,25 @@ func identityClientVersion(authURL string) (int, error) {
 		// User explicitely did not provide any version, it is empty.
 		return -1, nil
 	}
-	// The last part of the path should be the version #.
+	// The last part of the path should be the version #, prefixed with a 'v' or 'V'
 	// Example: https://keystone.foo:443/v3/
-	versionNumStr := strings.TrimPrefix(strings.ToLower(url.Path), "/v")
-	versionNumStr = strings.TrimSuffix(versionNumStr, "/")
-	if versionNumStr == url.Path || versionNumStr == "" {
-		// If nothing was trimmed, version specification was malformed.
-		// If nothing was left after trim, version number is not provided and,
-		// hence, version specification was malformed.
+	// Example: https://sharedhost.foo:443/identity/v3/
+	urlpath := strings.ToLower(url.Path)
+	urlpath, tail := path.Split(urlpath)
+	if len(tail) == 0 && len(urlpath) > 2 {
+		// trailing /, remove it and split again
+		urlpath, tail = path.Split(strings.TrimRight(urlpath, "/"))
+	}
+	versionNumStr := strings.TrimPrefix(tail, "v")
+	logger.Tracef("authURL: %s", authURL)
+	major, _, err := version.ParseMajorMinor(versionNumStr)
+	if len(tail) < 2 || tail[0] != 'v' || err != nil {
+		// There must be a '/v' in the URL path.
 		// At this stage only '/Vxxx' and '/vxxx' are valid where xxx is major.minor version.
 		// Return 0 as this is the lowest invalid number according to openstack codebase:
 		// -1 is reserved and has special handling; 1, 2, 3, etc are valid identity client versions.
 		return 0, errors.NotValidf("version part of identity url %s", authURL)
 	}
-	logger.Tracef("authURL: %s", authURL)
-	major, _, err := version.ParseMajorMinor(versionNumStr)
 	return major, err
 }
 
@@ -1001,6 +1006,25 @@ func (e *Environ) StartInstance(args environs.StartInstanceParams) (_ *environs.
 		networks = append(networks, nova.ServerNetworks{NetworkId: networkId})
 	}
 
+	machineName := resourceName(
+		e.namespace,
+		e.name,
+		args.InstanceConfig.MachineId,
+	)
+
+	if e.ecfg().useOpenstackGBP() {
+		client := e.neutron()
+		ptArg := neutron.PolicyTargetV2{
+			Name:                fmt.Sprintf("juju-policytarget-%s", machineName),
+			PolicyTargetGroupId: e.ecfg().policyTargetGroup(),
+		}
+		pt, err := client.CreatePolicyTargetV2(ptArg)
+		if err != nil {
+			return nil, errors.Trace(err)
+		}
+		networks = append(networks, nova.ServerNetworks{PortId: pt.PortId})
+	}
+
 	// For BUG 1680787: openstack: add support for neutron networks where port
 	// security is disabled.
 	// If any network specified for instance boot has PortSecurityEnabled equals
@@ -1009,6 +1033,10 @@ func (e *Environ) StartInstance(args environs.StartInstanceParams) (_ *environs.
 	if len(networks) > 0 && e.supportsNeutron() {
 		client := e.neutron()
 		for _, n := range networks {
+			if n.NetworkId == "" {
+				// It's a GBP network.
+				continue
+			}
 			net, err := client.GetNetworkV2(n.NetworkId)
 			if err != nil {
 				return nil, common.ZoneIndependentError(err)
@@ -1041,12 +1069,6 @@ func (e *Environ) StartInstance(args environs.StartInstanceParams) (_ *environs.
 		}
 	}
 
-	machineName := resourceName(
-		e.namespace,
-		e.name,
-		args.InstanceConfig.MachineId,
-	)
-
 	waitForActiveServerDetails := func(
 		client *nova.Client,
 		id string,
