@@ -44,5 +44,7 @@ func (c *defaultConfigurator) GetConfigDefaults() schema.Defaults {
 		"use-default-secgroup": false,
 		"network":              "",
 		"external-network":     "",
+		"use-openstack-gbp":    false,
+		"policy-target-group":  "",
 	}
 }
