@@ -79,15 +79,6 @@ type EnvironAPI interface {
 	StorageAPI
 }
 
-func (o *OracleEnviron) SetEnvironAPI(client EnvironAPI) {
-	if o == nil {
-		return
-	}
-	o.mutex.Lock()
-	defer o.mutex.Unlock()
-	o.client = client
-}
-
 // AvailabilityZones is defined in the common.ZonedEnviron interface
 func (o *OracleEnviron) AvailabilityZones() ([]common.AvailabilityZone, error) {
 	return []common.AvailabilityZone{
@@ -774,7 +765,7 @@ func (o *OracleEnviron) Provider() environs.EnvironProvider {
 }
 
 // PrecheckInstance is part of the environs.Environ interface.
-func (o *OracleEnviron) PrecheckInstance(series string, cons constraints.Value, placement string) error {
+func (o *OracleEnviron) PrecheckInstance(environs.PrecheckInstanceParams) error {
 	return nil
 }
 
