@@ -3,11 +3,13 @@
 
 package oracle
 
-import "github.com/juju/juju/storage"
+import (
+	"github.com/juju/juju/storage"
+)
 
 var (
-	DefaultTypes               = []storage.ProviderType{oracleStorageProvideType}
-	DefaultStorageProviderType = oracleStorageProvideType
+	DefaultTypes               = []storage.ProviderType{DefaultStorageProviderType}
+	DefaultStorageProviderType = oracleStorageProviderType
 	OracleVolumeType           = oracleVolumeType
 	OracleLatencyPool          = latencyPool
 	OracleCloudSchema          = cloudSchema
@@ -20,3 +22,16 @@ var (
 	ParseImageName             = parseImageName
 	CheckImageList             = checkImageList
 )
+
+func SetEnvironAPI(o *OracleEnviron, client EnvironAPI) {
+	if o == nil {
+		return
+	}
+	o.mutex.Lock()
+	defer o.mutex.Unlock()
+	o.client = client
+}
+
+func CreateHostname(o *OracleEnviron, id string) (string, error) {
+	return o.namespace.Hostname(id)
+}
