@@ -4,6 +4,7 @@
 package oracle_test
 
 import (
+	gitjujutesting "github.com/juju/testing"
 	gc "gopkg.in/check.v1"
 
 	"github.com/juju/juju/environs"
@@ -13,7 +14,9 @@ import (
 	"github.com/juju/juju/testing"
 )
 
-type storageSuite struct{}
+type storageSuite struct {
+	gitjujutesting.IsolationSuite
+}
 
 var _ = gc.Suite(&storageSuite{})
 
