@@ -7,21 +7,22 @@ import (
 	"github.com/juju/utils/arch"
 
 	"github.com/juju/juju/constraints"
+	"github.com/juju/juju/environs"
 )
 
 // PrecheckInstance is part of the environs.Environ interface.
-func (env *environ) PrecheckInstance(series string, cons constraints.Value, placement string) error {
-	if placement == "" {
+func (env *environ) PrecheckInstance(args environs.PrecheckInstanceParams) error {
+	if args.Placement == "" {
 		return nil
 	}
 	return env.withSession(func(env *sessionEnviron) error {
-		return env.PrecheckInstance(series, cons, placement)
+		return env.PrecheckInstance(args)
 	})
 }
 
 // PrecheckInstance is part of the environs.Environ interface.
-func (env *sessionEnviron) PrecheckInstance(series string, cons constraints.Value, placement string) error {
-	_, err := env.parsePlacement(placement)
+func (env *sessionEnviron) PrecheckInstance(args environs.PrecheckInstanceParams) error {
+	_, err := env.parsePlacement(args.Placement)
 	return err
 }
 
