@@ -1,8 +1,6 @@
 // Copyright 2015 Canonical Ltd.
 // Licensed under the AGPLv3, see LICENCE file for details.
 
-// +build !gccgo
-
 package vsphere
 
 import (
@@ -33,14 +31,14 @@ func init() {
 	simplestreams.RegisterStructTags(OvaFileMetadata{})
 }
 
-func findImageMetadata(env *environ, args environs.StartInstanceParams) (*OvaFileMetadata, error) {
+func findImageMetadata(env environs.Environ, args environs.StartInstanceParams) (*OvaFileMetadata, error) {
 	arches := args.Tools.Arches()
 	series := args.Tools.OneSeries()
 	ic := &imagemetadata.ImageConstraint{
 		LookupParams: simplestreams.LookupParams{
 			Series: []string{series},
 			Arches: arches,
-			Stream: env.ecfg.ImageStream(),
+			Stream: env.Config().ImageStream(),
 		},
 	}
 	sources, err := environs.ImageMetadataSources(env)
