@@ -5,13 +5,10 @@ package resource_test
 
 import (
 	"strings"
-	"time"
 
 	jc "github.com/juju/testing/checkers"
 	gc "gopkg.in/check.v1"
-	charmresource "gopkg.in/juju/charm.v6-unstable/resource"
-
-	"github.com/juju/juju/resource"
+	charmresource "gopkg.in/juju/charm.v6/resource"
 )
 
 func newFingerprint(c *gc.C, data string) charmresource.Fingerprint {
@@ -34,13 +31,3 @@ func newFullCharmResource(c *gc.C, name string) charmresource.Resource {
 		Fingerprint: newFingerprint(c, name),
 	}
 }
-
-func newFullResource(c *gc.C, name string) resource.Resource {
-	return resource.Resource{
-		Resource:      newFullCharmResource(c, name),
-		ID:            "a-application/" + name,
-		ApplicationID: "a-application",
-		Username:      "a-user",
-		Timestamp:     time.Now(),
-	}
-}
