@@ -20,7 +20,9 @@ import (
 var (
 	logger   = loggo.GetLogger("juju.worker.deployer.service")
 	renderer = &shell.PowershellRenderer{}
+)
 
+const (
 	// c_ERROR_SERVICE_DOES_NOT_EXIST is returned by the OS when trying to open
 	// an inexistent service
 	// https://msdn.microsoft.com/en-us/library/windows/desktop/ms684330%28v=vs.85%29.aspx
@@ -38,10 +40,6 @@ var (
 	// SeAssignPrimaryTokenPrivilege
 	// SeServiceLogonRight
 	jujudUser = ".\\jujud"
-
-	// File containing encrypted password for jujud user.
-	// TODO (gabriel-samfira): migrate this to a registry key
-	jujuPasswdFile = "C:\\Juju\\Jujud.pass"
 )
 
 // IsRunning returns whether or not windows is the local init system.
@@ -75,6 +73,9 @@ type ServiceManager interface {
 	// Exists checks whether the config of the installed service matches the
 	// config supplied to this function
 	Exists(name string, conf common.Conf) (bool, error)
+	// ChangeServicePassword can change the password of a service
+	// as long as it belongs to the user defined in this package
+	ChangeServicePassword(name, newPassword string) error
 }
 
 // Service represents a service running on the current system
@@ -95,7 +96,7 @@ func newService(name string, conf common.Conf, manager ServiceManager) *Service
 
 // NewService returns a new Service type
 func NewService(name string, conf common.Conf) (*Service, error) {
-	m, err := newServiceManager()
+	m, err := NewServiceManager()
 	if err != nil {
 		return nil, errors.Trace(err)
 	}
@@ -215,10 +216,10 @@ func (s *Service) Install() error {
 		return errors.Trace(err)
 	}
 	if installed {
-		return errors.Errorf("Service %s already installed", s.Service.Name)
+		return errors.Errorf("Service %s already installed", s.Name())
 	}
 
-	logger.Infof("Installing Service %v", s.Name)
+	logger.Infof("Installing Service %v", s.Name())
 	err = s.manager.Create(s.Name(), s.Conf())
 	if err != nil {
 		return errors.Trace(err)
@@ -228,7 +229,10 @@ func (s *Service) Install() error {
 
 // InstallCommands returns shell commands to install the service.
 func (s *Service) InstallCommands() ([]string, error) {
-	cmd := fmt.Sprintf(serviceInstallCommands[1:],
+	cmd := fmt.Sprintf(serviceCreateCommandTemplate[1:],
+		renderer.Quote(s.Service.Name),
+		renderer.Quote(s.Service.Conf.Desc),
+		renderer.Quote(s.Service.Conf.ExecStart),
 		renderer.Quote(s.Service.Name),
 		renderer.Quote(s.Service.Conf.Desc),
 		renderer.Quote(s.Service.Conf.ExecStart),
@@ -244,7 +248,11 @@ func (s *Service) StartCommands() ([]string, error) {
 	return []string{cmd}, nil
 }
 
-const serviceInstallCommands = `
-New-Service -Credential $jujuCreds -Name %s -DependsOn Winmgmt -DisplayName %s %s
+const serviceCreateCommandTemplate = `
+if ($jujuCreds) {
+  New-Service -Credential $jujuCreds -Name %s -DependsOn Winmgmt -DisplayName %s %s
+} else {
+  New-Service -Name %s -DependsOn Winmgmt -DisplayName %s %s
+}
 sc.exe failure %s reset=5 actions=restart/1000
 sc.exe failureflag %s 1`
