@@ -10,11 +10,11 @@ import (
 	"unsafe"
 
 	// https://bugs.launchpad.net/juju-core/+bug/1470820
-	"github.com/gabriel-samfira/sys/windows"
-	"github.com/gabriel-samfira/sys/windows/svc"
-	"github.com/gabriel-samfira/sys/windows/svc/mgr"
 	"github.com/juju/errors"
 	"github.com/juju/utils/series"
+	"golang.org/x/sys/windows"
+	"golang.org/x/sys/windows/svc"
+	"golang.org/x/sys/windows/svc/mgr"
 
 	"github.com/juju/juju/service/common"
 )
@@ -358,7 +358,11 @@ func (s *SvcManager) Delete(name string) error {
 func (s *SvcManager) Create(name string, conf common.Conf) error {
 	serviceStartName := "LocalSystem"
 	var passwd string
-	if !series.IsWindowsNano(series.HostSeries()) {
+	hostSeries, err := series.HostSeries()
+	if err != nil {
+		return errors.Trace(err)
+	}
+	if !series.IsWindowsNano(hostSeries) {
 		password, err := getPassword()
 		if err != nil {
 			return errors.Trace(err)
