@@ -698,15 +698,15 @@ func addMachineStorageAttachmentsOps(
 	volumes []volumeAttachmentTemplate,
 	filesystems []filesystemAttachmentTemplate,
 ) ([]txn.Op, error) {
-	var updates bson.D
+	var addToSet bson.D
 	assert := isAliveDoc
 	if len(volumes) > 0 {
 		volumeIds := make([]string, len(volumes))
 		for i, v := range volumes {
 			volumeIds[i] = v.tag.Id()
 		}
-		updates = append(updates, bson.DocElem{"$addToSet", bson.D{{
-			"volumes", bson.D{{"$each", volumeIds}}}},
+		addToSet = append(addToSet, bson.DocElem{
+			"volumes", bson.D{{"$each", volumeIds}},
 		})
 	}
 	if len(filesystems) > 0 {
@@ -723,8 +723,8 @@ func addMachineStorageAttachmentsOps(
 				withLocation = append(withLocation, f)
 			}
 		}
-		updates = append(updates, bson.DocElem{"$addToSet", bson.D{{
-			"filesystems", bson.D{{"$each", filesystemIds}}}},
+		addToSet = append(addToSet, bson.DocElem{
+			"filesystems", bson.D{{"$each", filesystemIds}},
 		})
 		if len(withLocation) > 0 {
 			if err := validateFilesystemMountPoints(machine, withLocation); err != nil {
@@ -740,11 +740,15 @@ func addMachineStorageAttachmentsOps(
 			})
 		}
 	}
+	var update interface{}
+	if len(addToSet) > 0 {
+		update = bson.D{{"$addToSet", addToSet}}
+	}
 	return []txn.Op{{
 		C:      machinesC,
 		Id:     machine.doc.Id,
 		Assert: assert,
-		Update: updates,
+		Update: update,
 	}}, nil
 }
 
@@ -802,8 +806,12 @@ func (st *State) maintainControllersOps(mdocs []*machineDoc, currentInfo *Contro
 			},
 		}},
 		Update: bson.D{
-			{"$addToSet", bson.D{{"machineids", bson.D{{"$each", newIds}}}}},
-			{"$addToSet", bson.D{{"votingmachineids", bson.D{{"$each", newVotingIds}}}}},
+			{"$addToSet",
+				bson.D{
+					{"machineids", bson.D{{"$each", newIds}}},
+					{"votingmachineids", bson.D{{"$each", newVotingIds}}},
+				},
+			},
 		},
 	}}
 	return ops, nil
@@ -815,8 +823,9 @@ func (st *State) maintainControllersOps(mdocs []*machineDoc, currentInfo *Contro
 // If placement is not empty, any new machines which may be required are started
 // according to the specified placement directives until the placement list is
 // exhausted; thereafter any new machines are started according to the constraints and series.
+// MachineID is the id of the machine where the apiserver is running.
 func (st *State) EnableHA(
-	numControllers int, cons constraints.Value, series string, placement []string,
+	numControllers int, cons constraints.Value, series string, placement []string, machineId string,
 ) (ControllersChanges, error) {
 
 	if numControllers < 0 || (numControllers != 0 && numControllers%2 != 1) {
@@ -842,7 +851,7 @@ func (st *State) EnableHA(
 			return nil, errors.New("cannot reduce controller count")
 		}
 
-		intent, err := st.enableHAIntentions(currentInfo, placement)
+		intent, err := st.enableHAIntentions(currentInfo, placement, machineId)
 		if err != nil {
 			return nil, err
 		}
@@ -992,7 +1001,7 @@ type enableHAIntent struct {
 //   demoting unavailable, voting machines;
 //   removing unavailable, non-voting, non-vote-holding machines;
 //   gathering available, non-voting machines that may be promoted;
-func (st *State) enableHAIntentions(info *ControllerInfo, placement []string) (*enableHAIntent, error) {
+func (st *State) enableHAIntentions(info *ControllerInfo, placement []string, machineId string) (*enableHAIntent, error) {
 	var intent enableHAIntent
 	for _, s := range placement {
 		// TODO(natefinch): unscoped placements shouldn't ever get here (though
@@ -1044,17 +1053,25 @@ func (st *State) enableHAIntentions(info *ControllerInfo, placement []string) (*
 			}
 			continue
 		}
-		if m.WantsVote() {
+
+		switch {
+		case m.WantsVote() && m.HasVote() && m.Id() == machineId:
+			// lp:1748275 - Shortly after bootstrap, it's possible that the
+			// controller hasn't pinged with agent presence yet, thereby
+			// failing the controllerAvailable check. So, don't demote the
+			// machine we're running on, if we're here, the agent must be running.
+			intent.maintain = append(intent.maintain, m)
+		case m.WantsVote():
 			// The machine wants to vote, so we simply set novote and allow it
 			// to run its course to have its vote removed by the worker that
 			// maintains the replicaset. We will replace it with an existing
 			// non-voting controller if there is one, starting a new one if
 			// not.
 			intent.demote = append(intent.demote, m)
-		} else if m.HasVote() {
+		case m.HasVote():
 			// The machine still has a vote, so keep it around for now.
 			intent.maintain = append(intent.maintain, m)
-		} else {
+		default:
 			// The machine neither wants to nor has a vote, so remove its
 			// JobManageModel job immediately.
 			intent.remove = append(intent.remove, m)
@@ -1078,8 +1095,10 @@ func convertControllerOps(m *Machine) []txn.Op {
 		C:  controllersC,
 		Id: modelGlobalKey,
 		Update: bson.D{
-			{"$addToSet", bson.D{{"votingmachineids", m.doc.Id}}},
-			{"$addToSet", bson.D{{"machineids", m.doc.Id}}},
+			{"$addToSet", bson.D{
+				{"votingmachineids", m.doc.Id},
+				{"machineids", m.doc.Id},
+			}},
 		},
 	}}
 }
