@@ -12,10 +12,9 @@ import (
 
 	"github.com/juju/errors"
 	jujutxn "github.com/juju/txn"
-	"github.com/juju/utils/featureflag"
 	"github.com/juju/utils/series"
-	"gopkg.in/juju/charm.v6-unstable"
-	csparams "gopkg.in/juju/charmrepo.v2-unstable/csclient/params"
+	"gopkg.in/juju/charm.v6"
+	csparams "gopkg.in/juju/charmrepo.v2/csclient/params"
 	"gopkg.in/juju/names.v2"
 	"gopkg.in/mgo.v2"
 	"gopkg.in/mgo.v2/bson"
@@ -23,7 +22,6 @@ import (
 
 	"github.com/juju/juju/constraints"
 	"github.com/juju/juju/core/leadership"
-	"github.com/juju/juju/feature"
 	"github.com/juju/juju/status"
 )
 
@@ -132,40 +130,69 @@ var errRefresh = stderrors.New("state seems inconsistent, refresh and try again"
 // some point; if the application has no units, and no relation involving the
 // application has any units in scope, they are all removed immediately.
 func (a *Application) Destroy() (err error) {
-	defer errors.DeferredAnnotatef(&err, "cannot destroy application %q", a)
 	defer func() {
 		if err == nil {
 			// This is a white lie; the document might actually be removed.
 			a.doc.Life = Dying
 		}
 	}()
-	app := &Application{st: a.st, doc: a.doc}
-	buildTxn := func(attempt int) ([]txn.Op, error) {
-		if attempt > 0 {
-			if err := app.Refresh(); errors.IsNotFound(err) {
-				return nil, jujutxn.ErrNoOperations
-			} else if err != nil {
-				return nil, err
-			}
-		}
-		switch ops, err := app.destroyOps(); err {
-		case errRefresh:
-		case errAlreadyDying:
+	return a.st.ApplyOperation(a.DestroyOperation())
+}
+
+// DestroyOperation returns a model operation that will destroy the application.
+func (a *Application) DestroyOperation() *DestroyApplicationOperation {
+	return &DestroyApplicationOperation{
+		app: &Application{st: a.st, doc: a.doc},
+	}
+}
+
+// DestroyApplicationOperation is a model operation for destroying an
+// application.
+type DestroyApplicationOperation struct {
+	// unit holds the unit to destroy.
+	app *Application
+
+	// DestroyStorage controls whether or not storage attached
+	// to units of the application are destroyed. If this is false,
+	// then detachable storage will be detached and left in the model.
+	DestroyStorage bool
+
+	// RemoveOffers controls whether or not application offers
+	// are removed. If this is false, then the operation will
+	// fail if there are any offers remaining.
+	RemoveOffers bool
+}
+
+// Build is part of the ModelOperation interface.
+func (op *DestroyApplicationOperation) Build(attempt int) ([]txn.Op, error) {
+	if attempt > 0 {
+		if err := op.app.Refresh(); errors.IsNotFound(err) {
 			return nil, jujutxn.ErrNoOperations
-		case nil:
-			return ops, nil
-		default:
+		} else if err != nil {
 			return nil, err
 		}
-		return nil, jujutxn.ErrTransientFailure
 	}
-	return a.st.run(buildTxn)
+	ops, err := op.app.destroyOps(op.DestroyStorage, op.RemoveOffers)
+	switch err {
+	case errRefresh:
+		return nil, jujutxn.ErrTransientFailure
+	case errAlreadyDying:
+		return nil, jujutxn.ErrNoOperations
+	case nil:
+		return ops, nil
+	}
+	return nil, err
+}
+
+// Done is part of the ModelOperation interface.
+func (op *DestroyApplicationOperation) Done(err error) error {
+	return errors.Annotatef(err, "cannot destroy application %q", op.app)
 }
 
 // destroyOps returns the operations required to destroy the application. If it
 // returns errRefresh, the application should be refreshed and the destruction
 // operations recalculated.
-func (a *Application) destroyOps() ([]txn.Op, error) {
+func (a *Application) destroyOps(destroyStorage, removeOffers bool) ([]txn.Op, error) {
 	if a.doc.Life == Dying {
 		return nil, errAlreadyDying
 	}
@@ -202,6 +229,19 @@ func (a *Application) destroyOps() ([]txn.Op, error) {
 		return nil, errors.Trace(err)
 	}
 	ops = append(ops, resOps...)
+
+	// We can't delete an application if it is being offered.
+	if !removeOffers {
+		countOp, n, err := countApplicationOffersRefOp(a.st, a.Name())
+		if err != nil {
+			return nil, errors.Trace(err)
+		}
+		if n != 0 {
+			return nil, errors.Errorf("application is used by %d offer%s", n, plural(n))
+		}
+		ops = append(ops, countOp)
+	}
+
 	// If the application has no units, and all its known relations will be
 	// removed, the application can also be removed.
 	if a.doc.UnitCount == 0 && a.doc.RelationCount == removeCount {
@@ -228,7 +268,12 @@ func (a *Application) destroyOps() ([]txn.Op, error) {
 	// about is that *some* unit is, or is not, keeping the application from
 	// being removed: the difference between 1 unit and 1000 is irrelevant.
 	if a.doc.UnitCount > 0 {
-		ops = append(ops, newCleanupOp(cleanupUnitsForDyingApplication, a.doc.Name))
+		cleanupOp := newCleanupOp(
+			cleanupUnitsForDyingApplication,
+			a.doc.Name,
+			destroyStorage,
+		)
+		ops = append(ops, cleanupOp)
 		notLastRefs = append(notLastRefs, bson.D{{"unitcount", bson.D{{"$gt", 0}}}}...)
 	} else {
 		notLastRefs = append(notLastRefs, bson.D{{"unitcount", 0}}...)
@@ -265,14 +310,20 @@ func removeResourcesOps(st *State, applicationID string) ([]txn.Op, error) {
 // removeOps returns the operations required to remove the application. Supplied
 // asserts will be included in the operation on the application document.
 func (a *Application) removeOps(asserts bson.D) ([]txn.Op, error) {
-	ops := []txn.Op{
-		{
-			C:      applicationsC,
-			Id:     a.doc.DocID,
-			Assert: asserts,
-			Remove: true,
-		},
+	ops := []txn.Op{{
+		C:      applicationsC,
+		Id:     a.doc.DocID,
+		Assert: asserts,
+		Remove: true,
+	}}
+
+	// Remove application offers.
+	removeOfferOps, err := removeApplicationOffersOps(a.st, a.doc.Name)
+	if err != nil {
+		return nil, errors.Trace(err)
 	}
+	ops = append(ops, removeOfferOps...)
+
 	// Note that appCharmDecRefOps might not catch the final decref
 	// when run in a transaction that decrefs more than once. So we
 	// avoid attempting to do the final cleanup in the ref dec ops and
@@ -291,7 +342,7 @@ func (a *Application) removeOps(asserts bson.D) ([]txn.Op, error) {
 	globalKey := a.globalKey()
 	ops = append(ops,
 		removeEndpointBindingsOp(globalKey),
-		removeConstraintsOp(a.st, globalKey),
+		removeConstraintsOp(globalKey),
 		annotationRemoveOp(a.st, globalKey),
 		removeLeadershipSettingsOp(name),
 		removeStatusOp(a.st, globalKey),
@@ -326,7 +377,7 @@ func (a *Application) setExposed(exposed bool) (err error) {
 		Assert: isAliveDoc,
 		Update: bson.D{{"$set", bson.D{{"exposed", exposed}}}},
 	}}
-	if err := a.st.runTransaction(ops); err != nil {
+	if err := a.st.db().RunTransaction(ops); err != nil {
 		return errors.Errorf("cannot set exposed flag for application %q to %v: %v", a, exposed, onAbort(err, errNotAlive))
 	}
 	a.doc.Exposed = exposed
@@ -474,6 +525,12 @@ func (a *Application) checkRelationsOps(ch *Charm, relations []*Relation) ([]txn
 
 func (a *Application) checkStorageUpgrade(newMeta, oldMeta *charm.Meta, units []*Unit) (_ []txn.Op, err error) {
 	// Make sure no storage instances are added or removed.
+
+	im, err := a.st.IAASModel()
+	if err != nil {
+		return nil, errors.Trace(err)
+	}
+
 	var ops []txn.Op
 	for name, oldStorageMeta := range oldMeta.Storage {
 		if _, ok := newMeta.Storage[name]; ok {
@@ -486,7 +543,7 @@ func (a *Application) checkStorageUpgrade(newMeta, oldMeta *charm.Meta, units []
 		// are no instances of the store, it can safely be
 		// removed.
 		if oldStorageMeta.Shared {
-			op, n, err := a.st.countEntityStorageInstances(a.Tag(), name)
+			op, n, err := im.countEntityStorageInstances(a.Tag(), name)
 			if err != nil {
 				return nil, errors.Trace(err)
 			}
@@ -496,7 +553,7 @@ func (a *Application) checkStorageUpgrade(newMeta, oldMeta *charm.Meta, units []
 			ops = append(ops, op)
 		} else {
 			for _, u := range units {
-				op, n, err := a.st.countEntityStorageInstances(u.Tag(), name)
+				op, n, err := im.countEntityStorageInstances(u.Tag(), name)
 				if err != nil {
 					return nil, errors.Trace(err)
 				}
@@ -632,6 +689,10 @@ func (a *Application) changeCharmOps(
 	// constraints, as incompatible charm changes will otherwise yield
 	// confusing error messages that would suggest the user has supplied
 	// invalid constraints.
+	im, err := a.st.IAASModel()
+	if err != nil {
+		return nil, errors.Trace(err)
+	}
 	oldCharm, _, err := a.Charm()
 	if err != nil {
 		return nil, errors.Trace(err)
@@ -659,14 +720,14 @@ func (a *Application) changeCharmOps(
 			delete(newStorageConstraints, name)
 		}
 	}
-	if err := addDefaultStorageConstraints(a.st, newStorageConstraints, ch.Meta()); err != nil {
+	if err := addDefaultStorageConstraints(im, newStorageConstraints, ch.Meta()); err != nil {
 		return nil, errors.Annotate(err, "adding default storage constraints")
 	}
-	if err := validateStorageConstraints(a.st, newStorageConstraints, ch.Meta()); err != nil {
+	if err := validateStorageConstraints(im, newStorageConstraints, ch.Meta()); err != nil {
 		return nil, errors.Annotate(err, "validating storage constraints")
 	}
 	newStorageConstraintsKey := applicationStorageConstraintsKey(a.doc.Name, ch.URL())
-	if _, err := readStorageConstraints(a.st, newStorageConstraintsKey); errors.IsNotFound(err) {
+	if _, err := readStorageConstraints(im.mb, newStorageConstraintsKey); errors.IsNotFound(err) {
 		storageConstraintsOp = createStorageConstraintsOp(
 			newStorageConstraintsKey, newStorageConstraints,
 		)
@@ -791,6 +852,12 @@ func (a *Application) upgradeStorageOps(
 	units []*Unit,
 	allStorageCons map[string]StorageConstraints,
 ) (_ []txn.Op, err error) {
+
+	im, err := a.st.IAASModel()
+	if err != nil {
+		return nil, errors.Trace(err)
+	}
+
 	// For each store, ensure that every unit has the minimum requirements.
 	// If a unit has an existing store, but its minimum count has been
 	// increased, we only add the shortfall; we do not necessarily add as
@@ -805,7 +872,7 @@ func (a *Application) upgradeStorageOps(
 				// cosntraints.
 				countMin = int(cons.Count)
 			}
-			unitOps, err := a.st.addUnitStorageOps(
+			_, unitOps, err := im.addUnitStorageOps(
 				meta, u, name, cons, countMin,
 			)
 			if err != nil {
@@ -995,7 +1062,7 @@ func (a *Application) SetCharm(cfg SetCharmConfig) (err error) {
 
 		return ops, nil
 	}
-	if err := a.st.run(buildTxn); err != nil {
+	if err := a.st.db().Run(buildTxn); err != nil {
 		return err
 	}
 	a.doc.CharmURL = cfg.Charm.URL()
@@ -1005,6 +1072,102 @@ func (a *Application) SetCharm(cfg SetCharmConfig) (err error) {
 	return nil
 }
 
+// UpdateApplicationSeries updates the series for the Application.
+func (a *Application) UpdateApplicationSeries(series string, force bool) (err error) {
+	buildTxn := func(attempt int) ([]txn.Op, error) {
+		if attempt > 0 {
+			// If we've tried once already and failed, re-evaluate the criteria.
+			if err := a.Refresh(); err != nil {
+				return nil, errors.Trace(err)
+			}
+		}
+		// Exit early if the Application series doesn't need to change.
+		if a.Series() == series {
+			return nil, jujutxn.ErrNoOperations
+		}
+
+		// Verify and gather data for the transaction operations.
+		err := a.VerifySupportedSeries(series, force)
+		if err != nil {
+			return nil, err
+		}
+		units, err := a.AllUnits()
+		if err != nil {
+			return nil, errors.Trace(err)
+		}
+		var subApps []*Application
+		var unit *Unit
+
+		if len(units) > 0 {
+			// All units have the same subordinates...
+			unit = units[0]
+			for _, n := range unit.SubordinateNames() {
+				app, err := a.st.Application(unitAppName(n))
+				if err != nil {
+					return nil, err
+				}
+				err = app.VerifySupportedSeries(series, force)
+				if err != nil {
+					return nil, err
+				}
+				subApps = append(subApps, app)
+			}
+		}
+
+		//Create the transaction operations
+		ops := []txn.Op{{
+			C:  applicationsC,
+			Id: a.doc.DocID,
+			Assert: bson.D{{"life", Alive},
+				{"charmurl", a.doc.CharmURL},
+				{"unitcount", a.doc.UnitCount}},
+			Update: bson.D{{"$set", bson.D{{"series", series}}}},
+		}}
+		if err != nil {
+			return nil, errors.Trace(err)
+		}
+		if unit != nil {
+			ops = append(ops, txn.Op{
+				C:  unitsC,
+				Id: unit.doc.DocID,
+				Assert: bson.D{{"life", Alive},
+					{"subordinates", unit.SubordinateNames()}},
+			})
+		}
+		for _, sub := range subApps {
+			ops = append(ops, txn.Op{
+				C:  applicationsC,
+				Id: sub.doc.DocID,
+				Assert: bson.D{{"life", Alive},
+					{"charmurl", sub.doc.CharmURL},
+					{"unitcount", sub.doc.UnitCount}},
+				Update: bson.D{{"$set", bson.D{{"series", series}}}},
+			})
+		}
+		return ops, nil
+	}
+
+	err = a.st.db().Run(buildTxn)
+	return errors.Annotatef(err, "cannot update series for %q to %s", a, series)
+}
+
+// VerifySupportedSeries verifies if the given series is supported by the
+// application.
+func (a *Application) VerifySupportedSeries(series string, force bool) error {
+	ch, _, err := a.Charm()
+	if err != nil {
+		return err
+	}
+	_, seriesSupportedErr := charm.SeriesForCharm(series, ch.Meta().Series)
+	if seriesSupportedErr != nil && !force {
+		return &ErrIncompatibleSeries{
+			SeriesList: ch.Meta().Series,
+			Series:     series,
+		}
+	}
+	return nil
+}
+
 // String returns the application name.
 func (a *Application) String() string {
 	return a.doc.Name
@@ -1029,7 +1192,7 @@ func (a *Application) Refresh() error {
 
 // newUnitName returns the next unit name.
 func (a *Application) newUnitName() (string, error) {
-	unitSeq, err := a.st.sequence(a.Tag().String())
+	unitSeq, err := sequence(a.st, a.Tag().String())
 	if err != nil {
 		return "", errors.Trace(err)
 	}
@@ -1043,7 +1206,11 @@ func (a *Application) newUnitName() (string, error) {
 // application will be assigned to a given principal. The asserts param can be used
 // to include additional assertions for the application document.  This method
 // assumes that the application already exists in the db.
-func (a *Application) addUnitOps(principalName string, asserts bson.D) (string, []txn.Op, error) {
+func (a *Application) addUnitOps(
+	principalName string,
+	args AddUnitParams,
+	asserts bson.D,
+) (string, []txn.Op, error) {
 	var cons constraints.Value
 	if !a.doc.Subordinate {
 		scons, err := a.Constraints()
@@ -1062,12 +1229,12 @@ func (a *Application) addUnitOps(principalName string, asserts bson.D) (string,
 	if err != nil {
 		return "", nil, err
 	}
-	args := applicationAddUnitOpsArgs{
+	names, ops, err := a.addUnitOpsWithCons(applicationAddUnitOpsArgs{
 		cons:          cons,
 		principalName: principalName,
 		storageCons:   storageCons,
-	}
-	names, ops, err := a.addUnitOpsWithCons(args)
+		attachStorage: args.AttachStorage,
+	})
 	if err != nil {
 		return names, ops, err
 	}
@@ -1081,6 +1248,7 @@ type applicationAddUnitOpsArgs struct {
 	principalName string
 	cons          constraints.Value
 	storageCons   map[string]StorageConstraints
+	attachStorage []names.StorageTag
 }
 
 // addApplicationUnitOps is just like addUnitOps but explicitly takes a
@@ -1111,6 +1279,35 @@ func (a *Application) addUnitOpsWithCons(args applicationAddUnitOpsArgs) (string
 		return "", nil, err
 	}
 
+	im, err := a.st.IAASModel()
+	if err != nil {
+		return "", nil, errors.Trace(err)
+	}
+
+	// Reduce the count of new storage created for each existing storage
+	// being attached.
+	var storageCons map[string]StorageConstraints
+	for _, tag := range args.attachStorage {
+		storageName, err := names.StorageName(tag.Id())
+		if err != nil {
+			return "", nil, errors.Trace(err)
+		}
+		if cons, ok := args.storageCons[storageName]; ok && cons.Count > 0 {
+			if storageCons == nil {
+				// We must not modify the conents of the original
+				// args.storageCons map, as it comes from the
+				// user. Make a copy and modify that.
+				storageCons = make(map[string]StorageConstraints)
+				for name, cons := range args.storageCons {
+					storageCons[name] = cons
+				}
+				args.storageCons = storageCons
+			}
+			cons.Count--
+			storageCons[storageName] = cons
+		}
+	}
+
 	// Add storage instances/attachments for the unit. If the
 	// application is subordinate, we'll add the machine storage
 	// if the principal is assigned to a machine. Otherwise, we
@@ -1124,8 +1321,8 @@ func (a *Application) addUnitOpsWithCons(args applicationAddUnitOpsArgs) (string
 		}
 		machineAssignable = pu
 	}
-	storageOps, numStorageAttachments, err := createStorageOps(
-		a.st,
+	storageOps, storageTags, numStorageAttachments, err := createStorageOps(
+		im,
 		unitTag,
 		charm.Meta(),
 		args.storageCons,
@@ -1135,6 +1332,40 @@ func (a *Application) addUnitOpsWithCons(args applicationAddUnitOpsArgs) (string
 	if err != nil {
 		return "", nil, errors.Trace(err)
 	}
+	for _, storageTag := range args.attachStorage {
+		si, err := im.storageInstance(storageTag)
+		if err != nil {
+			return "", nil, errors.Annotatef(
+				err, "attaching %s",
+				names.ReadableString(storageTag),
+			)
+		}
+		ops, err := im.attachStorageOps(
+			si,
+			unitTag,
+			a.doc.Series,
+			charm,
+			machineAssignable,
+		)
+		if err != nil {
+			return "", nil, errors.Trace(err)
+		}
+		storageOps = append(storageOps, ops...)
+		numStorageAttachments++
+		storageTags[si.StorageName()] = append(storageTags[si.StorageName()], storageTag)
+	}
+	for name, tags := range storageTags {
+		count := len(tags)
+		charmStorage := charm.Meta().Storage[name]
+		if err := validateCharmStorageCountChange(charmStorage, 0, count); err != nil {
+			return "", nil, errors.Trace(err)
+		}
+		incRefOp, err := increfEntityStorageOp(a.st, unitTag, name, count)
+		if err != nil {
+			return "", nil, errors.Trace(err)
+		}
+		storageOps = append(storageOps, incRefOp)
+	}
 
 	docID := a.st.docID(name)
 	globalKey := unitGlobalKey(name)
@@ -1148,7 +1379,7 @@ func (a *Application) addUnitOpsWithCons(args applicationAddUnitOpsArgs) (string
 		Principal:              args.principalName,
 		StorageAttachmentCount: numStorageAttachments,
 	}
-	now := a.st.clock.Now()
+	now := a.st.clock().Now()
 	agentStatusDoc := statusDoc{
 		Status:  status.Allocating,
 		Updated: now.UnixNano(),
@@ -1186,7 +1417,7 @@ func (a *Application) addUnitOpsWithCons(args applicationAddUnitOpsArgs) (string
 			Update: bson.D{{"$addToSet", bson.D{{"subordinates", name}}}},
 		})
 	} else {
-		ops = append(ops, createConstraintsOp(a.st, agentGlobalKey, args.cons))
+		ops = append(ops, createConstraintsOp(agentGlobalKey, args.cons))
 	}
 
 	// At the last moment we still have the statusDocs in scope, set the initial
@@ -1199,6 +1430,45 @@ func (a *Application) addUnitOpsWithCons(args applicationAddUnitOpsArgs) (string
 	return name, ops, nil
 }
 
+// applicationOffersRefCountKey returns a key for refcounting offers
+// for the specified application. Each time an offer is created, the
+// refcount is incremented, and the opposite happens on removal.
+func applicationOffersRefCountKey(appName string) string {
+	return fmt.Sprintf("offer#%s", appName)
+}
+
+// incApplicationOffersRefOp returns a txn.Op that increments the reference
+// count for an application offer.
+func incApplicationOffersRefOp(mb modelBackend, appName string) (txn.Op, error) {
+	refcounts, closer := mb.db().GetCollection(refcountsC)
+	defer closer()
+	offerRefCountKey := applicationOffersRefCountKey(appName)
+	incRefOp, err := nsRefcounts.CreateOrIncRefOp(refcounts, offerRefCountKey, 1)
+	return incRefOp, errors.Trace(err)
+}
+
+// countApplicationOffersRefOp returns the number of offers for an application,
+// along with a txn.Op that ensures that that does not change.
+func countApplicationOffersRefOp(mb modelBackend, appName string) (txn.Op, int, error) {
+	refcounts, closer := mb.db().GetCollection(refcountsC)
+	defer closer()
+	key := applicationOffersRefCountKey(appName)
+	return nsRefcounts.CurrentOp(refcounts, key)
+}
+
+// decApplicationOffersRefOp returns a txn.Op that decrements the reference
+// count for an application offer.
+func decApplicationOffersRefOp(mb modelBackend, appName string) (txn.Op, error) {
+	refcounts, closer := mb.db().GetCollection(refcountsC)
+	defer closer()
+	offerRefCountKey := applicationOffersRefCountKey(appName)
+	decRefOp, _, err := nsRefcounts.DyingDecRefOp(refcounts, offerRefCountKey)
+	if err != nil {
+		return txn.Op{}, errors.Trace(err)
+	}
+	return decRefOp, nil
+}
+
 // incUnitCountOp returns the operation to increment the application's unit count.
 func (a *Application) incUnitCountOp(asserts bson.D) txn.Op {
 	op := txn.Op{
@@ -1212,15 +1482,21 @@ func (a *Application) incUnitCountOp(asserts bson.D) txn.Op {
 	return op
 }
 
+// AddUnitParams contains parameters for the Application.AddUnit method.
+type AddUnitParams struct {
+	// AttachStorage identifies storage instances to attach to the unit.
+	AttachStorage []names.StorageTag
+}
+
 // AddUnit adds a new principal unit to the application.
-func (a *Application) AddUnit() (unit *Unit, err error) {
+func (a *Application) AddUnit(args AddUnitParams) (unit *Unit, err error) {
 	defer errors.DeferredAnnotatef(&err, "cannot add unit to application %q", a)
-	name, ops, err := a.addUnitOps("", nil)
+	name, ops, err := a.addUnitOps("", args, nil)
 	if err != nil {
 		return nil, err
 	}
 
-	if err := a.st.runTransaction(ops); err == txn.ErrAborted {
+	if err := a.st.db().RunTransaction(ops); err == txn.ErrAborted {
 		if alive, err := isAlive(a.st, applicationsC, a.doc.DocID); err != nil {
 			return nil, err
 		} else if !alive {
@@ -1236,7 +1512,7 @@ func (a *Application) AddUnit() (unit *Unit, err error) {
 // removeUnitOps returns the operations necessary to remove the supplied unit,
 // assuming the supplied asserts apply to the unit document.
 func (a *Application) removeUnitOps(u *Unit, asserts bson.D) ([]txn.Op, error) {
-	ops, err := u.destroyHostOps(a)
+	hostOps, err := u.destroyHostOps(a)
 	if err != nil {
 		return nil, err
 	}
@@ -1244,7 +1520,11 @@ func (a *Application) removeUnitOps(u *Unit, asserts bson.D) ([]txn.Op, error) {
 	if err != nil {
 		return nil, err
 	}
-	storageInstanceOps, err := removeStorageInstancesOps(a.st, u.Tag())
+	im, err := a.st.IAASModel()
+	if err != nil {
+		return nil, err
+	}
+	storageInstanceOps, err := removeStorageInstancesOps(im, u.Tag())
 	if err != nil {
 		return nil, err
 	}
@@ -1252,14 +1532,13 @@ func (a *Application) removeUnitOps(u *Unit, asserts bson.D) ([]txn.Op, error) {
 	if err != nil {
 		return nil, errors.Trace(err)
 	}
-	ops = append(ops, resOps...)
 
 	observedFieldsMatch := bson.D{
 		{"charmurl", u.doc.CharmURL},
 		{"machineid", u.doc.MachineId},
 	}
-	ops = append(ops,
-		txn.Op{
+	ops := []txn.Op{
+		{
 			C:      unitsC,
 			Id:     u.doc.DocID,
 			Assert: append(observedFieldsMatch, asserts...),
@@ -1268,12 +1547,15 @@ func (a *Application) removeUnitOps(u *Unit, asserts bson.D) ([]txn.Op, error) {
 		removeMeterStatusOp(a.st, u.globalMeterStatusKey()),
 		removeStatusOp(a.st, u.globalAgentKey()),
 		removeStatusOp(a.st, u.globalKey()),
-		removeConstraintsOp(a.st, u.globalAgentKey()),
+		removeConstraintsOp(u.globalAgentKey()),
 		annotationRemoveOp(a.st, u.globalKey()),
 		newCleanupOp(cleanupRemovedUnit, u.doc.Name),
-	)
+	}
 	ops = append(ops, portsOps...)
 	ops = append(ops, storageInstanceOps...)
+	ops = append(ops, resOps...)
+	ops = append(ops, hostOps...)
+
 	if u.doc.CharmURL != nil {
 		// If the unit has a different URL to the application, allow any final
 		// cleanup to happen; otherwise we just do it when the app itself is removed.
@@ -1497,7 +1779,7 @@ func (a *Application) UpdateLeaderSettings(token leadership.Token, updates map[s
 			Update: update,
 		}}, nil
 	}
-	return a.st.run(buildTxnWithLeadership(buildTxn, token))
+	return a.st.db().Run(buildTxnWithLeadership(buildTxn, token))
 }
 
 var ErrSubordinateConstraints = stderrors.New("constraints do not apply to subordinate applications")
@@ -1531,8 +1813,8 @@ func (a *Application) SetConstraints(cons constraints.Value) (err error) {
 		Id:     a.doc.DocID,
 		Assert: isAliveDoc,
 	}}
-	ops = append(ops, setConstraintsOp(a.st, a.globalKey(), cons))
-	return onAbort(a.st.runTransaction(ops), errNotAlive)
+	ops = append(ops, setConstraintsOp(a.globalKey(), cons))
+	return onAbort(a.st.db().RunTransaction(ops), errNotAlive)
 }
 
 // EndpointBindings returns the mapping for each endpoint name and the space
@@ -1595,7 +1877,7 @@ func (a *Application) SetMetricCredentials(b []byte) error {
 		}
 		return ops, nil
 	}
-	if err := a.st.run(buildTxn); err != nil {
+	if err := a.st.db().Run(buildTxn); err != nil {
 		if err == errNotAlive {
 			return errors.New("cannot update metric credentials: application " + err.Error())
 		}
@@ -1668,7 +1950,7 @@ func (a *Application) SetStatus(statusInfo status.StatusInfo) error {
 		status:    statusInfo.Status,
 		message:   statusInfo.Message,
 		rawData:   statusInfo.Data,
-		updated:   timeOrNow(statusInfo.Since, a.st.clock),
+		updated:   timeOrNow(statusInfo.Since, a.st.clock()),
 	})
 }
 
@@ -1748,8 +2030,8 @@ type addApplicationOpsArgs struct {
 // applications collection, along with all the associated expected other application
 // entries. This method is used by both the *State.AddApplication method and the
 // migration import code.
-func addApplicationOps(st *State, app *Application, args addApplicationOpsArgs) ([]txn.Op, error) {
-	charmRefOps, err := appCharmIncRefOps(st, args.applicationDoc.Name, args.applicationDoc.CharmURL, true)
+func addApplicationOps(mb modelBackend, app *Application, args addApplicationOpsArgs) ([]txn.Op, error) {
+	charmRefOps, err := appCharmIncRefOps(mb, args.applicationDoc.Name, args.applicationDoc.CharmURL, true)
 	if err != nil {
 		return nil, errors.Trace(err)
 	}
@@ -1760,12 +2042,12 @@ func addApplicationOps(st *State, app *Application, args addApplicationOpsArgs)
 	leadershipKey := leadershipSettingsKey(app.Name())
 
 	ops := []txn.Op{
-		createConstraintsOp(st, globalKey, args.constraints),
+		createConstraintsOp(globalKey, args.constraints),
 		createStorageConstraintsOp(storageConstraintsKey, args.storage),
 		createSettingsOp(settingsC, settingsKey, args.settings),
 		createSettingsOp(settingsC, leadershipKey, args.leadershipSettings),
-		createStatusOp(st, globalKey, args.statusDoc),
-		addModelApplicationRefOp(st, app.Name()),
+		createStatusOp(mb, globalKey, args.statusDoc),
+		addModelApplicationRefOp(mb, app.Name()),
 	}
 	ops = append(ops, charmRefOps...)
 	ops = append(ops, txn.Op{
@@ -1774,12 +2056,10 @@ func addApplicationOps(st *State, app *Application, args addApplicationOpsArgs)
 		Assert: txn.DocMissing,
 		Insert: args.applicationDoc,
 	})
-	if featureflag.Enabled(feature.CrossModelRelations) {
-		ops = append(ops, txn.Op{
-			C:      remoteApplicationsC,
-			Id:     app.Name(),
-			Assert: txn.DocMissing,
-		})
-	}
+	ops = append(ops, txn.Op{
+		C:      remoteApplicationsC,
+		Id:     app.Name(),
+		Assert: txn.DocMissing,
+	})
 	return ops, nil
 }
