@@ -9,10 +9,10 @@ import (
 	"github.com/juju/errors"
 	"gopkg.in/juju/charm.v6"
 	"gopkg.in/juju/names.v2"
-	"gopkg.in/mgo.v2"
 	"gopkg.in/mgo.v2/bson"
 	"gopkg.in/mgo.v2/txn"
 
+	"github.com/juju/juju/mongo"
 	"github.com/juju/juju/status"
 )
 
@@ -624,7 +624,7 @@ func (st *State) cleanupForceDestroyedMachine(machineId string) error {
 		return err
 	}
 	removePortsOps, err := machine.removePortsOps()
-	if err != nil {
+	if len(removePortsOps) == 0 || err != nil {
 		return err
 	}
 	return st.db().RunTransaction(removePortsOps)
@@ -906,7 +906,7 @@ func (st *State) cleanupAttachmentsForDyingFilesystem(filesystemId string) (err
 	return nil
 }
 
-func closeIter(iter *mgo.Iter, errOut *error, message string) {
+func closeIter(iter mongo.Iterator, errOut *error, message string) {
 	err := iter.Close()
 	if err == nil {
 		return
