@@ -9,7 +9,7 @@ import (
 	"github.com/juju/errors"
 	jc "github.com/juju/testing/checkers"
 	gc "gopkg.in/check.v1"
-	"gopkg.in/juju/charm.v6-unstable"
+	"gopkg.in/juju/charm.v6"
 	"gopkg.in/juju/names.v2"
 
 	"github.com/juju/juju/instance"
@@ -32,10 +32,10 @@ func (s *CleanupSuite) SetUpTest(c *gc.C) {
 
 func (s *CleanupSuite) TestCleanupDyingApplicationUnits(c *gc.C) {
 	// Create a application with some units.
-	mysql := s.AddTestingService(c, "mysql", s.AddTestingCharm(c, "mysql"))
+	mysql := s.AddTestingApplication(c, "mysql", s.AddTestingCharm(c, "mysql"))
 	units := make([]*state.Unit, 3)
 	for i := range units {
-		unit, err := mysql.AddUnit()
+		unit, err := mysql.AddUnit(state.AddUnitParams{})
 		c.Assert(err, jc.ErrorIsNil)
 		units[i] = unit
 	}
@@ -70,7 +70,7 @@ func (s *CleanupSuite) TestCleanupDyingApplicationUnits(c *gc.C) {
 func (s *CleanupSuite) TestCleanupDyingApplicationCharm(c *gc.C) {
 	// Create a application and a charm.
 	ch := s.AddTestingCharm(c, "mysql")
-	mysql := s.AddTestingService(c, "mysql", ch)
+	mysql := s.AddTestingApplication(c, "mysql", ch)
 
 	// Create a dummy archive blob.
 	stor := storage.NewStorage(s.State.ModelUUID(), s.State.MongoSession())
@@ -116,13 +116,13 @@ func (s *CleanupSuite) TestCleanupRemoteApplicationWithRelations(c *gc.C) {
 	}
 	remoteApp, err := s.State.AddRemoteApplication(state.AddRemoteApplicationParams{
 		Name:        "mysql",
-		SourceModel: s.State.ModelTag(),
+		SourceModel: s.IAASModel.ModelTag(),
 		Token:       "t0",
 		Endpoints:   mysqlEps,
 	})
 	c.Assert(err, jc.ErrorIsNil)
 
-	s.AddTestingService(c, "wordpress", s.AddTestingCharm(c, "wordpress"))
+	s.AddTestingApplication(c, "wordpress", s.AddTestingCharm(c, "wordpress"))
 	eps, err := s.State.InferEndpoints("wordpress", "mysql")
 	c.Assert(err, jc.ErrorIsNil)
 	_, err = s.State.AddRelation(eps[0], eps[1])
@@ -152,9 +152,11 @@ func (s *CleanupSuite) TestCleanupControllerModels(c *gc.C) {
 
 	// Destroy the controller and check the model is unaffected, but a
 	// cleanup for the model and applications has been scheduled.
-	controllerEnv, err := s.State.Model()
+	controllerModel, err := s.State.Model()
 	c.Assert(err, jc.ErrorIsNil)
-	err = controllerEnv.DestroyIncludingHosted()
+	err = controllerModel.Destroy(state.DestroyModelParams{
+		DestroyHostedModels: true,
+	})
 	c.Assert(err, jc.ErrorIsNil)
 
 	// Two cleanups should be scheduled. One to destroy the hosted
@@ -194,9 +196,9 @@ func (s *CleanupSuite) TestCleanupModelMachines(c *gc.C) {
 	s.assertDoesNotNeedCleanup(c)
 
 	// Destroy model, check cleanup queued.
-	env, err := s.State.Model()
+	model, err := s.State.Model()
 	c.Assert(err, jc.ErrorIsNil)
-	err = env.Destroy()
+	err = model.Destroy(state.DestroyModelParams{})
 	c.Assert(err, jc.ErrorIsNil)
 	s.assertNeedsCleanup(c)
 
@@ -218,10 +220,10 @@ func (s *CleanupSuite) TestCleanupModelApplications(c *gc.C) {
 	s.assertDoesNotNeedCleanup(c)
 
 	// Create a application with some units.
-	mysql := s.AddTestingService(c, "mysql", s.AddTestingCharm(c, "mysql"))
+	mysql := s.AddTestingApplication(c, "mysql", s.AddTestingCharm(c, "mysql"))
 	units := make([]*state.Unit, 3)
 	for i := range units {
-		unit, err := mysql.AddUnit()
+		unit, err := mysql.AddUnit(state.AddUnitParams{})
 		c.Assert(err, jc.ErrorIsNil)
 		units[i] = unit
 	}
@@ -231,7 +233,7 @@ func (s *CleanupSuite) TestCleanupModelApplications(c *gc.C) {
 	// unaffected, but a cleanup for the application has been scheduled.
 	model, err := s.State.Model()
 	c.Assert(err, jc.ErrorIsNil)
-	err = model.Destroy()
+	err = model.Destroy(state.DestroyModelParams{})
 	c.Assert(err, jc.ErrorIsNil)
 	s.assertNeedsCleanup(c)
 	s.assertCleanupRuns(c)
@@ -341,8 +343,8 @@ func (s *CleanupSuite) TestCleanupForceDestroyMachineCleansStorageAttachments(c
 	storage := map[string]state.StorageConstraints{
 		"data": makeStorageCons("loop", 1024, 1),
 	}
-	application := s.AddTestingServiceWithStorage(c, "storage-block", ch, storage)
-	u, err := application.AddUnit()
+	application := s.AddTestingApplicationWithStorage(c, "storage-block", ch, storage)
+	u, err := application.AddUnit(state.AddUnitParams{})
 	c.Assert(err, jc.ErrorIsNil)
 	err = u.AssignToMachine(machine)
 	c.Assert(err, jc.ErrorIsNil)
@@ -353,7 +355,7 @@ func (s *CleanupSuite) TestCleanupForceDestroyMachineCleansStorageAttachments(c
 	// this tag matches the storage instance created for the unit above.
 	storageTag := names.NewStorageTag("data/0")
 
-	sa, err := s.State.StorageAttachment(storageTag, u.UnitTag())
+	sa, err := s.IAASModel.StorageAttachment(storageTag, u.UnitTag())
 	c.Assert(err, jc.ErrorIsNil)
 	c.Assert(sa.Life(), gc.Equals, state.Alive)
 
@@ -362,12 +364,15 @@ func (s *CleanupSuite) TestCleanupForceDestroyMachineCleansStorageAttachments(c
 	c.Assert(err, jc.ErrorIsNil)
 	s.assertCleanupCount(c, 2)
 
-	// After running the cleanup, the storage attachment and instance
-	// should both be removed.
-	_, err = s.State.StorageAttachment(storageTag, u.UnitTag())
-	c.Assert(err, jc.Satisfies, errors.IsNotFound)
-	_, err = s.State.StorageInstance(storageTag)
+	// After running the cleanups, the storage attachment should
+	// have been removed; the storage instance should be floating,
+	// and will be removed along with the machine.
+	_, err = s.IAASModel.StorageAttachment(storageTag, u.UnitTag())
 	c.Assert(err, jc.Satisfies, errors.IsNotFound)
+	si, err := s.IAASModel.StorageInstance(storageTag)
+	c.Assert(err, jc.ErrorIsNil)
+	_, hasOwner := si.Owner()
+	c.Assert(hasOwner, jc.IsFalse)
 
 	// Check that the unit has been removed.
 	assertRemoved(c, u)
@@ -503,8 +508,8 @@ func (s *CleanupSuite) TestCleanupDyingUnitAlreadyRemoved(c *gc.C) {
 
 func (s *CleanupSuite) TestCleanupActions(c *gc.C) {
 	// Create a application with a unit.
-	dummy := s.AddTestingService(c, "dummy", s.AddTestingCharm(c, "dummy"))
-	unit, err := dummy.AddUnit()
+	dummy := s.AddTestingApplication(c, "dummy", s.AddTestingCharm(c, "dummy"))
+	unit, err := dummy.AddUnit(state.AddUnitParams{})
 	c.Assert(err, jc.ErrorIsNil)
 
 	// check no cleanups
@@ -545,8 +550,8 @@ func (s *CleanupSuite) TestCleanupActions(c *gc.C) {
 
 func (s *CleanupSuite) TestCleanupWithCompletedActions(c *gc.C) {
 	// Create a application with a unit.
-	dummy := s.AddTestingService(c, "dummy", s.AddTestingCharm(c, "dummy"))
-	unit, err := dummy.AddUnit()
+	dummy := s.AddTestingApplication(c, "dummy", s.AddTestingCharm(c, "dummy"))
+	unit, err := dummy.AddUnit(state.AddUnitParams{})
 	c.Assert(err, jc.ErrorIsNil)
 	s.assertDoesNotNeedCleanup(c)
 
@@ -578,8 +583,8 @@ func (s *CleanupSuite) TestCleanupStorageAttachments(c *gc.C) {
 	storage := map[string]state.StorageConstraints{
 		"data": makeStorageCons("loop", 1024, 1),
 	}
-	application := s.AddTestingServiceWithStorage(c, "storage-block", ch, storage)
-	u, err := application.AddUnit()
+	application := s.AddTestingApplicationWithStorage(c, "storage-block", ch, storage)
+	u, err := application.AddUnit(state.AddUnitParams{})
 	c.Assert(err, jc.ErrorIsNil)
 
 	// check no cleanups
@@ -588,19 +593,19 @@ func (s *CleanupSuite) TestCleanupStorageAttachments(c *gc.C) {
 	// this tag matches the storage instance created for the unit above.
 	storageTag := names.NewStorageTag("data/0")
 
-	sa, err := s.State.StorageAttachment(storageTag, u.UnitTag())
+	sa, err := s.IAASModel.StorageAttachment(storageTag, u.UnitTag())
 	c.Assert(err, jc.ErrorIsNil)
 	c.Assert(sa.Life(), gc.Equals, state.Alive)
 
-	// destroy unit and run cleanups; the attachment should be marked dying
+	// destroy unit and run cleanups; the storage should be detached
 	err = u.Destroy()
 	c.Assert(err, jc.ErrorIsNil)
 	s.assertCleanupRuns(c)
 
-	// After running the cleanup, the attachment should be dying.
-	sa, err = s.State.StorageAttachment(storageTag, u.UnitTag())
-	c.Assert(err, jc.ErrorIsNil)
-	c.Assert(sa.Life(), gc.Equals, state.Dying)
+	// After running the cleanup, the attachment should be removed
+	// (short-circuited, because volume was never attached).
+	_, err = s.IAASModel.StorageAttachment(storageTag, u.UnitTag())
+	c.Assert(err, jc.Satisfies, errors.IsNotFound)
 
 	// check no cleanups
 	s.assertDoesNotNeedCleanup(c)
@@ -611,8 +616,8 @@ func (s *CleanupSuite) TestCleanupStorageInstances(c *gc.C) {
 	storage := map[string]state.StorageConstraints{
 		"allecto": makeStorageCons("modelscoped-block", 1024, 1),
 	}
-	application := s.AddTestingServiceWithStorage(c, "storage-block", ch, storage)
-	u, err := application.AddUnit()
+	application := s.AddTestingApplicationWithStorage(c, "storage-block", ch, storage)
+	u, err := application.AddUnit(state.AddUnitParams{})
 	c.Assert(err, jc.ErrorIsNil)
 
 	// check no cleanups
@@ -621,25 +626,25 @@ func (s *CleanupSuite) TestCleanupStorageInstances(c *gc.C) {
 	// this tag matches the storage instance created for the unit above.
 	storageTag := names.NewStorageTag("allecto/0")
 
-	si, err := s.State.StorageInstance(storageTag)
+	si, err := s.IAASModel.StorageInstance(storageTag)
 	c.Assert(err, jc.ErrorIsNil)
 	c.Assert(si.Life(), gc.Equals, state.Alive)
 
 	// destroy storage instance and run cleanups
-	err = s.State.DestroyStorageInstance(storageTag)
+	err = s.IAASModel.DestroyStorageInstance(storageTag, true)
 	c.Assert(err, jc.ErrorIsNil)
-	si, err = s.State.StorageInstance(storageTag)
+	si, err = s.IAASModel.StorageInstance(storageTag)
 	c.Assert(err, jc.ErrorIsNil)
 	c.Assert(si.Life(), gc.Equals, state.Dying)
-	sa, err := s.State.StorageAttachment(storageTag, u.UnitTag())
+	sa, err := s.IAASModel.StorageAttachment(storageTag, u.UnitTag())
 	c.Assert(err, jc.ErrorIsNil)
 	c.Assert(sa.Life(), gc.Equals, state.Alive)
 	s.assertCleanupRuns(c)
 
-	// After running the cleanup, the attachment should be dying.
-	sa, err = s.State.StorageAttachment(storageTag, u.UnitTag())
-	c.Assert(err, jc.ErrorIsNil)
-	c.Assert(sa.Life(), gc.Equals, state.Dying)
+	// After running the cleanup, the attachment should be removed
+	// (short-circuited, because volume was never attached).
+	_, err = s.IAASModel.StorageAttachment(storageTag, u.UnitTag())
+	c.Assert(err, jc.Satisfies, errors.IsNotFound)
 
 	// check no cleanups
 	s.assertDoesNotNeedCleanup(c)
@@ -650,8 +655,8 @@ func (s *CleanupSuite) TestCleanupMachineStorage(c *gc.C) {
 	storage := map[string]state.StorageConstraints{
 		"data": makeStorageCons("modelscoped", 1024, 1),
 	}
-	application := s.AddTestingServiceWithStorage(c, "storage-block", ch, storage)
-	unit, err := application.AddUnit()
+	application := s.AddTestingApplicationWithStorage(c, "storage-block", ch, storage)
+	unit, err := application.AddUnit(state.AddUnitParams{})
 	c.Assert(err, jc.ErrorIsNil)
 	err = s.State.AssignUnit(unit, state.AssignCleanEmpty)
 	c.Assert(err, jc.ErrorIsNil)
@@ -676,7 +681,7 @@ func (s *CleanupSuite) TestCleanupMachineStorage(c *gc.C) {
 	c.Assert(err, jc.ErrorIsNil)
 	s.assertCleanupRuns(c)
 
-	vas, err := s.State.MachineVolumeAttachments(machine.MachineTag())
+	vas, err := s.IAASModel.MachineVolumeAttachments(machine.MachineTag())
 	c.Assert(err, jc.ErrorIsNil)
 	c.Assert(vas, gc.HasLen, 1)
 	c.Assert(vas[0].Life(), gc.Equals, state.Dying)
@@ -696,11 +701,11 @@ func (s *CleanupSuite) TestCleanupVolumeAttachments(c *gc.C) {
 	c.Assert(err, jc.ErrorIsNil)
 	s.assertDoesNotNeedCleanup(c)
 
-	err = s.State.DestroyVolume(names.NewVolumeTag("0/0"))
+	err = s.IAASModel.DestroyVolume(names.NewVolumeTag("0/0"))
 	c.Assert(err, jc.ErrorIsNil)
 	s.assertCleanupRuns(c)
 
-	attachment, err := s.State.VolumeAttachment(names.NewMachineTag("0"), names.NewVolumeTag("0/0"))
+	attachment, err := s.IAASModel.VolumeAttachment(names.NewMachineTag("0"), names.NewVolumeTag("0/0"))
 	c.Assert(err, jc.ErrorIsNil)
 	c.Assert(attachment.Life(), gc.Equals, state.Dying)
 }
@@ -716,17 +721,17 @@ func (s *CleanupSuite) TestCleanupFilesystemAttachments(c *gc.C) {
 	c.Assert(err, jc.ErrorIsNil)
 	s.assertDoesNotNeedCleanup(c)
 
-	err = s.State.DestroyFilesystem(names.NewFilesystemTag("0/0"))
+	err = s.IAASModel.DestroyFilesystem(names.NewFilesystemTag("0/0"))
 	c.Assert(err, jc.ErrorIsNil)
 	s.assertCleanupRuns(c)
 
-	attachment, err := s.State.FilesystemAttachment(names.NewMachineTag("0"), names.NewFilesystemTag("0/0"))
+	attachment, err := s.IAASModel.FilesystemAttachment(names.NewMachineTag("0"), names.NewFilesystemTag("0/0"))
 	c.Assert(err, jc.ErrorIsNil)
 	c.Assert(attachment.Life(), gc.Equals, state.Dying)
 }
 
 func (s *CleanupSuite) TestCleanupResourceBlob(c *gc.C) {
-	app := s.AddTestingService(c, "wp", s.AddTestingCharm(c, "wordpress"))
+	app := s.AddTestingApplication(c, "wp", s.AddTestingCharm(c, "wordpress"))
 	data := "ancient-debris"
 	res := resourcetesting.NewResource(c, nil, "mug", "wp", data).Resource
 	resources, err := s.State.Resources()
@@ -751,7 +756,7 @@ func (s *CleanupSuite) TestCleanupResourceBlob(c *gc.C) {
 }
 
 func (s *CleanupSuite) TestCleanupResourceBlobHandlesMissing(c *gc.C) {
-	app := s.AddTestingService(c, "wp", s.AddTestingCharm(c, "wordpress"))
+	app := s.AddTestingApplication(c, "wp", s.AddTestingCharm(c, "wordpress"))
 	data := "ancient-debris"
 	res := resourcetesting.NewResource(c, nil, "mug", "wp", data).Resource
 	resources, err := s.State.Resources()
