@@ -26,6 +26,20 @@ type storage struct {
 
 var _ Storage = (*storage)(nil)
 
+// expiryTime is the time after which non-custom image metadata
+// records will be deleted from the cache.
+const expiryTime = 5 * time.Minute
+
+// MongoIndexes returns the indexes to apply to the clouldimagemetadata collection.
+// We return an index that expires records containing a created-at field after 5 minutes.
+func MongoIndexes() []mgo.Index {
+	return []mgo.Index{{
+		Key:         []string{"expire-at"},
+		ExpireAfter: expiryTime,
+		Sparse:      true,
+	}}
+}
+
 // NewStorage constructs a new Storage that stores image metadata
 // in the provided data store.
 func NewStorage(collectionName string, store DataStore) Storage {
@@ -34,15 +48,26 @@ func NewStorage(collectionName string, store DataStore) Storage {
 
 var emptyMetadata = Metadata{}
 
+// SaveMetadataNoExpiry implements Storage.SaveMetadataNoExpiry and behaves as save-or-update.
+// Records will not expire.
+func (s *storage) SaveMetadataNoExpiry(metadata []Metadata) error {
+	return s.saveMetadata(metadata, false)
+}
+
 // SaveMetadata implements Storage.SaveMetadata and behaves as save-or-update.
+// Records will expire after a set time.
 func (s *storage) SaveMetadata(metadata []Metadata) error {
+	return s.saveMetadata(metadata, true)
+}
+
+func (s *storage) saveMetadata(metadata []Metadata, expires bool) error {
 	if len(metadata) == 0 {
 		return nil
 	}
 
 	newDocs := make([]imagesMetadataDoc, len(metadata))
 	for i, m := range metadata {
-		newDoc := s.mongoDoc(m)
+		newDoc := s.mongoDoc(m, expires)
 		if err := validateMetadata(&newDoc); err != nil {
 			return err
 		}
@@ -190,6 +215,10 @@ type imagesMetadataDoc struct {
 	// that there can be a public and custom image for the same attributes set.
 	Id string `bson:"_id"`
 
+	// ExpireAt is optional and records a time used in conjunction with the
+	// TTL index in order to expire the record.
+	ExpireAt time.Time `bson:"expire-at,omitempty"`
+
 	// ImageId is an image identifier.
 	ImageId string `bson:"image_id"`
 
@@ -252,11 +281,12 @@ func (m imagesMetadataDoc) metadata() Metadata {
 	return r
 }
 
-func (s *storage) mongoDoc(m Metadata) imagesMetadataDoc {
+func (s *storage) mongoDoc(m Metadata, expires bool) imagesMetadataDoc {
+	now := time.Now()
 	dateCreated := m.DateCreated
 	if dateCreated == 0 {
 		// TODO(fwereade): 2016-03-17 lp:1558657
-		dateCreated = time.Now().UnixNano()
+		dateCreated = now.UnixNano()
 	}
 	r := imagesMetadataDoc{
 		Id:              buildKey(m),
@@ -272,6 +302,9 @@ func (s *storage) mongoDoc(m Metadata) imagesMetadataDoc {
 		Source:          m.Source,
 		Priority:        m.Priority,
 	}
+	if expires {
+		r.ExpireAt = now
+	}
 	if m.RootStorageSize != nil {
 		r.RootStorageSize = *m.RootStorageSize
 	}
