@@ -109,7 +109,10 @@ func SetRetryHooks(c *gc.C, st *State, block, check func()) txntesting.Transacti
 
 func newRunnerForHooks(st *State) jujutxn.Runner {
 	db := st.database.(*database)
-	runner := jujutxn.NewRunner(jujutxn.RunnerParams{Database: db.raw})
+	runner := jujutxn.NewRunner(jujutxn.RunnerParams{
+		Database: db.raw,
+		Clock:    st.stateClock,
+	})
 	db.runner = runner
 	return runner
 }
@@ -308,7 +311,7 @@ func GetAllUpgradeInfos(st *State) ([]*UpgradeInfo, error) {
 	for iter.Next(&doc) {
 		out = append(out, &UpgradeInfo{st: st, doc: doc})
 	}
-	if err := iter.Err(); err != nil {
+	if err := iter.Close(); err != nil {
 		return nil, err
 	}
 	return out, nil
