@@ -1,4 +1,4 @@
-// Copyright 2013 Canonical Ltd.
+// Copyright Canonical Ltd. 2013
 // Licensed under the AGPLv3, see LICENCE file for details.
 
 package state_test
@@ -26,6 +26,7 @@ type InitializeSuite struct {
 	gitjujutesting.MgoSuite
 	testing.BaseSuite
 	State *state.State
+	Model *state.Model
 }
 
 var _ = gc.Suite(&InitializeSuite{})
@@ -55,6 +56,10 @@ func (s *InitializeSuite) openState(c *gc.C, modelTag names.ModelTag) {
 	})
 	c.Assert(err, jc.ErrorIsNil)
 	s.State = st
+
+	m, err := st.Model()
+	c.Assert(err, jc.ErrorIsNil)
+	s.Model = m
 }
 
 func (s *InitializeSuite) TearDownTest(c *gc.C) {
@@ -93,10 +98,11 @@ func (s *InitializeSuite) TestInitialize(c *gc.C) {
 	}
 	controllerCfg := testing.FakeControllerConfig()
 
-	st, err := state.Initialize(state.InitializeParams{
+	ctlr, st, err := state.Initialize(state.InitializeParams{
 		Clock:            clock.WallClock,
 		ControllerConfig: controllerCfg,
 		ControllerModelArgs: state.ModelArgs{
+			Type:                    state.ModelTypeIAAS,
 			Owner:                   owner,
 			Config:                  cfg,
 			CloudName:               "dummy",
@@ -118,14 +124,20 @@ func (s *InitializeSuite) TestInitialize(c *gc.C) {
 	})
 	c.Assert(err, jc.ErrorIsNil)
 	c.Assert(st, gc.NotNil)
-	modelTag := st.ModelTag()
+	m, err := st.Model()
+	c.Assert(err, jc.ErrorIsNil)
+	modelTag := m.ModelTag()
 	c.Assert(modelTag.Id(), gc.Equals, uuid)
+
 	err = st.Close()
 	c.Assert(err, jc.ErrorIsNil)
 
+	err = ctlr.Close()
+	c.Assert(err, jc.ErrorIsNil)
+
 	s.openState(c, modelTag)
 
-	cfg, err = s.State.ModelConfig()
+	cfg, err = s.Model.ModelConfig()
 	c.Assert(err, jc.ErrorIsNil)
 	expected := cfg.AllAttrs()
 	for k, v := range config.ConfigDefaults() {
@@ -186,10 +198,11 @@ func (s *InitializeSuite) TestInitializeWithInvalidCredentialType(c *gc.C) {
 	modelCfg := testing.ModelConfig(c)
 	controllerCfg := testing.FakeControllerConfig()
 	credentialTag := names.NewCloudCredentialTag("dummy/" + owner.Id() + "/borken")
-	_, err := state.Initialize(state.InitializeParams{
+	_, _, err := state.Initialize(state.InitializeParams{
 		Clock:            clock.WallClock,
 		ControllerConfig: controllerCfg,
 		ControllerModelArgs: state.ModelArgs{
+			Type:                    state.ModelTypeIAAS,
 			CloudName:               "dummy",
 			Owner:                   owner,
 			Config:                  modelCfg,
@@ -223,10 +236,11 @@ func (s *InitializeSuite) TestInitializeWithControllerInheritedConfig(c *gc.C) {
 	owner := names.NewLocalUserTag("initialize-admin")
 	controllerCfg := testing.FakeControllerConfig()
 
-	st, err := state.Initialize(state.InitializeParams{
+	ctlr, st, err := state.Initialize(state.InitializeParams{
 		Clock:            clock.WallClock,
 		ControllerConfig: controllerCfg,
 		ControllerModelArgs: state.ModelArgs{
+			Type:                    state.ModelTypeIAAS,
 			CloudName:               "dummy",
 			Owner:                   owner,
 			Config:                  cfg,
@@ -243,11 +257,17 @@ func (s *InitializeSuite) TestInitializeWithControllerInheritedConfig(c *gc.C) {
 	})
 	c.Assert(err, jc.ErrorIsNil)
 	c.Assert(st, gc.NotNil)
-	modelTag := st.ModelTag()
+	m, err := st.Model()
+	c.Assert(err, jc.ErrorIsNil)
+	modelTag := m.ModelTag()
 	c.Assert(modelTag.Id(), gc.Equals, uuid)
+
 	err = st.Close()
 	c.Assert(err, jc.ErrorIsNil)
 
+	err = ctlr.Close()
+	c.Assert(err, jc.ErrorIsNil)
+
 	s.openState(c, modelTag)
 
 	controllerInheritedConfig, err := s.State.ReadSettings(state.GlobalSettingsC, state.ControllerInheritedSettingsGlobalKey)
@@ -262,7 +282,7 @@ func (s *InitializeSuite) TestInitializeWithControllerInheritedConfig(c *gc.C) {
 	}
 	// Config as read from state has resources tags coerced to a map.
 	expected["resource-tags"] = map[string]string{}
-	cfg, err = s.State.ModelConfig()
+	cfg, err = s.Model.ModelConfig()
 	c.Assert(err, jc.ErrorIsNil)
 	c.Assert(cfg.AllAttrs(), jc.DeepEquals, expected)
 }
@@ -279,6 +299,7 @@ func (s *InitializeSuite) TestDoubleInitializeConfig(c *gc.C) {
 		Clock:            clock.WallClock,
 		ControllerConfig: controllerCfg,
 		ControllerModelArgs: state.ModelArgs{
+			Type:                    state.ModelTypeIAAS,
 			CloudName:               "dummy",
 			Owner:                   owner,
 			Config:                  cfg,
@@ -292,17 +313,19 @@ func (s *InitializeSuite) TestDoubleInitializeConfig(c *gc.C) {
 		MongoInfo:     mgoInfo,
 		MongoDialOpts: dialOpts,
 	}
-	st, err := state.Initialize(args)
+	ctlr, st, err := state.Initialize(args)
 	c.Assert(err, jc.ErrorIsNil)
+
 	err = st.Close()
 	c.Check(err, jc.ErrorIsNil)
 
-	st, err = state.Initialize(args)
+	err = ctlr.Close()
+	c.Check(err, jc.ErrorIsNil)
+
+	ctlr, st, err = state.Initialize(args)
 	c.Check(err, gc.ErrorMatches, "already initialized")
-	if !c.Check(st, gc.IsNil) {
-		err = st.Close()
-		c.Check(err, jc.ErrorIsNil)
-	}
+	c.Check(ctlr, gc.IsNil)
+	c.Check(st, gc.IsNil)
 }
 
 func (s *InitializeSuite) TestModelConfigWithAdminSecret(c *gc.C) {
@@ -337,6 +360,7 @@ func (s *InitializeSuite) testBadModelConfig(c *gc.C, update map[string]interfac
 		Clock:            clock.WallClock,
 		ControllerConfig: controllerCfg,
 		ControllerModelArgs: state.ModelArgs{
+			Type:                    state.ModelTypeIAAS,
 			CloudName:               "dummy",
 			CloudRegion:             "dummy-region",
 			Owner:                   owner,
@@ -352,20 +376,24 @@ func (s *InitializeSuite) testBadModelConfig(c *gc.C, update map[string]interfac
 		MongoInfo:     statetesting.NewMongoInfo(),
 		MongoDialOpts: mongotest.DialOpts(),
 	}
-	_, err = state.Initialize(args)
+	_, _, err = state.Initialize(args)
 	c.Assert(err, gc.ErrorMatches, expect)
 
 	args.ControllerModelArgs.Config = good
-	st, err := state.Initialize(args)
+	ctlr, st, err := state.Initialize(args)
 	c.Assert(err, jc.ErrorIsNil)
 	st.Close()
+	ctlr.Close()
 
-	s.openState(c, st.ModelTag())
-	err = s.State.UpdateModelConfig(update, remove)
+	s.openState(c, names.NewModelTag(st.ModelUUID()))
+	m, err := s.State.Model()
+	c.Assert(err, jc.ErrorIsNil)
+
+	err = m.UpdateModelConfig(update, remove)
 	c.Assert(err, gc.ErrorMatches, expect)
 
 	// ModelConfig remains inviolate.
-	cfg, err := s.State.ModelConfig()
+	cfg, err := s.Model.ModelConfig()
 	c.Assert(err, jc.ErrorIsNil)
 	goodWithDefaults, err := config.New(config.UseDefaults, good.AllAttrs())
 	c.Assert(err, jc.ErrorIsNil)
@@ -388,6 +416,7 @@ func (s *InitializeSuite) TestCloudConfigWithForbiddenValues(c *gc.C) {
 		Clock:            clock.WallClock,
 		ControllerConfig: controllerCfg,
 		ControllerModelArgs: state.ModelArgs{
+			Type:                    state.ModelTypeIAAS,
 			CloudName:               "dummy",
 			Owner:                   names.NewLocalUserTag("initialize-admin"),
 			Config:                  modelCfg,
@@ -405,7 +434,7 @@ func (s *InitializeSuite) TestCloudConfigWithForbiddenValues(c *gc.C) {
 	for _, badAttrName := range badAttrNames {
 		badAttrs := map[string]interface{}{badAttrName: "foo"}
 		args.ControllerInheritedConfig = badAttrs
-		_, err := state.Initialize(args)
+		_, _, err := state.Initialize(args)
 		c.Assert(err, gc.ErrorMatches, "local cloud config cannot contain .*")
 	}
 }
@@ -426,10 +455,11 @@ func (s *InitializeSuite) TestInitializeWithCloudRegionConfig(c *gc.C) {
 	owner := names.NewLocalUserTag("initialize-admin")
 	controllerCfg := testing.FakeControllerConfig()
 
-	st, err := state.Initialize(state.InitializeParams{
+	ctlr, st, err := state.Initialize(state.InitializeParams{
 		Clock:            clock.WallClock,
 		ControllerConfig: controllerCfg,
 		ControllerModelArgs: state.ModelArgs{
+			Type:                    state.ModelTypeIAAS,
 			CloudName:               "dummy",
 			Owner:                   owner,
 			Config:                  cfg,
@@ -446,11 +476,17 @@ func (s *InitializeSuite) TestInitializeWithCloudRegionConfig(c *gc.C) {
 	})
 	c.Assert(err, jc.ErrorIsNil)
 	c.Assert(st, gc.NotNil)
-	modelTag := st.ModelTag()
+	m, err := st.Model()
+	c.Assert(err, jc.ErrorIsNil)
+	modelTag := m.ModelTag()
 	c.Assert(modelTag.Id(), gc.Equals, uuid)
+
 	err = st.Close()
 	c.Assert(err, jc.ErrorIsNil)
 
+	err = ctlr.Close()
+	c.Assert(err, jc.ErrorIsNil)
+
 	s.openState(c, modelTag)
 
 	for k := range regionInheritedConfigIn {
@@ -484,10 +520,11 @@ func (s *InitializeSuite) TestInitializeWithCloudRegionMisses(c *gc.C) {
 	owner := names.NewLocalUserTag("initialize-admin")
 	controllerCfg := testing.FakeControllerConfig()
 
-	st, err := state.Initialize(state.InitializeParams{
+	ctlr, st, err := state.Initialize(state.InitializeParams{
 		Clock:            clock.WallClock,
 		ControllerConfig: controllerCfg,
 		ControllerModelArgs: state.ModelArgs{
+			Type:                    state.ModelTypeIAAS,
 			CloudName:               "dummy",
 			Owner:                   owner,
 			Config:                  cfg,
@@ -505,11 +542,17 @@ func (s *InitializeSuite) TestInitializeWithCloudRegionMisses(c *gc.C) {
 	})
 	c.Assert(err, jc.ErrorIsNil)
 	c.Assert(st, gc.NotNil)
-	modelTag := st.ModelTag()
+	m, err := st.Model()
+	c.Assert(err, jc.ErrorIsNil)
+	modelTag := m.ModelTag()
 	c.Assert(modelTag.Id(), gc.Equals, uuid)
+
 	err = st.Close()
 	c.Assert(err, jc.ErrorIsNil)
 
+	err = ctlr.Close()
+	c.Assert(err, jc.ErrorIsNil)
+
 	s.openState(c, modelTag)
 
 	var attrs map[string]interface{}
@@ -538,10 +581,11 @@ func (s *InitializeSuite) TestInitializeWithCloudRegionHits(c *gc.C) {
 	owner := names.NewLocalUserTag("initialize-admin")
 	controllerCfg := testing.FakeControllerConfig()
 
-	st, err := state.Initialize(state.InitializeParams{
+	ctlr, st, err := state.Initialize(state.InitializeParams{
 		Clock:            clock.WallClock,
 		ControllerConfig: controllerCfg,
 		ControllerModelArgs: state.ModelArgs{
+			Type:                    state.ModelTypeIAAS,
 			CloudName:               "dummy",
 			Owner:                   owner,
 			Config:                  cfg,
@@ -559,11 +603,17 @@ func (s *InitializeSuite) TestInitializeWithCloudRegionHits(c *gc.C) {
 	})
 	c.Assert(err, jc.ErrorIsNil)
 	c.Assert(st, gc.NotNil)
-	modelTag := st.ModelTag()
+	m, err := st.Model()
+	c.Assert(err, jc.ErrorIsNil)
+	modelTag := m.ModelTag()
 	c.Assert(modelTag.Id(), gc.Equals, uuid)
+
 	err = st.Close()
 	c.Assert(err, jc.ErrorIsNil)
 
+	err = ctlr.Close()
+	c.Assert(err, jc.ErrorIsNil)
+
 	s.openState(c, modelTag)
 
 	var attrs map[string]interface{}
