@@ -15,6 +15,7 @@ import (
 	"gopkg.in/mgo.v2/bson"
 	"gopkg.in/mgo.v2/txn"
 
+	"github.com/juju/juju/environs"
 	"github.com/juju/juju/network"
 )
 
@@ -113,7 +114,7 @@ func (m *Machine) RemoveAllLinkLayerDevices() error {
 		return errors.Trace(err)
 	}
 
-	return m.st.runTransaction(ops)
+	return m.st.db().RunTransaction(ops)
 }
 
 func (m *Machine) removeAllLinkLayerDevicesOps() ([]txn.Op, error) {
@@ -229,7 +230,7 @@ func (m *Machine) SetLinkLayerDevices(devicesArgs ...LinkLayerDeviceArgs) (err e
 		}
 		return append(ops, setDevicesOps...), nil
 	}
-	if err := m.st.run(buildTxn); err != nil {
+	if err := m.st.db().Run(buildTxn); err != nil {
 		return errors.Trace(err)
 	}
 	return nil
@@ -589,6 +590,10 @@ type LinkLayerDeviceAddress struct {
 
 	// GatewayAddress is the address of the gateway to use, which can be empty.
 	GatewayAddress string
+
+	// IsDefaultGateway is set to true if this address on this device is the
+	// default gw on a machine.
+	IsDefaultGateway bool
 }
 
 // SetDevicesAddresses sets the addresses of all devices in devicesAddresses,
@@ -605,7 +610,6 @@ type LinkLayerDeviceAddress struct {
 // - ErrProviderIDNotUnique, when one or more specified ProviderIDs are not unique.
 func (m *Machine) SetDevicesAddresses(devicesAddresses ...LinkLayerDeviceAddress) (err error) {
 	defer errors.DeferredAnnotatef(&err, "cannot set link-layer device addresses of machine %q", m.doc.Id)
-
 	if len(devicesAddresses) == 0 {
 		logger.Debugf("no device addresses to set")
 		return nil
@@ -651,7 +655,7 @@ func (m *Machine) SetDevicesAddresses(devicesAddresses ...LinkLayerDeviceAddress
 		}
 		return append(ops, setAddressesOps...), nil
 	}
-	if err := m.st.run(buildTxn); err != nil {
+	if err := m.st.db().Run(buildTxn); err != nil {
 		return errors.Trace(err)
 	}
 	return nil
@@ -760,6 +764,7 @@ func (m *Machine) newIPAddressDocFromArgs(args *LinkLayerDeviceAddress) (*ipAddr
 		DNSServers:       args.DNSServers,
 		DNSSearchDomains: args.DNSSearchDomains,
 		GatewayAddress:   args.GatewayAddress,
+		IsDefaultGateway: args.IsDefaultGateway,
 	}
 	return newDoc, nil
 }
@@ -853,7 +858,7 @@ func (m *Machine) RemoveAllAddresses() error {
 		return errors.Trace(err)
 	}
 
-	return m.st.runTransaction(ops)
+	return m.st.db().RunTransaction(ops)
 }
 
 func (m *Machine) removeAllAddressesOps() ([]txn.Op, error) {
@@ -1124,7 +1129,7 @@ func generateMACAddress() string {
 // MachineNetworkInfoResult contains an error or a list of NetworkInfo structures for a specific space.
 type MachineNetworkInfoResult struct {
 	NetworkInfos []network.NetworkInfo
-	Error        *error
+	Error        error
 }
 
 // Add address to a device in list or create a new device with this address.
@@ -1164,21 +1169,19 @@ func (m *Machine) GetNetworkInfoForSpaces(spaces set.Strings) map[string](Machin
 
 	var privateAddress network.Address
 
-	if spaces.Contains("") {
+	if spaces.Contains(environs.DefaultSpaceName) {
 		var err error
 		privateAddress, err = m.PrivateAddress()
 		if err != nil {
-			error := errors.Annotatef(err, "getting machine %q preferred private address", m.MachineTag())
-			results[""] = MachineNetworkInfoResult{Error: &error}
-			spaces.Remove("")
+			results[environs.DefaultSpaceName] = MachineNetworkInfoResult{Error: errors.Annotatef(err, "getting machine %q preferred private address", m.MachineTag())}
+			spaces.Remove(environs.DefaultSpaceName)
 		}
 	}
 
 	addresses, err := m.AllAddresses()
 	logger.Debugf("Looking for something from spaces %v in %v", spaces, addresses)
 	if err != nil {
-		newErr := errors.Annotate(err, "cannot get devices addresses")
-		result := MachineNetworkInfoResult{Error: &newErr}
+		result := MachineNetworkInfoResult{Error: errors.Annotate(err, "cannot get devices addresses")}
 		for space := range spaces {
 			if _, ok := results[space]; !ok {
 				results[space] = result
@@ -1201,44 +1204,41 @@ func (m *Machine) GetNetworkInfoForSpaces(spaces set.Strings) map[string](Machin
 				r := results[space]
 				r.NetworkInfos, err = addAddressToResult(r.NetworkInfos, addr)
 				if err != nil {
-					r.Error = &err
+					r.Error = err
 				} else {
 					results[space] = r
 				}
 			}
-			if spaces.Contains("") && privateAddress.Value == addr.Value() {
-				r := results[""]
+			if spaces.Contains(environs.DefaultSpaceName) && privateAddress.Value == addr.Value() {
+				r := results[environs.DefaultSpaceName]
 				r.NetworkInfos, err = addAddressToResult(r.NetworkInfos, addr)
 				if err != nil {
-					r.Error = &err
+					r.Error = err
 				} else {
-					results[""] = r
+					results[environs.DefaultSpaceName] = r
 				}
 			}
 		}
 	}
 
 	// For a spaceless environment we won't find a subnet that's linked to privateAddress,
-	// we have to work around that and at least return minimal information for --primary-address.
-	if r, filledPrivateAddress := results[""]; !filledPrivateAddress && spaces.Contains("") {
+	// we have to work around that and at least return minimal information.
+	if r, filledPrivateAddress := results[environs.DefaultSpaceName]; !filledPrivateAddress && spaces.Contains(environs.DefaultSpaceName) {
 		r.NetworkInfos = []network.NetworkInfo{{
 			Addresses: []network.InterfaceAddress{{
 				Address: privateAddress.Value,
 			}},
 		}}
-		results[""] = r
+		results[environs.DefaultSpaceName] = r
 	}
 	actualSpacesStr := network.QuoteSpaceSet(actualSpaces)
 
 	for space := range spaces {
 		if _, ok := results[space]; !ok {
-			newErr := errors.Errorf("machine %q has no devices in space %q, only spaces %s", m.doc.Id, space, actualSpacesStr)
 			results[space] = MachineNetworkInfoResult{
-				Error: &newErr,
+				Error: errors.Errorf("machine %q has no devices in space %q, only spaces %s", m.doc.Id, space, actualSpacesStr),
 			}
 		}
-		return results
 	}
-
 	return results
 }
