@@ -10,7 +10,7 @@ import (
 
 	"github.com/juju/errors"
 	"github.com/juju/loggo"
-	"gopkg.in/juju/charm.v6-unstable"
+	"gopkg.in/juju/charm.v6"
 	"gopkg.in/juju/names.v2"
 	"gopkg.in/mgo.v2"
 	"gopkg.in/mgo.v2/bson"
@@ -165,7 +165,7 @@ func (st *State) AddMetrics(batch BatchParam) (*MetricBatch, error) {
 		}}
 		return ops, nil
 	}
-	err = st.run(buildTxn)
+	err = st.db().Run(buildTxn)
 	if err != nil {
 		return nil, errors.Trace(err)
 	}
@@ -211,7 +211,7 @@ func (st *State) AddModelMetrics(batch ModelBatchParam) (*MetricBatch, error) {
 		}}
 		return ops, nil
 	}
-	err = st.run(buildTxn)
+	err = st.db().Run(buildTxn)
 	if err != nil {
 		return nil, errors.Trace(err)
 	}
@@ -302,7 +302,7 @@ func (st *State) MetricBatch(id string) (*MetricBatch, error) {
 // CleanupOldMetrics looks for metrics that are 24 hours old (or older)
 // and have been sent. Any metrics it finds are deleted.
 func (st *State) CleanupOldMetrics() error {
-	now := st.clock.Now()
+	now := st.clock().Now()
 	metrics, closer := st.db().GetCollection(metricsC)
 	defer closer()
 	// Nothing else in the system will interact with sent metrics, and nothing needs
@@ -436,7 +436,7 @@ func (m *MetricBatch) UniqueMetrics() []Metric {
 func (m *MetricBatch) SetSent(t time.Time) error {
 	deleteTime := t.UTC().Add(CleanupAge)
 	ops := setSentOps([]string{m.UUID()}, deleteTime)
-	if err := m.st.runTransaction(ops); err != nil {
+	if err := m.st.db().RunTransaction(ops); err != nil {
 		return errors.Annotatef(err, "cannot set metric sent for metric %q", m.UUID())
 	}
 
@@ -470,9 +470,9 @@ func setSentOps(batchUUIDs []string, deleteTime time.Time) []txn.Op {
 
 // SetMetricBatchesSent sets sent on each MetricBatch corresponding to the uuids provided.
 func (st *State) SetMetricBatchesSent(batchUUIDs []string) error {
-	deleteTime := st.clock.Now().UTC().Add(CleanupAge)
+	deleteTime := st.clock().Now().UTC().Add(CleanupAge)
 	ops := setSentOps(batchUUIDs, deleteTime)
-	if err := st.runTransaction(ops); err != nil {
+	if err := st.db().RunTransaction(ops); err != nil {
 		return errors.Annotatef(err, "cannot set metric sent in bulk call")
 	}
 	return nil
