@@ -34,7 +34,7 @@ func (s *MetricSuite) SetUpTest(c *gc.C) {
 }
 
 func (s *MetricSuite) TestAddNoMetrics(c *gc.C) {
-	now := s.State.NowToTheSecond()
+	now := state.NowToTheSecond(s.State)
 	_, err := s.State.AddMetrics(state.BatchParam{
 		UUID:     utils.MustNewUUID().String(),
 		CharmURL: s.meteredCharm.URL().String(),
@@ -57,7 +57,7 @@ func ensureUnitDead(c *gc.C, unit *state.Unit) {
 }
 
 func (s *MetricSuite) TestAddMetric(c *gc.C) {
-	now := s.State.NowToTheSecond()
+	now := state.NowToTheSecond(s.State)
 	modelUUID := s.State.ModelUUID()
 	m := state.Metric{"pings", "5", now}
 	metricBatch, err := s.State.AddMetrics(
@@ -95,7 +95,7 @@ func (s *MetricSuite) TestAddMetric(c *gc.C) {
 }
 
 func (s *MetricSuite) TestAddModelMetricMetric(c *gc.C) {
-	now := s.State.NowToTheSecond()
+	now := state.NowToTheSecond(s.State)
 	modelUUID := s.State.ModelUUID()
 	m := state.Metric{"pings", "5", now}
 	metricBatch, err := s.State.AddModelMetrics(
@@ -134,7 +134,7 @@ func (s *MetricSuite) TestAddModelMetricMetric(c *gc.C) {
 
 func (s *MetricSuite) TestAddMetricNonExistentUnit(c *gc.C) {
 	removeUnit(c, s.unit)
-	now := s.State.NowToTheSecond()
+	now := state.NowToTheSecond(s.State)
 	m := state.Metric{"pings", "5", now}
 	unitTag := names.NewUnitTag("test/0")
 	_, err := s.State.AddMetrics(
@@ -151,7 +151,7 @@ func (s *MetricSuite) TestAddMetricNonExistentUnit(c *gc.C) {
 
 func (s *MetricSuite) TestAddMetricDeadUnit(c *gc.C) {
 	ensureUnitDead(c, s.unit)
-	now := s.State.NowToTheSecond()
+	now := state.NowToTheSecond(s.State)
 	m := state.Metric{"pings", "5", now}
 	_, err := s.State.AddMetrics(
 		state.BatchParam{
@@ -166,7 +166,7 @@ func (s *MetricSuite) TestAddMetricDeadUnit(c *gc.C) {
 }
 
 func (s *MetricSuite) TestSetMetricSent(c *gc.C) {
-	now := s.State.NowToTheSecond()
+	now := state.NowToTheSecond(s.State)
 	m := state.Metric{"pings", "5", now}
 	added, err := s.State.AddMetrics(
 		state.BatchParam{
@@ -284,7 +284,7 @@ func (s *MetricSuite) TestCleanupMetricsIgnoreNotSent(c *gc.C) {
 }
 
 func (s *MetricSuite) TestAllMetricBatches(c *gc.C) {
-	now := s.State.NowToTheSecond()
+	now := state.NowToTheSecond(s.State)
 	m := state.Metric{"pings", "5", now}
 	_, err := s.State.AddMetrics(
 		state.BatchParam{
@@ -306,7 +306,7 @@ func (s *MetricSuite) TestAllMetricBatches(c *gc.C) {
 }
 
 func (s *MetricSuite) TestAllMetricBatchesCustomCharmURLAndUUID(c *gc.C) {
-	now := s.State.NowToTheSecond()
+	now := state.NowToTheSecond(s.State)
 	m := state.Metric{"pings", "5", now}
 	uuid := utils.MustNewUUID().String()
 	charmURL := "cs:quantal/metered-1"
@@ -331,7 +331,7 @@ func (s *MetricSuite) TestAllMetricBatchesCustomCharmURLAndUUID(c *gc.C) {
 }
 
 func (s *MetricSuite) TestMetricCredentials(c *gc.C) {
-	now := s.State.NowToTheSecond()
+	now := state.NowToTheSecond(s.State)
 	m := state.Metric{"pings", "5", now}
 	err := s.application.SetMetricCredentials([]byte("hello there"))
 	c.Assert(err, gc.IsNil)
@@ -388,7 +388,7 @@ func (s *MetricSuite) TestSetMetricBatchesSent(c *gc.C) {
 }
 
 func (s *MetricSuite) TestMetricsToSend(c *gc.C) {
-	now := s.State.NowToTheSecond()
+	now := state.NowToTheSecond(s.State)
 	m := []state.Metric{{Key: "pings", Value: "123", Time: now}}
 	s.Factory.MakeMetric(c, &factory.MetricParams{Unit: s.unit, Sent: false, Time: &now, Metrics: m})
 	s.Factory.MakeMetric(c, &factory.MetricParams{Unit: s.unit, Sent: false, Time: &now, Metrics: m})
@@ -400,7 +400,7 @@ func (s *MetricSuite) TestMetricsToSend(c *gc.C) {
 
 // TestMetricsToSendBatches checks that metrics are properly batched.
 func (s *MetricSuite) TestMetricsToSendBatches(c *gc.C) {
-	now := s.State.NowToTheSecond()
+	now := state.NowToTheSecond(s.State)
 	for i := 0; i < 6; i++ {
 		m := []state.Metric{{Key: "pings", Value: "123", Time: now}}
 		s.Factory.MakeMetric(c, &factory.MetricParams{Unit: s.unit, Sent: false, Time: &now, Metrics: m})
@@ -428,7 +428,7 @@ func (s *MetricSuite) TestMetricValidation(c *gc.C) {
 	nonMeteredUnit := s.Factory.MakeUnit(c, &factory.UnitParams{SetCharmURL: true})
 	meteredApplication := s.Factory.MakeApplication(c, &factory.ApplicationParams{Name: "metered-service", Charm: s.meteredCharm})
 	meteredUnit := s.Factory.MakeUnit(c, &factory.UnitParams{Application: meteredApplication, SetCharmURL: true})
-	dyingUnit, err := meteredApplication.AddUnit()
+	dyingUnit, err := meteredApplication.AddUnit(state.AddUnitParams{})
 	c.Assert(err, jc.ErrorIsNil)
 	err = dyingUnit.SetCharmURL(s.meteredCharm.URL())
 	c.Assert(err, jc.ErrorIsNil)
@@ -503,7 +503,7 @@ func (s *MetricSuite) TestMetricValidation(c *gc.C) {
 }
 
 func (s *MetricSuite) TestAddMetricDuplicateUUID(c *gc.C) {
-	now := s.State.NowToTheSecond()
+	now := state.NowToTheSecond(s.State)
 	mUUID := utils.MustNewUUID().String()
 	_, err := s.State.AddMetrics(
 		state.BatchParam{
@@ -552,7 +552,7 @@ func (s *MetricSuite) TestAddBuiltInMetric(c *gc.C) {
 	}
 	for _, test := range tests {
 		c.Logf("running test: %v", test.about)
-		now := s.State.NowToTheSecond()
+		now := state.NowToTheSecond(s.State)
 		modelUUID := s.State.ModelUUID()
 		m := state.Metric{"juju-units", test.value, now}
 		metricBatch, err := s.State.AddMetrics(
@@ -595,7 +595,7 @@ func (s *MetricSuite) TestAddBuiltInMetric(c *gc.C) {
 }
 
 func (s *MetricSuite) TestUnitMetricBatchesMatchesAllCharms(c *gc.C) {
-	now := s.State.NowToTheSecond()
+	now := state.NowToTheSecond(s.State)
 	m := state.Metric{"pings", "5", now}
 	_, err := s.State.AddMetrics(
 		state.BatchParam{
@@ -654,7 +654,7 @@ func (s *MetricLocalCharmSuite) SetUpTest(c *gc.C) {
 }
 
 func (s *MetricLocalCharmSuite) TestUnitMetricBatches(c *gc.C) {
-	now := s.State.NowToTheSecond()
+	now := state.NowToTheSecond(s.State)
 	m := state.Metric{"pings", "5", now}
 	m2 := state.Metric{"pings", "10", now}
 	_, err := s.State.AddMetrics(
@@ -667,7 +667,7 @@ func (s *MetricLocalCharmSuite) TestUnitMetricBatches(c *gc.C) {
 		},
 	)
 	c.Assert(err, jc.ErrorIsNil)
-	newUnit, err := s.application.AddUnit()
+	newUnit, err := s.application.AddUnit(state.AddUnitParams{})
 	c.Assert(err, jc.ErrorIsNil)
 	_, err = s.State.AddMetrics(
 		state.BatchParam{
@@ -700,7 +700,7 @@ func (s *MetricLocalCharmSuite) TestUnitMetricBatches(c *gc.C) {
 }
 
 func (s *MetricLocalCharmSuite) TestApplicationMetricBatches(c *gc.C) {
-	now := s.State.NowToTheSecond()
+	now := state.NowToTheSecond(s.State)
 	m := state.Metric{"pings", "5", now}
 	m2 := state.Metric{"pings", "10", now}
 	_, err := s.State.AddMetrics(
@@ -713,7 +713,7 @@ func (s *MetricLocalCharmSuite) TestApplicationMetricBatches(c *gc.C) {
 		},
 	)
 	c.Assert(err, jc.ErrorIsNil)
-	newUnit, err := s.application.AddUnit()
+	newUnit, err := s.application.AddUnit(state.AddUnitParams{})
 	c.Assert(err, jc.ErrorIsNil)
 	_, err = s.State.AddMetrics(
 		state.BatchParam{
@@ -744,7 +744,7 @@ func (s *MetricLocalCharmSuite) TestApplicationMetricBatches(c *gc.C) {
 }
 
 func (s *MetricLocalCharmSuite) TestModelMetricBatches(c *gc.C) {
-	now := s.State.NowToTheSecond()
+	now := state.NowToTheSecond(s.State)
 	// Add 2 metric batches to a single unit.
 	m := state.Metric{"pings", "5", now}
 	m2 := state.Metric{"pings", "10", now}
@@ -758,7 +758,7 @@ func (s *MetricLocalCharmSuite) TestModelMetricBatches(c *gc.C) {
 		},
 	)
 	c.Assert(err, jc.ErrorIsNil)
-	newUnit, err := s.application.AddUnit()
+	newUnit, err := s.application.AddUnit(state.AddUnitParams{})
 	c.Assert(err, jc.ErrorIsNil)
 	_, err = s.State.AddMetrics(
 		state.BatchParam{
@@ -827,7 +827,7 @@ func (s *MetricLocalCharmSuite) TestModelMetricBatches(c *gc.C) {
 }
 
 func (s *MetricLocalCharmSuite) TestMetricsSorted(c *gc.C) {
-	newUnit, err := s.application.AddUnit()
+	newUnit, err := s.application.AddUnit(state.AddUnitParams{})
 	c.Assert(err, jc.ErrorIsNil)
 
 	t0 := time.Date(2016, time.August, 16, 16, 00, 35, 0, time.Local)
@@ -896,7 +896,7 @@ func assertMetricBatchesTimeAscending(c *gc.C, batches []state.MetricBatch) {
 }
 
 func (s *MetricLocalCharmSuite) TestUnitMetricBatchesReturnsAllCharms(c *gc.C) {
-	now := s.State.NowToTheSecond()
+	now := state.NowToTheSecond(s.State)
 	m := state.Metric{"pings", "5", now}
 	_, err := s.State.AddMetrics(
 		state.BatchParam{
@@ -929,7 +929,7 @@ func (s *MetricLocalCharmSuite) TestUnitMetricBatchesReturnsAllCharms(c *gc.C) {
 }
 
 func (s *MetricLocalCharmSuite) TestUnique(c *gc.C) {
-	t0 := s.State.NowToTheSecond()
+	t0 := state.NowToTheSecond(s.State)
 	t1 := t0.Add(time.Second)
 	batch, err := s.State.AddMetrics(
 		state.BatchParam{
@@ -1012,7 +1012,7 @@ func mustCreateMeteredModel(c *gc.C, stateFactory *factory.Factory) (modelData,
 }
 
 func (s *CrossModelMetricSuite) TestMetricsAcrossEnvironments(c *gc.C) {
-	now := s.State.NowToTheSecond().Add(-48 * time.Hour)
+	now := state.NowToTheSecond(s.State).Add(-48 * time.Hour)
 	m := state.Metric{"pings", "5", now}
 	m1, err := s.models[0].state.AddMetrics(
 		state.BatchParam{
