@@ -28,6 +28,18 @@ func (s *metricsManagerSuite) TestDefaultsWritten(c *gc.C) {
 	c.Assert(mm.GracePeriod(), gc.Equals, 24*7*time.Hour)
 }
 
+func (s *metricsManagerSuite) TestNewMetricsManager(c *gc.C) {
+	state.SetBeforeHooks(c, s.State, func() {
+		_, err := s.State.MetricsManager()
+		c.Assert(err, jc.ErrorIsNil)
+	})
+	mm, err := s.State.MetricsManager()
+	c.Assert(err, jc.ErrorIsNil)
+	c.Assert(mm.LastSuccessfulSend(), gc.DeepEquals, time.Time{})
+	c.Assert(mm.ConsecutiveErrors(), gc.Equals, 0)
+	c.Assert(mm.GracePeriod(), gc.Equals, 24*7*time.Hour)
+}
+
 func (s *metricsManagerSuite) TestMetricsManagerCreatesThenReturns(c *gc.C) {
 	mm, err := s.State.MetricsManager()
 	c.Assert(err, jc.ErrorIsNil)
