@@ -755,14 +755,14 @@ func (i *importer) application(a description.Application) error {
 	// mean to use the default, i.e. don't set the value.
 	// There may have existed some applications with settings that contained
 	// nil values, see lp#1667199. When importing, we want these stripped.
-	removeNils(a.Settings())
+	removeNils(a.CharmConfig())
 
 	ops, err := addApplicationOps(i.st, app, addApplicationOpsArgs{
 		applicationDoc:     appDoc,
 		statusDoc:          statusDoc,
 		constraints:        i.constraints(a.Constraints()),
 		storage:            i.storageConstraints(a.StorageConstraints()),
-		settings:           a.Settings(),
+		settings:           a.CharmConfig(),
 		leadershipSettings: a.LeadershipSettings(),
 	})
 	if err != nil {
@@ -1100,10 +1100,24 @@ func (i *importer) relation(rel description.Relation) error {
 			Insert: relationDoc,
 		},
 	}
-	if rel.Status() != nil {
-		status := i.makeStatusDoc(rel.Status())
-		ops = append(ops, createStatusOp(i.im.mb, relationGlobalScope(rel.Id()), status))
+
+	var relStatusDoc statusDoc
+	relStatus := rel.Status()
+	if relStatus != nil {
+		relStatusDoc = i.makeStatusDoc(relStatus)
+	} else {
+		// Relations are marked as either
+		// joining or joined, depending on
+		// whether there are any units in scope.
+		relStatusDoc = statusDoc{
+			Status:  status.Joining,
+			Updated: time.Now().UnixNano(),
+		}
+		if relationDoc.UnitCount > 0 {
+			relStatusDoc.Status = status.Joined
+		}
 	}
+	ops = append(ops, createStatusOp(i.im.mb, relationGlobalScope(rel.Id()), relStatusDoc))
 
 	dbRelation := newRelation(i.st, relationDoc)
 	// Add an op that adds the relation scope document for each
@@ -1385,10 +1399,20 @@ func (i *importer) sshHostKeys() error {
 func (i *importer) cloudimagemetadata() error {
 	i.logger.Debugf("importing cloudimagemetadata")
 	images := i.model.CloudImageMetadata()
-	metadatas := make([]cloudimagemetadata.Metadata, len(images))
-	for index, image := range images {
-		metadatas[index] = cloudimagemetadata.Metadata{
-			cloudimagemetadata.MetadataAttributes{
+	var metadatas []cloudimagemetadata.Metadata
+	for _, image := range images {
+		// We only want to import custom (user defined metadata).
+		// Everything else *now* expires after a set time anyway and
+		// coming from Juju < 2.3.4 would result in non-expiring metadata.
+		if image.Source() != "custom" {
+			continue
+		}
+		var rootStoragePtr *uint64
+		if rootStorageSize, ok := image.RootStorageSize(); ok {
+			rootStoragePtr = &rootStorageSize
+		}
+		metadatas = append(metadatas, cloudimagemetadata.Metadata{
+			MetadataAttributes: cloudimagemetadata.MetadataAttributes{
 				Source:          image.Source(),
 				Stream:          image.Stream(),
 				Region:          image.Region(),
@@ -1396,12 +1420,13 @@ func (i *importer) cloudimagemetadata() error {
 				Series:          image.Series(),
 				Arch:            image.Arch(),
 				RootStorageType: image.RootStorageType(),
+				RootStorageSize: rootStoragePtr,
 				VirtType:        image.VirtType(),
 			},
-			image.Priority(),
-			image.ImageId(),
-			image.DateCreated(),
-		}
+			Priority:    image.Priority(),
+			ImageId:     image.ImageId(),
+			DateCreated: image.DateCreated(),
+		})
 	}
 	err := i.st.CloudImageMetadataStorage.SaveMetadata(metadatas)
 	if err != nil {
