@@ -672,6 +672,8 @@ func (s *MigrationImportSuite) TestRelations(c *gc.C) {
 	c.Assert(err, jc.ErrorIsNil)
 	rel, err := s.State.AddRelation(eps...)
 	c.Assert(err, jc.ErrorIsNil)
+	err = rel.SetStatus(status.StatusInfo{Status: status.Joined})
+	c.Assert(err, jc.ErrorIsNil)
 	wordpress_0 := s.Factory.MakeUnit(c, &factory.UnitParams{Application: wordpress})
 
 	ru, err := rel.Unit(wordpress_0)
@@ -696,7 +698,7 @@ func (s *MigrationImportSuite) TestRelations(c *gc.C) {
 
 	relStatus, err := rels[0].Status()
 	c.Assert(err, jc.ErrorIsNil)
-	c.Assert(relStatus.Status, gc.Equals, status.Joining)
+	c.Assert(relStatus.Status, gc.Equals, status.Joined)
 
 	ru, err = rels[0].Unit(units[0])
 	c.Assert(err, jc.ErrorIsNil)
@@ -706,6 +708,57 @@ func (s *MigrationImportSuite) TestRelations(c *gc.C) {
 	c.Assert(settings.Map(), gc.DeepEquals, relSettings)
 }
 
+func (s *MigrationImportSuite) assertRelationsMissingStatus(c *gc.C, hasUnits bool) {
+	wordpress := state.AddTestingApplication(c, s.State, "wordpress", state.AddTestingCharm(c, s.State, "wordpress"))
+	state.AddTestingApplication(c, s.State, "mysql", state.AddTestingCharm(c, s.State, "mysql"))
+	eps, err := s.State.InferEndpoints("mysql", "wordpress")
+	c.Assert(err, jc.ErrorIsNil)
+	rel, err := s.State.AddRelation(eps...)
+	c.Assert(err, jc.ErrorIsNil)
+
+	if hasUnits {
+		wordpress_0 := s.Factory.MakeUnit(c, &factory.UnitParams{Application: wordpress})
+		ru, err := rel.Unit(wordpress_0)
+		c.Assert(err, jc.ErrorIsNil)
+		relSettings := map[string]interface{}{
+			"name": "wordpress/0",
+		}
+		err = ru.EnterScope(relSettings)
+		c.Assert(err, jc.ErrorIsNil)
+	}
+
+	_, newSt := s.importModel(c, func(desc map[string]interface{}) {
+		relations := desc["relations"].(map[interface{}]interface{})
+		for _, item := range relations["relations"].([]interface{}) {
+			relation := item.(map[interface{}]interface{})
+			delete(relation, "status")
+		}
+	})
+
+	newWordpress, err := newSt.Application("wordpress")
+	c.Assert(err, jc.ErrorIsNil)
+	c.Assert(state.RelationCount(newWordpress), gc.Equals, 1)
+	rels, err := newWordpress.Relations()
+	c.Assert(err, jc.ErrorIsNil)
+	c.Assert(rels, gc.HasLen, 1)
+
+	relStatus, err := rels[0].Status()
+	c.Assert(err, jc.ErrorIsNil)
+	if hasUnits {
+		c.Assert(relStatus.Status, gc.Equals, status.Joined)
+	} else {
+		c.Assert(relStatus.Status, gc.Equals, status.Joining)
+	}
+}
+
+func (s *MigrationImportSuite) TestRelationsMissingStatusWithUnits(c *gc.C) {
+	s.assertRelationsMissingStatus(c, true)
+}
+
+func (s *MigrationImportSuite) TestRelationsMissingStatusNoUnits(c *gc.C) {
+	s.assertRelationsMissingStatus(c, false)
+}
+
 func (s *MigrationImportSuite) TestEndpointBindings(c *gc.C) {
 	// Endpoint bindings need both valid charms, applications, and spaces.
 	s.Factory.MakeSpace(c, &factory.SpaceParams{
@@ -993,31 +1046,54 @@ func (s *MigrationImportSuite) TestCloudImageMetadata(c *gc.C) {
 		RootStorageSize: &storageSize,
 		Source:          "test",
 	}
-	metadata := []cloudimagemetadata.Metadata{{attrs, 2, "1", 2}}
+	attrsCustom := cloudimagemetadata.MetadataAttributes{
+		Stream:          "stream",
+		Region:          "region-custom",
+		Version:         "14.04",
+		Series:          "trusty",
+		Arch:            "arch",
+		VirtType:        "virtType-test",
+		RootStorageType: "rootStorageType-test",
+		RootStorageSize: &storageSize,
+		Source:          "custom",
+	}
+	metadata := []cloudimagemetadata.Metadata{
+		{attrs, 2, "1", 2},
+		{attrsCustom, 3, "2", 3},
+	}
 
 	err := s.State.CloudImageMetadataStorage.SaveMetadata(metadata)
 	c.Assert(err, jc.ErrorIsNil)
 
-	_, newSt := s.importModel(c)
+	_, newSt := s.importModel(c, func(map[string]interface{}) {
+		// Image metadata collection is global so we need to delete it
+		// to properly test import.
+		all, err := s.State.CloudImageMetadataStorage.AllCloudImageMetadata()
+		c.Assert(err, jc.ErrorIsNil)
+		for _, m := range all {
+			err := s.State.CloudImageMetadataStorage.DeleteMetadata(m.ImageId)
+			c.Assert(err, jc.ErrorIsNil)
+		}
+	})
 	defer func() {
 		c.Assert(newSt.Close(), jc.ErrorIsNil)
 	}()
 
-	images, err := s.State.CloudImageMetadataStorage.AllCloudImageMetadata()
+	images, err := newSt.CloudImageMetadataStorage.AllCloudImageMetadata()
 	c.Assert(err, jc.ErrorIsNil)
 	c.Assert(images, gc.HasLen, 1)
 	image := images[0]
 	c.Check(image.Stream, gc.Equals, "stream")
-	c.Check(image.Region, gc.Equals, "region-test")
+	c.Check(image.Region, gc.Equals, "region-custom")
 	c.Check(image.Version, gc.Equals, "14.04")
 	c.Check(image.Arch, gc.Equals, "arch")
 	c.Check(image.VirtType, gc.Equals, "virtType-test")
 	c.Check(image.RootStorageType, gc.Equals, "rootStorageType-test")
 	c.Check(*image.RootStorageSize, gc.Equals, uint64(3))
-	c.Check(image.Source, gc.Equals, "test")
-	c.Check(image.Priority, gc.Equals, 2)
-	c.Check(image.ImageId, gc.Equals, "1")
-	c.Check(image.DateCreated, gc.Equals, int64(2))
+	c.Check(image.Source, gc.Equals, "custom")
+	c.Check(image.Priority, gc.Equals, 3)
+	c.Check(image.ImageId, gc.Equals, "2")
+	c.Check(image.DateCreated, gc.Equals, int64(3))
 }
 
 func (s *MigrationImportSuite) TestAction(c *gc.C) {
