@@ -20,6 +20,7 @@ import (
 	"github.com/juju/juju/constraints"
 	"github.com/juju/juju/environs/config"
 	"github.com/juju/juju/feature"
+	"github.com/juju/juju/mongo/utils"
 	"github.com/juju/juju/permission"
 	"github.com/juju/juju/status"
 	"github.com/juju/juju/storage"
@@ -197,18 +198,28 @@ func (st *State) Model() (*Model, error) {
 	return model, nil
 }
 
-// AllModelUUIDs returns the UUIDs for all models in the controller.
+// AllModelUUIDs returns the UUIDs for all non-dead models in the controller.
 // Results are sorted by (name, owner).
 func (st *State) AllModelUUIDs() ([]string, error) {
+	return st.filteredModelUUIDs(notDeadDoc)
+}
+
+// AllModelUUIDsIncludingDead returns the UUIDs for all models in the controller.
+// Results are sorted by (name, owner).
+func (st *State) AllModelUUIDsIncludingDead() ([]string, error) {
+	return st.filteredModelUUIDs(nil)
+}
+
+// filteredModelUUIDs returns all model uuids that match the filter.
+func (st *State) filteredModelUUIDs(filter bson.D) ([]string, error) {
 	models, closer := st.db().GetCollection(modelsC)
 	defer closer()
 
 	var docs []bson.M
-	err := models.Find(nil).Sort("name", "owner").Select(bson.M{"_id": 1}).All(&docs)
+	err := models.Find(filter).Sort("name", "owner").Select(bson.M{"_id": 1}).All(&docs)
 	if err != nil {
 		return nil, err
 	}
-
 	out := make([]string, len(docs))
 	for i, doc := range docs {
 		out[i] = doc["_id"].(string)
@@ -425,14 +436,14 @@ func (st *State) NewModel(args ModelArgs) (_ *Model, _ *State, err error) {
 		} else if envCount > 0 {
 			err = errors.AlreadyExistsf("model %q for %s", name, owner.Id())
 		} else {
-			err = errors.New("model already exists")
+			err = errors.Annotate(err, "failed to create new model")
 		}
 	}
 	if err != nil {
 		return nil, nil, errors.Trace(err)
 	}
 
-	err = newSt.start(st.controllerTag)
+	err = newSt.start(st.controllerTag, nil)
 	if err != nil {
 		return nil, nil, errors.Annotate(err, "could not start state for new model")
 	}
@@ -472,7 +483,7 @@ func validateCloudRegion(cloud jujucloud.Cloud, regionName string) (txn.Op, erro
 			return txn.Op{}, errors.Trace(err)
 		}
 		assertCloudRegionOp.Assert = bson.D{
-			{"regions." + region.Name, bson.D{{"$exists", true}}},
+			{"regions." + utils.EscapeKey(region.Name), bson.D{{"$exists", true}}},
 		}
 	} else {
 		if len(cloud.Regions) > 0 {
@@ -1102,7 +1113,7 @@ func (m *Model) destroyOps(
 				im, modelEntityRefs,
 			)
 			if err != nil {
-				return nil, err
+				return nil, errors.Trace(err)
 			}
 			prereqOps = storageOps
 		}
@@ -1130,18 +1141,18 @@ func (m *Model) destroyOps(
 		pool := NewStatePool(m.st)
 		defer pool.Close()
 
-		modelUUIDs, err := m.st.AllModelUUIDs()
+		modelUUIDs, err := m.st.AllModelUUIDsIncludingDead()
 		if err != nil {
 			return nil, errors.Trace(err)
 		}
 		var aliveEmpty, aliveNonEmpty, dying, dead int
-		for _, modelUUID := range modelUUIDs {
-			if modelUUID == m.UUID() {
+		for _, aModelUUID := range modelUUIDs {
+			if aModelUUID == m.UUID() {
 				// Ignore the controller model.
 				continue
 			}
 
-			st, release, err := pool.Get(modelUUID)
+			st, release, err := pool.Get(aModelUUID)
 			if err != nil {
 				// This model could have been removed.
 				if errors.IsNotFound(err) {
