@@ -5,7 +5,6 @@ package state_test
 
 import (
 	"fmt"
-	"sort"
 
 	"github.com/juju/errors"
 	jc "github.com/juju/testing/checkers"
@@ -27,15 +26,14 @@ type ModelUserSuite struct {
 var _ = gc.Suite(&ModelUserSuite{})
 
 func (s *ModelUserSuite) TestAddModelUser(c *gc.C) {
-	now := s.State.NowToTheSecond()
+	now := state.NowToTheSecond(s.State)
 	user := s.Factory.MakeUser(c,
 		&factory.UserParams{
 			Name:        "validusername",
 			NoModelUser: true,
 		})
 	createdBy := s.Factory.MakeUser(c, &factory.UserParams{Name: "createdby"})
-	modelUser, err := s.State.AddModelUser(
-		s.State.ModelUUID(),
+	modelUser, err := s.Model.AddUser(
 		state.UserAccessSpec{
 			User:      user.UserTag(),
 			CreatedBy: createdBy.UserTag(),
@@ -50,11 +48,11 @@ func (s *ModelUserSuite) TestAddModelUser(c *gc.C) {
 	c.Assert(modelUser.Access, gc.Equals, permission.WriteAccess)
 	c.Assert(modelUser.CreatedBy.Id(), gc.Equals, "createdby")
 	c.Assert(modelUser.DateCreated.Equal(now) || modelUser.DateCreated.After(now), jc.IsTrue)
-	when, err := s.State.LastModelConnection(modelUser.UserTag)
+	when, err := s.Model.LastModelConnection(modelUser.UserTag)
 	c.Assert(err, jc.Satisfies, state.IsNeverConnectedError)
 	c.Assert(when.IsZero(), jc.IsTrue)
 
-	modelUser, err = s.State.UserAccess(user.UserTag(), s.State.ModelTag())
+	modelUser, err = s.State.UserAccess(user.UserTag(), s.IAASModel.ModelTag())
 	c.Assert(err, jc.ErrorIsNil)
 	c.Assert(modelUser.UserID, gc.Equals, fmt.Sprintf("%s:validusername", s.modelTag.Id()))
 	c.Assert(modelUser.Object, gc.Equals, s.modelTag)
@@ -63,7 +61,7 @@ func (s *ModelUserSuite) TestAddModelUser(c *gc.C) {
 	c.Assert(modelUser.Access, gc.Equals, permission.WriteAccess)
 	c.Assert(modelUser.CreatedBy.Id(), gc.Equals, "createdby")
 	c.Assert(modelUser.DateCreated.Equal(now) || modelUser.DateCreated.After(now), jc.IsTrue)
-	when, err = s.State.LastModelConnection(modelUser.UserTag)
+	when, err = s.Model.LastModelConnection(modelUser.UserTag)
 	c.Assert(err, jc.Satisfies, state.IsNeverConnectedError)
 	c.Assert(when.IsZero(), jc.IsTrue)
 }
@@ -75,8 +73,7 @@ func (s *ModelUserSuite) TestAddReadOnlyModelUser(c *gc.C) {
 			NoModelUser: true,
 		})
 	createdBy := s.Factory.MakeUser(c, &factory.UserParams{Name: "createdby"})
-	modelUser, err := s.State.AddModelUser(
-		s.State.ModelUUID(),
+	modelUser, err := s.Model.AddUser(
 		state.UserAccessSpec{
 			User:      user.UserTag(),
 			CreatedBy: createdBy.UserTag(),
@@ -89,7 +86,7 @@ func (s *ModelUserSuite) TestAddReadOnlyModelUser(c *gc.C) {
 	c.Assert(modelUser.Access, gc.Equals, permission.ReadAccess)
 
 	// Make sure that it is set when we read the user out.
-	modelUser, err = s.State.UserAccess(user.UserTag(), s.State.ModelTag())
+	modelUser, err = s.State.UserAccess(user.UserTag(), s.IAASModel.ModelTag())
 	c.Assert(err, jc.ErrorIsNil)
 	c.Assert(modelUser.UserName, gc.Equals, "validusername")
 	c.Assert(modelUser.Access, gc.Equals, permission.ReadAccess)
@@ -102,8 +99,7 @@ func (s *ModelUserSuite) TestAddReadWriteModelUser(c *gc.C) {
 			NoModelUser: true,
 		})
 	createdBy := s.Factory.MakeUser(c, &factory.UserParams{Name: "createdby"})
-	modelUser, err := s.State.AddModelUser(
-		s.State.ModelUUID(),
+	modelUser, err := s.Model.AddUser(
 		state.UserAccessSpec{
 			User:      user.UserTag(),
 			CreatedBy: createdBy.UserTag(),
@@ -116,7 +112,7 @@ func (s *ModelUserSuite) TestAddReadWriteModelUser(c *gc.C) {
 	c.Assert(modelUser.Access, gc.Equals, permission.WriteAccess)
 
 	// Make sure that it is set when we read the user out.
-	modelUser, err = s.State.UserAccess(user.UserTag(), s.State.ModelTag())
+	modelUser, err = s.State.UserAccess(user.UserTag(), s.IAASModel.ModelTag())
 	c.Assert(err, jc.ErrorIsNil)
 	c.Assert(modelUser.UserName, gc.Equals, "validusername")
 	c.Assert(modelUser.Access, gc.Equals, permission.WriteAccess)
@@ -129,8 +125,7 @@ func (s *ModelUserSuite) TestAddAdminModelUser(c *gc.C) {
 			NoModelUser: true,
 		})
 	createdBy := s.Factory.MakeUser(c, &factory.UserParams{Name: "createdby"})
-	modelUser, err := s.State.AddModelUser(
-		s.State.ModelUUID(),
+	modelUser, err := s.Model.AddUser(
 		state.UserAccessSpec{
 			User:      user.UserTag(),
 			CreatedBy: createdBy.UserTag(),
@@ -143,7 +138,7 @@ func (s *ModelUserSuite) TestAddAdminModelUser(c *gc.C) {
 	c.Assert(modelUser.Access, gc.Equals, permission.AdminAccess)
 
 	// Make sure that it is set when we read the user out.
-	modelUser, err = s.State.UserAccess(user.UserTag(), s.State.ModelTag())
+	modelUser, err = s.State.UserAccess(user.UserTag(), s.IAASModel.ModelTag())
 	c.Assert(err, jc.ErrorIsNil)
 	c.Assert(modelUser.UserName, gc.Equals, "validusername")
 	c.Assert(modelUser.Access, gc.Equals, permission.AdminAccess)
@@ -156,8 +151,7 @@ func (s *ModelUserSuite) TestDefaultAccessModelUser(c *gc.C) {
 			NoModelUser: true,
 		})
 	createdBy := s.Factory.MakeUser(c, &factory.UserParams{Name: "createdby"})
-	modelUser, err := s.State.AddModelUser(
-		s.State.ModelUUID(),
+	modelUser, err := s.Model.AddUser(
 		state.UserAccessSpec{
 			User:      user.UserTag(),
 			CreatedBy: createdBy.UserTag(),
@@ -174,8 +168,7 @@ func (s *ModelUserSuite) TestSetAccessModelUser(c *gc.C) {
 			NoModelUser: true,
 		})
 	createdBy := s.Factory.MakeUser(c, &factory.UserParams{Name: "createdby"})
-	modelUser, err := s.State.AddModelUser(
-		s.State.ModelUUID(),
+	modelUser, err := s.Model.AddUser(
 		state.UserAccessSpec{
 			User:      user.UserTag(),
 			CreatedBy: createdBy.UserTag(),
@@ -184,9 +177,9 @@ func (s *ModelUserSuite) TestSetAccessModelUser(c *gc.C) {
 	c.Assert(err, jc.ErrorIsNil)
 	c.Assert(modelUser.Access, gc.Equals, permission.AdminAccess)
 
-	s.State.SetUserAccess(modelUser.UserTag, s.State.ModelTag(), permission.ReadAccess)
+	s.State.SetUserAccess(modelUser.UserTag, s.IAASModel.ModelTag(), permission.ReadAccess)
 
-	modelUser, err = s.State.UserAccess(user.UserTag(), s.State.ModelTag())
+	modelUser, err = s.State.UserAccess(user.UserTag(), s.IAASModel.ModelTag())
 	c.Assert(modelUser.Access, gc.Equals, permission.ReadAccess)
 }
 
@@ -194,8 +187,7 @@ func (s *ModelUserSuite) TestCaseUserNameVsId(c *gc.C) {
 	model, err := s.State.Model()
 	c.Assert(err, jc.ErrorIsNil)
 
-	user, err := s.State.AddModelUser(
-		s.State.ModelUUID(),
+	user, err := s.Model.AddUser(
 		state.UserAccessSpec{
 			User:      names.NewUserTag("Bob@RandomProvider"),
 			CreatedBy: model.Owner(),
@@ -211,8 +203,7 @@ func (s *ModelUserSuite) TestCaseSensitiveModelUserErrors(c *gc.C) {
 	c.Assert(err, jc.ErrorIsNil)
 	s.Factory.MakeModelUser(c, &factory.ModelUserParams{User: "Bob@ubuntuone"})
 
-	_, err = s.State.AddModelUser(
-		s.State.ModelUUID(),
+	_, err = s.Model.AddUser(
 		state.UserAccessSpec{
 			User:      names.NewUserTag("boB@ubuntuone"),
 			CreatedBy: model.Owner(),
@@ -227,14 +218,20 @@ func (s *ModelUserSuite) TestCaseInsensitiveLookupInMultiEnvirons(c *gc.C) {
 		f := factory.NewFactory(st1)
 		expectedUser := f.MakeModelUser(c, &factory.ModelUserParams{User: usernames[0]})
 
+		m1, err := st1.Model()
+		c.Assert(err, jc.ErrorIsNil)
+
+		m2, err := st2.Model()
+		c.Assert(err, jc.ErrorIsNil)
+
 		// assert case insensitive lookup for each username
 		for _, username := range usernames {
 			userTag := names.NewUserTag(username)
-			obtainedUser, err := st1.UserAccess(userTag, st1.ModelTag())
+			obtainedUser, err := st1.UserAccess(userTag, m1.ModelTag())
 			c.Assert(err, jc.ErrorIsNil)
 			c.Assert(obtainedUser, gc.DeepEquals, expectedUser)
 
-			_, err = st2.UserAccess(userTag, st2.ModelTag())
+			_, err = st2.UserAccess(userTag, m2.ModelTag())
 			c.Assert(errors.IsNotFound(err), jc.IsTrue)
 		}
 	}
@@ -263,8 +260,7 @@ func (s *ModelUserSuite) TestAddModelDisplayName(c *gc.C) {
 
 func (s *ModelUserSuite) TestAddModelNoUserFails(c *gc.C) {
 	createdBy := s.Factory.MakeUser(c, &factory.UserParams{Name: "createdby"})
-	_, err := s.State.AddModelUser(
-		s.State.ModelUUID(),
+	_, err := s.Model.AddUser(
 		state.UserAccessSpec{
 			User:      names.NewLocalUserTag("validusername"),
 			CreatedBy: createdBy.UserTag(),
@@ -275,8 +271,7 @@ func (s *ModelUserSuite) TestAddModelNoUserFails(c *gc.C) {
 
 func (s *ModelUserSuite) TestAddModelNoCreatedByUserFails(c *gc.C) {
 	user := s.Factory.MakeUser(c, &factory.UserParams{Name: "validusername"})
-	_, err := s.State.AddModelUser(
-		s.State.ModelUUID(),
+	_, err := s.Model.AddUser(
 		state.UserAccessSpec{
 			User:      user.UserTag(),
 			CreatedBy: names.NewLocalUserTag("createdby"),
@@ -287,31 +282,31 @@ func (s *ModelUserSuite) TestAddModelNoCreatedByUserFails(c *gc.C) {
 
 func (s *ModelUserSuite) TestRemoveModelUser(c *gc.C) {
 	user := s.Factory.MakeUser(c, &factory.UserParams{Name: "validUsername"})
-	_, err := s.State.UserAccess(user.UserTag(), s.State.ModelTag())
+	_, err := s.State.UserAccess(user.UserTag(), s.IAASModel.ModelTag())
 	c.Assert(err, jc.ErrorIsNil)
 
-	err = s.State.RemoveUserAccess(user.UserTag(), s.State.ModelTag())
+	err = s.State.RemoveUserAccess(user.UserTag(), s.IAASModel.ModelTag())
 	c.Assert(err, jc.ErrorIsNil)
 
-	_, err = s.State.UserAccess(user.UserTag(), s.State.ModelTag())
+	_, err = s.State.UserAccess(user.UserTag(), s.IAASModel.ModelTag())
 	c.Assert(err, jc.Satisfies, errors.IsNotFound)
 }
 
 func (s *ModelUserSuite) TestRemoveModelUserFails(c *gc.C) {
 	user := s.Factory.MakeUser(c, &factory.UserParams{NoModelUser: true})
-	err := s.State.RemoveUserAccess(user.UserTag(), s.State.ModelTag())
+	err := s.State.RemoveUserAccess(user.UserTag(), s.IAASModel.ModelTag())
 	c.Assert(err, jc.Satisfies, errors.IsNotFound)
 }
 
 func (s *ModelUserSuite) TestUpdateLastConnection(c *gc.C) {
-	now := s.State.NowToTheSecond()
+	now := state.NowToTheSecond(s.State)
 	createdBy := s.Factory.MakeUser(c, &factory.UserParams{Name: "createdby"})
 	user := s.Factory.MakeUser(c, &factory.UserParams{Name: "validusername", Creator: createdBy.Tag()})
-	modelUser, err := s.State.UserAccess(user.UserTag(), s.State.ModelTag())
+	modelUser, err := s.State.UserAccess(user.UserTag(), s.IAASModel.ModelTag())
 	c.Assert(err, jc.ErrorIsNil)
-	err = s.State.UpdateLastModelConnection(user.UserTag())
+	err = s.Model.UpdateLastModelConnection(user.UserTag())
 	c.Assert(err, jc.ErrorIsNil)
-	when, err := s.State.LastModelConnection(modelUser.UserTag)
+	when, err := s.Model.LastModelConnection(modelUser.UserTag)
 	c.Assert(err, jc.ErrorIsNil)
 	// It is possible that the update is done over a second boundary, so we need
 	// to check for after now as well as equal.
@@ -319,19 +314,20 @@ func (s *ModelUserSuite) TestUpdateLastConnection(c *gc.C) {
 }
 
 func (s *ModelUserSuite) TestUpdateLastConnectionTwoModelUsers(c *gc.C) {
-	now := s.State.NowToTheSecond()
+	now := state.NowToTheSecond(s.State)
 
 	// Create a user and add them to the inital model.
 	createdBy := s.Factory.MakeUser(c, &factory.UserParams{Name: "createdby"})
 	user := s.Factory.MakeUser(c, &factory.UserParams{Name: "validusername", Creator: createdBy.Tag()})
-	modelUser, err := s.State.UserAccess(user.UserTag(), s.State.ModelTag())
+	modelUser, err := s.State.UserAccess(user.UserTag(), s.IAASModel.ModelTag())
 	c.Assert(err, jc.ErrorIsNil)
 
 	// Create a second model and add the same user to this.
 	st2 := s.Factory.MakeModel(c, nil)
 	defer st2.Close()
-	modelUser2, err := st2.AddModelUser(
-		st2.ModelUUID(),
+	model2, err := st2.Model()
+	c.Assert(err, jc.ErrorIsNil)
+	modelUser2, err := model2.AddUser(
 		state.UserAccessSpec{
 			User:      user.UserTag(),
 			CreatedBy: createdBy.UserTag(),
@@ -343,161 +339,103 @@ func (s *ModelUserSuite) TestUpdateLastConnectionTwoModelUsers(c *gc.C) {
 	// separate last connections.
 
 	// Connect modelUser and get last connection.
-	err = s.State.UpdateLastModelConnection(user.UserTag())
+	err = s.Model.UpdateLastModelConnection(user.UserTag())
 	c.Assert(err, jc.ErrorIsNil)
-	when, err := s.State.LastModelConnection(modelUser.UserTag)
+	when, err := s.Model.LastModelConnection(modelUser.UserTag)
 	c.Assert(err, jc.ErrorIsNil)
 	c.Assert(when.After(now) || when.Equal(now), jc.IsTrue)
 
 	// Try to get last connection for modelUser2. As they have never connected,
 	// we expect to get an error.
-	_, err = st2.LastModelConnection(modelUser2.UserTag)
+	_, err = model2.LastModelConnection(modelUser2.UserTag)
 	c.Assert(err, gc.ErrorMatches, `never connected: "validusername"`)
 
 	// Connect modelUser2 and get last connection.
-	err = s.State.UpdateLastModelConnection(modelUser2.UserTag)
+	err = s.Model.UpdateLastModelConnection(modelUser2.UserTag)
 	c.Assert(err, jc.ErrorIsNil)
-	when, err = s.State.LastModelConnection(modelUser2.UserTag)
+	when, err = s.Model.LastModelConnection(modelUser2.UserTag)
 	c.Assert(err, jc.ErrorIsNil)
 	c.Assert(when.After(now) || when.Equal(now), jc.IsTrue)
 }
 
-func (s *ModelUserSuite) TestModelsForUserNone(c *gc.C) {
+func (s *ModelUserSuite) TestModelUUIDsForUserNone(c *gc.C) {
 	tag := names.NewUserTag("non-existent@remote")
-	models, err := s.State.ModelsForUser(tag)
+	models, err := s.State.ModelUUIDsForUser(tag)
 	c.Assert(err, jc.ErrorIsNil)
 	c.Assert(models, gc.HasLen, 0)
 }
 
-func (s *ModelUserSuite) TestModelsForUserNewLocalUser(c *gc.C) {
+func (s *ModelUserSuite) TestModelUUIDsForUserNewLocalUser(c *gc.C) {
 	user := s.Factory.MakeUser(c, &factory.UserParams{NoModelUser: true})
-	models, err := s.State.ModelsForUser(user.UserTag())
+	models, err := s.State.ModelUUIDsForUser(user.UserTag())
 	c.Assert(err, jc.ErrorIsNil)
 	c.Assert(models, gc.HasLen, 0)
 }
 
-func (s *ModelUserSuite) TestModelsForUser(c *gc.C) {
+func (s *ModelUserSuite) TestModelUUIDsForUser(c *gc.C) {
 	user := s.Factory.MakeUser(c, nil)
-	models, err := s.State.ModelsForUser(user.UserTag())
+	models, err := s.State.ModelUUIDsForUser(user.UserTag())
 	c.Assert(err, jc.ErrorIsNil)
-	c.Assert(models, gc.HasLen, 1)
-	c.Assert(models[0].UUID(), gc.Equals, s.State.ModelUUID())
-	st, err := s.State.ForModel(models[0].ModelTag())
+	c.Assert(models, jc.DeepEquals, []string{s.State.ModelUUID()})
+
+	modelTag := names.NewModelTag(models[0])
+	st, err := s.State.ForModel(modelTag)
 	c.Assert(err, jc.ErrorIsNil)
-	modelUser, err := s.State.UserAccess(user.UserTag(), models[0].ModelTag())
-	when, err := st.LastModelConnection(modelUser.UserTag)
+
+	access, err := s.State.UserAccess(user.UserTag(), modelTag)
+	when, err := s.Model.LastModelConnection(access.UserTag)
 	c.Assert(err, jc.Satisfies, state.IsNeverConnectedError)
 	c.Assert(when.IsZero(), jc.IsTrue)
 	c.Assert(st.Close(), jc.ErrorIsNil)
 }
 
-func (s *ModelUserSuite) TestDeadModelsForUser(c *gc.C) {
+func (s *ModelUserSuite) TestImportingModelUUIDsForUser(c *gc.C) {
 	user := s.Factory.MakeUser(c, nil)
-	models, err := s.State.ModelsForUser(user.UserTag())
+	models, err := s.State.ModelUUIDsForUser(user.UserTag())
 	c.Assert(err, jc.ErrorIsNil)
-	c.Assert(models, gc.HasLen, 1)
-	c.Assert(models[0].UUID(), gc.Equals, s.State.ModelUUID())
+	c.Assert(models, jc.DeepEquals, []string{s.State.ModelUUID()})
 
-	err = models[0].SetDead()
+	model, err := s.State.Model()
 	c.Assert(err, jc.ErrorIsNil)
-	models, err = s.State.ModelsForUser(user.UserTag())
+	err = model.SetMigrationMode(state.MigrationModeImporting)
+	c.Assert(err, jc.ErrorIsNil)
+
+	models, err = s.State.ModelUUIDsForUser(user.UserTag())
 	c.Assert(err, jc.ErrorIsNil)
 	c.Assert(models, gc.HasLen, 0)
 }
 
-func (s *ModelUserSuite) TestImportingModelsForUser(c *gc.C) {
-	user := s.Factory.MakeUser(c, nil)
-	models, err := s.State.ModelsForUser(user.UserTag())
-	c.Assert(err, jc.ErrorIsNil)
-	c.Assert(models, gc.HasLen, 1)
-	c.Assert(models[0].UUID(), gc.Equals, s.State.ModelUUID())
-
-	err = models[0].SetMigrationMode(state.MigrationModeImporting)
-	c.Assert(err, jc.ErrorIsNil)
-	models, err = s.State.ModelsForUser(user.UserTag())
-	c.Assert(err, jc.ErrorIsNil)
-	c.Assert(models, gc.HasLen, 0)
-}
-
-func (s *ModelUserSuite) newEnvWithOwner(c *gc.C, name string, owner names.UserTag) *state.Model {
-	// Don't use the factory to call MakeModel because it may at some
-	// time in the future be modified to do additional things.  Instead call
-	// the state method directly to create an model to make sure that
-	// the owner is able to access the model.
-	uuid, err := utils.NewUUID()
-	c.Assert(err, jc.ErrorIsNil)
-	cfg := testing.CustomModelConfig(c, testing.Attrs{
-		"name": name,
-		"uuid": uuid.String(),
-	})
-	model, st, err := s.State.NewModel(state.ModelArgs{
-		CloudName: "dummy", CloudRegion: "dummy-region", Config: cfg, Owner: owner,
-		StorageProviderRegistry: storage.StaticProviderRegistry{},
-	})
-	c.Assert(err, jc.ErrorIsNil)
-	defer st.Close()
-	return model
-}
-
-func (s *ModelUserSuite) TestModelsForUserEnvOwner(c *gc.C) {
+func (s *ModelUserSuite) TestModelUUIDsForUserModelOwner(c *gc.C) {
 	owner := names.NewUserTag("external@remote")
-	model := s.newEnvWithOwner(c, "test-model", owner)
+	model := s.newModelWithOwner(c, owner)
 
-	models, err := s.State.ModelsForUser(owner)
+	models, err := s.State.ModelUUIDsForUser(owner)
 	c.Assert(err, jc.ErrorIsNil)
-	c.Assert(models, gc.HasLen, 1)
-	s.checkSameModel(c, models[0].Model, model)
+	c.Assert(models, jc.DeepEquals, []string{model.UUID()})
 }
 
-func (s *ModelUserSuite) checkSameModel(c *gc.C, env1, env2 *state.Model) {
-	c.Check(env1.Name(), gc.Equals, env2.Name())
-	c.Check(env1.UUID(), gc.Equals, env2.UUID())
-}
-
-func (s *ModelUserSuite) newEnvWithUser(c *gc.C, name string, user names.UserTag) *state.Model {
-	envState := s.Factory.MakeModel(c, &factory.ModelParams{Name: name})
-	defer envState.Close()
-	newEnv, err := envState.Model()
-	c.Assert(err, jc.ErrorIsNil)
-
-	_, err = envState.AddModelUser(
-		envState.ModelUUID(),
-		state.UserAccessSpec{
-			User: user, CreatedBy: newEnv.Owner(),
-			Access: permission.ReadAccess,
-		})
-	c.Assert(err, jc.ErrorIsNil)
-	return newEnv
-}
-
-func (s *ModelUserSuite) TestModelsForUserOfNewEnv(c *gc.C) {
+func (s *ModelUserSuite) TestModelUUIDsForUserOfNewModel(c *gc.C) {
 	userTag := names.NewUserTag("external@remote")
-	model := s.newEnvWithUser(c, "test-model", userTag)
+	model := s.newModelWithUser(c, userTag)
 
-	models, err := s.State.ModelsForUser(userTag)
+	models, err := s.State.ModelUUIDsForUser(userTag)
 	c.Assert(err, jc.ErrorIsNil)
-	c.Assert(models, gc.HasLen, 1)
-	s.checkSameModel(c, models[0].Model, model)
+	c.Assert(models, jc.DeepEquals, []string{model.UUID()})
 }
 
-func (s *ModelUserSuite) TestModelsForUserMultiple(c *gc.C) {
+func (s *ModelUserSuite) TestModelUUIDsForUserMultiple(c *gc.C) {
 	userTag := names.NewUserTag("external@remote")
-	expected := []*state.Model{
-		s.newEnvWithUser(c, "user1", userTag),
-		s.newEnvWithUser(c, "user2", userTag),
-		s.newEnvWithUser(c, "user3", userTag),
-		s.newEnvWithOwner(c, "owner1", userTag),
-		s.newEnvWithOwner(c, "owner2", userTag),
+	expected := []string{
+		s.newModelWithUser(c, userTag).UUID(),
+		s.newModelWithUser(c, userTag).UUID(),
+		s.newModelWithUser(c, userTag).UUID(),
+		s.newModelWithOwner(c, userTag).UUID(),
+		s.newModelWithOwner(c, userTag).UUID(),
 	}
-	sort.Sort(UUIDOrder(expected))
 
-	models, err := s.State.ModelsForUser(userTag)
+	models, err := s.State.ModelUUIDsForUser(userTag)
 	c.Assert(err, jc.ErrorIsNil)
-	c.Assert(models, gc.HasLen, len(expected))
-	sort.Sort(userUUIDOrder(models))
-	for i := range expected {
-		s.checkSameModel(c, models[i].Model, expected[i])
-	}
+	c.Assert(models, jc.SameContents, expected)
 }
 
 func (s *ModelUserSuite) TestIsControllerAdmin(c *gc.C) {
@@ -536,16 +474,43 @@ func (s *ModelUserSuite) TestIsControllerAdminFromOtherState(c *gc.C) {
 	c.Assert(isAdmin, jc.IsTrue)
 }
 
-// UUIDOrder is used to sort the models into a stable order
-type UUIDOrder []*state.Model
+func (s *ModelUserSuite) newModelWithOwner(c *gc.C, owner names.UserTag) *state.Model {
+	// Don't use the factory to call MakeModel because it may at some
+	// time in the future be modified to do additional things.  Instead call
+	// the state method directly to create an model to make sure that
+	// the owner is able to access the model.
+	uuid, err := utils.NewUUID()
+	c.Assert(err, jc.ErrorIsNil)
+	uuidStr := uuid.String()
 
-func (a UUIDOrder) Len() int           { return len(a) }
-func (a UUIDOrder) Swap(i, j int)      { a[i], a[j] = a[j], a[i] }
-func (a UUIDOrder) Less(i, j int) bool { return a[i].UUID() < a[j].UUID() }
+	cfg := testing.CustomModelConfig(c, testing.Attrs{
+		"name": uuidStr[:8],
+		"uuid": uuidStr,
+	})
+	model, st, err := s.State.NewModel(state.ModelArgs{
+		Type:        state.ModelTypeIAAS,
+		CloudName:   "dummy",
+		CloudRegion: "dummy-region",
+		Config:      cfg,
+		Owner:       owner,
+		StorageProviderRegistry: storage.StaticProviderRegistry{},
+	})
+	c.Assert(err, jc.ErrorIsNil)
+	defer st.Close()
+	return model
+}
 
-// userUUIDOrder is used to sort the UserModels into a stable order
-type userUUIDOrder []*state.UserModel
+func (s *ModelUserSuite) newModelWithUser(c *gc.C, user names.UserTag) *state.Model {
+	st := s.Factory.MakeModel(c, nil)
+	defer st.Close()
+	newEnv, err := st.Model()
+	c.Assert(err, jc.ErrorIsNil)
 
-func (a userUUIDOrder) Len() int           { return len(a) }
-func (a userUUIDOrder) Swap(i, j int)      { a[i], a[j] = a[j], a[i] }
-func (a userUUIDOrder) Less(i, j int) bool { return a[i].UUID() < a[j].UUID() }
+	_, err = newEnv.AddUser(
+		state.UserAccessSpec{
+			User: user, CreatedBy: newEnv.Owner(),
+			Access: permission.ReadAccess,
+		})
+	c.Assert(err, jc.ErrorIsNil)
+	return newEnv
+}
