@@ -35,27 +35,24 @@ func (s *statePoolSuite) SetUpTest(c *gc.C) {
 	s.State2 = s.Factory.MakeModel(c, nil)
 	s.AddCleanup(func(*gc.C) { s.State2.Close() })
 	s.ModelUUID2 = s.State2.ModelUUID()
-
-	s.Pool = state.NewStatePool(s.State)
-	s.AddCleanup(func(*gc.C) { s.Pool.Close() })
 }
 
 func (s *statePoolSuite) TestGet(c *gc.C) {
-	st1, _, err := s.Pool.Get(s.ModelUUID1)
+	st1, _, err := s.StatePool.Get(s.ModelUUID1)
 	c.Assert(err, jc.ErrorIsNil)
 	c.Assert(st1.ModelUUID(), gc.Equals, s.ModelUUID1)
 
-	st2, _, err := s.Pool.Get(s.ModelUUID2)
+	st2, _, err := s.StatePool.Get(s.ModelUUID2)
 	c.Assert(err, jc.ErrorIsNil)
 	c.Assert(st2.ModelUUID(), gc.Equals, s.ModelUUID2)
 
 	// Check that the same instances are returned
 	// when a State for the same model is re-requested.
-	st1_, _, err := s.Pool.Get(s.ModelUUID1)
+	st1_, _, err := s.StatePool.Get(s.ModelUUID1)
 	c.Assert(err, jc.ErrorIsNil)
 	c.Assert(st1_, gc.Equals, st1)
 
-	st2_, _, err := s.Pool.Get(s.ModelUUID2)
+	st2_, _, err := s.StatePool.Get(s.ModelUUID2)
 	c.Assert(err, jc.ErrorIsNil)
 	c.Assert(st2_, gc.Equals, st2)
 }
@@ -63,31 +60,31 @@ func (s *statePoolSuite) TestGet(c *gc.C) {
 func (s *statePoolSuite) TestGetWithControllerModel(c *gc.C) {
 	// When a State for the controller model is requested, the same
 	// State that was original passed in should be returned.
-	st0, _, err := s.Pool.Get(s.ModelUUID)
+	st0, _, err := s.StatePool.Get(s.ModelUUID)
 	c.Assert(err, jc.ErrorIsNil)
 	c.Assert(st0, gc.Equals, s.State)
 }
 
 func (s *statePoolSuite) TestGetSystemState(c *gc.C) {
-	st0 := s.Pool.SystemState()
+	st0 := s.StatePool.SystemState()
 	c.Assert(st0, gc.Equals, s.State)
 }
 
 func (s *statePoolSuite) TestKillWorkers(c *gc.C) {
 	// Get some State instances via the pool and extract their
 	// internal workers.
-	st1, _, err := s.Pool.Get(s.ModelUUID1)
+	st1, _, err := s.StatePool.Get(s.ModelUUID1)
 	c.Assert(err, jc.ErrorIsNil)
 	w1 := state.GetInternalWorkers(st1)
 	workertest.CheckAlive(c, w1)
 
-	st2, _, err := s.Pool.Get(s.ModelUUID2)
+	st2, _, err := s.StatePool.Get(s.ModelUUID2)
 	c.Assert(err, jc.ErrorIsNil)
 	w2 := state.GetInternalWorkers(st2)
 	workertest.CheckAlive(c, w2)
 
 	// Now kill their workers.
-	s.Pool.KillWorkers()
+	s.StatePool.KillWorkers()
 
 	// Ensure the internal workers for each State died.
 	c.Check(workertest.CheckKilled(c, w1), jc.ErrorIsNil)
@@ -96,14 +93,14 @@ func (s *statePoolSuite) TestKillWorkers(c *gc.C) {
 
 func (s *statePoolSuite) TestClose(c *gc.C) {
 	// Get some State instances.
-	st1, _, err := s.Pool.Get(s.ModelUUID1)
+	st1, _, err := s.StatePool.Get(s.ModelUUID1)
 	c.Assert(err, jc.ErrorIsNil)
 
-	st2, _, err := s.Pool.Get(s.ModelUUID2)
+	st2, _, err := s.StatePool.Get(s.ModelUUID2)
 	c.Assert(err, jc.ErrorIsNil)
 
 	// Now close them.
-	err = s.Pool.Close()
+	err = s.StatePool.Close()
 	c.Assert(err, jc.ErrorIsNil)
 
 	// Confirm that controller State isn't closed.
@@ -112,27 +109,27 @@ func (s *statePoolSuite) TestClose(c *gc.C) {
 
 	// Ensure that new ones are returned if further States are
 	// requested.
-	st1_, _, err := s.Pool.Get(s.ModelUUID1)
+	st1_, _, err := s.StatePool.Get(s.ModelUUID1)
 	c.Assert(err, jc.ErrorIsNil)
 	c.Assert(st1_, gc.Not(gc.Equals), st1)
 
-	st2_, _, err := s.Pool.Get(s.ModelUUID2)
+	st2_, _, err := s.StatePool.Get(s.ModelUUID2)
 	c.Assert(err, jc.ErrorIsNil)
 	c.Assert(st2_, gc.Not(gc.Equals), st2)
 }
 
 func (s *statePoolSuite) TestTooManyReleases(c *gc.C) {
-	st, firstRelease, err := s.Pool.Get(s.ModelUUID1)
+	st, firstRelease, err := s.StatePool.Get(s.ModelUUID1)
 	c.Assert(err, jc.ErrorIsNil)
 	// Get a second reference to the same model
-	_, secondRelease, err := s.Pool.Get(s.ModelUUID1)
+	_, secondRelease, err := s.StatePool.Get(s.ModelUUID1)
 	c.Assert(err, jc.ErrorIsNil)
 
 	// Try to call the first releaser twice.
 	firstRelease()
 	firstRelease()
 
-	removed, err := s.Pool.Remove(s.ModelUUID1)
+	removed, err := s.StatePool.Remove(s.ModelUUID1)
 	c.Assert(err, jc.ErrorIsNil)
 	c.Assert(removed, jc.IsFalse)
 
@@ -145,7 +142,7 @@ func (s *statePoolSuite) TestTooManyReleases(c *gc.C) {
 }
 
 func (s *statePoolSuite) TestReleaseOnSystemStateUUID(c *gc.C) {
-	st, releaser, err := s.Pool.Get(s.ModelUUID)
+	st, releaser, err := s.StatePool.Get(s.ModelUUID)
 	c.Assert(err, jc.ErrorIsNil)
 	removed := releaser()
 	c.Assert(removed, jc.IsFalse)
@@ -153,7 +150,7 @@ func (s *statePoolSuite) TestReleaseOnSystemStateUUID(c *gc.C) {
 }
 
 func (s *statePoolSuite) TestRemoveSystemStateUUID(c *gc.C) {
-	removed, err := s.Pool.Remove(s.ModelUUID)
+	removed, err := s.StatePool.Remove(s.ModelUUID)
 	c.Assert(err, jc.ErrorIsNil)
 	c.Assert(removed, jc.IsFalse)
 	assertNotClosed(c, s.State)
@@ -170,7 +167,7 @@ func assertClosed(c *gc.C, st *state.State) {
 }
 
 func (s *statePoolSuite) TestRemoveWithNoRefsCloses(c *gc.C) {
-	st, releaser, err := s.Pool.Get(s.ModelUUID1)
+	st, releaser, err := s.StatePool.Get(s.ModelUUID1)
 	c.Assert(err, jc.ErrorIsNil)
 
 	// Confirm the state isn't closed.
@@ -178,7 +175,7 @@ func (s *statePoolSuite) TestRemoveWithNoRefsCloses(c *gc.C) {
 	c.Assert(removed, jc.IsFalse)
 	assertNotClosed(c, st)
 
-	removed, err = s.Pool.Remove(s.ModelUUID1)
+	removed, err = s.StatePool.Remove(s.ModelUUID1)
 	c.Assert(err, jc.ErrorIsNil)
 	c.Assert(removed, jc.IsTrue)
 
@@ -186,16 +183,16 @@ func (s *statePoolSuite) TestRemoveWithNoRefsCloses(c *gc.C) {
 }
 
 func (s *statePoolSuite) TestRemoveWithRefsClosesOnLastRelease(c *gc.C) {
-	st, firstRelease, err := s.Pool.Get(s.ModelUUID1)
+	st, firstRelease, err := s.StatePool.Get(s.ModelUUID1)
 	c.Assert(err, jc.ErrorIsNil)
-	_, secondRelease, err := s.Pool.Get(s.ModelUUID1)
+	_, secondRelease, err := s.StatePool.Get(s.ModelUUID1)
 	c.Assert(err, jc.ErrorIsNil)
 	// Now there are two references to the state.
 	// Sanity check!
 	assertNotClosed(c, st)
 
 	// Doesn't close while there are refs still held.
-	removed, err := s.Pool.Remove(s.ModelUUID1)
+	removed, err := s.StatePool.Remove(s.ModelUUID1)
 	c.Assert(err, jc.ErrorIsNil)
 	c.Assert(removed, jc.IsFalse)
 	assertNotClosed(c, st)
@@ -212,11 +209,11 @@ func (s *statePoolSuite) TestRemoveWithRefsClosesOnLastRelease(c *gc.C) {
 }
 
 func (s *statePoolSuite) TestGetRemovedNotAllowed(c *gc.C) {
-	_, _, err := s.Pool.Get(s.ModelUUID1)
+	_, _, err := s.StatePool.Get(s.ModelUUID1)
 	c.Assert(err, jc.ErrorIsNil)
-	_, err = s.Pool.Remove(s.ModelUUID1)
+	_, err = s.StatePool.Remove(s.ModelUUID1)
 	c.Assert(err, jc.ErrorIsNil)
-	_, _, err = s.Pool.Get(s.ModelUUID1)
+	_, _, err = s.StatePool.Get(s.ModelUUID1)
 	c.Assert(err, gc.ErrorMatches, fmt.Sprintf("model %v has been removed", s.ModelUUID1))
 	c.Assert(err, jc.Satisfies, errors.IsNotFound)
 }
