@@ -8,12 +8,15 @@ import (
 	"math/rand"
 	"sort"
 	"strconv"
+	"sync"
 	"time"
 
 	"github.com/juju/errors"
+	"github.com/juju/retry"
+	jujutesting "github.com/juju/testing"
 	jc "github.com/juju/testing/checkers"
 	gc "gopkg.in/check.v1"
-	"gopkg.in/juju/charm.v6-unstable"
+	"gopkg.in/juju/charm.v6"
 
 	"github.com/juju/juju/network"
 	"github.com/juju/juju/state"
@@ -56,8 +59,8 @@ func assertNotJoined(c *gc.C, ru *state.RelationUnit) {
 }
 
 func (s *RelationUnitSuite) TestReadSettingsErrors(c *gc.C) {
-	riak := s.AddTestingService(c, "riak", s.AddTestingCharm(c, "riak"))
-	u0, err := riak.AddUnit()
+	riak := s.AddTestingApplication(c, "riak", s.AddTestingCharm(c, "riak"))
+	u0, err := riak.AddUnit(state.AddUnitParams{})
 	c.Assert(err, jc.ErrorIsNil)
 	riakEP, err := riak.Endpoint("ring")
 	c.Assert(err, jc.ErrorIsNil)
@@ -172,7 +175,7 @@ func (s *RelationUnitSuite) TestRemoteUnitErrors(c *gc.C) {
 			Scope:     charm.ScopeGlobal,
 		}}})
 	c.Assert(err, jc.ErrorIsNil)
-	s.AddTestingService(c, "wordpress", s.AddTestingCharm(c, "wordpress"))
+	s.AddTestingApplication(c, "wordpress", s.AddTestingCharm(c, "wordpress"))
 
 	eps, err := s.State.InferEndpoints("mysql", "wordpress")
 	c.Assert(err, jc.ErrorIsNil)
@@ -267,20 +270,20 @@ func (s *RelationUnitSuite) TestContainerSettings(c *gc.C) {
 }
 
 func (s *RelationUnitSuite) TestContainerCreateSubordinate(c *gc.C) {
-	psvc := s.AddTestingService(c, "mysql", s.AddTestingCharm(c, "mysql"))
-	rsvc := s.AddTestingService(c, "logging", s.AddTestingCharm(c, "logging"))
+	papp := s.AddTestingApplication(c, "mysql", s.AddTestingCharm(c, "mysql"))
+	rapp := s.AddTestingApplication(c, "logging", s.AddTestingCharm(c, "logging"))
 	eps, err := s.State.InferEndpoints("mysql", "logging")
 	c.Assert(err, jc.ErrorIsNil)
 	rel, err := s.State.AddRelation(eps...)
 	c.Assert(err, jc.ErrorIsNil)
-	punit, err := psvc.AddUnit()
+	punit, err := papp.AddUnit(state.AddUnitParams{})
 	c.Assert(err, jc.ErrorIsNil)
 	pru, err := rel.Unit(punit)
 	c.Assert(err, jc.ErrorIsNil)
 
 	// Check that no units of the subordinate service exist.
 	assertSubCount := func(expect int) []*state.Unit {
-		runits, err := rsvc.AllUnits()
+		runits, err := rapp.AllUnits()
 		c.Assert(err, jc.ErrorIsNil)
 		c.Assert(runits, gc.HasLen, expect)
 		return runits
@@ -812,7 +815,7 @@ func (s *RelationUnitSuite) assertNoScopeChange(c *gc.C, ws ...*state.RelationSc
 	}
 }
 
-func (s *RelationUnitSuite) TestSettingsAddress(c *gc.C) {
+func (s *RelationUnitSuite) TestNetworksForRelation(c *gc.C) {
 	prr := newProReqRelation(c, &s.ConnSuite, charm.ScopeGlobal)
 	err := prr.pu0.AssignToNewMachine()
 	c.Assert(err, jc.ErrorIsNil)
@@ -826,10 +829,12 @@ func (s *RelationUnitSuite) TestSettingsAddress(c *gc.C) {
 		network.NewScopedAddress("4.3.2.1", network.ScopePublic),
 	)
 
-	address, err := prr.pru0.SettingsAddress()
+	boundSpace, ingress, egress, err := state.NetworksForRelation("", prr.pu0, prr.rel, nil)
 	c.Assert(err, jc.ErrorIsNil)
 
-	c.Assert(address, gc.DeepEquals, network.NewAddress("1.2.3.4"))
+	c.Assert(boundSpace, gc.Equals, "")
+	c.Assert(ingress, gc.DeepEquals, []string{"1.2.3.4"})
+	c.Assert(egress, gc.DeepEquals, []string{"1.2.3.4/32"})
 }
 
 func (s *RelationUnitSuite) addDevicesWithAddresses(c *gc.C, machine *state.Machine, addresses ...string) {
@@ -857,7 +862,7 @@ func (s *RelationUnitSuite) addDevicesWithAddresses(c *gc.C, machine *state.Mach
 	}
 }
 
-func (s *RelationUnitSuite) TestSettingsAddressWithSpaces(c *gc.C) {
+func (s *RelationUnitSuite) TestNetworksForRelationWithSpaces(c *gc.C) {
 	s.State.AddSubnet(state.SubnetInfo{CIDR: "1.2.0.0/16"})
 	s.State.AddSpace("space-1", "pid-1", []string{"1.2.0.0/16"}, false)
 	s.State.AddSubnet(state.SubnetInfo{CIDR: "2.2.0.0/16"})
@@ -894,13 +899,15 @@ func (s *RelationUnitSuite) TestSettingsAddressWithSpaces(c *gc.C) {
 
 	s.addDevicesWithAddresses(c, machine, "1.2.3.4/16", "2.2.3.4/16", "3.2.3.4/16", "4.3.2.1/16")
 
-	address, err := prr.pru0.SettingsAddress()
+	boundSpace, ingress, egress, err := state.NetworksForRelation("", prr.pu0, prr.rel, nil)
 	c.Assert(err, jc.ErrorIsNil)
 
-	c.Assert(address, gc.DeepEquals, network.NewAddress("2.2.3.4"))
+	c.Assert(boundSpace, gc.Equals, "space-3")
+	c.Assert(ingress, gc.DeepEquals, []string{"2.2.3.4"})
+	c.Assert(egress, gc.DeepEquals, []string{"2.2.3.4/32"})
 }
 
-func (s *RelationUnitSuite) TestSettingsAddressRemoteRelation(c *gc.C) {
+func (s *RelationUnitSuite) TestNetworksForRelationRemoteRelation(c *gc.C) {
 	prr := newRemoteProReqRelation(c, &s.ConnSuite)
 	err := prr.ru0.AssignToNewMachine()
 	c.Assert(err, jc.ErrorIsNil)
@@ -913,14 +920,17 @@ func (s *RelationUnitSuite) TestSettingsAddressRemoteRelation(c *gc.C) {
 		network.NewScopedAddress("1.2.3.4", network.ScopeCloudLocal),
 		network.NewScopedAddress("4.3.2.1", network.ScopePublic),
 	)
-
-	address, err := prr.rru0.SettingsAddress()
 	c.Assert(err, jc.ErrorIsNil)
 
-	c.Assert(address, gc.DeepEquals, network.NewScopedAddress("4.3.2.1", network.ScopePublic))
+	boundSpace, ingress, egress, err := state.NetworksForRelation("", prr.ru0, prr.rel, nil)
+	c.Assert(err, jc.ErrorIsNil)
+
+	c.Assert(boundSpace, gc.Equals, "")
+	c.Assert(ingress, gc.DeepEquals, []string{"4.3.2.1"})
+	c.Assert(egress, gc.DeepEquals, []string{"4.3.2.1/32"})
 }
 
-func (s *RelationUnitSuite) TestSettingsAddressRemoteRelationNoPublicAddr(c *gc.C) {
+func (s *RelationUnitSuite) TestNetworksForRelationRemoteRelationNoPublicAddr(c *gc.C) {
 	prr := newRemoteProReqRelation(c, &s.ConnSuite)
 	err := prr.ru0.AssignToNewMachine()
 	c.Assert(err, jc.ErrorIsNil)
@@ -932,10 +942,83 @@ func (s *RelationUnitSuite) TestSettingsAddressRemoteRelationNoPublicAddr(c *gc.
 	err = machine.SetProviderAddresses(
 		network.NewScopedAddress("1.2.3.4", network.ScopeCloudLocal),
 	)
-
-	address, err := prr.rru0.SettingsAddress()
 	c.Assert(err, jc.ErrorIsNil)
-	c.Assert(address, gc.DeepEquals, network.NewScopedAddress("1.2.3.4", network.ScopeCloudLocal))
+
+	boundSpace, ingress, egress, err := state.NetworksForRelation("", prr.ru0, prr.rel, nil)
+	c.Assert(err, jc.ErrorIsNil)
+
+	c.Assert(boundSpace, gc.Equals, "")
+	c.Assert(ingress, gc.DeepEquals, []string{"1.2.3.4"})
+	c.Assert(egress, gc.DeepEquals, []string{"1.2.3.4/32"})
+}
+
+func (s *RelationUnitSuite) TestNetworksForRelationRemoteRelationDelayedPublicAddress(c *gc.C) {
+	clk := jujutesting.NewClock(time.Now())
+	attemptMade := make(chan struct{}, 10)
+	s.PatchValue(&state.PublicAddressRetryArgs, func() retry.CallArgs {
+		return retry.CallArgs{
+			Clock:       clk,
+			Delay:       3 * time.Second,
+			MaxDuration: 30 * time.Second,
+			NotifyFunc: func(lastError error, attempt int) {
+				attemptMade <- struct{}{}
+			},
+		}
+	})
+	prr := newRemoteProReqRelation(c, &s.ConnSuite)
+	err := prr.ru0.AssignToNewMachine()
+	c.Assert(err, jc.ErrorIsNil)
+	id, err := prr.ru0.AssignedMachineId()
+	c.Assert(err, jc.ErrorIsNil)
+	machine, err := s.State.Machine(id)
+	c.Assert(err, jc.ErrorIsNil)
+
+	// Set up a public address after at least one attempt
+	// is made to get it.We record the err for checking later
+	// as gc.C is not thread safe.
+	var funcErr error
+	wg := sync.WaitGroup{}
+	go func() {
+		wg.Add(1)
+		defer wg.Done()
+		funcErr = clk.WaitAdvance(15*time.Second, time.Second, 1)
+		if funcErr != nil {
+			return
+		}
+		// Ensure we have a failed attempt to get public address.
+		select {
+		case <-attemptMade:
+			funcErr = clk.WaitAdvance(10*time.Second, time.Second, 1)
+			if funcErr != nil {
+				return
+			}
+		case <-time.After(coretesting.LongWait):
+			c.Fatal("waiting for public address attempt")
+		}
+
+		// Now set up the public address.
+		funcErr = machine.SetProviderAddresses(
+			network.NewScopedAddress("4.3.2.1", network.ScopePublic),
+		)
+		if funcErr != nil {
+			return
+		}
+		funcErr = clk.WaitAdvance(10*time.Second, time.Second, 1)
+		if funcErr != nil {
+			return
+		}
+	}()
+
+	boundSpace, ingress, egress, err := state.NetworksForRelation("", prr.ru0, prr.rel, nil)
+	c.Assert(err, jc.ErrorIsNil)
+
+	// Ensure there we no errors in the go routine.
+	wg.Wait()
+	c.Assert(funcErr, jc.ErrorIsNil)
+
+	c.Assert(boundSpace, gc.Equals, "")
+	c.Assert(ingress, gc.DeepEquals, []string{"4.3.2.1"})
+	c.Assert(egress, gc.DeepEquals, []string{"4.3.2.1/32"})
 }
 
 func (s *RelationUnitSuite) TestValidYes(c *gc.C) {
@@ -950,7 +1033,7 @@ func (s *RelationUnitSuite) TestValidYes(c *gc.C) {
 
 func (s *RelationUnitSuite) TestValidNo(c *gc.C) {
 	mysqlLogging := newProReqRelation(c, &s.ConnSuite, charm.ScopeContainer)
-	wpApp := s.AddTestingService(c, "wordpress", s.AddTestingCharm(c, "wordpress"))
+	wpApp := s.AddTestingApplication(c, "wordpress", s.AddTestingCharm(c, "wordpress"))
 	wpLogging := newProReqRelationForApps(c, &s.ConnSuite, wpApp, mysqlLogging.rsvc)
 
 	// We have logging related to mysql and to wordpress. We can
@@ -967,7 +1050,7 @@ func (s *RelationUnitSuite) TestValidNo(c *gc.C) {
 func (s *RelationUnitSuite) TestValidSubordinateToSubordinate(c *gc.C) {
 	// Relate mysql and logging...
 	mysqlLogging := newProReqRelation(c, &s.ConnSuite, charm.ScopeContainer)
-	monApp := s.AddTestingService(c, "monitoring", s.AddTestingCharm(c, "monitoring"))
+	monApp := s.AddTestingApplication(c, "monitoring", s.AddTestingCharm(c, "monitoring"))
 	// Relate mysql and monitoring - relation to a non-subordinate
 	// needed to trigger creation of monitoring unit.
 	mysqlMonitoring := newProReqRelationForApps(c, &s.ConnSuite, mysqlLogging.psvc, monApp)
@@ -1018,16 +1101,16 @@ func preventPeerUnitsDestroyRemove(c *gc.C, pr *PeerRelation) {
 }
 
 func newPeerRelation(c *gc.C, st *state.State) *PeerRelation {
-	svc := state.AddTestingService(c, st, "riak", state.AddTestingCharm(c, st, "riak"))
-	ep, err := svc.Endpoint("ring")
+	app := state.AddTestingApplication(c, st, "riak", state.AddTestingCharm(c, st, "riak"))
+	ep, err := app.Endpoint("ring")
 	c.Assert(err, jc.ErrorIsNil)
 	rel, err := st.EndpointsRelation(ep)
 	c.Assert(err, jc.ErrorIsNil)
-	pr := &PeerRelation{rel: rel, app: svc}
-	pr.u0, pr.ru0 = addRU(c, svc, rel, nil)
-	pr.u1, pr.ru1 = addRU(c, svc, rel, nil)
-	pr.u2, pr.ru2 = addRU(c, svc, rel, nil)
-	pr.u3, pr.ru3 = addRU(c, svc, rel, nil)
+	pr := &PeerRelation{rel: rel, app: app}
+	pr.u0, pr.ru0 = addRU(c, app, rel, nil)
+	pr.u1, pr.ru1 = addRU(c, app, rel, nil)
+	pr.u2, pr.ru2 = addRU(c, app, rel, nil)
+	pr.u3, pr.ru3 = addRU(c, app, rel, nil)
 	return pr
 }
 
@@ -1046,23 +1129,23 @@ func preventProReqUnitsDestroyRemove(c *gc.C, prr *ProReqRelation) {
 }
 
 func newProReqRelation(c *gc.C, s *ConnSuite, scope charm.RelationScope) *ProReqRelation {
-	papp := s.AddTestingService(c, "mysql", s.AddTestingCharm(c, "mysql"))
+	papp := s.AddTestingApplication(c, "mysql", s.AddTestingCharm(c, "mysql"))
 	var rapp *state.Application
 	if scope == charm.ScopeGlobal {
-		rapp = s.AddTestingService(c, "wordpress", s.AddTestingCharm(c, "wordpress"))
+		rapp = s.AddTestingApplication(c, "wordpress", s.AddTestingCharm(c, "wordpress"))
 	} else {
-		rapp = s.AddTestingService(c, "logging", s.AddTestingCharm(c, "logging"))
+		rapp = s.AddTestingApplication(c, "logging", s.AddTestingCharm(c, "logging"))
 	}
 	return newProReqRelationForApps(c, s, papp, rapp)
 }
 
 func newProReqRelationWithBindings(c *gc.C, s *ConnSuite, scope charm.RelationScope, pbindings, rbindings map[string]string) *ProReqRelation {
-	papp := s.AddTestingServiceWithBindings(c, "mysql", s.AddTestingCharm(c, "mysql"), pbindings)
+	papp := s.AddTestingApplicationWithBindings(c, "mysql", s.AddTestingCharm(c, "mysql"), pbindings)
 	var rapp *state.Application
 	if scope == charm.ScopeGlobal {
-		rapp = s.AddTestingServiceWithBindings(c, "wordpress", s.AddTestingCharm(c, "wordpress"), rbindings)
+		rapp = s.AddTestingApplicationWithBindings(c, "wordpress", s.AddTestingCharm(c, "wordpress"), rbindings)
 	} else {
-		rapp = s.AddTestingServiceWithBindings(c, "logging", s.AddTestingCharm(c, "logging"), rbindings)
+		rapp = s.AddTestingApplicationWithBindings(c, "logging", s.AddTestingCharm(c, "logging"), rbindings)
 	}
 	return newProReqRelationForApps(c, s, papp, rapp)
 }
@@ -1099,7 +1182,7 @@ func addRU(c *gc.C, svc *state.Application, rel *state.Relation, principal *stat
 	// relation's scope as the principal.
 	var u *state.Unit
 	if principal == nil {
-		unit, err := svc.AddUnit()
+		unit, err := svc.AddUnit(state.AddUnitParams{})
 		c.Assert(err, jc.ErrorIsNil)
 		u = unit
 	} else {
@@ -1152,18 +1235,18 @@ func newRemoteProReqRelation(c *gc.C, s *ConnSuite) *RemoteProReqRelation {
 			Scope:     charm.ScopeGlobal,
 		}}})
 	c.Assert(err, jc.ErrorIsNil)
-	rsvc := s.AddTestingService(c, "wordpress", s.AddTestingCharm(c, "wordpress"))
+	rapp := s.AddTestingApplication(c, "wordpress", s.AddTestingCharm(c, "wordpress"))
 
 	eps, err := s.State.InferEndpoints("mysql", "wordpress")
 	c.Assert(err, jc.ErrorIsNil)
 	rel, err := s.State.AddRelation(eps...)
 	c.Assert(err, jc.ErrorIsNil)
 
-	prr := &RemoteProReqRelation{rel: rel, psvc: psvc, rsvc: rsvc}
+	prr := &RemoteProReqRelation{rel: rel, psvc: psvc, rsvc: rapp}
 	prr.pru0 = addRemoteRU(c, rel, "mysql/0")
 	prr.pru1 = addRemoteRU(c, rel, "mysql/1")
-	prr.ru0, prr.rru0 = addRU(c, rsvc, rel, nil)
-	prr.ru1, prr.rru1 = addRU(c, rsvc, rel, nil)
+	prr.ru0, prr.rru0 = addRU(c, rapp, rel, nil)
+	prr.ru1, prr.rru1 = addRU(c, rapp, rel, nil)
 	return prr
 }
 
@@ -1188,8 +1271,8 @@ type WatchScopeSuite struct {
 var _ = gc.Suite(&WatchScopeSuite{})
 
 func (s *WatchScopeSuite) TestPeer(c *gc.C) {
-	// Create a service and get a peer relation.
-	riak := s.AddTestingService(c, "riak", s.AddTestingCharm(c, "riak"))
+	// Create an application and get a peer relation.
+	riak := s.AddTestingApplication(c, "riak", s.AddTestingCharm(c, "riak"))
 	riakEP, err := riak.Endpoint("ring")
 	c.Assert(err, jc.ErrorIsNil)
 	rels, err := riak.Relations()
@@ -1197,14 +1280,14 @@ func (s *WatchScopeSuite) TestPeer(c *gc.C) {
 	c.Assert(rels, gc.HasLen, 1)
 	rel := rels[0]
 
-	// Add some units to the service and set their private addresses; get
+	// Add some units to the application and set their private addresses; get
 	// the relevant RelationUnits.
 	// (Private addresses should be set by their unit agents on
 	// startup; this test does not include that, but Join expects
 	// the information to be available, and uses it to populate the
 	// relation settings node.)
 	addUnit := func(i int) *state.RelationUnit {
-		unit, err := riak.AddUnit()
+		unit, err := riak.AddUnit(state.AddUnitParams{})
 		c.Assert(err, jc.ErrorIsNil)
 		err = unit.AssignToNewMachine()
 		c.Assert(err, jc.ErrorIsNil)
@@ -1323,10 +1406,10 @@ func (s *WatchScopeSuite) TestPeer(c *gc.C) {
 
 func (s *WatchScopeSuite) TestProviderRequirerGlobal(c *gc.C) {
 	// Create a pair of services and a relation between them.
-	mysql := s.AddTestingService(c, "mysql", s.AddTestingCharm(c, "mysql"))
+	mysql := s.AddTestingApplication(c, "mysql", s.AddTestingCharm(c, "mysql"))
 	mysqlEP, err := mysql.Endpoint("server")
 	c.Assert(err, jc.ErrorIsNil)
-	wordpress := s.AddTestingService(c, "wordpress", s.AddTestingCharm(c, "wordpress"))
+	wordpress := s.AddTestingApplication(c, "wordpress", s.AddTestingCharm(c, "wordpress"))
 	wordpressEP, err := wordpress.Endpoint("db")
 	c.Assert(err, jc.ErrorIsNil)
 	rel, err := s.State.AddRelation(mysqlEP, wordpressEP)
@@ -1334,7 +1417,7 @@ func (s *WatchScopeSuite) TestProviderRequirerGlobal(c *gc.C) {
 
 	// Add some units to the services and set their private addresses.
 	addUnit := func(srv *state.Application, sub string, ep state.Endpoint) *state.RelationUnit {
-		unit, err := srv.AddUnit()
+		unit, err := srv.AddUnit(state.AddUnitParams{})
 		c.Assert(err, jc.ErrorIsNil)
 		ru, err := rel.Unit(unit)
 		c.Assert(err, jc.ErrorIsNil)
@@ -1453,10 +1536,10 @@ func (s *WatchScopeSuite) TestProviderRequirerGlobal(c *gc.C) {
 
 func (s *WatchScopeSuite) TestProviderRequirerContainer(c *gc.C) {
 	// Create a pair of services and a relation between them.
-	mysql := s.AddTestingService(c, "mysql", s.AddTestingCharm(c, "mysql"))
+	mysql := s.AddTestingApplication(c, "mysql", s.AddTestingCharm(c, "mysql"))
 	mysqlEP, err := mysql.Endpoint("juju-info")
 	c.Assert(err, jc.ErrorIsNil)
-	logging := s.AddTestingService(c, "logging", s.AddTestingCharm(c, "logging"))
+	logging := s.AddTestingApplication(c, "logging", s.AddTestingCharm(c, "logging"))
 	loggingEP, err := logging.Endpoint("info")
 	c.Assert(err, jc.ErrorIsNil)
 	rel, err := s.State.AddRelation(mysqlEP, loggingEP)
@@ -1467,7 +1550,7 @@ func (s *WatchScopeSuite) TestProviderRequirerContainer(c *gc.C) {
 
 	// Add some units to the services and set their private addresses.
 	addUnits := func(i int) (*state.RelationUnit, *state.RelationUnit) {
-		msu, err := mysql.AddUnit()
+		msu, err := mysql.AddUnit(state.AddUnitParams{})
 		c.Assert(err, jc.ErrorIsNil)
 		msru, err := rel.Unit(msu)
 		c.Assert(err, jc.ErrorIsNil)
@@ -1600,10 +1683,10 @@ var _ = gc.Suite(&WatchUnitsSuite{})
 
 func (s *WatchUnitsSuite) TestProviderRequirerGlobal(c *gc.C) {
 	// Create a pair of services and a relation between them.
-	mysql := s.AddTestingService(c, "mysql", s.AddTestingCharm(c, "mysql"))
+	mysql := s.AddTestingApplication(c, "mysql", s.AddTestingCharm(c, "mysql"))
 	mysqlEP, err := mysql.Endpoint("server")
 	c.Assert(err, jc.ErrorIsNil)
-	wordpress := s.AddTestingService(c, "wordpress", s.AddTestingCharm(c, "wordpress"))
+	wordpress := s.AddTestingApplication(c, "wordpress", s.AddTestingCharm(c, "wordpress"))
 	wordpressEP, err := wordpress.Endpoint("db")
 	c.Assert(err, jc.ErrorIsNil)
 	rel, err := s.State.AddRelation(mysqlEP, wordpressEP)
@@ -1611,7 +1694,7 @@ func (s *WatchUnitsSuite) TestProviderRequirerGlobal(c *gc.C) {
 
 	// Add some units to the services and set their private addresses.
 	addUnit := func(srv *state.Application) *state.RelationUnit {
-		unit, err := srv.AddUnit()
+		unit, err := srv.AddUnit(state.AddUnitParams{})
 		c.Assert(err, jc.ErrorIsNil)
 		ru, err := rel.Unit(unit)
 		c.Assert(err, jc.ErrorIsNil)
@@ -1652,10 +1735,10 @@ func (s *WatchUnitsSuite) TestProviderRequirerGlobal(c *gc.C) {
 
 func (s *WatchUnitsSuite) TestProviderRequirerContainer(c *gc.C) {
 	// Create a pair of services and a relation between them.
-	mysql := s.AddTestingService(c, "mysql", s.AddTestingCharm(c, "mysql"))
+	mysql := s.AddTestingApplication(c, "mysql", s.AddTestingCharm(c, "mysql"))
 	mysqlEP, err := mysql.Endpoint("juju-info")
 	c.Assert(err, jc.ErrorIsNil)
-	logging := s.AddTestingService(c, "logging", s.AddTestingCharm(c, "logging"))
+	logging := s.AddTestingApplication(c, "logging", s.AddTestingCharm(c, "logging"))
 	loggingEP, err := logging.Endpoint("info")
 	c.Assert(err, jc.ErrorIsNil)
 	rel, err := s.State.AddRelation(mysqlEP, loggingEP)
