@@ -4,6 +4,7 @@
 package state
 
 import (
+	"encoding/json"
 	"fmt"
 	"sort"
 	"time"
@@ -11,8 +12,9 @@ import (
 	"github.com/juju/errors"
 	jujutxn "github.com/juju/txn"
 	"github.com/juju/utils/set"
-	"gopkg.in/juju/charm.v6-unstable"
+	"gopkg.in/juju/charm.v6"
 	"gopkg.in/juju/names.v2"
+	"gopkg.in/macaroon.v1"
 	"gopkg.in/mgo.v2"
 	"gopkg.in/mgo.v2/bson"
 	"gopkg.in/mgo.v2/txn"
@@ -33,7 +35,7 @@ type RemoteApplication struct {
 type remoteApplicationDoc struct {
 	DocID           string              `bson:"_id"`
 	Name            string              `bson:"name"`
-	OfferName       string              `bson:"offer-name"`
+	OfferUUID       string              `bson:"offer-uuid"`
 	URL             string              `bson:"url,omitempty"`
 	SourceModelUUID string              `bson:"source-model-uuid"`
 	Endpoints       []remoteEndpointDoc `bson:"endpoints"`
@@ -42,6 +44,7 @@ type remoteApplicationDoc struct {
 	Life            Life                `bson:"life"`
 	RelationCount   int                 `bson:"relationcount"`
 	IsConsumerProxy bool                `bson:"is-consumer-proxy"`
+	Macaroon        string              `bson:"macaroon,omitempty"`
 }
 
 // remoteEndpointDoc represents the internal state of a remote application endpoint in MongoDB.
@@ -140,9 +143,9 @@ func (s *RemoteApplication) Name() string {
 	return s.doc.Name
 }
 
-// OfferName returns the name on te offering side.
-func (s *RemoteApplication) OfferName() string {
-	return s.doc.OfferName
+// OfferUUID returns the offer UUID.
+func (s *RemoteApplication) OfferUUID() string {
+	return s.doc.OfferUUID
 }
 
 // URL returns the remote service URL, and a boolean indicating whether or not
@@ -156,7 +159,7 @@ func (s *RemoteApplication) URL() (string, bool) {
 // model to identify the service in future communications.
 func (s *RemoteApplication) Token() (string, error) {
 	r := s.st.RemoteEntities()
-	return r.GetToken(s.SourceModel(), s.Tag())
+	return r.GetToken(s.Tag())
 }
 
 // Tag returns a name identifying the application.
@@ -276,7 +279,7 @@ func (s *RemoteApplication) Destroy() (err error) {
 		}
 		return nil, jujutxn.ErrTransientFailure
 	}
-	return s.st.run(buildTxn)
+	return s.st.db().Run(buildTxn)
 }
 
 // destroyOps returns the operations required to destroy the application. If it
@@ -318,7 +321,11 @@ func (s *RemoteApplication) destroyOps() ([]txn.Op, error) {
 	// removed, the application can also be removed.
 	if s.doc.RelationCount == removeCount {
 		hasLastRefs := bson.D{{"life", Alive}, {"relationcount", removeCount}}
-		return append(ops, s.removeOps(hasLastRefs)...), nil
+		removeOps, err := s.removeOps(hasLastRefs)
+		if err != nil {
+			return nil, err
+		}
+		return append(ops, removeOps...), nil
 	}
 	// In all other cases, application removal will be handled as a consequence
 	// of the removal of the relation referencing it. If any  relations have
@@ -347,7 +354,7 @@ func (s *RemoteApplication) destroyOps() ([]txn.Op, error) {
 
 // removeOps returns the operations required to remove the application. Supplied
 // asserts will be included in the operation on the application document.
-func (s *RemoteApplication) removeOps(asserts bson.D) []txn.Op {
+func (s *RemoteApplication) removeOps(asserts bson.D) ([]txn.Op, error) {
 	r := s.st.RemoteEntities()
 	ops := []txn.Op{
 		{
@@ -357,9 +364,10 @@ func (s *RemoteApplication) removeOps(asserts bson.D) []txn.Op {
 			Remove: true,
 		},
 		removeStatusOp(s.st, s.globalKey()),
-		r.removeRemoteEntityOp(s.SourceModel(), s.Tag()),
 	}
-	return ops
+	tokenOps := r.removeRemoteEntityOps(s.Tag())
+	ops = append(ops, tokenOps...)
+	return ops, nil
 }
 
 // Status returns the status of the remote application.
@@ -378,7 +386,7 @@ func (s *RemoteApplication) SetStatus(info status.StatusInfo) error {
 		status:    info.Status,
 		message:   info.Message,
 		rawData:   info.Data,
-		updated:   timeOrNow(info.Since, s.st.clock),
+		updated:   timeOrNow(info.Since, s.st.clock()),
 	})
 }
 
@@ -501,12 +509,24 @@ func (s *RemoteApplication) AddEndpoints(eps []charm.Relation) error {
 		}
 		return ops, nil
 	}
-	if err := s.st.run(buildTxn); err != nil {
+	if err := s.st.db().Run(buildTxn); err != nil {
 		return errors.Trace(err)
 	}
 	return s.Refresh()
 }
 
+func (s *RemoteApplication) Macaroon() (*macaroon.Macaroon, error) {
+	if s.doc.Macaroon == "" {
+		return nil, nil
+	}
+	var mac macaroon.Macaroon
+	err := json.Unmarshal([]byte(s.doc.Macaroon), &mac)
+	if err != nil {
+		return nil, errors.Trace(err)
+	}
+	return &mac, nil
+}
+
 // String returns the application name.
 func (s *RemoteApplication) String() string {
 	return s.doc.Name
@@ -541,8 +561,8 @@ type AddRemoteApplicationParams struct {
 	// match the application name in the URL, or the name in the remote model.
 	Name string
 
-	// OfferName is the name the offering side has given to the remote application.
-	OfferName string
+	// OfferUUID is the UUID of the offer.
+	OfferUUID string
 
 	// URL is either empty, or the URL that the remote application was offered
 	// with on the hosting model.
@@ -568,6 +588,9 @@ type AddRemoteApplicationParams struct {
 	// IsConsumerProxy is true when a remote application is created as a result
 	// of a registration operation from a remote model.
 	IsConsumerProxy bool
+
+	// Macaroon is used for authentication on the offering side.
+	Macaroon *macaroon.Macaroon
 }
 
 // Validate returns an error if there's a problem with the
@@ -579,8 +602,8 @@ func (p AddRemoteApplicationParams) Validate() error {
 	if p.URL != "" {
 		// URL may be empty, to represent remote applications corresponding
 		// to consumers of an offered application.
-		if _, err := crossmodel.ParseApplicationURL(p.URL); err != nil {
-			return errors.Annotate(err, "validating offered application URL")
+		if _, err := crossmodel.ParseOfferURL(p.URL); err != nil {
+			return errors.Annotate(err, "validating offer URL")
 		}
 	}
 	if p.SourceModel == (names.ModelTag{}) {
@@ -614,17 +637,26 @@ func (st *State) AddRemoteApplication(args AddRemoteApplicationParams) (_ *Remot
 		return nil, errors.Errorf("model is no longer alive")
 	}
 
+	var macJSON string
+	if args.Macaroon != nil {
+		b, err := json.Marshal(args.Macaroon)
+		if err != nil {
+			return nil, errors.Trace(err)
+		}
+		macJSON = string(b)
+	}
 	applicationID := st.docID(args.Name)
 	// Create the application addition operations.
 	appDoc := &remoteApplicationDoc{
 		DocID:           applicationID,
 		Name:            args.Name,
-		OfferName:       args.OfferName,
+		OfferUUID:       args.OfferUUID,
 		SourceModelUUID: args.SourceModel.Id(),
 		URL:             args.URL,
 		Bindings:        args.Bindings,
 		Life:            Alive,
 		IsConsumerProxy: args.IsConsumerProxy,
+		Macaroon:        macJSON,
 	}
 	eps := make([]remoteEndpointDoc, len(args.Endpoints))
 	for i, ep := range args.Endpoints {
@@ -660,12 +692,6 @@ func (st *State) AddRemoteApplication(args AddRemoteApplicationParams) (_ *Remot
 	}
 	appDoc.Spaces = spaces
 	app := newRemoteApplication(st, appDoc)
-	statusDoc := statusDoc{
-		ModelUUID:  st.ModelUUID(),
-		Status:     status.Unknown,
-		StatusInfo: "waiting for remote connection",
-		Updated:    time.Now().UnixNano(),
-	}
 
 	buildTxn := func(attempt int) ([]txn.Op, error) {
 		// If we've tried once already and failed, check that
@@ -689,7 +715,6 @@ func (st *State) AddRemoteApplication(args AddRemoteApplicationParams) (_ *Remot
 		}
 		ops := []txn.Op{
 			model.assertActiveOp(),
-			createStatusOp(st, app.globalKey(), statusDoc),
 			{
 				C:      remoteApplicationsC,
 				Id:     appDoc.Name,
@@ -701,16 +726,22 @@ func (st *State) AddRemoteApplication(args AddRemoteApplicationParams) (_ *Remot
 				Assert: txn.DocMissing,
 			},
 		}
+		if !args.IsConsumerProxy {
+			statusDoc := statusDoc{
+				ModelUUID: st.ModelUUID(),
+				Status:    status.Unknown,
+				Updated:   time.Now().UnixNano(),
+			}
+			ops = append(ops, createStatusOp(st, app.globalKey(), statusDoc))
+		}
 		// If we know the token, import it.
 		if args.Token != "" {
-			importRemoteEntityOps := st.RemoteEntities().importRemoteEntityOps(
-				args.SourceModel, app.Tag(), args.Token,
-			)
+			importRemoteEntityOps := st.RemoteEntities().importRemoteEntityOps(app.Tag(), args.Token)
 			ops = append(ops, importRemoteEntityOps...)
 		}
 		return ops, nil
 	}
-	if err = st.run(buildTxn); err != nil {
+	if err = st.db().Run(buildTxn); err != nil {
 		return nil, errors.Trace(err)
 	}
 	return app, nil
@@ -767,29 +798,3 @@ func (st *State) AllRemoteApplications() (applications []*RemoteApplication, err
 	}
 	return applications, nil
 }
-
-// RemoteConnectionStatus returns summary information about connections to the specified offer.
-func (st *State) RemoteConnectionStatus(offerName string) (*RemoteConnectionStatus, error) {
-	applicationsCollection, closer := st.db().GetCollection(remoteApplicationsC)
-	defer closer()
-
-	count, err := applicationsCollection.Find(bson.D{{"offer-name", offerName}}).Count()
-	if err != nil {
-		return nil, errors.Errorf("cannot get remote connection status for offer %q", offerName)
-	}
-	return &RemoteConnectionStatus{
-		count: count,
-	}, nil
-}
-
-// RemoteConnectionStatus holds summary information about connections
-// to an application offer.
-type RemoteConnectionStatus struct {
-	count int
-}
-
-// ConnectionCount returns the number of remote applications
-// related to an offer.
-func (r *RemoteConnectionStatus) ConnectionCount() int {
-	return r.count
-}
