@@ -470,7 +470,8 @@ func (s *ResourcePersistenceSuite) TestNewResourcePendingResourceOpsExists(c *gc
 	// TODO(macgreagoir) We need to keep using time.Now() for now, while we
 	// have NewResolvePendingResourceOps returning LastPolled based on
 	// timeNow(). lp:1558657
-	lastPolled := time.Now().UTC().Round(time.Second)
+	// Note: truncate the time to remove monotonic time for Go 1.9+.
+	lastPolled := time.Now().UTC().Round(time.Second).Truncate(1)
 
 	ops, err := p.NewResolvePendingResourceOps(stored.ID, stored.PendingID)
 	c.Assert(err, jc.ErrorIsNil)
@@ -572,6 +573,55 @@ func (s *ResourcePersistenceSuite) TestNewResourcePendingResourceOpsNotFound(c *
 	})
 }
 
+func (s *ResourcePersistenceSuite) TestRemoveResourcesCleansUpUniqueStoragePaths(c *gc.C) {
+	// We shouldn't schedule multiple cleanups for the same path (when
+	// application and units use the same resource).
+	appResource, appDoc := newPersistenceResource(c, "appa", "yipyip")
+	_, unitDoc := newPersistenceUnitResource(c, "appa", "appa/0", "yipyip")
+	s.base.ReturnAll = []resourceDoc{appDoc, unitDoc}
+	p := NewResourcePersistence(s.base)
+
+	ops, err := p.NewRemoveResourcesOps("appa")
+	c.Assert(err, jc.ErrorIsNil)
+
+	var cleanups []txn.Op
+	for _, op := range ops {
+		if op.C == cleanupsC {
+			cleanups = append(cleanups, op)
+		}
+	}
+	c.Assert(cleanups, gc.HasLen, 1)
+	c.Assert(cleanups[0].Insert, gc.Not(gc.IsNil))
+	c.Assert(cleanups[0].Insert.(*cleanupDoc).Kind, gc.Equals, cleanupResourceBlob)
+	c.Assert(cleanups[0].Insert.(*cleanupDoc).Prefix, gc.Equals, appResource.storagePath)
+}
+
+func (s *ResourcePersistenceSuite) TestRemovePendingAppResources(c *gc.C) {
+	_, appDoc1 := newPersistenceResource(c, "appa", "yipyip")
+	appDoc1.DocID += "#pending-freewifi"
+	appDoc1.PendingID = "freewifi"
+	appResource2, appDoc2 := newPersistenceResource(c, "appa", "momo")
+	appDoc2.DocID += "#pending-parallax"
+	appDoc2.PendingID = "parallax"
+	_, unitDoc := newPersistenceUnitResource(c, "appa", "appa/0", "momo")
+	unitDoc.DocID += "#pending-parallax"
+	unitDoc.PendingID = "parallax"
+	s.base.ReturnAll = []resourceDoc{appDoc1, appDoc2, unitDoc}
+	p := NewResourcePersistence(s.base)
+
+	ops, err := p.NewRemovePendingAppResourcesOps("appa", map[string]string{"momo": "parallax"})
+	c.Assert(err, jc.ErrorIsNil)
+	c.Assert(ops, gc.HasLen, 2)
+	c.Assert(ops[0], gc.DeepEquals, txn.Op{
+		C:      resourcesC,
+		Id:     appDoc2.DocID,
+		Remove: true,
+	})
+	c.Assert(ops[1].Insert, gc.Not(gc.IsNil))
+	c.Assert(ops[1].Insert.(*cleanupDoc).Kind, gc.Equals, cleanupResourceBlob)
+	c.Assert(ops[1].Insert.(*cleanupDoc).Prefix, gc.Equals, appResource2.storagePath)
+}
+
 func newPersistenceUnitResources(c *gc.C, serviceID, unitID string, resources []resource.Resource) ([]resource.Resource, []resourceDoc) {
 	var unitResources []resource.Resource
 	var docs []resourceDoc
