@@ -6,6 +6,7 @@ package state_test
 import (
 	"time"
 
+	gitjujutesting "github.com/juju/testing"
 	jc "github.com/juju/testing/checkers"
 	gc "gopkg.in/check.v1"
 
@@ -28,7 +29,7 @@ func (s *SingularSuite) SetUpTest(c *gc.C) {
 func (s *SingularSuite) TestClaimBadLease(c *gc.C) {
 	claimer := s.State.SingularClaimer()
 	err := claimer.Claim("xxx", "machine-123", time.Minute)
-	c.Check(err, gc.ErrorMatches, `cannot claim lease "xxx": expected environ UUID`)
+	c.Check(err, gc.ErrorMatches, `cannot claim lease "xxx": expected controller or model UUID`)
 }
 
 func (s *SingularSuite) TestClaimBadHolder(c *gc.C) {
@@ -44,14 +45,23 @@ func (s *SingularSuite) TestClaimBadDuration(c *gc.C) {
 }
 
 func (s *SingularSuite) TestClaim(c *gc.C) {
+	// We claim in the same test for the model and controller,
+	// swapping which machine gets to hold which lease to
+	// demonstrate that the model and controller leases are
+	// independent.
+	s.testClaim(c, s.modelTag.Id(), "machine-123", "machine-456")
+	s.testClaim(c, s.State.ControllerUUID(), "machine-456", "machine-123")
+}
+
+func (s *SingularSuite) testClaim(c *gc.C, leaseId, machineA, machineB string) {
 	claimer := s.State.SingularClaimer()
-	err := claimer.Claim(s.modelTag.Id(), "machine-123", time.Minute)
+	err := claimer.Claim(leaseId, machineA, time.Minute)
 	c.Assert(err, jc.ErrorIsNil)
 
-	err = claimer.Claim(s.modelTag.Id(), "machine-123", time.Minute)
+	err = claimer.Claim(leaseId, machineA, time.Minute)
 	c.Assert(err, jc.ErrorIsNil)
 
-	err = claimer.Claim(s.modelTag.Id(), "machine-456", time.Minute)
+	err = claimer.Claim(leaseId, machineB, time.Minute)
 	c.Assert(err, gc.Equals, lease.ErrClaimDenied)
 }
 
@@ -64,6 +74,10 @@ func (s *SingularSuite) TestExpire(c *gc.C) {
 		wait <- claimer.WaitUntilExpired(s.modelTag.Id())
 	}()
 
+	g, err := s.State.GlobalClockUpdater()
+	c.Assert(err, jc.ErrorIsNil)
+	err = g.Advance(coretesting.ShortWait)
+	c.Assert(err, jc.ErrorIsNil)
 	s.Clock.Advance(coretesting.ShortWait)
 	select {
 	case err := <-wait:
@@ -71,6 +85,8 @@ func (s *SingularSuite) TestExpire(c *gc.C) {
 	case <-time.After(coretesting.ShortWait):
 	}
 
+	err = g.Advance(time.Hour)
+	c.Assert(err, jc.ErrorIsNil)
 	s.Clock.Advance(time.Hour)
 	select {
 	case err := <-wait:
@@ -82,3 +98,15 @@ func (s *SingularSuite) TestExpire(c *gc.C) {
 	err = claimer.Claim(s.modelTag.Id(), "machine-456", time.Minute)
 	c.Assert(err, jc.ErrorIsNil)
 }
+
+func (s *SingularSuite) TestSingularClaimerRestarts(c *gc.C) {
+	claimer := s.State.SingularClaimer()
+
+	// SetClockForTesting will restart the workers, and
+	// will have replaced them by the time it returns.
+	s.State.SetClockForTesting(gitjujutesting.NewClock(time.Time{}))
+
+	err := claimer.Claim(s.modelTag.Id(), "machine-123", time.Minute)
+	c.Assert(err, jc.ErrorIsNil)
+
+}
