@@ -15,6 +15,7 @@ import (
 
 	"github.com/juju/errors"
 	"github.com/juju/loggo"
+	"github.com/juju/pubsub"
 	jujutxn "github.com/juju/txn"
 	"github.com/juju/utils"
 	"github.com/juju/utils/clock"
@@ -30,7 +31,6 @@ import (
 	"gopkg.in/mgo.v2/txn"
 
 	"github.com/juju/juju/apiserver/params"
-	"github.com/juju/juju/audit"
 	"github.com/juju/juju/constraints"
 	coreglobalclock "github.com/juju/juju/core/globalclock"
 	"github.com/juju/juju/core/lease"
@@ -40,7 +40,6 @@ import (
 	"github.com/juju/juju/permission"
 	"github.com/juju/juju/state/cloudimagemetadata"
 	"github.com/juju/juju/state/globalclock"
-	stateaudit "github.com/juju/juju/state/internal/audit"
 	statelease "github.com/juju/juju/state/lease"
 	"github.com/juju/juju/state/presence"
 	"github.com/juju/juju/state/watcher"
@@ -344,7 +343,7 @@ func (st *State) ForModel(modelTag names.ModelTag) (*State, error) {
 	if err != nil {
 		return nil, errors.Trace(err)
 	}
-	if err := newSt.start(st.controllerTag); err != nil {
+	if err := newSt.start(st.controllerTag, nil); err != nil {
 		return nil, errors.Trace(err)
 	}
 	return newSt, nil
@@ -356,7 +355,7 @@ func (st *State) ForModel(modelTag names.ModelTag) (*State, error) {
 //   * creating cloud metadata storage
 //
 // start will close the *State if it fails.
-func (st *State) start(controllerTag names.ControllerTag) (err error) {
+func (st *State) start(controllerTag names.ControllerTag, hub *pubsub.SimpleHub) (err error) {
 	defer func() {
 		if err == nil {
 			return
@@ -388,7 +387,7 @@ func (st *State) start(controllerTag names.ControllerTag) (err error) {
 	// now we've set up leaseClientId, we can use workersFactory
 
 	logger.Infof("starting standard state workers")
-	workers, err := newWorkers(st)
+	workers, err := newWorkers(st, hub)
 	if err != nil {
 		return errors.Trace(err)
 	}
@@ -538,6 +537,9 @@ func (st *State) getPingBatcher() *presence.PingBatcher {
 // getTxnLogCollection returns the raw mongodb txns collection, which is
 // needed to interact with the state/watcher package.
 func (st *State) getTxnLogCollection() *mgo.Collection {
+	if st.Ping() != nil {
+		st.session.Refresh()
+	}
 	return st.session.DB(jujuDB).C(txnLogC)
 }
 
@@ -650,6 +652,7 @@ func (st *State) checkCanUpgrade(currentVersion, newVersion string) error {
 			DocID string `bson:"_id"`
 		}
 		iter := collection.Find(sel).Select(bson.D{{"_id", 1}}).Iter()
+		defer iter.Close()
 		for iter.Next(&doc) {
 			localID, err := st.strictLocalID(doc.DocID)
 			if err != nil {
@@ -687,7 +690,7 @@ func IsUpgradeInProgressError(err error) bool {
 // cannot be higher than the controller version.
 func (st *State) SetModelAgentVersion(newVersion version.Number, ignoreAgentVersions bool) (err error) {
 	if newVersion.Compare(jujuversion.Current) > 0 && !st.IsController() {
-		return errors.Errorf("a hosted model cannot have a higher version than the server model: %s > %s",
+		return errors.Errorf("model cannot be upgraded to %s while the controller is %s: upgrade 'controller' model first",
 			newVersion.String(),
 			jujuversion.Current,
 		)
@@ -1112,7 +1115,9 @@ func (st *State) AddApplication(args AddApplicationArgs) (_ *Application, err er
 			for _, supportedSeries := range supportedSeries {
 				os, err := series.GetOSFromSeries(supportedSeries)
 				if err != nil {
-					return nil, errors.Trace(err)
+					// If we can't figure out a series written in the charm
+					// just skip it.
+					continue
 				}
 				supportedOperatingSystems[os] = true
 			}
@@ -2239,20 +2244,6 @@ func (st *State) networkEntityGlobalKey(globalKey string, providerId network.Id)
 	return st.docID(globalKey + ":" + string(providerId))
 }
 
-// PutAuditEntryFn returns a function which will persist
-// audit.AuditEntry instances to the database.
-func (st *State) PutAuditEntryFn() func(audit.AuditEntry) error {
-	insert := func(collectionName string, docs ...interface{}) error {
-		collection, closeCollection := st.db().GetCollection(collectionName)
-		defer closeCollection()
-
-		writeableCollection := collection.Writeable()
-
-		return errors.Trace(writeableCollection.Insert(docs...))
-	}
-	return stateaudit.PutAuditEntryFn(auditingC, insert)
-}
-
 // SetSLA sets the SLA on the current connected model.
 func (st *State) SetSLA(level, owner string, credentials []byte) error {
 	model, err := st.Model()
@@ -2329,7 +2320,10 @@ func (st *State) SetClockForTesting(clock clock.Clock) error {
 		return errors.Trace(err)
 	}
 	st.stateClock = clock
-	err = st.start(st.controllerTag)
+	if db, ok := st.database.(*database); ok {
+		db.clock = clock
+	}
+	err = st.start(st.controllerTag, nil)
 	if err != nil {
 		return errors.Trace(err)
 	}
