@@ -34,11 +34,11 @@ func (s *FilesystemStatusSuite) SetUpTest(c *gc.C) {
 	})
 	c.Assert(err, jc.ErrorIsNil)
 
-	filesystemAttachments, err := s.State.MachineFilesystemAttachments(machine.MachineTag())
+	filesystemAttachments, err := s.IAASModel.MachineFilesystemAttachments(machine.MachineTag())
 	c.Assert(err, jc.ErrorIsNil)
 	c.Assert(filesystemAttachments, gc.HasLen, 1)
 
-	filesystem, err := s.State.Filesystem(filesystemAttachments[0].Filesystem())
+	filesystem, err := s.IAASModel.Filesystem(filesystemAttachments[0].Filesystem())
 	c.Assert(err, jc.ErrorIsNil)
 
 	s.machine = machine
@@ -119,7 +119,7 @@ func (s *FilesystemStatusSuite) checkGetSetStatus(c *gc.C) {
 	err := s.filesystem.SetStatus(sInfo)
 	c.Check(err, jc.ErrorIsNil)
 
-	filesystem, err := s.State.Filesystem(s.filesystem.FilesystemTag())
+	filesystem, err := s.IAASModel.Filesystem(s.filesystem.FilesystemTag())
 	c.Assert(err, jc.ErrorIsNil)
 
 	statusInfo, err := filesystem.Status()
@@ -135,21 +135,21 @@ func (s *FilesystemStatusSuite) checkGetSetStatus(c *gc.C) {
 }
 
 func (s *FilesystemStatusSuite) TestGetSetStatusDying(c *gc.C) {
-	err := s.State.DestroyFilesystem(s.filesystem.FilesystemTag())
+	err := s.IAASModel.DestroyFilesystem(s.filesystem.FilesystemTag())
 	c.Assert(err, jc.ErrorIsNil)
 
 	s.checkGetSetStatus(c)
 }
 
 func (s *FilesystemStatusSuite) TestGetSetStatusDead(c *gc.C) {
-	err := s.State.DestroyFilesystem(s.filesystem.FilesystemTag())
+	err := s.IAASModel.DestroyFilesystem(s.filesystem.FilesystemTag())
 	c.Assert(err, jc.ErrorIsNil)
-	err = s.State.DetachFilesystem(s.machine.MachineTag(), s.filesystem.FilesystemTag())
+	err = s.IAASModel.DetachFilesystem(s.machine.MachineTag(), s.filesystem.FilesystemTag())
 	c.Assert(err, jc.ErrorIsNil)
-	err = s.State.RemoveFilesystemAttachment(s.machine.MachineTag(), s.filesystem.FilesystemTag())
+	err = s.IAASModel.RemoveFilesystemAttachment(s.machine.MachineTag(), s.filesystem.FilesystemTag())
 	c.Assert(err, jc.ErrorIsNil)
 
-	filesystem, err := s.State.Filesystem(s.filesystem.FilesystemTag())
+	filesystem, err := s.IAASModel.Filesystem(s.filesystem.FilesystemTag())
 	c.Assert(err, jc.ErrorIsNil)
 	c.Assert(filesystem.Life(), gc.Equals, state.Dead)
 
@@ -188,7 +188,7 @@ func (s *FilesystemStatusSuite) TestSetStatusPendingUnprovisioned(c *gc.C) {
 }
 
 func (s *FilesystemStatusSuite) TestSetStatusPendingProvisioned(c *gc.C) {
-	err := s.State.SetFilesystemInfo(s.filesystem.FilesystemTag(), state.FilesystemInfo{
+	err := s.IAASModel.SetFilesystemInfo(s.filesystem.FilesystemTag(), state.FilesystemInfo{
 		FilesystemId: "fs-id",
 	})
 	c.Assert(err, jc.ErrorIsNil)
