@@ -38,7 +38,7 @@ func (s *VolumeStatusSuite) SetUpTest(c *gc.C) {
 	c.Assert(err, jc.ErrorIsNil)
 	c.Assert(volumeAttachments, gc.HasLen, 1)
 
-	volume, err := s.State.Volume(volumeAttachments[0].Volume())
+	volume, err := s.IAASModel.Volume(volumeAttachments[0].Volume())
 	c.Assert(err, jc.ErrorIsNil)
 
 	s.machine = machine
@@ -125,7 +125,7 @@ func (s *VolumeStatusSuite) checkGetSetStatus(c *gc.C, volumeStatus status.Statu
 	err := s.volume.SetStatus(sInfo)
 	c.Check(err, jc.ErrorIsNil)
 
-	volume, err := s.State.Volume(s.volume.VolumeTag())
+	volume, err := s.IAASModel.Volume(s.volume.VolumeTag())
 	c.Assert(err, jc.ErrorIsNil)
 
 	statusInfo, err := volume.Status()
@@ -141,21 +141,21 @@ func (s *VolumeStatusSuite) checkGetSetStatus(c *gc.C, volumeStatus status.Statu
 }
 
 func (s *VolumeStatusSuite) TestGetSetStatusDying(c *gc.C) {
-	err := s.State.DestroyVolume(s.volume.VolumeTag())
+	err := s.IAASModel.DestroyVolume(s.volume.VolumeTag())
 	c.Assert(err, jc.ErrorIsNil)
 
 	s.checkGetSetStatus(c, status.Attaching)
 }
 
 func (s *VolumeStatusSuite) TestGetSetStatusDead(c *gc.C) {
-	err := s.State.DestroyVolume(s.volume.VolumeTag())
+	err := s.IAASModel.DestroyVolume(s.volume.VolumeTag())
 	c.Assert(err, jc.ErrorIsNil)
-	err = s.State.DetachVolume(s.machine.MachineTag(), s.volume.VolumeTag())
+	err = s.IAASModel.DetachVolume(s.machine.MachineTag(), s.volume.VolumeTag())
 	c.Assert(err, jc.ErrorIsNil)
-	err = s.State.RemoveVolumeAttachment(s.machine.MachineTag(), s.volume.VolumeTag())
+	err = s.IAASModel.RemoveVolumeAttachment(s.machine.MachineTag(), s.volume.VolumeTag())
 	c.Assert(err, jc.ErrorIsNil)
 
-	volume, err := s.State.Volume(s.volume.VolumeTag())
+	volume, err := s.IAASModel.Volume(s.volume.VolumeTag())
 	c.Assert(err, jc.ErrorIsNil)
 	c.Assert(volume.Life(), gc.Equals, state.Dead)
 
@@ -194,7 +194,7 @@ func (s *VolumeStatusSuite) TestSetStatusPendingUnprovisioned(c *gc.C) {
 }
 
 func (s *VolumeStatusSuite) TestSetStatusPendingProvisioned(c *gc.C) {
-	err := s.State.SetVolumeInfo(s.volume.VolumeTag(), state.VolumeInfo{
+	err := s.IAASModel.SetVolumeInfo(s.volume.VolumeTag(), state.VolumeInfo{
 		VolumeId: "vol-ume",
 	})
 	c.Assert(err, jc.ErrorIsNil)
