@@ -38,11 +38,19 @@ type SubnetInfo struct {
 	// SpaceName is the name of the space the subnet is associated with. It
 	// can be empty if the subnet is not associated with a space yet.
 	SpaceName string
+
+	// FanUnderlay is the CIDR of the local underlaying fan network, it allows easy
+	// identification of the device the FAN is running on. Empty if not a FAN subnet.
+	FanLocalUnderlay string
+
+	// FanOverlay is the CIDR of the complete FAN setup. Empty if not a FAN subnet.
+	FanOverlay string
 }
 
 type Subnet struct {
-	st  *State
-	doc subnetDoc
+	st        *State
+	doc       subnetDoc
+	spaceName string
 }
 
 type subnetDoc struct {
@@ -56,8 +64,10 @@ type subnetDoc struct {
 	AvailabilityZone  string `bson:"availabilityzone,omitempty"`
 	// TODO: add IsPublic to SubnetArgs, add an IsPublic method and add
 	// IsPublic to migration import/export.
-	IsPublic  bool   `bson:"is-public,omitempty"`
-	SpaceName string `bson:"space-name,omitempty"`
+	IsPublic         bool   `bson:"is-public,omitempty"`
+	SpaceName        string `bson:"space-name,omitempty"`
+	FanLocalUnderlay string `bson:"fan-local-underlay,omitempty"`
+	FanOverlay       string `bson:"fan-overlay,omitempty"`
 }
 
 // Life returns whether the subnet is Alive, Dying or Dead.
@@ -80,6 +90,14 @@ func (s *Subnet) GoString() string {
 	return s.String()
 }
 
+func (s *Subnet) FanOverlay() string {
+	return s.doc.FanOverlay
+}
+
+func (s *Subnet) FanLocalUnderlay() string {
+	return s.doc.FanLocalUnderlay
+}
+
 // EnsureDead sets the Life of the subnet to Dead, if it's Alive. If the subnet
 // is already Dead, no error is returned. When the subnet is no longer Alive or
 // already removed, errNotAlive is returned.
@@ -97,7 +115,7 @@ func (s *Subnet) EnsureDead() (err error) {
 		Assert: isAliveDoc,
 	}}
 
-	txnErr := s.st.runTransaction(ops)
+	txnErr := s.st.db().RunTransaction(ops)
 	if txnErr == nil {
 		s.doc.Life = Dead
 		return nil
@@ -126,7 +144,7 @@ func (s *Subnet) Remove() (err error) {
 		ops = append(ops, op)
 	}
 
-	txnErr := s.st.runTransaction(ops)
+	txnErr := s.st.db().RunTransaction(ops)
 	if txnErr == nil {
 		return nil
 	}
@@ -158,7 +176,7 @@ func (s *Subnet) AvailabilityZone() string {
 // SpaceName returns the space the subnet is associated with. If the subnet is
 // not associated with a space it will be the empty string.
 func (s *Subnet) SpaceName() string {
-	return s.doc.SpaceName
+	return s.spaceName
 }
 
 // ProviderNetworkId returns the provider id of the network containing
@@ -199,6 +217,16 @@ func (s *Subnet) Refresh() error {
 	if err != nil {
 		return errors.Errorf("cannot refresh subnet %q: %v", s, err)
 	}
+	s.spaceName = s.doc.SpaceName
+	if s.doc.FanLocalUnderlay != "" {
+		overlayDoc := &subnetDoc{}
+		err = subnets.FindId(s.doc.FanLocalUnderlay).One(overlayDoc)
+		if err != nil {
+			return errors.Annotatef(err, "Can't find underlay network %v for FAN %v", s.doc.FanLocalUnderlay, s.doc.CIDR)
+		} else {
+			s.spaceName = overlayDoc.SpaceName
+		}
+	}
 	return nil
 }
 
@@ -230,7 +258,7 @@ func (st *State) AddSubnet(args SubnetInfo) (subnet *Subnet, err error) {
 		}
 		return ops, nil
 	}
-	err = st.run(buildTxn)
+	err = st.db().Run(buildTxn)
 	if err != nil {
 		return nil, errors.Trace(err)
 	}
@@ -249,8 +277,10 @@ func (st *State) newSubnetFromArgs(args SubnetInfo) (*Subnet, error) {
 		ProviderNetworkId: string(args.ProviderNetworkId),
 		AvailabilityZone:  args.AvailabilityZone,
 		SpaceName:         args.SpaceName,
+		FanLocalUnderlay:  args.FanLocalUnderlay,
+		FanOverlay:        args.FanOverlay,
 	}
-	subnet := &Subnet{doc: subDoc, st: st}
+	subnet := &Subnet{doc: subDoc, st: st, spaceName: args.SpaceName}
 	err := subnet.Validate()
 	if err != nil {
 		return nil, errors.Trace(err)
@@ -270,6 +300,8 @@ func (st *State) addSubnetOps(args SubnetInfo) []txn.Op {
 		ProviderNetworkId: string(args.ProviderNetworkId),
 		AvailabilityZone:  args.AvailabilityZone,
 		SpaceName:         args.SpaceName,
+		FanLocalUnderlay:  args.FanLocalUnderlay,
+		FanOverlay:        args.FanOverlay,
 	}
 	ops := []txn.Op{
 		{
@@ -298,7 +330,18 @@ func (st *State) Subnet(cidr string) (*Subnet, error) {
 	if err != nil {
 		return nil, errors.Annotatef(err, "cannot get subnet %q", cidr)
 	}
-	return &Subnet{st, *doc}, nil
+	spaceName := doc.SpaceName
+	if doc.FanLocalUnderlay != "" {
+		overlayDoc := &subnetDoc{}
+		err = subnets.FindId(doc.FanLocalUnderlay).One(overlayDoc)
+		if err != nil {
+			return nil, errors.Annotatef(err, "Can't find underlay network %v for FAN %v", doc.FanLocalUnderlay, doc.CIDR)
+		} else {
+			spaceName = overlayDoc.SpaceName
+		}
+	}
+
+	return &Subnet{st, *doc, spaceName}, nil
 }
 
 // AllSubnets returns all known subnets in the model.
@@ -311,8 +354,18 @@ func (st *State) AllSubnets() (subnets []*Subnet, err error) {
 	if err != nil {
 		return nil, errors.Annotatef(err, "cannot get all subnets")
 	}
+	cidrToSpace := make(map[string]string)
 	for _, doc := range docs {
-		subnets = append(subnets, &Subnet{st, doc})
+		cidrToSpace[doc.CIDR] = doc.SpaceName
+	}
+	for _, doc := range docs {
+		spaceName := doc.SpaceName
+		if doc.FanLocalUnderlay != "" {
+			if space, ok := cidrToSpace[doc.FanLocalUnderlay]; ok {
+				spaceName = space
+			}
+		}
+		subnets = append(subnets, &Subnet{st, doc, spaceName})
 	}
 	return subnets, nil
 }
