@@ -2445,7 +2445,7 @@ func (u *Unit) RunningActions() ([]Action, error) {
 // reestablish normal workflow. The retryHooks parameter informs
 // whether to attempt to reexecute previous failed hooks or to continue
 // as if they had succeeded before.
-func (u *Unit) Resolve(noretryHooks bool) error {
+func (u *Unit) Resolve(retryHooks bool) error {
 	// We currently check agent status to see if a unit is
 	// in error state. As the new Juju Health work is completed,
 	// this will change to checking the unit status.
@@ -2456,9 +2456,9 @@ func (u *Unit) Resolve(noretryHooks bool) error {
 	if statusInfo.Status != status.Error {
 		return errors.Errorf("unit %q is not in an error state", u)
 	}
-	mode := ResolvedRetryHooks
-	if noretryHooks {
-		mode = ResolvedNoHooks
+	mode := ResolvedNoHooks
+	if retryHooks {
+		mode = ResolvedRetryHooks
 	}
 	return u.SetResolved(mode)
 }
