@@ -20,6 +20,7 @@ import (
 	"github.com/juju/juju/cloud"
 	"github.com/juju/juju/controller"
 	"github.com/juju/juju/environs/config"
+	"github.com/juju/juju/mongo/utils"
 	"github.com/juju/juju/state/globalclock"
 	"github.com/juju/juju/state/lease"
 	"github.com/juju/juju/status"
@@ -101,7 +102,7 @@ func RenameAddModelPermission(st *State) error {
 			Update: bson.D{{"$set", bson.D{{"access", "add-model"}}}},
 		})
 	}
-	if err := iter.Err(); err != nil {
+	if err := iter.Close(); err != nil {
 		return errors.Trace(err)
 	}
 	return st.runRawTransaction(ops)
@@ -228,7 +229,7 @@ func stripLocalFromFields(st *State, collName string, fields ...string) ([]txn.O
 			}}...)
 		}
 	}
-	if err := iter.Err(); err != nil {
+	if err := iter.Close(); err != nil {
 		return nil, errors.Trace(err)
 	}
 	return ops, nil
@@ -261,7 +262,7 @@ func AddMigrationAttempt(st *State) error {
 			Update: bson.D{{"$set", bson.D{{"attempt", attempt}}}},
 		})
 	}
-	if err := iter.Err(); err != nil {
+	if err := iter.Close(); err != nil {
 		return errors.Annotate(err, "iterating migrations")
 	}
 
@@ -408,7 +409,7 @@ func updateLegacyLXDCloudsOps(st *State, endpoint string) ([]txn.Op, error) {
 		for _, region := range c.Regions {
 			if region.Endpoint == "" {
 				set = append(set, bson.DocElem{
-					"regions." + region.Name + ".endpoint",
+					"regions." + utils.EscapeKey(region.Name) + ".endpoint",
 					endpoint,
 				})
 			}
@@ -429,6 +430,7 @@ func updateLegacyLXDCredentialsOps(st *State, cred cloud.Credential) ([]txn.Op,
 	coll, closer := st.db().GetRawCollection(cloudCredentialsC)
 	defer closer()
 	iter := coll.Find(bson.M{"auth-type": "empty"}).Iter()
+	defer iter.Close()
 	var doc cloudCredentialDoc
 	for iter.Next(&doc) {
 		cloudCredentialTag, err := doc.cloudCredentialTag()
@@ -447,7 +449,7 @@ func updateLegacyLXDCredentialsOps(st *State, cred cloud.Credential) ([]txn.Op,
 		upgradesLogger.Infof("updating credential %q: %v", cloudCredentialTag, op)
 		ops = append(ops, op)
 	}
-	if err := iter.Err(); err != nil {
+	if err := iter.Close(); err != nil {
 		return nil, errors.Trace(err)
 	}
 	return ops, nil
@@ -472,6 +474,7 @@ func UpgradeNoProxyDefaults(st *State) error {
 	coll, closer := st.db().GetRawCollection(settingsC)
 	defer closer()
 	iter := coll.Find(bson.D{}).Iter()
+	defer iter.Close()
 	var doc settingsDoc
 	for iter.Next(&doc) {
 		noProxyVal := doc.Settings[config.NoProxyKey]
@@ -481,13 +484,15 @@ func UpgradeNoProxyDefaults(st *State) error {
 		}
 		noProxy = upgradeNoProxy(noProxy)
 		doc.Settings[config.NoProxyKey] = noProxy
-		ops = append(ops,
-			txn.Op{
-				C:      settingsC,
-				Id:     doc.DocID,
-				Assert: txn.DocExists,
-				Update: bson.M{"$set": bson.M{"settings": doc.Settings}},
-			})
+		ops = append(ops, txn.Op{
+			C:      settingsC,
+			Id:     doc.DocID,
+			Assert: txn.DocExists,
+			Update: bson.M{"$set": bson.M{"settings": doc.Settings}},
+		})
+	}
+	if err := iter.Close(); err != nil {
+		return errors.Trace(err)
 	}
 	if len(ops) > 0 {
 		return errors.Trace(st.runRawTransaction(ops))
@@ -600,6 +605,7 @@ func RemoveNilValueApplicationSettings(st *State) error {
 	coll, closer := st.db().GetRawCollection(settingsC)
 	defer closer()
 	iter := coll.Find(bson.M{"_id": bson.M{"$regex": "^.*:a#.*"}}).Iter()
+	defer iter.Close()
 	var ops []txn.Op
 	var doc settingsDoc
 	for iter.Next(&doc) {
@@ -620,6 +626,9 @@ func RemoveNilValueApplicationSettings(st *State) error {
 			})
 		}
 	}
+	if err := iter.Close(); err != nil {
+		return errors.Trace(err)
+	}
 	if len(ops) > 0 {
 		return errors.Trace(st.runRawTransaction(ops))
 	}
@@ -667,30 +676,41 @@ func maybeUpdateSettings(settings map[string]interface{}, key string, value inte
 	return false
 }
 
-// AddStatusHistoryPruneSettings adds the model settings
-// to control log pruning if they are missing.
-func AddStatusHistoryPruneSettings(st *State) error {
-	coll, closer := st.db().GetRawCollection(settingsC)
-	defer closer()
-
+// applyToAllModelSettings iterates the model settings documents and applys the
+// passed in function to them.  If the function returns 'true' it indicates the
+// settings have been modified, and they should be written back to the
+// database.
+// Note that if there are any problems with updating settings, then none of the
+// changes will be applied, as they are all updated in a single transaction.
+func applyToAllModelSettings(st *State, change func(*settingsDoc) (bool, error)) error {
 	uuids, err := st.AllModelUUIDs()
 	if err != nil {
 		return errors.Trace(err)
 	}
+
+	coll, closer := st.db().GetRawCollection(settingsC)
+	defer closer()
+
 	var ids []string
 	for _, uuid := range uuids {
 		ids = append(ids, uuid+":e")
 	}
 
 	iter := coll.Find(bson.M{"_id": bson.M{"$in": ids}}).Iter()
+	defer iter.Close()
+
 	var ops []txn.Op
 	var doc settingsDoc
 	for iter.Next(&doc) {
-		settingsChanged :=
-			maybeUpdateSettings(doc.Settings, config.MaxStatusHistoryAge, config.DefaultStatusHistoryAge)
-		settingsChanged =
-			maybeUpdateSettings(doc.Settings, config.MaxStatusHistorySize, config.DefaultStatusHistorySize) || settingsChanged
+		settingsChanged, err := change(&doc)
+		if err != nil {
+			return errors.Trace(err)
+		}
 		if settingsChanged {
+			// Note: some settings contain embedded '.' or '$' characters and
+			// need to be encoded to save back to the database. We don't need
+			// to do that here, only because we don't use those keys in our
+			// Model and Controller settings documents.
 			ops = append(ops, txn.Op{
 				C:      settingsC,
 				Id:     doc.DocID,
@@ -699,46 +719,43 @@ func AddStatusHistoryPruneSettings(st *State) error {
 			})
 		}
 	}
+	if err := iter.Close(); err != nil {
+		return errors.Trace(err)
+	}
 	if len(ops) > 0 {
 		return errors.Trace(st.runRawTransaction(ops))
 	}
 	return nil
 }
 
-// AddActionPruneSettings adds the model settings
+// AddStatusHistoryPruneSettings adds the model settings
 // to control log pruning if they are missing.
-func AddActionPruneSettings(st *State) error {
-	coll, closer := st.db().GetRawCollection(settingsC)
-	defer closer()
-
-	uuids, err := st.AllModelUUIDs()
+func AddStatusHistoryPruneSettings(st *State) error {
+	err := applyToAllModelSettings(st, func(doc *settingsDoc) (bool, error) {
+		settingsChanged :=
+			maybeUpdateSettings(doc.Settings, config.MaxStatusHistoryAge, config.DefaultStatusHistoryAge)
+		settingsChanged =
+			maybeUpdateSettings(doc.Settings, config.MaxStatusHistorySize, config.DefaultStatusHistorySize) || settingsChanged
+		return settingsChanged, nil
+	})
 	if err != nil {
 		return errors.Trace(err)
 	}
-	var ids []string
-	for _, uuid := range uuids {
-		ids = append(ids, uuid+":e")
-	}
+	return nil
+}
 
-	iter := coll.Find(bson.M{"_id": bson.M{"$in": ids}}).Iter()
-	var ops []txn.Op
-	var doc settingsDoc
-	for iter.Next(&doc) {
+// AddActionPruneSettings adds the model settings
+// to control log pruning if they are missing.
+func AddActionPruneSettings(st *State) error {
+	err := applyToAllModelSettings(st, func(doc *settingsDoc) (bool, error) {
 		settingsChanged :=
 			maybeUpdateSettings(doc.Settings, config.MaxActionResultsAge, config.DefaultActionResultsAge)
 		settingsChanged =
 			maybeUpdateSettings(doc.Settings, config.MaxActionResultsSize, config.DefaultActionResultsSize) || settingsChanged
-		if settingsChanged {
-			ops = append(ops, txn.Op{
-				C:      settingsC,
-				Id:     doc.DocID,
-				Assert: txn.DocExists,
-				Update: bson.M{"$set": bson.M{"settings": doc.Settings}},
-			})
-		}
-	}
-	if len(ops) > 0 {
-		return errors.Trace(st.runRawTransaction(ops))
+		return settingsChanged, nil
+	})
+	if err != nil {
+		return errors.Trace(err)
 	}
 	return nil
 }
@@ -747,35 +764,13 @@ func AddActionPruneSettings(st *State) error {
 // to control how often to run the update-status hook
 // if they are missing.
 func AddUpdateStatusHookSettings(st *State) error {
-	coll, closer := st.db().GetRawCollection(settingsC)
-	defer closer()
-
-	uuids, err := st.AllModelUUIDs()
-	if err != nil {
-		return errors.Trace(err)
-	}
-	var ids []string
-	for _, uuid := range uuids {
-		ids = append(ids, uuid+":e")
-	}
-
-	iter := coll.Find(bson.M{"_id": bson.M{"$in": ids}}).Iter()
-	var ops []txn.Op
-	var doc settingsDoc
-	for iter.Next(&doc) {
+	err := applyToAllModelSettings(st, func(doc *settingsDoc) (bool, error) {
 		settingsChanged :=
 			maybeUpdateSettings(doc.Settings, config.UpdateStatusHookInterval, config.DefaultUpdateStatusHookInterval)
-		if settingsChanged {
-			ops = append(ops, txn.Op{
-				C:      settingsC,
-				Id:     doc.DocID,
-				Assert: txn.DocExists,
-				Update: bson.M{"$set": bson.M{"settings": doc.Settings}},
-			})
-		}
-	}
-	if len(ops) > 0 {
-		return errors.Trace(st.runRawTransaction(ops))
+		return settingsChanged, nil
+	})
+	if err != nil {
+		return errors.Trace(err)
 	}
 	return nil
 }
@@ -897,8 +892,9 @@ func SplitLogCollections(st *State) error {
 	seen := set.NewStrings()
 
 	iter := oldLogs.Find(nil).Iter()
-	var doc bson.M
+	defer iter.Close()
 
+	var doc bson.M
 	for iter.Next(&doc) {
 		modelUUID := doc["e"].(string)
 		newCollName := logCollectionName(modelUUID)
@@ -923,6 +919,9 @@ func SplitLogCollections(st *State) error {
 		}
 		doc = nil
 	}
+	if err := iter.Close(); err != nil {
+		return errors.Trace(err)
+	}
 
 	// drop the old collection
 	if err := oldLogs.DropCollection(); err != nil {
@@ -996,6 +995,7 @@ func CorrectRelationUnitCounts(st *State) error {
 	}
 	relationsToUpdate := set.NewStrings()
 	iter := scopesColl.Find(nil).Iter()
+	defer iter.Close()
 
 	for iter.Next(&scope) {
 		// Scope key looks like: r#<relation id>#[<principal unit for container scope>#]<role>#<unit>
@@ -1090,6 +1090,7 @@ func collectRelationInfo(coll *mgo.Collection) (map[string]*relationUnitCountInf
 	}
 
 	iter := coll.Find(nil).Iter()
+	defer iter.Close()
 	for iter.Next(&doc) {
 		endpoints := set.NewStrings()
 		for _, epDoc := range doc.Endpoints {
@@ -1155,6 +1156,7 @@ func AddModelEnvironVersion(st *State) error {
 
 	var ops []txn.Op
 	iter := coll.Find(nil).Iter()
+	defer iter.Close()
 	for iter.Next(&doc) {
 		if doc.EnvironVersion != nil {
 			continue
@@ -1185,6 +1187,7 @@ func AddModelType(st *State) error {
 
 	var ops []txn.Op
 	iter := coll.Find(nil).Iter()
+	defer iter.Close()
 	for iter.Next(&doc) {
 		if doc.Type != "" {
 			continue
@@ -1240,8 +1243,20 @@ func migrateModelLeasesToGlobalTime(st *State) error {
 				Remove: true,
 			})
 			if doc.Type != "lease" {
+				upgradesLogger.Tracef("deleting old lease doc %q", doc.DocID)
 				continue
 			}
+			// Check if the target exists
+			if _, err := lease.LookupLease(coll, doc.Namespace, doc.Name); err == nil {
+				// target already exists, it takes precedence over an old doc, which we still want to delete
+				upgradesLogger.Infof("new lease %q %q already exists, simply deleting old lease %q",
+					doc.Namespace, doc.Name, doc.DocID)
+				continue
+			} else if err != mgo.ErrNotFound {
+				// We got an unknown error looking up this doc, don't suppress it
+				return nil, err
+			}
+			upgradesLogger.Tracef("migrating lease %q to new lease structure", doc.DocID)
 			claimOps, err := lease.ClaimLeaseOps(
 				doc.Namespace,
 				doc.Name,
@@ -1311,3 +1326,124 @@ func addRelationStatus(st *State) error {
 	})
 	return errors.Annotate(err, "adding relation status")
 }
+
+// MoveOldAuditLog renames the no-longer-needed audit.log collection
+// to old-audit.log if it has any rows - if it's empty it deletes it.
+func MoveOldAuditLog(st *State) error {
+	names, err := st.MongoSession().DB("juju").CollectionNames()
+	if err != nil {
+		return errors.Trace(err)
+	}
+	if !set.NewStrings(names...).Contains("audit.log") {
+		// No audit log collection to move.
+		return nil
+	}
+
+	coll, closer := st.db().GetRawCollection("audit.log")
+	defer closer()
+
+	rows, err := coll.Count()
+	if err != nil {
+		return errors.Trace(err)
+	}
+	if rows == 0 {
+		return errors.Trace(coll.DropCollection())
+	}
+	session := st.MongoSession()
+	renameCommand := bson.D{
+		{"renameCollection", "juju.audit.log"},
+		{"to", "juju.old-audit.log"},
+	}
+	return errors.Trace(session.Run(renameCommand, nil))
+}
+
+// DeleteCloudImageMetadata deletes any non-custom cloud
+// image metadata records from the cloudimagemetadata collection.
+func DeleteCloudImageMetadata(st *State) error {
+	coll, closer := st.db().GetRawCollection(cloudimagemetadataC)
+	defer closer()
+
+	bulk := coll.Bulk()
+	bulk.Unordered()
+	bulk.RemoveAll(bson.D{{"source", bson.D{{"$ne", "custom"}}}})
+	_, err := bulk.Run()
+	return errors.Annotate(err, "deleting cloud image metadata records")
+}
+
+// UpgradeDefaultContainerImageStreamConfig ensures that the config value for
+// container-image-stream is set to its default value, "released".
+func UpgradeContainerImageStreamDefault(st *State) error {
+	err := applyToAllModelSettings(st, func(doc *settingsDoc) (bool, error) {
+		ciStreamVal, keySet := doc.Settings[config.ContainerImageStreamKey]
+		if keySet {
+			if ciStream, _ := ciStreamVal.(string); ciStream != "" {
+				return false, nil
+			}
+		}
+		doc.Settings[config.ContainerImageStreamKey] = "released"
+		return true, nil
+	})
+	if err != nil {
+		return errors.Trace(err)
+	}
+	return nil
+}
+
+// RemoveContainerImageStreamFromNonModelSettings
+// In 2.3.6 we accidentally had an upgrade step that added
+// "container-image-stream": "released" to all settings documents, not just the
+// ones relating to Model data.
+// This removes it from all the ones that aren't model docs if it is exactly
+// what we would have added in 2.3.6
+func RemoveContainerImageStreamFromNonModelSettings(st *State) error {
+	uuids, err := st.AllModelUUIDs()
+	if err != nil {
+		return errors.Trace(err)
+	}
+
+	modelDocIDs := set.NewStrings()
+	for _, uuid := range uuids {
+		modelDocIDs.Add(uuid + ":e")
+	}
+	coll, closer := st.db().GetRawCollection(settingsC)
+	defer closer()
+
+	iter := coll.Find(nil).Iter()
+	defer iter.Close()
+
+	// This is the key for the field that was accidentally added in 2.3.6
+	// settings.container-image-stream
+	const dbSettingsKey = "settings." + config.ContainerImageStreamKey
+
+	var ops []txn.Op
+	var doc settingsDoc
+	for iter.Next(&doc) {
+		if modelDocIDs.Contains(doc.DocID) {
+			// this is a model document, whatever was set here should stay
+			continue
+		}
+		if stream, ok := doc.Settings[config.ContainerImageStreamKey]; !ok {
+			// doesn't contain ContainerImageStreamKey
+			continue
+		} else if stream != "released" {
+			// definitely wasn't set by the 2.3.6 upgrade step
+			continue
+		}
+		// We just unset the one field that we accidentally set before, so we
+		// don't have to worry about serialization of the other keys in the
+		// document.
+		ops = append(ops, txn.Op{
+			C:      settingsC,
+			Id:     doc.DocID,
+			Assert: txn.DocExists,
+			Update: bson.M{"$unset": bson.M{dbSettingsKey: 1}},
+		})
+	}
+	if err := iter.Close(); err != nil {
+		return errors.Trace(err)
+	}
+	if len(ops) > 0 {
+		return errors.Trace(st.runRawTransaction(ops))
+	}
+	return nil
+}
