@@ -20,6 +20,7 @@ import (
 
 	"github.com/juju/juju/cloud"
 	"github.com/juju/juju/environs/config"
+	"github.com/juju/juju/state/cloudimagemetadata"
 	"github.com/juju/juju/storage/provider"
 	coretesting "github.com/juju/juju/testing"
 )
@@ -1907,6 +1908,89 @@ func (s *upgradesSuite) TestMigrateLeasesToGlobalTime(c *gc.C) {
 	)
 }
 
+func (s *upgradesSuite) TestMigrateLeasesToGlobalTimeWithNewTarget(c *gc.C) {
+	// It is possible that API servers will try to coordinate the singular lease before we can get to the upgrade steps.
+	// While upgrading leases, if we encounter any leases that already exist in the new GlobalTime format, they should
+	// be considered authoritative, and the old lease should just be deleted.
+	leases, closer := s.state.db().GetRawCollection(leasesC)
+	defer closer()
+
+	// Use the non-controller model to ensure we can run the function
+	// across multiple models.
+	otherState := s.makeModel(c, "crack-up", coretesting.Attrs{})
+	defer otherState.Close()
+
+	uuid := otherState.ModelUUID()
+
+	err := leases.Insert(bson.M{
+		"_id":        uuid + ":some-garbage",
+		"model-uuid": uuid,
+	}, bson.M{
+		"_id":        uuid + ":clock#some-namespace#some-name#",
+		"model-uuid": uuid,
+		"type":       "clock",
+	}, bson.M{
+		"_id":        uuid + ":lease#some-namespace#some-name#",
+		"model-uuid": uuid,
+		"type":       "lease",
+		"namespace":  "some-namespace",
+		"name":       "some-name",
+		"holder":     "hand",
+		"expiry":     "later",
+		"writer":     "ghost",
+	}, bson.M{
+		"_id":        uuid + ":lease#some-namespace2#some-name2#",
+		"model-uuid": uuid,
+		"type":       "lease",
+		"namespace":  "some-namespace2",
+		"name":       "some-name2",
+		"holder":     "hand",
+		"expiry":     "later",
+		"writer":     "ghost",
+	}, bson.M{
+		// some-namespace2 has already been created in the new format
+		"_id":        uuid + ":some-namespace2#some-name2#",
+		"model-uuid": uuid,
+		"namespace":  "some-namespace2",
+		"name":       "some-name2",
+		"holder":     "foot",
+		"start":      int64(0),
+		"duration":   int64(time.Minute),
+		"writer":     "gobble",
+	})
+	c.Assert(err, jc.ErrorIsNil)
+
+	// - garbage doc is left alone has it has no "type" field
+	// - clock doc is removed, but no replacement required
+	// - lease doc is removed and replaced
+	// - second old lease doc is removed, and the new lease doc is not overwritten
+	expectedLeases := []bson.M{{
+		"_id":        uuid + ":some-garbage",
+		"model-uuid": uuid,
+	}, {
+		"_id":        uuid + ":some-namespace#some-name#",
+		"model-uuid": uuid,
+		"namespace":  "some-namespace",
+		"name":       "some-name",
+		"holder":     "hand",
+		"start":      int64(0),
+		"duration":   int64(time.Minute),
+		"writer":     "ghost",
+	}, {
+		"_id":        uuid + ":some-namespace2#some-name2#",
+		"model-uuid": uuid,
+		"namespace":  "some-namespace2",
+		"name":       "some-name2",
+		"holder":     "foot",
+		"start":      int64(0),
+		"duration":   int64(time.Minute),
+		"writer":     "gobble",
+	}}
+	s.assertUpgradedData(c, MigrateLeasesToGlobalTime,
+		expectUpgradedData{leases, expectedLeases},
+	)
+}
+
 func (s *upgradesSuite) TestAddRelationStatus(c *gc.C) {
 	// Set a test clock so we can dictate the
 	// time set in the new status doc.
@@ -1978,3 +2062,260 @@ func (s *upgradesSuite) TestAddRelationStatus(c *gc.C) {
 		expectUpgradedData{statuses, expectedStatuses},
 	)
 }
+
+func (s *upgradesSuite) TestMoveOldAuditLogNoRecords(c *gc.C) {
+	// Ensure an empty audit log collection exists.
+	auditLog, closer := s.state.db().GetRawCollection("audit.log")
+	defer closer()
+	err := auditLog.Create(&mgo.CollectionInfo{})
+	c.Assert(err, jc.ErrorIsNil)
+
+	// Sanity check.
+	count, err := auditLog.Count()
+	c.Assert(err, jc.ErrorIsNil)
+	c.Assert(count, gc.Equals, 0)
+
+	err = MoveOldAuditLog(s.state)
+	c.Assert(err, jc.ErrorIsNil)
+
+	db := s.state.MongoSession().DB("juju")
+	names, err := db.CollectionNames()
+	c.Assert(err, jc.ErrorIsNil)
+	c.Assert(set.NewStrings(names...).Contains("audit.log"), jc.IsFalse)
+
+	err = MoveOldAuditLog(s.state)
+	c.Assert(err, jc.ErrorIsNil)
+}
+
+func (s *upgradesSuite) TestMoveOldAuditLogRename(c *gc.C) {
+	auditLog, closer := s.state.db().GetRawCollection("audit.log")
+	defer closer()
+	oldLog, oldCloser := s.state.db().GetRawCollection("old-audit.log")
+	defer oldCloser()
+
+	// Put some rows into audit log and check that they're moved.
+	data := []bson.M{
+		{"_id": "band", "king": "gizzard", "lizard": "wizard"},
+		{"_id": "song", "crumbling": "castle"},
+	}
+	err := auditLog.Insert(data[0], data[1])
+	c.Assert(err, jc.ErrorIsNil)
+	s.assertUpgradedData(c, MoveOldAuditLog,
+		expectUpgradedData{oldLog, data},
+	)
+
+	db := s.state.MongoSession().DB("juju")
+	names, err := db.CollectionNames()
+	c.Assert(err, jc.ErrorIsNil)
+	c.Assert(set.NewStrings(names...).Contains("audit.log"), jc.IsFalse)
+}
+
+func (s *upgradesSuite) TestDeleteCloudImageMetadata(c *gc.C) {
+	stor := cloudimagemetadata.NewStorage(cloudimagemetadataC, &environMongo{s.state})
+	attrs1 := cloudimagemetadata.MetadataAttributes{
+		Stream:  "stream",
+		Region:  "region-test",
+		Version: "14.04",
+		Series:  "trusty",
+		Arch:    "arch",
+		Source:  "custom",
+	}
+	attrs2 := cloudimagemetadata.MetadataAttributes{
+		Stream:  "chalk",
+		Region:  "nether",
+		Version: "12.04",
+		Series:  "precise",
+		Arch:    "amd64",
+		Source:  "test",
+	}
+	now := time.Now().UnixNano()
+	added := []cloudimagemetadata.Metadata{
+		{attrs1, 0, "1", now},
+		{attrs2, 0, "2", now},
+	}
+	err := stor.SaveMetadataNoExpiry(added)
+	c.Assert(err, jc.ErrorIsNil)
+
+	expected := []bson.M{{
+		"_id":               "stream:region-test:trusty:arch:::custom",
+		"date_created":      now,
+		"image_id":          "1",
+		"priority":          0,
+		"stream":            "stream",
+		"region":            "region-test",
+		"series":            "trusty",
+		"arch":              "arch",
+		"root_storage_size": int64(0),
+		"source":            "custom",
+	}}
+
+	coll, closer := s.state.db().GetRawCollection(cloudimagemetadataC)
+	defer closer()
+	s.assertUpgradedData(c, DeleteCloudImageMetadata, expectUpgradedData{coll, expected})
+}
+
+func (s *upgradesSuite) TestUpgradeContainerImageStreamDefault(c *gc.C) {
+	// Value not set
+	m1 := s.makeModel(c, "m1", coretesting.Attrs{
+		"other-setting": "val",
+	})
+	defer m1.Close()
+	// Value set to the empty string
+	m2 := s.makeModel(c, "m2", coretesting.Attrs{
+		"container-image-stream": "",
+		"other-setting":          "val",
+	})
+	defer m2.Close()
+	// Value set to something other that default
+	m3 := s.makeModel(c, "m3", coretesting.Attrs{
+		"container-image-stream": "daily",
+	})
+	defer m3.Close()
+
+	settingsColl, settingsCloser := s.state.db().GetRawCollection(settingsC)
+	defer settingsCloser()
+	// To simulate a 2.3.5 without any setting, delete the record from it.
+	err := settingsColl.UpdateId(m1.ModelUUID()+":e",
+		bson.M{"$unset": bson.M{"settings.container-image-stream": 1}},
+	)
+	c.Assert(err, jc.ErrorIsNil)
+	// And an extra document from somewhere else that we shouldn't touch
+	err = settingsColl.Insert(
+		bson.M{
+			"_id":      "not-a-model",
+			"settings": bson.M{"other-setting": "val"},
+		},
+	)
+	c.Assert(err, jc.ErrorIsNil)
+
+	// Read all the settings from the database, but make sure to change the
+	// documents we think we're changing, and the rest should go through
+	// unchanged.
+	var rawSettings bson.M
+	iter := settingsColl.Find(nil).Sort("_id").Iter()
+	defer iter.Close()
+
+	expectedSettings := []bson.M{}
+
+	expectedChanges := map[string]bson.M{
+		m1.ModelUUID() + ":e": bson.M{"container-image-stream": "released", "other-setting": "val"},
+		m2.ModelUUID() + ":e": bson.M{"container-image-stream": "released", "other-setting": "val"},
+		m3.ModelUUID() + ":e": bson.M{"container-image-stream": "daily"},
+		"not-a-model":         bson.M{"other-setting": "val"},
+	}
+	for iter.Next(&rawSettings) {
+		expSettings := copyMap(rawSettings, nil)
+		delete(expSettings, "txn-queue")
+		delete(expSettings, "txn-revno")
+		delete(expSettings, "version")
+		id, ok := expSettings["_id"]
+		c.Assert(ok, jc.IsTrue)
+		idStr, ok := id.(string)
+		c.Assert(ok, jc.IsTrue)
+		c.Assert(idStr, gc.Not(gc.Equals), "")
+		if changes, ok := expectedChanges[idStr]; ok {
+			raw, ok := expSettings["settings"]
+			c.Assert(ok, jc.IsTrue)
+			settings, ok := raw.(bson.M)
+			c.Assert(ok, jc.IsTrue)
+			for k, v := range changes {
+				settings[k] = v
+			}
+		}
+		expectedSettings = append(expectedSettings, expSettings)
+	}
+	c.Assert(iter.Close(), jc.ErrorIsNil)
+
+	s.assertUpgradedData(c, UpgradeContainerImageStreamDefault,
+		expectUpgradedData{settingsColl, expectedSettings},
+	)
+}
+
+func (s *upgradesSuite) TestRemoveContainerImageStreamFromNonModelSettings(c *gc.C) {
+	// a model with a valid setting
+	m1 := s.makeModel(c, "m1", coretesting.Attrs{
+		"other-setting":          "val",
+		"container-image-stream": "released",
+	})
+	defer m1.Close()
+	// a model with a custom setting
+	m2 := s.makeModel(c, "m2", coretesting.Attrs{
+		"container-image-stream": "daily",
+		"other-setting":          "val",
+	})
+	defer m2.Close()
+
+	settingsColl, settingsCloser := s.state.db().GetRawCollection(settingsC)
+	defer settingsCloser()
+	// A document that isn't a model with an accidental setting
+	err := settingsColl.Insert(
+		bson.M{
+			"_id": "not-a-model",
+			"settings": bson.M{
+				"container-image-stream":                   "released",
+				"other-setting":                            "val",
+				unescapeReplacer.Replace("dotted.setting"): "value",
+				unescapeReplacer.Replace("dollar$setting"): "value",
+			},
+		},
+	)
+	c.Assert(err, jc.ErrorIsNil)
+	// A document that doesn't have the setting
+	err = settingsColl.Insert(
+		bson.M{
+			"_id": "applicationsetting",
+			"settings": bson.M{
+				"other-setting": "val",
+			},
+		},
+	)
+	c.Assert(err, jc.ErrorIsNil)
+	// A document that has the setting, but it shouldn't be touched because it is a custom value.
+	err = settingsColl.Insert(
+		bson.M{
+			"_id": "otherapplication",
+			"settings": bson.M{
+				"container-image-stream": "custom",
+				"other-setting":          "val",
+			},
+		},
+	)
+	c.Assert(err, jc.ErrorIsNil)
+
+	// Read all the settings from the database, and change the 'not-a-model'
+	// content which has 'container-image-stream' that needs to be removed.
+	// documents we think we're changing, and the rest should go through
+	// unchanged.
+	var rawSettings bson.M
+	iter := settingsColl.Find(nil).Sort("_id").Iter()
+	defer iter.Close()
+
+	expectedSettings := []bson.M{}
+
+	for iter.Next(&rawSettings) {
+		expSettings := copyMap(rawSettings, nil)
+		delete(expSettings, "txn-queue")
+		delete(expSettings, "txn-revno")
+		delete(expSettings, "version")
+		id, ok := expSettings["_id"]
+		idStr, ok := id.(string)
+		c.Assert(ok, jc.IsTrue)
+		c.Assert(idStr, gc.Not(gc.Equals), "")
+		if idStr == "not-a-model" {
+			raw, ok := expSettings["settings"]
+			c.Assert(ok, jc.IsTrue)
+			settings, ok := raw.(bson.M)
+			c.Assert(ok, jc.IsTrue)
+			delete(settings, "container-image-stream")
+		}
+		expectedSettings = append(expectedSettings, expSettings)
+	}
+	c.Assert(iter.Close(), jc.ErrorIsNil)
+
+	// Note that the assertions on this are very hard to read for humans,
+	// because Settings documents have a ton of keys and nested sub documents.
+	// But it is a more accurate depiction of what is in that table.
+	s.assertUpgradedData(c, RemoveContainerImageStreamFromNonModelSettings,
+		expectUpgradedData{settingsColl, expectedSettings},
+	)
+}
