@@ -48,7 +48,26 @@ func (st *State) userPermission(objectGlobalKey, subjectGlobalKey string) (*user
 	id := permissionID(objectGlobalKey, subjectGlobalKey)
 	err := permissions.FindId(id).One(&result.doc)
 	if err == mgo.ErrNotFound {
-		return nil, errors.NotFoundf("user permissions for user %q", id)
+		return nil, errors.NotFoundf("user permission for %q on %q", subjectGlobalKey, objectGlobalKey)
+	}
+	return result, nil
+}
+
+// usersPermissions returns all permissions for a given object.
+func (st *State) usersPermissions(objectGlobalKey string) ([]*userPermission, error) {
+	permissions, closer := st.db().GetCollection(permissionsC)
+	defer closer()
+
+	var matchingPermissions []permissionDoc
+	findExpr := fmt.Sprintf("^%s#.*$", objectGlobalKey)
+	if err := permissions.Find(
+		bson.D{{"_id", bson.D{{"$regex", findExpr}}}},
+	).All(&matchingPermissions); err != nil {
+		return nil, err
+	}
+	var result []*userPermission
+	for _, pDoc := range matchingPermissions {
+		result = append(result, &userPermission{doc: pDoc})
 	}
 	return result, nil
 }
@@ -63,29 +82,11 @@ func (st *State) controllerUserPermission(objectGlobalKey, subjectGlobalKey stri
 	id := permissionID(objectGlobalKey, subjectGlobalKey)
 	err := permissions.FindId(id).One(&result.doc)
 	if err == mgo.ErrNotFound {
-		return nil, errors.NotFoundf("user permissions for user %q", id)
+		return nil, errors.NotFoundf("user permission for %q on %q", subjectGlobalKey, objectGlobalKey)
 	}
 	return result, nil
 }
 
-// isReadOnly returns whether or not the user has write access or only
-// read access to the model.
-func (p *userPermission) isReadOnly() bool {
-	return stringToAccess(p.doc.Access) == permission.NoAccess || stringToAccess(p.doc.Access) == permission.ReadAccess
-}
-
-// isAdmin is a convenience method that
-// returns whether or not the user has permission.AdminAccess.
-func (p *userPermission) isAdmin() bool {
-	return stringToAccess(p.doc.Access) == permission.AdminAccess
-}
-
-// isReadWrite is a convenience method that
-// returns whether or not the user has permission.WriteAccess.
-func (p *userPermission) isReadWrite() bool {
-	return stringToAccess(p.doc.Access) == permission.WriteAccess
-}
-
 func (p *userPermission) access() permission.Access {
 	return stringToAccess(p.doc.Access)
 }
