@@ -46,6 +46,13 @@ type Volume interface {
 	// if it has not already been provisioned. Params returns true if the
 	// returned parameters are usable for provisioning, otherwise false.
 	Params() (VolumeParams, bool)
+
+	// Detachable reports whether or not the volume is detachable.
+	Detachable() bool
+
+	// Releasing reports whether or not the volume is to be released
+	// from the model when it is Dying/Dead.
+	Releasing() bool
 }
 
 // VolumeAttachment describes an attachment of a volume to a machine.
@@ -72,7 +79,7 @@ type VolumeAttachment interface {
 }
 
 type volume struct {
-	st  *State
+	im  *IAASModel
 	doc volumeDoc
 }
 
@@ -86,6 +93,7 @@ type volumeDoc struct {
 	Name            string        `bson:"name"`
 	ModelUUID       string        `bson:"model-uuid"`
 	Life            Life          `bson:"life"`
+	Releasing       bool          `bson:"releasing,omitempty"`
 	StorageId       string        `bson:"storageid,omitempty"`
 	AttachmentCount int           `bson:"attachmentcount"`
 	Info            *VolumeInfo   `bson:"info,omitempty"`
@@ -116,6 +124,11 @@ type VolumeParams struct {
 	// that the volume is to be assigned to.
 	storage names.StorageTag
 
+	// volumeInfo, if non-empty, is the information for an already
+	// provisioned volume. This is only set when creating a volume
+	// entity for an existing volume.
+	volumeInfo *VolumeInfo
+
 	Pool string `bson:"pool"`
 	Size uint64 `bson:"size"`
 }
@@ -145,7 +158,7 @@ type VolumeAttachmentParams struct {
 }
 
 // validate validates the contents of the volume document.
-func (v *volume) validate() error {
+func (v *volumeDoc) validate() error {
 	return nil
 }
 
@@ -194,14 +207,19 @@ func (v *volume) Params() (VolumeParams, bool) {
 	return *v.doc.Params, true
 }
 
+// Releasing is required to imeplement Volume.
+func (v *volume) Releasing() bool {
+	return v.doc.Releasing
+}
+
 // Status is required to implement StatusGetter.
 func (v *volume) Status() (status.StatusInfo, error) {
-	return v.st.VolumeStatus(v.VolumeTag())
+	return v.im.VolumeStatus(v.VolumeTag())
 }
 
 // SetStatus is required to implement StatusSetter.
 func (v *volume) SetStatus(volumeStatus status.StatusInfo) error {
-	return v.st.SetVolumeStatus(v.VolumeTag(), volumeStatus.Status, volumeStatus.Message, volumeStatus.Data, volumeStatus.Since)
+	return v.im.SetVolumeStatus(v.VolumeTag(), volumeStatus.Status, volumeStatus.Message, volumeStatus.Data, volumeStatus.Since)
 }
 
 // Volume is required to implement VolumeAttachment.
@@ -236,13 +254,58 @@ func (v *volumeAttachment) Params() (VolumeAttachmentParams, bool) {
 }
 
 // Volume returns the Volume with the specified name.
-func (st *State) Volume(tag names.VolumeTag) (Volume, error) {
-	v, err := st.volumeByTag(tag)
+func (im *IAASModel) Volume(tag names.VolumeTag) (Volume, error) {
+	v, err := im.volumeByTag(tag)
 	return v, err
 }
 
-func (st *State) volumes(query interface{}) ([]*volume, error) {
-	coll, cleanup := st.db().GetCollection(volumesC)
+func (im *IAASModel) volumes(query interface{}) ([]*volume, error) {
+	docs, err := getVolumeDocs(im.mb.db(), query)
+	if err != nil {
+		return nil, errors.Trace(err)
+	}
+	volumes := make([]*volume, len(docs))
+	for i := range docs {
+		volumes[i] = &volume{im, docs[i]}
+	}
+	return volumes, nil
+}
+
+func (im *IAASModel) volumeByTag(tag names.VolumeTag) (*volume, error) {
+	doc, err := getVolumeDocByTag(im.mb.db(), tag)
+	if err != nil {
+		return nil, errors.Trace(err)
+	}
+	return &volume{im, doc}, nil
+}
+
+func (im *IAASModel) volume(query bson.D, description string) (*volume, error) {
+	doc, err := getVolumeDoc(im.mb.db(), query, description)
+	if err != nil {
+		return nil, errors.Trace(err)
+	}
+	return &volume{im, doc}, nil
+}
+
+func getVolumeDocByTag(db Database, tag names.VolumeTag) (volumeDoc, error) {
+	return getVolumeDoc(db, bson.D{{"_id", tag.Id()}}, fmt.Sprintf("volume %q", tag.Id()))
+}
+
+func getVolumeDoc(db Database, query bson.D, description string) (volumeDoc, error) {
+	docs, err := getVolumeDocs(db, query)
+	if err != nil {
+		return volumeDoc{}, errors.Trace(err)
+	}
+	if len(docs) == 0 {
+		return volumeDoc{}, errors.NotFoundf("%s", description)
+	} else if len(docs) != 1 {
+		return volumeDoc{}, errors.Errorf("expected 1 volume, got %d", len(docs))
+	}
+	return docs[0], nil
+}
+
+func getVolumeDocs(db Database, query interface{}) ([]volumeDoc, error) {
+	coll, cleanup := db.GetCollection(volumesC)
 	defer cleanup()
 
 	var docs []volumeDoc
@@ -250,32 +313,12 @@ func (st *State) volumes(query interface{}) ([]*volume, error) {
 	if err != nil {
 		return nil, errors.Annotate(err, "querying volumes")
 	}
-	volumes := make([]*volume, len(docs))
-	for i := range docs {
-		volume := &volume{st, docs[i]}
-		if err := volume.validate(); err != nil {
+	for _, doc := range docs {
+		if err := doc.validate(); err != nil {
 			return nil, errors.Annotate(err, "validating volume")
 		}
-		volumes[i] = volume
 	}
-	return volumes, nil
-}
-
-func (st *State) volume(query bson.D, description string) (*volume, error) {
-	volumes, err := st.volumes(query)
-	if err != nil {
-		return nil, errors.Trace(err)
-	}
-	if len(volumes) == 0 {
-		return nil, errors.NotFoundf("%s", description)
-	} else if len(volumes) != 1 {
-		return nil, errors.Errorf("expected 1 volume, got %d", len(volumes))
-	}
-	return volumes[0], nil
-}
-
-func (st *State) volumeByTag(tag names.VolumeTag) (*volume, error) {
-	return st.volume(bson.D{{"_id", tag.Id()}}, fmt.Sprintf("volume %q", tag.Id()))
+	return docs, nil
 }
 
 func volumesToInterfaces(volumes []*volume) []Volume {
@@ -286,8 +329,8 @@ func volumesToInterfaces(volumes []*volume) []Volume {
 	return result
 }
 
-func (st *State) storageInstanceVolume(tag names.StorageTag) (*volume, error) {
-	return st.volume(
+func (im *IAASModel) storageInstanceVolume(tag names.StorageTag) (*volume, error) {
+	return im.volume(
 		bson.D{{"storageid", tag.Id()}},
 		fmt.Sprintf("volume for storage instance %q", tag.Id()),
 	)
@@ -295,15 +338,15 @@ func (st *State) storageInstanceVolume(tag names.StorageTag) (*volume, error) {
 
 // StorageInstanceVolume returns the Volume assigned to the specified
 // storage instance.
-func (st *State) StorageInstanceVolume(tag names.StorageTag) (Volume, error) {
-	v, err := st.storageInstanceVolume(tag)
+func (im *IAASModel) StorageInstanceVolume(tag names.StorageTag) (Volume, error) {
+	v, err := im.storageInstanceVolume(tag)
 	return v, err
 }
 
 // VolumeAttachment returns the VolumeAttachment corresponding to
 // the specified volume and machine.
-func (st *State) VolumeAttachment(machine names.MachineTag, volume names.VolumeTag) (VolumeAttachment, error) {
-	coll, cleanup := st.db().GetCollection(volumeAttachmentsC)
+func (im *IAASModel) VolumeAttachment(machine names.MachineTag, volume names.VolumeTag) (VolumeAttachment, error) {
+	coll, cleanup := im.mb.db().GetCollection(volumeAttachmentsC)
 	defer cleanup()
 
 	var att volumeAttachment
@@ -318,8 +361,8 @@ func (st *State) VolumeAttachment(machine names.MachineTag, volume names.VolumeT
 
 // MachineVolumeAttachments returns all of the VolumeAttachments for the
 // specified machine.
-func (st *State) MachineVolumeAttachments(machine names.MachineTag) ([]VolumeAttachment, error) {
-	attachments, err := st.volumeAttachments(bson.D{{"machineid", machine.Id()}})
+func (im *IAASModel) MachineVolumeAttachments(machine names.MachineTag) ([]VolumeAttachment, error) {
+	attachments, err := im.volumeAttachments(bson.D{{"machineid", machine.Id()}})
 	if err != nil {
 		return nil, errors.Annotatef(err, "getting volume attachments for machine %q", machine.Id())
 	}
@@ -328,16 +371,16 @@ func (st *State) MachineVolumeAttachments(machine names.MachineTag) ([]VolumeAtt
 
 // VolumeAttachments returns all of the VolumeAttachments for the specified
 // volume.
-func (st *State) VolumeAttachments(volume names.VolumeTag) ([]VolumeAttachment, error) {
-	attachments, err := st.volumeAttachments(bson.D{{"volumeid", volume.Id()}})
+func (im *IAASModel) VolumeAttachments(volume names.VolumeTag) ([]VolumeAttachment, error) {
+	attachments, err := im.volumeAttachments(bson.D{{"volumeid", volume.Id()}})
 	if err != nil {
 		return nil, errors.Annotatef(err, "getting volume attachments for volume %q", volume.Id())
 	}
 	return attachments, nil
 }
 
-func (st *State) volumeAttachments(query bson.D) ([]VolumeAttachment, error) {
-	coll, cleanup := st.db().GetCollection(volumeAttachmentsC)
+func (im *IAASModel) volumeAttachments(query bson.D) ([]VolumeAttachment, error) {
+	coll, cleanup := im.mb.db().GetCollection(volumeAttachmentsC)
 	defer cleanup()
 
 	var docs []volumeAttachmentDoc
@@ -366,7 +409,7 @@ func IsContainsFilesystem(err error) bool {
 // removeMachineVolumesOps returns txn.Ops to remove non-persistent volumes
 // bound or attached to the specified machine. This is used when the given
 // machine is being removed from state.
-func (st *State) removeMachineVolumesOps(m *Machine) ([]txn.Op, error) {
+func (im *IAASModel) removeMachineVolumesOps(m *Machine) ([]txn.Op, error) {
 	// A machine cannot transition to Dead if it has any detachable storage
 	// attached, so any attachments are for machine-bound storage.
 	//
@@ -379,7 +422,7 @@ func (st *State) removeMachineVolumesOps(m *Machine) ([]txn.Op, error) {
 	// Therefore, there may be volumes that are bound, but not attached,
 	// to the machine.
 
-	machineVolumes, err := st.volumes(bson.D{{"machineid", m.Id()}})
+	machineVolumes, err := im.volumes(bson.D{{"machineid", m.Id()}})
 	if err != nil {
 		return nil, errors.Trace(err)
 	}
@@ -393,7 +436,6 @@ func (st *State) removeMachineVolumesOps(m *Machine) ([]txn.Op, error) {
 		})
 	}
 	for _, v := range machineVolumes {
-		volumeId := v.Tag().Id()
 		if v.doc.StorageId != "" {
 			// The volume is assigned to a storage instance;
 			// make sure we also remove the storage instance.
@@ -410,39 +452,42 @@ func (st *State) removeMachineVolumesOps(m *Machine) ([]txn.Op, error) {
 				},
 			)
 		}
-		ops = append(ops,
-			txn.Op{
-				C:      volumesC,
-				Id:     volumeId,
-				Assert: txn.DocExists,
-				Remove: true,
-			},
-			removeModelVolumeRefOp(st, volumeId),
-		)
+		ops = append(ops, im.removeVolumeOps(v.VolumeTag())...)
 	}
 	return ops, nil
 }
 
 // isDetachableVolumeTag reports whether or not the volume with the specified
 // tag is detachable.
-func isDetachableVolumeTag(st *State, tag names.VolumeTag) (bool, error) {
-	volume, err := st.volumeByTag(tag)
+func isDetachableVolumeTag(db Database, tag names.VolumeTag) (bool, error) {
+	doc, err := getVolumeDocByTag(db, tag)
 	if err != nil {
 		return false, errors.Trace(err)
 	}
-	return volume.detachable(), nil
+	return detachableVolumeDoc(&doc), nil
 }
 
-// detachable reports whether or not the volume is detachable.
-func (v *volume) detachable() bool {
-	return v.doc.MachineId == ""
+// Detachable reports whether or not the volume is detachable.
+func (v *volume) Detachable() bool {
+	return detachableVolumeDoc(&v.doc)
+}
+
+func (v *volume) pool() string {
+	if v.doc.Info != nil {
+		return v.doc.Info.Pool
+	}
+	return v.doc.Params.Pool
+}
+
+func detachableVolumeDoc(doc *volumeDoc) bool {
+	return doc.MachineId == ""
 }
 
 // isDetachableVolumePool reports whether or not the given storage
 // pool will create a volume that is not inherently bound to a machine,
 // and therefore can be detached.
-func isDetachableVolumePool(st *State, pool string) (bool, error) {
-	_, provider, err := poolStorageProvider(st, pool)
+func isDetachableVolumePool(im *IAASModel, pool string) (bool, error) {
+	_, provider, err := poolStorageProvider(im, pool)
 	if err != nil {
 		return false, errors.Trace(err)
 	}
@@ -472,24 +517,24 @@ func isDetachableVolumePool(st *State, pool string) (bool, error) {
 // IsContainsFilesystem error if the volume contains an attached filesystem; the
 // filesystem attachment must be removed first. DetachVolume will fail for
 // inherently machine-bound volumes.
-func (st *State) DetachVolume(machine names.MachineTag, volume names.VolumeTag) (err error) {
+func (im *IAASModel) DetachVolume(machine names.MachineTag, volume names.VolumeTag) (err error) {
 	defer errors.DeferredAnnotatef(&err, "detaching volume %s from machine %s", volume.Id(), machine.Id())
 	// If the volume is backing a filesystem, the volume cannot be detached
 	// until the filesystem has been detached.
-	if _, err := st.volumeFilesystemAttachment(machine, volume); err == nil {
+	if _, err := im.volumeFilesystemAttachment(machine, volume); err == nil {
 		return &errContainsFilesystem{errors.New("volume contains attached filesystem")}
 	} else if !errors.IsNotFound(err) {
 		return errors.Trace(err)
 	}
 	buildTxn := func(attempt int) ([]txn.Op, error) {
-		va, err := st.VolumeAttachment(machine, volume)
+		va, err := im.VolumeAttachment(machine, volume)
 		if err != nil {
 			return nil, errors.Trace(err)
 		}
 		if va.Life() != Alive {
 			return nil, jujutxn.ErrNoOperations
 		}
-		detachable, err := isDetachableVolumeTag(st, volume)
+		detachable, err := isDetachableVolumeTag(im.mb.db(), volume)
 		if err != nil {
 			return nil, errors.Trace(err)
 		}
@@ -498,15 +543,15 @@ func (st *State) DetachVolume(machine names.MachineTag, volume names.VolumeTag)
 		}
 		return detachVolumeOps(machine, volume), nil
 	}
-	return st.run(buildTxn)
+	return im.mb.db().Run(buildTxn)
 }
 
-func (st *State) volumeFilesystemAttachment(machine names.MachineTag, volume names.VolumeTag) (FilesystemAttachment, error) {
-	filesystem, err := st.VolumeFilesystem(volume)
+func (im *IAASModel) volumeFilesystemAttachment(machine names.MachineTag, volume names.VolumeTag) (FilesystemAttachment, error) {
+	filesystem, err := im.VolumeFilesystem(volume)
 	if err != nil {
 		return nil, errors.Trace(err)
 	}
-	return st.FilesystemAttachment(machine, filesystem.FilesystemTag())
+	return im.FilesystemAttachment(machine, filesystem.FilesystemTag())
 }
 
 func detachVolumeOps(m names.MachineTag, v names.VolumeTag) []txn.Op {
@@ -520,10 +565,10 @@ func detachVolumeOps(m names.MachineTag, v names.VolumeTag) []txn.Op {
 
 // RemoveVolumeAttachment removes the volume attachment from state.
 // RemoveVolumeAttachment will fail if the attachment is not Dying.
-func (st *State) RemoveVolumeAttachment(machine names.MachineTag, volume names.VolumeTag) (err error) {
+func (im *IAASModel) RemoveVolumeAttachment(machine names.MachineTag, volume names.VolumeTag) (err error) {
 	defer errors.DeferredAnnotatef(&err, "removing attachment of volume %s from machine %s", volume.Id(), machine.Id())
 	buildTxn := func(attempt int) ([]txn.Op, error) {
-		attachment, err := st.VolumeAttachment(machine, volume)
+		attachment, err := im.VolumeAttachment(machine, volume)
 		if errors.IsNotFound(err) && attempt > 0 {
 			// We only ignore IsNotFound on attempts after the
 			// first, since we expect the volume attachment to
@@ -536,13 +581,13 @@ func (st *State) RemoveVolumeAttachment(machine names.MachineTag, volume names.V
 		if attachment.Life() != Dying {
 			return nil, errors.New("volume attachment is not dying")
 		}
-		v, err := st.volumeByTag(volume)
+		v, err := im.volumeByTag(volume)
 		if err != nil {
 			return nil, errors.Trace(err)
 		}
 		return removeVolumeAttachmentOps(machine, v), nil
 	}
-	return st.run(buildTxn)
+	return im.mb.db().Run(buildTxn)
 }
 
 func removeVolumeAttachmentOps(m names.MachineTag, v *volume) []txn.Op {
@@ -567,11 +612,7 @@ func removeVolumeAttachmentOps(m names.MachineTag, v *volume) []txn.Op {
 // count for a given machine storage entity (volume or filesystem), given its
 // current attachment count and lifecycle state. If the attachment count goes
 // to zero, then the entity should become Dead.
-func machineStorageDecrefOp(
-	collection, id string,
-	attachmentCount int, life Life,
-	machine names.MachineTag,
-) txn.Op {
+func machineStorageDecrefOp(collection, id string, attachmentCount int, life Life, machine names.MachineTag) txn.Op {
 	op := txn.Op{
 		C:  collection,
 		Id: id,
@@ -624,15 +665,15 @@ func machineStorageDecrefOp(
 // destroyed and removed from state at some point in the future. DestroyVolume
 // will fail with an IsContainsFilesystem error if the volume contains a
 // filesystem; the filesystem must be fully removed first.
-func (st *State) DestroyVolume(tag names.VolumeTag) (err error) {
+func (im *IAASModel) DestroyVolume(tag names.VolumeTag) (err error) {
 	defer errors.DeferredAnnotatef(&err, "destroying volume %s", tag.Id())
-	if _, err := st.VolumeFilesystem(tag); err == nil {
+	if _, err := im.VolumeFilesystem(tag); err == nil {
 		return &errContainsFilesystem{errors.New("volume contains filesystem")}
 	} else if !errors.IsNotFound(err) {
 		return errors.Trace(err)
 	}
 	buildTxn := func(attempt int) ([]txn.Op, error) {
-		volume, err := st.volumeByTag(tag)
+		volume, err := im.volumeByTag(tag)
 		if errors.IsNotFound(err) && attempt > 0 {
 			// On the first attempt, we expect it to exist.
 			return nil, jujutxn.ErrNoOperations
@@ -652,35 +693,41 @@ func (st *State) DestroyVolume(tag names.VolumeTag) (err error) {
 			{{"storageid", ""}},
 			{{"storageid", bson.D{{"$exists", false}}}},
 		}}}
-		return destroyVolumeOps(st, volume, hasNoStorageAssignment)
+		return destroyVolumeOps(im, volume, false, hasNoStorageAssignment)
 	}
-	return st.run(buildTxn)
+	return im.mb.db().Run(buildTxn)
 }
 
-func destroyVolumeOps(st *State, v *volume, extraAssert bson.D) ([]txn.Op, error) {
+func destroyVolumeOps(im *IAASModel, v *volume, release bool, extraAssert bson.D) ([]txn.Op, error) {
 	baseAssert := append(isAliveDoc, extraAssert...)
+	setFields := bson.D{}
+	if release {
+		setFields = append(setFields, bson.DocElem{"releasing", true})
+	}
 	if v.doc.AttachmentCount == 0 {
 		hasNoAttachments := bson.D{{"attachmentcount", 0}}
+		setFields = append(setFields, bson.DocElem{"life", Dead})
 		return []txn.Op{{
 			C:      volumesC,
 			Id:     v.doc.Name,
 			Assert: append(hasNoAttachments, baseAssert...),
-			Update: bson.D{{"$set", bson.D{{"life", Dead}}}},
+			Update: bson.D{{"$set", setFields}},
 		}}, nil
 	}
 	hasAttachments := bson.D{{"attachmentcount", bson.D{{"$gt", 0}}}}
+	setFields = append(setFields, bson.DocElem{"life", Dying})
 	ops := []txn.Op{{
 		C:      volumesC,
 		Id:     v.doc.Name,
 		Assert: append(hasAttachments, baseAssert...),
-		Update: bson.D{{"$set", bson.D{{"life", Dying}}}},
+		Update: bson.D{{"$set", setFields}},
 	}}
-	if !v.detachable() {
+	if !v.Detachable() {
 		// This volume cannot be directly detached, so we do not
 		// issue a cleanup. Since there can (should!) be only one
 		// attachment for the lifetime of the filesystem, we can
 		// look it up and destroy it along with the filesystem.
-		attachments, err := st.VolumeAttachments(v.VolumeTag())
+		attachments, err := im.VolumeAttachments(v.VolumeTag())
 		if err != nil {
 			return nil, errors.Trace(err)
 		}
@@ -706,10 +753,10 @@ func destroyVolumeOps(st *State, v *volume, extraAssert bson.D) ([]txn.Op, error
 
 // RemoveVolume removes the volume from state. RemoveVolume will fail if
 // the volume is not Dead, which implies that it still has attachments.
-func (st *State) RemoveVolume(tag names.VolumeTag) (err error) {
+func (im *IAASModel) RemoveVolume(tag names.VolumeTag) (err error) {
 	defer errors.DeferredAnnotatef(&err, "removing volume %s", tag.Id())
 	buildTxn := func(attempt int) ([]txn.Op, error) {
-		volume, err := st.Volume(tag)
+		volume, err := im.Volume(tag)
 		if errors.IsNotFound(err) {
 			return nil, jujutxn.ErrNoOperations
 		} else if err != nil {
@@ -718,26 +765,30 @@ func (st *State) RemoveVolume(tag names.VolumeTag) (err error) {
 		if volume.Life() != Dead {
 			return nil, errors.New("volume is not dead")
 		}
-		return []txn.Op{
-			{
-				C:      volumesC,
-				Id:     tag.Id(),
-				Assert: txn.DocExists,
-				Remove: true,
-			},
-			removeModelVolumeRefOp(st, tag.Id()),
-			removeStatusOp(st, volumeGlobalKey(tag.Id())),
-		}, nil
+		return im.removeVolumeOps(tag), nil
+	}
+	return im.mb.db().Run(buildTxn)
+}
+
+func (im *IAASModel) removeVolumeOps(tag names.VolumeTag) []txn.Op {
+	return []txn.Op{
+		{
+			C:      volumesC,
+			Id:     tag.Id(),
+			Assert: txn.DocExists,
+			Remove: true,
+		},
+		removeModelVolumeRefOp(im.mb, tag.Id()),
+		removeStatusOp(im.mb, volumeGlobalKey(tag.Id())),
 	}
-	return st.run(buildTxn)
 }
 
 // newVolumeName returns a unique volume name.
 // If the machine ID supplied is non-empty, the
 // volume ID will incorporate it as the volume's
 // machine scope.
-func newVolumeName(st *State, machineId string) (string, error) {
-	seq, err := st.sequence("volume")
+func newVolumeName(mb modelBackend, machineId string) (string, error) {
+	seq, err := sequence(mb, "volume")
 	if err != nil {
 		return "", errors.Trace(err)
 	}
@@ -752,57 +803,65 @@ func newVolumeName(st *State, machineId string) (string, error) {
 // parameters. If the supplied machine ID is non-empty, and the storage
 // provider is machine-scoped, then the volume will be scoped to that
 // machine.
-func (st *State) addVolumeOps(params VolumeParams, machineId string) ([]txn.Op, names.VolumeTag, error) {
-	params, err := st.volumeParamsWithDefaults(params, machineId)
+func (im *IAASModel) addVolumeOps(params VolumeParams, machineId string) ([]txn.Op, names.VolumeTag, error) {
+	params, err := im.volumeParamsWithDefaults(params, machineId)
 	if err != nil {
 		return nil, names.VolumeTag{}, errors.Trace(err)
 	}
-	detachable, err := isDetachableVolumePool(st, params.Pool)
+	detachable, err := isDetachableVolumePool(im, params.Pool)
 	if err != nil {
 		return nil, names.VolumeTag{}, errors.Trace(err)
 	}
 	origMachineId := machineId
-	machineId, err = st.validateVolumeParams(params, machineId)
+	machineId, err = im.validateVolumeParams(params, machineId)
 	if err != nil {
 		return nil, names.VolumeTag{}, errors.Annotate(err, "validating volume params")
 	}
-	name, err := newVolumeName(st, machineId)
+	name, err := newVolumeName(im.mb, machineId)
 	if err != nil {
 		return nil, names.VolumeTag{}, errors.Annotate(err, "cannot generate volume name")
 	}
-	status := statusDoc{
+	statusDoc := statusDoc{
 		Status:  status.Pending,
-		Updated: st.clock.Now().UnixNano(),
+		Updated: im.mb.clock().Now().UnixNano(),
 	}
 	doc := volumeDoc{
 		Name:      name,
 		StorageId: params.storage.Id(),
-		Params:    &params,
-		// Every volume is created with one attachment.
-		AttachmentCount: 1,
+	}
+	if params.volumeInfo != nil {
+		// We're importing an already provisioned volume into the
+		// model. Set provisioned info rather than params, and set
+		// the status to "detached".
+		statusDoc.Status = status.Detached
+		doc.Info = params.volumeInfo
+	} else {
+		// Every new volume is created with one attachment.
+		doc.Params = &params
+		doc.AttachmentCount = 1
 	}
 	if !detachable {
 		doc.MachineId = origMachineId
 	}
-	return st.newVolumeOps(doc, status), names.NewVolumeTag(name), nil
+	return im.newVolumeOps(doc, statusDoc), names.NewVolumeTag(name), nil
 }
 
-func (st *State) newVolumeOps(doc volumeDoc, status statusDoc) []txn.Op {
+func (im *IAASModel) newVolumeOps(doc volumeDoc, status statusDoc) []txn.Op {
 	return []txn.Op{
-		createStatusOp(st, volumeGlobalKey(doc.Name), status),
+		createStatusOp(im.mb, volumeGlobalKey(doc.Name), status),
 		{
 			C:      volumesC,
 			Id:     doc.Name,
 			Assert: txn.DocMissing,
 			Insert: &doc,
 		},
-		addModelVolumeRefOp(st, doc.Name),
+		addModelVolumeRefOp(im.mb, doc.Name),
 	}
 }
 
-func (st *State) volumeParamsWithDefaults(params VolumeParams, machineId string) (VolumeParams, error) {
+func (im *IAASModel) volumeParamsWithDefaults(params VolumeParams, machineId string) (VolumeParams, error) {
 	if params.Pool == "" {
-		modelConfig, err := st.ModelConfig()
+		modelConfig, err := im.ModelConfig()
 		if err != nil {
 			return VolumeParams{}, errors.Trace(err)
 		}
@@ -822,8 +881,8 @@ func (st *State) volumeParamsWithDefaults(params VolumeParams, machineId string)
 
 // validateVolumeParams validates the volume parameters, and returns the
 // machine ID to use as the scope in the volume tag.
-func (st *State) validateVolumeParams(params VolumeParams, machineId string) (maybeMachineId string, _ error) {
-	if err := validateStoragePool(st, params.Pool, storage.StorageKindBlock, &machineId); err != nil {
+func (im *IAASModel) validateVolumeParams(params VolumeParams, machineId string) (maybeMachineId string, _ error) {
+	if err := validateStoragePool(im, params.Pool, storage.StorageKindBlock, &machineId); err != nil {
 		return "", err
 	}
 	if params.Size == 0 {
@@ -851,8 +910,9 @@ func ParseVolumeAttachmentId(id string) (names.MachineTag, names.VolumeTag, erro
 }
 
 type volumeAttachmentTemplate struct {
-	tag    names.VolumeTag
-	params VolumeAttachmentParams
+	tag      names.VolumeTag
+	params   VolumeAttachmentParams
+	existing bool
 }
 
 // createMachineVolumeAttachmentInfo creates volume attachments
@@ -873,6 +933,14 @@ func createMachineVolumeAttachmentsOps(machineId string, attachments []volumeAtt
 				Params:  &paramsCopy,
 			},
 		}
+		if attachment.existing {
+			ops = append(ops, txn.Op{
+				C:      volumesC,
+				Id:     attachment.tag.Id(),
+				Assert: txn.DocExists,
+				Update: bson.D{{"$inc", bson.D{{"attachmentcount", 1}}}},
+			})
+		}
 	}
 	return ops
 }
@@ -881,15 +949,11 @@ func createMachineVolumeAttachmentsOps(machineId string, attachments []volumeAtt
 // info for the specified machine. Each volume attachment info
 // structure is keyed by the name of the volume it corresponds
 // to.
-func setMachineVolumeAttachmentInfo(
-	st *State,
-	machineId string,
-	attachments map[names.VolumeTag]VolumeAttachmentInfo,
-) (err error) {
+func setMachineVolumeAttachmentInfo(im *IAASModel, machineId string, attachments map[names.VolumeTag]VolumeAttachmentInfo) (err error) {
 	defer errors.DeferredAnnotatef(&err, "cannot set volume attachment info for machine %s", machineId)
 	machineTag := names.NewMachineTag(machineId)
 	for volumeTag, info := range attachments {
-		if err := st.setVolumeAttachmentInfo(machineTag, volumeTag, info); err != nil {
+		if err := im.setVolumeAttachmentInfo(machineTag, volumeTag, info); err != nil {
 			return errors.Annotatef(err, "setting attachment info for volume %s", volumeTag.Id())
 		}
 	}
@@ -898,9 +962,9 @@ func setMachineVolumeAttachmentInfo(
 
 // SetVolumeAttachmentInfo sets the VolumeAttachmentInfo for the specified
 // volume attachment.
-func (st *State) SetVolumeAttachmentInfo(machineTag names.MachineTag, volumeTag names.VolumeTag, info VolumeAttachmentInfo) (err error) {
+func (im *IAASModel) SetVolumeAttachmentInfo(machineTag names.MachineTag, volumeTag names.VolumeTag, info VolumeAttachmentInfo) (err error) {
 	defer errors.DeferredAnnotatef(&err, "cannot set info for volume attachment %s:%s", volumeTag.Id(), machineTag.Id())
-	v, err := st.Volume(volumeTag)
+	v, err := im.Volume(volumeTag)
 	if err != nil {
 		return errors.Trace(err)
 	}
@@ -911,23 +975,19 @@ func (st *State) SetVolumeAttachmentInfo(machineTag names.MachineTag, volumeTag
 		return errors.Trace(err)
 	}
 	// Also ensure the machine is provisioned.
-	m, err := st.Machine(machineTag.Id())
+	m, err := im.st.Machine(machineTag.Id())
 	if err != nil {
 		return errors.Trace(err)
 	}
 	if _, err := m.InstanceId(); err != nil {
 		return errors.Trace(err)
 	}
-	return st.setVolumeAttachmentInfo(machineTag, volumeTag, info)
+	return im.setVolumeAttachmentInfo(machineTag, volumeTag, info)
 }
 
-func (st *State) setVolumeAttachmentInfo(
-	machineTag names.MachineTag,
-	volumeTag names.VolumeTag,
-	info VolumeAttachmentInfo,
-) error {
+func (im *IAASModel) setVolumeAttachmentInfo(machineTag names.MachineTag, volumeTag names.VolumeTag, info VolumeAttachmentInfo) error {
 	buildTxn := func(attempt int) ([]txn.Op, error) {
-		va, err := st.VolumeAttachment(machineTag, volumeTag)
+		va, err := im.VolumeAttachment(machineTag, volumeTag)
 		if err != nil {
 			return nil, errors.Trace(err)
 		}
@@ -940,7 +1000,7 @@ func (st *State) setVolumeAttachmentInfo(
 		)
 		return ops, nil
 	}
-	return st.run(buildTxn)
+	return im.mb.db().Run(buildTxn)
 }
 
 func setVolumeAttachmentInfoOps(machine names.MachineTag, volume names.VolumeTag, info VolumeAttachmentInfo, unsetParams bool) []txn.Op {
@@ -964,9 +1024,9 @@ func setVolumeAttachmentInfoOps(machine names.MachineTag, volume names.VolumeTag
 // setProvisionedVolumeInfo sets the initial info for newly
 // provisioned volumes. If non-empty, machineId must be the
 // machine ID associated with the volumes.
-func setProvisionedVolumeInfo(st *State, volumes map[names.VolumeTag]VolumeInfo) error {
+func setProvisionedVolumeInfo(im *IAASModel, volumes map[names.VolumeTag]VolumeInfo) error {
 	for volumeTag, info := range volumes {
-		if err := st.SetVolumeInfo(volumeTag, info); err != nil {
+		if err := im.SetVolumeInfo(volumeTag, info); err != nil {
 			return errors.Trace(err)
 		}
 	}
@@ -974,7 +1034,7 @@ func setProvisionedVolumeInfo(st *State, volumes map[names.VolumeTag]VolumeInfo)
 }
 
 // SetVolumeInfo sets the VolumeInfo for the specified volume.
-func (st *State) SetVolumeInfo(tag names.VolumeTag, info VolumeInfo) (err error) {
+func (im *IAASModel) SetVolumeInfo(tag names.VolumeTag, info VolumeInfo) (err error) {
 	defer errors.DeferredAnnotatef(&err, "cannot set info for volume %q", tag.Id())
 	if info.VolumeId == "" {
 		return errors.New("volume ID not set")
@@ -982,7 +1042,7 @@ func (st *State) SetVolumeInfo(tag names.VolumeTag, info VolumeInfo) (err error)
 	// TODO(axw) we should reject info without VolumeId set; can't do this
 	// until the providers all set it correctly.
 	buildTxn := func(attempt int) ([]txn.Op, error) {
-		v, err := st.Volume(tag)
+		v, err := im.Volume(tag)
 		if err != nil {
 			return nil, errors.Trace(err)
 		}
@@ -1007,7 +1067,7 @@ func (st *State) SetVolumeInfo(tag names.VolumeTag, info VolumeInfo) (err error)
 		ops = append(ops, setVolumeInfoOps(tag, info, unsetParams)...)
 		return ops, nil
 	}
-	return st.run(buildTxn)
+	return im.mb.db().Run(buildTxn)
 }
 
 func validateVolumeInfoChange(newInfo, oldInfo VolumeInfo) error {
@@ -1045,8 +1105,8 @@ func setVolumeInfoOps(tag names.VolumeTag, info VolumeInfo, unsetParams bool) []
 }
 
 // AllVolumes returns all Volumes scoped to the model.
-func (st *State) AllVolumes() ([]Volume, error) {
-	volumes, err := st.volumes(nil)
+func (im *IAASModel) AllVolumes() ([]Volume, error) {
+	volumes, err := im.volumes(nil)
 	if err != nil {
 		return nil, errors.Annotate(err, "cannot get volumes")
 	}
@@ -1058,12 +1118,12 @@ func volumeGlobalKey(name string) string {
 }
 
 // VolumeStatus returns the status of the specified volume.
-func (st *State) VolumeStatus(tag names.VolumeTag) (status.StatusInfo, error) {
-	return getStatus(st.db(), volumeGlobalKey(tag.Id()), "volume")
+func (im *IAASModel) VolumeStatus(tag names.VolumeTag) (status.StatusInfo, error) {
+	return getStatus(im.mb.db(), volumeGlobalKey(tag.Id()), "volume")
 }
 
 // SetVolumeStatus sets the status of the specified volume.
-func (st *State) SetVolumeStatus(tag names.VolumeTag, volumeStatus status.Status, info string, data map[string]interface{}, updated *time.Time) error {
+func (im *IAASModel) SetVolumeStatus(tag names.VolumeTag, volumeStatus status.Status, info string, data map[string]interface{}, updated *time.Time) error {
 	switch volumeStatus {
 	case status.Attaching, status.Attached, status.Detaching, status.Detached, status.Destroying:
 	case status.Error:
@@ -1073,7 +1133,7 @@ func (st *State) SetVolumeStatus(tag names.VolumeTag, volumeStatus status.Status
 	case status.Pending:
 		// If a volume is not yet provisioned, we allow its status
 		// to be set back to pending (when a retry is to occur).
-		v, err := st.Volume(tag)
+		v, err := im.Volume(tag)
 		if err != nil {
 			return errors.Trace(err)
 		}
@@ -1085,12 +1145,12 @@ func (st *State) SetVolumeStatus(tag names.VolumeTag, volumeStatus status.Status
 	default:
 		return errors.Errorf("cannot set invalid status %q", volumeStatus)
 	}
-	return setStatus(st.db(), setStatusParams{
+	return setStatus(im.mb.db(), setStatusParams{
 		badge:     "volume",
 		globalKey: volumeGlobalKey(tag.Id()),
 		status:    volumeStatus,
 		message:   info,
 		rawData:   data,
-		updated:   timeOrNow(updated, st.clock),
+		updated:   timeOrNow(updated, im.mb.clock()),
 	})
 }
