@@ -14,6 +14,7 @@ import (
 	"github.com/juju/juju/provider/dummy"
 	"github.com/juju/juju/state"
 	"github.com/juju/juju/state/testing"
+	"github.com/juju/juju/storage"
 	"github.com/juju/juju/storage/poolmanager"
 	"github.com/juju/juju/storage/provider"
 )
@@ -44,10 +45,10 @@ func (s *VolumeStateSuite) assertMachineVolume(c *gc.C, unit *state.Unit) {
 	assignedMachineId, err := unit.AssignedMachineId()
 	c.Assert(err, jc.ErrorIsNil)
 
-	storageAttachments, err := s.State.UnitStorageAttachments(unit.UnitTag())
+	storageAttachments, err := s.IAASModel.UnitStorageAttachments(unit.UnitTag())
 	c.Assert(err, jc.ErrorIsNil)
 	c.Assert(storageAttachments, gc.HasLen, 1)
-	storageInstance, err := s.State.StorageInstance(storageAttachments[0].StorageInstance())
+	storageInstance, err := s.IAASModel.StorageInstance(storageAttachments[0].StorageInstance())
 	c.Assert(err, jc.ErrorIsNil)
 
 	volume := s.storageInstanceVolume(c, storageInstance.StorageTag())
@@ -62,7 +63,8 @@ func (s *VolumeStateSuite) assertMachineVolume(c *gc.C, unit *state.Unit) {
 
 	machine, err := s.State.Machine(assignedMachineId)
 	c.Assert(err, jc.ErrorIsNil)
-	volumeAttachments, err := s.State.MachineVolumeAttachments(machine.MachineTag())
+
+	volumeAttachments, err := s.IAASModel.MachineVolumeAttachments(machine.MachineTag())
 	c.Assert(err, jc.ErrorIsNil)
 	c.Assert(volumeAttachments, gc.HasLen, 1)
 	c.Assert(volumeAttachments[0].Volume(), gc.Equals, volume.VolumeTag())
@@ -72,10 +74,10 @@ func (s *VolumeStateSuite) assertMachineVolume(c *gc.C, unit *state.Unit) {
 	_, ok = volumeAttachments[0].Params()
 	c.Assert(ok, jc.IsTrue)
 
-	_, err = s.State.VolumeAttachment(machine.MachineTag(), volume.VolumeTag())
+	_, err = s.IAASModel.VolumeAttachment(machine.MachineTag(), volume.VolumeTag())
 	c.Assert(err, jc.ErrorIsNil)
 
-	assertMachineStorageRefs(c, s.State, machine.MachineTag())
+	assertMachineStorageRefs(c, s.IAASModel, machine.MachineTag())
 }
 
 func (s *VolumeStateSuite) TestAddServiceInvalidPool(c *gc.C) {
@@ -112,10 +114,13 @@ func (s *VolumeStateSuite) TestAddServiceNoUserDefaultPool(c *gc.C) {
 
 func (s *VolumeStateSuite) TestAddServiceDefaultPool(c *gc.C) {
 	// Register a default pool.
-	pm := poolmanager.New(state.NewStateSettings(s.State), dummy.StorageProviders())
+	pm := poolmanager.New(state.NewStateSettings(s.State), storage.ChainedProviderRegistry{
+		dummy.StorageProviders(),
+		provider.CommonStorageProviders(),
+	})
 	_, err := pm.Create("default-block", provider.LoopProviderType, map[string]interface{}{})
 	c.Assert(err, jc.ErrorIsNil)
-	err = s.State.UpdateModelConfig(map[string]interface{}{
+	err = s.IAASModel.UpdateModelConfig(map[string]interface{}{
 		"storage-default-block-source": "default-block",
 	}, nil)
 	c.Assert(err, jc.ErrorIsNil)
@@ -124,7 +129,7 @@ func (s *VolumeStateSuite) TestAddServiceDefaultPool(c *gc.C) {
 	storage := map[string]state.StorageConstraints{
 		"data": makeStorageCons("", 1024, 1),
 	}
-	app := s.AddTestingServiceWithStorage(c, "storage-block", ch, storage)
+	app := s.AddTestingApplicationWithStorage(c, "storage-block", ch, storage)
 	cons, err := app.StorageConstraints()
 	c.Assert(err, jc.ErrorIsNil)
 	c.Assert(cons, jc.DeepEquals, map[string]state.StorageConstraints{
@@ -151,7 +156,7 @@ func (s *VolumeStateSuite) TestSetVolumeInfo(c *gc.C) {
 	s.assertVolumeUnprovisioned(c, volumeTag)
 
 	volumeInfoSet := state.VolumeInfo{Size: 123, Persistent: true, VolumeId: "vol-ume"}
-	err = s.State.SetVolumeInfo(volume.VolumeTag(), volumeInfoSet)
+	err = s.IAASModel.SetVolumeInfo(volume.VolumeTag(), volumeInfoSet)
 	c.Assert(err, jc.ErrorIsNil)
 	volumeInfoSet.Pool = "loop-pool" // taken from params
 	s.assertVolumeInfo(c, volumeTag, volumeInfoSet)
@@ -167,7 +172,7 @@ func (s *VolumeStateSuite) TestSetVolumeInfoNoVolumeId(c *gc.C) {
 	s.assertVolumeUnprovisioned(c, volumeTag)
 
 	volumeInfoSet := state.VolumeInfo{Size: 123, Persistent: true}
-	err = s.State.SetVolumeInfo(volume.VolumeTag(), volumeInfoSet)
+	err = s.IAASModel.SetVolumeInfo(volume.VolumeTag(), volumeInfoSet)
 	c.Assert(err, gc.ErrorMatches, `cannot set info for volume "0/0": volume ID not set`)
 }
 
@@ -197,7 +202,7 @@ func (s *VolumeStateSuite) TestSetVolumeInfoNoStorageAssigned(c *gc.C) {
 	m, err := s.State.Machine(machines[0].Id())
 	c.Assert(err, jc.ErrorIsNil)
 
-	volumeAttachments, err := s.State.MachineVolumeAttachments(m.MachineTag())
+	volumeAttachments, err := s.IAASModel.MachineVolumeAttachments(m.MachineTag())
 	c.Assert(err, jc.ErrorIsNil)
 	c.Assert(volumeAttachments, gc.HasLen, 1)
 	volumeTag := volumeAttachments[0].Volume()
@@ -208,7 +213,7 @@ func (s *VolumeStateSuite) TestSetVolumeInfoNoStorageAssigned(c *gc.C) {
 
 	s.assertVolumeUnprovisioned(c, volumeTag)
 	volumeInfoSet := state.VolumeInfo{Size: 123, VolumeId: "vol-ume"}
-	err = s.State.SetVolumeInfo(volume.VolumeTag(), volumeInfoSet)
+	err = s.IAASModel.SetVolumeInfo(volume.VolumeTag(), volumeInfoSet)
 	c.Assert(err, jc.ErrorIsNil)
 	volumeInfoSet.Pool = "loop-pool" // taken from params
 	s.assertVolumeInfo(c, volumeTag, volumeInfoSet)
@@ -222,14 +227,14 @@ func (s *VolumeStateSuite) TestSetVolumeInfoImmutable(c *gc.C) {
 	volumeTag := volume.VolumeTag()
 
 	volumeInfoSet := state.VolumeInfo{Size: 123, VolumeId: "vol-ume"}
-	err = s.State.SetVolumeInfo(volume.VolumeTag(), volumeInfoSet)
+	err = s.IAASModel.SetVolumeInfo(volume.VolumeTag(), volumeInfoSet)
 	c.Assert(err, jc.ErrorIsNil)
 
 	// The first call to SetVolumeInfo takes the pool name from
 	// the params; the second does not, but it must not change
 	// either. Callers are expected to get the existing info and
 	// update it, leaving immutable values intact.
-	err = s.State.SetVolumeInfo(volume.VolumeTag(), volumeInfoSet)
+	err = s.IAASModel.SetVolumeInfo(volume.VolumeTag(), volumeInfoSet)
 	c.Assert(err, gc.ErrorMatches, `cannot set info for volume "0/0": cannot change pool from "loop-pool" to ""`)
 
 	volumeInfoSet.Pool = "loop-pool"
@@ -247,7 +252,7 @@ func (s *VolumeStateSuite) TestWatchVolumeAttachment(c *gc.C) {
 	volume := s.storageInstanceVolume(c, storageTag)
 	volumeTag := volume.VolumeTag()
 
-	w := s.State.WatchVolumeAttachment(machineTag, volumeTag)
+	w := s.IAASModel.WatchVolumeAttachment(machineTag, volumeTag)
 	defer testing.AssertStop(c, w)
 	wc := testing.NewNotifyWatcherC(c, s.State, w)
 	wc.AssertOneChange()
@@ -258,11 +263,11 @@ func (s *VolumeStateSuite) TestWatchVolumeAttachment(c *gc.C) {
 	c.Assert(err, jc.ErrorIsNil)
 
 	// volume attachment will NOT react to volume changes
-	err = s.State.SetVolumeInfo(volumeTag, state.VolumeInfo{VolumeId: "vol-123"})
+	err = s.IAASModel.SetVolumeInfo(volumeTag, state.VolumeInfo{VolumeId: "vol-123"})
 	c.Assert(err, jc.ErrorIsNil)
 	wc.AssertNoChange()
 
-	err = s.State.SetVolumeAttachmentInfo(
+	err = s.IAASModel.SetVolumeAttachmentInfo(
 		machineTag, volumeTag, state.VolumeAttachmentInfo{
 			DeviceName: "xvdf1",
 		},
@@ -274,14 +279,14 @@ func (s *VolumeStateSuite) TestWatchVolumeAttachment(c *gc.C) {
 func (s *VolumeStateSuite) TestWatchModelVolumes(c *gc.C) {
 	app := s.setupMixedScopeStorageApplication(c, "block")
 	addUnit := func() {
-		u, err := app.AddUnit()
+		u, err := app.AddUnit(state.AddUnitParams{})
 		c.Assert(err, jc.ErrorIsNil)
 		err = s.State.AssignUnit(u, state.AssignCleanEmpty)
 		c.Assert(err, jc.ErrorIsNil)
 	}
 	addUnit()
 
-	w := s.State.WatchModelVolumes()
+	w := s.IAASModel.WatchModelVolumes()
 	defer testing.AssertStop(c, w)
 	wc := testing.NewStringsWatcherC(c, s.State, w)
 	wc.AssertChangeInSingleEvent("0", "1") // initial
@@ -291,22 +296,22 @@ func (s *VolumeStateSuite) TestWatchModelVolumes(c *gc.C) {
 	wc.AssertChangeInSingleEvent("4", "5")
 	wc.AssertNoChange()
 
-	volume, err := s.State.Volume(names.NewVolumeTag("0"))
+	volume, err := s.IAASModel.Volume(names.NewVolumeTag("0"))
 	c.Assert(err, jc.ErrorIsNil)
 	storageTag, err := volume.StorageInstance()
 	c.Assert(err, jc.ErrorIsNil)
-	removeStorageInstance(c, s.State, storageTag)
-	err = s.State.DestroyVolume(names.NewVolumeTag("0"))
+	removeStorageInstance(c, s.IAASModel, storageTag)
+	err = s.IAASModel.DestroyVolume(names.NewVolumeTag("0"))
 	c.Assert(err, jc.ErrorIsNil)
 	wc.AssertChangeInSingleEvent("0") // dying
 	wc.AssertNoChange()
 
-	err = s.State.DetachVolume(names.NewMachineTag("0"), names.NewVolumeTag("0"))
+	err = s.IAASModel.DetachVolume(names.NewMachineTag("0"), names.NewVolumeTag("0"))
 	c.Assert(err, jc.ErrorIsNil)
-	err = s.State.RemoveVolumeAttachment(names.NewMachineTag("0"), names.NewVolumeTag("0"))
+	err = s.IAASModel.RemoveVolumeAttachment(names.NewMachineTag("0"), names.NewVolumeTag("0"))
 	c.Assert(err, jc.ErrorIsNil)
 
-	err = s.State.RemoveVolume(names.NewVolumeTag("0"))
+	err = s.IAASModel.RemoveVolume(names.NewVolumeTag("0"))
 	c.Assert(err, jc.ErrorIsNil)
 	wc.AssertChangeInSingleEvent("0") // removed
 	wc.AssertNoChange()
@@ -315,14 +320,14 @@ func (s *VolumeStateSuite) TestWatchModelVolumes(c *gc.C) {
 func (s *VolumeStateSuite) TestWatchEnvironVolumeAttachments(c *gc.C) {
 	app := s.setupMixedScopeStorageApplication(c, "block")
 	addUnit := func() {
-		u, err := app.AddUnit()
+		u, err := app.AddUnit(state.AddUnitParams{})
 		c.Assert(err, jc.ErrorIsNil)
 		err = s.State.AssignUnit(u, state.AssignCleanEmpty)
 		c.Assert(err, jc.ErrorIsNil)
 	}
 	addUnit()
 
-	w := s.State.WatchEnvironVolumeAttachments()
+	w := s.IAASModel.WatchModelVolumeAttachments()
 	defer testing.AssertStop(c, w)
 	wc := testing.NewStringsWatcherC(c, s.State, w)
 	wc.AssertChangeInSingleEvent("0:0", "0:1") // initial
@@ -332,12 +337,12 @@ func (s *VolumeStateSuite) TestWatchEnvironVolumeAttachments(c *gc.C) {
 	wc.AssertChangeInSingleEvent("1:4", "1:5")
 	wc.AssertNoChange()
 
-	err := s.State.DetachVolume(names.NewMachineTag("0"), names.NewVolumeTag("0"))
+	err := s.IAASModel.DetachVolume(names.NewMachineTag("0"), names.NewVolumeTag("0"))
 	c.Assert(err, jc.ErrorIsNil)
 	wc.AssertChangeInSingleEvent("0:0") // dying
 	wc.AssertNoChange()
 
-	err = s.State.RemoveVolumeAttachment(names.NewMachineTag("0"), names.NewVolumeTag("0"))
+	err = s.IAASModel.RemoveVolumeAttachment(names.NewMachineTag("0"), names.NewVolumeTag("0"))
 	c.Assert(err, jc.ErrorIsNil)
 	wc.AssertChangeInSingleEvent("0:0") // removed
 	wc.AssertNoChange()
@@ -346,14 +351,14 @@ func (s *VolumeStateSuite) TestWatchEnvironVolumeAttachments(c *gc.C) {
 func (s *VolumeStateSuite) TestWatchMachineVolumes(c *gc.C) {
 	app := s.setupMixedScopeStorageApplication(c, "block", "machinescoped", "modelscoped")
 	addUnit := func() {
-		u, err := app.AddUnit()
+		u, err := app.AddUnit(state.AddUnitParams{})
 		c.Assert(err, jc.ErrorIsNil)
 		err = s.State.AssignUnit(u, state.AssignCleanEmpty)
 		c.Assert(err, jc.ErrorIsNil)
 	}
 	addUnit()
 
-	w := s.State.WatchMachineVolumes(names.NewMachineTag("0"))
+	w := s.IAASModel.WatchMachineVolumes(names.NewMachineTag("0"))
 	defer testing.AssertStop(c, w)
 	wc := testing.NewStringsWatcherC(c, s.State, w)
 	wc.AssertChangeInSingleEvent("0/0", "0/1") // initial
@@ -363,21 +368,21 @@ func (s *VolumeStateSuite) TestWatchMachineVolumes(c *gc.C) {
 	// no change, since we're only interested in the one machine.
 	wc.AssertNoChange()
 
-	volume, err := s.State.Volume(names.NewVolumeTag("0/0"))
+	volume, err := s.IAASModel.Volume(names.NewVolumeTag("0/0"))
 	c.Assert(err, jc.ErrorIsNil)
 	storageTag, err := volume.StorageInstance()
 	c.Assert(err, jc.ErrorIsNil)
-	removeStorageInstance(c, s.State, storageTag)
-	err = s.State.DestroyVolume(volume.VolumeTag())
+	removeStorageInstance(c, s.IAASModel, storageTag)
+	err = s.IAASModel.DestroyVolume(volume.VolumeTag())
 	c.Assert(err, jc.ErrorIsNil)
 	wc.AssertChangeInSingleEvent("0/0") // dying
 	wc.AssertNoChange()
 
-	err = s.State.DestroyVolume(names.NewVolumeTag("0/0"))
+	err = s.IAASModel.DestroyVolume(names.NewVolumeTag("0/0"))
 	c.Assert(err, jc.ErrorIsNil)
-	err = s.State.RemoveVolumeAttachment(names.NewMachineTag("0"), names.NewVolumeTag("0/0"))
+	err = s.IAASModel.RemoveVolumeAttachment(names.NewMachineTag("0"), names.NewVolumeTag("0/0"))
 	c.Assert(err, jc.ErrorIsNil)
-	err = s.State.RemoveVolume(names.NewVolumeTag("0/0"))
+	err = s.IAASModel.RemoveVolume(names.NewVolumeTag("0/0"))
 	c.Assert(err, jc.ErrorIsNil)
 	wc.AssertChangeInSingleEvent("0/0") // removed
 	wc.AssertNoChange()
@@ -387,7 +392,7 @@ func (s *VolumeStateSuite) TestWatchMachineVolumeAttachments(c *gc.C) {
 	app := s.setupMixedScopeStorageApplication(c, "block", "machinescoped", "modelscoped")
 	addUnit := func(to *state.Machine) (u *state.Unit, m *state.Machine) {
 		var err error
-		u, err = app.AddUnit()
+		u, err = app.AddUnit(state.AddUnitParams{})
 		c.Assert(err, jc.ErrorIsNil)
 		if to != nil {
 			err = u.AssignToMachine(to)
@@ -396,15 +401,12 @@ func (s *VolumeStateSuite) TestWatchMachineVolumeAttachments(c *gc.C) {
 		}
 		err = s.State.AssignUnit(u, state.AssignCleanEmpty)
 		c.Assert(err, jc.ErrorIsNil)
-		mid, err := u.AssignedMachineId()
-		c.Assert(err, jc.ErrorIsNil)
-		m, err = s.State.Machine(mid)
-		c.Assert(err, jc.ErrorIsNil)
+		m = unitMachine(c, s.State, u)
 		return u, m
 	}
 	_, m0 := addUnit(nil)
 
-	w := s.State.WatchMachineVolumeAttachments(names.NewMachineTag("0"))
+	w := s.IAASModel.WatchMachineVolumeAttachments(names.NewMachineTag("0"))
 	defer testing.AssertStop(c, w)
 	wc := testing.NewStringsWatcherC(c, s.State, w)
 	wc.AssertChangeInSingleEvent("0:0/0", "0:0/1") // initial
@@ -414,19 +416,19 @@ func (s *VolumeStateSuite) TestWatchMachineVolumeAttachments(c *gc.C) {
 	// no change, since we're only interested in the one machine.
 	wc.AssertNoChange()
 
-	err := s.State.DetachVolume(names.NewMachineTag("0"), names.NewVolumeTag("2"))
+	err := s.IAASModel.DetachVolume(names.NewMachineTag("0"), names.NewVolumeTag("2"))
 	c.Assert(err, jc.ErrorIsNil)
 	// no change, since we're only interested in attachments of
 	// machine-scoped volumes.
 	wc.AssertNoChange()
 
-	removeVolumeStorageInstance(c, s.State, names.NewVolumeTag("0/0"))
-	err = s.State.DestroyVolume(names.NewVolumeTag("0/0"))
+	removeVolumeStorageInstance(c, s.IAASModel, names.NewVolumeTag("0/0"))
+	err = s.IAASModel.DestroyVolume(names.NewVolumeTag("0/0"))
 	c.Assert(err, jc.ErrorIsNil)
 	wc.AssertChangeInSingleEvent("0:0/0") // dying
 	wc.AssertNoChange()
 
-	err = s.State.RemoveVolumeAttachment(names.NewMachineTag("0"), names.NewVolumeTag("0/0"))
+	err = s.IAASModel.RemoveVolumeAttachment(names.NewMachineTag("0"), names.NewVolumeTag("0/0"))
 	c.Assert(err, jc.ErrorIsNil)
 	wc.AssertChangeInSingleEvent("0:0/0") // removed
 	wc.AssertNoChange()
@@ -461,7 +463,7 @@ func (s *VolumeStateSuite) TestParseVolumeAttachmentIdError(c *gc.C) {
 func (s *VolumeStateSuite) TestAllVolumes(c *gc.C) {
 	_, expected, _ := s.assertCreateVolumes(c)
 
-	volumes, err := s.State.AllVolumes()
+	volumes, err := s.IAASModel.AllVolumes()
 	c.Assert(err, jc.ErrorIsNil)
 	tags := make([]names.VolumeTag, len(volumes))
 	for i, v := range volumes {
@@ -483,18 +485,18 @@ func (s *VolumeStateSuite) assertCreateVolumes(c *gc.C) (_ *state.Machine, all,
 		}},
 	})
 	c.Assert(err, jc.ErrorIsNil)
-	assertMachineStorageRefs(c, s.State, machine.MachineTag())
+	assertMachineStorageRefs(c, s.IAASModel, machine.MachineTag())
 
 	volume1 := s.volume(c, names.NewVolumeTag("0"))
 	volume2 := s.volume(c, names.NewVolumeTag("0/1"))
 	volume3 := s.volume(c, names.NewVolumeTag("2"))
 
 	volumeInfoSet := state.VolumeInfo{Size: 123, Persistent: true, VolumeId: "vol-1"}
-	err = s.State.SetVolumeInfo(volume1.VolumeTag(), volumeInfoSet)
+	err = s.IAASModel.SetVolumeInfo(volume1.VolumeTag(), volumeInfoSet)
 	c.Assert(err, jc.ErrorIsNil)
 
 	volumeInfoSet = state.VolumeInfo{Size: 456, Persistent: false, VolumeId: "vol-2"}
-	err = s.State.SetVolumeInfo(volume2.VolumeTag(), volumeInfoSet)
+	err = s.IAASModel.SetVolumeInfo(volume2.VolumeTag(), volumeInfoSet)
 	c.Assert(err, jc.ErrorIsNil)
 
 	all = []names.VolumeTag{
@@ -517,21 +519,37 @@ func (s *VolumeStateSuite) TestRemoveStorageInstanceDestroysAndUnassignsVolume(c
 
 	err = u.Destroy()
 	c.Assert(err, jc.ErrorIsNil)
-	err = s.State.DestroyStorageInstance(storageTag)
+
+	// Provision volume attachment so that detaching the storage
+	// attachment does not short-circuit.
+	defer state.SetBeforeHooks(c, s.State, func() {
+		machine := unitMachine(c, s.State, u)
+		err = machine.SetProvisioned("inst-id", "fake_nonce", nil)
+		c.Assert(err, jc.ErrorIsNil)
+		err = s.IAASModel.SetVolumeInfo(volume.VolumeTag(), state.VolumeInfo{VolumeId: "vol-123"})
+		c.Assert(err, jc.ErrorIsNil)
+		err = s.IAASModel.SetVolumeAttachmentInfo(
+			machine.MachineTag(), volume.VolumeTag(),
+			state.VolumeAttachmentInfo{DeviceName: "xvdf1"},
+		)
+		c.Assert(err, jc.ErrorIsNil)
+	}).Check()
+
+	err = s.IAASModel.DestroyStorageInstance(storageTag, true)
 	c.Assert(err, jc.ErrorIsNil)
-	err = s.State.DetachStorage(storageTag, u.UnitTag())
+	err = s.IAASModel.DetachStorage(storageTag, u.UnitTag())
 	c.Assert(err, jc.ErrorIsNil)
 
 	// The storage instance and attachment are dying, but not yet
 	// removed from state. The volume should still be assigned.
 	s.storageInstanceVolume(c, storageTag)
 
-	err = s.State.RemoveStorageAttachment(storageTag, u.UnitTag())
+	err = s.IAASModel.RemoveStorageAttachment(storageTag, u.UnitTag())
 	c.Assert(err, jc.ErrorIsNil)
 
 	// The storage instance is now gone; the volume should no longer
 	// be assigned to the storage.
-	_, err = s.State.StorageInstanceVolume(storageTag)
+	_, err = s.IAASModel.StorageInstanceVolume(storageTag)
 	c.Assert(err, gc.ErrorMatches, `volume for storage instance "data/0" not found`)
 
 	// The volume should still exist, but it should be dying.
@@ -539,6 +557,54 @@ func (s *VolumeStateSuite) TestRemoveStorageInstanceDestroysAndUnassignsVolume(c
 	c.Assert(v.Life(), gc.Equals, state.Dying)
 }
 
+func (s *VolumeStateSuite) TestReleaseStorageInstanceVolumeReleasing(c *gc.C) {
+	_, u, storageTag := s.setupSingleStorage(c, "block", "modelscoped")
+	err := s.State.AssignUnit(u, state.AssignCleanEmpty)
+	c.Assert(err, jc.ErrorIsNil)
+	volume := s.storageInstanceVolume(c, storageTag)
+	c.Assert(err, jc.ErrorIsNil)
+	c.Assert(volume.Releasing(), jc.IsFalse)
+	err = s.IAASModel.SetVolumeInfo(volume.VolumeTag(), state.VolumeInfo{VolumeId: "vol-123"})
+	c.Assert(err, jc.ErrorIsNil)
+
+	err = u.Destroy()
+	c.Assert(err, jc.ErrorIsNil)
+	err = s.IAASModel.ReleaseStorageInstance(storageTag, true)
+	c.Assert(err, jc.ErrorIsNil)
+	err = s.IAASModel.DetachStorage(storageTag, u.UnitTag())
+	c.Assert(err, jc.ErrorIsNil)
+
+	// The volume should should be dying, and releasing.
+	volume = s.volume(c, volume.VolumeTag())
+	c.Assert(volume.Life(), gc.Equals, state.Dying)
+	c.Assert(volume.Releasing(), jc.IsTrue)
+}
+
+func (s *VolumeStateSuite) TestReleaseStorageInstanceVolumeUnreleasable(c *gc.C) {
+	_, u, storageTag := s.setupSingleStorage(c, "block", "modelscoped-unreleasable")
+	err := s.State.AssignUnit(u, state.AssignCleanEmpty)
+	c.Assert(err, jc.ErrorIsNil)
+	volume := s.storageInstanceVolume(c, storageTag)
+	c.Assert(err, jc.ErrorIsNil)
+	c.Assert(volume.Releasing(), jc.IsFalse)
+	err = s.IAASModel.SetVolumeInfo(volume.VolumeTag(), state.VolumeInfo{VolumeId: "vol-123"})
+	c.Assert(err, jc.ErrorIsNil)
+
+	err = u.Destroy()
+	c.Assert(err, jc.ErrorIsNil)
+	err = s.IAASModel.ReleaseStorageInstance(storageTag, true)
+	c.Assert(err, gc.ErrorMatches,
+		`cannot release storage "data/0": storage provider "modelscoped-unreleasable" does not support releasing storage`,
+	)
+	err = s.IAASModel.DetachStorage(storageTag, u.UnitTag())
+	c.Assert(err, jc.ErrorIsNil)
+
+	// The volume should should still be alive.
+	volume = s.volume(c, volume.VolumeTag())
+	c.Assert(volume.Life(), gc.Equals, state.Alive)
+	c.Assert(volume.Releasing(), jc.IsFalse)
+}
+
 func (s *VolumeStateSuite) TestSetVolumeAttachmentInfoVolumeNotProvisioned(c *gc.C) {
 	_, u, storageTag := s.setupSingleStorage(c, "block", "loop-pool")
 	err := s.State.AssignUnit(u, state.AssignCleanEmpty)
@@ -550,7 +616,7 @@ func (s *VolumeStateSuite) TestSetVolumeAttachmentInfoVolumeNotProvisioned(c *gc
 	volume := s.storageInstanceVolume(c, storageTag)
 	volumeTag := volume.VolumeTag()
 
-	err = s.State.SetVolumeAttachmentInfo(
+	err = s.IAASModel.SetVolumeAttachmentInfo(
 		machineTag, volumeTag, state.VolumeAttachmentInfo{
 			DeviceName: "xvdf1",
 		},
@@ -561,7 +627,7 @@ func (s *VolumeStateSuite) TestSetVolumeAttachmentInfoVolumeNotProvisioned(c *gc
 func (s *VolumeStateSuite) TestDestroyVolume(c *gc.C) {
 	volume, _ := s.setupMachineScopedVolumeAttachment(c)
 	assertDestroy := func() {
-		err := s.State.DestroyVolume(volume.VolumeTag())
+		err := s.IAASModel.DestroyVolume(volume.VolumeTag())
 		c.Assert(err, jc.ErrorIsNil)
 		volume = s.volume(c, volume.VolumeTag())
 		c.Assert(volume.Life(), gc.Equals, state.Dying)
@@ -571,7 +637,7 @@ func (s *VolumeStateSuite) TestDestroyVolume(c *gc.C) {
 }
 
 func (s *VolumeStateSuite) TestDestroyVolumeNotFound(c *gc.C) {
-	err := s.State.DestroyVolume(names.NewVolumeTag("0"))
+	err := s.IAASModel.DestroyVolume(names.NewVolumeTag("0"))
 	c.Assert(err, gc.ErrorMatches, `destroying volume 0: volume "0" not found`)
 	c.Assert(err, jc.Satisfies, errors.IsNotFound)
 }
@@ -581,28 +647,27 @@ func (s *VolumeStateSuite) TestDestroyVolumeStorageAssigned(c *gc.C) {
 	storageTag, err := volume.StorageInstance()
 	c.Assert(err, jc.ErrorIsNil)
 
-	err = s.State.DestroyVolume(volume.VolumeTag())
+	err = s.IAASModel.DestroyVolume(volume.VolumeTag())
 	c.Assert(err, gc.ErrorMatches, "destroying volume 0: volume is assigned to storage data/0")
 
 	err = u.Destroy()
 	c.Assert(err, jc.ErrorIsNil)
-	removeStorageInstance(c, s.State, storageTag)
-	err = s.State.DestroyVolume(volume.VolumeTag())
+	removeStorageInstance(c, s.IAASModel, storageTag)
+	err = s.IAASModel.DestroyVolume(volume.VolumeTag())
 	c.Assert(err, jc.ErrorIsNil)
 }
 
 func (s *VolumeStateSuite) TestDestroyVolumeNoAttachments(c *gc.C) {
 	volume, machine := s.setupModelScopedVolumeAttachment(c)
-
-	err := s.State.DetachVolume(machine.MachineTag(), volume.VolumeTag())
+	err := s.IAASModel.DetachVolume(machine.MachineTag(), volume.VolumeTag())
 	c.Assert(err, jc.ErrorIsNil)
 
 	defer state.SetBeforeHooks(c, s.State, func() {
-		err := s.State.RemoveVolumeAttachment(machine.MachineTag(), volume.VolumeTag())
+		err := s.IAASModel.RemoveVolumeAttachment(machine.MachineTag(), volume.VolumeTag())
 		c.Assert(err, jc.ErrorIsNil)
 	}).Check()
 
-	err = s.State.DestroyVolume(volume.VolumeTag())
+	err = s.IAASModel.DestroyVolume(volume.VolumeTag())
 	c.Assert(err, jc.ErrorIsNil)
 	volume = s.volume(c, volume.VolumeTag())
 
@@ -613,14 +678,15 @@ func (s *VolumeStateSuite) TestDestroyVolumeNoAttachments(c *gc.C) {
 
 func (s *VolumeStateSuite) TestRemoveVolume(c *gc.C) {
 	volume, machine := s.setupMachineScopedVolumeAttachment(c)
-	err := s.State.DestroyVolume(volume.VolumeTag())
+
+	err := s.IAASModel.DestroyVolume(volume.VolumeTag())
 	c.Assert(err, jc.ErrorIsNil)
-	err = s.State.RemoveVolumeAttachment(machine.MachineTag(), volume.VolumeTag())
+	err = s.IAASModel.RemoveVolumeAttachment(machine.MachineTag(), volume.VolumeTag())
 	c.Assert(err, jc.ErrorIsNil)
 	assertRemove := func() {
-		err = s.State.RemoveVolume(volume.VolumeTag())
+		err = s.IAASModel.RemoveVolume(volume.VolumeTag())
 		c.Assert(err, jc.ErrorIsNil)
-		_, err = s.State.Volume(volume.VolumeTag())
+		_, err = s.IAASModel.Volume(volume.VolumeTag())
 		c.Assert(err, jc.Satisfies, errors.IsNotFound)
 	}
 	defer state.SetBeforeHooks(c, s.State, assertRemove).Check()
@@ -628,24 +694,24 @@ func (s *VolumeStateSuite) TestRemoveVolume(c *gc.C) {
 }
 
 func (s *VolumeStateSuite) TestRemoveVolumeNotFound(c *gc.C) {
-	err := s.State.RemoveVolume(names.NewVolumeTag("42"))
+	err := s.IAASModel.RemoveVolume(names.NewVolumeTag("42"))
 	c.Assert(err, jc.ErrorIsNil)
 }
 
 func (s *VolumeStateSuite) TestRemoveVolumeNotDead(c *gc.C) {
 	volume, _ := s.setupMachineScopedVolumeAttachment(c)
-	err := s.State.RemoveVolume(volume.VolumeTag())
+	err := s.IAASModel.RemoveVolume(volume.VolumeTag())
 	c.Assert(err, gc.ErrorMatches, "removing volume 0/0: volume is not dead")
-	err = s.State.DestroyVolume(volume.VolumeTag())
+	err = s.IAASModel.DestroyVolume(volume.VolumeTag())
 	c.Assert(err, jc.ErrorIsNil)
-	err = s.State.RemoveVolume(volume.VolumeTag())
+	err = s.IAASModel.RemoveVolume(volume.VolumeTag())
 	c.Assert(err, gc.ErrorMatches, "removing volume 0/0: volume is not dead")
 }
 
 func (s *VolumeStateSuite) TestDetachVolume(c *gc.C) {
 	volume, machine := s.setupModelScopedVolumeAttachment(c)
 	assertDetach := func() {
-		err := s.State.DetachVolume(machine.MachineTag(), volume.VolumeTag())
+		err := s.IAASModel.DetachVolume(machine.MachineTag(), volume.VolumeTag())
 		c.Assert(err, jc.ErrorIsNil)
 		attachment := s.volumeAttachment(c, machine.MachineTag(), volume.VolumeTag())
 		c.Assert(attachment.Life(), gc.Equals, state.Dying)
@@ -657,15 +723,15 @@ func (s *VolumeStateSuite) TestDetachVolume(c *gc.C) {
 func (s *VolumeStateSuite) TestRemoveLastVolumeAttachment(c *gc.C) {
 	volume, machine := s.setupModelScopedVolumeAttachment(c)
 
-	err := s.State.DetachVolume(machine.MachineTag(), volume.VolumeTag())
+	err := s.IAASModel.DetachVolume(machine.MachineTag(), volume.VolumeTag())
 	c.Assert(err, jc.ErrorIsNil)
 
-	err = s.State.DestroyVolume(volume.VolumeTag())
+	err = s.IAASModel.DestroyVolume(volume.VolumeTag())
 	c.Assert(err, jc.ErrorIsNil)
 	volume = s.volume(c, volume.VolumeTag())
 	c.Assert(volume.Life(), gc.Equals, state.Dying)
 
-	err = s.State.RemoveVolumeAttachment(machine.MachineTag(), volume.VolumeTag())
+	err = s.IAASModel.RemoveVolumeAttachment(machine.MachineTag(), volume.VolumeTag())
 	c.Assert(err, jc.ErrorIsNil)
 
 	// The volume was Dying when the last attachment was
@@ -677,17 +743,17 @@ func (s *VolumeStateSuite) TestRemoveLastVolumeAttachment(c *gc.C) {
 func (s *VolumeStateSuite) TestRemoveLastVolumeAttachmentConcurrently(c *gc.C) {
 	volume, machine := s.setupModelScopedVolumeAttachment(c)
 
-	err := s.State.DetachVolume(machine.MachineTag(), volume.VolumeTag())
+	err := s.IAASModel.DetachVolume(machine.MachineTag(), volume.VolumeTag())
 	c.Assert(err, jc.ErrorIsNil)
 
 	defer state.SetBeforeHooks(c, s.State, func() {
-		err := s.State.DestroyVolume(volume.VolumeTag())
+		err := s.IAASModel.DestroyVolume(volume.VolumeTag())
 		c.Assert(err, jc.ErrorIsNil)
 		volume := s.volume(c, volume.VolumeTag())
 		c.Assert(volume.Life(), gc.Equals, state.Dying)
 	}).Check()
 
-	err = s.State.RemoveVolumeAttachment(machine.MachineTag(), volume.VolumeTag())
+	err = s.IAASModel.RemoveVolumeAttachment(machine.MachineTag(), volume.VolumeTag())
 	c.Assert(err, jc.ErrorIsNil)
 
 	// Last attachment was removed, and the volume was (concurrently)
@@ -697,19 +763,20 @@ func (s *VolumeStateSuite) TestRemoveLastVolumeAttachmentConcurrently(c *gc.C) {
 }
 
 func (s *VolumeStateSuite) TestRemoveVolumeAttachmentNotFound(c *gc.C) {
-	err := s.State.RemoveVolumeAttachment(names.NewMachineTag("42"), names.NewVolumeTag("42"))
+	err := s.IAASModel.RemoveVolumeAttachment(names.NewMachineTag("42"), names.NewVolumeTag("42"))
 	c.Assert(err, jc.Satisfies, errors.IsNotFound)
 	c.Assert(err, gc.ErrorMatches, `removing attachment of volume 42 from machine 42: volume "42" on machine "42" not found`)
 }
 
 func (s *VolumeStateSuite) TestRemoveVolumeAttachmentConcurrently(c *gc.C) {
 	volume, machine := s.setupMachineScopedVolumeAttachment(c)
-	err := s.State.DestroyVolume(volume.VolumeTag())
+
+	err := s.IAASModel.DestroyVolume(volume.VolumeTag())
 	c.Assert(err, jc.ErrorIsNil)
 	remove := func() {
-		err := s.State.RemoveVolumeAttachment(machine.MachineTag(), volume.VolumeTag())
+		err := s.IAASModel.RemoveVolumeAttachment(machine.MachineTag(), volume.VolumeTag())
 		c.Assert(err, jc.ErrorIsNil)
-		assertMachineStorageRefs(c, s.State, machine.MachineTag())
+		assertMachineStorageRefs(c, s.IAASModel, machine.MachineTag())
 	}
 	defer state.SetBeforeHooks(c, s.State, remove).Check()
 	remove()
@@ -717,7 +784,8 @@ func (s *VolumeStateSuite) TestRemoveVolumeAttachmentConcurrently(c *gc.C) {
 
 func (s *VolumeStateSuite) TestRemoveVolumeAttachmentAlive(c *gc.C) {
 	volume, machine := s.setupMachineScopedVolumeAttachment(c)
-	err := s.State.RemoveVolumeAttachment(machine.MachineTag(), volume.VolumeTag())
+
+	err := s.IAASModel.RemoveVolumeAttachment(machine.MachineTag(), volume.VolumeTag())
 	c.Assert(err, gc.ErrorMatches, "removing attachment of volume 0/0 from machine 0: volume attachment is not dying")
 }
 
@@ -742,16 +810,16 @@ func (s *VolumeStateSuite) TestRemoveMachineRemovesVolumes(c *gc.C) {
 	c.Assert(err, jc.ErrorIsNil)
 
 	volumeInfoSet := state.VolumeInfo{Size: 123, Persistent: true, VolumeId: "vol-1"}
-	err = s.State.SetVolumeInfo(names.NewVolumeTag("0/1"), volumeInfoSet)
+	err = s.IAASModel.SetVolumeInfo(names.NewVolumeTag("0/1"), volumeInfoSet)
 	c.Assert(err, jc.ErrorIsNil)
 	volumeInfoSet = state.VolumeInfo{Size: 456, Persistent: false, VolumeId: "vol-2"}
-	err = s.State.SetVolumeInfo(names.NewVolumeTag("0/3"), volumeInfoSet)
+	err = s.IAASModel.SetVolumeInfo(names.NewVolumeTag("0/3"), volumeInfoSet)
 	c.Assert(err, jc.ErrorIsNil)
 	volumeInfoSet = state.VolumeInfo{Size: 789, Persistent: false, VolumeId: "vol-3"}
-	err = s.State.SetVolumeInfo(names.NewVolumeTag("4"), volumeInfoSet)
+	err = s.IAASModel.SetVolumeInfo(names.NewVolumeTag("4"), volumeInfoSet)
 	c.Assert(err, jc.ErrorIsNil)
 
-	allVolumes, err := s.State.AllVolumes()
+	allVolumes, err := s.IAASModel.AllVolumes()
 	c.Assert(err, jc.ErrorIsNil)
 
 	persistentVolumes := make([]state.Volume, 0, len(allVolumes))
@@ -774,13 +842,13 @@ func (s *VolumeStateSuite) TestRemoveMachineRemovesVolumes(c *gc.C) {
 	c.Assert(machine.Remove(), jc.ErrorIsNil)
 
 	// Machine is gone: non-detachable storage should be done too.
-	allVolumes, err = s.State.AllVolumes()
+	allVolumes, err = s.IAASModel.AllVolumes()
 	c.Assert(err, jc.ErrorIsNil)
 	// We should only have the persistent volume remaining.
 	c.Assert(allVolumes, gc.HasLen, 1)
 	c.Assert(allVolumes[0].Tag().String(), gc.Equals, "volume-0")
 
-	attachments, err := s.State.MachineVolumeAttachments(machine.MachineTag())
+	attachments, err := s.IAASModel.MachineVolumeAttachments(machine.MachineTag())
 	c.Assert(err, jc.ErrorIsNil)
 	c.Assert(attachments, gc.HasLen, 0)
 }
@@ -842,9 +910,9 @@ func (s *VolumeStateSuite) TestVolumeMachineScoped(c *gc.C) {
 	volume := s.volume(c, names.NewVolumeTag("0/0"))
 	c.Assert(volume.Life(), gc.Equals, state.Alive)
 
-	err = s.State.DestroyVolume(volume.VolumeTag())
+	err = s.IAASModel.DestroyVolume(volume.VolumeTag())
 	c.Assert(err, jc.ErrorIsNil)
-	err = s.State.RemoveVolumeAttachment(machine.MachineTag(), volume.VolumeTag())
+	err = s.IAASModel.RemoveVolumeAttachment(machine.MachineTag(), volume.VolumeTag())
 	c.Assert(err, jc.ErrorIsNil)
 	volume = s.volume(c, volume.VolumeTag())
 	c.Assert(volume.Life(), gc.Equals, state.Dead)
@@ -856,7 +924,7 @@ func (s *VolumeStateSuite) TestVolumeMachineScoped(c *gc.C) {
 	c.Assert(err, jc.ErrorIsNil)
 	err = machine.Remove()
 	c.Assert(err, jc.ErrorIsNil)
-	volume, err = s.State.Volume(volume.VolumeTag())
+	volume, err = s.IAASModel.Volume(volume.VolumeTag())
 	c.Assert(err, jc.Satisfies, errors.IsNotFound)
 }
 
@@ -872,7 +940,7 @@ func (s *VolumeStateSuite) TestVolumeBindingStorage(c *gc.C) {
 	// We must destroy the unit before we can remove the storage.
 	err = u.Destroy()
 	c.Assert(err, jc.ErrorIsNil)
-	removeStorageInstance(c, s.State, storageTag)
+	removeStorageInstance(c, s.IAASModel, storageTag)
 	volume = s.volume(c, volume.VolumeTag())
 	c.Assert(volume.Life(), gc.Equals, state.Dying)
 }
@@ -881,9 +949,8 @@ func (s *VolumeStateSuite) setupStorageVolumeAttachment(c *gc.C) (state.Volume,
 	_, u, storageTag := s.setupSingleStorage(c, "block", "modelscoped")
 	err := s.State.AssignUnit(u, state.AssignCleanEmpty)
 	c.Assert(err, jc.ErrorIsNil)
-	assignedMachineId, err := u.AssignedMachineId()
-	c.Assert(err, jc.ErrorIsNil)
-	return s.storageInstanceVolume(c, storageTag), s.machine(c, assignedMachineId), u
+	machine := unitMachine(c, s.State, u)
+	return s.storageInstanceVolume(c, storageTag), machine, u
 }
 
 func (s *VolumeStateSuite) setupModelScopedVolumeAttachment(c *gc.C) (state.Volume, *state.Machine) {
@@ -903,33 +970,31 @@ func (s *VolumeStateSuite) setupVolumeAttachment(c *gc.C, pool string) (state.Vo
 		}},
 	})
 	c.Assert(err, jc.ErrorIsNil)
-	volumeAttachments, err := s.State.MachineVolumeAttachments(machine.MachineTag())
+	volumeAttachments, err := s.IAASModel.MachineVolumeAttachments(machine.MachineTag())
 	c.Assert(err, jc.ErrorIsNil)
 	c.Assert(volumeAttachments, gc.HasLen, 1)
-	volume, err := s.State.Volume(volumeAttachments[0].Volume())
+	volume, err := s.IAASModel.Volume(volumeAttachments[0].Volume())
 	c.Assert(err, jc.ErrorIsNil)
 	return volume, machine
 }
 
-func removeVolumeStorageInstance(c *gc.C, st *state.State, volumeTag names.VolumeTag) {
-	volume, err := st.Volume(volumeTag)
+func removeVolumeStorageInstance(c *gc.C, im *state.IAASModel, volumeTag names.VolumeTag) {
+	volume, err := im.Volume(volumeTag)
 	c.Assert(err, jc.ErrorIsNil)
 	storageTag, err := volume.StorageInstance()
 	c.Assert(err, jc.ErrorIsNil)
-	removeStorageInstance(c, st, storageTag)
+	removeStorageInstance(c, im, storageTag)
 }
 
-func removeStorageInstance(c *gc.C, st *state.State, storageTag names.StorageTag) {
-	err := st.DestroyStorageInstance(storageTag)
+func removeStorageInstance(c *gc.C, im *state.IAASModel, storageTag names.StorageTag) {
+	err := im.DestroyStorageInstance(storageTag, true)
 	c.Assert(err, jc.ErrorIsNil)
-	attachments, err := st.StorageAttachments(storageTag)
+	attachments, err := im.StorageAttachments(storageTag)
 	c.Assert(err, jc.ErrorIsNil)
 	for _, a := range attachments {
-		err = st.DetachStorage(storageTag, a.Unit())
-		c.Assert(err, jc.ErrorIsNil)
-		err = st.RemoveStorageAttachment(storageTag, a.Unit())
+		err = im.DetachStorage(storageTag, a.Unit())
 		c.Assert(err, jc.ErrorIsNil)
 	}
-	_, err = st.StorageInstance(storageTag)
+	_, err = im.StorageInstance(storageTag)
 	c.Assert(err, jc.Satisfies, errors.IsNotFound)
 }
