@@ -183,6 +183,27 @@ const (
 	Busy Status = "busy"
 )
 
+const (
+	// Status values specific to relations.
+
+	// Joining is used to signify that a relation should become joined soon.
+	Joining Status = "joining"
+
+	// Joined is the normal status for a healthy, alive relation.
+	Joined Status = "joined"
+
+	// Broken is the status for when a relation life goes to Dead.
+	Broken Status = "broken"
+
+	// Suspended is used to signify that a relation will be temporarily broken
+	// pending action to resume it.
+	Suspending Status = "suspending"
+
+	// Suspended is used to signify that a relation is temporarily broken pending
+	// action to resume it.
+	Suspended Status = "suspended"
+)
+
 const (
 	// Status values that are common to several entities.
 
