@@ -4,7 +4,6 @@
 package status
 
 import (
-	"fmt"
 	"time"
 
 	"github.com/juju/errors"
@@ -69,90 +68,16 @@ type DetailedStatus struct {
 // History holds many DetailedStatus,
 type History []DetailedStatus
 
-// push will take an input DetailedStatus and put it as element
-// 0 of the current history pushing all other elements by 1
-// and return the last element, now out of the slice.
-func (h *History) push(new DetailedStatus) DetailedStatus {
-	ch := *h
-	old := ch[0]
-	for i := 0; i < len(ch)-1; i++ {
-		ch[i] = ch[i+1]
-	}
-	ch[len(ch)-1] = new
-	return old
-}
-
-// SquashLogs will find repetitions of N consequent status log entries into just
-// one appearance of them and information about repetition.
-func (h *History) SquashLogs(cycleSize int) History {
-	statuses := *h
-	if len(statuses) <= cycleSize {
-		return statuses
-	}
-	buffer := History{}
-	for i := 0; i < cycleSize; i++ {
-		buffer = append(buffer, statuses[i])
-	}
-	result := []DetailedStatus{}
-	// TODO(perrito666) 2016-05-02 lp:1558657
-	now := time.Now()
-	var repeat int
-	var i int
-	repeatStatus := DetailedStatus{
-		Status: Idle,
-		Info:   "",
-		Since:  &now,
-	}
-
-	for i = cycleSize; i < len(statuses); {
-		if i+cycleSize > len(statuses) {
-			break
-		}
-		subset := statuses[i : i+cycleSize]
-		repetition := true
-		for j := range subset {
-			if subset[j].Status != buffer[j].Status || subset[j].Info != buffer[j].Info {
-				repetition = false
-			}
-		}
-		if repetition {
-			repeat++
-			i = i + cycleSize
-			continue
-		}
-		if repeat > 0 {
-			rstatus := fmt.Sprintf("last %d statuses repeated %d times", cycleSize, repeat)
-			repeat = 0
-			repeatStatus.Info = rstatus
-			for j := 0; j < cycleSize; j++ {
-				result = append(result, buffer.push(subset[j]))
-			}
-			result = append(result, repeatStatus)
-			i = i + cycleSize
-			continue
-		}
-		result = append(result, buffer.push(statuses[i]))
-		i++
-	}
-	for j := 0; j < cycleSize; j++ {
-		result = append(result, buffer[j])
-	}
-	if repeat > 0 {
-		rstatus := fmt.Sprintf("last %d statuses repeated %d times", cycleSize, repeat)
-		repeatStatus.Info = rstatus
-		result = append(result, repeatStatus)
-	}
-	if i < len(statuses)-1 {
-		result = append(result, statuses[i+1:]...)
-	}
-
-	return result
-}
-
 // HistoryKind represents the possible types of
 // status history entries.
+//
 type HistoryKind string
 
+// IMPORTANT DEV NOTE: when changing this HistoryKind list in anyway, these may need to be revised:
+//
+// * HistoryKind.Valid()
+// * AllHistoryKind()
+// * command help for 'show-status-log' describing these kinds.
 const (
 	// KindUnit represents agent and workload combined.
 	KindUnit HistoryKind = "unit"
@@ -185,3 +110,16 @@ func (k HistoryKind) Valid() bool {
 	}
 	return false
 }
+
+// AllHistoryKind will return all valid HistoryKinds.
+func AllHistoryKind() map[HistoryKind]string {
+	return map[HistoryKind]string{
+		KindUnit:              "statuses for specified unit and its workload",
+		KindUnitAgent:         "statuses from the agent that is managing a unit",
+		KindWorkload:          "statuses for unit's workload",
+		KindMachineInstance:   "statuses that occur due to provisioning of a machine",
+		KindMachine:           "status of the agent that is managing a machine",
+		KindContainerInstance: "statuses from the agent that is managing containers",
+		KindContainer:         "statuses from the containers only and not their host machines",
+	}
+}
