@@ -35,7 +35,7 @@ type ProviderRegistry interface {
 
 	// StorageProvider returns the storage provider with the given
 	// provider type. StorageProvider must return an errors satisfying
-	// errors.IsNotFound if the registry does not contain said the
+	// errors.IsNotFound if the registry does not contain the
 	// specified provider type.
 	StorageProvider(ProviderType) (Provider, error)
 }
@@ -72,6 +72,11 @@ type Provider interface {
 	// created at the time a machine is provisioned.
 	Dynamic() bool
 
+	// Releasable reports whether or not the storage provider is capable
+	// of releasing dynamic storage, with either ReleaseVolumes or
+	// ReleaseFilesystems.
+	Releasable() bool
+
 	// DefaultPools returns the default storage pools for this provider,
 	// to register in each new model.
 	DefaultPools() []*Config
@@ -102,6 +107,10 @@ type VolumeSource interface {
 	// volume IDs.
 	DestroyVolumes(volIds []string) ([]error, error)
 
+	// ReleaseVolumes releases the volumes with the specified provider
+	// volume IDs from the model/controller.
+	ReleaseVolumes(volIds []string) ([]error, error)
+
 	// ValidateVolumeParams validates the provided volume creation
 	// parameters, returning an error if they are invalid.
 	ValidateVolumeParams(params VolumeParams) error
@@ -142,6 +151,10 @@ type FilesystemSource interface {
 	// providerd filesystem IDs.
 	DestroyFilesystems(fsIds []string) ([]error, error)
 
+	// ReleaseFilesystems releases the filesystems with the specified provider
+	// filesystem IDs from the model/controller.
+	ReleaseFilesystems(volIds []string) ([]error, error)
+
 	// AttachFilesystems attaches filesystems to machines.
 	//
 	// AttachFilesystems must be idempotent; it may be called even if
@@ -160,6 +173,46 @@ type FilesystemSource interface {
 	DetachFilesystems(params []FilesystemAttachmentParams) ([]error, error)
 }
 
+// FilesystemImporter provides an interface for importing filesystems
+// into the controller/model.
+//
+// TODO(axw) make this part of FilesystemSource?
+type FilesystemImporter interface {
+	// ImportFilesystem updates the filesystem with the specified
+	// filesystem provider ID with the given resource tags, so that
+	// it is seen as being managed by this Juju controller/model.
+	// ImportFilesystem returns the filesystem information to store
+	// in the model.
+	//
+	// Implementations of ImportFilesystem should validate that the
+	// filesystem is not in use before allowing the import to proceed.
+	// Once it is imported, it is assumed to be in a detached state.
+	ImportFilesystem(
+		filesystemId string,
+		resourceTags map[string]string,
+	) (FilesystemInfo, error)
+}
+
+// VolumeImporter provides an interface for importing volumes
+// into the controller/model.
+//
+// TODO(axw) make this part of VolumeSource?
+type VolumeImporter interface {
+	// ImportVolume updates the volume with the specified volume
+	// provider ID with the given resource tags, so that it is
+	// seen as being managed by this Juju controller/model.
+	// ImportVolume returns the volume information to store
+	// in the model.
+	//
+	// Implementations of ImportVolume should validate that the
+	// volume is not in use before allowing the import to proceed.
+	// Once it is imported, it is assumed to be in a detached state.
+	ImportVolume(
+		volumeId string,
+		resourceTags map[string]string,
+	) (VolumeInfo, error)
+}
+
 // VolumeParams is a fully specified set of parameters for volume creation,
 // derived from one or more of user-specified storage constraints, a
 // storage pool definition, and charm storage metadata.
@@ -306,13 +359,6 @@ type CreateFilesystemsResult struct {
 	Error      error
 }
 
-// DescribeFilesystemsResult contains the result of a FilesystemSource.DescribeFilesystems call
-// for one filesystem. Filesystem should only be used if Error is nil.
-type DescribeFilesystemsResult struct {
-	FilesystemInfo *FilesystemInfo
-	Error          error
-}
-
 // AttachFilesystemsResult contains the result of a FilesystemSource.AttachFilesystems call
 // for one filesystem. FilesystemAttachment should only be used if Error is nil.
 type AttachFilesystemsResult struct {
