@@ -81,6 +81,11 @@ func (*rootfsProvider) Dynamic() bool {
 	return true
 }
 
+// Releasable is defined on the Provider interface.
+func (*rootfsProvider) Releasable() bool {
+	return false
+}
+
 // DefaultPools is defined on the Provider interface.
 func (*rootfsProvider) DefaultPools() []*storage.Config {
 	return nil
@@ -188,6 +193,11 @@ func (s *rootfsFilesystemSource) DestroyFilesystems(filesystemIds []string) ([]e
 	return make([]error, len(filesystemIds)), nil
 }
 
+// ReleaseFilesystems is defined on the FilesystemSource interface.
+func (s *rootfsFilesystemSource) ReleaseFilesystems(filesystemIds []string) ([]error, error) {
+	return make([]error, len(filesystemIds)), nil
+}
+
 // AttachFilesystems is defined on the FilesystemSource interface.
 func (s *rootfsFilesystemSource) AttachFilesystems(args []storage.FilesystemAttachmentParams) ([]storage.AttachFilesystemsResult, error) {
 	results := make([]storage.AttachFilesystemsResult, len(args))
