@@ -11,6 +11,7 @@ import (
 	"time"
 
 	gitjujutesting "github.com/juju/testing"
+	utilscert "github.com/juju/utils/cert"
 
 	"github.com/juju/juju/cert"
 )
@@ -52,8 +53,10 @@ func verifyCertificates() error {
 }
 
 func mustNewCA() (string, string) {
-	cert.KeyBits = 512
-	caCert, caKey, err := cert.NewCA("juju testing", "1234-ABCD-IS-NOT-A-REAL-UUID", time.Now().AddDate(10, 0, 0))
+	caCert, caKey, err := cert.NewCA(
+		"juju testing",
+		"1234-ABCD-IS-NOT-A-REAL-UUID",
+		time.Now().AddDate(10, 0, 0))
 	if err != nil {
 		panic(err)
 	}
@@ -61,7 +64,6 @@ func mustNewCA() (string, string) {
 }
 
 func mustNewServer() (string, string) {
-	cert.KeyBits = 512
 	var hostnames []string
 	srvCert, srvKey, err := cert.NewServer(CACert, CAKey, time.Now().AddDate(10, 0, 0), hostnames)
 	if err != nil {
@@ -71,7 +73,7 @@ func mustNewServer() (string, string) {
 }
 
 func mustParseCertAndKey(certPEM, keyPEM string) (*x509.Certificate, *rsa.PrivateKey) {
-	cert, key, err := cert.ParseCertAndKey(certPEM, keyPEM)
+	cert, key, err := utilscert.ParseCertAndKey(certPEM, keyPEM)
 	if err != nil {
 		panic(err)
 	}
