@@ -12,7 +12,7 @@ import (
 	jc "github.com/juju/testing/checkers"
 	"github.com/juju/utils"
 	gc "gopkg.in/check.v1"
-	"gopkg.in/juju/charm.v6-unstable"
+	"gopkg.in/juju/charm.v6"
 	"gopkg.in/juju/names.v2"
 
 	"github.com/juju/juju/instance"
@@ -96,7 +96,7 @@ func (s *factorySuite) TestMakeUserParams(c *gc.C) {
 	c.Assert(err, jc.Satisfies, state.IsNeverLoggedInError)
 	c.Assert(savedLastLogin, gc.Equals, lastLogin)
 
-	_, err = s.State.UserAccess(user.UserTag(), s.State.ModelTag())
+	_, err = s.State.UserAccess(user.UserTag(), s.IAASModel.ModelTag())
 	c.Assert(err, jc.ErrorIsNil)
 }
 
@@ -131,7 +131,7 @@ func (s *factorySuite) TestMakeUserNoModelUser(c *gc.C) {
 
 	_, err := s.State.User(user.UserTag())
 	c.Assert(err, jc.ErrorIsNil)
-	_, err = s.State.UserAccess(user.UserTag(), s.State.ModelTag())
+	_, err = s.State.UserAccess(user.UserTag(), s.IAASModel.ModelTag())
 	c.Assert(err, jc.Satisfies, errors.IsNotFound)
 }
 
@@ -172,7 +172,7 @@ func (s *factorySuite) TestMakeModelUserParams(c *gc.C) {
 		DisplayName: "Foo Bar",
 	})
 
-	saved, err := s.State.UserAccess(modelUser.UserTag, s.State.ModelTag())
+	saved, err := s.State.UserAccess(modelUser.UserTag, s.IAASModel.ModelTag())
 	c.Assert(err, jc.ErrorIsNil)
 	c.Assert(saved.Object.Id(), gc.Equals, modelUser.Object.Id())
 	c.Assert(saved.UserName, gc.Equals, "foobar")
@@ -189,7 +189,7 @@ func (s *factorySuite) TestMakeModelUserInvalidCreatedBy(c *gc.C) {
 	}
 
 	c.Assert(invalidFunc, gc.PanicMatches, `interface conversion: .*`)
-	saved, err := s.State.UserAccess(names.NewLocalUserTag("bob"), s.State.ModelTag())
+	saved, err := s.State.UserAccess(names.NewLocalUserTag("bob"), s.IAASModel.ModelTag())
 	c.Assert(err, jc.Satisfies, errors.IsNotFound)
 	c.Assert(saved, gc.DeepEquals, permission.UserAccess{})
 }
@@ -202,7 +202,7 @@ func (s *factorySuite) TestMakeModelUserNonLocalUser(c *gc.C) {
 		CreatedBy:   creator.UserTag(),
 	})
 
-	saved, err := s.State.UserAccess(modelUser.UserTag, s.State.ModelTag())
+	saved, err := s.State.UserAccess(modelUser.UserTag, s.IAASModel.ModelTag())
 	c.Assert(err, jc.ErrorIsNil)
 	c.Assert(saved.Object.Id(), gc.Equals, modelUser.Object.Id())
 	c.Assert(saved.UserName, gc.Equals, "foobar@ubuntuone")
@@ -265,12 +265,12 @@ func (s *factorySuite) TestMakeMachine(c *gc.C) {
 	c.Assert(machine.PasswordValid(password), jc.IsTrue)
 
 	assertVolume := func(name string, size uint64) {
-		volume, err := s.State.Volume(names.NewVolumeTag(name))
+		volume, err := s.IAASModel.Volume(names.NewVolumeTag(name))
 		c.Assert(err, jc.ErrorIsNil)
 		volParams, ok := volume.Params()
 		c.Assert(ok, jc.IsTrue)
 		c.Assert(volParams, jc.DeepEquals, state.VolumeParams{Pool: "loop", Size: size})
-		volAttachments, err := s.State.VolumeAttachments(volume.VolumeTag())
+		volAttachments, err := s.IAASModel.VolumeAttachments(volume.VolumeTag())
 		c.Assert(err, jc.ErrorIsNil)
 		c.Assert(volAttachments, gc.HasLen, 1)
 		c.Assert(volAttachments[0].Machine(), gc.Equals, machine.Tag())
@@ -278,12 +278,12 @@ func (s *factorySuite) TestMakeMachine(c *gc.C) {
 	assertVolume(machine.Id()+"/0", 2048) // backing the filesystem
 	assertVolume(machine.Id()+"/1", 1024)
 
-	filesystem, err := s.State.Filesystem(names.NewFilesystemTag(machine.Id() + "/0"))
+	filesystem, err := s.IAASModel.Filesystem(names.NewFilesystemTag(machine.Id() + "/0"))
 	c.Assert(err, jc.ErrorIsNil)
 	fsParams, ok := filesystem.Params()
 	c.Assert(ok, jc.IsTrue)
 	c.Assert(fsParams, jc.DeepEquals, state.FilesystemParams{Pool: "loop", Size: 2048})
-	fsAttachments, err := s.State.MachineFilesystemAttachments(machine.Tag().(names.MachineTag))
+	fsAttachments, err := s.IAASModel.MachineFilesystemAttachments(machine.Tag().(names.MachineTag))
 	c.Assert(err, jc.ErrorIsNil)
 	c.Assert(fsAttachments, gc.HasLen, 1)
 	c.Assert(fsAttachments[0].Machine(), gc.Equals, machine.Tag())
@@ -509,7 +509,10 @@ func (s *factorySuite) TestMakeModelNil(c *gc.C) {
 	c.Assert(err, jc.ErrorIsNil)
 	c.Assert(env.Owner(), gc.Equals, origEnv.Owner())
 
-	cfg, err := st.ModelConfig()
+	m, err := st.Model()
+	c.Assert(err, jc.ErrorIsNil)
+
+	cfg, err := m.ModelConfig()
 	c.Assert(err, jc.ErrorIsNil)
 	c.Assert(cfg.AllAttrs()["default-series"], gc.Equals, "xenial")
 }
@@ -533,7 +536,9 @@ func (s *factorySuite) TestMakeModel(c *gc.C) {
 	c.Assert(env.UUID() == s.State.ModelUUID(), jc.IsFalse)
 	c.Assert(env.Owner(), gc.Equals, owner.UserTag())
 
-	cfg, err := st.ModelConfig()
+	m, err := st.Model()
+	c.Assert(err, jc.ErrorIsNil)
+	cfg, err := m.ModelConfig()
 	c.Assert(err, jc.ErrorIsNil)
 	c.Assert(cfg.AllAttrs()["default-series"], gc.Equals, "precise")
 }
