@@ -3,9 +3,7 @@
 
 package testing
 
-import (
-	gc "gopkg.in/check.v1"
-)
+import gc "gopkg.in/check.v1"
 
 type GitSuite struct {
 	BaseSuite
