@@ -13,6 +13,7 @@ import (
 	"net/http"
 	"net/url"
 	"os"
+	"path/filepath"
 	"strconv"
 	"strings"
 	"syscall"
@@ -215,6 +216,27 @@ func isSupportedAPIVersion(version string) bool {
 func newRawClient(remote Remote) (*lxd.Client, error) {
 	host := remote.Host
 
+	// LXD socket is different depending on installation method
+	// We prefer upstream's preference of snap installed LXD
+	debianSocket := filepath.FromSlash("/var/lib/lxd")
+	snapSocket := filepath.FromSlash("/var/snap/lxd/common/lxd")
+	envSocket := os.Getenv("LXD_DIR")
+
+	// The current lxdclient code ONLY looks at LXD_DIR, as it has no understanding
+	// of snaps. This forces us to set LXD_DIR to set the socket path.
+	// This should be updated once lxdclient dependency is updated
+	if envSocket != "" {
+		logger.Debugf("Using environment LXD_DIR as socket path: %q", envSocket)
+	} else {
+		if _, err := os.Stat(snapSocket); err == nil {
+			logger.Debugf("Using LXD snap socket: %q", snapSocket)
+			os.Setenv("LXD_DIR", snapSocket)
+		} else {
+			logger.Debugf("LXD snap socket not found, falling back to debian socket: %q", debianSocket)
+			os.Setenv("LXD_DIR", debianSocket)
+		}
+	}
+
 	if remote.ID() == remoteIDForLocal && host == "" {
 		host = "unix://" + lxdshared.VarPath("unix.socket")
 	} else {
@@ -473,7 +495,7 @@ Please configure LXD by running:
 
 	installText := `
 Please install LXD by running:
-	$ sudo apt-get install lxd
+	$ sudo snap install lxd
 and then configure it with:
 	$ newgrp lxd
 	$ lxd init
