@@ -6,7 +6,6 @@
 package lxdclient
 
 import (
-	"fmt"
 	"io"
 	"strings"
 
@@ -50,16 +49,6 @@ type instanceClient struct {
 	remote string
 }
 
-func deviceProperties(device Device) []string {
-	var props []string
-
-	for k, v := range device {
-		props = append(props, fmt.Sprintf("%s=%s", k, v))
-	}
-
-	return props
-}
-
 func (client *instanceClient) addInstance(spec InstanceSpec) error {
 	imageRemote := spec.ImageRemote
 	if imageRemote == "" {
