@@ -11,7 +11,10 @@ import (
 
 var NewInstanceSummary = newInstanceSummary
 
-type RawInstanceClient rawInstanceClient
+type (
+	RawInstanceClient rawInstanceClient
+	RawStorageClient  rawStorageClient
+)
 
 func NewInstanceClient(raw RawInstanceClient) *instanceClient {
 	return &instanceClient{
@@ -20,6 +23,13 @@ func NewInstanceClient(raw RawInstanceClient) *instanceClient {
 	}
 }
 
+func NewStorageClient(raw RawStorageClient, supported bool) *storageClient {
+	return &storageClient{
+		raw:       raw,
+		supported: supported,
+	}
+}
+
 func PatchGenerateCertificate(s *testing.CleanupSuite, cert, key string) {
 	s.PatchValue(&generateCertificate, func() ([]byte, []byte, error) {
 		return []byte(cert), []byte(key), nil
