@@ -14,8 +14,7 @@ type fakeAPI struct{}
 
 func (*fakeAPI) ConnectionInfo() (params.DeployerConnectionValues, error) {
 	return params.DeployerConnectionValues{
-		StateAddresses: []string{"s1:123", "s2:123"},
-		APIAddresses:   []string{"a1:123", "a2:123"},
+		APIAddresses: []string{"a1:123", "a2:123"},
 	}, nil
 }
 
@@ -29,7 +28,7 @@ func NewTestSimpleContext(agentConfig agent.Config, logDir string, data *svctest
 			return svc, nil
 		},
 		listServices: func() ([]string, error) {
-			return data.InstalledNames.Values(), nil
+			return data.InstalledNames(), nil
 		},
 	}
 }
