@@ -5,8 +5,8 @@ package fortress
 
 import (
 	"github.com/juju/errors"
+	worker "gopkg.in/juju/worker.v1"
 
-	"github.com/juju/juju/worker"
 	"github.com/juju/juju/worker/dependency"
 )
 
