@@ -5,8 +5,7 @@ package gate
 
 import (
 	"github.com/juju/errors"
-
-	"github.com/juju/juju/worker"
+	worker "gopkg.in/juju/worker.v1"
 )
 
 func NewFlagWorker(gate Waiter) (worker.Worker, error) {
