@@ -5,10 +5,10 @@ package hostkeyreporter
 
 import (
 	"github.com/juju/errors"
+	worker "gopkg.in/juju/worker.v1"
 
 	"github.com/juju/juju/api/base"
 	apihostkeyreporter "github.com/juju/juju/api/hostkeyreporter"
-	"github.com/juju/juju/worker"
 )
 
 func NewFacade(apiCaller base.APICaller) (Facade, error) {
