@@ -127,12 +127,23 @@ func (p *updater) startMachines(tags []names.MachineTag) error {
 			if err != nil {
 				return errors.Trace(err)
 			}
-			// We don't poll manual machines.
+			// We don't poll manual machines, instead we're setting the status to 'running'
+			// as we don't have any better information from the provider, see lp:1678981
 			isManual, err := m.IsManual()
 			if err != nil {
 				return errors.Trace(err)
 			}
 			if isManual {
+				statusInfo, err := m.Status()
+				if err != nil {
+					return errors.Trace(err)
+				}
+				machineStatus := status.Status(statusInfo.Status)
+				if machineStatus != status.Running {
+					if err = m.SetInstanceStatus(status.Running, "Manually provisioned machine", nil); err != nil {
+						logger.Errorf("cannot set instance status on %q: %v", m, err)
+					}
+				}
 				continue
 			}
 			c = make(chan struct{})
@@ -201,6 +212,9 @@ func machineLoop(context machineContext, m machine, lifeChanged <-chan struct{},
 				// We have no addresses or not started - poll increasingly rarely
 				// until we do.
 				pollInterval = time.Duration(float64(pollInterval) * ShortPollBackoff)
+				if pollInterval > LongPoll {
+					pollInterval = LongPoll
+				}
 			}
 		}
 		return nil
