@@ -5,11 +5,11 @@ package lifeflag
 
 import (
 	"github.com/juju/errors"
+	worker "gopkg.in/juju/worker.v1"
 
 	"github.com/juju/juju/api/base"
 	"github.com/juju/juju/api/lifeflag"
 	"github.com/juju/juju/api/watcher"
-	"github.com/juju/juju/worker"
 )
 
 func NewWorker(config Config) (worker.Worker, error) {
