@@ -1,4 +1,7 @@
-package logsender
+// Copyright 2015 Canonical Ltd.
+// Licensed under the AGPLv3, see LICENCE file for details.
+
+package logsender_test
 
 import (
 	stdtesting "testing"
