@@ -13,7 +13,7 @@ import (
 	"github.com/juju/loggo"
 	"github.com/juju/utils/clock"
 	"github.com/juju/version"
-	"gopkg.in/juju/charm.v6-unstable"
+	"gopkg.in/juju/charm.v6"
 	"gopkg.in/juju/names.v2"
 
 	"github.com/juju/juju/api"
@@ -182,9 +182,10 @@ func New(config Config) (*Worker, error) {
 // Worker waits until a migration is active and its configured
 // Fortress is locked down, and then orchestrates a model migration.
 type Worker struct {
-	catacomb catacomb.Catacomb
-	config   Config
-	logger   loggo.Logger
+	catacomb    catacomb.Catacomb
+	config      Config
+	logger      loggo.Logger
+	lastFailure string
 }
 
 // Kill implements worker.Worker.
@@ -276,6 +277,7 @@ func (w *Worker) setInfoStatus(s string, a ...interface{}) {
 }
 
 func (w *Worker) setErrorStatus(s string, a ...interface{}) {
+	w.lastFailure = fmt.Sprintf(s, a...)
 	w.setStatusAndLog(w.logger.Errorf, s, a...)
 }
 
@@ -407,6 +409,11 @@ func (w *Worker) transferModel(targetInfo coremigration.TargetInfo, modelUUID st
 		return errors.Annotate(err, "failed to import model into target controller")
 	}
 
+	if wrench.IsActive("migrationmaster", "die-in-export") {
+		// Simulate a abort causing failure to test last status not over written.
+		return errors.New("wrench in the transferModel works")
+	}
+
 	w.setInfoStatus("uploading model binaries into target controller")
 	wrapper := &uploadWrapper{targetClient, modelUUID}
 	err = w.config.UploadBinaries(migration.UploadBinariesConfig{
@@ -612,7 +619,7 @@ func (w *Worker) doREAP() (coremigration.Phase, error) {
 }
 
 func (w *Worker) doABORT(targetInfo coremigration.TargetInfo, modelUUID string) (coremigration.Phase, error) {
-	w.setInfoStatus("aborted, removing model from target controller")
+	w.setInfoStatus("aborted, removing model from target controller: %s", w.lastFailure)
 	if err := w.removeImportedModel(targetInfo, modelUUID); err != nil {
 		// This isn't fatal. Removing the imported model is a best
 		// efforts attempt so just report the error and proceed.
@@ -675,40 +682,6 @@ func (w *Worker) waitForActiveMigration() (coremigration.MigrationStatus, error)
 	}
 }
 
-func (w *Worker) waitForMigrationEnd() error {
-	w.logger.Infof("migration is externally managed. waiting for completion")
-	watcher, err := w.config.Facade.Watch()
-	if err != nil {
-		return errors.Annotate(err, "watching for migration")
-	}
-	if err := w.catacomb.Add(watcher); err != nil {
-		return errors.Trace(err)
-	}
-	defer watcher.Kill()
-
-	for {
-		select {
-		case <-w.catacomb.Dying():
-			return w.catacomb.ErrDying()
-		case <-watcher.Changes():
-		}
-
-		status, err := w.config.Facade.MigrationStatus()
-		if err != nil {
-			return errors.Annotate(err, "retrieving migration status")
-		}
-		w.logger.Infof("migration phase is now %v", status.Phase)
-		if status.Phase.IsTerminal() {
-			if modelHasMigrated(status.Phase) {
-				w.logger.Infof("migration is complete")
-				return ErrMigrated
-			}
-			w.logger.Infof("migration has aborted")
-			return ErrInactive
-		}
-	}
-}
-
 // Possible values for waitForMinion's waitPolicy argument.
 const failFast = false  // Stop waiting at first minion failure report
 const waitForAll = true // Wait for all minion reports to arrive (or timeout)
