@@ -6,6 +6,7 @@ package peergrouper
 import (
 	"fmt"
 	"sort"
+	"strings"
 
 	"github.com/juju/replicaset"
 
@@ -35,14 +36,19 @@ func desiredPeerGroup(info *peerGroupInfo) ([]replicaset.Member, map[*machineTra
 	}
 	changed := false
 	members, extra, maxId := info.membersMap()
-	logger.Debugf("calculating desired peer group")
-	line := "members: ..."
+	lines := make([]string, 0)
+	lines = append(lines, fmt.Sprintf("calculated desired peer group\ndesired voting members: (maxId: %d)", maxId))
 	for tracker, replMem := range members {
-		line = fmt.Sprintf("%s\n   %#v: rs_id=%d, rs_addr=%s", line, tracker, replMem.Id, replMem.Address)
+		lines = append(lines, fmt.Sprintf("\n   %#v: rs_id=%d, rs_addr=%s", tracker, replMem.Id, replMem.Address))
 	}
-	logger.Debugf(line)
-	logger.Debugf("extra: %#v", extra)
-	logger.Debugf("maxId: %v", maxId)
+	if len(extra) > 0 {
+		lines = append(lines, "\nother members:")
+		for _, replMem := range extra {
+			vote := (replMem.Votes != nil && *replMem.Votes > 0)
+			lines = append(lines, fmt.Sprintf("\n   rs_id=%d, rs_addr=%s, tags=%v, vote=%t", replMem.Id, replMem.Address, replMem.Tags, vote))
+		}
+	}
+	logger.Debugf(strings.Join(lines, ""))
 
 	// We may find extra peer group members if the machines
 	// have been removed or their controller status removed.
@@ -220,23 +226,26 @@ func addNewMembers(
 	mongoSpace network.SpaceName,
 ) {
 	for _, m := range toKeep {
-		hasAddress := m.SelectMongoHostPort(mongoSpace) != ""
-		if members[m] == nil && hasAddress {
-			// This machine was not previously in the members list,
-			// so add it (as non-voting). We maintain the
-			// id manually to make it easier for tests.
-			maxId++
-			member := &replicaset.Member{
-				Tags: map[string]string{
-					jujuMachineKey: m.Id(),
-				},
-				Id: maxId,
-			}
-			members[m] = member
-			setVoting(m, false)
-		} else if !hasAddress {
-			logger.Debugf("ignoring machine %q with no address", m.Id())
+		if members[m] != nil {
+			continue
 		}
+		hasAddress := m.SelectMongoHostPort(mongoSpace) != ""
+		if !hasAddress {
+			logger.Debugf("ignoring machine %q with no address", m.Id())
+			continue
+		}
+		// This machine was not previously in the members list,
+		// so add it (as non-voting). We maintain the
+		// id manually to make it easier for tests.
+		maxId++
+		member := &replicaset.Member{
+			Tags: map[string]string{
+				jujuMachineKey: m.Id(),
+			},
+			Id: maxId,
+		}
+		members[m] = member
+		setVoting(m, false)
 	}
 }
 
