@@ -6,6 +6,10 @@ package provisioner
 import (
 	"sort"
 
+	"github.com/juju/version"
+
+	apiprovisioner "github.com/juju/juju/api/provisioner"
+	"github.com/juju/juju/environs"
 	"github.com/juju/juju/environs/config"
 	"github.com/juju/juju/watcher"
 )
@@ -14,9 +18,11 @@ func SetObserver(p Provisioner, observer chan<- *config.Config) {
 	var configObserver *configObserver
 	if ep, ok := p.(*environProvisioner); ok {
 		configObserver = &ep.configObserver
+		configObserver.catacomb = &ep.catacomb
 	} else {
 		cp := p.(*containerProvisioner)
 		configObserver = &cp.configObserver
+		configObserver.catacomb = &cp.catacomb
 	}
 	configObserver.Lock()
 	configObserver.observer = observer
@@ -35,6 +41,7 @@ var (
 	RetryStrategyDelay       = &retryStrategyDelay
 	RetryStrategyCount       = &retryStrategyCount
 	GetObservedNetworkConfig = &getObservedNetworkConfig
+	CombinedCloudInitData    = combinedCloudInitData
 )
 
 var ClassifyMachine = classifyMachine
@@ -52,3 +59,14 @@ func GetCopyAvailabilityZoneMachines(p ProvisionerTask) []AvailabilityZoneMachin
 	}
 	return retvalues
 }
+
+func SetupToStartMachine(p ProvisionerTask, machine *apiprovisioner.Machine, version *version.Number) (
+	environs.StartInstanceParams,
+	error,
+) {
+	return p.(*provisionerTask).setupToStartMachine(machine, version)
+}
+
+func GetAPIProvisionerState(p Provisioner) *apiprovisioner.State {
+	return p.(*environProvisioner).st
+}
