@@ -14,19 +14,20 @@ import (
 	gitjujutesting "github.com/juju/testing"
 	jc "github.com/juju/testing/checkers"
 	"github.com/juju/utils/arch"
-	"github.com/juju/utils/series"
 	gc "gopkg.in/check.v1"
 	"gopkg.in/juju/names.v2"
 
 	"github.com/juju/juju/agent"
 	"github.com/juju/juju/api/common"
 	"github.com/juju/juju/apiserver/params"
+	"github.com/juju/juju/cloudconfig/instancecfg"
 	"github.com/juju/juju/container"
 	"github.com/juju/juju/container/kvm"
 	"github.com/juju/juju/container/kvm/mock"
 	kvmtesting "github.com/juju/juju/container/kvm/testing"
 	"github.com/juju/juju/environs"
 	"github.com/juju/juju/instance"
+	supportedversion "github.com/juju/juju/juju/version"
 	"github.com/juju/juju/network"
 	"github.com/juju/juju/state"
 	coretesting "github.com/juju/juju/testing"
@@ -45,6 +46,7 @@ type kvmBrokerSuite struct {
 	kvmSuite
 	agentConfig agent.Config
 	api         *fakeAPI
+	manager     *fakeContainerManager
 }
 
 var _ = gc.Suite(&kvmBrokerSuite{})
@@ -76,6 +78,9 @@ func (s *kvmBrokerSuite) SetUpTest(c *gc.C) {
 		c.Skip("Skipping kvm tests on windows")
 	}
 	s.kvmSuite.SetUpTest(c)
+	s.PatchValue(&provisioner.GetMachineCloudInitData, func(_ string) (map[string]interface{}, error) {
+		return nil, nil
+	})
 	var err error
 	s.agentConfig, err = agent.NewAgentConfig(
 		agent.AgentConfigParams{
@@ -91,6 +96,7 @@ func (s *kvmBrokerSuite) SetUpTest(c *gc.C) {
 		})
 	c.Assert(err, jc.ErrorIsNil)
 	s.api = NewFakeAPI()
+	s.manager = &fakeContainerManager{}
 }
 
 func (s *kvmBrokerSuite) startInstance(c *gc.C, broker environs.InstanceBroker, machineId string) (*environs.StartInstanceResult, error) {
@@ -104,6 +110,10 @@ func (s *kvmBrokerSuite) newKVMBroker(c *gc.C) (environs.InstanceBroker, error)
 	return provisioner.NewKVMBroker(s.api.PrepareHost, s.api, manager, s.agentConfig)
 }
 
+func (s *kvmBrokerSuite) newKVMBrokerFakeManager(c *gc.C) (environs.InstanceBroker, error) {
+	return provisioner.NewKVMBroker(s.api.PrepareHost, s.api, s.manager, s.agentConfig)
+}
+
 func (s *kvmBrokerSuite) maintainInstance(c *gc.C, broker environs.InstanceBroker, machineId string) {
 	callMaintainInstance(c, s, broker, machineId)
 }
@@ -248,6 +258,82 @@ func (s *kvmBrokerSuite) TestStartInstancePopulatesFallbackNetworkInfo(c *gc.C)
 	c.Assert(err, gc.ErrorMatches, "container address allocation not supported")
 }
 
+func (s *kvmBrokerSuite) TestStartInstanceWithCloudInitUserData(c *gc.C) {
+	broker, brokerErr := s.newKVMBrokerFakeManager(c)
+	c.Assert(brokerErr, jc.ErrorIsNil)
+
+	_, err := s.startInstance(c, broker, "1/kvm/0")
+	c.Assert(err, jc.ErrorIsNil)
+
+	s.manager.CheckCallNames(c, "CreateContainer")
+	call := s.manager.Calls()[0]
+	c.Assert(call.Args[0], gc.FitsTypeOf, &instancecfg.InstanceConfig{})
+	instanceConfig := call.Args[0].(*instancecfg.InstanceConfig)
+	assertCloudInitUserData(instanceConfig.CloudInitUserData, map[string]interface{}{
+		"packages":        []interface{}{"python-keystoneclient", "python-glanceclient"},
+		"preruncmd":       []interface{}{"mkdir /tmp/preruncmd", "mkdir /tmp/preruncmd2"},
+		"postruncmd":      []interface{}{"mkdir /tmp/postruncmd", "mkdir /tmp/postruncmd2"},
+		"package_upgrade": false,
+	}, c)
+}
+
+func (s *kvmBrokerSuite) TestStartInstanceWithContainerInheritProperties(c *gc.C) {
+	s.PatchValue(&provisioner.GetMachineCloudInitData, func(_ string) (map[string]interface{}, error) {
+		return map[string]interface{}{
+			"packages":   []interface{}{"python-novaclient"},
+			"fake-entry": []interface{}{"testing-garbage"},
+			"apt": map[interface{}]interface{}{
+				"primary": []interface{}{
+					map[interface{}]interface{}{
+						"arches": []interface{}{"default"},
+						"uri":    "http://archive.ubuntu.com/ubuntu",
+					},
+				},
+				"security": []interface{}{
+					map[interface{}]interface{}{
+						"arches": []interface{}{"default"},
+						"uri":    "http://archive.ubuntu.com/ubuntu",
+					},
+				},
+			},
+			"ca-certs": map[interface{}]interface{}{
+				"remove-defaults": true,
+				"trusted":         []interface{}{"-----BEGIN CERTIFICATE-----\nYOUR-ORGS-TRUSTED-CA-CERT-HERE\n-----END CERTIFICATE-----\n"},
+			},
+		}, nil
+	})
+	s.api.fakeContainerConfig.ContainerInheritProperties = "ca-certs,apt-security"
+
+	broker, brokerErr := s.newKVMBrokerFakeManager(c)
+	c.Assert(brokerErr, jc.ErrorIsNil)
+
+	_, err := s.startInstance(c, broker, "1/kvm/0")
+	c.Assert(err, jc.ErrorIsNil)
+
+	s.manager.CheckCallNames(c, "CreateContainer")
+	call := s.manager.Calls()[0]
+	c.Assert(call.Args[0], gc.FitsTypeOf, &instancecfg.InstanceConfig{})
+	instanceConfig := call.Args[0].(*instancecfg.InstanceConfig)
+	assertCloudInitUserData(instanceConfig.CloudInitUserData, map[string]interface{}{
+		"packages":        []interface{}{"python-keystoneclient", "python-glanceclient"},
+		"preruncmd":       []interface{}{"mkdir /tmp/preruncmd", "mkdir /tmp/preruncmd2"},
+		"postruncmd":      []interface{}{"mkdir /tmp/postruncmd", "mkdir /tmp/postruncmd2"},
+		"package_upgrade": false,
+		"apt": map[string]interface{}{
+			"security": []interface{}{
+				map[interface{}]interface{}{
+					"arches": []interface{}{"default"},
+					"uri":    "http://archive.ubuntu.com/ubuntu",
+				},
+			},
+		},
+		"ca-certs": map[interface{}]interface{}{
+			"remove-defaults": true,
+			"trusted":         []interface{}{"-----BEGIN CERTIFICATE-----\nYOUR-ORGS-TRUSTED-CA-CERT-HERE\n-----END CERTIFICATE-----\n"},
+		},
+	}, c)
+}
+
 type kvmProvisionerSuite struct {
 	CommonProvisionerSuite
 	kvmSuite
@@ -345,7 +431,7 @@ func (s *kvmProvisionerSuite) TestDoesNotStartEnvironMachines(c *gc.C) {
 	defer workertest.CleanKill(c, p)
 
 	// Check that an instance is not provisioned when the machine is created.
-	_, err := s.State.AddMachine(series.LatestLts(), state.JobHostUnits)
+	_, err := s.State.AddMachine(supportedversion.SupportedLTS(), state.JobHostUnits)
 	c.Assert(err, jc.ErrorIsNil)
 
 	s.expectNoEvents(c)
@@ -362,7 +448,7 @@ func (s *kvmProvisionerSuite) TestDoesNotHaveRetryWatcher(c *gc.C) {
 
 func (s *kvmProvisionerSuite) addContainer(c *gc.C) *state.Machine {
 	template := state.MachineTemplate{
-		Series: series.LatestLts(),
+		Series: supportedversion.SupportedLTS(),
 		Jobs:   []state.MachineJob{state.JobHostUnits},
 	}
 	container, err := s.State.AddMachineInsideMachine(template, "0", instance.KVM)
@@ -398,8 +484,3 @@ func (s *kvmProvisionerSuite) TestKVMProvisionerObservesConfigChanges(c *gc.C) {
 	defer workertest.CleanKill(c, p)
 	s.assertProvisionerObservesConfigChanges(c, p)
 }
-
-type kvmFakeBridger struct {
-	brokerSuite      *kvmBrokerSuite
-	provisionerSuite *kvmProvisionerSuite
-}
