@@ -10,6 +10,7 @@ import (
 	"time"
 
 	"github.com/juju/errors"
+	"github.com/juju/loggo"
 	jc "github.com/juju/testing/checkers"
 	"github.com/juju/utils"
 	"github.com/juju/utils/arch"
@@ -18,7 +19,7 @@ import (
 	"github.com/juju/version"
 	gc "gopkg.in/check.v1"
 	"gopkg.in/juju/names.v2"
-	worker "gopkg.in/juju/worker.v1"
+	"gopkg.in/juju/worker.v1"
 
 	"github.com/juju/juju/agent"
 	"github.com/juju/juju/api"
@@ -36,6 +37,7 @@ import (
 	"github.com/juju/juju/environs/tools"
 	"github.com/juju/juju/instance"
 	"github.com/juju/juju/juju/testing"
+	supportedversion "github.com/juju/juju/juju/version"
 	"github.com/juju/juju/network"
 	providercommon "github.com/juju/juju/provider/common"
 	"github.com/juju/juju/provider/dummy"
@@ -65,7 +67,7 @@ type CommonProvisionerSuite struct {
 
 func (s *CommonProvisionerSuite) assertProvisionerObservesConfigChanges(c *gc.C, p provisioner.Provisioner) {
 	// Inject our observer into the provisioner
-	cfgObserver := make(chan *config.Config, 1)
+	cfgObserver := make(chan *config.Config)
 	provisioner.SetObserver(p, cfgObserver)
 
 	// Switch to reaping on All machines.
@@ -80,7 +82,7 @@ func (s *CommonProvisionerSuite) assertProvisionerObservesConfigChanges(c *gc.C,
 	// Wait for the PA to load the new configuration. We wait for the change we expect
 	// like this because sometimes we pick up the initial harvest config (destroyed)
 	// rather than the one we change to (all).
-	received := []string{}
+	var received []string
 	timeout := time.After(coretesting.LongWait)
 	for {
 		select {
@@ -470,7 +472,7 @@ func (s *CommonProvisionerSuite) addMachine() (*state.Machine, error) {
 
 func (s *CommonProvisionerSuite) addMachineWithConstraints(cons constraints.Value) (*state.Machine, error) {
 	return s.BackingState.AddOneMachine(state.MachineTemplate{
-		Series:      series.LatestLts(),
+		Series:      supportedversion.SupportedLTS(),
 		Jobs:        []state.MachineJob{state.JobHostUnits},
 		Constraints: cons,
 	})
@@ -480,7 +482,7 @@ func (s *CommonProvisionerSuite) addMachines(number int) ([]*state.Machine, erro
 	templates := make([]state.MachineTemplate, number)
 	for i, _ := range templates {
 		templates[i] = state.MachineTemplate{
-			Series:      series.LatestLts(),
+			Series:      supportedversion.SupportedLTS(),
 			Jobs:        []state.MachineJob{state.JobHostUnits},
 			Constraints: s.defaultConstraints,
 		}
@@ -489,7 +491,7 @@ func (s *CommonProvisionerSuite) addMachines(number int) ([]*state.Machine, erro
 }
 
 func (s *CommonProvisionerSuite) enableHA(c *gc.C, n int) []*state.Machine {
-	changes, err := s.BackingState.EnableHA(n, s.defaultConstraints, series.LatestLts(), nil)
+	changes, err := s.BackingState.EnableHA(n, s.defaultConstraints, supportedversion.SupportedLTS(), nil, "")
 	c.Assert(err, jc.ErrorIsNil)
 	added := make([]*state.Machine, len(changes.Added))
 	for i, mid := range changes.Added {
@@ -585,6 +587,58 @@ func (s *ProvisionerSuite) TestPossibleTools(c *gc.C) {
 	)
 }
 
+var validCloudInitUserData = `
+packages:
+  - 'python-keystoneclient'
+  - 'python-glanceclient'
+preruncmd:
+  - mkdir /tmp/preruncmd
+  - mkdir /tmp/preruncmd2
+postruncmd:
+  - mkdir /tmp/postruncmd
+  - mkdir /tmp/postruncmd2
+package_upgrade: false
+`[1:]
+
+func (s *ProvisionerSuite) TestSetUpToStartMachine(c *gc.C) {
+	attrs := map[string]interface{}{
+		config.CloudInitUserDataKey: validCloudInitUserData,
+	}
+	s.Model.UpdateModelConfig(attrs, nil)
+
+	task := s.newProvisionerTask(
+		c,
+		config.HarvestAll,
+		s.Environ,
+		s.provisioner,
+		&mockDistributionGroupFinder{},
+		mockToolsFinder{},
+	)
+	defer workertest.CleanKill(c, task)
+
+	machine, err := s.addMachine()
+	c.Assert(err, jc.ErrorIsNil)
+
+	result, err := s.provisioner.Machines(machine.MachineTag())
+	c.Assert(err, gc.IsNil)
+	c.Assert(result, gc.HasLen, 1)
+	c.Assert(result[0].Err, gc.IsNil)
+	apiMachine := result[0].Machine
+
+	v, err := apiMachine.ModelAgentVersion()
+	c.Assert(err, jc.ErrorIsNil)
+
+	startInstanceParams, err := provisioner.SetupToStartMachine(task, apiMachine, v)
+	c.Assert(err, jc.ErrorIsNil)
+	cloudInitUserData := startInstanceParams.InstanceConfig.CloudInitUserData
+	c.Assert(cloudInitUserData, gc.DeepEquals, map[string]interface{}{
+		"packages":        []interface{}{"python-keystoneclient", "python-glanceclient"},
+		"preruncmd":       []interface{}{"mkdir /tmp/preruncmd", "mkdir /tmp/preruncmd2"},
+		"postruncmd":      []interface{}{"mkdir /tmp/postruncmd", "mkdir /tmp/postruncmd2"},
+		"package_upgrade": false},
+	)
+}
+
 func (s *ProvisionerSuite) TestProvisionerSetsErrorStatusWhenNoToolsAreAvailable(c *gc.C) {
 	p := s.newEnvironProvisioner(c)
 	defer workertest.CleanKill(c, p)
@@ -746,7 +800,7 @@ func (s *ProvisionerSuite) TestProvisioningDoesNotOccurForLXD(c *gc.C) {
 
 	// make a container on the machine we just created
 	template := state.MachineTemplate{
-		Series: series.LatestLts(),
+		Series: supportedversion.SupportedLTS(),
 		Jobs:   []state.MachineJob{state.JobHostUnits},
 	}
 	container, err := s.State.AddMachineInsideMachine(template, m.Id(), instance.LXD)
@@ -774,7 +828,7 @@ func (s *ProvisionerSuite) TestProvisioningDoesNotOccurForKVM(c *gc.C) {
 
 	// make a container on the machine we just created
 	template := state.MachineTemplate{
-		Series: series.LatestLts(),
+		Series: supportedversion.SupportedLTS(),
 		Jobs:   []state.MachineJob{state.JobHostUnits},
 	}
 	container, err := s.State.AddMachineInsideMachine(template, m.Id(), instance.KVM)
@@ -1053,7 +1107,7 @@ func (s *ProvisionerSuite) TestProvisioningMachinesFailsWithEmptySpaces(c *gc.C)
 
 func (s *CommonProvisionerSuite) addMachineWithRequestedVolumes(volumes []state.MachineVolumeParams, cons constraints.Value) (*state.Machine, error) {
 	return s.BackingState.AddOneMachine(state.MachineTemplate{
-		Series:      series.LatestLts(),
+		Series:      supportedversion.SupportedLTS(),
 		Jobs:        []state.MachineJob{state.JobHostUnits},
 		Constraints: cons,
 		Volumes:     volumes,
@@ -1201,7 +1255,7 @@ type mockDistributionGroupFinder struct {
 func (mock *mockDistributionGroupFinder) DistributionGroupByMachineId(tags ...names.MachineTag) ([]apiprovisioner.DistributionGroupResult, error) {
 	result := make([]apiprovisioner.DistributionGroupResult, len(tags))
 	if len(mock.groups) == 0 {
-		for i, _ := range tags {
+		for i := range tags {
 			result[i] = apiprovisioner.DistributionGroupResult{[]string{}, nil}
 		}
 	} else {
@@ -1450,6 +1504,9 @@ func (s *ProvisionerSuite) TestProvisionerRetriesTransientErrors(c *gc.C) {
 	task := s.newProvisionerTask(c, config.HarvestAll, e, s.provisioner, &mockDistributionGroupFinder{}, mockToolsFinder{})
 	defer workertest.CleanKill(c, task)
 
+	logger := loggo.GetLogger("juju.provisioner")
+	logger.SetLogLevel(loggo.TRACE)
+
 	// Provision some machines, some will be started first time,
 	// another will require retries.
 	m1, err := s.addMachine()
@@ -1466,11 +1523,13 @@ func (s *ProvisionerSuite) TestProvisionerRetriesTransientErrors(c *gc.C) {
 	// mockBroker will fail to start machine-3 several times;
 	// keep setting the transient flag to retry until the
 	// instance has started.
-	thatsAllFolks := make(chan struct{})
+	runSetStatusGoroutine := make(chan struct{})
+	setStatusGoroutineDone := make(chan struct{})
 	go func() {
+		defer close(setStatusGoroutineDone)
 		for {
 			select {
-			case <-thatsAllFolks:
+			case <-runSetStatusGoroutine:
 				return
 			case <-time.After(coretesting.ShortWait):
 				now := time.Now()
@@ -1480,13 +1539,20 @@ func (s *ProvisionerSuite) TestProvisionerRetriesTransientErrors(c *gc.C) {
 					Data:    map[string]interface{}{"transient": true},
 					Since:   &now,
 				}
+				logger.Infof("setting instance status provisioning error as transient for m3")
 				err := m3.SetInstanceStatus(sInfo)
-				c.Assert(err, jc.ErrorIsNil)
+				c.Check(err, jc.ErrorIsNil)
 			}
 		}
 	}()
 	s.checkStartInstance(c, m3)
-	close(thatsAllFolks)
+	close(runSetStatusGoroutine)
+
+	select {
+	case <-setStatusGoroutineDone:
+	case <-time.After(coretesting.LongWait):
+		c.Fatalf("status setting goroutine didn't stop")
+	}
 
 	// Machine 4 is never provisioned.
 	statusInfo, err := m4.InstanceStatus()
@@ -1645,11 +1711,11 @@ func (s *ProvisionerSuite) TestAvailabilityZoneMachinesStartMachinesWithDG(c *gc
 	// Per provider dummy, there will be 3 available availability zones.
 	s.PatchValue(&apiserverprovisioner.ErrorRetryWaitDelay, 5*time.Millisecond)
 	dgFinder := &mockDistributionGroupFinder{groups: map[names.MachineTag][]string{
-		names.NewMachineTag("1"): []string{"3, 4"},
-		names.NewMachineTag("2"): []string{},
-		names.NewMachineTag("3"): []string{"1, 4"},
-		names.NewMachineTag("4"): []string{"1, 3"},
-		names.NewMachineTag("5"): []string{},
+		names.NewMachineTag("1"): {"3, 4"},
+		names.NewMachineTag("2"): {},
+		names.NewMachineTag("3"): {"1, 4"},
+		names.NewMachineTag("4"): {"1, 3"},
+		names.NewMachineTag("5"): {},
 	}}
 
 	task := s.newProvisionerTask(c, config.HarvestDestroyed, s.Environ, s.provisioner, dgFinder, mockToolsFinder{})
@@ -1676,11 +1742,11 @@ func (s *ProvisionerSuite) TestAvailabilityZoneMachinesStartMachinesAZFailuresWi
 		},
 	}
 	dgFinder := &mockDistributionGroupFinder{groups: map[names.MachineTag][]string{
-		names.NewMachineTag("1"): []string{"4", "5"},
-		names.NewMachineTag("2"): []string{"3"},
-		names.NewMachineTag("3"): []string{"2"},
-		names.NewMachineTag("4"): []string{"1", "5"},
-		names.NewMachineTag("5"): []string{"1", "4"},
+		names.NewMachineTag("1"): {"4", "5"},
+		names.NewMachineTag("2"): {"3"},
+		names.NewMachineTag("3"): {"2"},
+		names.NewMachineTag("4"): {"1", "5"},
+		names.NewMachineTag("5"): {"1", "4"},
 	}}
 	retryStrategy := provisioner.NewRetryStrategy(0*time.Second, 2)
 	task := s.newProvisionerTaskWithRetryStrategy(c, config.HarvestDestroyed,
@@ -1701,10 +1767,10 @@ func (s *ProvisionerSuite) TestProvisioningMachinesSingleMachineDGFailure(c *gc.
 	// ensure the other machines are still provisioned.
 	dgFinder := &mockDistributionGroupFinder{
 		groups: map[names.MachineTag][]string{
-			names.NewMachineTag("2"): []string{"3", "5"},
-			names.NewMachineTag("3"): []string{"2", "5"},
-			names.NewMachineTag("4"): []string{"1"},
-			names.NewMachineTag("5"): []string{"2", "3"},
+			names.NewMachineTag("2"): {"3", "5"},
+			names.NewMachineTag("3"): {"2", "5"},
+			names.NewMachineTag("4"): {"1"},
+			names.NewMachineTag("5"): {"2", "3"},
 		},
 	}
 	task := s.newProvisionerTask(c, config.HarvestDestroyed, s.Environ, s.provisioner, dgFinder, mockToolsFinder{})
@@ -1806,7 +1872,7 @@ func (s *ProvisionerSuite) TestProvisioningMachinesClearAZFailures(c *gc.C) {
 	machine, err := s.addMachine()
 	c.Assert(err, jc.ErrorIsNil)
 	s.checkStartInstance(c, machine)
-	count := e.retryCount[machine.Id()]
+	count := e.getRetryCount(machine.Id())
 	c.Assert(count, gc.Equals, 3)
 	machineAZ, err := machine.AvailabilityZone()
 	c.Assert(err, jc.ErrorIsNil)
@@ -1824,11 +1890,11 @@ func (s *ProvisionerSuite) TestProvisioningMachinesDerivedAZ(c *gc.C) {
 			"5": {whenSucceed: 1, err: providercommon.ZoneIndependentError(errors.New("arf"))},
 		},
 		derivedAZ: map[string][]string{
-			"1": []string{"fail-zone"},
-			"2": []string{"zone4"},
-			"3": []string{"zone1", "zone4"},
-			"4": []string{"zone1"},
-			"5": []string{"zone3"},
+			"1": {"fail-zone"},
+			"2": {"zone4"},
+			"3": {"zone1", "zone4"},
+			"4": {"zone1"},
+			"5": {"zone3"},
 		},
 	}
 	retryStrategy := provisioner.NewRetryStrategy(5*time.Millisecond, 2)
@@ -1842,9 +1908,22 @@ func (s *ProvisionerSuite) TestProvisioningMachinesDerivedAZ(c *gc.C) {
 	mSucceed := machines[2:]
 
 	s.checkStartInstances(c, mSucceed)
-	c.Assert(e.retryCount[mFail[1].Id()], gc.Equals, 3)
-	c.Assert(e.retryCount[mSucceed[0].Id()], gc.Equals, 1)
-	c.Assert(e.retryCount[mSucceed[2].Id()], gc.Equals, 1)
+	c.Assert(e.getRetryCount(mSucceed[0].Id()), gc.Equals, 1)
+	c.Assert(e.getRetryCount(mSucceed[2].Id()), gc.Equals, 1)
+
+	// This synchronisation addresses a potential race condition.
+	// It can happen that upon successful return from checkStartInstances
+	// The machine(s) arranged for provisioning failure have not yet been
+	// retried the specified number of times; so we wait.
+	id := mFail[1].Id()
+	timeout := time.After(coretesting.LongWait)
+	for e.getRetryCount(id) < 3 {
+		select {
+		case <-timeout:
+			c.Fatalf("Failed provision of %q did not retry 3 times", id)
+		default:
+		}
+	}
 
 	_, err = mFail[0].InstanceId()
 	c.Assert(err, jc.Satisfies, errors.IsNotProvisioned)
@@ -1925,6 +2004,13 @@ func (b *mockBroker) StartInstance(args environs.StartInstanceParams) (*environs
 	return nil, returnError
 }
 
+func (b *mockBroker) getRetryCount(id string) int {
+	b.mu.Lock()
+	retries := b.retryCount[id]
+	b.mu.Unlock()
+	return retries
+}
+
 // ZonedEnviron necessary for provisionerTask.populateAvailabilityZoneMachines where
 // mockBroker used.
 
