@@ -22,6 +22,12 @@ import (
 
 const attachedVolumeId = "1"
 const needsInstanceVolumeId = "23"
+const noAttachmentVolumeId = "66"
+
+var (
+	releasingVolumeId     = "2"
+	releasingFilesystemId = "2"
+)
 
 var dyingVolumeAttachmentId = params.MachineStorageId{
 	MachineTag:    "machine-0",
@@ -169,8 +175,6 @@ func (v *mockVolumeAccessor) VolumeBlockDevices(ids []params.MachineStorageId) (
 func (v *mockVolumeAccessor) VolumeParams(volumes []names.VolumeTag) ([]params.VolumeParamsResult, error) {
 	var result []params.VolumeParamsResult
 	for _, tag := range volumes {
-		// Parameters are returned regardless of whether the volume
-		// exists; this is to support destruction.
 		volumeParams := params.VolumeParams{
 			VolumeTag: tag.String(),
 			Size:      1024,
@@ -182,18 +186,40 @@ func (v *mockVolumeAccessor) VolumeParams(volumes []names.VolumeTag) ([]params.V
 				"very": "fancy",
 			},
 		}
-		volumeParams.Attachment = &params.VolumeAttachmentParams{
-			VolumeTag:  tag.String(),
-			MachineTag: "machine-1",
-			InstanceId: string(v.provisionedMachines["machine-1"]),
-			Provider:   "dummy",
-			ReadOnly:   tag.String() == "volume-1",
+		if tag.Id() != noAttachmentVolumeId {
+			volumeParams.Attachment = &params.VolumeAttachmentParams{
+				VolumeTag:  tag.String(),
+				MachineTag: "machine-1",
+				InstanceId: string(v.provisionedMachines["machine-1"]),
+				Provider:   "dummy",
+				ReadOnly:   tag.String() == "volume-1",
+			}
 		}
 		result = append(result, params.VolumeParamsResult{Result: volumeParams})
 	}
 	return result, nil
 }
 
+func (v *mockVolumeAccessor) RemoveVolumeParams(volumes []names.VolumeTag) ([]params.RemoveVolumeParamsResult, error) {
+	var result []params.RemoveVolumeParamsResult
+	for _, tag := range volumes {
+		v, ok := v.provisionedVolumes[tag.String()]
+		if !ok {
+			result = append(result, params.RemoveVolumeParamsResult{
+				Error: &params.Error{Code: params.CodeNotProvisioned},
+			})
+			continue
+		}
+		volumeParams := params.RemoveVolumeParams{
+			Provider: "dummy",
+			VolumeId: v.Info.VolumeId,
+			Destroy:  tag.Id() != releasingVolumeId,
+		}
+		result = append(result, params.RemoveVolumeParamsResult{Result: volumeParams})
+	}
+	return result, nil
+}
+
 func (v *mockVolumeAccessor) VolumeAttachmentParams(ids []params.MachineStorageId) ([]params.VolumeAttachmentParamsResult, error) {
 	var result []params.VolumeAttachmentParamsResult
 	for _, id := range ids {
@@ -252,7 +278,7 @@ func (m *mockFilesystemAccessor) provisionFilesystem(tag names.FilesystemTag) pa
 	f := params.Filesystem{
 		FilesystemTag: tag.String(),
 		Info: params.FilesystemInfo{
-			FilesystemId: "vol-" + tag.Id(),
+			FilesystemId: "fs-" + tag.Id(),
 		},
 	}
 	m.provisionedFilesystems[tag.String()] = f
@@ -298,8 +324,6 @@ func (v *mockFilesystemAccessor) FilesystemAttachments(ids []params.MachineStora
 func (v *mockFilesystemAccessor) FilesystemParams(filesystems []names.FilesystemTag) ([]params.FilesystemParamsResult, error) {
 	results := make([]params.FilesystemParamsResult, len(filesystems))
 	for i, tag := range filesystems {
-		// Parameters are returned regardless of whether the filesystem
-		// exists; this is to support destruction.
 		filesystemParams := params.FilesystemParams{
 			FilesystemTag: tag.String(),
 			Size:          1024,
@@ -318,6 +342,26 @@ func (v *mockFilesystemAccessor) FilesystemParams(filesystems []names.Filesystem
 	return results, nil
 }
 
+func (v *mockFilesystemAccessor) RemoveFilesystemParams(filesystems []names.FilesystemTag) ([]params.RemoveFilesystemParamsResult, error) {
+	results := make([]params.RemoveFilesystemParamsResult, len(filesystems))
+	for i, tag := range filesystems {
+		f, ok := v.provisionedFilesystems[tag.String()]
+		if !ok {
+			results = append(results, params.RemoveFilesystemParamsResult{
+				Error: &params.Error{Code: params.CodeNotProvisioned},
+			})
+			continue
+		}
+		filesystemParams := params.RemoveFilesystemParams{
+			Provider:     "dummy",
+			FilesystemId: f.Info.FilesystemId,
+			Destroy:      tag.Id() != releasingFilesystemId,
+		}
+		results[i] = params.RemoveFilesystemParamsResult{Result: filesystemParams}
+	}
+	return results, nil
+}
+
 func (f *mockFilesystemAccessor) FilesystemAttachmentParams(ids []params.MachineStorageId) ([]params.FilesystemAttachmentParamsResult, error) {
 	var result []params.FilesystemAttachmentParamsResult
 	for _, id := range ids {
@@ -430,7 +474,9 @@ type dummyProvider struct {
 	detachVolumesFunc            func([]storage.VolumeAttachmentParams) ([]error, error)
 	detachFilesystemsFunc        func([]storage.FilesystemAttachmentParams) ([]error, error)
 	destroyVolumesFunc           func([]string) ([]error, error)
+	releaseVolumesFunc           func([]string) ([]error, error)
 	destroyFilesystemsFunc       func([]string) ([]error, error)
+	releaseFilesystemsFunc       func([]string) ([]error, error)
 	validateVolumeParamsFunc     func(storage.VolumeParams) error
 	validateFilesystemParamsFunc func(storage.FilesystemParams) error
 }
@@ -506,6 +552,14 @@ func (s *dummyVolumeSource) DestroyVolumes(volumeIds []string) ([]error, error)
 	return make([]error, len(volumeIds)), nil
 }
 
+// ReleaseVolumes destroys volumes.
+func (s *dummyVolumeSource) ReleaseVolumes(volumeIds []string) ([]error, error) {
+	if s.provider.releaseVolumesFunc != nil {
+		return s.provider.releaseVolumesFunc(volumeIds)
+	}
+	return make([]error, len(volumeIds)), nil
+}
+
 // AttachVolumes attaches volumes to machines.
 func (s *dummyVolumeSource) AttachVolumes(params []storage.VolumeAttachmentParams) ([]storage.AttachVolumesResult, error) {
 	if s.provider != nil && s.provider.attachVolumesFunc != nil {
@@ -578,6 +632,14 @@ func (s *dummyFilesystemSource) DestroyFilesystems(filesystemIds []string) ([]er
 	return make([]error, len(filesystemIds)), nil
 }
 
+// ReleaseFilesystems destroys filesystems.
+func (s *dummyFilesystemSource) ReleaseFilesystems(filesystemIds []string) ([]error, error) {
+	if s.provider.releaseFilesystemsFunc != nil {
+		return s.provider.releaseFilesystemsFunc(filesystemIds)
+	}
+	return make([]error, len(filesystemIds)), nil
+}
+
 // AttachFilesystems attaches filesystems to machines.
 func (s *dummyFilesystemSource) AttachFilesystems(params []storage.FilesystemAttachmentParams) ([]storage.AttachFilesystemsResult, error) {
 	if s.provider != nil && s.provider.attachFilesystemsFunc != nil {
@@ -643,6 +705,10 @@ func (s *mockManagedFilesystemSource) DestroyFilesystems(filesystemIds []string)
 	return make([]error, len(filesystemIds)), nil
 }
 
+func (s *mockManagedFilesystemSource) ReleaseFilesystems(filesystemIds []string) ([]error, error) {
+	return make([]error, len(filesystemIds)), nil
+}
+
 func (s *mockManagedFilesystemSource) AttachFilesystems(args []storage.FilesystemAttachmentParams) ([]storage.AttachFilesystemsResult, error) {
 	results := make([]storage.AttachFilesystemsResult, len(args))
 	for i, arg := range args {
