@@ -5,11 +5,11 @@ package undertaker
 
 import (
 	"github.com/juju/errors"
+	worker "gopkg.in/juju/worker.v1"
 
 	"github.com/juju/juju/api/base"
 	"github.com/juju/juju/api/undertaker"
 	"github.com/juju/juju/api/watcher"
-	"github.com/juju/juju/worker"
 )
 
 // NewFacade creates a Facade from a base.APICaller, by calling the
