@@ -6,12 +6,15 @@ package hook_test
 import (
 	jc "github.com/juju/testing/checkers"
 	gc "gopkg.in/check.v1"
-	"gopkg.in/juju/charm.v4/hooks"
+	"gopkg.in/juju/charm.v6/hooks"
 
+	"github.com/juju/juju/testing"
 	"github.com/juju/juju/worker/uniter/hook"
 )
 
-type InfoSuite struct{}
+type InfoSuite struct {
+	testing.BaseSuite
+}
 
 var _ = gc.Suite(&InfoSuite{})
 
@@ -37,17 +40,16 @@ var validateTests = []struct {
 	{hook.Info{Kind: hooks.ConfigChanged}, ""},
 	{hook.Info{Kind: hooks.CollectMetrics}, ""},
 	{hook.Info{Kind: hooks.MeterStatusChanged}, ""},
-	{
-		hook.Info{Kind: hooks.Action},
-		`action id "" cannot be parsed as an action tag`,
-	},
-	{hook.Info{Kind: hooks.Action, ActionId: "badadded-0123-4567-89ab-cdef01234567"}, ""},
+	{hook.Info{Kind: hooks.Action}, "hooks.Kind Action is deprecated"},
 	{hook.Info{Kind: hooks.UpgradeCharm}, ""},
 	{hook.Info{Kind: hooks.Stop}, ""},
 	{hook.Info{Kind: hooks.RelationJoined, RemoteUnit: "x"}, ""},
 	{hook.Info{Kind: hooks.RelationChanged, RemoteUnit: "x"}, ""},
 	{hook.Info{Kind: hooks.RelationDeparted, RemoteUnit: "x"}, ""},
 	{hook.Info{Kind: hooks.RelationBroken}, ""},
+	{hook.Info{Kind: hooks.StorageAttached}, `invalid storage ID ""`},
+	{hook.Info{Kind: hooks.StorageAttached, StorageId: "data/0"}, ""},
+	{hook.Info{Kind: hooks.StorageDetaching, StorageId: "data/0"}, ""},
 }
 
 func (s *InfoSuite) TestValidate(c *gc.C) {
