@@ -8,10 +8,9 @@ import (
 	"github.com/juju/testing"
 	jc "github.com/juju/testing/checkers"
 	gc "gopkg.in/check.v1"
-	corecharm "gopkg.in/juju/charm.v6-unstable"
-	"gopkg.in/juju/charm.v6-unstable/hooks"
+	corecharm "gopkg.in/juju/charm.v6"
+	"gopkg.in/juju/charm.v6/hooks"
 
-	"github.com/juju/juju/worker/uniter/charm"
 	"github.com/juju/juju/worker/uniter/hook"
 	"github.com/juju/juju/worker/uniter/operation"
 )
@@ -303,32 +302,6 @@ func (s *DeploySuite) TestPrepareSuccess_Upgrade_PreserveNoHook(c *gc.C) {
 	}
 }
 
-func (s *DeploySuite) testExecuteConflictError(c *gc.C, newDeploy newDeploy) {
-	callbacks := NewDeployCallbacks()
-	deployer := &MockDeployer{
-		MockNotifyRevert:   &MockNoArgs{},
-		MockNotifyResolved: &MockNoArgs{},
-		MockStage:          &MockStage{},
-		MockDeploy:         &MockNoArgs{err: charm.ErrConflict},
-	}
-	factory := operation.NewFactory(operation.FactoryParams{
-		Deployer:  deployer,
-		Callbacks: callbacks,
-	})
-	charmURL := curl("cs:quantal/nyancat-4")
-	op, err := newDeploy(factory, charmURL)
-	c.Assert(err, jc.ErrorIsNil)
-	_, err = op.Prepare(operation.State{})
-	c.Assert(err, jc.ErrorIsNil)
-
-	newState, err := op.Execute(operation.State{})
-	c.Check(newState, gc.IsNil)
-	c.Check(err, gc.ErrorMatches, "cannot deploy charm cs:quantal/nyancat-4")
-	ok := operation.IsDeployConflictError(err)
-	c.Check(ok, jc.IsTrue)
-	c.Check(deployer.MockDeploy.called, jc.IsTrue)
-}
-
 func (s *DeploySuite) TestExecuteConflictError_Install(c *gc.C) {
 	s.testExecuteError(c, (operation.Factory).NewInstall)
 }
