@@ -8,8 +8,8 @@ import (
 	jc "github.com/juju/testing/checkers"
 	utilexec "github.com/juju/utils/exec"
 	gc "gopkg.in/check.v1"
-	corecharm "gopkg.in/juju/charm.v5"
-	"gopkg.in/juju/charm.v5/hooks"
+	corecharm "gopkg.in/juju/charm.v6"
+	"gopkg.in/juju/charm.v6/hooks"
 
 	"github.com/juju/juju/worker/uniter/hook"
 	"github.com/juju/juju/worker/uniter/operation"
@@ -28,7 +28,7 @@ func (s *FactorySuite) SetUpTest(c *gc.C) {
 	// verifying that inadequate args to the factory methods will produce
 	// the expected errors; and that the results of same get a string
 	// representation that does not depend on the factory attributes.
-	s.factory = operation.NewFactory(nil, nil, nil, nil, nil)
+	s.factory = operation.NewFactory(operation.FactoryParams{})
 }
 
 func (s *FactorySuite) testNewDeployError(c *gc.C, newDeploy newDeploy) {
@@ -70,14 +70,14 @@ func (s *FactorySuite) TestNewUpgradeString(c *gc.C) {
 func (s *FactorySuite) TestNewRevertUpgradeString(c *gc.C) {
 	s.testNewDeployString(c,
 		(operation.Factory).NewRevertUpgrade,
-		"clear resolved flag and switch upgrade to",
+		"switch upgrade to",
 	)
 }
 
 func (s *FactorySuite) TestNewResolvedUpgradeString(c *gc.C) {
 	s.testNewDeployString(c,
 		(operation.Factory).NewResolvedUpgrade,
-		"clear resolved flag and continue upgrade to",
+		"continue upgrade to",
 	)
 }
 
@@ -179,10 +179,6 @@ func (s *FactorySuite) TestNewHookError_Run(c *gc.C) {
 	s.testNewHookError(c, (operation.Factory).NewRunHook)
 }
 
-func (s *FactorySuite) TestNewHookError_Retry(c *gc.C) {
-	s.testNewHookError(c, (operation.Factory).NewRetryHook)
-}
-
 func (s *FactorySuite) TestNewHookError_Skip(c *gc.C) {
 	s.testNewHookError(c, (operation.Factory).NewSkipHook)
 }
@@ -193,15 +189,6 @@ func (s *FactorySuite) TestNewHookString_Run(c *gc.C) {
 	c.Check(op.String(), gc.Equals, "run install hook")
 }
 
-func (s *FactorySuite) TestNewHookString_Retry(c *gc.C) {
-	op, err := s.factory.NewRetryHook(hook.Info{
-		Kind:       hooks.RelationBroken,
-		RelationId: 123,
-	})
-	c.Check(err, jc.ErrorIsNil)
-	c.Check(op.String(), gc.Equals, "clear resolved flag and run relation-broken (123) hook")
-}
-
 func (s *FactorySuite) TestNewHookString_Skip(c *gc.C) {
 	op, err := s.factory.NewSkipHook(hook.Info{
 		Kind:       hooks.RelationJoined,
@@ -209,13 +196,7 @@ func (s *FactorySuite) TestNewHookString_Skip(c *gc.C) {
 		RelationId: 123,
 	})
 	c.Check(err, jc.ErrorIsNil)
-	c.Check(op.String(), gc.Equals, "clear resolved flag and skip run relation-joined (123; foo/22) hook")
-}
-
-func (s *FactorySuite) TestNewUpdateRelationsString(c *gc.C) {
-	op, err := s.factory.NewUpdateRelations([]int{1, 2, 3})
-	c.Check(err, jc.ErrorIsNil)
-	c.Check(op.String(), gc.Equals, "update relations [1 2 3]")
+	c.Check(op.String(), gc.Equals, "skip run relation-joined (123; foo/22) hook")
 }
 
 func (s *FactorySuite) TestNewAcceptLeadershipString(c *gc.C) {
