@@ -8,11 +8,12 @@ import (
 	"github.com/juju/testing"
 	jc "github.com/juju/testing/checkers"
 	gc "gopkg.in/check.v1"
-	"gopkg.in/juju/charm.v5/hooks"
+	"gopkg.in/juju/charm.v6/hooks"
 
 	"github.com/juju/juju/worker/uniter/hook"
 	"github.com/juju/juju/worker/uniter/operation"
 	"github.com/juju/juju/worker/uniter/runner"
+	"github.com/juju/juju/worker/uniter/runner/context"
 )
 
 type RunActionSuite struct {
@@ -29,7 +30,10 @@ func (s *RunActionSuite) TestPrepareErrorBadActionAndFailSucceeds(c *gc.C) {
 	callbacks := &RunActionCallbacks{
 		MockFailAction: &MockFailAction{err: errors.New("squelch")},
 	}
-	factory := operation.NewFactory(nil, runnerFactory, callbacks, nil, nil)
+	factory := operation.NewFactory(operation.FactoryParams{
+		RunnerFactory: runnerFactory,
+		Callbacks:     callbacks,
+	})
 	op, err := factory.NewAction(someActionId)
 	c.Assert(err, jc.ErrorIsNil)
 
@@ -49,7 +53,10 @@ func (s *RunActionSuite) TestPrepareErrorBadActionAndFailErrors(c *gc.C) {
 	callbacks := &RunActionCallbacks{
 		MockFailAction: &MockFailAction{},
 	}
-	factory := operation.NewFactory(nil, runnerFactory, callbacks, nil, nil)
+	factory := operation.NewFactory(operation.FactoryParams{
+		RunnerFactory: runnerFactory,
+		Callbacks:     callbacks,
+	})
 	op, err := factory.NewAction(someActionId)
 	c.Assert(err, jc.ErrorIsNil)
 
@@ -65,7 +72,9 @@ func (s *RunActionSuite) TestPrepareErrorActionNotAvailable(c *gc.C) {
 	runnerFactory := &MockRunnerFactory{
 		MockNewActionRunner: &MockNewActionRunner{err: runner.ErrActionNotAvailable},
 	}
-	factory := operation.NewFactory(nil, runnerFactory, nil, nil, nil)
+	factory := operation.NewFactory(operation.FactoryParams{
+		RunnerFactory: runnerFactory,
+	})
 	op, err := factory.NewAction(someActionId)
 	c.Assert(err, jc.ErrorIsNil)
 
@@ -79,7 +88,9 @@ func (s *RunActionSuite) TestPrepareErrorOther(c *gc.C) {
 	runnerFactory := &MockRunnerFactory{
 		MockNewActionRunner: &MockNewActionRunner{err: errors.New("foop")},
 	}
-	factory := operation.NewFactory(nil, runnerFactory, nil, nil, nil)
+	factory := operation.NewFactory(operation.FactoryParams{
+		RunnerFactory: runnerFactory,
+	})
 	op, err := factory.NewAction(someActionId)
 	c.Assert(err, jc.ErrorIsNil)
 
@@ -89,9 +100,54 @@ func (s *RunActionSuite) TestPrepareErrorOther(c *gc.C) {
 	c.Assert(*runnerFactory.MockNewActionRunner.gotActionId, gc.Equals, someActionId)
 }
 
+func (s *RunActionSuite) TestPrepareCtxCalled(c *gc.C) {
+	ctx := &MockContext{actionData: &context.ActionData{Name: "some-action-name"}}
+	runnerFactory := &MockRunnerFactory{
+		MockNewActionRunner: &MockNewActionRunner{
+			runner: &MockRunner{
+				context: ctx,
+			},
+		},
+	}
+	factory := operation.NewFactory(operation.FactoryParams{
+		RunnerFactory: runnerFactory,
+	})
+	op, err := factory.NewAction(someActionId)
+	c.Assert(err, jc.ErrorIsNil)
+
+	newState, err := op.Prepare(operation.State{})
+	c.Assert(err, jc.ErrorIsNil)
+	c.Assert(newState, gc.NotNil)
+	ctx.CheckCall(c, 0, "Prepare")
+}
+
+func (s *RunActionSuite) TestPrepareCtxError(c *gc.C) {
+	ctx := &MockContext{actionData: &context.ActionData{Name: "some-action-name"}}
+	ctx.SetErrors(errors.New("ctx prepare error"))
+	runnerFactory := &MockRunnerFactory{
+		MockNewActionRunner: &MockNewActionRunner{
+			runner: &MockRunner{
+				context: ctx,
+			},
+		},
+	}
+	factory := operation.NewFactory(operation.FactoryParams{
+		RunnerFactory: runnerFactory,
+	})
+	op, err := factory.NewAction(someActionId)
+	c.Assert(err, jc.ErrorIsNil)
+
+	newState, err := op.Prepare(operation.State{})
+	c.Assert(err, gc.ErrorMatches, `ctx prepare error`)
+	c.Assert(newState, gc.IsNil)
+	ctx.CheckCall(c, 0, "Prepare")
+}
+
 func (s *RunActionSuite) TestPrepareSuccessCleanState(c *gc.C) {
 	runnerFactory := NewRunActionRunnerFactory(errors.New("should not call"))
-	factory := operation.NewFactory(nil, runnerFactory, nil, nil, nil)
+	factory := operation.NewFactory(operation.FactoryParams{
+		RunnerFactory: runnerFactory,
+	})
 	op, err := factory.NewAction(someActionId)
 	c.Assert(err, jc.ErrorIsNil)
 
@@ -107,19 +163,20 @@ func (s *RunActionSuite) TestPrepareSuccessCleanState(c *gc.C) {
 
 func (s *RunActionSuite) TestPrepareSuccessDirtyState(c *gc.C) {
 	runnerFactory := NewRunActionRunnerFactory(errors.New("should not call"))
-	factory := operation.NewFactory(nil, runnerFactory, nil, nil, nil)
+	factory := operation.NewFactory(operation.FactoryParams{
+		RunnerFactory: runnerFactory,
+	})
 	op, err := factory.NewAction(someActionId)
 	c.Assert(err, jc.ErrorIsNil)
 
 	newState, err := op.Prepare(overwriteState)
 	c.Assert(err, jc.ErrorIsNil)
 	c.Assert(newState, jc.DeepEquals, &operation.State{
-		Kind:               operation.RunAction,
-		Step:               operation.Pending,
-		ActionId:           &someActionId,
-		Started:            true,
-		CollectMetricsTime: 1234567,
-		Hook:               &hook.Info{Kind: hooks.Install},
+		Kind:     operation.RunAction,
+		Step:     operation.Pending,
+		ActionId: &someActionId,
+		Started:  true,
+		Hook:     &hook.Info{Kind: hooks.Install},
 	})
 	c.Assert(*runnerFactory.MockNewActionRunner.gotActionId, gc.Equals, someActionId)
 }
@@ -140,12 +197,11 @@ func (s *RunActionSuite) TestExecuteSuccess(c *gc.C) {
 		description: "preserves appropriate fields",
 		before:      overwriteState,
 		after: operation.State{
-			Kind:               operation.RunAction,
-			Step:               operation.Done,
-			ActionId:           &someActionId,
-			Hook:               &hook.Info{Kind: hooks.Install},
-			Started:            true,
-			CollectMetricsTime: 1234567,
+			Kind:     operation.RunAction,
+			Step:     operation.Done,
+			ActionId: &someActionId,
+			Hook:     &hook.Info{Kind: hooks.Install},
+			Started:  true,
 		},
 	}}
 
@@ -153,7 +209,10 @@ func (s *RunActionSuite) TestExecuteSuccess(c *gc.C) {
 		c.Logf("test %d: %s", i, test.description)
 		runnerFactory := NewRunActionRunnerFactory(nil)
 		callbacks := &RunActionCallbacks{}
-		factory := operation.NewFactory(nil, runnerFactory, callbacks, nil, nil)
+		factory := operation.NewFactory(operation.FactoryParams{
+			RunnerFactory: runnerFactory,
+			Callbacks:     callbacks,
+		})
 		op, err := factory.NewAction(someActionId)
 		c.Assert(err, jc.ErrorIsNil)
 		midState, err := op.Prepare(test.before)
@@ -182,42 +241,38 @@ func (s *RunActionSuite) TestCommit(c *gc.C) {
 	}, {
 		description: "preserves only appropriate fields, no hook",
 		before: operation.State{
-			Kind:               operation.Continue,
-			Step:               operation.Pending,
-			Started:            true,
-			CollectMetricsTime: 1234567,
-			CharmURL:           curl("cs:quantal/wordpress-2"),
-			ActionId:           &randomActionId,
+			Kind:     operation.Continue,
+			Step:     operation.Pending,
+			Started:  true,
+			CharmURL: curl("cs:quantal/wordpress-2"),
+			ActionId: &randomActionId,
 		},
 		after: operation.State{
-			Kind:               operation.Continue,
-			Step:               operation.Pending,
-			Started:            true,
-			CollectMetricsTime: 1234567,
+			Kind:    operation.Continue,
+			Step:    operation.Pending,
+			Started: true,
 		},
 	}, {
 		description: "preserves only appropriate fields, with hook",
 		before: operation.State{
-			Kind:               operation.Continue,
-			Step:               operation.Pending,
-			Started:            true,
-			CollectMetricsTime: 1234567,
-			CharmURL:           curl("cs:quantal/wordpress-2"),
-			ActionId:           &randomActionId,
-			Hook:               &hook.Info{Kind: hooks.Install},
+			Kind:     operation.Continue,
+			Step:     operation.Pending,
+			Started:  true,
+			CharmURL: curl("cs:quantal/wordpress-2"),
+			ActionId: &randomActionId,
+			Hook:     &hook.Info{Kind: hooks.Install},
 		},
 		after: operation.State{
-			Kind:               operation.RunHook,
-			Step:               operation.Pending,
-			Hook:               &hook.Info{Kind: hooks.Install},
-			Started:            true,
-			CollectMetricsTime: 1234567,
+			Kind:    operation.RunHook,
+			Step:    operation.Pending,
+			Hook:    &hook.Info{Kind: hooks.Install},
+			Started: true,
 		},
 	}}
 
 	for i, test := range stateChangeTests {
 		c.Logf("test %d: %s", i, test.description)
-		factory := operation.NewFactory(nil, nil, nil, nil, nil)
+		factory := operation.NewFactory(operation.FactoryParams{})
 		op, err := factory.NewAction(someActionId)
 		c.Assert(err, jc.ErrorIsNil)
 
@@ -227,7 +282,7 @@ func (s *RunActionSuite) TestCommit(c *gc.C) {
 }
 
 func (s *RunActionSuite) TestNeedsGlobalMachineLock(c *gc.C) {
-	factory := operation.NewFactory(nil, nil, nil, nil, nil)
+	factory := operation.NewFactory(operation.FactoryParams{})
 	op, err := factory.NewAction(someActionId)
 	c.Assert(err, jc.ErrorIsNil)
 	c.Assert(op.NeedsGlobalMachineLock(), jc.IsTrue)
