@@ -9,6 +9,7 @@ import (
 	"github.com/juju/errors"
 
 	"github.com/juju/juju/worker/uniter/runner"
+	"github.com/juju/juju/worker/uniter/runner/context"
 )
 
 type runCommands struct {
@@ -39,7 +40,7 @@ func (rc *runCommands) String() string {
 // Prepare ensures the commands can be run. It never returns a state change.
 // Prepare is part of the Operation interface.
 func (rc *runCommands) Prepare(state State) (*State, error) {
-	rnr, err := rc.runnerFactory.NewCommandRunner(runner.CommandInfo{
+	rnr, err := rc.runnerFactory.NewCommandRunner(context.CommandInfo{
 		RelationId:      rc.args.RelationId,
 		RemoteUnitName:  rc.args.RemoteUnitName,
 		ForceRemoteUnit: rc.args.ForceRemoteUnit,
@@ -47,7 +48,12 @@ func (rc *runCommands) Prepare(state State) (*State, error) {
 	if err != nil {
 		return nil, err
 	}
+	err = rnr.Context().Prepare()
+	if err != nil {
+		return nil, errors.Trace(err)
+	}
 	rc.runner = rnr
+
 	return nil, nil
 }
 
@@ -55,20 +61,23 @@ func (rc *runCommands) Prepare(state State) (*State, error) {
 // state change.
 // Execute is part of the Operation interface.
 func (rc *runCommands) Execute(state State) (*State, error) {
+	logger.Tracef("run commands: %s", rc)
 	if err := rc.callbacks.SetExecutingStatus("running commands"); err != nil {
 		return nil, errors.Trace(err)
 	}
 
 	response, err := rc.runner.RunCommands(rc.args.Commands)
 	switch err {
-	case runner.ErrRequeueAndReboot:
+	case context.ErrRequeueAndReboot:
 		logger.Warningf("cannot requeue external commands")
 		fallthrough
-	case runner.ErrReboot:
+	case context.ErrReboot:
+		rc.sendResponse(response, nil)
 		err = ErrNeedsReboot
+	default:
+		rc.sendResponse(response, err)
 	}
-	rc.sendResponse(response, err)
-	return nil, nil
+	return nil, err
 }
 
 // Commit does nothing.
