@@ -13,14 +13,15 @@ import (
 	"github.com/juju/testing"
 	jc "github.com/juju/testing/checkers"
 	gc "gopkg.in/check.v1"
-	"gopkg.in/juju/charm.v6-unstable"
-	"gopkg.in/juju/charm.v6-unstable/hooks"
+	"gopkg.in/juju/charm.v6"
+	"gopkg.in/juju/charm.v6/hooks"
 	"gopkg.in/juju/names.v2"
 
 	apitesting "github.com/juju/juju/api/base/testing"
 	"github.com/juju/juju/api/uniter"
 	"github.com/juju/juju/apiserver/common"
 	"github.com/juju/juju/apiserver/params"
+	"github.com/juju/juju/core/leadership"
 	"github.com/juju/juju/state/multiwatcher"
 	coretesting "github.com/juju/juju/testing"
 	"github.com/juju/juju/worker/uniter/hook"
@@ -28,6 +29,7 @@ import (
 	"github.com/juju/juju/worker/uniter/relation"
 	"github.com/juju/juju/worker/uniter/remotestate"
 	"github.com/juju/juju/worker/uniter/resolver"
+	"github.com/juju/juju/worker/uniter/runner/context"
 )
 
 /*
@@ -46,8 +48,9 @@ return curated values for each API call.
 type relationsSuite struct {
 	coretesting.BaseSuite
 
-	stateDir     string
-	relationsDir string
+	stateDir              string
+	relationsDir          string
+	leadershipContextFunc relation.LeadershipContextFunc
 }
 
 var _ = gc.Suite(&relationsSuite{})
@@ -78,7 +81,7 @@ func mockAPICaller(c *gc.C, callNumber *int32, apiCalls ...apiCall) apitesting.A
 			c.Check(index < len(apiCalls), jc.IsTrue)
 			call := apiCalls[index]
 			c.Logf("request %d, %s", index, request)
-			c.Check(version, gc.Equals, 6)
+			c.Check(version, gc.Equals, 7)
 			c.Check(id, gc.Equals, "")
 			c.Check(request, gc.Equals, call.request)
 			c.Check(arg, jc.DeepEquals, call.args)
@@ -94,6 +97,15 @@ func mockAPICaller(c *gc.C, callNumber *int32, apiCalls ...apiCall) apitesting.A
 	return apiCaller
 }
 
+type stubLeadershipContext struct {
+	context.LeadershipContext
+	isLeader bool
+}
+
+func (stub *stubLeadershipContext) IsLeader() (bool, error) {
+	return stub.isLeader, nil
+}
+
 var minimalMetadata = `
 name: wordpress
 summary: "test"
@@ -109,6 +121,9 @@ func (s *relationsSuite) SetUpTest(c *gc.C) {
 	err = ioutil.WriteFile(filepath.Join(s.stateDir, "metadata.yaml"), []byte(minimalMetadata), 0755)
 	c.Assert(err, jc.ErrorIsNil)
 	s.relationsDir = filepath.Join(c.MkDir(), "relations")
+	s.leadershipContextFunc = func(accessor context.LeadershipSettingsAccessor, tracker leadership.Tracker) context.LeadershipContext {
+		return &stubLeadershipContext{isLeader: true}
+	}
 }
 
 func assertNumCalls(c *gc.C, numCalls *int32, expected int32) {
@@ -121,14 +136,22 @@ func (s *relationsSuite) setupRelations(c *gc.C) relation.Relations {
 	abort := make(chan struct{})
 
 	var numCalls int32
-	unitEntity := params.Entities{Entities: []params.Entity{params.Entity{Tag: "unit-wordpress-0"}}}
+	unitEntity := params.Entities{Entities: []params.Entity{{Tag: "unit-wordpress-0"}}}
 	apiCaller := mockAPICaller(c, &numCalls,
-		uniterAPICall("Life", unitEntity, params.LifeResults{Results: []params.LifeResult{{Life: params.Alive}}}, nil),
+		uniterAPICall("Refresh", unitEntity, params.UnitRefreshResults{Results: []params.UnitRefreshResult{{Life: params.Alive, Resolved: params.ResolvedNone, Series: "quantal"}}}, nil),
 		uniterAPICall("GetPrincipal", unitEntity, params.StringBoolResults{Results: []params.StringBoolResult{{Result: "", Ok: false}}}, nil),
-		uniterAPICall("JoinedRelations", unitEntity, params.StringsResults{Results: []params.StringsResult{{Result: []string{}}}}, nil),
+		uniterAPICall("RelationsStatus", unitEntity, params.RelationUnitStatusResults{Results: []params.RelationUnitStatusResult{{RelationResults: []params.RelationUnitStatus{}}}}, nil),
 	)
 	st := uniter.NewState(apiCaller, unitTag)
-	r, err := relation.NewRelations(st, unitTag, s.stateDir, s.relationsDir, abort)
+	r, err := relation.NewRelations(
+		relation.RelationsConfig{
+			State:                st,
+			UnitTag:              unitTag,
+			CharmDir:             s.stateDir,
+			RelationsDir:         s.relationsDir,
+			NewLeadershipContext: s.leadershipContextFunc,
+			Abort:                abort,
+		})
 	c.Assert(err, jc.ErrorIsNil)
 	assertNumCalls(c, &numCalls, 3)
 	return r
@@ -140,12 +163,15 @@ func (s *relationsSuite) TestNewRelationsNoRelations(c *gc.C) {
 	c.Assert(r.GetInfo(), gc.HasLen, 0)
 }
 
-func (s *relationsSuite) TestNewRelationsWithExistingRelations(c *gc.C) {
+func (s *relationsSuite) assertNewRelationsWithExistingRelations(c *gc.C, isLeader bool) {
 	unitTag := names.NewUnitTag("wordpress/0")
 	abort := make(chan struct{})
+	s.leadershipContextFunc = func(accessor context.LeadershipSettingsAccessor, tracker leadership.Tracker) context.LeadershipContext {
+		return &stubLeadershipContext{isLeader: isLeader}
+	}
 
 	var numCalls int32
-	unitEntity := params.Entities{Entities: []params.Entity{params.Entity{Tag: "unit-wordpress-0"}}}
+	unitEntity := params.Entities{Entities: []params.Entity{{Tag: "unit-wordpress-0"}}}
 	relationUnits := params.RelationUnits{RelationUnits: []params.RelationUnit{
 		{Relation: "relation-wordpress.db#mysql.db", Unit: "unit-wordpress-0"},
 	}}
@@ -161,20 +187,39 @@ func (s *relationsSuite) TestNewRelationsWithExistingRelations(c *gc.C) {
 				}},
 		},
 	}
+	relationStatus := params.RelationStatusArgs{Args: []params.RelationStatusArg{{
+		RelationId: 1,
+		Status:     params.Joined,
+	}}}
 
-	apiCaller := mockAPICaller(c, &numCalls,
-		uniterAPICall("Life", unitEntity, params.LifeResults{Results: []params.LifeResult{{Life: params.Alive}}}, nil),
+	apiCalls := []apiCall{
+		uniterAPICall("Refresh", unitEntity, params.UnitRefreshResults{Results: []params.UnitRefreshResult{{Life: params.Alive, Resolved: params.ResolvedNone, Series: "quantal"}}}, nil),
 		uniterAPICall("GetPrincipal", unitEntity, params.StringBoolResults{Results: []params.StringBoolResult{{Result: "", Ok: false}}}, nil),
-		uniterAPICall("JoinedRelations", unitEntity, params.StringsResults{Results: []params.StringsResult{{Result: []string{"relation-wordpress:db mysql:db"}}}}, nil),
+		uniterAPICall("RelationsStatus", unitEntity, params.RelationUnitStatusResults{Results: []params.RelationUnitStatusResult{
+			{RelationResults: []params.RelationUnitStatus{{RelationTag: "relation-wordpress:db mysql:db", InScope: true}}}}}, nil),
 		uniterAPICall("Relation", relationUnits, relationResults, nil),
 		uniterAPICall("Relation", relationUnits, relationResults, nil),
 		uniterAPICall("Watch", unitEntity, params.NotifyWatchResults{Results: []params.NotifyWatchResult{{NotifyWatcherId: "1"}}}, nil),
 		uniterAPICall("EnterScope", relationUnits, params.ErrorResults{Results: []params.ErrorResult{{}}}, nil),
-	)
+	}
+	if isLeader {
+		apiCalls = append(apiCalls,
+			uniterAPICall("SetRelationStatus", relationStatus, noErrorResult, nil),
+		)
+	}
+	apiCaller := mockAPICaller(c, &numCalls, apiCalls...)
 	st := uniter.NewState(apiCaller, unitTag)
-	r, err := relation.NewRelations(st, unitTag, s.stateDir, s.relationsDir, abort)
+	r, err := relation.NewRelations(
+		relation.RelationsConfig{
+			State:                st,
+			UnitTag:              unitTag,
+			CharmDir:             s.stateDir,
+			RelationsDir:         s.relationsDir,
+			NewLeadershipContext: s.leadershipContextFunc,
+			Abort:                abort,
+		})
 	c.Assert(err, jc.ErrorIsNil)
-	assertNumCalls(c, &numCalls, 7)
+	assertNumCalls(c, &numCalls, int32(len(apiCalls)))
 
 	info := r.GetInfo()
 	c.Assert(info, gc.HasLen, 1)
@@ -186,19 +231,35 @@ func (s *relationsSuite) TestNewRelationsWithExistingRelations(c *gc.C) {
 	c.Assert(oneInfo.MemberNames, gc.HasLen, 0)
 }
 
+func (s *relationsSuite) TestNewRelationsWithExistingRelationsLeader(c *gc.C) {
+	s.assertNewRelationsWithExistingRelations(c, true)
+}
+
+func (s *relationsSuite) TestNewRelationsWithExistingRelationsNotLeader(c *gc.C) {
+	s.assertNewRelationsWithExistingRelations(c, false)
+}
+
 func (s *relationsSuite) TestNextOpNothing(c *gc.C) {
 	unitTag := names.NewUnitTag("wordpress/0")
 	abort := make(chan struct{})
 
 	var numCalls int32
-	unitEntity := params.Entities{Entities: []params.Entity{params.Entity{Tag: "unit-wordpress-0"}}}
+	unitEntity := params.Entities{Entities: []params.Entity{{Tag: "unit-wordpress-0"}}}
 	apiCaller := mockAPICaller(c, &numCalls,
-		uniterAPICall("Life", unitEntity, params.LifeResults{Results: []params.LifeResult{{Life: params.Alive}}}, nil),
+		uniterAPICall("Refresh", unitEntity, params.UnitRefreshResults{Results: []params.UnitRefreshResult{{Life: params.Alive, Resolved: params.ResolvedNone, Series: "quantal"}}}, nil),
 		uniterAPICall("GetPrincipal", unitEntity, params.StringBoolResults{Results: []params.StringBoolResult{{Result: "", Ok: false}}}, nil),
-		uniterAPICall("JoinedRelations", unitEntity, params.StringsResults{Results: []params.StringsResult{{Result: []string{}}}}, nil),
+		uniterAPICall("RelationsStatus", unitEntity, params.RelationUnitStatusResults{Results: []params.RelationUnitStatusResult{{RelationResults: []params.RelationUnitStatus{}}}}, nil),
 	)
 	st := uniter.NewState(apiCaller, unitTag)
-	r, err := relation.NewRelations(st, unitTag, s.stateDir, s.relationsDir, abort)
+	r, err := relation.NewRelations(
+		relation.RelationsConfig{
+			State:                st,
+			UnitTag:              unitTag,
+			CharmDir:             s.stateDir,
+			RelationsDir:         s.relationsDir,
+			NewLeadershipContext: s.leadershipContextFunc,
+			Abort:                abort,
+		})
 	c.Assert(err, jc.ErrorIsNil)
 	assertNumCalls(c, &numCalls, 3)
 
@@ -214,7 +275,7 @@ func (s *relationsSuite) TestNextOpNothing(c *gc.C) {
 }
 
 func relationJoinedAPICalls() []apiCall {
-	unitEntity := params.Entities{Entities: []params.Entity{params.Entity{Tag: "unit-wordpress-0"}}}
+	unitEntity := params.Entities{Entities: []params.Entity{{Tag: "unit-wordpress-0"}}}
 	relationResults := params.RelationResults{
 		Results: []params.RelationResult{
 			{
@@ -230,15 +291,20 @@ func relationJoinedAPICalls() []apiCall {
 	relationUnits := params.RelationUnits{RelationUnits: []params.RelationUnit{
 		{Relation: "relation-wordpress.db#mysql.db", Unit: "unit-wordpress-0"},
 	}}
+	relationStatus := params.RelationStatusArgs{Args: []params.RelationStatusArg{{
+		RelationId: 1,
+		Status:     params.Joined,
+	}}}
 	apiCalls := []apiCall{
-		uniterAPICall("Life", unitEntity, params.LifeResults{Results: []params.LifeResult{{Life: params.Alive}}}, nil),
+		uniterAPICall("Refresh", unitEntity, params.UnitRefreshResults{Results: []params.UnitRefreshResult{{Life: params.Alive, Resolved: params.ResolvedNone, Series: "quantal"}}}, nil),
 		uniterAPICall("GetPrincipal", unitEntity, params.StringBoolResults{Results: []params.StringBoolResult{{Result: "", Ok: false}}}, nil),
-		uniterAPICall("JoinedRelations", unitEntity, params.StringsResults{Results: []params.StringsResult{{Result: []string{}}}}, nil),
+		uniterAPICall("RelationsStatus", unitEntity, params.RelationUnitStatusResults{Results: []params.RelationUnitStatusResult{{RelationResults: []params.RelationUnitStatus{}}}}, nil),
 		uniterAPICall("RelationById", params.RelationIds{RelationIds: []int{1}}, relationResults, nil),
 		uniterAPICall("Relation", relationUnits, relationResults, nil),
 		uniterAPICall("Relation", relationUnits, relationResults, nil),
 		uniterAPICall("Watch", unitEntity, params.NotifyWatchResults{Results: []params.NotifyWatchResult{{NotifyWatcherId: "1"}}}, nil),
 		uniterAPICall("EnterScope", relationUnits, params.ErrorResults{Results: []params.ErrorResult{{}}}, nil),
+		uniterAPICall("SetRelationStatus", relationStatus, noErrorResult, nil),
 	}
 	return apiCalls
 }
@@ -249,7 +315,15 @@ func (s *relationsSuite) assertHookRelationJoined(c *gc.C, numCalls *int32, apiC
 
 	apiCaller := mockAPICaller(c, numCalls, apiCalls...)
 	st := uniter.NewState(apiCaller, unitTag)
-	r, err := relation.NewRelations(st, unitTag, s.stateDir, s.relationsDir, abort)
+	r, err := relation.NewRelations(
+		relation.RelationsConfig{
+			State:                st,
+			UnitTag:              unitTag,
+			CharmDir:             s.stateDir,
+			RelationsDir:         s.relationsDir,
+			NewLeadershipContext: s.leadershipContextFunc,
+			Abort:                abort,
+		})
 	c.Assert(err, jc.ErrorIsNil)
 	assertNumCalls(c, numCalls, 3)
 
@@ -260,8 +334,9 @@ func (s *relationsSuite) assertHookRelationJoined(c *gc.C, numCalls *int32, apiC
 	}
 	remoteState := remotestate.Snapshot{
 		Relations: map[int]remotestate.RelationSnapshot{
-			1: remotestate.RelationSnapshot{
-				Life: params.Alive,
+			1: {
+				Life:      params.Alive,
+				Suspended: false,
 				Members: map[string]int64{
 					"wordpress": 1,
 				},
@@ -271,7 +346,7 @@ func (s *relationsSuite) assertHookRelationJoined(c *gc.C, numCalls *int32, apiC
 	relationsResolver := relation.NewRelationsResolver(r)
 	op, err := relationsResolver.NextOp(localState, remoteState, &mockOperations{})
 	c.Assert(err, jc.ErrorIsNil)
-	assertNumCalls(c, numCalls, 8)
+	assertNumCalls(c, numCalls, 9)
 	c.Assert(op.String(), gc.Equals, "run hook relation-joined on unit with relation 1")
 
 	// Commit the operation so we save local state for any next operation.
@@ -325,13 +400,15 @@ func (s *relationsSuite) TestHookRelationChanged(c *gc.C) {
 	// members, due to the "changed pending" local persistent
 	// state.
 	s.assertHookRelationChanged(c, r, remotestate.RelationSnapshot{
-		Life: params.Alive,
+		Life:      params.Alive,
+		Suspended: false,
 	}, &numCalls)
 
 	// wordpress starts at 1, changing to 2 should trigger a
 	// relation-changed hook.
 	s.assertHookRelationChanged(c, r, remotestate.RelationSnapshot{
-		Life: params.Alive,
+		Life:      params.Alive,
+		Suspended: false,
 		Members: map[string]int64{
 			"wordpress": 2,
 		},
@@ -351,10 +428,48 @@ func (s *relationsSuite) TestHookRelationChanged(c *gc.C) {
 	}, &numCalls)
 }
 
+func (s *relationsSuite) TestHookRelationChangedSuspended(c *gc.C) {
+	var numCalls int32
+	apiCalls := relationJoinedAPICalls()
+	r := s.assertHookRelationJoined(c, &numCalls, apiCalls...)
+
+	// There will be an initial relation-changed regardless of
+	// members, due to the "changed pending" local persistent
+	// state.
+	s.assertHookRelationChanged(c, r, remotestate.RelationSnapshot{
+		Life:      params.Alive,
+		Suspended: true,
+	}, &numCalls)
+	c.Assert(r.GetInfo()[1].RelationUnit.Relation().Suspended(), jc.IsTrue)
+
+	numCallsBefore := numCalls
+
+	localState := resolver.LocalState{
+		State: operation.State{
+			Kind: operation.Continue,
+		},
+	}
+	remoteState := remotestate.Snapshot{
+		Relations: map[int]remotestate.RelationSnapshot{
+			1: {
+				Life:      params.Alive,
+				Suspended: true,
+			},
+		},
+	}
+
+	relationsResolver := relation.NewRelationsResolver(r)
+	op, err := relationsResolver.NextOp(localState, remoteState, &mockOperations{})
+	c.Assert(err, jc.ErrorIsNil)
+	assertNumCalls(c, &numCalls, numCallsBefore)
+	c.Assert(op.String(), gc.Equals, "run hook relation-departed on unit with relation 1")
+}
+
 func (s *relationsSuite) assertHookRelationDeparted(c *gc.C, numCalls *int32, apiCalls ...apiCall) relation.Relations {
 	r := s.assertHookRelationJoined(c, numCalls, apiCalls...)
 	s.assertHookRelationChanged(c, r, remotestate.RelationSnapshot{
-		Life: params.Alive,
+		Life:      params.Alive,
+		Suspended: false,
 	}, numCalls)
 	numCallsBefore := *numCalls
 
@@ -365,7 +480,7 @@ func (s *relationsSuite) assertHookRelationDeparted(c *gc.C, numCalls *int32, ap
 	}
 	remoteState := remotestate.Snapshot{
 		Relations: map[int]remotestate.RelationSnapshot{
-			1: remotestate.RelationSnapshot{
+			1: {
 				Life: params.Dying,
 				Members: map[string]int64{
 					"wordpress": 1,
@@ -407,7 +522,7 @@ func (s *relationsSuite) TestHookRelationBroken(c *gc.C) {
 	}
 	remoteState := remotestate.Snapshot{
 		Relations: map[int]remotestate.RelationSnapshot{
-			1: remotestate.RelationSnapshot{
+			1: {
 				Life: params.Dying,
 			},
 		},
@@ -415,10 +530,70 @@ func (s *relationsSuite) TestHookRelationBroken(c *gc.C) {
 	relationsResolver := relation.NewRelationsResolver(r)
 	op, err := relationsResolver.NextOp(localState, remoteState, &mockOperations{})
 	c.Assert(err, jc.ErrorIsNil)
-	assertNumCalls(c, &numCalls, 8)
+	assertNumCalls(c, &numCalls, 9)
 	c.Assert(op.String(), gc.Equals, "run hook relation-broken on unit with relation 1")
 }
 
+func (s *relationsSuite) TestHookRelationBrokenWhenSuspended(c *gc.C) {
+	var numCalls int32
+	apiCalls := relationJoinedAPICalls()
+
+	r := s.assertHookRelationDeparted(c, &numCalls, apiCalls...)
+
+	localState := resolver.LocalState{
+		State: operation.State{
+			Kind: operation.Continue,
+		},
+	}
+	remoteState := remotestate.Snapshot{
+		Relations: map[int]remotestate.RelationSnapshot{
+			1: {
+				Life:      params.Alive,
+				Suspended: true,
+			},
+		},
+	}
+	relationsResolver := relation.NewRelationsResolver(r)
+	op, err := relationsResolver.NextOp(localState, remoteState, &mockOperations{})
+	c.Assert(err, jc.ErrorIsNil)
+	assertNumCalls(c, &numCalls, 9)
+	c.Assert(op.String(), gc.Equals, "run hook relation-broken on unit with relation 1")
+}
+
+func (s *relationsSuite) TestHookRelationBrokenOnlyOnce(c *gc.C) {
+	var numCalls int32
+	apiCalls := relationJoinedAPICalls()
+	relationUnits := params.RelationUnits{RelationUnits: []params.RelationUnit{
+		{Relation: "relation-wordpress.db#mysql.db", Unit: "unit-wordpress-0"},
+	}}
+	apiCalls = append(apiCalls,
+		uniterAPICall("LeaveScope", relationUnits, params.ErrorResults{Results: []params.ErrorResult{{}}}, nil),
+	)
+
+	r := s.assertHookRelationDeparted(c, &numCalls, apiCalls...)
+
+	localState := resolver.LocalState{
+		State: operation.State{
+			Kind: operation.Continue,
+		},
+	}
+	remoteState := remotestate.Snapshot{
+		Relations: map[int]remotestate.RelationSnapshot{
+			1: {
+				Life:      params.Alive,
+				Suspended: true,
+			},
+		},
+	}
+	relationsResolver := relation.NewRelationsResolver(r)
+
+	// Remove the state directory to check that the hook is not run again.
+	err := os.RemoveAll(s.relationsDir)
+	c.Assert(err, jc.ErrorIsNil)
+	_, err = relationsResolver.NextOp(localState, remoteState, &mockOperations{})
+	c.Assert(errors.Cause(err), gc.Equals, resolver.ErrNoOperation)
+}
+
 func (s *relationsSuite) TestCommitHook(c *gc.C) {
 	var numCalls int32
 	apiCalls := relationJoinedAPICalls()
@@ -460,7 +635,7 @@ func (s *relationsSuite) TestImplicitRelationNoHooks(c *gc.C) {
 	unitTag := names.NewUnitTag("wordpress/0")
 	abort := make(chan struct{})
 
-	unitEntity := params.Entities{Entities: []params.Entity{params.Entity{Tag: "unit-wordpress-0"}}}
+	unitEntity := params.Entities{Entities: []params.Entity{{Tag: "unit-wordpress-0"}}}
 	relationResults := params.RelationResults{
 		Results: []params.RelationResult{
 			{
@@ -476,21 +651,34 @@ func (s *relationsSuite) TestImplicitRelationNoHooks(c *gc.C) {
 	relationUnits := params.RelationUnits{RelationUnits: []params.RelationUnit{
 		{Relation: "relation-wordpress.juju-info#juju-info.juju-info", Unit: "unit-wordpress-0"},
 	}}
+	relationStatus := params.RelationStatusArgs{Args: []params.RelationStatusArg{{
+		RelationId: 1,
+		Status:     params.Joined,
+	}}}
 	apiCalls := []apiCall{
-		uniterAPICall("Life", unitEntity, params.LifeResults{Results: []params.LifeResult{{Life: params.Alive}}}, nil),
+		uniterAPICall("Refresh", unitEntity, params.UnitRefreshResults{Results: []params.UnitRefreshResult{{Life: params.Alive, Resolved: params.ResolvedNone, Series: "quantal"}}}, nil),
 		uniterAPICall("GetPrincipal", unitEntity, params.StringBoolResults{Results: []params.StringBoolResult{{Result: "", Ok: false}}}, nil),
-		uniterAPICall("JoinedRelations", unitEntity, params.StringsResults{Results: []params.StringsResult{{Result: []string{}}}}, nil),
+		uniterAPICall("RelationsStatus", unitEntity, params.RelationUnitStatusResults{Results: []params.RelationUnitStatusResult{{RelationResults: []params.RelationUnitStatus{}}}}, nil),
 		uniterAPICall("RelationById", params.RelationIds{RelationIds: []int{1}}, relationResults, nil),
 		uniterAPICall("Relation", relationUnits, relationResults, nil),
 		uniterAPICall("Relation", relationUnits, relationResults, nil),
 		uniterAPICall("Watch", unitEntity, params.NotifyWatchResults{Results: []params.NotifyWatchResult{{NotifyWatcherId: "1"}}}, nil),
 		uniterAPICall("EnterScope", relationUnits, params.ErrorResults{Results: []params.ErrorResult{{}}}, nil),
+		uniterAPICall("SetRelationStatus", relationStatus, noErrorResult, nil),
 	}
 
 	var numCalls int32
 	apiCaller := mockAPICaller(c, &numCalls, apiCalls...)
 	st := uniter.NewState(apiCaller, unitTag)
-	r, err := relation.NewRelations(st, unitTag, s.stateDir, s.relationsDir, abort)
+	r, err := relation.NewRelations(
+		relation.RelationsConfig{
+			State:                st,
+			UnitTag:              unitTag,
+			CharmDir:             s.stateDir,
+			RelationsDir:         s.relationsDir,
+			NewLeadershipContext: s.leadershipContextFunc,
+			Abort:                abort,
+		})
 	c.Assert(err, jc.ErrorIsNil)
 
 	localState := resolver.LocalState{
@@ -500,7 +688,7 @@ func (s *relationsSuite) TestImplicitRelationNoHooks(c *gc.C) {
 	}
 	remoteState := remotestate.Snapshot{
 		Relations: map[int]remotestate.RelationSnapshot{
-			1: remotestate.RelationSnapshot{
+			1: {
 				Life: params.Alive,
 				Members: map[string]int64{
 					"wordpress": 1,
@@ -516,14 +704,19 @@ func (s *relationsSuite) TestImplicitRelationNoHooks(c *gc.C) {
 var (
 	noErrorResult  = params.ErrorResults{Results: []params.ErrorResult{{}}}
 	nrpeUnitTag    = names.NewUnitTag("nrpe/0")
-	nrpeUnitEntity = params.Entities{Entities: []params.Entity{params.Entity{Tag: nrpeUnitTag.String()}}}
+	nrpeUnitEntity = params.Entities{Entities: []params.Entity{{Tag: nrpeUnitTag.String()}}}
 )
 
 func subSubRelationAPICalls() []apiCall {
-	joinedRelationsResults := params.StringsResults{Results: []params.StringsResult{{Result: []string{
-		"relation-wordpress:juju-info nrpe:general-info",
-		"relation-ntp:nrpe-external-master nrpe:external-master",
-	}}}}
+	relationStatusResults := params.RelationUnitStatusResults{Results: []params.RelationUnitStatusResult{{
+		RelationResults: []params.RelationUnitStatus{{
+			RelationTag: "relation-wordpress:juju-info nrpe:general-info",
+			InScope:     true,
+		}, {
+			RelationTag: "relation-ntp:nrpe-external-master nrpe:external-master",
+			InScope:     true,
+		},
+		}}}}
 	relationUnits1 := params.RelationUnits{RelationUnits: []params.RelationUnit{
 		{Relation: "relation-wordpress.juju-info#nrpe.general-info", Unit: "unit-nrpe-0"},
 	}}
@@ -564,19 +757,29 @@ func subSubRelationAPICalls() []apiCall {
 			},
 		}},
 	}
+	relationStatus1 := params.RelationStatusArgs{Args: []params.RelationStatusArg{{
+		RelationId: 1,
+		Status:     params.Joined,
+	}}}
+	relationStatus2 := params.RelationStatusArgs{Args: []params.RelationStatusArg{{
+		RelationId: 2,
+		Status:     params.Joined,
+	}}}
 
 	return []apiCall{
-		uniterAPICall("Life", nrpeUnitEntity, params.LifeResults{Results: []params.LifeResult{{Life: params.Alive}}}, nil),
+		uniterAPICall("Refresh", nrpeUnitEntity, params.UnitRefreshResults{Results: []params.UnitRefreshResult{{Life: params.Alive, Resolved: params.ResolvedNone, Series: "quantal"}}}, nil),
 		uniterAPICall("GetPrincipal", nrpeUnitEntity, params.StringBoolResults{Results: []params.StringBoolResult{{Result: "unit-wordpress-0", Ok: true}}}, nil),
-		uniterAPICall("JoinedRelations", nrpeUnitEntity, joinedRelationsResults, nil),
+		uniterAPICall("RelationsStatus", nrpeUnitEntity, relationStatusResults, nil),
 		uniterAPICall("Relation", relationUnits1, relationResults1, nil),
 		uniterAPICall("Relation", relationUnits2, relationResults2, nil),
 		uniterAPICall("Relation", relationUnits1, relationResults1, nil),
 		uniterAPICall("Watch", nrpeUnitEntity, params.NotifyWatchResults{Results: []params.NotifyWatchResult{{NotifyWatcherId: "1"}}}, nil),
 		uniterAPICall("EnterScope", relationUnits1, noErrorResult, nil),
+		uniterAPICall("SetRelationStatus", relationStatus1, noErrorResult, nil),
 		uniterAPICall("Relation", relationUnits2, relationResults2, nil),
 		uniterAPICall("Watch", nrpeUnitEntity, params.NotifyWatchResults{Results: []params.NotifyWatchResult{{NotifyWatcherId: "2"}}}, nil),
 		uniterAPICall("EnterScope", relationUnits2, noErrorResult, nil),
+		uniterAPICall("SetRelationStatus", relationStatus2, noErrorResult, nil),
 	}
 }
 
@@ -594,7 +797,15 @@ func (s *relationsSuite) TestSubSubPrincipalRelationDyingDestroysUnit(c *gc.C) {
 	apiCaller := mockAPICaller(c, &numCalls, apiCalls...)
 
 	st := uniter.NewState(apiCaller, nrpeUnitTag)
-	r, err := relation.NewRelations(st, nrpeUnitTag, s.stateDir, s.relationsDir, make(chan struct{}))
+	r, err := relation.NewRelations(
+		relation.RelationsConfig{
+			State:                st,
+			UnitTag:              nrpeUnitTag,
+			CharmDir:             s.stateDir,
+			RelationsDir:         s.relationsDir,
+			NewLeadershipContext: s.leadershipContextFunc,
+			Abort:                make(chan struct{}),
+		})
 	c.Assert(err, jc.ErrorIsNil)
 	assertNumCalls(c, &numCalls, callsBeforeDestroy)
 
@@ -610,13 +821,13 @@ func (s *relationsSuite) TestSubSubPrincipalRelationDyingDestroysUnit(c *gc.C) {
 
 	remoteState := remotestate.Snapshot{
 		Relations: map[int]remotestate.RelationSnapshot{
-			1: remotestate.RelationSnapshot{
+			1: {
 				Life: params.Dying,
 				Members: map[string]int64{
 					"wordpress/0": 1,
 				},
 			},
-			2: remotestate.RelationSnapshot{
+			2: {
 				Life: params.Alive,
 				Members: map[string]int64{
 					"ntp/0": 1,
@@ -643,7 +854,15 @@ func (s *relationsSuite) TestSubSubOtherRelationDyingNotDestroyed(c *gc.C) {
 	apiCaller := mockAPICaller(c, &numCalls, apiCalls...)
 
 	st := uniter.NewState(apiCaller, nrpeUnitTag)
-	r, err := relation.NewRelations(st, nrpeUnitTag, s.stateDir, s.relationsDir, make(chan struct{}))
+	r, err := relation.NewRelations(
+		relation.RelationsConfig{
+			State:                st,
+			UnitTag:              nrpeUnitTag,
+			CharmDir:             s.stateDir,
+			RelationsDir:         s.relationsDir,
+			NewLeadershipContext: s.leadershipContextFunc,
+			Abort:                make(chan struct{}),
+		})
 	c.Assert(err, jc.ErrorIsNil)
 	assertNumCalls(c, &numCalls, expectedCalls)
 
@@ -659,13 +878,13 @@ func (s *relationsSuite) TestSubSubOtherRelationDyingNotDestroyed(c *gc.C) {
 
 	remoteState := remotestate.Snapshot{
 		Relations: map[int]remotestate.RelationSnapshot{
-			1: remotestate.RelationSnapshot{
+			1: {
 				Life: params.Alive,
 				Members: map[string]int64{
 					"wordpress/0": 1,
 				},
 			},
-			2: remotestate.RelationSnapshot{
+			2: {
 				Life: params.Dying,
 				Members: map[string]int64{
 					"ntp/0": 1,
