@@ -5,7 +5,7 @@ package uniter
 
 import (
 	"github.com/juju/errors"
-	"gopkg.in/juju/charm.v6-unstable/hooks"
+	"gopkg.in/juju/charm.v6/hooks"
 
 	"github.com/juju/juju/apiserver/params"
 	"github.com/juju/juju/worker/uniter/hook"
@@ -260,7 +260,8 @@ func (s *uniterResolver) nextOp(
 		return opFactory.NewUpgrade(remoteState.CharmURL)
 	}
 
-	if localState.ConfigVersion != remoteState.ConfigVersion {
+	if localState.ConfigVersion != remoteState.ConfigVersion ||
+		localState.Series != remoteState.Series {
 		return opFactory.NewRunHook(hook.Info{Kind: hooks.ConfigChanged})
 	}
 
