@@ -15,7 +15,7 @@ import (
 	"github.com/juju/loggo"
 	"github.com/juju/utils/clock"
 	"github.com/juju/utils/proxy"
-	"gopkg.in/juju/charm.v6-unstable"
+	"gopkg.in/juju/charm.v6"
 	"gopkg.in/juju/names.v2"
 
 	"github.com/juju/juju/api/base"
@@ -23,6 +23,7 @@ import (
 	"github.com/juju/juju/apiserver/params"
 	"github.com/juju/juju/network"
 	"github.com/juju/juju/status"
+	"github.com/juju/juju/version"
 	"github.com/juju/juju/worker/uniter/runner/jujuc"
 )
 
@@ -589,6 +590,7 @@ func (context *HookContext) HookVars(paths Paths) ([]string, error) {
 		"JUJU_MACHINE_ID="+context.assignedMachineTag.Id(),
 		"JUJU_PRINCIPAL_UNIT="+context.principal,
 		"JUJU_AVAILABILITY_ZONE="+context.availabilityzone,
+		"JUJU_VERSION="+version.Current.String(),
 	)
 	if r, err := context.HookRelation(); err == nil {
 		vars = append(vars,
@@ -795,11 +797,6 @@ func (ctx *HookContext) killCharmHook() error {
 	}
 }
 
-// NetworkConfig returns the network config for the given bindingName.
-func (ctx *HookContext) NetworkConfig(bindingName string) ([]params.NetworkConfig, error) {
-	return ctx.unit.NetworkConfig(bindingName)
-}
-
 // UnitWorkloadVersion returns the version of the workload reported by
 // the current unit.
 func (ctx *HookContext) UnitWorkloadVersion() (string, error) {
@@ -837,7 +834,11 @@ func (ctx *HookContext) SetUnitWorkloadVersion(version string) error {
 	return result.OneError()
 }
 
-// NetworkInfo returns the network info for the given bindingNames.
-func (ctx *HookContext) NetworkInfo(bindingNames []string) (map[string]params.NetworkInfoResult, error) {
-	return ctx.unit.NetworkInfo(bindingNames)
+// NetworkInfo returns the network info for the given bindings on the given relation.
+func (ctx *HookContext) NetworkInfo(bindingNames []string, relationId int) (map[string]params.NetworkInfoResult, error) {
+	var relId *int
+	if relationId != -1 {
+		relId = &relationId
+	}
+	return ctx.unit.NetworkInfo(bindingNames, relId)
 }
