@@ -13,7 +13,7 @@ import (
 	"github.com/juju/utils"
 	"github.com/juju/utils/fs"
 	gc "gopkg.in/check.v1"
-	"gopkg.in/juju/charm.v6-unstable/hooks"
+	"gopkg.in/juju/charm.v6/hooks"
 	"gopkg.in/juju/names.v2"
 
 	"github.com/juju/juju/apiserver/params"
@@ -62,6 +62,12 @@ func (s *ContextFactorySuite) setUpCacheMethods(c *gc.C) {
 	c.Assert(err, jc.ErrorIsNil)
 }
 
+func (s *ContextFactorySuite) Model(c *gc.C) *state.Model {
+	m, err := s.State.Model()
+	c.Assert(err, jc.ErrorIsNil)
+	return m
+}
+
 func (s *ContextFactorySuite) updateCache(relId int, unitName string, settings params.Settings) {
 	context.UpdateCachedSettings(s.factory, relId, unitName, settings)
 }
@@ -141,7 +147,7 @@ func (s *ContextFactorySuite) TestNewCommandContextLeadershipContext(c *gc.C) {
 func (s *ContextFactorySuite) TestNewActionContextLeadershipContext(c *gc.C) {
 	s.testLeadershipContextWiring(c, func() *context.HookContext {
 		s.SetCharm(c, "dummy")
-		action, err := s.State.EnqueueAction(s.unit.Tag(), "snapshot", nil)
+		action, err := s.Model(c).EnqueueAction(s.unit.Tag(), "snapshot", nil)
 		c.Assert(err, jc.ErrorIsNil)
 
 		actionData := &context.ActionData{
@@ -176,28 +182,28 @@ func (s *ContextFactorySuite) TestNewHookContextWithStorage(c *gc.C) {
 	sCons := map[string]state.StorageConstraints{
 		"data": {Pool: "", Size: 1024, Count: 1},
 	}
-	service := s.AddTestingServiceWithStorage(c, "storage-block", ch, sCons)
+	service := s.AddTestingApplicationWithStorage(c, "storage-block", ch, sCons)
 	s.machine = nil // allocate a new machine
 	unit := s.AddUnit(c, service)
 
-	storageAttachments, err := s.State.UnitStorageAttachments(unit.UnitTag())
+	storageAttachments, err := s.IAASModel.UnitStorageAttachments(unit.UnitTag())
 	c.Assert(err, jc.ErrorIsNil)
 	c.Assert(storageAttachments, gc.HasLen, 1)
 	storageTag := storageAttachments[0].StorageInstance()
 
-	volume, err := s.State.StorageInstanceVolume(storageTag)
+	volume, err := s.IAASModel.StorageInstanceVolume(storageTag)
 	c.Assert(err, jc.ErrorIsNil)
 	volumeTag := volume.VolumeTag()
 	machineTag := s.machine.MachineTag()
 
-	err = s.State.SetVolumeInfo(
+	err = s.IAASModel.SetVolumeInfo(
 		volumeTag, state.VolumeInfo{
 			VolumeId: "vol-123",
 			Size:     456,
 		},
 	)
 	c.Assert(err, jc.ErrorIsNil)
-	err = s.State.SetVolumeAttachmentInfo(
+	err = s.IAASModel.SetVolumeAttachmentInfo(
 		machineTag, volumeTag, state.VolumeAttachmentInfo{
 			DeviceName: "sdb",
 		},
@@ -237,7 +243,7 @@ func (s *ContextFactorySuite) TestNewHookContextWithStorage(c *gc.C) {
 
 func (s *ContextFactorySuite) TestActionContext(c *gc.C) {
 	s.SetCharm(c, "dummy")
-	action, err := s.State.EnqueueAction(s.unit.Tag(), "snapshot", nil)
+	action, err := s.Model(c).EnqueueAction(s.unit.Tag(), "snapshot", nil)
 	c.Assert(err, jc.ErrorIsNil)
 
 	actionData := &context.ActionData{
