@@ -7,10 +7,10 @@ import (
 	"fmt"
 
 	"github.com/juju/cmd"
+	"github.com/juju/cmd/cmdtesting"
 	jc "github.com/juju/testing/checkers"
 	gc "gopkg.in/check.v1"
 
-	"github.com/juju/juju/testing"
 	"github.com/juju/juju/worker/uniter/runner/jujuc"
 )
 
@@ -69,7 +69,7 @@ func (s *ActionFailSuite) TestActionFail(c *gc.C) {
 	}, {
 		summary: "extra arguments are an error, leaving the action not failed",
 		command: []string{"a failure message", "something else"},
-		errMsg:  "error: unrecognized args: [\"something else\"]\n",
+		errMsg:  "ERROR unrecognized args: [\"something else\"]\n",
 		code:    2,
 	}}
 
@@ -78,7 +78,7 @@ func (s *ActionFailSuite) TestActionFail(c *gc.C) {
 		hctx := &actionFailContext{}
 		com, err := jujuc.NewCommand(hctx, cmdString("action-fail"))
 		c.Assert(err, jc.ErrorIsNil)
-		ctx := testing.Context(c)
+		ctx := cmdtesting.Context(c)
 		code := cmd.Main(com, ctx, t.command)
 		c.Check(code, gc.Equals, t.code)
 		c.Check(bufferString(ctx.Stderr), gc.Equals, t.errMsg)
@@ -91,23 +91,26 @@ func (s *ActionFailSuite) TestNonActionSetActionFailedFails(c *gc.C) {
 	hctx := &nonActionFailContext{}
 	com, err := jujuc.NewCommand(hctx, cmdString("action-fail"))
 	c.Assert(err, jc.ErrorIsNil)
-	ctx := testing.Context(c)
+	ctx := cmdtesting.Context(c)
 	code := cmd.Main(com, ctx, []string{"oops"})
 	c.Check(code, gc.Equals, 1)
-	c.Check(bufferString(ctx.Stderr), gc.Equals, "error: not running an action\n")
+	c.Check(bufferString(ctx.Stderr), gc.Equals, "ERROR not running an action\n")
 	c.Check(bufferString(ctx.Stdout), gc.Equals, "")
 }
 
 func (s *ActionFailSuite) TestHelp(c *gc.C) {
-	hctx := &Context{}
+	hctx, _ := s.NewHookContext()
 	com, err := jujuc.NewCommand(hctx, cmdString("action-fail"))
 	c.Assert(err, jc.ErrorIsNil)
-	ctx := testing.Context(c)
+	ctx := cmdtesting.Context(c)
 	code := cmd.Main(com, ctx, []string{"--help"})
 	c.Assert(code, gc.Equals, 0)
-	c.Assert(bufferString(ctx.Stdout), gc.Equals, `usage: action-fail ["<failure message>"]
-purpose: set action fail status with message
+	c.Assert(bufferString(ctx.Stdout), gc.Equals, `Usage: action-fail ["<failure message>"]
 
+Summary:
+set action fail status with message
+
+Details:
 action-fail sets the action's fail state with a given error message.  Using
 action-fail without a failure message will set a default message indicating a
 problem with the action.
