@@ -8,10 +8,10 @@ import (
 	"time"
 
 	"github.com/juju/cmd"
+	"github.com/juju/cmd/cmdtesting"
 	jc "github.com/juju/testing/checkers"
 	gc "gopkg.in/check.v1"
 
-	"github.com/juju/juju/testing"
 	"github.com/juju/juju/worker/uniter/runner/jujuc"
 )
 
@@ -25,12 +25,14 @@ func (s *AddMetricSuite) TestHelp(c *gc.C) {
 	hctx := s.GetHookContext(c, -1, "")
 	com, err := jujuc.NewCommand(hctx, cmdString("add-metric"))
 	c.Assert(err, jc.ErrorIsNil)
-	ctx := testing.Context(c)
+	ctx := cmdtesting.Context(c)
 	code := cmd.Main(com, ctx, []string{"--help"})
 	c.Assert(code, gc.Equals, 0)
 	c.Assert(bufferString(ctx.Stdout), gc.Equals, `
-usage: add-metric key1=value1 [key2=value2 ...]
-purpose: send metrics
+Usage: add-metric key1=value1 [key2=value2 ...]
+
+Summary:
+add metrics
 `[1:])
 	c.Assert(bufferString(ctx.Stderr), gc.Equals, "")
 }
@@ -59,7 +61,7 @@ func (s *AddMetricSuite) TestAddMetric(c *gc.C) {
 			true,
 			2,
 			"",
-			"error: no metrics specified\n",
+			"ERROR no metrics specified\n",
 			nil,
 		}, {
 			"invalid argument format",
@@ -67,7 +69,7 @@ func (s *AddMetricSuite) TestAddMetric(c *gc.C) {
 			true,
 			2,
 			"",
-			"error: expected \"key=value\", got \"key\"\n",
+			"ERROR expected \"key=value\", got \"key\"\n",
 			nil,
 		}, {
 			"invalid argument format",
@@ -75,7 +77,31 @@ func (s *AddMetricSuite) TestAddMetric(c *gc.C) {
 			true,
 			2,
 			"",
-			"error: expected \"key=value\", got \"=key\"\n",
+			"ERROR expected \"key=value\", got \"=key\"\n",
+			nil,
+		}, {
+			"invalid argument format, whitespace key",
+			[]string{"add-metric", " =value"},
+			true,
+			2,
+			"",
+			"ERROR expected \"key=value\", got \"=value\"\n",
+			nil,
+		}, {
+			"invalid argument format, whitespace key and value",
+			[]string{"add-metric", " \t =  \n"},
+			true,
+			2,
+			"",
+			"ERROR expected \"key=value\", got \"=\"\n",
+			nil,
+		}, {
+			"invalid argument format, whitespace value",
+			[]string{"add-metric", " key =  "},
+			true,
+			2,
+			"",
+			"ERROR expected \"key=value\", got \"key=\"\n",
 			nil,
 		}, {
 			"multiple metrics",
@@ -91,15 +117,31 @@ func (s *AddMetricSuite) TestAddMetric(c *gc.C) {
 			true,
 			2,
 			"",
-			"error: key \"key\" specified more than once\n",
+			"ERROR key \"key\" specified more than once\n",
 			nil,
+		}, {
+			"newline in metric value",
+			[]string{"add-metric", "key=60\n", "key2\t=\t30", "\tkey3 =\t15"},
+			true,
+			0,
+			"",
+			"",
+			[]jujuc.Metric{{"key", "60", time.Now()}, {"key2", "30", time.Now()}, {"key3", "15", time.Now()}},
 		}, {
 			"can't add metrics",
 			[]string{"add-metric", "key=60", "key2=50.4"},
 			false,
 			1,
 			"",
-			"error: cannot record metric: metrics disabled\n",
+			"ERROR cannot record metric: metrics disabled\n",
+			nil,
+		}, {
+			"cannot add builtin metric",
+			[]string{"add-metric", "juju-key=50"},
+			true,
+			1,
+			"",
+			"ERROR juju-key uses a reserved prefix\n",
 			nil,
 		}}
 	for i, t := range testCases {
@@ -108,7 +150,7 @@ func (s *AddMetricSuite) TestAddMetric(c *gc.C) {
 		hctx.canAddMetrics = t.canAddMetrics
 		com, err := jujuc.NewCommand(hctx, cmdString(t.cmd[0]))
 		c.Assert(err, jc.ErrorIsNil)
-		ctx := testing.Context(c)
+		ctx := cmdtesting.Context(c)
 		ret := cmd.Main(com, ctx, t.cmd[1:])
 		c.Check(ret, gc.Equals, t.result)
 		c.Check(bufferString(ctx.Stdout), gc.Equals, t.stdout)
