@@ -5,6 +5,7 @@ package jujuc_test
 
 import (
 	"github.com/juju/cmd"
+	"github.com/juju/cmd/cmdtesting"
 	"github.com/juju/errors"
 	jc "github.com/juju/testing/checkers"
 	gc "gopkg.in/check.v1"
@@ -36,23 +37,23 @@ func (s *isLeaderSuite) TestInitSuccess(c *gc.C) {
 func (s *isLeaderSuite) TestFormatError(c *gc.C) {
 	command, err := jujuc.NewIsLeaderCommand(nil)
 	c.Assert(err, jc.ErrorIsNil)
-	runContext := testing.Context(c)
+	runContext := cmdtesting.Context(c)
 	code := cmd.Main(command, runContext, []string{"--format", "bad"})
 	c.Check(code, gc.Equals, 2)
 	c.Check(bufferString(runContext.Stdout), gc.Equals, "")
-	c.Check(bufferString(runContext.Stderr), gc.Equals, `error: invalid value "bad" for flag --format: unknown format "bad"`+"\n")
+	c.Check(bufferString(runContext.Stderr), gc.Equals, `ERROR invalid value "bad" for flag --format: unknown format "bad"`+"\n")
 }
 
 func (s *isLeaderSuite) TestIsLeaderError(c *gc.C) {
 	jujucContext := &isLeaderContext{err: errors.New("pow")}
 	command, err := jujuc.NewIsLeaderCommand(jujucContext)
 	c.Assert(err, jc.ErrorIsNil)
-	runContext := testing.Context(c)
+	runContext := cmdtesting.Context(c)
 	code := cmd.Main(command, runContext, nil)
 	c.Check(code, gc.Equals, 1)
 	c.Check(jujucContext.called, jc.IsTrue)
 	c.Check(bufferString(runContext.Stdout), gc.Equals, "")
-	c.Check(bufferString(runContext.Stderr), gc.Equals, "error: leadership status unknown: pow\n")
+	c.Check(bufferString(runContext.Stderr), gc.Equals, "ERROR leadership status unknown: pow\n")
 }
 
 func (s *isLeaderSuite) TestFormatDefaultYes(c *gc.C) {
@@ -91,7 +92,7 @@ func (s *isLeaderSuite) testOutput(c *gc.C, leader bool, args []string, expect s
 	jujucContext := &isLeaderContext{leader: leader}
 	command, err := jujuc.NewIsLeaderCommand(jujucContext)
 	c.Assert(err, jc.ErrorIsNil)
-	runContext := testing.Context(c)
+	runContext := cmdtesting.Context(c)
 	code := cmd.Main(command, runContext, args)
 	c.Check(code, gc.Equals, 0)
 	c.Check(jujucContext.called, jc.IsTrue)
@@ -103,7 +104,7 @@ func (s *isLeaderSuite) testParseOutput(c *gc.C, leader bool, args []string, che
 	jujucContext := &isLeaderContext{leader: leader}
 	command, err := jujuc.NewIsLeaderCommand(jujucContext)
 	c.Assert(err, jc.ErrorIsNil)
-	runContext := testing.Context(c)
+	runContext := cmdtesting.Context(c)
 	code := cmd.Main(command, runContext, args)
 	c.Check(code, gc.Equals, 0)
 	c.Check(jujucContext.called, jc.IsTrue)
