@@ -37,11 +37,20 @@ var portsTests = []struct {
 	{[]string{"close-port", "443/udp"}, makeRanges("99/tcp")},
 	{[]string{"open-port", "123/udp"}, makeRanges("99/tcp", "123/udp")},
 	{[]string{"close-port", "9999/UDP"}, makeRanges("99/tcp", "123/udp")},
+	{[]string{"open-port", "icmp"}, makeRanges("icmp", "99/tcp", "123/udp")},
 }
 
 func makeRanges(stringRanges ...string) []network.PortRange {
 	var results []network.PortRange
 	for _, s := range stringRanges {
+		if s == "icmp" {
+			results = append(results, network.PortRange{
+				FromPort: -1,
+				ToPort:   -1,
+				Protocol: "icmp",
+			})
+			continue
+		}
 		if strings.Contains(s, "-") {
 			parts := strings.Split(s, "-")
 			fromPort, _ := strconv.Atoi(parts[0])
@@ -75,7 +84,7 @@ func (s *PortsSuite) TestOpenClose(c *gc.C) {
 		c.Assert(err, jc.ErrorIsNil)
 		ctx := cmdtesting.Context(c)
 		code := cmd.Main(com, ctx, t.cmd[1:])
-		c.Assert(code, gc.Equals, 0)
+		c.Check(code, gc.Equals, 0)
 		c.Assert(bufferString(ctx.Stdout), gc.Equals, "")
 		c.Assert(bufferString(ctx.Stderr), gc.Equals, "")
 		hctx.info.CheckPorts(c, t.expect)
@@ -89,16 +98,17 @@ var badPortsTests = []struct {
 	{nil, "no port or range specified"},
 	{[]string{"0"}, `port must be in the range \[1, 65535\]; got "0"`},
 	{[]string{"65536"}, `port must be in the range \[1, 65535\]; got "65536"`},
-	{[]string{"two"}, `expected <port>\[/<protocol>\] or <from>-<to>\[/<protocol>\]; got "two"`},
-	{[]string{"80/http"}, `protocol must be "tcp" or "udp"; got "http"`},
-	{[]string{"blah/blah/blah"}, `expected <port>\[/<protocol>\] or <from>-<to>\[/<protocol>\]; got "blah/blah/blah"`},
+	{[]string{"two"}, `expected <port>\[/<protocol>\] or <from>-<to>\[/<protocol>\] or icmp; got "two"`},
+	{[]string{"80/http"}, `protocol must be "tcp", "udp", or "icmp"; got "http"`},
+	{[]string{"blah/blah/blah"}, `expected <port>\[/<protocol>\] or <from>-<to>\[/<protocol>\] or icmp; got "blah/blah/blah"`},
 	{[]string{"123", "haha"}, `unrecognized args: \["haha"\]`},
 	{[]string{"1-0"}, `invalid port range 1-0/tcp; expected fromPort <= toPort`},
 	{[]string{"-42"}, `flag provided but not defined: -4`},
 	{[]string{"99999/UDP"}, `port must be in the range \[1, 65535\]; got "99999"`},
-	{[]string{"9999/foo"}, `protocol must be "tcp" or "udp"; got "foo"`},
-	{[]string{"80-90/http"}, `protocol must be "tcp" or "udp"; got "http"`},
+	{[]string{"9999/foo"}, `protocol must be "tcp", "udp", or "icmp"; got "foo"`},
+	{[]string{"80-90/http"}, `protocol must be "tcp", "udp", or "icmp"; got "http"`},
 	{[]string{"20-10/tcp"}, `invalid port range 20-10/tcp; expected fromPort <= toPort`},
+	{[]string{"80/icmp"}, `protocol "icmp" doesn't support any ports; got "80"`},
 }
 
 func (s *PortsSuite) TestBadArgs(c *gc.C) {
@@ -119,7 +129,7 @@ func (s *PortsSuite) TestHelp(c *gc.C) {
 	c.Assert(err, jc.ErrorIsNil)
 	flags := cmdtesting.NewFlagSet()
 	c.Assert(string(open.Info().Help(flags)), gc.Equals, `
-Usage: open-port <port>[/<protocol>] or <from>-<to>[/<protocol>]
+Usage: open-port <port>[/<protocol>] or <from>-<to>[/<protocol>] or icmp
 
 Summary:
 register a port or range to open
@@ -131,7 +141,7 @@ The port range will only be open while the application is exposed.
 	close, err := jujuc.NewCommand(hctx, cmdString("close-port"))
 	c.Assert(err, jc.ErrorIsNil)
 	c.Assert(string(close.Info().Help(flags)), gc.Equals, `
-Usage: close-port <port>[/<protocol>] or <from>-<to>[/<protocol>]
+Usage: close-port <port>[/<protocol>] or <from>-<to>[/<protocol>] or icmp
 
 Summary:
 ensure a port or range is always closed
