@@ -6,7 +6,6 @@ package runner_test
 import (
 	"fmt"
 	"io/ioutil"
-	"os"
 	"path/filepath"
 	"runtime"
 	"strings"
@@ -15,23 +14,24 @@ import (
 	"github.com/juju/errors"
 	envtesting "github.com/juju/testing"
 	jc "github.com/juju/testing/checkers"
-	"github.com/juju/utils"
+	"github.com/juju/utils/exec"
+	"github.com/juju/utils/proxy"
 	gc "gopkg.in/check.v1"
+	"gopkg.in/juju/charm.v6/hooks"
 
+	"github.com/juju/juju/worker/uniter/hook"
 	"github.com/juju/juju/worker/uniter/runner"
+	"github.com/juju/juju/worker/uniter/runner/context"
+	runnertesting "github.com/juju/juju/worker/uniter/runner/testing"
 )
 
 type RunCommandSuite struct {
-	HookContextSuite
+	ContextSuite
 }
 
 var _ = gc.Suite(&RunCommandSuite{})
 
-func (s *RunCommandSuite) getHookContext(c *gc.C) *runner.HookContext {
-	uuid, err := utils.NewUUID()
-	c.Assert(err, jc.ErrorIsNil)
-	return s.HookContextSuite.getHookContext(c, uuid.String(), -1, "", noProxies)
-}
+var noProxies = proxy.Settings{}
 
 func (s *RunCommandSuite) TestRunCommandsEnvStdOutAndErrAndRC(c *gc.C) {
 	// TODO(bogdanteleaga): powershell throws another exit status code when
@@ -40,8 +40,9 @@ func (s *RunCommandSuite) TestRunCommandsEnvStdOutAndErrAndRC(c *gc.C) {
 	if runtime.GOOS == "windows" {
 		c.Skip("bug 1403084: Have to figure out a good way to output to stderr from powershell")
 	}
-	ctx := s.getHookContext(c)
-	paths := NewRealPaths(c)
+	ctx, err := s.contextFactory.HookContext(hook.Info{Kind: hooks.ConfigChanged})
+	c.Assert(err, jc.ErrorIsNil)
+	paths := runnertesting.NewRealPaths(c)
 	runner := runner.NewRunner(ctx, paths)
 
 	commands := `
@@ -53,13 +54,13 @@ exit 42
 	c.Assert(err, jc.ErrorIsNil)
 
 	c.Assert(result.Code, gc.Equals, 42)
-	c.Assert(strings.TrimRight(string(result.Stdout), "\r\n"), gc.Equals, paths.charm)
+	c.Assert(strings.TrimRight(string(result.Stdout), "\r\n"), gc.Equals, paths.GetCharmDir())
 	c.Assert(strings.TrimRight(string(result.Stderr), "\r\n"), gc.Equals, "this is standard err")
 	c.Assert(ctx.GetProcess(), gc.NotNil)
 }
 
 type RunHookSuite struct {
-	HookContextSuite
+	ContextSuite
 }
 
 var _ = gc.Suite(&RunHookSuite{})
@@ -113,12 +114,12 @@ var runHookTests = []struct {
 }
 
 func (s *RunHookSuite) TestRunHook(c *gc.C) {
-	uuid, err := utils.NewUUID()
-	c.Assert(err, jc.ErrorIsNil)
 	for i, t := range runHookTests {
 		c.Logf("\ntest %d: %s; perm %v", i, t.summary, t.spec.perm)
-		ctx := s.getHookContext(c, uuid.String(), t.relid, t.remote, noProxies)
-		paths := NewRealPaths(c)
+		ctx, err := s.contextFactory.HookContext(hook.Info{Kind: hooks.ConfigChanged})
+		c.Assert(err, jc.ErrorIsNil)
+
+		paths := runnertesting.NewRealPaths(c)
 		rnr := runner.NewRunner(ctx, paths)
 		var hookExists bool
 		if t.spec.perm != 0 {
@@ -126,15 +127,15 @@ func (s *RunHookSuite) TestRunHook(c *gc.C) {
 			spec.dir = "hooks"
 			spec.name = hookName
 			c.Logf("makeCharm %#v", spec)
-			makeCharm(c, spec, paths.charm)
+			makeCharm(c, spec, paths.GetCharmDir())
 			hookExists = true
 		}
 		t0 := time.Now()
-		err := rnr.RunHook("something-happened")
+		err = rnr.RunHook("something-happened")
 		if t.err == "" && hookExists {
 			c.Assert(err, jc.ErrorIsNil)
 		} else if !hookExists {
-			c.Assert(runner.IsMissingHookError(err), jc.IsTrue)
+			c.Assert(context.IsMissingHookError(err), jc.IsTrue)
 		} else {
 			c.Assert(err, gc.ErrorMatches, t.err)
 		}
@@ -146,52 +147,70 @@ func (s *RunHookSuite) TestRunHook(c *gc.C) {
 
 type MockContext struct {
 	runner.Context
-	actionData   *runner.ActionData
-	expectPid    int
-	flushBadge   string
-	flushFailure error
-	flushResult  error
+	actionData      *context.ActionData
+	actionParams    map[string]interface{}
+	actionParamsErr error
+	actionResults   map[string]interface{}
+	expectPid       int
+	flushBadge      string
+	flushFailure    error
+	flushResult     error
 }
 
 func (ctx *MockContext) UnitName() string {
 	return "some-unit/999"
 }
 
-func (ctx *MockContext) HookVars(paths runner.Paths) []string {
-	return []string{"VAR=value"}
+func (ctx *MockContext) HookVars(paths context.Paths) ([]string, error) {
+	return []string{"VAR=value"}, nil
 }
 
-func (ctx *MockContext) ActionData() (*runner.ActionData, error) {
+func (ctx *MockContext) ActionData() (*context.ActionData, error) {
 	if ctx.actionData == nil {
 		return nil, errors.New("blam")
 	}
 	return ctx.actionData, nil
 }
 
-func (ctx *MockContext) SetProcess(process *os.Process) {
-	ctx.expectPid = process.Pid
+func (ctx *MockContext) SetProcess(process context.HookProcess) {
+	ctx.expectPid = process.Pid()
 }
 
-func (ctx *MockContext) FlushContext(badge string, failure error) error {
+func (ctx *MockContext) Prepare() error {
+	return nil
+}
+
+func (ctx *MockContext) Flush(badge string, failure error) error {
 	ctx.flushBadge = badge
 	ctx.flushFailure = failure
 	return ctx.flushResult
 }
 
+func (ctx *MockContext) ActionParams() (map[string]interface{}, error) {
+	return ctx.actionParams, ctx.actionParamsErr
+}
+
+func (ctx *MockContext) UpdateActionResults(keys []string, value string) error {
+	for _, key := range keys {
+		ctx.actionResults[key] = value
+	}
+	return nil
+}
+
 type RunMockContextSuite struct {
 	envtesting.IsolationSuite
-	paths RealPaths
+	paths runnertesting.RealPaths
 }
 
 var _ = gc.Suite(&RunMockContextSuite{})
 
 func (s *RunMockContextSuite) SetUpTest(c *gc.C) {
 	s.IsolationSuite.SetUpTest(c)
-	s.paths = NewRealPaths(c)
+	s.paths = runnertesting.NewRealPaths(c)
 }
 
 func (s *RunMockContextSuite) assertRecordedPid(c *gc.C, expectPid int) {
-	path := filepath.Join(s.paths.charm, "pid")
+	path := filepath.Join(s.paths.GetCharmDir(), "pid")
 	content, err := ioutil.ReadFile(path)
 	c.Assert(err, jc.ErrorIsNil)
 	expectContent := fmt.Sprintf("%d", expectPid)
@@ -207,7 +226,7 @@ func (s *RunMockContextSuite) TestRunHookFlushSuccess(c *gc.C) {
 		dir:  "hooks",
 		name: hookName,
 		perm: 0700,
-	}, s.paths.charm)
+	}, s.paths.GetCharmDir())
 	actualErr := runner.NewRunner(ctx, s.paths).RunHook("something-happened")
 	c.Assert(actualErr, gc.Equals, expectErr)
 	c.Assert(ctx.flushBadge, gc.Equals, "something-happened")
@@ -225,7 +244,7 @@ func (s *RunMockContextSuite) TestRunHookFlushFailure(c *gc.C) {
 		name: hookName,
 		perm: 0700,
 		code: 123,
-	}, s.paths.charm)
+	}, s.paths.GetCharmDir())
 	actualErr := runner.NewRunner(ctx, s.paths).RunHook("something-happened")
 	c.Assert(actualErr, gc.Equals, expectErr)
 	c.Assert(ctx.flushBadge, gc.Equals, "something-happened")
@@ -237,13 +256,13 @@ func (s *RunMockContextSuite) TestRunActionFlushSuccess(c *gc.C) {
 	expectErr := errors.New("pew pew pew")
 	ctx := &MockContext{
 		flushResult: expectErr,
-		actionData:  &runner.ActionData{},
+		actionData:  &context.ActionData{},
 	}
 	makeCharm(c, hookSpec{
 		dir:  "actions",
 		name: hookName,
 		perm: 0700,
-	}, s.paths.charm)
+	}, s.paths.GetCharmDir())
 	actualErr := runner.NewRunner(ctx, s.paths).RunAction("something-happened")
 	c.Assert(actualErr, gc.Equals, expectErr)
 	c.Assert(ctx.flushBadge, gc.Equals, "something-happened")
@@ -255,14 +274,14 @@ func (s *RunMockContextSuite) TestRunActionFlushFailure(c *gc.C) {
 	expectErr := errors.New("pew pew pew")
 	ctx := &MockContext{
 		flushResult: expectErr,
-		actionData:  &runner.ActionData{},
+		actionData:  &context.ActionData{},
 	}
 	makeCharm(c, hookSpec{
 		dir:  "actions",
 		name: hookName,
 		perm: 0700,
 		code: 123,
-	}, s.paths.charm)
+	}, s.paths.GetCharmDir())
 	actualErr := runner.NewRunner(ctx, s.paths).RunAction("something-happened")
 	c.Assert(actualErr, gc.Equals, expectErr)
 	c.Assert(ctx.flushBadge, gc.Equals, "something-happened")
@@ -270,6 +289,53 @@ func (s *RunMockContextSuite) TestRunActionFlushFailure(c *gc.C) {
 	s.assertRecordedPid(c, ctx.expectPid)
 }
 
+func (s *RunMockContextSuite) TestRunActionParamsFailure(c *gc.C) {
+	expectErr := errors.New("stork")
+	ctx := &MockContext{
+		actionData:      &context.ActionData{},
+		actionParamsErr: expectErr,
+	}
+	actualErr := runner.NewRunner(ctx, s.paths).RunAction("juju-run")
+	c.Assert(errors.Cause(actualErr), gc.Equals, expectErr)
+}
+
+func (s *RunMockContextSuite) TestRunActionSuccessful(c *gc.C) {
+	ctx := &MockContext{
+		actionData: &context.ActionData{},
+		actionParams: map[string]interface{}{
+			"command": "echo 1",
+			"timeout": 0,
+		},
+		actionResults: map[string]interface{}{},
+	}
+	err := runner.NewRunner(ctx, s.paths).RunAction("juju-run")
+	c.Assert(err, jc.ErrorIsNil)
+	c.Assert(ctx.flushBadge, gc.Equals, "juju-run")
+	c.Assert(ctx.flushFailure, gc.IsNil)
+	c.Assert(ctx.actionResults["Code"], gc.Equals, "0")
+	c.Assert(strings.TrimRight(ctx.actionResults["Stdout"].(string), "\r\n"), gc.Equals, "1")
+	c.Assert(ctx.actionResults["Stderr"], gc.Equals, "")
+}
+
+func (s *RunMockContextSuite) TestRunActionCancelled(c *gc.C) {
+	timeout := 1 * time.Nanosecond
+	ctx := &MockContext{
+		actionData: &context.ActionData{},
+		actionParams: map[string]interface{}{
+			"command": "sleep 10",
+			"timeout": float64(timeout.Nanoseconds()),
+		},
+		actionResults: map[string]interface{}{},
+	}
+	err := runner.NewRunner(ctx, s.paths).RunAction("juju-run")
+	c.Assert(err, jc.ErrorIsNil)
+	c.Assert(ctx.flushBadge, gc.Equals, "juju-run")
+	c.Assert(ctx.flushFailure, gc.Equals, exec.ErrCancelled)
+	c.Assert(ctx.actionResults["Code"], gc.Equals, nil)
+	c.Assert(ctx.actionResults["Stdout"], gc.Equals, nil)
+	c.Assert(ctx.actionResults["Stderr"], gc.Equals, nil)
+}
+
 func (s *RunMockContextSuite) TestRunCommandsFlushSuccess(c *gc.C) {
 	expectErr := errors.New("pew pew pew")
 	ctx := &MockContext{
