@@ -6,9 +6,7 @@ package storage
 import (
 	"gopkg.in/juju/names.v2"
 
-	"github.com/juju/juju/apiserver/params"
 	"github.com/juju/juju/worker/uniter/hook"
-	"github.com/juju/juju/worker/uniter/resolver"
 )
 
 type State interface {
@@ -41,7 +39,3 @@ func ReadAllStateFiles(dirPath string) (map[names.StorageTag]State, error) {
 	}
 	return states, nil
 }
-
-func SetStorageLife(resolver resolver.Resolver, life map[names.StorageTag]params.Life) {
-	resolver.(*storageResolver).life = life
-}
