@@ -15,7 +15,7 @@ import (
 	"github.com/juju/utils"
 	"github.com/juju/utils/clock"
 	"github.com/juju/utils/exec"
-	corecharm "gopkg.in/juju/charm.v6-unstable"
+	corecharm "gopkg.in/juju/charm.v6"
 	"gopkg.in/juju/names.v2"
 	"gopkg.in/juju/worker.v1"
 
@@ -430,9 +430,15 @@ func (u *Uniter) init(unitTag names.UnitTag) (err error) {
 		return errors.Trace(err)
 	}
 	relations, err := relation.NewRelations(
-		u.st, unitTag, u.paths.State.CharmDir,
-		u.paths.State.RelationsDir, u.catacomb.Dying(),
-	)
+		relation.RelationsConfig{
+			State:                u.st,
+			UnitTag:              unitTag,
+			Tracker:              u.leadershipTracker,
+			NewLeadershipContext: context.NewLeadershipContext,
+			CharmDir:             u.paths.State.CharmDir,
+			RelationsDir:         u.paths.State.RelationsDir,
+			Abort:                u.catacomb.Dying(),
+		})
 	if err != nil {
 		return errors.Annotatef(err, "cannot create relations")
 	}
