@@ -26,7 +26,7 @@ import (
 	utilexec "github.com/juju/utils/exec"
 	"github.com/juju/utils/proxy"
 	gc "gopkg.in/check.v1"
-	corecharm "gopkg.in/juju/charm.v6-unstable"
+	corecharm "gopkg.in/juju/charm.v6"
 	"gopkg.in/juju/names.v2"
 	"gopkg.in/juju/worker.v1"
 	goyaml "gopkg.in/yaml.v2"
@@ -85,6 +85,7 @@ type context struct {
 	dataDir                string
 	s                      *UniterSuite
 	st                     *state.State
+	im                     *state.IAASModel
 	api                    *apiuniter.State
 	apiConn                api.Connection
 	leaderClaimer          coreleadership.Claimer
@@ -189,24 +190,6 @@ func (ctx *context) writeHook(c *gc.C, path string, good bool) {
 	ctx.writeExplicitHook(c, path, content)
 }
 
-func (ctx *context) writeActions(c *gc.C, path string, names []string) {
-	for _, name := range names {
-		ctx.writeAction(c, path, name)
-	}
-}
-
-func (ctx *context) writeMetricsYaml(c *gc.C, path string) {
-	metricsYamlPath := filepath.Join(path, "metrics.yaml")
-	var metricsYamlFull []byte = []byte(`
-metrics:
-  pings:
-    type: gauge
-    description: sample metric
-`)
-	err := ioutil.WriteFile(metricsYamlPath, []byte(metricsYamlFull), 0755)
-	c.Assert(err, jc.ErrorIsNil)
-}
-
 func (ctx *context) writeAction(c *gc.C, path, name string) {
 	actionPath := filepath.Join(path, "actions", name)
 	action := actions[name]
@@ -362,10 +345,6 @@ func (s createCharm) step(c *gc.C, ctx *context) {
 	step(c, ctx, addCharm{dir, curl(s.revision)})
 }
 
-func (s createCharm) charmURL() string {
-	return curl(s.revision).String()
-}
-
 type addCharm struct {
 	dir  *corecharm.CharmDir
 	curl *corecharm.URL
@@ -403,24 +382,24 @@ func (s serveCharm) step(c *gc.C, ctx *context) {
 	}
 }
 
-type createServiceAndUnit struct {
-	serviceName string
-	storage     map[string]state.StorageConstraints
+type createApplicationAndUnit struct {
+	applicationName string
+	storage         map[string]state.StorageConstraints
 }
 
-func (csau createServiceAndUnit) step(c *gc.C, ctx *context) {
-	if csau.serviceName == "" {
-		csau.serviceName = "u"
+func (csau createApplicationAndUnit) step(c *gc.C, ctx *context) {
+	if csau.applicationName == "" {
+		csau.applicationName = "u"
 	}
 	sch, err := ctx.st.Charm(curl(0))
 	c.Assert(err, jc.ErrorIsNil)
-	svc := ctx.s.AddTestingServiceWithStorage(c, csau.serviceName, sch, csau.storage)
-	unit, err := svc.AddUnit()
+	app := ctx.s.AddTestingApplicationWithStorage(c, csau.applicationName, sch, csau.storage)
+	unit, err := app.AddUnit(state.AddUnitParams{})
 	c.Assert(err, jc.ErrorIsNil)
 
 	// Assign the unit to a provisioned machine to match expected state.
 	assertAssignUnit(c, ctx.st, unit)
-	ctx.svc = svc
+	ctx.svc = app
 	ctx.unit = unit
 
 	ctx.apiLogin(c)
@@ -434,7 +413,7 @@ type createUniter struct {
 
 func (s createUniter) step(c *gc.C, ctx *context) {
 	step(c, ctx, ensureStateWorker{})
-	step(c, ctx, createServiceAndUnit{})
+	step(c, ctx, createApplicationAndUnit{})
 	if s.minion {
 		step(c, ctx, forceMinion{})
 	}
@@ -877,7 +856,9 @@ type waitActionResults struct {
 }
 
 func (s waitActionResults) step(c *gc.C, ctx *context) {
-	resultsWatcher := ctx.st.WatchActionResults()
+	m, err := ctx.st.Model()
+	c.Assert(err, jc.ErrorIsNil)
+	resultsWatcher := m.WatchActionResults()
 	defer func() {
 		c.Assert(resultsWatcher.Stop(), gc.IsNil)
 	}()
@@ -984,7 +965,9 @@ type addAction struct {
 }
 
 func (s addAction) step(c *gc.C, ctx *context) {
-	_, err := ctx.st.EnqueueAction(ctx.unit.Tag(), s.name, s.params)
+	m, err := ctx.st.Model()
+	c.Assert(err, jc.ErrorIsNil)
+	_, err = m.EnqueueAction(ctx.unit.Tag(), s.name, s.params)
 	c.Assert(err, jc.ErrorIsNil)
 }
 
@@ -1135,7 +1118,7 @@ func (s addRelation) step(c *gc.C, ctx *context) {
 		panic("don't add two relations!")
 	}
 	if ctx.relatedSvc == nil {
-		ctx.relatedSvc = ctx.s.AddTestingService(c, "mysql", ctx.s.AddTestingCharm(c, "mysql"))
+		ctx.relatedSvc = ctx.s.AddTestingApplication(c, "mysql", ctx.s.AddTestingCharm(c, "mysql"))
 	}
 	eps, err := ctx.st.InferEndpoints("u", "mysql")
 	c.Assert(err, jc.ErrorIsNil)
@@ -1168,7 +1151,7 @@ func (s addRelation) step(c *gc.C, ctx *context) {
 type addRelationUnit struct{}
 
 func (s addRelationUnit) step(c *gc.C, ctx *context) {
-	u, err := ctx.relatedSvc.AddUnit()
+	u, err := ctx.relatedSvc.AddUnit(state.AddUnitParams{})
 	c.Assert(err, jc.ErrorIsNil)
 	ru, err := ctx.relation.Unit(u)
 	c.Assert(err, jc.ErrorIsNil)
@@ -1229,7 +1212,7 @@ type addSubordinateRelation struct {
 
 func (s addSubordinateRelation) step(c *gc.C, ctx *context) {
 	if _, err := ctx.st.Application("logging"); errors.IsNotFound(err) {
-		ctx.s.AddTestingService(c, "logging", ctx.s.AddTestingCharm(c, "logging"))
+		ctx.s.AddTestingApplication(c, "logging", ctx.s.AddTestingCharm(c, "logging"))
 	}
 	eps, err := ctx.st.InferEndpoints("logging", "u:"+s.ifce)
 	c.Assert(err, jc.ErrorIsNil)
@@ -1456,7 +1439,9 @@ func (s setProxySettings) step(c *gc.C, ctx *context) {
 		"ftp-proxy":   s.Ftp,
 		"no-proxy":    s.NoProxy,
 	}
-	err := ctx.st.UpdateModelConfig(attrs, nil)
+	m, err := ctx.st.Model()
+	c.Assert(err, jc.ErrorIsNil)
+	err = m.UpdateModelConfig(attrs, nil)
 	c.Assert(err, jc.ErrorIsNil)
 }
 
@@ -1734,18 +1719,18 @@ func (*mockCharmDirGuard) Lockdown(_ fortress.Abort) error { return nil }
 type provisionStorage struct{}
 
 func (s provisionStorage) step(c *gc.C, ctx *context) {
-	storageAttachments, err := ctx.st.UnitStorageAttachments(ctx.unit.UnitTag())
+	storageAttachments, err := ctx.im.UnitStorageAttachments(ctx.unit.UnitTag())
 	c.Assert(err, jc.ErrorIsNil)
 	c.Assert(storageAttachments, gc.HasLen, 1)
 
-	filesystem, err := ctx.st.StorageInstanceFilesystem(storageAttachments[0].StorageInstance())
+	filesystem, err := ctx.im.StorageInstanceFilesystem(storageAttachments[0].StorageInstance())
 	c.Assert(err, jc.ErrorIsNil)
 
 	filesystemInfo := state.FilesystemInfo{
 		Size:         1024,
 		FilesystemId: "fs-id",
 	}
-	err = ctx.st.SetFilesystemInfo(filesystem.FilesystemTag(), filesystemInfo)
+	err = ctx.im.SetFilesystemInfo(filesystem.FilesystemTag(), filesystemInfo)
 	c.Assert(err, jc.ErrorIsNil)
 
 	machineId, err := ctx.unit.AssignedMachineId()
@@ -1754,7 +1739,7 @@ func (s provisionStorage) step(c *gc.C, ctx *context) {
 	filesystemAttachmentInfo := state.FilesystemAttachmentInfo{
 		MountPoint: "/srv/wordpress/content",
 	}
-	err = ctx.st.SetFilesystemAttachmentInfo(
+	err = ctx.im.SetFilesystemAttachmentInfo(
 		names.NewMachineTag(machineId),
 		filesystem.FilesystemTag(),
 		filesystemAttachmentInfo,
@@ -1765,10 +1750,10 @@ func (s provisionStorage) step(c *gc.C, ctx *context) {
 type destroyStorageAttachment struct{}
 
 func (s destroyStorageAttachment) step(c *gc.C, ctx *context) {
-	storageAttachments, err := ctx.st.UnitStorageAttachments(ctx.unit.UnitTag())
+	storageAttachments, err := ctx.im.UnitStorageAttachments(ctx.unit.UnitTag())
 	c.Assert(err, jc.ErrorIsNil)
 	c.Assert(storageAttachments, gc.HasLen, 1)
-	err = ctx.st.DetachStorage(
+	err = ctx.im.DetachStorage(
 		storageAttachments[0].StorageInstance(),
 		ctx.unit.UnitTag(),
 	)
@@ -1778,7 +1763,7 @@ func (s destroyStorageAttachment) step(c *gc.C, ctx *context) {
 type verifyStorageDetached struct{}
 
 func (s verifyStorageDetached) step(c *gc.C, ctx *context) {
-	storageAttachments, err := ctx.st.UnitStorageAttachments(ctx.unit.UnitTag())
+	storageAttachments, err := ctx.im.UnitStorageAttachments(ctx.unit.UnitTag())
 	c.Assert(err, jc.ErrorIsNil)
 	c.Assert(storageAttachments, gc.HasLen, 0)
 }
