@@ -6,7 +6,7 @@ package workertest
 import (
 	"errors"
 
-	"github.com/juju/juju/worker"
+	"gopkg.in/juju/worker.v1"
 )
 
 type NotAWatcher struct {
