@@ -43,18 +43,25 @@ var (
 	logger    = loggo.GetLogger("juju.testing")
 
 	// regular expression to match output of mongod
-	waitingForConnectionsRe = regexp.MustCompile(".*initandlisten.*waiting for connections.*")
+	waitingForConnectionsRe = regexp.MustCompile(".*waiting for connections.*")
+
+	mongo32 = version.Number{Major: 3, Minor: 2}
 
 	// After version 3.2 we shouldn't use --nojournal - it makes the
 	// WiredTiger storage engine much slower.
 	// https://jira.mongodb.org/browse/SERVER-21198
-	useJournalMongoVersion = version.Number{Major: 3, Minor: 2}
-	mongoVersion           lazyMongoVersion
+	useJournalMongoVersion = mongo32
+
+	// From mongo 3.2 onwards, we can specify a storage engine.
+	storageEngineMongoVersion = mongo32
+
+	installedMongod mongodCache
 )
 
 const (
 	// Maximum number of times to attempt starting mongod.
 	maxStartMongodAttempts = 5
+
 	// The default password to use when connecting to the mongo database.
 	DefaultMongoPassword = "conn-from-name-secret"
 )
@@ -113,6 +120,11 @@ func (m *MgoInstance) Port() int {
 	return m.port
 }
 
+// SSLEnabled reports whether or not SSL is enabled for the MongoDB server.
+func (m *MgoInstance) SSLEnabled() bool {
+	return m.certs != nil
+}
+
 // We specify a timeout to mgo.Dial, to prevent
 // mongod failures hanging the tests.
 const mgoDialTimeout = 60 * time.Second
@@ -122,6 +134,12 @@ const mgoDialTimeout = 60 * time.Second
 // MongoDB server.
 type MgoSuite struct {
 	Session *mgo.Session
+
+	// DebugMgo controls whether SetUpSuite enables mgo logging and
+	// debugging. Set this before calling SetUpSuite. Enabling either
+	// logging or debugging in mgo adds a significant overhead to the
+	// Juju tests, so they are disabled by default.
+	DebugMgo bool
 }
 
 // generatePEM receives server certificate and the server private key
@@ -212,7 +230,6 @@ func (inst *MgoInstance) run() error {
 		"--nssize", "1",
 		"--noprealloc",
 		"--smallfiles",
-		"--nohttpinterface",
 		"--oplogSize", "10",
 		"--ipv6",
 		"--setParameter", "enableTestCommands=1",
@@ -228,27 +245,31 @@ func (inst *MgoInstance) run() error {
 	}
 	if inst.certs != nil {
 		mgoargs = append(mgoargs,
-			"--sslOnNormalPorts",
+			"--sslMode", "requireSSL",
 			"--sslPEMKeyFile", filepath.Join(inst.dir, "server.pem"),
-			"--sslPEMKeyPassword", "ignored")
+			"--sslPEMKeyPassword=ignored")
 	}
-	version, err := mongoVersion.Get()
+
+	mongopath, version, err := installedMongod.Get()
 	if err != nil {
 		return err
 	}
+	logger.Debugf("using mongod at: %q (version=%s)", mongopath, version)
+
 	if version.Compare(useJournalMongoVersion) == -1 {
 		mgoargs = append(mgoargs, "--nojournal")
 	}
+	if version.Compare(storageEngineMongoVersion) >= 0 {
+		storageEngine := mongoStorageEngine()
+		if storageEngine != "" {
+			mgoargs = append(mgoargs, "--storageEngine", storageEngine)
+		}
+	}
 
 	if inst.Params != nil {
 		mgoargs = append(mgoargs, inst.Params...)
 	}
-	mongopath, err := getMongod()
-	if err != nil {
-		return err
-	}
-	logger.Debugf("found mongod at: %q", mongopath)
-	if mongopath == "/usr/lib/juju/bin/mongod" {
+	if mongopath == "/usr/lib/juju/bin/mongod" || mongopath == "/usr/lib/juju/mongo3.2/bin/mongod" {
 		inst.WithoutV8 = true
 	}
 	server := exec.Command(mongopath, mgoargs...)
@@ -310,12 +331,59 @@ func (inst *MgoInstance) run() error {
 	return nil
 }
 
-func getMongod() (string, error) {
-	// The last path is needed in tests on CentOS where PATH is being completely removed
-	paths := []string{"mongod", "/usr/lib/juju/bin/mongod", "/usr/local/bin/mongod"}
-	if path := os.Getenv("JUJU_MONGOD"); path != "" {
-		paths = append([]string{path}, paths...)
+func mongoStorageEngine() string {
+	storageEngine := os.Getenv("JUJU_MONGO_STORAGE_ENGINE")
+	if storageEngine != "" {
+		return storageEngine
 	}
+	switch runtime.GOARCH {
+	case "386", "amd64":
+		// On x86(_64), mmapv1 should always be available. If not
+		// overridden via the environment variable above, we use
+		// mmapv1 by default for the best performance in tests.
+		return "mmapv1"
+	}
+	return "" // use the default
+}
+
+// mongodCache looks up mongod path and version and caches the result.
+type mongodCache struct {
+	sync.Mutex
+	path    string
+	version version.Number
+	done    bool
+}
+
+func (cache *mongodCache) Get() (string, version.Number, error) {
+	cache.Lock()
+	defer cache.Unlock()
+	if !cache.done {
+		var err error
+		cache.path, err = getMongod()
+		if err != nil {
+			return "", version.Zero, errors.Trace(err)
+		}
+		cache.version, err = detectMongoVersion(cache.path)
+		if err != nil {
+			return "", version.Zero, errors.Trace(err)
+		}
+		cache.done = true
+	}
+	return cache.path, cache.version, nil
+}
+
+func getMongod() (string, error) {
+	// Prefer $JUJU_MONGOD and then newer MongoDBs.
+	var paths []string
+	if path := os.Getenv("JUJU_MONGOD"); path != "" {
+		paths = append(paths, path)
+	}
+	paths = append(paths,
+		"/usr/lib/juju/mongo3.2/bin/mongod",
+		"mongod",
+		"/usr/lib/juju/bin/mongod",
+		"/usr/local/bin/mongod", // Needed on CentOS where $PATH is being completely removed
+	)
 	var err error
 	var mongopath string
 	for _, path := range paths {
@@ -331,11 +399,7 @@ func getMongod() (string, error) {
 // The mongod --version line starts with this prefix.
 const versionLinePrefix = "db version v"
 
-func detectMongoVersion() (version.Number, error) {
-	mongoPath, err := getMongod()
-	if err != nil {
-		return version.Zero, errors.Trace(err)
-	}
+func detectMongoVersion(mongoPath string) (version.Number, error) {
 	output, err := exec.Command(mongoPath, "--version").Output()
 	if err != nil {
 		return version.Zero, errors.Trace(err)
@@ -357,29 +421,9 @@ func detectMongoVersion() (version.Number, error) {
 	if err != nil {
 		return version.Zero, errors.Trace(err)
 	}
-	logger.Debugf("detected mongod version %v", ver)
 	return ver, nil
 }
 
-// lazyMongoVersion stores the mongod version once we've got it.
-type lazyMongoVersion struct {
-	sync.Mutex
-	version version.Number
-}
-
-func (v *lazyMongoVersion) Get() (version.Number, error) {
-	v.Lock()
-	defer v.Unlock()
-	if v.version == version.Zero {
-		ver, err := detectMongoVersion()
-		if err != nil {
-			return version.Zero, errors.Trace(err)
-		}
-		v.version = ver
-	}
-	return v.version, nil
-}
-
 func (inst *MgoInstance) kill(sig os.Signal) {
 	inst.server.Process.Signal(sig)
 	<-inst.exited
@@ -433,8 +477,10 @@ func (s *mgoLogger) Output(calldepth int, message string) error {
 }
 
 func (s *MgoSuite) SetUpSuite(c *gc.C) {
-	mgo.SetLogger(&mgoLogger{loggo.GetLogger("mgo")})
-	mgo.SetDebug(true)
+	if s.DebugMgo {
+		mgo.SetLogger(&mgoLogger{loggo.GetLogger("mgo")})
+		mgo.SetDebug(true)
+	}
 	if MgoServer.addr == "" {
 		c.Fatalf("No Mongo Server Address, MgoSuite tests must be run with MgoTestPackage")
 	}
@@ -495,8 +541,20 @@ func (s *MgoSuite) TearDownSuite(c *gc.C) {
 	err := MgoServer.Reset()
 	c.Assert(err, jc.ErrorIsNil)
 	utils.FastInsecureHash = false
-	mgo.SetDebug(false)
-	mgo.SetLogger(nil)
+	if s.DebugMgo {
+		mgo.SetDebug(false)
+		mgo.SetLogger(nil)
+	}
+}
+
+// MustDial returns a new connection to the MongoDB server, and panics on
+// errors.
+func (inst *MgoInstance) MustDial() *mgo.Session {
+	s, err := mgo.DialWithInfo(inst.DialInfo())
+	if err != nil {
+		panic(err)
+	}
+	return s
 }
 
 // Dial returns a new connection to the MongoDB server.
@@ -600,6 +658,11 @@ func clearCollections(db *mgo.Database) error {
 		if strings.HasPrefix(name, "system.") {
 			continue
 		}
+		if name == "oplog.rs" && db.Name == "local" {
+			// 3.4 prevents you from deleting your local replicaset information
+			// while part of a replica. Arguably it was never safe to do.
+			continue
+		}
 		collection := db.C(name)
 		clearFunc := clearNormalCollection
 		capped, err := isCapped(collection)
@@ -680,7 +743,7 @@ func (inst *MgoInstance) Reset() error {
 	logger.Infof("reset successfully reset admin password")
 	for _, name := range dbnames {
 		switch name {
-		case "local", "config":
+		case "local", "config", "admin":
 			// don't delete these
 			continue
 		}
@@ -844,6 +907,8 @@ func NewProxiedSession(c *gc.C) *ProxiedSession {
 	mgoInfo.Addrs = []string{proxy.Addr()}
 	session, err := mgo.DialWithInfo(mgoInfo)
 	c.Assert(err, gc.IsNil)
+	err = session.Ping()
+	c.Assert(err, jc.ErrorIsNil)
 	return &ProxiedSession{
 		Session:  session,
 		TCPProxy: proxy,
