@@ -7,6 +7,7 @@ import (
 	"time"
 
 	"github.com/juju/loggo"
+	"github.com/juju/testing"
 	jc "github.com/juju/testing/checkers"
 	gc "gopkg.in/check.v1"
 	"gopkg.in/mgo.v2/bson"
@@ -31,6 +32,7 @@ func (s *PruneSuite) maybePruneWithTimestamp(c *gc.C, pruneFactor float32, times
 		Database:                  s.db,
 		TransactionCollectionName: s.txns.Name,
 		ChangeLogName:             s.txns.Name + ".log",
+		Clock:					   testing.NewClock(time.Now()),
 	})
 	err := r.MaybePruneTransactions(jujutxn.PruneOptions{
 		PruneFactor:        pruneFactor,
