@@ -9,7 +9,6 @@ import (
 	"fmt"
 	"os"
 	"path/filepath"
-	"strings"
 	"syscall"
 	"unsafe"
 
@@ -23,6 +22,27 @@ const (
 
 //sys moveFileEx(lpExistingFileName *uint16, lpNewFileName *uint16, dwFlags uint32) (err error) = MoveFileExW
 
+// MoveFile atomically moves the source file to the destination, returning
+// whether the file was moved successfully. If the destination already exists,
+// it returns an error rather than overwrite it.
+func MoveFile(source, destination string) (bool, error) {
+	src, err := syscall.UTF16PtrFromString(source)
+	if err != nil {
+		return false, &os.LinkError{"move", source, destination, err}
+	}
+	dest, err := syscall.UTF16PtrFromString(destination)
+	if err != nil {
+		return false, &os.LinkError{"move", source, destination, err}
+	}
+
+	// see http://msdn.microsoft.com/en-us/library/windows/desktop/aa365240(v=vs.85).aspx
+	if err := moveFileEx(src, dest, movefile_write_through); err != nil {
+		return false, &os.LinkError{"move", source, destination, err}
+	}
+	return true, nil
+
+}
+
 // ReplaceFile atomically replaces the destination file or directory with the source.
 // The errors that are returned are identical to those returned by os.Rename.
 func ReplaceFile(source, destination string) error {
@@ -44,33 +64,17 @@ func ReplaceFile(source, destination string) error {
 
 // MakeFileURL returns a proper file URL for the given path/directory
 func MakeFileURL(in string) string {
-	var volumeName string
-	if strings.HasPrefix(in, "file://") {
-		volumeName = filepath.VolumeName(strings.TrimPrefix(in, "file://"))
-	} else {
-		volumeName = filepath.VolumeName(in)
+	in = filepath.ToSlash(in)
+	// for windows at least should be <letter>: to be considered valid
+	// so we cant do anything with less than that.
+	if len(in) < 2 {
+		return in
 	}
-
-	if volumeName != "" {
-		// Strip colon
-		volumeName = strings.TrimSuffix(volumeName, ":")
-
-		// Do not apply function twice
-		if strings.HasPrefix(in, "file://\\\\localhost\\") {
-			return in
-		}
-
-		in = filepath.ToSlash(in)
-		prefix := "file://\\\\localhost\\" + volumeName + "$"
-		if strings.HasPrefix(in, volumeName) {
-			return prefix + strings.TrimPrefix(in, volumeName+":")
-		}
-		if strings.HasPrefix(in, "file://"+volumeName) {
-			return prefix + strings.TrimPrefix(in, "file://"+volumeName+":")
-		}
+	if string(in[1]) != ":" {
+		return in
 	}
-
-	return in
+	// since go 1.6 http client will only take this format.
+	return "file://" + in
 }
 
 func getUserSID(username string) (string, error) {
