@@ -13,42 +13,9 @@ import (
 	"os"
 	"strings"
 	"unicode"
-
-	goyaml "gopkg.in/yaml.v1"
 )
 
-// WriteYaml marshals obj as yaml and then writes it to a file, atomically,
-// by first writing a sibling with the suffix ".preparing" and then moving
-// the sibling to the real path.
-func WriteYaml(path string, obj interface{}) error {
-	data, err := goyaml.Marshal(obj)
-	if err != nil {
-		return err
-	}
-	prep := path + ".preparing"
-	f, err := os.OpenFile(prep, os.O_WRONLY|os.O_CREATE|os.O_SYNC, 0644)
-	if err != nil {
-		return err
-	}
-	_, err = f.Write(data)
-	// Explicitly close the file before moving it. This is needed on Windows
-	// where the OS will not allow us to move a file that still has an open file handle
-	f.Close()
-	if err != nil {
-		return err
-	}
-	return ReplaceFile(prep, path)
-}
-
-// ReadYaml unmarshals the yaml contained in the file at path into obj. See
-// goyaml.Unmarshal.
-func ReadYaml(path string, obj interface{}) error {
-	data, err := ioutil.ReadFile(path)
-	if err != nil {
-		return err
-	}
-	return goyaml.Unmarshal(data, obj)
-}
+// TODO(ericsnow) Move the quoting helpers into the shell package?
 
 // ShQuote quotes s so that when read by bash, no metacharacters
 // within s will be interpreted as such.
@@ -57,6 +24,48 @@ func ShQuote(s string) string {
 	return `'` + strings.Replace(s, `'`, `'"'"'`, -1) + `'`
 }
 
+// WinPSQuote quotes s so that when read by powershell, no metacharacters
+// within s will be interpreted as such.
+func WinPSQuote(s string) string {
+	// See http://ss64.com/ps/syntax-esc.html#quotes.
+	// Double quotes inside single quotes are fine, double single quotes inside
+	// single quotes, not so much so. Having double quoted strings inside single
+	// quoted strings, ensure no expansion happens.
+	return `'` + strings.Replace(s, `'`, `"`, -1) + `'`
+}
+
+// WinCmdQuote quotes s so that when read by cmd.exe, no metacharacters
+// within s will be interpreted as such.
+func WinCmdQuote(s string) string {
+	// See http://blogs.msdn.com/b/twistylittlepassagesallalike/archive/2011/04/23/everyone-quotes-arguments-the-wrong-way.aspx.
+	quoted := winCmdQuote(s)
+	return winCmdEscapeMeta(quoted)
+}
+
+func winCmdQuote(s string) string {
+	var escaped string
+	for _, c := range s {
+		switch c {
+		case '\\', '"':
+			escaped += `\`
+		}
+		escaped += string(c)
+	}
+	return `"` + escaped + `"`
+}
+
+func winCmdEscapeMeta(str string) string {
+	const meta = `()%!^"<>&|`
+	var newStr string
+	for _, c := range str {
+		if strings.Contains(meta, string(c)) {
+			newStr += "^"
+		}
+		newStr += string(c)
+	}
+	return newStr
+}
+
 // CommandString flattens a sequence of command arguments into a
 // string suitable for executing in a shell, escaping slashes,
 // variables and quotes as necessary; each argument is double-quoted
