@@ -34,40 +34,41 @@ import (
 */
 
 var dbUpdates = []dbUpdate{
-	dbUpdate{version: 1, run: dbUpdateFromV0},
-	dbUpdate{version: 2, run: dbUpdateFromV1},
-	dbUpdate{version: 3, run: dbUpdateFromV2},
-	dbUpdate{version: 4, run: dbUpdateFromV3},
-	dbUpdate{version: 5, run: dbUpdateFromV4},
-	dbUpdate{version: 6, run: dbUpdateFromV5},
-	dbUpdate{version: 7, run: dbUpdateFromV6},
-	dbUpdate{version: 8, run: dbUpdateFromV7},
-	dbUpdate{version: 9, run: dbUpdateFromV8},
-	dbUpdate{version: 10, run: dbUpdateFromV9},
-	dbUpdate{version: 11, run: dbUpdateFromV10},
-	dbUpdate{version: 12, run: dbUpdateFromV11},
-	dbUpdate{version: 13, run: dbUpdateFromV12},
-	dbUpdate{version: 14, run: dbUpdateFromV13},
-	dbUpdate{version: 15, run: dbUpdateFromV14},
-	dbUpdate{version: 16, run: dbUpdateFromV15},
-	dbUpdate{version: 17, run: dbUpdateFromV16},
-	dbUpdate{version: 18, run: dbUpdateFromV17},
-	dbUpdate{version: 19, run: dbUpdateFromV18},
-	dbUpdate{version: 20, run: dbUpdateFromV19},
-	dbUpdate{version: 21, run: dbUpdateFromV20},
-	dbUpdate{version: 22, run: dbUpdateFromV21},
-	dbUpdate{version: 23, run: dbUpdateFromV22},
-	dbUpdate{version: 24, run: dbUpdateFromV23},
-	dbUpdate{version: 25, run: dbUpdateFromV24},
-	dbUpdate{version: 26, run: dbUpdateFromV25},
-	dbUpdate{version: 27, run: dbUpdateFromV26},
-	dbUpdate{version: 28, run: dbUpdateFromV27},
-	dbUpdate{version: 29, run: dbUpdateFromV28},
-	dbUpdate{version: 30, run: dbUpdateFromV29},
-	dbUpdate{version: 31, run: dbUpdateFromV30},
-	dbUpdate{version: 32, run: dbUpdateFromV31},
-	dbUpdate{version: 33, run: dbUpdateFromV32},
-	dbUpdate{version: 34, run: dbUpdateFromV33},
+	{version: 1, run: dbUpdateFromV0},
+	{version: 2, run: dbUpdateFromV1},
+	{version: 3, run: dbUpdateFromV2},
+	{version: 4, run: dbUpdateFromV3},
+	{version: 5, run: dbUpdateFromV4},
+	{version: 6, run: dbUpdateFromV5},
+	{version: 7, run: dbUpdateFromV6},
+	{version: 8, run: dbUpdateFromV7},
+	{version: 9, run: dbUpdateFromV8},
+	{version: 10, run: dbUpdateFromV9},
+	{version: 11, run: dbUpdateFromV10},
+	{version: 12, run: dbUpdateFromV11},
+	{version: 13, run: dbUpdateFromV12},
+	{version: 14, run: dbUpdateFromV13},
+	{version: 15, run: dbUpdateFromV14},
+	{version: 16, run: dbUpdateFromV15},
+	{version: 17, run: dbUpdateFromV16},
+	{version: 18, run: dbUpdateFromV17},
+	{version: 19, run: dbUpdateFromV18},
+	{version: 20, run: dbUpdateFromV19},
+	{version: 21, run: dbUpdateFromV20},
+	{version: 22, run: dbUpdateFromV21},
+	{version: 23, run: dbUpdateFromV22},
+	{version: 24, run: dbUpdateFromV23},
+	{version: 25, run: dbUpdateFromV24},
+	{version: 26, run: dbUpdateFromV25},
+	{version: 27, run: dbUpdateFromV26},
+	{version: 28, run: dbUpdateFromV27},
+	{version: 29, run: dbUpdateFromV28},
+	{version: 30, run: dbUpdateFromV29},
+	{version: 31, run: dbUpdateFromV30},
+	{version: 32, run: dbUpdateFromV31},
+	{version: 33, run: dbUpdateFromV32},
+	{version: 34, run: dbUpdateFromV33},
+	{version: 35, run: dbUpdateFromV34},
 }
 
 type dbUpdate struct {
@@ -124,6 +125,42 @@ func dbUpdatesApplyAll(d *Daemon) error {
 }
 
 // Schema updates begin here
+func dbUpdateFromV34(currentVersion int, version int, d *Daemon) error {
+	stmt := `
+CREATE TABLE IF NOT EXISTS storage_pools (
+    id INTEGER PRIMARY KEY AUTOINCREMENT NOT NULL,
+    name VARCHAR(255) NOT NULL,
+    driver VARCHAR(255) NOT NULL,
+    UNIQUE (name)
+);
+CREATE TABLE IF NOT EXISTS storage_pools_config (
+    id INTEGER PRIMARY KEY AUTOINCREMENT NOT NULL,
+    storage_pool_id INTEGER NOT NULL,
+    key VARCHAR(255) NOT NULL,
+    value TEXT,
+    UNIQUE (storage_pool_id, key),
+    FOREIGN KEY (storage_pool_id) REFERENCES storage_pools (id) ON DELETE CASCADE
+);
+CREATE TABLE IF NOT EXISTS storage_volumes (
+    id INTEGER PRIMARY KEY AUTOINCREMENT NOT NULL,
+    name VARCHAR(255) NOT NULL,
+    storage_pool_id INTEGER NOT NULL,
+    type INTEGER NOT NULL,
+    UNIQUE (storage_pool_id, name, type),
+    FOREIGN KEY (storage_pool_id) REFERENCES storage_pools (id) ON DELETE CASCADE
+);
+CREATE TABLE IF NOT EXISTS storage_volumes_config (
+    id INTEGER PRIMARY KEY AUTOINCREMENT NOT NULL,
+    storage_volume_id INTEGER NOT NULL,
+    key VARCHAR(255) NOT NULL,
+    value TEXT,
+    UNIQUE (storage_volume_id, key),
+    FOREIGN KEY (storage_volume_id) REFERENCES storage_volumes (id) ON DELETE CASCADE
+);`
+	_, err := d.db.Exec(stmt)
+	return err
+}
+
 func dbUpdateFromV33(currentVersion int, version int, d *Daemon) error {
 	stmt := `
 CREATE TABLE IF NOT EXISTS networks (
@@ -537,7 +574,7 @@ func dbUpdateFromV11(currentVersion int, version int, d *Daemon) error {
 
 			// Rsync
 			// containers/<container>/snapshots/<snap0>
-			//   to
+			// to
 			// snapshots/<container>/<snap0>
 			output, err := storageRsyncCopy(oldPath, newPath)
 			if err != nil {
