@@ -12,6 +12,7 @@ import (
 	"os/exec"
 	"path"
 	"path/filepath"
+	"regexp"
 	"sort"
 	"strconv"
 	"strings"
@@ -55,7 +56,217 @@ type usbDevice struct {
 	minor int
 }
 
-func createUSBDevice(action string, vendor string, product string, major string, minor string, busnum string, devnum string) (usbDevice, error) {
+// /dev/nvidia[0-9]+
+type nvidiaGpuCards struct {
+	path  string
+	major int
+	minor int
+	id    string
+}
+
+// {/dev/nvidiactl, /dev/nvidia-uvm, ...}
+type nvidiaGpuDevices struct {
+	path  string
+	major int
+	minor int
+}
+
+// /dev/dri/card0. If we detect that vendor == nvidia, then nvidia will contain
+// the corresponding nvidia car, e.g. {/dev/dri/card1 --> /dev/nvidia1}.
+type gpuDevice struct {
+	vendorid  string
+	productid string
+	id        string // card id e.g. 0
+	// If related devices have the same PCI address as the GPU we should
+	// mount them all. Meaning if we detect /dev/dri/card0,
+	// /dev/dri/controlD64, and /dev/dri/renderD128 with the same PCI
+	// address, then they should all be made available in the container.
+	pci    string
+	nvidia nvidiaGpuCards
+
+	path  string
+	major int
+	minor int
+}
+
+func (g *gpuDevice) isNvidiaGpu() bool {
+	return strings.EqualFold(g.vendorid, "10de")
+}
+
+type cardIds struct {
+	id  string
+	pci string
+}
+
+func deviceLoadGpu() ([]gpuDevice, []nvidiaGpuDevices, error) {
+	const DRI_PATH = "/sys/bus/pci/devices"
+	var gpus []gpuDevice
+	var nvidiaDevices []nvidiaGpuDevices
+	var cards []cardIds
+
+	ents, err := ioutil.ReadDir(DRI_PATH)
+	if err != nil {
+		if os.IsNotExist(err) {
+			return nil, nil, nil
+		}
+		return nil, nil, err
+	}
+
+	isNvidia := false
+	for _, ent := range ents {
+		// The pci address == the name of the directory. So let's use
+		// this cheap way of retrieving it.
+		pciAddr := ent.Name()
+
+		// Make sure that we are dealing with a GPU by looking whether
+		// the "drm" subfolder exists.
+		drm := filepath.Join(DRI_PATH, pciAddr, "drm")
+		drmEnts, err := ioutil.ReadDir(drm)
+		if err != nil {
+			if os.IsNotExist(err) {
+				continue
+			}
+		}
+
+		// Retrieve vendor ID.
+		vendorIdPath := filepath.Join(DRI_PATH, pciAddr, "vendor")
+		vendorId, err := ioutil.ReadFile(vendorIdPath)
+		if err != nil {
+			if os.IsNotExist(err) {
+				continue
+			}
+		}
+
+		// Retrieve device ID.
+		productIdPath := filepath.Join(DRI_PATH, pciAddr, "device")
+		productId, err := ioutil.ReadFile(productIdPath)
+		if err != nil {
+			if os.IsNotExist(err) {
+				continue
+			}
+		}
+
+		// Store all associated subdevices, e.g. controlD64, renderD128.
+		// The name of the directory == the last part of the
+		// /dev/dri/controlD64 path. So ent.Name() will give us
+		// controlD64.
+		for _, drmEnt := range drmEnts {
+			vendorTmp := strings.TrimSpace(string(vendorId))
+			productTmp := strings.TrimSpace(string(productId))
+			vendorTmp = strings.TrimPrefix(vendorTmp, "0x")
+			productTmp = strings.TrimPrefix(productTmp, "0x")
+			tmpGpu := gpuDevice{
+				pci:       pciAddr,
+				vendorid:  vendorTmp,
+				productid: productTmp,
+				path:      filepath.Join("/dev/dri", drmEnt.Name()),
+			}
+
+			majMinPath := filepath.Join(drm, drmEnt.Name(), "dev")
+			majMinByte, err := ioutil.ReadFile(majMinPath)
+			if err != nil {
+				if os.IsNotExist(err) {
+					continue
+				}
+			}
+			majMin := strings.TrimSpace(string(majMinByte))
+			majMinSlice := strings.Split(string(majMin), ":")
+			if len(majMinSlice) != 2 {
+				continue
+			}
+			majorInt, err := strconv.Atoi(majMinSlice[0])
+			if err != nil {
+				continue
+			}
+			minorInt, err := strconv.Atoi(majMinSlice[1])
+			if err != nil {
+				continue
+			}
+
+			tmpGpu.major = majorInt
+			tmpGpu.minor = minorInt
+
+			isCard, err := regexp.MatchString("^card[0-9]+", drmEnt.Name())
+			if err != nil {
+				continue
+			}
+
+			if isCard {
+				// If it is a card it's minor number will be its id.
+				tmpGpu.id = strconv.Itoa(minorInt)
+				tmp := cardIds{
+					id:  tmpGpu.id,
+					pci: tmpGpu.pci,
+				}
+				cards = append(cards, tmp)
+			}
+			// Find matching /dev/nvidia* entry for /dev/dri/card*
+			if tmpGpu.isNvidiaGpu() && isCard {
+				if !isNvidia {
+					isNvidia = true
+				}
+				nvidiaPath := "/dev/nvidia" + strconv.Itoa(tmpGpu.minor)
+				stat := syscall.Stat_t{}
+				err := syscall.Stat(nvidiaPath, &stat)
+				if err != nil {
+					continue
+				}
+				tmpGpu.nvidia.path = nvidiaPath
+				tmpGpu.nvidia.major = int(stat.Rdev / 256)
+				tmpGpu.nvidia.minor = int(stat.Rdev % 256)
+				tmpGpu.nvidia.id = strconv.Itoa(tmpGpu.nvidia.minor)
+			}
+			gpus = append(gpus, tmpGpu)
+		}
+	}
+
+	// We detected a Nvidia card, so let's collect all other nvidia devices
+	// that are not /dev/nvidia[0-9]+.
+	if isNvidia {
+		nvidiaEnts, err := ioutil.ReadDir("/dev")
+		if err != nil {
+			if os.IsNotExist(err) {
+				return nil, nil, err
+			}
+		}
+		validNvidia, err := regexp.Compile(`^nvidia[^0-9]+`)
+		if err != nil {
+			return nil, nil, err
+		}
+		for _, nvidiaEnt := range nvidiaEnts {
+			if !validNvidia.MatchString(nvidiaEnt.Name()) {
+				continue
+			}
+			nvidiaPath := filepath.Join("/dev", nvidiaEnt.Name())
+			stat := syscall.Stat_t{}
+			err = syscall.Stat(nvidiaPath, &stat)
+			if err != nil {
+				continue
+			}
+			tmpNividiaGpu := nvidiaGpuDevices{
+				path:  nvidiaPath,
+				major: int(stat.Rdev / 256),
+				minor: int(stat.Rdev % 256),
+			}
+			nvidiaDevices = append(nvidiaDevices, tmpNividiaGpu)
+		}
+
+	}
+
+	// Since we'll give users to ability to specify and id we need to group
+	// devices on the same PCI that belong to the same card by id.
+	for _, card := range cards {
+		for i := 0; i < len(gpus); i++ {
+			if gpus[i].pci == card.pci {
+				gpus[i].id = card.id
+			}
+		}
+	}
+
+	return gpus, nvidiaDevices, nil
+}
+
+func createUSBDevice(action string, vendor string, product string, major string, minor string, busnum string, devnum string, devname string) (usbDevice, error) {
 	majorInt, err := strconv.Atoi(major)
 	if err != nil {
 		return usbDevice{}, err
@@ -66,18 +277,24 @@ func createUSBDevice(action string, vendor string, product string, major string,
 		return usbDevice{}, err
 	}
 
-	busnumInt, err := strconv.Atoi(busnum)
-	if err != nil {
-		return usbDevice{}, err
-	}
+	path := devname
+	if devname == "" {
+		busnumInt, err := strconv.Atoi(busnum)
+		if err != nil {
+			return usbDevice{}, err
+		}
 
-	devnumInt, err := strconv.Atoi(devnum)
-	if err != nil {
-		return usbDevice{}, err
+		devnumInt, err := strconv.Atoi(devnum)
+		if err != nil {
+			return usbDevice{}, err
+		}
+		path = fmt.Sprintf("/dev/bus/usb/%03d/%03d", busnumInt, devnumInt)
+	} else {
+		if !filepath.IsAbs(devname) {
+			path = fmt.Sprintf("/dev/%s", devname)
+		}
 	}
 
-	path := fmt.Sprintf("/dev/bus/usb/%03d/%03d", busnumInt, devnumInt)
-
 	return usbDevice{
 		action,
 		vendor,
@@ -187,14 +404,19 @@ func deviceNetlinkListener() (chan []string, chan []string, chan usbDevice, erro
 				if !ok {
 					continue
 				}
+
 				minor, ok := props["MINOR"]
 				if !ok {
 					continue
 				}
+
+				devname, ok := props["DEVNAME"]
+
 				busnum, ok := props["BUSNUM"]
 				if !ok {
 					continue
 				}
+
 				devnum, ok := props["DEVNUM"]
 				if !ok {
 					continue
@@ -216,6 +438,7 @@ func deviceNetlinkListener() (chan []string, chan []string, chan usbDevice, erro
 					minor,
 					busnum,
 					devnum,
+					devname,
 				)
 				if err != nil {
 					shared.LogError("error reading usb device", log.Ctx{"err": err, "path": props["PHYSDEVPATH"]})
@@ -492,13 +715,13 @@ func deviceUSBEvent(d *Daemon, usb usbDevice) {
 			}
 
 			if usb.action == "add" {
-				err := c.insertUSBDevice(m, usb)
+				err := c.insertUnixDeviceNum(m, usb.major, usb.minor, usb.path)
 				if err != nil {
 					shared.LogError("failed to create usb device", log.Ctx{"err": err, "usb": usb, "container": c.Name()})
 					return
 				}
 			} else if usb.action == "remove" {
-				err := c.removeUSBDevice(m, usb)
+				err := c.removeUnixDeviceNum(m, usb.major, usb.minor, usb.path)
 				if err != nil {
 					shared.LogError("failed to remove usb device", log.Ctx{"err": err, "usb": usb, "container": c.Name()})
 					return
@@ -848,7 +1071,7 @@ func deviceGetParentBlocks(path string) ([]string, error) {
 		// Accessible zfs filesystems
 		poolName := strings.Split(device[1], "/")[0]
 
-		output, err := exec.Command("zpool", "status", poolName).CombinedOutput()
+		output, err := exec.Command("zpool", "status", "-P", "-L", poolName).CombinedOutput()
 		if err != nil {
 			return nil, fmt.Errorf("Failed to query zfs filesystem information for %s: %s", device[1], output)
 		}
@@ -883,12 +1106,6 @@ func deviceGetParentBlocks(path string) ([]string, error) {
 						devices = append(devices, dev)
 					}
 				}
-			} else if deviceIsBlockdev(fmt.Sprintf("/dev/%s", fields[0])) {
-				path = fmt.Sprintf("/dev/%s", fields[0])
-			} else if deviceIsBlockdev(fmt.Sprintf("/dev/disk/by-id/%s", fields[0])) {
-				path = fmt.Sprintf("/dev/disk/by-id/%s", fields[0])
-			} else if deviceIsBlockdev(fmt.Sprintf("/dev/mapper/%s", fields[0])) {
-				path = fmt.Sprintf("/dev/mapper/%s", fields[0])
 			} else {
 				continue
 			}
@@ -991,7 +1208,7 @@ func loadRawValues(p string) (map[string]string, error) {
 		"devnum":    "",
 	}
 
-	for k, _ := range values {
+	for k := range values {
 		v, err := ioutil.ReadFile(path.Join(p, k))
 		if err != nil {
 			return nil, err
@@ -1039,6 +1256,7 @@ func deviceLoadUsb() ([]usbDevice, error) {
 			parts[1],
 			values["busnum"],
 			values["devnum"],
+			values["devname"],
 		)
 		if err != nil {
 			if os.IsNotExist(err) {
