@@ -23,7 +23,10 @@ import (
 	"gopkg.in/yaml.v2"
 
 	"github.com/lxc/lxd/shared"
+	"github.com/lxc/lxd/shared/api"
 	"github.com/lxc/lxd/shared/logging"
+	"github.com/lxc/lxd/shared/osarch"
+	"github.com/lxc/lxd/shared/version"
 
 	log "gopkg.in/inconshreveable/log15.v2"
 )
@@ -32,7 +35,7 @@ import (
    The CPU and I/O load of publish is such that running multiple ones in
    parallel takes longer than running them serially.
 
-   Additionaly, publishing the same container or container snapshot
+   Additionally, publishing the same container or container snapshot
    twice would lead to storage problem, not to mention a conflict at the
    end for whichever finishes last. */
 var imagePublishLock sync.Mutex
@@ -77,7 +80,7 @@ func detectCompression(fname string) ([]string, string, error) {
 
 }
 
-func unpack(d *Daemon, file string, path string) error {
+func unpack(d *Daemon, file string, path string, sType storageType) error {
 	extractArgs, extension, err := detectCompression(file)
 	if err != nil {
 		return err
@@ -126,8 +129,8 @@ func unpack(d *Daemon, file string, path string) error {
 
 		// Check if we're running out of space
 		if int64(fs.Bfree) < int64(2*fs.Bsize) {
-			if d.Storage.GetStorageType() == storageTypeLvm {
-				return fmt.Errorf("Unable to unpack image, run out of disk space (consider increasing storage.lvm_volume_size).")
+			if sType == storageTypeLvm {
+				return fmt.Errorf("Unable to unpack image, run out of disk space (consider increasing your pool's volume.size).")
 			} else {
 				return fmt.Errorf("Unable to unpack image, run out of disk space.")
 			}
@@ -146,8 +149,8 @@ func unpack(d *Daemon, file string, path string) error {
 	return nil
 }
 
-func unpackImage(d *Daemon, imagefname string, destpath string) error {
-	err := unpack(d, imagefname, destpath)
+func unpackImage(d *Daemon, imagefname string, destpath string, sType storageType) error {
+	err := unpack(d, imagefname, destpath, sType)
 	if err != nil {
 		return err
 	}
@@ -159,7 +162,7 @@ func unpackImage(d *Daemon, imagefname string, destpath string) error {
 			return fmt.Errorf("Error creating rootfs directory")
 		}
 
-		err = unpack(d, imagefname+".rootfs", rootfsPath)
+		err = unpack(d, imagefname+".rootfs", rootfsPath, sType)
 		if err != nil {
 			return err
 		}
@@ -206,15 +209,6 @@ type templateEntry struct {
 	Properties map[string]string `yaml:"properties"`
 }
 
-type imagePostReq struct {
-	Filename             string            `json:"filename"`
-	Public               bool              `json:"public"`
-	Source               map[string]string `json:"source"`
-	Properties           map[string]string `json:"properties"`
-	AutoUpdate           bool              `json:"auto_update"`
-	CompressionAlgorithm string            `json:"compression_algorithm"`
-}
-
 type imageMetadata struct {
 	Architecture string                    `yaml:"architecture"`
 	CreationDate int64                     `yaml:"creation_date"`
@@ -227,8 +221,8 @@ type imageMetadata struct {
  * This function takes a container or snapshot from the local image server and
  * exports it as an image.
  */
-func imgPostContInfo(d *Daemon, r *http.Request, req imagePostReq,
-	builddir string) (info shared.ImageInfo, err error) {
+func imgPostContInfo(d *Daemon, r *http.Request, req api.ImagesPost,
+	builddir string) (info api.Image, err error) {
 
 	info.Properties = map[string]string{}
 	name := req.Source["name"]
@@ -270,7 +264,7 @@ func imgPostContInfo(d *Daemon, r *http.Request, req imagePostReq,
 	}
 	defer os.Remove(tarfile.Name())
 
-	if err := c.Export(tarfile); err != nil {
+	if err := c.Export(tarfile, req.Properties); err != nil {
 		tarfile.Close()
 		return info, err
 	}
@@ -319,13 +313,13 @@ func imgPostContInfo(d *Daemon, r *http.Request, req imagePostReq,
 		return info, err
 	}
 
-	info.Architecture, _ = shared.ArchitectureName(c.Architecture())
+	info.Architecture, _ = osarch.ArchitectureName(c.Architecture())
 	info.Properties = req.Properties
 
 	return info, nil
 }
 
-func imgPostRemoteInfo(d *Daemon, req imagePostReq, op *operation) error {
+func imgPostRemoteInfo(d *Daemon, req api.ImagesPost, op *operation) error {
 	var err error
 	var hash string
 
@@ -337,7 +331,7 @@ func imgPostRemoteInfo(d *Daemon, req imagePostReq, op *operation) error {
 		return fmt.Errorf("must specify one of alias or fingerprint for init from image")
 	}
 
-	hash, err = d.ImageDownload(op, req.Source["server"], req.Source["protocol"], req.Source["certificate"], req.Source["secret"], hash, false, req.AutoUpdate)
+	hash, err = d.ImageDownload(op, req.Source["server"], req.Source["protocol"], req.Source["certificate"], req.Source["secret"], hash, false, req.AutoUpdate, "")
 	if err != nil {
 		return err
 	}
@@ -354,7 +348,7 @@ func imgPostRemoteInfo(d *Daemon, req imagePostReq, op *operation) error {
 
 	// Update the DB record if needed
 	if req.Public || req.AutoUpdate || req.Filename != "" || len(req.Properties) > 0 {
-		err = dbImageUpdate(d.db, id, req.Filename, info.Size, req.Public, req.AutoUpdate, info.Architecture, info.CreationDate, info.ExpiryDate, info.Properties)
+		err = dbImageUpdate(d.db, id, req.Filename, info.Size, req.Public, req.AutoUpdate, info.Architecture, info.CreatedAt, info.ExpiresAt, info.Properties)
 		if err != nil {
 			return err
 		}
@@ -368,29 +362,19 @@ func imgPostRemoteInfo(d *Daemon, req imagePostReq, op *operation) error {
 	return nil
 }
 
-func imgPostURLInfo(d *Daemon, req imagePostReq, op *operation) error {
+func imgPostURLInfo(d *Daemon, req api.ImagesPost, op *operation) error {
 	var err error
 
 	if req.Source["url"] == "" {
 		return fmt.Errorf("Missing URL")
 	}
 
-	// Resolve the image URL
-	tlsConfig, err := shared.GetTLSConfig("", "", "", nil)
+	myhttp, err := d.httpClient("")
 	if err != nil {
 		return err
 	}
 
-	tr := &http.Transport{
-		TLSClientConfig: tlsConfig,
-		Dial:            shared.RFC3493Dialer,
-		Proxy:           d.proxy,
-	}
-
-	myhttp := http.Client{
-		Transport: tr,
-	}
-
+	// Resolve the image URL
 	head, err := http.NewRequest("HEAD", req.Source["url"], nil)
 	if err != nil {
 		return err
@@ -401,9 +385,9 @@ func imgPostURLInfo(d *Daemon, req imagePostReq, op *operation) error {
 		architecturesStr = append(architecturesStr, fmt.Sprintf("%d", arch))
 	}
 
-	head.Header.Set("User-Agent", shared.UserAgent)
+	head.Header.Set("User-Agent", version.UserAgent)
 	head.Header.Set("LXD-Server-Architectures", strings.Join(architecturesStr, ", "))
-	head.Header.Set("LXD-Server-Version", shared.Version)
+	head.Header.Set("LXD-Server-Version", version.Version)
 
 	raw, err := myhttp.Do(head)
 	if err != nil {
@@ -421,7 +405,7 @@ func imgPostURLInfo(d *Daemon, req imagePostReq, op *operation) error {
 	}
 
 	// Import the image
-	hash, err = d.ImageDownload(op, url, "direct", "", "", hash, false, req.AutoUpdate)
+	hash, err = d.ImageDownload(op, url, "direct", "", "", hash, false, req.AutoUpdate, "")
 	if err != nil {
 		return err
 	}
@@ -437,7 +421,7 @@ func imgPostURLInfo(d *Daemon, req imagePostReq, op *operation) error {
 	}
 
 	if req.Public || req.AutoUpdate || req.Filename != "" || len(req.Properties) > 0 {
-		err = dbImageUpdate(d.db, id, req.Filename, info.Size, req.Public, req.AutoUpdate, info.Architecture, info.CreationDate, info.ExpiryDate, info.Properties)
+		err = dbImageUpdate(d.db, id, req.Filename, info.Size, req.Public, req.AutoUpdate, info.Architecture, info.CreatedAt, info.ExpiresAt, info.Properties)
 		if err != nil {
 			return err
 		}
@@ -452,7 +436,7 @@ func imgPostURLInfo(d *Daemon, req imagePostReq, op *operation) error {
 }
 
 func getImgPostInfo(d *Daemon, r *http.Request,
-	builddir string, post *os.File) (info shared.ImageInfo, err error) {
+	builddir string, post *os.File) (info api.Image, err error) {
 
 	var imageMeta *imageMetadata
 	logger := logging.AddContext(shared.Log, log.Ctx{"function": "getImgPostInfo"})
@@ -468,14 +452,14 @@ func getImgPostInfo(d *Daemon, r *http.Request,
 	sha256 := sha256.New()
 	var size int64
 
-	// Create a temporary file for the image tarball
-	imageTarf, err := ioutil.TempFile(builddir, "lxd_tar_")
-	if err != nil {
-		return info, err
-	}
-	defer os.Remove(imageTarf.Name())
-
 	if ctype == "multipart/form-data" {
+		// Create a temporary file for the image tarball
+		imageTarf, err := ioutil.TempFile(builddir, "lxd_tar_")
+		if err != nil {
+			return info, err
+		}
+		defer os.Remove(imageTarf.Name())
+
 		// Parse the POST data
 		post.Seek(0, 0)
 		mr := multipart.NewReader(post, ctypeParams["boundary"])
@@ -577,9 +561,8 @@ func getImgPostInfo(d *Daemon, r *http.Request,
 		}
 	} else {
 		post.Seek(0, 0)
-		size, err = io.Copy(io.MultiWriter(imageTarf, sha256), post)
+		size, err = io.Copy(sha256, post)
 		info.Size = size
-		imageTarf.Close()
 		logger.Debug("Tar size", log.Ctx{"size": size})
 		if err != nil {
 			logger.Error(
@@ -605,7 +588,7 @@ func getImgPostInfo(d *Daemon, r *http.Request,
 			return info, err
 		}
 
-		imageMeta, err = getImageMetadata(imageTarf.Name())
+		imageMeta, err = getImageMetadata(post.Name())
 		if err != nil {
 			logger.Error(
 				"Failed to get image metadata",
@@ -614,21 +597,21 @@ func getImgPostInfo(d *Daemon, r *http.Request,
 		}
 
 		imgfname := shared.VarPath("images", info.Fingerprint)
-		err = shared.FileMove(imageTarf.Name(), imgfname)
+		err = shared.FileMove(post.Name(), imgfname)
 		if err != nil {
 			logger.Error(
 				"Failed to move the tarfile",
 				log.Ctx{
 					"err":    err,
-					"source": imageTarf.Name(),
+					"source": post.Name(),
 					"dest":   imgfname})
 			return info, err
 		}
 	}
 
 	info.Architecture = imageMeta.Architecture
-	info.CreationDate = time.Unix(imageMeta.CreationDate, 0)
-	info.ExpiryDate = time.Unix(imageMeta.ExpiryDate, 0)
+	info.CreatedAt = time.Unix(imageMeta.CreationDate, 0)
+	info.ExpiresAt = time.Unix(imageMeta.ExpiryDate, 0)
 
 	info.Properties = imageMeta.Properties
 	if len(propHeaders) > 0 {
@@ -643,15 +626,32 @@ func getImgPostInfo(d *Daemon, r *http.Request,
 	return info, nil
 }
 
-func imageBuildFromInfo(d *Daemon, info shared.ImageInfo) (metadata map[string]string, err error) {
-	err = d.Storage.ImageCreate(info.Fingerprint)
-	if err != nil {
-		os.Remove(shared.VarPath("images", info.Fingerprint))
-		os.Remove(shared.VarPath("images", info.Fingerprint) + ".rootfs")
-
-		return metadata, err
+// imageCreateInPool() creates a new storage volume in a given storage pool for
+// the image. No entry in the images database will be created. This implies that
+// imageCreateinPool() should only be called when an image already exists in the
+// database and hence has already a storage volume in at least one storage pool.
+func imageCreateInPool(d *Daemon, info *api.Image, storagePool string) error {
+	if storagePool == "" {
+		return fmt.Errorf("No storage pool specified.")
 	}
 
+	// Initialize a new storage interface.
+	s, err := storagePoolInit(d, storagePool)
+	if err != nil {
+		return err
+	}
+
+	// Create the storage volume for the image on the requested storage
+	// pool.
+	err = s.ImageCreate(info.Fingerprint)
+	if err != nil {
+		return err
+	}
+
+	return nil
+}
+
+func imageBuildFromInfo(d *Daemon, info *api.Image) (metadata map[string]string, err error) {
 	err = dbImageInsert(
 		d.db,
 		info.Fingerprint,
@@ -660,8 +660,8 @@ func imageBuildFromInfo(d *Daemon, info shared.ImageInfo) (metadata map[string]s
 		info.Public,
 		info.AutoUpdate,
 		info.Architecture,
-		info.CreationDate,
-		info.ExpiryDate,
+		info.CreatedAt,
+		info.ExpiresAt,
 		info.Properties)
 	if err != nil {
 		return metadata, err
@@ -711,7 +711,7 @@ func imagesPost(d *Daemon, r *http.Request) Response {
 	decoder := json.NewDecoder(post)
 	imageUpload := false
 
-	req := imagePostReq{}
+	req := api.ImagesPost{}
 	err = decoder.Decode(&req)
 	if err != nil {
 		if r.Header.Get("Content-Type") == "application/json" {
@@ -727,7 +727,7 @@ func imagesPost(d *Daemon, r *http.Request) Response {
 
 	// Begin background operation
 	run := func(op *operation) error {
-		var info shared.ImageInfo
+		var info api.Image
 
 		// Setup the cleanup function
 		defer cleanup(builddir, post)
@@ -767,7 +767,7 @@ func imagesPost(d *Daemon, r *http.Request) Response {
 			imagePublishLock.Unlock()
 		}
 
-		metadata, err := imageBuildFromInfo(d, info)
+		metadata, err := imageBuildFromInfo(d, &info)
 		if err != nil {
 			return err
 		}
@@ -815,7 +815,7 @@ func getImageMetadata(fname string) (*imageMetadata, error) {
 		return nil, fmt.Errorf("Could not parse %s: %v", metadataName, err)
 	}
 
-	_, err = shared.ArchitectureId(metadata.Architecture)
+	_, err = osarch.ArchitectureId(metadata.Architecture)
 	if err != nil {
 		return nil, err
 	}
@@ -834,11 +834,11 @@ func doImagesGet(d *Daemon, recursion bool, public bool) (interface{}, error) {
 	}
 
 	resultString := make([]string, len(results))
-	resultMap := make([]*shared.ImageInfo, len(results))
+	resultMap := make([]*api.Image, len(results))
 	i := 0
 	for _, name := range results {
 		if !recursion {
-			url := fmt.Sprintf("/%s/images/%s", shared.APIVersion, name)
+			url := fmt.Sprintf("/%s/images/%s", version.APIVersion, name)
 			resultString[i] = url
 		} else {
 			image, response := doImageGet(d, name, public)
@@ -895,38 +895,86 @@ func autoUpdateImages(d *Daemon) {
 			continue
 		}
 
+		// Get the IDs of all storage pools on which a storage volume
+		// for the requested image currently exists.
+		poolIDs, err := dbImageGetPools(d.db, fp)
+		if err != nil {
+			continue
+		}
+
+		// Translate the IDs to poolNames.
+		poolNames, err := dbImageGetPoolNamesFromIDs(d.db, poolIDs)
+		if err != nil {
+			continue
+		}
+
 		shared.LogDebug("Processing image", log.Ctx{"fp": fp, "server": source.Server, "protocol": source.Protocol, "alias": source.Alias})
 
-		hash, err := d.ImageDownload(nil, source.Server, source.Protocol, "", "", source.Alias, false, true)
-		if hash == fp {
-			shared.LogDebug("Already up to date", log.Ctx{"fp": fp})
-			continue
-		} else if err != nil {
-			shared.LogError("Failed to update the image", log.Ctx{"err": err, "fp": fp})
+		// Update the image on each pool where it currently exists.
+		for _, poolName := range poolNames {
+			hash, err := d.ImageDownload(nil, source.Server, source.Protocol, "", "", source.Alias, false, true, poolName)
+			if hash == fp {
+				shared.LogDebug("Already up to date", log.Ctx{"fp": fp})
+				continue
+			} else if err != nil {
+				shared.LogError("Failed to update the image", log.Ctx{"err": err, "fp": fp})
+				continue
+			}
+
+			newId, _, err := dbImageGet(d.db, hash, false, true)
+			if err != nil {
+				shared.LogError("Error loading image", log.Ctx{"err": err, "fp": hash})
+				continue
+			}
+
+			err = dbImageLastAccessUpdate(d.db, hash, info.LastUsedAt)
+			if err != nil {
+				shared.LogError("Error setting last use date", log.Ctx{"err": err, "fp": hash})
+				continue
+			}
+
+			err = dbImageAliasesMove(d.db, id, newId)
+			if err != nil {
+				shared.LogError("Error moving aliases", log.Ctx{"err": err, "fp": hash})
+				continue
+			}
+
+			err = doDeleteImageFromPool(d, fp, poolName)
+			if err != nil {
+				shared.LogError("Error deleting image", log.Ctx{"err": err, "fp": fp})
+			}
+		}
+
+		nImg, err := dbImageGetPools(d.db, fp)
+		if err != nil {
 			continue
 		}
 
-		newId, _, err := dbImageGet(d.db, hash, false, true)
-		if err != nil {
-			shared.LogError("Error loading image", log.Ctx{"err": err, "fp": hash})
+		if len(nImg) > 0 {
 			continue
 		}
 
-		err = dbImageLastAccessUpdate(d.db, hash, info.LastUsedDate)
-		if err != nil {
-			shared.LogError("Error setting last use date", log.Ctx{"err": err, "fp": hash})
-			continue
+		// Remove main image file.
+		fname := shared.VarPath("images", fp)
+		if shared.PathExists(fname) {
+			err = os.Remove(fname)
+			if err != nil {
+				shared.LogDebugf("Error deleting image file %s: %s", fname, err)
+			}
 		}
 
-		err = dbImageAliasesMove(d.db, id, newId)
-		if err != nil {
-			shared.LogError("Error moving aliases", log.Ctx{"err": err, "fp": hash})
-			continue
+		// Remove the rootfs file for the image.
+		fname = shared.VarPath("images", fp) + ".rootfs"
+		if shared.PathExists(fname) {
+			err = os.Remove(fname)
+			if err != nil {
+				shared.LogDebugf("Error deleting image file %s: %s", fname, err)
+			}
 		}
 
-		err = doDeleteImage(d, fp)
-		if err != nil {
-			shared.LogError("Error deleting image", log.Ctx{"err": err, "fp": fp})
+		// Remove the database entry for the image.
+		if err = dbImageDelete(d.db, id); err != nil {
+			shared.LogDebugf("Error deleting image from database %s: %s", fname, err)
 		}
 	}
 
@@ -936,7 +984,7 @@ func autoUpdateImages(d *Daemon) {
 func pruneExpiredImages(d *Daemon) {
 	shared.LogInfof("Pruning expired images")
 
-	// Get the list of expires images
+	// Get the list of expired images.
 	expiry := daemonConfig["images.remote_cache_expiry"].GetInt64()
 	images, err := dbImagesGetExpired(d.db, expiry)
 	if err != nil {
@@ -946,52 +994,69 @@ func pruneExpiredImages(d *Daemon) {
 
 	// Delete them
 	for _, fp := range images {
-		if err := doDeleteImage(d, fp); err != nil {
-			shared.LogError("Error deleting image", log.Ctx{"err": err, "fp": fp})
+		// Get the IDs of all storage pools on which a storage volume
+		// for the requested image currently exists.
+		poolIDs, err := dbImageGetPools(d.db, fp)
+		if err != nil {
+			continue
+		}
+
+		// Translate the IDs to poolNames.
+		poolNames, err := dbImageGetPoolNamesFromIDs(d.db, poolIDs)
+		if err != nil {
+			continue
+		}
+
+		for _, pool := range poolNames {
+			err := doDeleteImageFromPool(d, fp, pool)
+			if err != nil {
+				shared.LogDebugf("Error deleting image %s from storage pool %: %s", fp, pool, err)
+				continue
+			}
+		}
+
+		// Remove main image file.
+		fname := shared.VarPath("images", fp)
+		if shared.PathExists(fname) {
+			err = os.Remove(fname)
+			if err != nil {
+				shared.LogDebugf("Error deleting image file %s: %s", fname, err)
+			}
+		}
+
+		// Remove the rootfs file for the image.
+		fname = shared.VarPath("images", fp) + ".rootfs"
+		if shared.PathExists(fname) {
+			err = os.Remove(fname)
+			if err != nil {
+				shared.LogDebugf("Error deleting image file %s: %s", fname, err)
+			}
+		}
+
+		imgID, _, err := dbImageGet(d.db, fp, false, false)
+		if err != nil {
+			shared.LogDebugf("Error retrieving image info %s: %s", fp, err)
+		}
+
+		// Remove the database entry for the image.
+		if err = dbImageDelete(d.db, imgID); err != nil {
+			shared.LogDebugf("Error deleting image %s from database: %s", fp, err)
 		}
 	}
 
 	shared.LogInfof("Done pruning expired images")
 }
 
-func doDeleteImage(d *Daemon, fingerprint string) error {
-	id, imgInfo, err := dbImageGet(d.db, fingerprint, false, false)
+func doDeleteImageFromPool(d *Daemon, fingerprint string, storagePool string) error {
+	// Initialize a new storage interface.
+	s, err := storagePoolVolumeImageInit(d, storagePool, fingerprint)
 	if err != nil {
 		return err
 	}
 
-	// get storage before deleting images/$fp because we need to
-	// look at the path
-	s, err := storageForImage(d, imgInfo)
+	// Delete the storage volume for the image from the storage pool.
+	err = s.ImageDelete(fingerprint)
 	if err != nil {
-		shared.LogError("error detecting image storage backend", log.Ctx{"fingerprint": imgInfo.Fingerprint, "err": err})
-	} else {
-		// Remove the image from storage backend
-		if err = s.ImageDelete(imgInfo.Fingerprint); err != nil {
-			shared.LogError("error deleting the image from storage backend", log.Ctx{"fingerprint": imgInfo.Fingerprint, "err": err})
-		}
-	}
-
-	// Remove main image file
-	fname := shared.VarPath("images", imgInfo.Fingerprint)
-	if shared.PathExists(fname) {
-		err = os.Remove(fname)
-		if err != nil {
-			shared.LogDebugf("Error deleting image file %s: %s", fname, err)
-		}
-	}
-
-	// Remove the rootfs file
-	fname = shared.VarPath("images", imgInfo.Fingerprint) + ".rootfs"
-	if shared.PathExists(fname) {
-		err = os.Remove(fname)
-		if err != nil {
-			shared.LogDebugf("Error deleting image file %s: %s", fname, err)
-		}
-	}
-
-	// Remove the DB entry
-	if err = dbImageDelete(d.db, id); err != nil {
 		return err
 	}
 
@@ -1001,8 +1066,55 @@ func doDeleteImage(d *Daemon, fingerprint string) error {
 func imageDelete(d *Daemon, r *http.Request) Response {
 	fingerprint := mux.Vars(r)["fingerprint"]
 
+	deleteFromAllPools := func() error {
+		// Use the fingerprint we received in a LIKE query and use the full
+		// fingerprint we receive from the database in all further queries.
+		imgID, imgInfo, err := dbImageGet(d.db, fingerprint, false, false)
+		if err != nil {
+			return err
+		}
+
+		poolIDs, err := dbImageGetPools(d.db, imgInfo.Fingerprint)
+		if err != nil {
+			return err
+		}
+
+		pools, err := dbImageGetPoolNamesFromIDs(d.db, poolIDs)
+		if err != nil {
+			return err
+		}
+
+		for _, pool := range pools {
+			err := doDeleteImageFromPool(d, imgInfo.Fingerprint, pool)
+			if err != nil {
+				return err
+			}
+		}
+
+		// Remove main image file.
+		fname := shared.VarPath("images", imgInfo.Fingerprint)
+		if shared.PathExists(fname) {
+			err = os.Remove(fname)
+			if err != nil {
+				shared.LogDebugf("Error deleting image file %s: %s", fname, err)
+			}
+		}
+
+		// Remove the rootfs file for the image.
+		fname = shared.VarPath("images", imgInfo.Fingerprint) + ".rootfs"
+		if shared.PathExists(fname) {
+			err = os.Remove(fname)
+			if err != nil {
+				shared.LogDebugf("Error deleting image file %s: %s", fname, err)
+			}
+		}
+
+		// Remove the database entry for the image.
+		return dbImageDelete(d.db, imgID)
+	}
+
 	rmimg := func(op *operation) error {
-		return doDeleteImage(d, fingerprint)
+		return deleteFromAllPools()
 	}
 
 	resources := map[string][]string{}
@@ -1016,7 +1128,7 @@ func imageDelete(d *Daemon, r *http.Request) Response {
 	return OperationResponse(op)
 }
 
-func doImageGet(d *Daemon, fingerprint string, public bool) (*shared.ImageInfo, Response) {
+func doImageGet(d *Daemon, fingerprint string, public bool) (*api.Image, Response) {
 	_, imgInfo, err := dbImageGet(d.db, fingerprint, public, false)
 	if err != nil {
 		return nil, SmartError(err)
@@ -1073,12 +1185,6 @@ func imageGet(d *Daemon, r *http.Request) Response {
 	return SyncResponseETag(true, info, etag)
 }
 
-type imagePutReq struct {
-	Properties map[string]string `json:"properties"`
-	Public     bool              `json:"public"`
-	AutoUpdate bool              `json:"auto_update"`
-}
-
 func imagePut(d *Daemon, r *http.Request) Response {
 	// Get current value
 	fingerprint := mux.Vars(r)["fingerprint"]
@@ -1094,12 +1200,12 @@ func imagePut(d *Daemon, r *http.Request) Response {
 		return PreconditionFailed(err)
 	}
 
-	req := imagePutReq{}
+	req := api.ImagePut{}
 	if err := json.NewDecoder(r.Body).Decode(&req); err != nil {
 		return BadRequest(err)
 	}
 
-	err = dbImageUpdate(d.db, id, info.Filename, info.Size, req.Public, req.AutoUpdate, info.Architecture, info.CreationDate, info.ExpiryDate, req.Properties)
+	err = dbImageUpdate(d.db, id, info.Filename, info.Size, req.Public, req.AutoUpdate, info.Architecture, info.CreatedAt, info.ExpiresAt, req.Properties)
 	if err != nil {
 		return SmartError(err)
 	}
@@ -1135,7 +1241,7 @@ func imagePatch(d *Daemon, r *http.Request) Response {
 		return BadRequest(err)
 	}
 
-	req := imagePutReq{}
+	req := api.ImagePut{}
 	if err := json.NewDecoder(rdr2).Decode(&req); err != nil {
 		return BadRequest(err)
 	}
@@ -1165,7 +1271,7 @@ func imagePatch(d *Daemon, r *http.Request) Response {
 		info.Properties = properties
 	}
 
-	err = dbImageUpdate(d.db, id, info.Filename, info.Size, info.Public, info.AutoUpdate, info.Architecture, info.CreationDate, info.ExpiryDate, info.Properties)
+	err = dbImageUpdate(d.db, id, info.Filename, info.Size, info.Public, info.AutoUpdate, info.Architecture, info.CreatedAt, info.ExpiresAt, info.Properties)
 	if err != nil {
 		return SmartError(err)
 	}
@@ -1175,19 +1281,8 @@ func imagePatch(d *Daemon, r *http.Request) Response {
 
 var imageCmd = Command{name: "images/{fingerprint}", untrustedGet: true, get: imageGet, put: imagePut, delete: imageDelete, patch: imagePatch}
 
-type aliasPostReq struct {
-	Name        string `json:"name"`
-	Description string `json:"description"`
-	Target      string `json:"target"`
-}
-
-type aliasPutReq struct {
-	Description string `json:"description"`
-	Target      string `json:"target"`
-}
-
 func aliasesPost(d *Daemon, r *http.Request) Response {
-	req := aliasPostReq{}
+	req := api.ImageAliasesPost{}
 	if err := json.NewDecoder(r.Body).Decode(&req); err != nil {
 		return BadRequest(err)
 	}
@@ -1212,7 +1307,7 @@ func aliasesPost(d *Daemon, r *http.Request) Response {
 		return InternalError(err)
 	}
 
-	return SyncResponseLocation(true, nil, fmt.Sprintf("/%s/images/aliases/%s", shared.APIVersion, req.Name))
+	return SyncResponseLocation(true, nil, fmt.Sprintf("/%s/images/aliases/%s", version.APIVersion, req.Name))
 }
 
 func aliasesGet(d *Daemon, r *http.Request) Response {
@@ -1227,11 +1322,11 @@ func aliasesGet(d *Daemon, r *http.Request) Response {
 		return BadRequest(err)
 	}
 	responseStr := []string{}
-	responseMap := shared.ImageAliases{}
+	responseMap := []api.ImageAliasesEntry{}
 	for _, res := range results {
 		name = res[0].(string)
 		if !recursion {
-			url := fmt.Sprintf("/%s/images/aliases/%s", shared.APIVersion, name)
+			url := fmt.Sprintf("/%s/images/aliases/%s", version.APIVersion, name)
 			responseStr = append(responseStr, url)
 
 		} else {
@@ -1290,7 +1385,7 @@ func aliasPut(d *Daemon, r *http.Request) Response {
 		return PreconditionFailed(err)
 	}
 
-	req := aliasPutReq{}
+	req := api.ImageAliasesEntryPut{}
 	if err := json.NewDecoder(r.Body).Decode(&req); err != nil {
 		return BadRequest(err)
 	}
@@ -1367,7 +1462,7 @@ func aliasPatch(d *Daemon, r *http.Request) Response {
 func aliasPost(d *Daemon, r *http.Request) Response {
 	name := mux.Vars(r)["name"]
 
-	req := aliasPostReq{}
+	req := api.ImageAliasesEntryPost{}
 	if err := json.NewDecoder(r.Body).Decode(&req); err != nil {
 		return BadRequest(err)
 	}
@@ -1388,7 +1483,7 @@ func aliasPost(d *Daemon, r *http.Request) Response {
 		return SmartError(err)
 	}
 
-	return SyncResponseLocation(true, nil, fmt.Sprintf("/%s/images/aliases/%s", shared.APIVersion, req.Name))
+	return SyncResponseLocation(true, nil, fmt.Sprintf("/%s/images/aliases/%s", version.APIVersion, req.Name))
 }
 
 func imageExport(d *Daemon, r *http.Request) Response {
