@@ -11,8 +11,8 @@ import (
 	"net/http"
 	"net/url"
 	"os"
+	"os/exec"
 	"path/filepath"
-	"strconv"
 	"strings"
 	"sync"
 
@@ -167,8 +167,9 @@ func NewMigrationSource(c container) (*migrationSourceWs, error) {
 	}
 
 	if c.IsRunning() {
-		if err := findCriu("source"); err != nil {
-			return nil, err
+		_, err := exec.LookPath("criu")
+		if err != nil {
+			return nil, fmt.Errorf("Unable to perform container live migration. CRIU isn't installed on the source server.")
 		}
 
 		ret.live = true
@@ -291,15 +292,16 @@ func (s *migrationSourceWs) Do(migrateOp *operation) error {
 	criuType := CRIUType_CRIU_RSYNC.Enum()
 	if !s.live {
 		criuType = nil
-
-		err := s.container.StorageStart()
-		if err != nil {
-			return err
-		}
-
-		defer s.container.StorageStop()
 	}
 
+	// Storage needs to start unconditionally now, since we need to
+	// initialize a new storage interface.
+	err := s.container.StorageStart()
+	if err != nil {
+		return err
+	}
+	defer s.container.StorageStop()
+
 	idmaps := make([]*IDMapType, 0)
 
 	idmapset := s.container.IdmapSet()
@@ -375,10 +377,12 @@ func (s *migrationSourceWs) Do(migrateOp *operation) error {
 		return err
 	}
 
-	if err := driver.SendWhileRunning(s.fsConn); err != nil {
+	if err := driver.SendWhileRunning(s.fsConn, migrateOp); err != nil {
 		return abort(err)
 	}
 
+	restoreSuccess := make(chan bool, 1)
+	dumpSuccess := make(chan error, 1)
 	if s.live {
 		if header.Criu == nil {
 			return abort(fmt.Errorf("Got no CRIU socket type for live migration"))
@@ -390,7 +394,6 @@ func (s *migrationSourceWs) Do(migrateOp *operation) error {
 		if err != nil {
 			return abort(err)
 		}
-		defer os.RemoveAll(checkpointDir)
 
 		if lxc.VersionAtLeast(2, 0, 4) {
 			/* What happens below is slightly convoluted. Due to various
@@ -412,6 +415,7 @@ func (s *migrationSourceWs) Do(migrateOp *operation) error {
 			dumpDone := make(chan bool, 1)
 			actionScriptOpSecret, err := shared.RandomCryptoString()
 			if err != nil {
+				os.RemoveAll(checkpointDir)
 				return abort(err)
 			}
 
@@ -420,13 +424,9 @@ func (s *migrationSourceWs) Do(migrateOp *operation) error {
 				nil,
 				nil,
 				func(op *operation) error {
-					_, err := migrateOp.WaitFinal(-1)
-					if err != nil {
-						return err
-					}
-
-					if migrateOp.status != shared.Success {
-						return fmt.Errorf("restore failed: %s", op.status.String())
+					result := <-restoreSuccess
+					if !result {
+						return fmt.Errorf("restore failed, failing CRIU")
 					}
 					return nil
 				},
@@ -453,32 +453,36 @@ func (s *migrationSourceWs) Do(migrateOp *operation) error {
 				},
 			)
 			if err != nil {
+				os.RemoveAll(checkpointDir)
 				return abort(err)
 			}
 
 			if err := writeActionScript(checkpointDir, actionScriptOp.url, actionScriptOpSecret); err != nil {
+				os.RemoveAll(checkpointDir)
 				return abort(err)
 			}
 
 			_, err = actionScriptOp.Run()
 			if err != nil {
+				os.RemoveAll(checkpointDir)
 				return abort(err)
 			}
 
-			migrateDone := make(chan error, 1)
 			go func() {
-				migrateDone <- s.container.Migrate(lxc.MIGRATE_DUMP, checkpointDir, "migration", true, true)
+				dumpSuccess <- s.container.Migrate(lxc.MIGRATE_DUMP, checkpointDir, "migration", true, true)
+				os.RemoveAll(checkpointDir)
 			}()
 
 			select {
 			/* the checkpoint failed, let's just abort */
-			case err = <-migrateDone:
+			case err = <-dumpSuccess:
 				return abort(err)
 			/* the dump finished, let's continue on to the restore */
 			case <-dumpDone:
 				shared.LogDebugf("Dump finished, continuing with restore...")
 			}
 		} else {
+			defer os.RemoveAll(checkpointDir)
 			if err := s.container.Migrate(lxc.MIGRATE_DUMP, checkpointDir, "migration", true, false); err != nil {
 				return abort(err)
 			}
@@ -491,7 +495,7 @@ func (s *migrationSourceWs) Do(migrateOp *operation) error {
 		 * no reason to do these in parallel. In the future when we're using
 		 * p.haul's protocol, it will make sense to do these in parallel.
 		 */
-		if err := RsyncSend(shared.AddSlash(checkpointDir), s.criuConn); err != nil {
+		if err := RsyncSend(shared.AddSlash(checkpointDir), s.criuConn, nil); err != nil {
 			return abort(err)
 		}
 
@@ -508,6 +512,14 @@ func (s *migrationSourceWs) Do(migrateOp *operation) error {
 		return err
 	}
 
+	if s.live {
+		restoreSuccess <- *msg.Success
+		err := <-dumpSuccess
+		if err != nil {
+			shared.LogErrorf("dump failed after successful restore?: %q", err)
+		}
+	}
+
 	if !*msg.Success {
 		return fmt.Errorf(*msg.Message)
 	}
@@ -585,11 +597,11 @@ func NewMigrationSink(args *MigrationSinkArgs) (*migrationSink, error) {
 		sink.src.live = ok
 	}
 
-	err = findCriu("destination")
+	_, err = exec.LookPath("criu")
 	if sink.push && sink.dest.live && err != nil {
-		return nil, err
+		return nil, fmt.Errorf("Unable to perform container live migration. CRIU isn't installed on the destination server.")
 	} else if sink.src.live && err != nil {
-		return nil, err
+		return nil, fmt.Errorf("Unable to perform container live migration. CRIU isn't installed on the destination server.")
 	}
 
 	return &sink, nil
@@ -784,7 +796,8 @@ func (c *migrationSink) Do(migrateOp *operation) error {
 			} else {
 				fsConn = c.src.fsConn
 			}
-			if err := mySink(live, c.src.container, header.Snapshots, fsConn, srcIdmap); err != nil {
+
+			if err := mySink(live, c.src.container, snapshots, fsConn, srcIdmap, migrateOp); err != nil {
 				fsTransfer <- err
 				return
 			}
@@ -813,7 +826,7 @@ func (c *migrationSink) Do(migrateOp *operation) error {
 			} else {
 				criuConn = c.src.criuConn
 			}
-			if err := RsyncRecv(shared.AddSlash(imagesDir), criuConn); err != nil {
+			if err := RsyncRecv(shared.AddSlash(imagesDir), criuConn, nil); err != nil {
 				restore <- err
 				return
 			}
@@ -866,46 +879,3 @@ func (c *migrationSink) Do(migrateOp *operation) error {
 		}
 	}
 }
-
-/*
- * Similar to forkstart, this is called when lxd is invoked as:
- *
- *    lxd forkmigrate <container> <lxcpath> <path_to_config> <path_to_criu_images> <preserves_inodes>
- *
- * liblxc's restore() sets up the processes in such a way that the monitor ends
- * up being a child of the process that calls it, in our case lxd. However, we
- * really want the monitor to be daemonized, so we fork again. Additionally, we
- * want to fork for the same reasons we do forkstart (i.e. reduced memory
- * footprint when we fork tasks that will never free golang's memory, etc.)
- */
-func MigrateContainer(args []string) error {
-	if len(args) != 6 {
-		return fmt.Errorf("Bad arguments %q", args)
-	}
-
-	name := args[1]
-	lxcpath := args[2]
-	configPath := args[3]
-	imagesDir := args[4]
-	preservesInodes, err := strconv.ParseBool(args[5])
-
-	c, err := lxc.NewContainer(name, lxcpath)
-	if err != nil {
-		return err
-	}
-
-	if err := c.LoadConfigFile(configPath); err != nil {
-		return err
-	}
-
-	/* see https://github.com/golang/go/issues/13155, startContainer, and dc3a229 */
-	os.Stdin.Close()
-	os.Stdout.Close()
-	os.Stderr.Close()
-
-	return c.Migrate(lxc.MIGRATE_RESTORE, lxc.MigrateOptions{
-		Directory:       imagesDir,
-		Verbose:         true,
-		PreservesInodes: preservesInodes,
-	})
-}
