@@ -12,10 +12,6 @@ btrfs_setup() {
     echo "Couldn't find the btrfs binary"; false
   fi
 
-  truncate -s 100G "${TEST_DIR}/$(basename "${LXD_DIR}").btrfs"
-  mkfs.btrfs "${TEST_DIR}/$(basename "${LXD_DIR}").btrfs"
-
-  mount -o loop "${TEST_DIR}/$(basename "${LXD_DIR}").btrfs" "${LXD_DIR}"
 }
 
 btrfs_configure() {
@@ -24,6 +20,9 @@ btrfs_configure() {
 
   LXD_DIR=$1
 
+  lxc storage create "lxdtest-$(basename "${LXD_DIR}")" btrfs size=100GB
+  lxc profile device add default root disk path="/" pool="lxdtest-$(basename "${LXD_DIR}")"
+
   echo "==> Configuring btrfs backend in ${LXD_DIR}"
 }
 
@@ -34,7 +33,4 @@ btrfs_teardown() {
   LXD_DIR=$1
 
   echo "==> Tearing down btrfs backend in ${LXD_DIR}"
-
-  umount -l "${LXD_DIR}"
-  rm -f "${TEST_DIR}/$(basename "${LXD_DIR}").btrfs"
 }
