@@ -17,11 +17,11 @@ limitations under the License.
 package datacenter
 
 import (
+	"context"
 	"flag"
 
 	"github.com/vmware/govmomi/govc/cli"
 	"github.com/vmware/govmomi/govc/flags"
-	"golang.org/x/net/context"
 )
 
 type create struct {
@@ -49,7 +49,7 @@ func (cmd *create) Process(ctx context.Context) error {
 }
 
 func (cmd *create) Run(ctx context.Context, f *flag.FlagSet) error {
-	folder, err := cmd.FolderOrRoot()
+	folder, err := cmd.FolderOrDefault("/")
 	if err != nil {
 		return err
 	}
