@@ -154,7 +154,11 @@ func toInfoList(devices object.VirtualDeviceList) []infoDevice {
 	var res []infoDevice
 
 	for _, device := range devices {
-		res = append(res, infoDevice{devices.Name(device), device})
+		res = append(res, infoDevice{
+			Name:              devices.Name(device),
+			Type:              devices.TypeName(device),
+			BaseVirtualDevice: device,
+		})
 	}
 
 	return res
@@ -162,6 +166,7 @@ func toInfoList(devices object.VirtualDeviceList) []infoDevice {
 
 type infoDevice struct {
 	Name string
+	Type string
 	types.BaseVirtualDevice
 }
 
@@ -173,7 +178,7 @@ func (d *infoDevice) MarshalJSON() ([]byte, error) {
 
 	// TODO: make use of "inline" tag if it comes to be: https://github.com/golang/go/issues/6213
 
-	return append([]byte(fmt.Sprintf(`{"Name":"%s",`, d.Name)), b[1:]...), err
+	return append([]byte(fmt.Sprintf(`{"Name":"%s","Type":"%s",`, d.Name, d.Type)), b[1:]...), err
 }
 
 type infoResult struct {
