@@ -1,5 +1,5 @@
 /*
-Copyright (c) 2014-2015 VMware, Inc. All Rights Reserved.
+Copyright (c) 2014-2018 VMware, Inc. All Rights Reserved.
 
 Licensed under the Apache License, Version 2.0 (the "License");
 you may not use this file except in compliance with the License.
@@ -25,10 +25,12 @@ import (
 	"flag"
 	"fmt"
 	"io/ioutil"
+	"net/http"
 	"net/url"
 	"os"
 	"path/filepath"
 	"strings"
+	"time"
 
 	"github.com/vmware/govmomi/session"
 	"github.com/vmware/govmomi/vim25"
@@ -70,9 +72,9 @@ type ClientFlag struct {
 	vimVersion    string
 	tlsCaCerts    string
 	tlsKnownHosts string
-	tlsHostHash   string
+	client        *vim25.Client
 
-	client *vim25.Client
+	Login func(context.Context, *vim25.Client) error
 }
 
 var (
@@ -92,6 +94,7 @@ func NewClientFlag(ctx context.Context) (*ClientFlag, context.Context) {
 	}
 
 	v := &ClientFlag{}
+	v.Login = v.login
 	v.DebugFlag, ctx = NewDebugFlag(ctx)
 	ctx = context.WithValue(ctx, clientFlagKey, v)
 	return v, ctx
@@ -275,6 +278,18 @@ func (flag *ClientFlag) configure(sc *soap.Client) (soap.RoundTripper, error) {
 		return nil, err
 	}
 
+	if t, ok := sc.Transport.(*http.Transport); ok {
+		var err error
+
+		value := os.Getenv("GOVC_TLS_HANDSHAKE_TIMEOUT")
+		if value != "" {
+			t.TLSHandshakeTimeout, err = time.ParseDuration(value)
+			if err != nil {
+				return nil, err
+			}
+		}
+	}
+
 	// Retry twice when a temporary I/O error occurs.
 	// This means a maximum of 3 attempts.
 	return vim25.Retry(sc, vim25.TemporaryNetworkError(3)), nil
@@ -385,13 +400,42 @@ func (flag *ClientFlag) SetRootCAs(c *soap.Client) error {
 	return nil
 }
 
+func (flag *ClientFlag) login(ctx context.Context, c *vim25.Client) error {
+	m := session.NewManager(c)
+	u := flag.url.User
+
+	if u.Username() == "" {
+		if !c.IsVC() {
+			// If no username is provided, try to acquire a local ticket.
+			// When invoked remotely, ESX returns an InvalidRequestFault.
+			// So, rather than return an error here, fallthrough to Login() with the original User to
+			// to avoid what would be a confusing error message.
+			luser, lerr := flag.localTicket(ctx, m)
+			if lerr == nil {
+				// We are running directly on an ESX or Workstation host and can use the ticket with Login()
+				u = luser
+			} else {
+				flag.persist = true // Not persisting, but this avoids the call to Logout()
+				return nil          // Avoid SaveSession for non-authenticated session
+			}
+		}
+	}
+
+	if flag.cert != "" {
+		err := m.LoginExtensionByCertificate(ctx, u.Username(), "")
+		if err != nil {
+			return err
+		}
+	}
+
+	return m.Login(ctx, u)
+}
+
 func (flag *ClientFlag) newClient() (*vim25.Client, error) {
 	ctx := context.TODO()
 	sc := soap.NewClient(flag.url, flag.insecure)
-	isTunnel := false
 
 	if flag.cert != "" {
-		isTunnel = true
 		cert, err := tls.LoadX509KeyPair(flag.cert, flag.key)
 		if err != nil {
 			return nil, err
@@ -413,35 +457,11 @@ func (flag *ClientFlag) newClient() (*vim25.Client, error) {
 	// Set client, since we didn't pass it in the constructor
 	c.Client = sc
 
-	m := session.NewManager(c)
-	u := flag.url.User
-
-	if u.Username() == "" {
-		// Assume we are running on an ESX or Workstation host if no username is provided
-		u, err = flag.localTicket(ctx, m)
-		if err != nil {
-			return nil, err
-		}
-	}
-
-	if isTunnel {
-		err = m.LoginExtensionByCertificate(ctx, u.Username(), "")
-		if err != nil {
-			return nil, err
-		}
-	} else {
-		err = m.Login(ctx, u)
-		if err != nil {
-			return nil, err
-		}
-	}
-
-	err = flag.saveClient(c)
-	if err != nil {
+	if err := flag.Login(ctx, c); err != nil {
 		return nil, err
 	}
 
-	return c, nil
+	return c, flag.saveClient(c)
 }
 
 func (flag *ClientFlag) localTicket(ctx context.Context, m *session.Manager) (*url.Userinfo, error) {
@@ -458,6 +478,11 @@ func (flag *ClientFlag) localTicket(ctx context.Context, m *session.Manager) (*u
 	return url.UserPassword(ticket.UserName, string(password)), nil
 }
 
+func isDevelopmentVersion(apiVersion string) bool {
+	// Skip version check for development builds which can be in the form of "r4A70F" or "6.5.x"
+	return strings.Count(apiVersion, ".") == 0 || strings.HasSuffix(apiVersion, ".x")
+}
+
 // apiVersionValid returns whether or not the API version supported by the
 // server the client is connected to is not recent enough.
 func apiVersionValid(c *vim25.Client, minVersionString string) error {
@@ -467,23 +492,22 @@ func apiVersionValid(c *vim25.Client, minVersionString string) error {
 	}
 
 	apiVersion := c.ServiceContent.About.ApiVersion
-	if strings.HasSuffix(apiVersion, ".x") {
-		// Skip version check for development builds
+	if isDevelopmentVersion(apiVersion) {
 		return nil
 	}
 
 	realVersion, err := ParseVersion(apiVersion)
 	if err != nil {
-		return err
+		return fmt.Errorf("Error parsing API version %q: %s", apiVersion, err)
 	}
 
 	minVersion, err := ParseVersion(minVersionString)
 	if err != nil {
-		return err
+		return fmt.Errorf("Error parsing %s=%q: %s", envMinAPIVersion, minVersionString, err)
 	}
 
 	if !minVersion.Lte(realVersion) {
-		err = fmt.Errorf("Require API version %s, connected to API version %s (set %s to override)",
+		err = fmt.Errorf("Require API version %q, connected to API version %q (set %s to override)",
 			minVersionString,
 			c.ServiceContent.About.ApiVersion,
 			envMinAPIVersion)
