@@ -17,10 +17,9 @@ limitations under the License.
 package importx
 
 import (
+	"context"
 	"flag"
 
-	"golang.org/x/net/context"
-
 	"github.com/vmware/govmomi/govc/cli"
 	"github.com/vmware/govmomi/object"
 	"github.com/vmware/govmomi/vim25/types"
@@ -44,7 +43,10 @@ func (cmd *ova) Run(ctx context.Context, f *flag.FlagSet) error {
 		return err
 	}
 
-	cmd.Archive = &TapeArchive{fpath}
+	archive := &TapeArchive{path: fpath}
+	archive.Client = cmd.Client
+
+	cmd.Archive = archive
 
 	moref, err := cmd.Import(fpath)
 	if err != nil {
