@@ -322,13 +322,13 @@ var devices = VirtualDeviceList([]types.BaseVirtualDevice{
 		},
 		StorageIOAllocation: &types.StorageIOAllocationInfo{
 			DynamicData: types.DynamicData{},
-			Limit:       -1,
+			Limit:       types.NewInt64(-1),
 			Shares: &types.SharesInfo{
 				DynamicData: types.DynamicData{},
 				Shares:      1000,
 				Level:       "normal",
 			},
-			Reservation: 0,
+			Reservation: types.NewInt32(0),
 		},
 		DiskObjectId:          "3-3000",
 		VFlashCacheConfigInfo: (*types.VirtualDiskVFlashCacheConfigInfo)(nil),
@@ -376,13 +376,13 @@ var devices = VirtualDeviceList([]types.BaseVirtualDevice{
 		},
 		StorageIOAllocation: &types.StorageIOAllocationInfo{
 			DynamicData: types.DynamicData{},
-			Limit:       -1,
+			Limit:       types.NewInt64(-1),
 			Shares: &types.SharesInfo{
 				DynamicData: types.DynamicData{},
 				Shares:      1000,
 				Level:       "normal",
 			},
-			Reservation: 0,
+			Reservation: types.NewInt32(0),
 		},
 		DiskObjectId:          "3-3002",
 		VFlashCacheConfigInfo: (*types.VirtualDiskVFlashCacheConfigInfo)(nil),
@@ -731,7 +731,7 @@ func TestSerialPort(t *testing.T) {
 	if err != nil {
 		t.Error(err)
 	}
-	devices.ConnectSerialPort(device, "telnet://:33233", false)
+	devices.ConnectSerialPort(device, "telnet://:33233", false, "")
 }
 
 func TestPrimaryMacAddress(t *testing.T) {
@@ -793,6 +793,10 @@ func TestBootOrder(t *testing.T) {
 	if len(list.BootOrder([]string{DeviceTypeDisk})) != 0 {
 		t.Error("expected 0 disks")
 	}
+
+	if len(list.BootOrder([]string{DeviceTypeNone})) != 1 {
+		t.Error("expected 1")
+	}
 }
 
 func TestName(t *testing.T) {
@@ -851,3 +855,23 @@ func TestName(t *testing.T) {
 		}
 	}
 }
+
+func TestChildDisk(t *testing.T) {
+	disks := devices.SelectByType((*types.VirtualDisk)(nil))
+
+	for _, disk := range disks {
+		child := disks.ChildDisk(disk.(*types.VirtualDisk))
+		name := child.Backing.(*types.VirtualDiskFlatVer2BackingInfo).VirtualDeviceFileBackingInfo.FileName
+
+		p := new(DatastorePath)
+		p.FromString(name)
+
+		if p.Datastore != "datastore1" {
+			t.Fatal(p.Datastore)
+		}
+
+		if p.Path != "" {
+			t.Fatal(p.Path)
+		}
+	}
+}
