@@ -621,162 +621,6 @@ func (s *S) TestTxnQueueStashStressTest(c *C) {
 	}
 }
 
-func (s *S) checkTxnQueueLength(c *C, expectedQueueLength int) {
-	txn.SetDebug(false)
-	txn.SetChaos(txn.Chaos{
-		KillChance: 1,
-		Breakpoint: "set-applying",
-	})
-	defer txn.SetChaos(txn.Chaos{})
-	err := s.accounts.Insert(M{"_id": 0, "balance": 100})
-	c.Assert(err, IsNil)
-	ops := []txn.Op{{
-		C:      "accounts",
-		Id:     0,
-		Update: M{"$inc": M{"balance": 100}},
-	}}
-	for i := 0; i < expectedQueueLength; i++ {
-		err := s.runner.Run(ops, "", nil)
-		c.Assert(err, Equals, txn.ErrChaos)
-	}
-	txn.SetDebug(true)
-	// Now that we've filled up the queue, we should see that there are 1000
-	// items in the queue, and the error applying a new one will change.
-	var doc bson.M
-	err = s.accounts.FindId(0).One(&doc)
-	c.Assert(err, IsNil)
-	c.Check(len(doc["txn-queue"].([]interface{})), Equals, expectedQueueLength)
-	err = s.runner.Run(ops, "", nil)
-	c.Check(err, ErrorMatches, `txn-queue for 0 in "accounts" has too many transactions \(\d+\)`)
-	// The txn-queue should not have grown
-	err = s.accounts.FindId(0).One(&doc)
-	c.Assert(err, IsNil)
-	c.Check(len(doc["txn-queue"].([]interface{})), Equals, expectedQueueLength)
-}
-
-func (s *S) TestTxnQueueDefaultMaxSize(c *C) {
-	s.runner.SetOptions(txn.DefaultRunnerOptions())
-	s.checkTxnQueueLength(c, 1000)
-}
-
-func (s *S) TestTxnQueueCustomMaxSize(c *C) {
-	opts := txn.DefaultRunnerOptions()
-	opts.MaxTxnQueueLength = 100
-	s.runner.SetOptions(opts)
-	s.checkTxnQueueLength(c, 100)
-}
-
-func (s *S) TestTxnQueueMultipleDocs(c *C) {
-	expectedLength := 100
-	maxDocs := 110
-	opts := txn.DefaultRunnerOptions()
-	opts.MaxTxnQueueLength = expectedLength
-	s.runner.SetOptions(opts)
-	txn.SetDebug(false)
-	createOps := []txn.Op{{
-		C:      "accounts",
-		Id:     0,
-		Insert: M{"balance": 1000},
-	}}
-	for i := 1; i < maxDocs; i++ {
-		createOps = append(createOps, txn.Op{
-			C:      "accounts",
-			Id:     i,
-			Insert: M{"balance": 0},
-		})
-	}
-	err := s.runner.Run(createOps, "", nil)
-	c.Assert(err, IsNil)
-	// Force a bad transaction into the queue
-	badTxnId := "deadbeef1234567812345678_12345678"
-	err = s.accounts.UpdateId(0, M{"$set": M{"txn-queue": []string{badTxnId}}})
-	c.Assert(err, IsNil)
-	for i := 1; i < expectedLength; i++ {
-		ops := []txn.Op{{
-			C:      "accounts",
-			Id:     0,
-			Update: M{"$inc": M{"balance": -1}},
-		}, {
-			C:      "accounts",
-			Id:     i,
-			Update: M{"$inc": M{"balance": 1}},
-		}}
-		err = s.runner.Run(ops, "", nil)
-		c.Assert(err, NotNil)
-		c.Assert(err, ErrorMatches, `cannot find transaction ObjectIdHex."deadbeef1234567812345678".`)
-	}
-	// Now that we've filled up the txn-queue of the first document, any
-	// further changes should be aborted
-	var doc bson.M
-	err = s.accounts.FindId(0).One(&doc)
-	c.Assert(err, IsNil)
-	c.Check(len(doc["txn-queue"].([]interface{})), Equals, expectedLength)
-	txn.SetDebug(true)
-	for i := 100; i < maxDocs; i++ {
-		ops := []txn.Op{{
-			C:      "accounts",
-			Id:     0,
-			Update: M{"$inc": M{"balance": -1}},
-		}, {
-			C:      "accounts",
-			Id:     i,
-			Update: M{"$inc": M{"balance": 1}},
-		}}
-		err = s.runner.Run(ops, "", nil)
-		c.Assert(err, NotNil)
-		c.Check(err, ErrorMatches, `txn-queue for 0 in "accounts" has too many transactions \(\d+\)`)
-	}
-	err = s.accounts.FindId(0).One(&doc)
-	c.Assert(err, IsNil)
-	c.Check(len(doc["txn-queue"].([]interface{})), Equals, expectedLength)
-	err = s.accounts.UpdateId(0, M{"$pullAll": M{"txn-queue": []string{badTxnId}}})
-	c.Assert(err, IsNil)
-	c.Log("Updated removing the invalid transaction")
-	// Now we should be able to cleanup
-	err = s.runner.ResumeAll()
-	c.Assert(err, IsNil)
-	c.Log("resumed all")
-}
-
-func (s *S) TestTxnQueueUnlimited(c *C) {
-	opts := txn.DefaultRunnerOptions()
-	// A value of 0 should mean 'unlimited'
-	opts.MaxTxnQueueLength = 0
-	s.runner.SetOptions(opts)
-	// it isn't possible to actually prove 'unlimited' but we can prove that
-	// we at least can insert more than the default number of transactions
-	// without getting a 'too many transactions' failure.
-	txn.SetDebug(false)
-	txn.SetChaos(txn.Chaos{
-		KillChance: 1,
-		// Use set-prepared because we are adding more transactions than
-		// other tests, and this speeds up setup time a bit
-		Breakpoint: "set-prepared",
-	})
-	defer txn.SetChaos(txn.Chaos{})
-	err := s.accounts.Insert(M{"_id": 0, "balance": 100})
-	c.Assert(err, IsNil)
-	ops := []txn.Op{{
-		C:      "accounts",
-		Id:     0,
-		Update: M{"$inc": M{"balance": 100}},
-	}}
-	for i := 0; i < 1100; i++ {
-		err := s.runner.Run(ops, "", nil)
-		c.Assert(err, Equals, txn.ErrChaos)
-	}
-	txn.SetDebug(true)
-	var doc bson.M
-	err = s.accounts.FindId(0).One(&doc)
-	c.Assert(err, IsNil)
-	c.Check(len(doc["txn-queue"].([]interface{})), Equals, 1100)
-	err = s.runner.Run(ops, "", nil)
-	c.Check(err, Equals, txn.ErrChaos)
-	err = s.accounts.FindId(0).One(&doc)
-	c.Assert(err, IsNil)
-	c.Check(len(doc["txn-queue"].([]interface{})), Equals, 1101)
-}
-
 func (s *S) TestPurgeMissingPipelineSizeLimit(c *C) {
 	// This test ensures that PurgeMissing can handle very large
 	// txn-queue fields. Previous iterations of PurgeMissing would
